<?php

final class PhabricatorVeryWowEnglishTranslation
  extends PhutilTranslation {

  public function getLocaleCode() {
    return 'en_W*';
  }

  protected function getTranslations() {
    return array(
      'Search' => 'Search! Wow!',
      'Review Code' => 'Wow! Code Review! Wow!',
      'Tasks and Bugs' => 'Much Bug! Very Bad!',
      'Cancel' => 'Nope!',
      'Advanced Search' => 'Much Search!',
      'No search results.' => 'No results! Wow!',
      'Send' => 'Bark Bark!',
      'Partial' => 'Pawtial',
      'Partial Upload' => 'Pawtial Upload',
      'Submit' => 'Such Send!',
      'Create Task' => 'Very Create Task',
      'Create' => 'Very Create',
      'Host and Browse Repositories' => 'Many Repositories',
      'Okay' => 'Wow!',
      'Save Preferences' => 'Make Such Saves',
      'Edit Query' => 'Play With Query',
      'Hide Query' => 'Bury Query',
      'Execute Query' => 'Throw the Query',
      'Wiki' => 'Wow-ki',
      'Get Organized' => 'Such Organized',
      'Explore More Applications' => 'Browse For More Wow',
      'Prototype' => 'Chew Toy',
      'Continue' => 'Bark And Run',
      'Countdown to Events' => 'To the Moon!',
    );
  }
}
