/**
 * @provides phabricator-fatal-config-template-css
 */
body {
  overflow-y: scroll;
  background: #f9f9f9;
  margin: 0;
  padding: 0;
  font: 13px/1.231 'Segoe UI', 'Segoe UI Web Regular', 'Segoe UI Symbol',
    'Helvetica Neue', Helvetica, Arial, sans-serif;
  text-align: left;
  -webkit-text-size-adjust: none;
}
