/**
 * @provides phui-button-css
 */


button,
a.button {
  font: {$basefont};
  -webkit-font-smoothing: antialiased;
  -webkit-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none;
}

button.phabricator-action-view-item {
  -webkit-font-smoothing: auto;
}

button::-moz-focus-inner {
  padding: 0;
  border: 0;
}

button,
a.button,
a.button:visited,
input[type="submit"] {
  background-color: {$blue};
  border: 1px solid {$blue};
  color: white;
  cursor: pointer;
  font-weight: bold;
  font-size: {$normalfontsize};
  display: inline-block;
  padding: 3px 12px 4px;
  text-align: center;
  white-space: nowrap;
  border-radius: 3px;
}

/* Buttons with images (full size only) */
button.icon,
a.icon,
a.icon:visited {
  padding-left: 0;
  position: relative;
  text-indent: 29px;
}

button.green,
a.green,
a.green:visited {
  background-color: {$green};
  border-color: {$green};
}

button.grey,
input[type="submit"].grey,
a.grey,
a.grey:visited {
  background-color: {$lightgreybackground};
  border-color: {$lightgreyborder};
  color: {$darkgreytext};
}

button.simple,
input[type="submit"].simple,
a.simple,
a.simple:visited {
  background: #fff;
  color: {$blue};
  border: 1px solid {$blue};
}

a.simple.current {
  background: {$lightblue};
}

button.simple .phui-icon-view,
input[type="submit"].simple .phui-icon-view,
a.simple .phui-icon-view,
a.simple:visited .phui-icon-view {
  color: {$blue};
}

a.disabled,
button.disabled,
button[disabled] {
  filter:alpha(opacity=50);
  -moz-opacity: 0.5;
  -khtml-opacity: 0.5;
  opacity: 0.5;
}

a.phuix-dropdown-open {
  color: {$greytext};
}

a.button:hover,
button:hover {
  text-decoration: none;
  background-color: {$sky};
}

a.button.grey:hover,
button.grey:hover {
  background-color: #fff;
}

a.button.green:hover,
button.green:hover {
  background-color: #0DAD48;
}

a.button.simple:hover,
button.simple:hover {
  background-color: {$blue};
  color: #fff;
}

a.button.simple:hover .phui-icon-view,
button.simple:hover .phui-icon-view {
  color: #fff;
}

a.button.simple .phui-icon-view {
  border: none;
}

a.button.simple.phuix-dropdown-open {
  background-color: #fff;
  color: {$blue};
  box-shadow: none;
}

a.button.simple.phuix-dropdown-open:hover .phui-icon-view {
  color: {$blue};
}

body a.button.disabled:hover,
body button.disabled:hover,
body a.button.disabled:active,
body button.disabled:active {
  box-shadow: none;
}

button.small,
a.small,
a.small:visited {
  padding: 2px 8px;
  height: auto;
  font-size: {$smallestfontsize};
  line-height: 16px;
}

button.link {
  display: inline;
  border: none;
  background: transparent;
  font-weight: normal;
  padding: 0;
  margin: 0;
  font-size: inherit;
  border-bottom: none;
  text-decoration: none;
  color: #19558D;
  -webkit-box-shadow: none;
  -moz-box-shadow: none;
  box-shadow: none;
}

button.link:hover {
  text-decoration: underline;
}

.phuix-dropdown-menu {
  position: absolute;
  width: 240px;
  background: #fff;
  margin-top: -1px;
  padding: 5px 0;
  box-shadow: {$dropshadow};
  border: 1px solid {$blueborder};
  border-radius: 3px;
  margin-bottom: 16px;
}

.phuix-dropdown-menu a:focus {
  /* We automatically focus links in dropdown menus for assistive devices, but
     this is distracting for visual user agents. */
  outline: none;
}

a.policy-control {
  width: 240px;
  text-align: left;
}

a.policy-control .caret {
  float: right;
}

.caret {
  display: inline-block;
  width: 0;
  height: 0;
  vertical-align: top;
  border-top: 5px solid #fff;
  border-right: 5px solid transparent;
  border-left: 5px solid transparent;
  content: "";
}

.caret-right {
  display: inline-block;
  width: 0;
  height: 0;
  vertical-align: middle;
  border-left: 7px solid {$greytext};
  border-top: 5px solid transparent;
  border-bottom: 5px solid transparent;
  content: "";
  margin-bottom: 4px;
}

.caret-left {
  display: inline-block;
  width: 0;
  height: 0;
  vertical-align: middle;
  border-right: 7px solid {$greytext};
  border-bottom: 5px solid transparent;
  border-top: 5px solid transparent;
  content: "";
  margin-bottom: 4px;
}

.dropdown .caret {
  margin-top: 7px;
  margin-left: 6px;
}

.small.dropdown .caret {
  margin-top: 6px;
}

.grey.dropdown .caret {
  border-top-color: #000;
}

/* Icons */
.button.has-icon {
  position: relative;
}

.button .phui-icon-view {
  display: inline-block;
  position: absolute;
  top: 6px;
  left: 10px;
}

.button.icon-last .phui-icon-view {
  left: auto;
  right: 10px;
}

.phui-button-bar .button .phui-icon-view {
  left: 9px;
}

.button.has-icon .phui-button-text {
  margin-left: 16px;
}

.button.has-icon.icon-last .phui-button-text {
  margin: 0 16px 0 0;
}

/* Login Buttons */

.button.big.has-icon {
  padding: 4px 20px 4px 14px;
  border-radius: 4px;
  text-align: left;
}

.button.big.has-icon .phui-button-text {
  margin-left: 30px;
  display: block;
}

.button.big.has-icon .phui-button-subtext {
  color: {$greytext};
  font-size: {$smallerfontsize};
  line-height: 15px;
  font-weight: normal;
}

/* PHUI Button Bar */

.phui-button-bar-borderless .button {
  border: 0;
  background-color: transparent;
  padding-left: 10px;
  padding-right: 10px;
}

.phui-button-bar-borderless .button .phui-icon-view {
  font-size: 15px;
  color: rgba(55,55,55,.4);
}

.phui-button-bar-borderless .button:hover {
  background-color: transparent;
  border-radius: 3px;
}

.phui-button-bar-borderless .button:hover .phui-icon-view {
  color: rgba(55,55,55,.9);
}

.phui-button-bar-borderless .button {
  border: 0;
}

.phui-button-bar a.button.has-icon {
  display: inline-block;
  height: 18px;
  width: 6px;
}

.phui-button-bar .phui-button-bar-first {
  border-top-right-radius: 0px;
  border-bottom-right-radius: 0px;
}

.phui-button-bar .phui-button-bar-middle {
  border-radius: 0;
  border-left: none;
}

.phui-button-bar .phui-button-bar-last {
  border-left: none;
  border-top-left-radius: 0px;
  border-bottom-left-radius: 0px;
}

.phui-button-bar .button.simple:hover {
  border-color: {$lightblueborder};
  background-color: #fff;
  color: {$sky};
}

.phui-button-bar .button.simple:hover .phui-icon-view {
  border-color: {$lightblueborder};
  color: {$sky};
}
