<?php

// TODO: Unify with similar Repository constants
final class DifferentialRevisionControlSystem extends Phobject {

  const SVN         = 'svn';
  const GIT         = 'git';
  const MERCURIAL   = 'hg';

}
