/**
 * @provides phabricator-application-launch-view-css
 */


/* - Application List ----------------------------------------------------------

  Spacing container for the list of large application buttons.

*/


.application-tile-group {
  overflow: hidden;
}


/* - Application Launch Button -------------------------------------------------

  Spacing container for the list of large application buttons.

*/

a.phabricator-application-launch-container,
div.phabricator-application-launch-container {
  display: block;
  float: left;
  overflow: hidden;
  position: relative;
  text-decoration: none;
  width: 100%;
  border-top-right-radius: 3px;
  border-bottom-right-radius: 3px;
  padding: 4px 0;
}

.device-phone div.phabricator-application-launch-container {
  display: none;
}

.phabricator-application-launch-icon {
  position: absolute;
  width: 38px;
  height: 18px;
  top: 6px;
  left: 0;
  font-size: 18px;
  text-align: center;
  vertical-align: bottom;
  color: {$darkbluetext};
  text-shadow: {$whitetextshadow};
}

.device-desktop a.phabricator-application-launch-container:hover {
  background-color: rgba(0,0,0,.07);
  text-decoration: none;
}

.device-desktop a.phabricator-application-launch-container:hover
  .phabricator-application-launch-icon {
  color: {$sky};
}

.phabricator-application-launch-name {
  display: block;
  font-weight: bold;
  color: {$darkbluetext};
  font-size: {$normalfontsize};
  margin-left: 36px;
}

.phabricator-application-launch-description {
  color: {$bluetext};
  font-size: {$smallestfontsize};
  margin-left: 36px;
  text-overflow: ellipsis;
  width: 150px;
  overflow: hidden;
  white-space: nowrap;
  display: inline-block;
  padding: 2px 0 0 0;
}

.phabricator-application-launch-attention {
  position: absolute;
  top: 8px;
  right: 8px;
  color: {$darkbluetext};
  font-weight: bold;
  font-size: {$smallerfontsize};
}

.phabricator-application-attention-count {
  color: {$fire};
}

a.phabricator-application-launch-phone-only {
  display: none;
}

.device-phone a.phabricator-application-launch-phone-only {
  display: block;
}
