/**
 * @provides differential-changeset-view-css
 * @requires phui-inline-comment-view-css
 */

.differential-changeset {
  position: relative;
  margin: 0;
  padding-top: 32px;
  overflow-x: auto;

  /* Fixes what seems to be a layout bug in Firefox which causes scrollbars,
     to appear unpredictably, see discussion in T7690. */
  overflow-y: hidden;
}

.device-phone .differential-changeset {
  overflow-x: scroll;
  -webkit-overflow-scrolling: touch;
}

.differential-diff {
  background: #fff;
  width: 100%;
  border-top: 1px solid {$lightblueborder};
  border-bottom: 1px solid {$lightblueborder};
  table-layout: fixed;
}

.differential-diff.diff-2up {
  min-width: 780px;
}

.differential-diff col.num {
  width: 45px;
}

.device .differential-diff.diff-1up col.num {
  width: 32px;
}

.differential-diff.diff-2up col.left,
.differential-diff.diff-2up col.right {
  width: 49.25%;
}

.differential-diff.diff-1up col.unified {
  width: 99.5%;
}

.differential-diff col.copy {
  width: 0.5%;
}

.differential-diff col.cov {
  width: 1%;
}

.differential-diff td {
  vertical-align: top;
  white-space: pre-wrap;
  word-wrap: break-word;
  padding: 0 8px 1px;
  line-height: 16px;
}

.device .differential-diff td {
  padding: 0 4px 1px;
}

.device .differential-diff .inline td {
  padding: 4px;
}

.differential-diff td .zwsp {
  position: absolute;
  width: 0;
}

.differential-diff th {
  text-align: right;
  padding: 2px 6px 0px 0px;
  vertical-align: top;
  background: {$lightbluebackground};
  color: {$bluetext};
  cursor: pointer;
  border-right: 1px solid {$thinblueborder};
  overflow: hidden;

  -moz-user-select: -moz-none;
  -khtml-user-select: none;
  -webkit-user-select: none;
  -ms-user-select: none;
  user-select: none;
}

.differential-diff th.selected {
  background-color: {$sh-yellowbackground};
}

.differential-changeset-immutable .differential-diff th {
  cursor: auto;
}

.differential-diff td.old {
  background: #ffd0d0;
}

.differential-diff td.new {
  background: #d0ffd0;
}

.differential-diff td.old-rebase {
  background: #ffeeee;
}

.differential-diff td.new-rebase {
  background: #eeffee;
}

.differential-diff td.old-full,
.differential-diff td.old span.bright {
  background: #ffaaaa;
}

.differential-diff td.new-full,
.differential-diff td.new span.bright {
  background: #aaffaa;
}

.differential-diff td.copy {
  min-width: 0.5%;
  width: 0.5%;
  padding: 0;
}

.differential-diff td.new-copy,
.differential-diff td.new-copy span.bright {
  background: {$lightyellow};
}

.differential-diff td.new-move,
.differential-diff td.new-move span.bright {
  background: {$yellow};
}

.differential-diff td.comment {
  background: #dddddd;
}

.differential-diff td.cov {
  padding: 0;
}

.diffusion-source td.cov {
  padding: 0 8px;
}

td.cov-U {
  background: #dd8866;
}

td.cov-C {
  background: #66bbff;
}

td.cov-N {
  background: #ddeeff;
}

td.cov-X {
  background: #aa00aa;
}

td.cov-I {
  background: {$lightgreybackground};
}

.differential-diff td.source-cov-C,
.differential-diff td.source-cov-C span.bright {
  background: #cceeff;
}

.differential-diff td.source-cov-U,
.differential-diff td.source-cov-U span.bright {
  background: #ffbb99;
}

.differential-diff td.source-cov-N,
.differential-diff td.source-cov-N span.bright {
  background: #f3f6ff;
}

.differential-diff td.show-more,
.differential-diff th.show-context-line,
.differential-diff td.show-context,
.differential-diff td.differential-shield {
  background: {$lightbluebackground};
  padding: 12px 0;
  border-top: 1px solid {$thinblueborder};
  border-bottom: 1px solid {$thinblueborder};
}

.device .differential-diff td.show-more,
.device .differential-diff th.show-context-line,
.device .differential-diff td.show-context,
.device .differential-diff td.differential-shield {
  padding: 6px 0;
}

.differential-diff td.show-more,
.differential-diff td.differential-shield {
  font: {$basefont};
  font-size: {$smallerfontsize};
  white-space: normal;
}

.differential-diff td.show-more {
  text-align: center;
  color: {$bluetext};
}

.differential-diff th.show-context-line {
  padding-right: 6px;
}

.differential-diff td.show-context {
  padding-left: 14px;
}

.differential-diff td.differential-shield {
  text-align: center;
}

.differential-diff td.differential-shield a {
  font-weight: bold;
}

.differential-diff .differential-image-diff {
  background-image: url(/rsrc/image/checker_light.png);
}

.differential-diff .differential-image-diff:hover {
  background-image: url(/rsrc/image/checker_dark.png);
}

.differential-diff .differential-image-diff td {
  padding: 8px;
}

.differential-image-stage {
  overflow: auto;
}

.differential-meta-notice {
  border-top: 1px solid {$sh-lightyellowborder};
  border-bottom: 1px solid {$sh-lightyellowborder};
  background-color: {$sh-yellowbackground};
  padding: 12px;
}

.differential-meta-notice + .differential-diff {
  border-top: none;
}

.differential-changeset h1 {
  font-size: {$biggestfontsize};
  padding: 2px 0 12px 12px;
  line-height: 20px;
  color: #000;
}

.device-phone .differential-changeset h1 {
  word-break: break-word;
  margin-right: 8px;
}

.differential-reticle {
  background-color: {$sh-yellowbackground};
  border: 1px solid {$sh-yellowborder};
  position: absolute;
  opacity: 0.5;
  top: 0;
  left: 0;
  box-sizing: border-box;
}

.differential-diff .inline td {
  padding: 8px 12px;
}

.differential-loading {
  border-top: 1px solid {$yellow};
  border-bottom: 1px solid {$yellow};
  background-color: {$lightyellow};
  padding: 12px;
  text-align: center;
}

.differential-collapse-undo {
  color: {$darkbluetext};
  padding: 12px;
  border: 1px solid {$blue};
  text-align: center;
  background-color: {$lightblue};
}

.differential-collapse-undo a {
  font-weight: bold;
}

.differential-file-icon-header .phui-icon-view {
  display: inline-block;
  margin: 0 6px 2px 0;
  vertical-align: middle;
  font-size: 14px;
}

.device-phone .differential-file-icon-header .phui-icon-view {
  display: none;
}

.differential-changeset-buttons {
  float: right;
  margin-right: 8px;
}

.device-phone .differential-changeset-buttons {
  float: none;
  margin: 0 0 8px 4px;
}

.differential-changeset-buttons a.button {
  margin-left: 8px;
}

.differential-property-table {
  margin: 12px;
  background: {$lightgreybackground};
  border: 1px solid {$lightblueborder};
  border-bottom: 1px solid {$blueborder};
}

.differential-property-table td em {
  color: {$lightgreytext};
}

.differential-property-table td.oval {
  background: #ffd0d0;
  width: 50%;
}

.differential-property-table td.nval {
  background: #d0ffd0;
  width: 50%;
}

tr.differential-inline-hidden {
  display: none;
}

tr.differential-inline-loading {
  opacity: 0.5;
}
