<?php

final class PhabricatorPHIDConstants extends Phobject {

  const PHID_TYPE_UNKNOWN = '????';
  const PHID_TYPE_MAGIC   = '!!!!';
  const PHID_TYPE_STRY    = 'STRY';
  const PHID_TYPE_TOBJ    = 'TOBJ';
  const PHID_TYPE_LEGB    = 'LEGB';

  const PHID_TYPE_XCMT    = 'XCMT';

  const PHID_TYPE_XOBJ    = 'XOBJ';

  const PHID_TYPE_VOID    = 'VOID';
  const PHID_VOID         = 'PHID-VOID-00000000000000000000';

}
