/**
 * @provides phabricator-hovercard-view-css
 */

.jx-hovercard-container {
  position: absolute;
}

.phabricator-hovercard-wrapper {
  float: left;
  width: 400px;
}

.device-phone .phabricator-hovercard-wrapper {
  float: left;
  width: 300px;
}

.phabricator-hovercard-container {
  float: left;
  width: 100%;
  box-shadow: {$dropshadow};
  border: 1px solid {$blueborder};
  border-radius: 3px;
  background-color: #fff;
}

.phabricator-hovercard-head .phui-header-shell {
  padding: 6px 8px 6px 12px;
  background-color: {$bluebackground};
  border-top-left-radius: 3px;
  border-top-right-radius: 3px;
}

.phabricator-hovercard-head .phui-header-header {
  font-size: 14px;
}

.phabricator-hovercard-head .phui-tag-type-state {
  color: {$darkbluetext};
  text-shadow: none;
  font-weight: normal;
}

.phabricator-hovercard-tags {
  float: right;
  white-space: normal;
}

.phabricator-hovercard-body {
  padding: 12px;
  color: {$darkgreytext};
  border-bottom-right-radius: 3px;
  border-bottom-left-radius: 3px;
  position: relative;
}

.phabricator-hovercard-body-item {
  margin: 4px 0 0 0;
}

.phabricator-hovercard-body-header {
  font-size: 14px;
  padding-bottom: 4px;
  color: {$darkgreytext};
  line-height: 18px;
}

.phabricator-hovercard-body .phabricator-hovercard-body-image {
  width: 58px;
}

.phabricator-hovercard-body .phabricator-hovercard-body-details {
  margin-left: 58px;
}

.phabricator-hovercard-body .profile-header-picture-frame {
  float: left;
  width: 50px;
  height: 50px;
  background-position: center;
  background-repeat: no-repeat;
  background-size: 100%;
}

.hovercard-badges {
  margin: 8px 0 0 0;
}

.hovercard-badges .phui-badge-flex-item {
  float: left;
}

.phabricator-hovercard-tail {
  width: 396px;
  float: left;
  padding: 2px;
  border-top: 1px solid {$thinblueborder};
  border-bottom-left-radius: 3px;
  border-bottom-right-radius: 3px;
}

.phabricator-hovercard-tail button,
.phabricator-hovercard-tail a.button {
  margin: 3px;
}
