/**
 * @provides phabricator-ui-example-css
 */

.phabricator-ui-example-note {
  padding: .5em;
  margin: 2em 1em 0.5em;
  background: #ffffdd;
  border: 1px solid #f5e178;
}

.phabricator-ui-example-note tt {
  font-weight: bold;
}

.jxui-example-container {
  padding: 2em;
  margin: 4em auto;
  border: 4px solid black;
  background: #dfdfdf;
  text-align: center;
  width: 50%;
}

.jxui-example-container a {
  margin: 1em;
}

.jxui-example-container a.jxui-active {
  box-shadow: 0 0 30px #ffff00;
}

.jxui-blue-background {
  background: {$blue};
}

.jxui-red-border {
  border-color: {$red};
}

.phui-example-icon-name {
  display: inline-block;
  width: 180px;
  padding-bottom: 8px;
  white-space: nowrap;
}

.phui-icon-view.phui-example-icon-transform {
  display: block;
  padding-bottom: 8px;
}
