<?php

final class DifferentialUnitTestResult extends Phobject {

  const RESULT_PASS     = 'pass';
  const RESULT_FAIL     = 'fail';
  const RESULT_SKIP     = 'skip';
  const RESULT_BROKEN   = 'broken';
  const RESULT_UNSOUND  = 'unsound';

}
