CREATE TABLE {$NAMESPACE}_drydock.drydock_command (
  id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
  authorPHID VARBINARY(64) NOT NULL,
  targetPHID VARBINARY(64) NOT NULL,
  command VARCHAR(32) NOT NULL COLLATE {$COLLATE_TEXT},
  isConsumed BOOL NOT NULL,
  dateCreated INT UNSIGNED NOT NULL,
  dateModified INT UNSIGNED NOT NULL,
  KEY `key_target` (targetPHID, isConsumed)
) ENGINE=InnoDB, COLLATE {$COLLATE_TEXT};
