/**
 * @provides phabricator-nav-view-css
 */

.jx-drag-col {
  cursor: col-resize;
}

.phabricator-nav-local,
.phabricator-nav-drag {
  display: none;
}

.device-desktop .has-local-nav .phabricator-nav-local,
.device-desktop .has-local-nav .phabricator-nav-drag {
  display: block;
}

.device .phabricator-side-menu-home .phabricator-nav-local {
  display: block;
}

.device-desktop .phabricator-side-menu-home .phabricator-nav-content,
.device-tablet .phabricator-side-menu-home .phabricator-nav-content {
  margin-left: 205px;
}

.phabricator-nav-local {
  width: 205px;
  position: absolute;
  left: 0;
  white-space: nowrap;
  overflow-x: hidden;
  overflow-y: auto;
  margin-top: 8px;
}

.phabricator-side-menu-home .phabricator-nav-local {
  margin-top: 16px;
}

.phabricator-nav-drag {
  position: fixed;
  top: 0;
  bottom: 0;
  left: 205px;
  width: 7px;

  cursor: col-resize;
  background: #f5f5f5;
  border-style: solid;
  border-width: 0 1px 0 1px;
  border-color: #fff #999c9e #fff #999c9e;

  box-shadow: inset -1px 0px 1px rgba(0, 0, 0, 0.15);

  background-image: url(/rsrc/image/divot.png);
  background-position: center;
  background-repeat: no-repeat;
}

.phabricator-nav-content {
  overflow: hidden;
}

.device-desktop .phabricator-standard-page-body .has-drag-nav
  .phabricator-nav-content {
  margin-left: 212px;
}

.device-desktop .has-local-nav .phabricator-nav-content {
  margin-left: 205px;
}

.device-phone .phabricator-side-menu-home .phabricator-nav-content {
  display: none;
}

.device-phone .phabricator-side-menu-home .phabricator-nav-local {
  width: 100%;
  padding-top: 0;
  margin-top: 0;
}
