/**
 * @provides phabricator-dashboard-css
 */

.dashboard-view {
  margin: 16px;
}

.device-phone .dashboard-view {
  margin: 8px;
}

.dashboard-view .phui-object-box {
  margin: 0 0 16px 0;
}

.dashboard-view .phui-object-box .phui-object-box {
  margin: 0;
}

.device-desktop .aphront-multi-column-fluid .aphront-multi-column-2-up
.aphront-multi-column-column-outer.half {
  width: 50%;
}

.device-desktop .aphront-multi-column-fluid .aphront-multi-column-2-up
.aphront-multi-column-column-outer.third {
  width: 33.34%;
}

.device-desktop .aphront-multi-column-fluid .aphront-multi-column-2-up
.aphront-multi-column-column-outer.thirds {
  width: 66.66%;
}

.aphront-multi-column-fluid
.aphront-multi-column-column-outer.grippable
.aphront-multi-column-column .dashboard-pane {
  cursor: move;
}

.aphront-multi-column-fluid
.aphront-multi-column-column .drag-ghost {
  list-style-type: none;
  margin: 16px;
}

.aphront-multi-column-fluid
.aphront-multi-column-column
.dashboard-panel-placeholder {
  display: none;
}

.aphront-multi-column-fluid
.aphront-multi-column-column.dashboard-column-empty
.dashboard-panel-placeholder {
  display: block;
  padding: 24px;
  margin: 16px 16px 0px 16px;
  text-decoration: none;
  border: 1px {$greyborder} dashed;
  color: {$greytext};
}

.aphront-multi-column-fluid
.aphront-multi-column-column.drag-target-list
.dashboard-panel-placeholder {
  display: none;
}

.aphront-multi-column-fluid
.aphront-multi-column-column-outer
.aphront-multi-column-column .phui-info-view {
  margin: 0;
}
