/**
 * @provides phui-timeline-view-css
 */

.phui-timeline-view {
  padding: 0 16px;
  background-image: url('/rsrc/image/BFCFDA.png');
  background-repeat: repeat-y;
  background-position: 94px;
}

.device .phui-timeline-view {
  padding: 0 8px;
}

.device-tablet .phui-timeline-view {
  background-position: 24px;
}

.device-phone .phui-timeline-view {
  padding: 0;
  background-position: 24px;
}

.phui-timeline-major-event .phui-timeline-group {
  border-left: 1px solid {$lightblueborder};
  border-right: 1px solid {$lightblueborder};
  border-radius: 3px;
}

.device-desktop .phui-timeline-event-view {
  margin-left: 62px;
  position: relative;
}

.device-desktop .phui-timeline-event-view.phui-timeline-minor-event {
  margin-left: 65px;
}

.device-desktop .phui-timeline-spacer {
  min-height: 16px;
}

.device-desktop .phui-timeline-event-view.the-worlds-end {
  background: {$lightblueborder};
  width: 9px;
  height: 9px;
  border-radius: 2px;
  margin-left: 74px;
}

.device-desktop .phui-timeline-wedge {
  border-bottom: 1px solid {$lightblueborder};
  position: absolute;
  width: 12px;
}

.device-phone .phui-timeline-minor-event,
.device-tablet .phui-timeline-minor-event {
  padding-left: 3px;
}

.phui-timeline-major-event .phui-timeline-content {
  border-top: 1px solid {$lightblueborder};
  border-bottom: 1px solid {$lightblueborder};
  border-radius: 3px;
}

.phui-timeline-title {
  line-height: 22px;
  min-height: 19px;
  position: relative;
  color: {$bluetext};
}

.phui-timeline-minor-event .phui-timeline-title {
  padding: 4px 8px 4px 33px;
}

.phui-timeline-title a {
  font-weight: bold;
  color: {$darkbluetext};
}

.device-desktop .phui-timeline-wedge {
  left: -12px;
}

.device-desktop .phui-timeline-major-event .phui-timeline-wedge {
  top: 26px;
}

.device-desktop .phui-timeline-minor-event .phui-timeline-wedge {
  top: 13px;
  left: -18px;
  width: 20px;
}

.phui-timeline-image {
  background-repeat: no-repeat;
  position: absolute;
  border-radius: 3px;
  box-shadow: {$borderinset};
  background-size: 100%;
  display: block;
}

.device-desktop .phui-timeline-major-event .phui-timeline-image {
  width: 50px;
  height: 50px;
  top: 0px;
  left: -62px;
}

.device-desktop .phui-timeline-minor-event .phui-timeline-image {
  width: 28px;
  height: 28px;
  background-size: 28px auto;
  left: -41px;
}

.phui-timeline-major-event .phui-timeline-title {
  background: {$lightbluebackground};
  min-height: 22px;
  border-top-right-radius: 3px;
}

.phui-timeline-title + .phui-timeline-title {
  border-radius: 0;
}

.phui-timeline-title {
  padding: 5px 8px;
  overflow-x: auto;
  overflow-y: hidden;
}

.phui-timeline-title-with-icon {
  padding-left: 38px;
}

.phui-timeline-title-with-menu {
  padding-right: 36px;
}

.phui-timeline-view .phui-icon-view.phui-timeline-token {
  vertical-align: middle;
  margin-right: 4px;
}

.phui-timeline-token.strikethrough {
  position: relative;
}

.phui-timeline-token.strikethrough:before {
  position: absolute;
  content: "";
  left: 0;
  top: 50%;
  right: 0;
  border-top: 3px solid;
  border-color: {$darkbluetext};

  -webkit-transform:rotate(-40deg);
  -moz-transform:rotate(-40deg);
  -ms-transform:rotate(-40deg);
  -o-transform:rotate(-40deg);
  transform:rotate(-40deg);
}

.phui-timeline-major-event .phui-timeline-content
  .phui-timeline-core-content {
  padding: 16px 12px;
  line-height: 18px;
  background: #fff;
  border-bottom-left-radius: 3px;
  border-bottom-right-radius: 3px;
}

.phui-timeline-core-content {
  overflow-x: auto;
}

.phui-timeline-core-content .comment-deleted {
  font-style: italic;
}

.device .phui-timeline-event-view {
  min-height: 23px;
  position: relative;
}

.device-phone .phui-timeline-event-view {
  margin: 0 8px;
}

.device .phui-timeline-image {
  display: none;
}

.device .phui-timeline-spacer {
  min-height: 8px;
  border-width: 0;
}

.phui-timeline-spacer.phui-timeline-spacer-bold {
  border-bottom: 4px solid {$lightblueborder};
  margin: 0;
}

.phui-timeline-spacer-bold + .phui-timeline-spacer {
  background-color: #ebecee;
}

.phui-timeline-icon-fill {
  position: absolute;
  width: 32px;
  height: 32px;
  background-color: {$lightblueborder};
  top: 0;
  left: 0;
  text-align: center;
}

.phui-icon-view.phui-timeline-icon:before {
  font-size: 15px;
}

.phui-timeline-minor-event .phui-timeline-icon-fill {
  height: 28px;
  width: 28px;
  border-radius: 3px;
}

.phui-timeline-icon-fill .phui-timeline-icon {
  margin-top: 8px;
}

.phui-timeline-minor-event .phui-timeline-icon-fill .phui-timeline-icon {
  margin-top: 7px;
}

.phui-timeline-extra,
.phui-timeline-extra .phabricator-content-source-view {
  font-size: {$smallestfontsize};
  font-weight: normal;
  color: {$lightbluetext};
}

.phui-timeline-title .phui-timeline-extra a {
  font-weight: normal;
}

.device-desktop .phui-timeline-extra {
  float: right;
}

.device .phui-timeline-extra {
  display: inline-block;
  line-height: 16px;
  margin-left: 8px;
  white-space: nowrap;
}

.device-phone .phui-timeline-extra {
  display: block;
  margin: 0;
}

.phui-timeline-icon-fill-red {
  background-color: {$red};
}

.phui-timeline-icon-fill-orange {
  background-color: {$orange};
}

.phui-timeline-icon-fill-yellow {
  background-color: {$yellow};
}

.phui-timeline-icon-fill-green {
  background-color: {$green};
}

.phui-timeline-icon-fill-sky {
  background-color: {$sky};
}

.phui-timeline-icon-fill-blue {
  background-color: {$blue};
}

.phui-timeline-icon-fill-indigo {
  background-color: {$indigo};
}

.phui-timeline-icon-fill-violet {
  background-color: {$violet};
}

.phui-timeline-icon-fill-grey {
  background-color: #888;
}

.phui-timeline-icon-fill-black {
  background-color: #333;
}

.phui-timeline-shell.anchor-target {
  background: {$lightyellow};
  padding: 4px;
  margin: -4px;
}

.phui-timeline-preview-header {
  background: #e0e3ec;
  color: {$darkgreytext};
  padding: 4px 1.25%;
  border: solid {$blueborder} 1px 0;
}

.phui-timeline-change-details {
  padding: 10px 0;
  border-style: solid;
  border-color: #efefef;
  border-width: 1px 0;
}

.phui-timeline-older-transactions-are-hidden {
  background: {$sh-yellowbackground};
  border: 1px solid {$sh-yellowborder};
  text-align: center;
  padding: 12px;
  color: {$darkgreytext};
  cursor: pointer;
  border-radius: 3px;
}

.device-phone .phui-timeline-older-transactions-are-hidden {
  margin: 0 8px;
}

.phui-timeline-title .phui-timeline-extra-information a {
  font-weight: normal;
  color: {$bluetext};
}

.phui-timeline-comment-actions .phui-icon-view {
  width: 16px;
  height: 16px;
  font-size: 16px;
  text-align: center;
  overflow: hidden;
}

.phui-timeline-menu {
  position: absolute;
  right: 3px;
  top: 6px;
  width: 28px;
  height: 22px;
  text-align: center;
  line-height: 22px;
  font-size: 15px;
  border-left: 1px solid {$lightblueborder};
}

.phui-timeline-menu:focus {
  outline: none;
}

.phui-timeline-menu .phui-icon-view {
  color: {$lightgreytext};
}

a.phui-timeline-menu .phui-icon-view {
  color: {$bluetext};
}

.device-desktop a.phui-timeline-menu:hover .phui-icon-view {
  color: {$darkgreytext};
}

.phui-timeline-menu.phuix-dropdown-open {
  background: {$hovergrey};
}

.phui-timeline-view + .phui-object-box {
  margin-top: 0;
}

.phui-timeline-badges {
  position: absolute;
  left: -64px;
  top: 52px;
  width: 54px;
}

.phui-timeline-badges .phui-badge-mini {
  height: 18px;
  width: 18px;
  line-height: 16px;
}

.phui-timeline-badges .phui-badge-mini .phui-icon-view {
  font-size: 10px;
}
