<?php

final class PHUI extends Phobject {

  const MARGIN_SMALL = 'ms';
  const MARGIN_MEDIUM = 'mm';
  const MARGIN_LARGE = 'ml';

  const MARGIN_SMALL_LEFT = 'msl';
  const MARGIN_MEDIUM_LEFT = 'mml';
  const MARGIN_LARGE_LEFT = 'mll';

  const MARGIN_SMALL_RIGHT = 'msr';
  const MARGIN_MEDIUM_RIGHT = 'mmr';
  const MARGIN_LARGE_RIGHT = 'mlr';

  const MARGIN_SMALL_BOTTOM = 'msb';
  const MARGIN_MEDIUM_BOTTOM = 'mmb';
  const MARGIN_LARGE_BOTTOM = 'mlb';

  const MARGIN_SMALL_TOP = 'mst';
  const MARGIN_MEDIUM_TOP = 'mmt';
  const MARGIN_LARGE_TOP = 'mlt';

  const PADDING_SMALL = 'ps';
  const PADDING_MEDIUM = 'pm';
  const PADDING_LARGE = 'pl';

  const PADDING_SMALL_LEFT = 'psl';
  const PADDING_MEDIUM_LEFT = 'pml';
  const PADDING_LARGE_LEFT = 'pll';

  const PADDING_SMALL_RIGHT = 'psr';
  const PADDING_MEDIUM_RIGHT = 'pmr';
  const PADDING_LARGE_RIGHT = 'plr';

  const PADDING_SMALL_BOTTOM = 'psb';
  const PADDING_MEDIUM_BOTTOM = 'pmb';
  const PADDING_LARGE_BOTTOM = 'plb';

  const PADDING_SMALL_TOP = 'pst';
  const PADDING_MEDIUM_TOP = 'pmt';
  const PADDING_LARGE_TOP = 'plt';

  const TEXT_BOLD = 'phui-text-bold';
  const TEXT_UPPERCASE = 'phui-text-uppercase';
  const TEXT_STRIKE = 'phui-text-strike';

  const TEXT_RED = 'phui-text-red';
  const TEXT_ORANGE = 'phui-text-orange';
  const TEXT_YELLOW = 'phui-text-yellow';
  const TEXT_GREEN = 'phui-text-green';
  const TEXT_BLUE = 'phui-text-blue';
  const TEXT_INDIGO = 'phui-text-indigo';
  const TEXT_VIOLET = 'phui-text-violet';
  const TEXT_WHITE = 'phui-text-white';
  const TEXT_BLACK = 'phui-text-black';

}
