/**
 * @requires herald-rule-editor
 *           javelin-behavior
 * @provides javelin-behavior-herald-rule-editor
 * @javelin
 */

JX.behavior('herald-rule-editor', function(config) {
  new JX.HeraldRuleEditor(config);
});
