<?php

final class ConpherenceUpdateActions extends ConpherenceConstants {

  const METADATA = 'metadata';
  const MESSAGE = 'message';
  const DRAFT = 'draft';
  const JOIN_ROOM = 'join_room';
  const ADD_PERSON = 'add_person';
  const REMOVE_PERSON = 'remove_person';
  const NOTIFICATIONS = 'notifications';
  const ADD_STATUS = 'add_status';
  const LOAD = 'load';
}
