CREATE TABLE {$NAMESPACE}_search.search_profilepanelconfiguration (
  id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
  phid VARBINARY(64) NOT NULL,
  profilePHID VARBINARY(64) NOT NULL,
  panelKey VARCHAR(64) NOT NULL COLLATE {$COLLATE_TEXT},
  builtinKey VARCHAR(64) COLLATE {$COLLATE_TEXT},
  panelOrder INT UNSIGNED,
  visibility VARCHAR(32) NOT NULL COLLATE {$COLLATE_TEXT},
  panelProperties LONGTEXT NOT NULL COLLATE {$COLLATE_TEXT},
  dateCreated INT UNSIGNED NOT NULL,
  dateModified INT UNSIGNED NOT NULL,
  KEY `key_profile` (profilePHID, panelOrder)
) ENGINE=InnoDB, COLLATE {$COLLATE_TEXT};
