/**
 * @provides phabricator-slowvote-css
 */

.slowvote-embed {
  background: #fff;
  border-color: {$lightblueborder};
  border-radius: 3px;
}

.slowvote-header {
  font-weight: bold;
  line-height: 16px;
  border-bottom: 1px solid #bbbbbb;
}

.slowvote-description {
  color: {$greytext};
  font-size: {$biggerfontsize};
  padding: 12px 4px 4px;
}

.slowvote-header-content {
  padding: 8px;
}

.slowvote-body-content {
  padding: 8px 4px 0;
}

.slowvote-option-label {
  border: 1px solid {$sh-lightgreyborder};
  display: block;
  position: relative;
  padding: 8px 4px;
  cursor: pointer;
  font-weight: bold;
  overflow: hidden;
  background-color: {$lightgreybackground};
}

.device-desktop .slowvote-option-label:hover {
  border-color: {$sh-blueborder};
}

.slowvote-bar {
  position: absolute;
  top: 0;
  left: 0;
  bottom: 0;
  overflow: hidden;
  background-color: {$sh-bluebackground};
}

.device-phone .slowvote-bar {
  display: none;
}

.slowvote-control-offset {
  white-space: nowrap;
  position: absolute;
  left: 32px;
  top: 8px;
  width: 100%;
  color: {$darkbluetext};
  text-shadow: {$whitetextshadow};
}

.device-phone .slowvote-control-offset {
  white-space: normal;
  position: static;
  padding: 0 4px 8px;
}

.slowvote-option-label-group {
  margin: 0 0 12px;
}

.slowvote-option-label input[type="radio"],
.slowvote-option-label input[type="checkbox"] {
  margin: 0 12px 0 8px;
  font-weight: bold;
}

.slowvote-above-the-bar {
  position: relative;
}

.slowvote-status {
  color: {$darkbluetext};
  font-weight: normal;
  padding: 2px 0;
  line-height: 15px;
  text-align: right;
  font-size: {$smallerfontsize};
  top: 0;
  right: 8px;
  position: absolute;
  text-shadow: {$whitetextshadow};
}

.slowvote-voters {
  padding-top: 8px;
}

.slowvote-voter {
  display: inline-block;
  width: 25px;
  height: 25px;
  background-size: 25px 25px;
  border-radius: 3px;
  margin: 0 4px;
  box-shadow: {$borderinset};
}

.slowvote-footer {
  border-top-width: 1px;
  border-top-style: solid;
  border-top-color: {$thinblueborder};
  position: relative;
}

.slowvote-footer-content {
  padding: 8px;
  overflow: hidden;
}

.slowvote-footer-content .slowvote-hint {
  line-height: 24px;
  color: {$lightgreytext};
}

.slowvote-footer-content button {
  float: right;
}
