/**
 * @requires owners-path-editor
 *           javelin-behavior
 * @provides javelin-behavior-owners-path-editor
 * @javelin
 */

JX.behavior('owners-path-editor', function(config) {
  new JX.OwnersPathEditor(config);
});
