/**
 * @provides phui-icon-view-css
 */

.phui-icon-example .phui-icon-view {
  display: inline-block;
  vertical-align: top;
}

.phui-icon-view.sprite-tokens {
  height: 16px;
  width: 16px;
  display: inline-block;
  vertical-align: top;
}

.phui-icon-view.sprite-login {
  height: 28px;
  width: 28px;
}

.phui-icon-view.phuihead-medium {
  height: 50px;
  width: 50px;
}

.phui-icon-view.phuihead-small {
  height: 35px;
  width: 35px;
  background-size: 35px;
}

.phui-icon-has-text:before {
  margin-right: 4px;
}

a.phui-icon-view:hover {
  text-decoration: none;
  color: {$sky};
}

/* - Icon in a Circle ------------------------------------------------------- */

.phui-icon-circle {
  border: 1px solid {$lightblueborder};
  border-radius: 24px;
  height: 24px;
  width: 24px;
  text-align: center;
  display: inline-block;
  cursor: pointer;
}

.phui-icon-circle.circle-medium {
  height: 36px;
  width: 36px;
  border-radius: 36px;
}

.phui-icon-circle .phui-icon-view {
  height: 24px;
  width: 24px;
  font-size: 11px;
  line-height: 24px;
  color: {$lightblueborder};
  cursor: pointer;
}

.phui-icon-circle.circle-medium .phui-icon-view {
  font-size: 18px;
  line-height: 36px;
}

a.phui-icon-circle:hover {
  text-decoration: none;
  border-color: {$sky};
  cursor: pointer;
}

a.phui-icon-circle:hover .phui-icon-view {
  color: {$sky};
}
