/**
 * @provides phui-document-view-css
 */

.phui-document-view {
  margin-bottom: 16px;
  border-radius: 3px;
  position: relative;
}

.phui-document-view .phui-header-shell {
  padding: 16px;
  background-color: {$bluebackground};
}

.phui-document-content {
  border-radius: 3px;
}

.device-desktop .phui-document-view {
  border: 1px solid {$lightblueborder};
  max-width: 960px;
  margin: 16px auto;
}

.device-desktop .phui-document-box {
  max-width: 996px;
  margin: 24px auto;
}

.device-desktop .phui-document-fluid .phui-document-view {
  max-width: none;
  margin: 16px;
}

.phui-crumbs-view + .phui-document-fluid .phui-document-view {
  margin-top: 0;
}

.device-desktop .phui-document-view.phui-offset-view {
  max-width: 800px;
}

/* Fix so that Phriction Document preview is the same width as the document */
.device-desktop .phui-remarkup-preview .phui-document-view {
  width: 800px;
}

.phui-document-sidenav {
  position:absolute;
  width: 200px;
  text-overflow: ellipsis;
  top: 0;
  right: 0;
}

.device-phone .phui-document-sidenav {
  position: static;
  width: auto;
  border-top: 1px solid {$thinblueborder};
  border-bottom: 1px solid {$thinblueborder};
}

.device-phone .phui-sidenav-view .phui-document-inner {
  margin: 0;
}

.phui-sidenav-view .phui-document-inner {
  margin-right: 200px;
  border-right: 1px solid {$thinblueborder};
  background: #fff;
}

.phui-document-content .phui-header-shell {
  border-top: none;
  border-bottom: 1px solid {$lightblueborder};
}

.phui-document-content
  .phui-header-shell.phui-header-no-backgound {
  border-bottom: 1px solid {$thinblueborder};
  margin: 0 0 16px 0;
}

.phui-document-content
  .phui-header-shell.phui-header-no-backgound
  .phui-header-view {
  padding: 8px 0 4px;
}

.phui-document-content .phui-property-list-container {
  border-bottom: 1px solid {$thinblueborder};
  background-color: {$lightgreybackground};
}

.legalpad .phui-document-content .phui-property-list-view {
  border: none;
  box-shadow: none;
  border-radius: 3px;
  margin: 16px 0 0  0;
  background-color: {$bluebackground};
}

.phui-document-content {
  background: #fff;
  overflow: hidden;
}

.phui-document-content .phabricator-remarkup {
  padding: 16px;
  font-size: 14px;
}

.phui-document-view .phui-header-action-links .phui-mobile-menu {
  display: block;
}

.device-phone .phui-document-content .phabricator-remarkup {
  padding: 8px;
}

.device-desktop .phui-document-content .phabricator-action-list-view {
  display: none;
}

.phui-document-content .phabricator-remarkup .remarkup-code-block {
  clear: both;
  margin: 16px 0;
}

.device-desktop .phui-document-offset {
  padding-right: 120px;
}

.phui-document-view .phui-info-severity-nodata {
  background-color: {$lightgreybackground};
}

body .phui-document-view .phui-header-shell.phui-bleed-header {
  padding: 0;
}

.phui-document-view .phui-property-list-section-header {
  padding: 20px 24px 0px;
  border-top: none;
}

.phui-document-view .phui-property-list-text-content {
  padding: 0 24px 4px;
}

.phui-document-view .PhabricatorMonospaced,
.phui-document-view .phabricator-remarkup .remarkup-code-block .remarkup-code {
  font: 11px/16px "Menlo", "Consolas", "Monaco", monospace;
}

.platform-windows .phui-document-view .PhabricatorMonospaced,
.platform-windows .phui-document-view .phabricator-remarkup .remarkup-code-block
  .remarkup-code {
  font: 12px/16px "Menlo", "Consolas", "Monaco", monospace;
}
