/**
 * @provides differential-core-view-css
 */

.differential-primary-pane {
  margin-bottom: 32px;
}

.differential-panel {
  padding: 16px;
}

.differential-panel h1 {
  border-bottom: 1px solid #aaaa99;
  padding-bottom: 8px;
  margin-bottom: 8px;
}

.differential-unselectable tr td:nth-of-type(1) {
  -moz-user-select: -moz-none;
  -khtml-user-select: none;
  -webkit-user-select: none;
  -ms-user-select: none;
  user-select: none;
}
