/**
 * @provides pholio-inline-comments-css
 */

.pholio-inline-comment-head {
  padding: 12px 0 4px;
  color: {$greytext};
}

.pholio-inline-comment-head a {
  font-weight: bold;
}

.pholio-transaction-inline-comment .phui-image-mask {
  margin-bottom: 8px;
}
