/**
 * @provides conpherence-widget-pane-css
 */

.conpherence-widget-pane,
.loading .widgets-loading-mask {
  position: fixed;
  right: 0px;
  top: 76px;
  bottom: 0;
  width: 240px;
  border-width: 0 0 0 1px;
  border-color: {$lightblueborder};
  border-style: solid;
  overflow-y: auto;
  -webkit-overflow-scrolling: touch;
}

.page-has-warning .conpherence-widget-pane,
.page-had-warning .loading .widgets-loading-mask {
  top: 110px;
}

.device .conpherence-widget-pane,
.device .loading .widgets-loading-mask {
  top: 44px;
  width: 100%;
}

.conpherence-widget-pane .widgets-loading-mask {
  opacity: .6;
  background: #fff;
  display: none;
}

.loading .widgets-loading-mask {
  display: block;
}

.conpherence-widget-pane .aphront-form-input {
  margin: 0;
  width: 100%;
}

.conpherence-widget-pane .aphront-form-inset {
  border: 0;
}

.conpherence-widget-pane .widgets-header .phui-header-shell {
  padding: 6px 8px;
}

.conpherence-widget-pane .widgets-header .phui-header-header {
  font-size: {$biggerfontsize};
}

.device .conpherence-widget-pane .widgets-header {
  display: none;
}

.conpherence-widget-pane .widgets-header .caret {
  float: none;
  height: 0px;
  width: 0px;
  margin: 10px 0 0 4px;
  border-top-color: #000;
}

.conpherence-widget-pane .widgets-header .phui-icon-view.disabled {
  color: {$lightgreytext};
}

.device-desktop .conpherence-layout .device-widgets-selector {
  display: none;
}

.conpherence-widget-pane .widgets-body {
  position: fixed;
  overflow-y: auto;
  bottom: 0;
  top: 76px;
  width: 100%;
}

#widgets-settings {
  padding: 3px 6px;
}

.device-desktop .conpherence-widget-pane .widgets-body {
  top: 111px;
  width: 240px;
}

.device-desktop .page-has-warning .conpherence-widget-pane .widgets-body {
  top: 142px;
}

.conpherence-widget-pane .widget-icon {
  display: block;
  height: 14px;
  width: 14px;
}

.conpherence-widget-pane .phabricator-remarkup-embed-layout-link {
  padding-bottom: 1px;
}

/* people widget */
.conpherence-widget-pane .people-widget-header .add-people-widget {
  padding: 10px 0 5px 0;
  overflow: hidden;
}

.conpherence-widget-pane .people-widget-header .add-people-widget
.aphront-form-control-tokenizer {
  float: left;
  width: 150px;
  padding: 0px 0px 0px 10px
}

.device .conpherence-widget-pane .people-widget-header .add-people-widget
.aphront-form-control-tokenizer {
  width: 70%;
}

.conpherence-widget-pane .people-widget-header .add-people-widget
.people-add-button {
  float: right;
  margin: 2px 8px 0px 0px;
  padding: 3px 16px 4px 16px;
}

#widgets-people {
  margin-top: 4px;
}

.conpherence-widget-pane .person-entry {
  padding: 4px 8px;
}

.conpherence-widget-pane .person-entry:hover {
  background-color: {$lightgreybackground};
}

.conpherence-widget-pane .person-entry a {
  float: left;
  font-weight: bold;
  line-height: 22px;
  color: {$darkbluetext};
}

.conpherence-widget-pane .person-entry a img {
  height: 24px;
  width: 24px;
  border-radius: 3px;
}

.conpherence-widget-pane .person-entry .pic {
  float: left;
  width: 30px;
}

.conpherence-widget-pane .person-entry .remove {
  float: right;
  width: 20px;
  height: 20px;
  font-size: 18px;
}

.person-entry .remove:hover .phui-icon-view {
  color: {$red};
}

.conpherence-widget-pane .person-entry .remove:hover {
  text-decoration: none;
}

.conpherence-widget-pane .person-entry .remove .close-icon {
  color: #bfbfbf;
}

.conpherence-widget-pane .person-entry .remove:hover .close-icon {
  color: #000;
}

/* settings widget */
.conpherence-widget-pane .title-update,
.conpherence-widget-pane .notifications-update {
  margin: 3px 0px 0px 4px;
}

.conpherence-widget-pane .no-settings {
  width: 200px;
  padding: 20px;
  text-align: center;
  color: {$greytext};
}

.device .conpherence-widget-pane .no-settings {
  width: 60px;
  margin: 0 auto 0 auto;
}
