/**
 * @provides maniphest-task-summary-css
 */
.device-phone .maniphest-batch-editor,
.device-phone .maniphest-task-updated {
  display: none;
}

.maniphest-batch-editor {
  border-top: 1px solid {$thinblueborder};
}

.maniphest-batch-editor-layout {
  width: 100%;
}

.maniphest-batch-editor-layout td {
  padding: 12px 4px 8px;
  white-space: nowrap;
}

.maniphest-batch-editor-layout a.button,
.maniphest-batch-editor-layout button {
  margin: 0px 4px;
}

.maniphest-batch-editor-layout .batch-select-submit-cell {
  text-align: right;
}

#batch-select-status-cell {
  text-align: right;
  color: {$greytext};
  font-size: {$smallestfontsize};
  vertical-align: middle;
  width: 100%;
}

.maniphest-board-link {
  color: {$bluetext};
}
