/**
 * @provides diviner-shared-css
 */

.diviner-monospace {
  font-family: monospace;
  font-size: 13px;
}

.diviner-table-view {
  margin: 0 0 24px;
}

.device-phone .diviner-table-view {
  margin: 0 8px 16px;
}

.diviner-table-header {
  padding: 4px 16px;
  background-color: {$greybackground};
  border-left: 1px solid {$lightblueborder};
  border-right: 1px solid {$lightblueborder};
  border-top: 1px solid {$lightblueborder};
  display: inline-block;
  border-top-left-radius: 3px;
  border-top-right-radius: 3px;
  font-weight: bold;
  color: {$bluetext};
}

.diviner-return-table-view,
.diviner-parameter-table-view {
  width: 100%;
  background: {$lightgreybackground};
  border: 1px solid {$lightblueborder};
}

.diviner-return-table-type,
.diviner-parameter-table-type {
  padding: 6px 8px 6px 12px;
  white-space: nowrap;
  text-align: right;
  color: {$bluetext};
  width: 20%;
}

.diviner-parameter-table-name {
  padding: 6px 8px;
  white-space: nowrap;
  font-weight: bold;
}

.diviner-return-table-docs,
.diviner-parameter-table-docs {
  padding: 6px 12px 6px 8px;
  width: 80%;
  background-color: #fff;
}

.diviner-document-section {
  padding: 0 0 16px 0;
}

.device-phone .diviner-document-section {
  padding: 0 8px 16px 0;
}

.phui-header-shell.diviner-section-header .phui-header-header {
  color: #000;
  font-size: 20px;
}

body .diviner-view .diviner-document-section
  .phui-header-shell.diviner-section-header {
  padding: 0 24px 8px 0;
  border-bottom: 1px solid {$thinblueborder};
  margin: 32px 0 16px;
}

body .diviner-document-section .phui-header-shell.phui-bleed-header {
  padding: 16px 0 0 0;
  border-bottom: none;
}

body .diviner-view .diviner-section-content .phui-header-shell {
  margin: 24px 0 8px;
  padding: 0;
  border: none;
}

.diviner-message-not-documented {
  color: {$lightgreytext};
  font-style: italic;
  margin: 12px 0 32px;
  font-size: {$biggerfontsize};
}

.diviner-document-section .diviner-message-not-documented {
  margin-left: 0;
  margin-right: 0;
}

.phui-document-content .phabricator-remarkup.diviner-remarkup-section {
  padding: 16px 0 32px 0;
}

.diviner-atom-signature-name {
  font-weight: bold;
  color: {$violet};
}

.diviner-atom-signature {
  font-weight: normal;
  color: #000;
}

.phui-header-view .phui-header-header a.diviner-atom-signature-name {
  color: {$violet};
}

.phui-header-view .phui-header-header a.diviner-atom-signature-name:hover {
  text-decoration: underline !important;
}

.diviner-list .diviner-atom-signature {
  margin-left: -16px;
}

.diviner-list a {
  font-weight: bold;
}

.diviner-method-implementation-header {
  color: {$greytext};
  margin-bottom: -8px;
}

.diviner-method-implementation-inherited {
  color: {$darkgreytext};
}

.diviner-method-implementation-inherited .diviner-method-implementation-header {
  color: {$lightgreytext};
}

/**
 * Fix excessive padding between method headers and method documentation for
 * methods with no inherited context.
 */
.diviner-document-section .phui-header-shell + .phabricator-remarkup {
  padding-top: 0;
}

.diviner-list {
  padding-left: 17px;
}

.diviner-list li {
  line-height: 20px;
  padding-bottom: 12px;
}

/* - Book Index ----------------------------------------------------------------

  How big lists of book indexes get displayed

*/

.diviner-book-item {
  display: block;
  padding: 6px 16px;
  margin: 4px -16px;
}

.diviner-book-item:hover {
  text-decoration: none;
}

.device-desktop .diviner-book-item:hover .diviner-book-item-title {
  color: {$violet};
}

.diviner-book-item-title {
  font-weight: bold;
  font-size: 15px;
  color: {$darkbluetext};
}

.diviner-book-item-type {
  color: {$lightgreytext};
  padding-left: 8px
}

.diviner-book-item-subtitle {
  display: block;
  line-height: 20px;
  color: {$lightgreytext};
}
