/**
 * @provides phui-property-list-view-css
 */

.phui-property-list-view .keyboard-shortcuts-available {
  float: right;
  height: 16px;
  margin: 12px 10px -28px 0px;
  padding: 0px 20px 0px 0px;
  vertical-align: middle;
  color: {$greytext};
  text-align: right;
  font-size: {$smallestfontsize};
  background:
    url('/rsrc/image/icon/fatcow/key_question.png') right center no-repeat;
}

.device .keyboard-shortcuts-available {
  display: none;
}

.phui-property-group-noninitial,
.phui-property-list-section-noninitial {
  border-color: {$thinblueborder};
  border-style: solid;
  border-width: 1px 0 0;
}

.device-desktop .phui-property-list-container {
  padding: 12px 0 12px 0;
  width: 100%;
}

.device .phui-property-list-container {
  padding: 12px 0 4px 0;
}

.phui-property-list-key {
  color: {$bluetext};
  font-weight: bold;
  overflow: hidden;
  white-space: nowrap;
}

.device-desktop .phui-property-list-key {
  width: 12%;
  margin-left: 1%;
  text-align: right;
  float: left;
  clear: left;
  margin-bottom: 4px;
}

.device-desktop .phui-property-list-has-actions .phui-property-list-key {
  width: 18%;
}

.phui-property-list-properties-wrap.phui-property-list-stacked {
  width: auto;
  float: none;
}

.device .phui-property-list-key,
.phui-property-list-stacked .phui-property-list-properties
  .phui-property-list-key {
    padding-left: 4px;
    text-align: left;
    margin-left: 0;
    width: auto;
    float: none;
}

.phui-property-list-value {
  color: {$darkgreytext};
}

.device-desktop .phui-property-list-value {
  width: 84%;
  margin-left: 1%;
  float: left;
  margin-bottom: 4px;
}

.device-desktop .phui-property-list-has-actions .phui-property-list-value {
  width: 78%;
}

.device .phui-property-list-value,
.phui-property-list-stacked .phui-property-list-properties
  .phui-property-list-value {
    padding: 0 8px;
    margin-bottom: 8px;
    width: auto;
    word-break: break-word;
    float: none;
}

.phui-property-list-section-header {
  color: {$bluetext};
  padding: 16px 4px 0px;
  text-transform: uppercase;
  font-weight: 700;
  border-color: {$thinblueborder};
  border-style: solid;
  border-width: 1px 0 0;
}

.phui-property-list-container + .phui-property-list-text-content {
  border-color: {$thinblueborder};
  border-style: solid;
  border-width: 1px 0 0;
}

.phui-property-list-section-noninitial .phui-property-list-section-header {
  border-top: none;
}

.device .phui-property-list-section-header {
  padding-left: 4px;
}

.phui-property-list-section-header-icon .phui-icon-view {
  display: inline-block;
  margin: -2px 4px -2px 0;
}

.phui-property-list-text-content {
  padding: 12px 4px;
  overflow: hidden;
}

.phui-property-list-raw-content {
  padding: 0px;
  overflow: hidden;
}

/* In the common case where we immediately follow a header, move back up 30px
   so we snuggle next to the header. */
.device-desktop .phui-header-view
  + .phabricator-action-list-view {
  margin-top: -30px;
}

.device-desktop .phui-header-view
  + .phabricator-action-list-view
  + .phui-property-list-view {
  margin-top: 0px;
}


.phui-property-list-image {
  margin: auto;
  max-width: 95%;
}

.phui-property-list-audio {
  display: block;
  margin: 16px auto;
  width: 50%;
  min-width: 240px;
}

/* When tags appear in property lists, give them a little more vertical
   spacing. */
.phui-property-list-view .phui-tag-view {
  margin: 2px 0;
}

.phui-property-list-has-actions .phui-property-list-properties-wrap {
  float: left;
  width: 78%;
}

.device .phui-property-list-properties-wrap {
  width: auto;
  border: none;
  float: none;
  overflow: auto;
}

.phui-property-list-actions {
  width: 20%;
  float: right;
  margin-right: 12px;
  border-left: 1px solid {$thinblueborder};
}

!print .phui-property-list-actions {
  display: none;
}

.device .phui-property-list-actions {
  float: none;
  width: auto;
  margin: -12px 0 12px 0;
  border: none;
}

.phui-property-list-image-content img {
  margin: 20px auto;
  background: url('/rsrc/image/checker_light.png');
  border: 1px solid {$lightblueborder};
}

.device-desktop .phui-property-list-image-content img:hover {
  background: url('/rsrc/image/checker_dark.png');
}

/* - Dashboards ------------------------------------------------------------ */

.dashboard-panel .phui-property-list-section {
  border-left: 1px solid {$lightblueborder};
  border-right: 1px solid {$lightblueborder};
  border-bottom: 1px solid {$blueborder};
}
