/**
 * @provides phabricator-notification-menu-css
 */

.phabricator-notification-menu {
  background: #fff;
  font-size: {$smallestfontsize};
  word-wrap: break-word;
  overflow-y: auto;
  box-shadow: {$dropshadow};
  border: 1px solid {$blueborder};
  border-radius: 3px;
}

.phabricator-notification .phabricator-notification-date {
  margin-left: 8px;
  color: {$lightgreytext};
}

.phabricator-notification-menu-loading {
  text-align: center;
  padding: 10px 0;
  color: {$lightgreytext};
}

.device-desktop .phabricator-notification-menu,
.device-tablet .phabricator-notification-menu {
  position: absolute;
  width: 360px;
  top: 42px;
}

.device-phone .phabricator-notification-menu {
  border-bottom: 1px solid {$thinblueborder};
  width: 94%;
  max-width: 390px;
  top: 42px !important;
  left: 3% !important;
  position: absolute;

}

.phabricator-notification-list.pm {
  padding: 0;
}

.phabricator-notification-list .phabricator-notification {
  padding: 10px 4px;
}

.phabricator-notification {
  padding: 6px 8px;
}

.phabricator-notification-menu .phabricator-notification {
  cursor: pointer;
}

.device-desktop .phabricator-notification-menu .phabricator-notification:hover {
  background: {$hoverblue};
}

.device-desktop .phabricator-notification-menu
 .phabricator-notification-unread.phabricator-notification:hover {
  background: {$hoverselectedblue};
}

.phabricator-notification + .phabricator-notification {
  border-top: 1px solid {$hoverborder};
}

.no-notifications {
  color: {$lightgreytext};
}

.phabricator-notification-list .phabricator-notification-unread,
.phabricator-notification-menu .phabricator-notification-unread {
  background: {$hoverblue};
}

.phabricator-notification-header {
  font-weight: bold;
  padding: 10px 8px;
  font-size: {$smallerfontsize};
  border-bottom: 1px solid {$thinblueborder};
}

.phabricator-notification-header a {
  color: {$darkgreytext};
}

.phabricator-notification-header a:hover {
  text-decoration: underline;
}

.phabricator-notification-header .phabricator-notification-clear-all {
  color: {$anchor};
  float: right;
  font-weight: normal;
}

.phabricator-notification-footer {
  background: {$greybackground};
  border-top: 1px solid {$thinblueborder};
  border-bottom-left-radius: 3px;
  border-bottom-right-radius: 3px;
  padding: 8px;
  font-size: {$smallerfontsize};
  color: {$darkgreytext};
}

.phabricator-notification-footer a {
  color: {$darkgreytext};
}

.phabricator-notification-footer a:hover {
  text-decoration: underline;
}

.phabricator-notification-menu .aphlict-connection-status {
  color: {$lightgreytext};
}

.aphlict-connection-status .aphlict-connection-status-connected {
  color: {$green};
}

.aphlict-connection-status .aphlict-connection-status-error {
  color: {$red};
}
