CREATE TABLE {$NAMESPACE}_policy.policy (
  id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
  phid VARCHAR(64) NOT NULL COLLATE utf8_bin,
  rules LONGTEXT NOT NULL COLLATE utf8_bin,
  defaultAction VARCHAR(32) NOT NULL COLLATE utf8_bin,
  dateCreated INT UNSIGNED NOT NULL,
  dateModified INT UNSIGNED NOT NULL,
  UNIQUE KEY (phid)
) ENGINE=InnoDB, COLLATE utf8_general_ci;
