CREATE TABLE {$NAMESPACE}_worker.worker_bulktask (
  id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
  bulkJobPHID VARBINARY(64) NOT NULL,
  objectPHID VARBINARY(64) NOT NULL,
  status VARCHAR(32) COLLATE {$COLLATE_TEXT} NOT NULL,
  data LONGTEXT COLLATE {$COLLATE_TEXT} NOT NULL,
  KEY `key_job` (bulkJobPHID, status),
  KEY `key_object` (objectPHID)
) ENGINE=InnoDB, COLLATE {$COLLATE_TEXT};
