/**
 * @provides phui-info-view-css
 */

.phui-info-view {
  border-style: solid;
  border-width: 1px;
  margin: 16px;
  padding: 12px;
  border-radius: 3px;
}

.device .phui-info-view {
  margin: 8px;
}

.phui-info-view .phui-form-view {
  padding: 0;
}

.phui-info-view-body {
  line-height: 1.6em;
}

.phui-info-view-body tt {
  padding: 0 2px;
  background-color: rgba(55,55,55,.1);
}

.phui-info-view-actions {
  margin-top: -3px;
  margin-bottom: -4px;
  float: right;
}

.phui-info-view-actions .button {
  margin-left: 4px;
}

.phui-info-view-head + .phui-info-view-body {
  padding-top: 4px;
}

h1.phui-info-view-head {
  font-weight: bold;
  font-size: {$biggerfontsize};
  line-height: 1.2em;
}

.phui-info-view-list {
  margin: 0 0 0 16px;
  list-style: disc;
  line-height: 1.5em;
}

.phui-info-view .phui-info-view-actions .button:hover {
  background: #fff;
  box-shadow: none;
}

.phui-info-severity-error,
.phui-info-severity-error a.button {
  border-color: {$sh-redborder};
  background: {$sh-redbackground};
}

.phui-info-severity-error a.button {
  color: {$sh-redtext};
}

.phui-info-severity-warning,
.phui-info-severity-warning a.button {
  border-color: {$sh-yellowborder};
  background: {$sh-yellowbackground};
}

.phui-info-severity-warning a.button {
  color: {$sh-yellowtext};
}

.phui-info-severity-notice,
.phui-info-severity-notice a.button {
  border-color: {$sh-blueborder};
  background: {$sh-bluebackground};
}

.phui-info-severity-notice a.button {
  color: {$sh-bluetext};
}

.phui-info-severity-nodata,
.phui-info-severity-nodata a.button {
  border-color: {$lightgreyborder};
  background: #fff;
}

.phui-info-severity-nodata a.button {
  color: {$greytext};
}

.phui-info-severity-success,
.phui-info-severity-success a.button {
  border-color: {$sh-greenborder};
  background: {$sh-greenbackground};
}

.phui-info-severity-success a.button {
  color: {$sh-greentext};
}

.aphront-dialog-body .phui-info-view {
  margin: 0 0 8px 0;
}

.phui-crumbs-view + .phui-info-view {
  margin-top: 0;
}

.phui-crumbs-view.phui-crumbs-border + .phui-info-view {
  margin-top: 16px;
}
