/**
 * @provides phui-header-view-css
 */

.phui-header-shell {
  border-bottom: 1px solid {$thinblueborder};
  overflow: hidden;
  padding: 0 4px 12px;
}

.phui-header-view {
  display: table;
  width: 100%
}

.phui-header-row {
  display: table-row;
}

.phui-header-col1 {
  display: table-cell;
  vertical-align: middle;
  width: 62px;
}

.phui-header-col2 {
  display: table-cell;
  vertical-align: middle;
  word-break: break-word;
}

.phui-header-col3 {
  display: table-cell;
  vertical-align: middle;
}

.device-phone .phui-header-col3 {
  vertical-align: top;
}

body .phui-header-shell.phui-header-no-backgound {
  background-color: transparent;
  border: none;
}

body .phui-header-shell.phui-bleed-header {
  background-color: #fff;
  border-bottom: 1px solid {$thinblueborder};
  width: auto;
  margin: 16px;
}

body .phui-header-shell.phui-bleed-header
  .phui-header-view {
  padding: 8px 24px 8px 0;
  color: {$darkbluetext};
}

.phui-header-shell + .phabricator-form-view {
  border-top-width: 0;
}

.phui-property-list-view + .diviner-document-section {
  margin-top: -1px;
}

.phui-header-view {
  position: relative;
  font-size: {$normalfontsize};
}

.phui-header-header {
  font-size: 16px;
  line-height: 24px;
  color: {$darkbluetext};
}

.phui-object-box .phui-header-tall .phui-header-header,
.phui-document-view .phui-header-tall .phui-header-header {
  font-size: 18px;
}

.phui-header-view .phui-header-header a {
  color: {$darkbluetext};
}

.device-desktop .phui-header-view .phui-header-header a:hover {
  text-decoration: none;
  color: {$blue};
}

.phui-header-view .phui-header-action-links {
  float: right;
}

.phui-object-box .phui-header-view .phui-header-action-links {
  margin-right: 4px;
  font-size: {$normalfontsize};
}

.device-phone .phui-header-action-link .phui-button-text {
  visibility: hidden;
  width: 0;
  margin-left: 8px;
}

.device-phone .phui-header-action-link.button .phui-icon-view {
  width: 12px;
  text-align: center;
}

.phui-header-divider {
  margin: 0 4px;
  font-weight: normal;
  color: {$lightbluetext};
}

.phui-header-tags {
  margin-left: 12px;
  font-size: {$normalfontsize};
}

.phui-header-tags .phui-tag-view {
  margin-left: 4px;
}

.phui-header-image {
  display: inline-block;
  background-repeat: no-repeat;
  background-size: 100%;
  box-shadow: {$borderinset};
  width: 50px;
  height: 50px;
}

.phui-header-subheader {
  font-weight: normal;
  font-size: {$biggerfontsize};
  margin-top: 8px;
}

.phui-header-subheader .phui-icon-view {
  display: inline-block;
  margin: -2px 4px -2px 0;
  font-size: 15px;
}

.phui-header-subheader,
.phui-header-subheader .policy-link {
  color: {$darkbluetext};
}

.policy-header-callout,
.phui-header-subheader .phui-header-status {
  padding: 3px 9px;
  border-radius: 3px;
  background: rgba(71, 87, 120, 0.06);
  margin-right: 8px;
}

.policy-header-callout.policy-adjusted-weaker {
  background: {$sh-greenbackground};
}

.policy-header-callout.policy-adjusted-weaker .policy-link,
.policy-header-callout.policy-adjusted-weaker .phui-icon-view {
  color: {$sh-greentext};
}

.policy-header-callout.policy-adjusted-stronger {
  background: {$sh-redbackground};
}

.policy-header-callout.policy-adjusted-stronger .policy-link,
.policy-header-callout.policy-adjusted-stronger .phui-icon-view {
  color: {$sh-redtext};
}

.policy-header-callout.policy-adjusted-different {
  background: {$sh-orangebackground};
}

.policy-header-callout.policy-adjusted-different .policy-link,
.policy-header-callout.policy-adjusted-different .phui-icon-view {
  color: {$sh-orangetext};
}

.phui-header-subheader .phui-header-status-dark {
  color: {$sh-indigotext};
  background: {$sh-indigobackground};
  margin-right: 8px;
}

.phui-header-subheader .phui-header-status-dark .phui-icon-view {
  color: {$sh-indigotext};
}

.phui-header-subheader .phui-header-status-red {
  color: {$sh-redtext};
  background: {$sh-redbackground};
}

.phui-header-subheader .phui-header-status-green {
  color: {$sh-greentext};
  background: {$sh-greenbackground};
}

.phui-header-action-links .phui-mobile-menu {
  display: none;
}

.device .phui-header-action-links .phui-mobile-menu {
  display: inline-block;
}

.phui-header-action-list {
  float: right;
}

.phui-header-action-list li {
  margin: 0 0 0 8px;
  float: right;
}

.phui-header-action-list li.phui-header-action-icon {
  height: 18px;
  width: 16px;
}

.phui-header-action-list .phui-header-action-icon .phui-icon-view {
  font-size: 16px;
  line-height: 20px;
  display: block;
}

.spaces-name {
  color: {$lightbluetext};
}

.phui-object-box .phui-header-tall .spaces-name {
  font-size: 18px;
}

.spaces-name .phui-handle {
  color: {$sh-redtext};
}

.phui-header-subheader .phui-badge-flex-view {
  display: inline;
  margin-right: 4px;
}

.phui-header-subheader .phui-badge-flex-view:after {
  display: inline;
}
