/**
 * @provides setup-issue-css
 */
.setup-issue-background {
  padding: 12px 0;
}

.setup-issue-shell {
  max-width: 760px;
  margin: 16px auto;
}

.setup-issue {
  background: #fff;
  border: 1px solid #BFCFDA;
  padding: 8px;
  border-radius: 3px;
}

.setup-issue p {
  margin: 12px 0;
}

.setup-issue table {
  width: 100%;
  border-collapse: collapse;
  border: 1px solid #BFCFDA;
}

.setup-issue table th {
  text-align: right;
  width: 30%;
  background: #F8F9FC;
  border: 1px solid #BFCFDA;
  padding: 8px;
}

.setup-issue table td {
  border: 1px solid #BFCFDA;
  padding: 8px;
}

.setup-issue pre {
  width: auto;
  border: 1px solid #BFCFDA;
  padding: 10px 2%;
  background: #F8F9FC;
  overflow-x: auto;
}

.setup-issue tt {
  color: black;
  background: #EBECEE;
  padding: 2px 4px;
}

.setup-issue em {
  font-weight: bold;
}

.setup-issue-instructions {
  font-size: 14px;
  padding: 12px 0;
  line-height: 20px;
  background: #fff;
  border-bottom: 1px solid #BFCFDA;
  margin: 0 12px;
}

.setup-fatal .setup-issue-instructions {
  color: #c0392b;
  background: #f4dddb;
  padding: 12px;
  margin: 0 0 12px;
  border-bottom: 1px solid #c0392b;
}

.setup-issue-name {
  color: #464C5C;
  padding: 4px 8px 12px;
  border-bottom: 1px solid #BFCFDA;
  font-size: 15px;
  font-weight: bold;
}

.setup-issue-tail {
  margin-top: 12px;
}

.setup-issue-status {
  margin: 12px 4px 0;
  padding: 12px;
  background: #FDF5D4;
  color: #bc7837;
  border: 1px solid #bc7837;
  border-radius: 3px;
}

.setup-issue-actions {
  padding: 8px 12px;
  border-top: 1px solid #dfdfdf;
  background-color: #f7f7f7;
  border-bottom-right-radius: 3px;
  border-bottom-left-radius: 3px;
  overflow: hidden;
  margin: 0 -8px -8px -8px;
}

.setup-issue-next {
  padding: 12px;
  border: 1px solid #BFCFDA;
  background: #daeaf3;
  text-align: center;
  font-size: 16px;
  margin: 12px 0;
  color: #2980b9;
  border-radius: 3px;
}

.setup-issue-config {
  padding: 8px 12px;
}

.setup-issue ul {
  width: 90%;
  margin: 1em auto;
  list-style: circle;
}

.setup-issue-debug {
  margin-top: 8px;
  padding: 4px;
  text-align: right;
  color: #74777D;
}
