<?php

// NOTE: This migration moved existing symbols from Arcanist Projects to
// Repositories. It stopped running cleanly about two months later, after
// Spaces were introduced. Since this data is not important and can be
// trivially regenerated, just stop running the migration. See T8691.
