<?php

// This was a complex migration that modernized old search queries. It was
// removed after giving installs a year to perform it because it would be
// unreasonably complex to maintain and users can easily recreate queries.
