/**
 * @provides phui-feed-story-css
 */

.phui-object-box .phui-box.phui-feed-story {
  border-bottom: 1px solid {$thinblueborder};
}

.phui-object-box .phui-box.phui-feed-story:last-child {
  border: none;
}

.phui-feed-story-head .phui-feed-story-actor-image {
  width: 35px;
  height: 35px;
  background-size: 35px;
  float: left;
  margin-right: 8px;
  box-shadow: {$borderinset};
  border-radius: 3px;
}

.phui-feed-story-head {
  padding: 12px 4px;
  overflow: hidden;
  color: {$greytext};
  line-height: 16px;
  word-break: break-word;
}

.phui-feed-story-head .phui-link-person {
  color: {$darkgreytext};
  font-weight: bold;
}

.phui-feed-story-body {
  margin: 4px 4px 8px;
  color: {$darkgreytext};
  word-break: break-word;
  max-height: 300px;
  overflow: hidden;
}

.phui-feed-story-foot {
  font-size: {$smallerfontsize};
  padding: 8px 4px 12px;
}

.phui-feed-story-foot,
.phui-feed-story-foot a {
  color: {$greytext};
}

.phui-feed-story-foot .phui-icon-view {
  margin-right: 5px;
}

.phui-feed-story-bigtext-post {
  line-height: 18px;
  color: {$darkgreytext};
}

.phui-feed-story-bigtext-post h3 {
  font-size: {$biggestfontsize};
  line-height: 18px;
  color: {$darkgreytext};
  margin: 0 0 5px 0;
}

.phui-feed-token-bar {
  margin-top: 8px;
  border-top: 1px solid #e7e7e7;
  padding-top: 8px;
}

.phui-feed-token-bar .phui-icon-view {
  margin-right: 2px;
  display: inline-block;
}

.phui-feed-story-action-list {
  float: right;
  padding-top: 4px;
}

.phui-feed-story-action-item {
  float: right;
  padding-left: 2px;
  height: 18px;
  width: 18px;
  font-size: 18px;
}

.phui-feed-story-action-list .phui-icon-view {
  display: block;
}
