/**
 * @provides phui-box-css
 */

.phui-box-border {
  border: 1px solid {$lightblueborder};
  background-color: #fff;
  border-radius: 3px;
}

.phui-box.focus {
  box-shadow: 0 0 5px 5px rgba(255, 255, 0, 0.90);
}

.phui-box-grey {
  background-color: rgba(71,87,120,0.1);
  border-radius: 3px;
}

.phui-box-blue {
  background-color: {$bluebackground};
  border-radius: 3px;
}
