<?php

final class PhortuneErrCode extends PhortuneConstants {

  // NOTE: These constants also appear in Javascript.

  const ERR_CC_INVALID_NUMBER = 'cc:invalid:number';
  const ERR_CC_INVALID_CVC    = 'cc:invalid:cvc';
  const ERR_CC_INVALID_EXPIRY = 'cc:invalid:expiry';

}
