/**
 * @provides herald-css
 */

.herald-action-table,
.herald-condition-table {
  margin-top: 8px;
}

.herald-condition-table select {
  width: 160px;
}

.device-phone .herald-condition-table select {
  width: 90px;
}

.herald-action-table td,
.herald-condition-table td {
  padding: 2px 8px 2px 0;
  vertical-align: middle;
}

.herald-action-table td.remove-column,
.herald-condition-table td.remove-column {
  padding: 2px 0 2px 4px;
}

.herald-condition-table td.value {
  width: 100%;
}

.herald-action-table td input,
.herald-condition-table td input {
  width: 95%;
  max-width: 460px;
  padding: 2px 4px;
}

.herald-action-table td.target {
  width: 100%;
}

.herald-list-description {
  color: {$darkgreytext};
  padding: 8px 0;
}

.herald-list-icon {
  margin-left: 12px;
  margin-right: 2px;
}

.herald-list-item {
  padding-bottom: 4px;
  color: {$greytext};
}
