<?php

// NOTE: This migration moved existing symbol configuration from Arcanist
// Projects to Repositories. Like "20150503.repositorysymbols.2.php", it stopped
// running cleanly about two months later, after Spaces were introduced. Since
// this data is easy to rebuild, just stop running the migration. See T8691 and
// T8697.
