/**
 * @provides phui-icon-set-selector-css
 */

button.icon-button {
  background: #f7f7f7;
  border: 1px solid {$lightblueborder};
  position: relative;
  width: 16px;
  height: 16px;
  padding: 12px;
  margin: 4px;
  text-shadow: none;
  box-shadow: none;
  box-sizing: content-box;
}

.icon-grid {
  text-align: center;
}

.icon-icon + .icon-icon {
  margin-left: 4px;
}

button.icon-button.selected {
  background: {$bluebackground};
  border: 1px solid {$blueborder};
}
