/**
 * @provides phui-text-css
 */


/* Styles */
.phui-text-bold {
  font-weight: bold;
}

.phui-text-uppercase {
  text-transform: uppercase;
}

.phui-text-strike {
  text-decoration: line-through;
}


/* Colors */
.phui-text-red {
  color: {$red};
}

.phui-text-orange {
  color: {$orange};
}

.phui-text-yellow {
  color: {$yellow};
}

.phui-text-green {
  color: {$green}
}

.phui-text-blue {
  color: {$blue};
}

.phui-text-indigo {
  color: {$indigo};
}

.phui-text-violet {
  color: {$violet};
}

.phui-text-white {
  color: #fff;
}

.phui-text-black {
  color: #333;
}
