/**
 * @provides releeph-core
 */


/* Colors: match differential colors */

.releeph-request-comment {
  border-color: #ddd;
}

.releeph-request-comment-pusher {
  background:   #8DEE8D;
  border-color: #096;
}

.releeph-request-comment-pusher div {
  background:   #8DEE8D;
}

/* The diff size bar */

.diff-bar {
  border: 0px;
}

.diff-bar div {
  width: 100px;
  border: 1px solid;
  border-top-color: #A4A4A4;
  border-right-color: #BBB;
  border-bottom-color: #D5D5D5;
  border-left-color: #BBB;
  background: white;
  float: left;
  margin-right: 1em;
}

.diff-bar div div {
  height: 10px;
}

.diff-bar span {
  color: #555;
}
