/**
 * @provides differential-revision-list-css
 */


.revision-age-fresh,
.revision-age-stale,
.revision-age-old {
  margin: -4px -8px;
  padding: 4px 8px;
}

.revision-age-fresh {
  background: #dff1d8;
}

.revision-age-stale {
  background: #fcf9e2;
}

.revision-age-old {
  background: #f3dede;
}
