/**
 * @provides inline-comment-summary-css
 */

.transaction-comment + .phabricator-inline-summary-table {
  margin-top: 8px;
}

.phabricator-inline-summary-table {
  border-collapse: collapse;
  border-style: hidden;
  width: 100%;
  margin-top: -16px;
}

.phabricator-inline-summary-table .inline-comment-summary-table-header {
  font-weight: bold;
  padding: 16px 1px 8px;
  background: #fff;
  color: #000;
  border-bottom: 1px solid {$thinblueborder};
}

.phabricator-inline-summary-table .inline-table-dolumn  {
  padding: 4px 8px;
  white-space: nowrap;
  color: {$darkbluetext};
  background: white;
}

.phabricator-inline-summary-table td.inline-line-number {
  padding: 0;
  width: 100px;
  white-space: nowrap;
  background: #F8F9FC;
  font-family: "Menlo", "Consolas", monospace;
  font-size: {$smallestfontsize};
  color: {$bluetext};
  border-right: 1px solid {$thinblueborder};
}

.phabricator-inline-summary-table td.inline-which-diff {
  color: {$greytext};
  width: 120px;
}

.phabricator-inline-summary-table td.inline-summary-content {
  vertical-align: top;
  white-space: normal;
}

/* NOTE: These two rules provide a larger hit target for clicking line numbers
   (by letting you click the entire cell) and a visual indicator that you're on
   target (by highlighting the entire cell). */

.phabricator-inline-summary-table td.inline-line-number a.num {
  padding: 4px 8px;
  display: block;
  color: {$bluetext};
}

.phabricator-inline-summary-table td.inline-line-number:hover a.num {
  background: rgba(150,150,150,.1);
  text-decoration: none;
  color: {$darkbluetext};
}

.phui-timeline-core-content .phabricator-inline-summary:first-child {
  border: none;
  margin-top: 0;
  padding-top: 0;
}
