/**
 * @provides maniphest-report-css
 */

table.aphront-table-view tr.aggregate,
table.aphront-table-view tr.alt-aggregate {
  font-size: {$normalfontsize};
  font-weight: bold;
}

table.aphront-table-view tr.month,
table.aphront-table-view tr.alt-month {
  background: {$backdrop};
  font-weight: bold;
}

span.red {
  color: {$red};
  font-weight: bold;
}

span.green {
  color: {$green};
  font-weight: bold;
}
