/**
 * @provides auth-css
 */

.phabricator-login-buttons {
  max-width: 508px;
  margin: 16px auto;
}

.phabricator-login-buttons .phabricator-login-button .button {
  width: 246px;
}

.device-desktop .phabricator-login-buttons .aphront-multi-column-column-last {
  text-align: right;
}

.device .phabricator-login-buttons {
  text-align: center;
}

.phabricator-link-button {
  text-align: center;
}

.auth-account-view {
  background-color: {$lightbluebackground};
  border: 1px solid {$thinblueborder};
  border-radius: 3px;
  min-height: 50px;
  position: relative;
  padding: 4px 4px 4px 64px;
}

.auth-account-view-profile-image {
  width: 50px;
  height: 50px;
  top: 6px;
  left: 6px;

  background-repeat: no-repeat;
  background-size: 100%;
  box-shadow: {$borderinset};
  position: absolute;
}

.auth-account-view-name {
  font-weight: bold;
}

.auth-account-view-provider-name {
  color: {$lightgreytext};
}

.auth-account-view-account-uri {
  word-break: break-word;
}
