/**
 * @provides phui-form-css
 */

select,
textarea,
input[type="text"],
input[type="password"],
input[type="datetime"],
input[type="datetime-local"],
input[type="date"],
input[type="month"],
input[type="time"],
input[type="week"],
input[type="number"],
input[type="email"],
input[type="url"],
input[type="search"],
input[type="tel"],
input[type="color"],
div.jx-tokenizer-container {
  display: inline-block;
  height: 28px;
  line-height: 18px;
  color: #333;
  vertical-align: middle;
  font: {$basefont};
  -webkit-font-smoothing: antialiased;
}

textarea,
input[type="text"],
input[type="password"],
input[type="datetime"],
input[type="datetime-local"],
input[type="date"],
input[type="month"],
input[type="time"],
input[type="week"],
input[type="number"],
input[type="email"],
input[type="url"],
input[type="search"],
input[type="tel"],
input[type="color"],
div.jx-tokenizer-container {
  padding: 4px 6px;
  background-color: #ffffff;
  border: 1px solid {$blueborder};

  -webkit-box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075);
  -moz-box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075);
  box-shadow: inset 0 1px 1px rgba(0, 0, 0, 0.075);

  -webkit-transition: border linear .05s, box-shadow linear .05s;
  -moz-transition: border linear .05s, box-shadow linear .05s;
  -o-transition: border linear .05s, box-shadow linear .05s;
  transition: border linear .05s, box-shadow linear .05s;

  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;

  /* iOS Safari */
  -webkit-appearance: none;
  border-radius: 0;
}

textarea:focus,
input[type="text"]:focus,
input[type="password"]:focus,
input[type="datetime"]:focus,
input[type="datetime-local"]:focus,
input[type="date"]:focus,
input[type="month"]:focus,
input[type="time"]:focus,
input[type="week"]:focus,
input[type="number"]:focus,
input[type="email"]:focus,
input[type="url"]:focus,
input[type="search"]:focus,
input[type="tel"]:focus,
input[type="color"]:focus,
div.jx-tokenizer-container-focused {
  border-color: rgba(82, 168, 236, 0.8);
  outline: 0;
  /* IE6-9 */

  -webkit-box-shadow:
    inset 0 1px 1px rgba(0,0,0,.075),
    0 0 8px rgba(82,168,236,.6);
  -moz-box-shadow:
    inset 0 1px 1px rgba(0,0,0,.075),
    0 0 8px rgba(82,168,236,.6);
  box-shadow:
    inset 0 1px 1px rgba(0,0,0,.075),
    0 0 8px rgba(82,168,236,.6);
}
input[type="radio"],
input[type="checkbox"] {
  margin: 4px 0 0;
  margin-top: 1px \9;
  /* IE8-9 */
  line-height: normal;
}

select {
  height: 24px;
  line-height: 24px;
  border: 1px solid {$lightgreyborder};
  background-color: #ffffff;
  min-width: 140px;
}

select[multiple],
select[size] {
  height: auto;
}

select:focus,
input[type="file"]:focus,
input[type="radio"]:focus,
input[type="checkbox"]:focus {
  outline: thin dotted #333;
  outline: 5px auto -webkit-focus-ring-color;
  outline-offset: -2px;
}

input:-moz-placeholder,
textarea:-moz-placeholder {
  color: {$lightgreytext};
}

input:-ms-input-placeholder,
textarea:-ms-input-placeholder {
  color: {$lightgreytext};
}

input::-webkit-input-placeholder,
textarea::-webkit-input-placeholder {
  color: {$lightgreytext};
}

select[disabled],
input[disabled],
textarea[disabled],
.disabled-control {
  opacity: 0.5;
}

.aphront-space-select-control-knob {
  margin: 2px 8px 2px 0;
}

.device .aphront-space-select-control-knob {
  margin-bottom: 8px;
}
