/**
 * @provides aphront-dark-console-css
 */

.dark-console {
  background:     #444444;
  color:          #eeeeee;
  width:          100%;
  font-family:    "Verdana";
  font-size:      11px;
  position:       relative;
}

.dark-console a {
  color: #cccccc;
}

.dark-console-requests,
.dark-console-tabs {
  position: absolute;
  overflow-y: auto;
  top: 0;
  left: 0;
  bottom: 0;
  width: 15%;
  padding: 8px 0;
}

.dark-console-requests,
.dark-console-tabs,
.dark-console-panel,
.dark-console-load {
  border-left: 1px solid #111111;
  box-shadow: -2px 0px 2px rgba(0, 0, 0, 0.25);
}

.dark-console-requests {
  background: #222222;
}

.dark-console-tabs {
  background: #333333;
  left: 15%;
}

.dark-console-panel,
.dark-console-load {
  position: relative;
  min-height: 320px;
}

.dark-console-panel {
  margin-left: 30%;
  background: #444444;
}

.dark-console-requests a.dark-console-request,
.dark-console-tabs a.dark-console-tab {
  display: block;
  padding: 6px;
  overflow: hidden;
  background: #444444;
  margin: 3px 0;
  border-color: {$greytext};
  border-width: 1px 0;
  border-style: solid;
}

.dark-console-requests a.dark-selected,
.dark-console-tabs a.dark-selected {
  background: #0066aa;
}

.dark-console-requests a.dark-console-request:hover,
.dark-console-tabs a.dark-console-tab:hover {
  background: #1188cc;
}

.dark-console-tabs a.dark-console-tab {
  text-align: right;
}

.dark-console-load {
  background-image: url(/rsrc/image/darkload.gif);
  background-position: center center;
  background-repeat: no-repeat;
  background-color: #000;
  margin-left: 15%;
}

.dark-console .aphront-table-view {
  font-size: 11px;
  background: {$lightgreytext};
  color: #eeeeee;
  width: 100%;
  border-color: #333333;
  margin: 8px 0;
}

.dark-console .aphront-table-view th {
  text-shadow: none;
  font-family: "Verdana";
  font-size: 11px;
  background: #333333;
  color: #ffffff;
}

.dark-console .aphront-table-view td {
  font-size: 11px;
}

.dark-console .aphront-table-view td.header {
  background: #444444;
  color: #ffffff;
  min-width: 200px;
}

.dark-console .aphront-table-view tr.alt {
  background: {$greytext};
}

.dark-console .aphront-table-view tr.highlight {
  background: #001A66;
}

.dark-console .aphront-table-view tr.no-data td {
  color: #dddddd;
}

.dark-console-panel-core {
  padding: 12px;
}

.explain-sev-1 {
  color: #33ff33;
}

.explain-sev-2 {
  color: #99ff33;
}

.explain-sev-3 {
  color: #ccff33;
}

.explain-sev-4 {
  color: #ffff33;
}

.explain-sev-5 {
  color: #ffcc33;
}

.explain-sev-6 {
  color: #ffffff;
  font-weight: bold;
  background: #aa0000;
  padding: 0 1em;
  border: 2px solid #ffff00;
}

.explain-sev-7 {
  color: #aaaaaa;
}

.dark-console-panel-header {
  padding: 8px 4px 0;
}

.dark-console-panel-header h1 {
  font-size: 15px;
}

.dark-console-panel-header .button {
  margin-top: .5em;
  float: right;
}

.dark-console-panel a.bright-link {
  color: #00cfff;
  font-weight: bold;
}

.dark-console iframe {
  width: 98%;
  margin: .5em 1%;
  height: 450px;
  border: 0;
}

.dark-console-no-content {
  padding: 1.5em 2em;
  font-style: italic;
}

.dark-console-request-log {
  border-bottom:  2px solid #000000;
}

.dark-console-panel-RequestLog {
  max-height: 10em;
  overflow: auto;
}

.dark-console-panel-request-log-separator {
    background-color: #e8e8e8;
    border-bottom: 1px solid #b7b7b7;
    border-top: 1px solid #b7b7b7;
    height: 2px;
}

.dark-console-panel-ErrorLog {
  max-height:       500px;
  overflow:         auto;
}

.dark-console-panel-error-details {
  display: none;
}
