/**
 * @provides harbormaster-css
 */

.harbormaster-artifact-io {
  margin: 4px 0 4px 8px;
  padding: 8px;
}

.harbormaster-artifact-summary-header {
  font-weight: bold;
  margin-bottom: 2px;
  color: {$darkbluetext};
}

.harbormaster-empty-logs-are-hidden {
  background: {$lightyellow};
  border: 1px solid {$yellow};
  text-align: center;
  padding: 12px;
  color: {$darkgreytext};
}
