/**
 * @provides aphront-tooltip-css
 */

.jx-tooltip-container {
  position: absolute;
  padding: 5px;
}

.jx-tooltip-inner {
  position: relative;
  background: rgba(0,0,0, .9);
  border-radius: 3px;
}

.jx-tooltip {
  color: #f9f9f9;
  font-size: {$normalfontsize};
  padding: 6px 8px;
  overflow: hidden;
  white-space: pre-wrap;
}

.jx-tooltip:after {
  border: solid transparent;
  content: " ";
  height: 0;
  width: 0;
  position: absolute;
  pointer-events: none;
  border-color: rgba(0, 0, 0, 0);
  border-width: 5px;
}

.jx-tooltip-align-E {
  margin-left: 5px;
}

.jx-tooltip-align-E .jx-tooltip:after {
  margin-top: -5px;
  border-right-color: #000;
  right: 100%;
  top: 50%;
}

.jx-tooltip-align-E {
  margin-right: 5px;
}

.jx-tooltip-align-W .jx-tooltip:after {
  margin-top: -5px;
  border-left-color: #000;
  left: 100%;
  top: 50%;
}

.jx-tooltip-align-N {
  margin-bottom: 5px;
}

.jx-tooltip-align-N .jx-tooltip:after {
  margin-left: -5px;
  border-top-color: #000;
  top: 100%;
  left: 50%;
}

.jx-tooltip-align-N {
  margin-top: 5px;
}

.jx-tooltip-align-S .jx-tooltip:after {
  margin-left: -5px;
  border-bottom-color: #000;
  bottom: 100%;
  left: 50%;
}
