/**
 * @provides phui-calendar-list-css
 */

.phui-calendar-list-container {
  width: 300px;
}

.device-phone .phui-calendar-list-container {
  width: auto;
}

.phui-calendar-list-container .phui-object-box {
  border-bottom: none;
  margin: 0;
}

.phui-calendar-list-container:last-child .phui-object-box {
  border-bottom: 1px solid {$blueborder};
}

.phui-calendar-list-container .phui-object-box .phui-header-shell h1 {
  padding: 6px 0;
}

.phui-calendar-list {
  padding: 16px 12px;
}

.phui-calendar-list-item {
  position: relative;
  line-height: 18px;
  height: 18px;
}

.phui-calendar-list-item a {
  display: block;
  width: auto;
  min-height: 18px;
}

.phui-calendar-list-item-icon {
  position: absolute;
  left: 0;
  top: 3px;
}

.phui-calendar-list-title {
 width: 200px;
 overflow: hidden;
 text-overflow: ellipsis;
 white-space: nowrap;
 position: absolute;
 left: 20px;
 top: 0;
}

.phui-calendar-viewer-invited .phui-calendar-list-title {
  font-weight: bold;
}

.phui-calendar-list-item .phui-calendar-list-time {
  position: absolute;
  top: 0;
  width: 60px;
  right: 0;
  color: {$lightgreytext};
  text-align: right;
  line-height: 20px;
}

.phui-calendar-list-item-empty {
  color: {$lightgreytext};
}
