/**
 * @provides phabricator-side-menu-view-css
 */

.phabricator-basic-nav .phabricator-side-menu .phui-list-item-view {
  display: block;
  white-space: nowrap;
  text-decoration: none;
  font-size: 13px;
  -webkit-font-smoothing: antialiased;
}

.phabricator-basic-nav .phabricator-side-menu .phui-list-item-href {
  display: block;
  padding: 6px 8px 6px 24px;
  color: {$darkbluetext};
  border-top-right-radius: 3px;
  border-bottom-right-radius: 3px;
}

.phabricator-basic-nav .phabricator-side-menu .phui-list-item-selected {
  background-color: rgba(0,0,0,.05);
  border-left: 4px solid {$sky};
  border-top-right-radius: 3px;
  border-bottom-right-radius: 3px;
  font-weight: bold;
}

.device-desktop .phabricator-basic-nav .phabricator-side-menu
  .phui-list-item-selected
  a.phui-list-item-href:hover {
  background-color: rgba(0,0,0,.05);
}

.phabricator-basic-nav .phabricator-side-menu .phui-list-item-selected
  .phui-list-item-href {
  padding-left: 20px;
}

.phabricator-basic-nav .phabricator-side-menu .phui-list-item-type-label {
  padding: 6px 8px 4px 12px;
  color: {$darkbluetext};
  text-transform: uppercase;
  font-size: 12px;
  font-weight: bold;
  border-style: solid;
}

.device-desktop .phabricator-basic-nav .phabricator-side-menu
  a.phui-list-item-href:hover {
  text-decoration: none;
  background-color: rgba(0,0,0,.07);
}
