<?php
/**
 * Mime Type: text/html
 * Mime Type: application/html+xml
 *
 * @author Nathan Good <me@nategood.com>
 */

namespace Httpful\Handlers;

class XHtmlHandler extends MimeHandlerAdapter
{
    // @todo add html specific parsing
    // see DomDocument::load http://docs.php.net/manual/en/domdocument.loadhtml.php
}