/**
 * @provides phabricator-zindex-css
 */

.keyboard-focus-focus-reticle {
  z-index: 1;
}

.device .phabricator-action-list-view.phabricator-action-list-toggle,
.device-desktop .phui-document-content
  .phabricator-action-list-view.phabricator-action-list-toggle {
  z-index: 1;
}

.device-desktop .phui-timeline-minor-event .phui-timeline-image {
  z-index: 2;
}

.differential-reticle {
  z-index: 2;
}

.differential-changeset {
  z-index: 2;
}

.pholio-new-inline-comment {
  z-index: 2;
}

.slowvote-bar {
  z-index: 2;
}

div.phui-calendar-day-event {
  z-index: 2;
}

.slowvote-above-the-bar {
  z-index: 3;
}

.phui-timeline-icon-fill {
  z-index: 3;
}

.phui-crumbs-view {
  z-index: 3;
}

.phabricator-nav-local {
  z-index: 4;
}

.project-board-header {
  z-index: 4;
}

.conpherence-layout .conpherence-no-threads {
  z-index: 4;
}

.conpherence-menu-pane {
  z-index: 4;
}

.phabricator-nav-drag {
  z-index: 4;
}

.setup-warning-callout {
  z-index: 5;
}

.loading .messages-loading-mask,
.loading .widgets-loading-mask {
  z-index: 5;
}

.dark-console {
  z-index: 5;
}

.drag-dragging {
  z-index: 5;
}

.phui-calendar-date-number  {
  z-index: 5;
}

.phabricator-main-menu {
  z-index: 6;
}

.aphront-developer-error-callout {
  z-index: 6;
}

.conpherence-durable-column {
  z-index: 7;
}

.jx-scrollbar-bar {
  z-index: 8;
}

.differential-haunt-mode-1 .differential-add-comment-panel,
.differential-haunt-mode-2 .differential-add-comment-panel {
  z-index: 8;
}

.device-desktop .phabricator-notification-menu {
  z-index: 9;
}

.jx-mask {
  z-index: 10;
}

.jx-notification-container {
  z-index: 11;
}

.phabricator-global-upload-instructions {
  z-index: 11;
}

.lightbox-attachment {
  z-index: 12;
}

div.jx-typeahead-results {
  z-index: 13;
}

.jx-client-dialog {
  z-index: 14;
}

.fancy-datepicker {
  z-index: 15;
}

.jx-hovercard-container {
  z-index: 17;
}

.pholio-device-lightbox {
  z-index: 20;
}

.phuix-autocomplete {
  z-index: 21;
}

.phuix-dropdown-menu {
  z-index: 32;
}

.busy {
  z-index: 40;
}

.remarkup-control-fullscreen-mode {
  z-index: 50;
}

.jx-tooltip-container {
  z-index: 51;
}

.audible .aural-only {
  z-index: 100;
}
