/**
 * @provides sprite-menu-css
 * @generated
 */

.sprite-menu {
  background-image: url(/rsrc/image/sprite-menu.png);
  background-repeat: no-repeat;
}

@media
only screen and (min-device-pixel-ratio: 1.5),
only screen and (-webkit-min-device-pixel-ratio: 1.5),
only screen and (min-resolution: 1.5dppx) {
  .sprite-menu {
    background-image: url(/rsrc/image/sprite-menu-X2.png);
    background-size: 97px 123px;
  }
}


.dark-logo {
  background-position: 0px 0px;
}

.dark-eye {
  background-position: 0px -82px;
}

.light-logo {
  background-position: 0px -41px;
}

.light-eye {
  background-position: -41px -82px;
}
