/**
 * @provides policy-css
 */

.policy-capability-explanation .phui-icon-view {
  display: inline-block;
  vertical-align: text-top;
}

.policy-capability-explanation {
  margin: 8px 0 0;
}

.policy-capability-explanation a {
  line-height: 14px;
  margin-left: 4px;
  color: {$bluetext};
}
