/**
 * @provides font-aleo
 * @requires phui-fontkit-css
 */

@font-face {
    font-family: 'Aleo';
    font-weight: bold;
    font-style: normal;
    src: url(/rsrc/externals/font/aleo/aleo-bold.eot);
    src: url(/rsrc/externals/font/aleo/aleo-bold.eot?#iefix)
          format('embedded-opentype'),
         url(/rsrc/externals/font/aleo/aleo-bold.woff2)
          format('woff2'),
         url(/rsrc/externals/font/aleo/aleo-bold.woff)
          format('woff'),
         url(/rsrc/externals/font/aleo/aleo-bold.ttf)
          format('truetype'),
         url(/rsrc/externals/font/aleo/aleo-bold.svg#aleo-bold)
          format('svg');

}

@font-face {
    font-family: 'Aleo';
    font-weight: normal;
    font-style: normal;
    src: url(/rsrc/externals/font/aleo/aleo-regular.eot);
    src: url(/rsrc/externals/font/aleo/aleo-regular.eot?#iefix)
          format('embedded-opentype'),
         url(/rsrc/externals/font/aleo/aleo-regular.woff2)
          format('woff2'),
         url(/rsrc/externals/font/aleo/aleo-regular.woff)
          format('woff'),
         url(/rsrc/externals/font/aleo/aleo-regular.ttf)
          format('truetype'),
         url(/rsrc/externals/font/aleo/aleo-regular.svg#aleo-regular)
          format('svg');

}
