<?php

/**
 * Allow infrastructure to automagically create "mentioned" transactions
 * - actually TYPE_EDGE transactions that add "mentioned" edges - to the
 * implementing object.
 */
interface PhabricatorMentionableInterface {}
