/**
 * @provides phabricator-source-code-view-css
 */

.phabricator-source-code-container {
  overflow-x: auto;
  overflow-y: hidden;
  border: 1px solid {$sh-lightyellowborder};
  border-radius: 3px;
  background-color: #FFFEF5;
}

.phui-object-item .phabricator-source-code-container {
  margin-left: 8px;
}

.phabricator-source-code-view tr:first-child * {
  padding-top: 8px;
}

.phabricator-source-code-view tr:last-child * {
  padding-bottom: 8px;
}

.phabricator-source-code {
  white-space: pre-wrap;
  padding: 2px 8px 1px;
  width: 100%;
}

.phabricator-source-line {
  background-color: {$sh-yellowbackground};
  text-align: right;
  padding: 2px 6px 1px 12px;
  border-right: 1px solid {$sh-lightyellowborder};
  color: {$sh-yellowtext};

  /* When the user selects rows of source, don't visibly select the line
     numbers beside them. We use JS to strip the line numbers out when the user
     copies the text. */
  -moz-user-select: -moz-none;
  -khtml-user-select: none;
  -webkit-user-select: none;
  -ms-user-select: none;
  user-select: none;
  white-space: nowrap;
}

th.phabricator-source-line a {
  color: {$darkbluetext};
}

th.phabricator-source-line:hover {
  background: {$sh-lightyellowborder};
  cursor: pointer;
}

.phabricator-source-highlight {
  background: {$sh-yellowbackground};
}

.phabricator-source-code-summary {
  padding-bottom: 8px;
}

/* If a Paste has enormously long lines, truncate them in the summary on the
   list page. They'll be fully visible on the Paste itself. */
.phabricator-source-code-summary .phabricator-source-code-container {
  overflow-x: hidden;
}

.phabricator-source-code-summary .phabricator-source-code {
  white-space: nowrap;
}
