/**
 * @provides config-welcome-css
 */

.phui-document-content .config-welcome-box .phabricator-remarkup {
  padding: 0 16px 16px;
  position: relative;
}

.config-welcome-box {
  padding: 8px 0;
}

.config-welcome-content {
  margin-left: 44px;
}

.config-welcome-box .config-welcome-icon {
  width: 32px;
  float: left;
  text-align: center;
  margin-left: 16px;
}
