/**
 * @provides lightbox-attachment-css
 */


.lightbox-attached {
  overflow: hidden;
}

.lightbox-attachment {
  position: fixed;
  top: 0;
  left: 0;
  bottom: 0;
  right: 0;
  overflow-y: auto;
}

.lightbox-attachment img {
  margin: 3% auto 0;
  max-height: 90%;
  max-width: 90%;
}

.lightbox-attachment .loading {
  position: absolute;
  top: -9999px;
}

.lightbox-attachment .attachment-name {
  width: 100%;
  color: #F2F2F2;
  line-height: 30px;
  text-align: center;
}

.lightbox-attachment .lightbox-status {
  background: #010101;
  color: #F2F2F2;
  line-height: 30px;
  position: fixed;
  bottom: 0px;
  width: 100%;
}

.lightbox-attachment .lightbox-status .lightbox-status-txt {
  padding: 0px 0px 0px 20px;
}

.lightbox-attachment .lightbox-status .lightbox-download {
  padding: 0px 20px 0px 0px;
  float: right;
}

.lightbox-attachment .lightbox-status .lightbox-download
.lightbox-download-form {
  display: inline;
}
.lightbox-attachment .lightbox-status .lightbox-download
.lightbox-download-form button {
  border: 0;
  background: #010101;
}
.lightbox-attachment .lightbox-status .lightbox-download
.lightbox-download-form button:hover {
  background: #333;
}

.lightbox-attachment .lightbox-close {
  top: 22px;
  right: 20px;
  position: fixed;
  display: block;
  height: 26px;
  width: 26px;
  background: url('/rsrc/image/icon/lightbox/close-2.png');
}
.lightbox-attachment .lightbox-close:hover {
  background: url('/rsrc/image/icon/lightbox/close-hover-2.png');
}

.lightbox-attachment .lightbox-left {
  top: 46%;
  left: 20px;
  position: fixed;
  display: block;
  height: 38px;
  width: 21px;
  background: url('/rsrc/image/icon/lightbox/left-arrow-2.png');
}
.lightbox-attachment .lightbox-left:hover {
  background: url('/rsrc/image/icon/lightbox/left-arrow-hover-2.png');
}

.lightbox-attachment .lightbox-right {
  top: 46%;
  right: 20px;
  position: fixed;
  display: block;
  height: 38px;
  width: 21px;
  background: url('/rsrc/image/icon/lightbox/right-arrow-2.png');
}
.lightbox-attachment .lightbox-right:hover {
  background: url('/rsrc/image/icon/lightbox/right-arrow-hover-2.png');
}
