/**
 * @provides maniphest-batch-editor
 */
.maniphest-batch-actions-table {
  width: 100%;
  margin: 12px 0;
}

.maniphest-batch-actions-table td {
  padding: 4px 8px;
  vertical-align: middle;
}

.batch-editor-input {
  width: 100%;
  text-align: left;
}
