/**
 * @provides phui-info-panel-css
 */

.phui-info-panel .phui-object-box .phui-header-has-image {
  padding: 2px 0 0 2px;
}

.phui-info-panel .phui-object-box .phui-header-image {
  margin: 0 8px 0 0;
}

.phui-info-panel-table {
  border-collapse: collapse;
  border-style: hidden;
  width: 100%;
}

.phui-info-panel-table td,
.phui-info-panel-table th {
  border: 1px solid {$thinblueborder};
}

.phui-info-panel-table-cell {
  padding: 8px;
}

.phui-info-panel-number,
.phui-info-panel-number a {
  font-size: 30px;
  font-weight: bold;
  color: {$lightgreytext};
  -webkit-font-smoothing: antialiased;
}

.phui-info-panel-text,
.phui-info-panel-text a {
  color: {$lightgreytext};
}

.phui-info-panel-number a:hover,
.phui-info-panel-text a:hover {
  color: {$greytext};
  text-decoration: none;
}

.phui-info-panel-progress {
  background: {$green};
  height: 6px;
}
