/**
 * @provides phui-remarkup-preview-css
 */

.phui-remarkup-preview .phui-object-box {
  padding: 0;
}

.phui-preview-header {
  font-size: {$smallerfontsize};
  font-weight: bold;
  color: {$bluetext};
  text-transform: uppercase;
  padding: 8px;
  background-color: {$lightgreybackground};
  border-bottom: 1px solid {$thinblueborder};
  display: block;
  border-top-left-radius: 3px;
  border-top-right-radius: 3px;
}

.phui-preview-body {
  padding: 16px;
}

.phui-document-inner .phui-preview-header {
  padding: 16px 0 0 16px;
}

.device-phone .phui-panel-preview {
  display: none;
}

.phui-preview-loading-text {
  color: {$lightgreytext};
}

/**
 * TODO: Classes implementing PhabricatorMarkupInterface are of differing
 * mindsets about whether output should be wrapped in a `phabricator-remarkup`
 * <div /> or not. It should probably move to the Engine in all cases, but
 * until we do that get rid of the extra spacing generated by the inner div.
 */
body .phui-remarkup-preview .phabricator-remarkup .phabricator-remarkup {
  padding: 0;
  margin: 0;
}
