/**
 * @provides maniphest-task-edit-css
 */

.maniphest-auxiliary-header {
  font-weight: bold;
  font-size: {$biggerfontsize};
  color: {$darkbluetext};
  border-bottom: 1px solid {$lightgreytext};
  padding: 12px 0 4px;
  margin-bottom: 4px;
}
