/**
 * @provides phui-workboard-view-css
 */

.phui-workboard-view {
  width: 100%;
}

.device-desktop .phui-workboard-view-shadow {
  overflow-x: auto;
  position: absolute;
  top: 96px;
  bottom: 0;
  left: 0;
  right: 0;
}

.device-desktop .page-has-warning .phui-workboard-view-shadow {
  top: 132px;
}

.device-desktop.with-durable-column .phui-workboard-view-shadow {
  right: 300px;
}

.device-desktop.with-durable-margin .phui-workboard-view-shadow {
  right: 312px;
}

.phui-workboard-view-shadow::-webkit-scrollbar {
  height: 12px;
  width: 12px;
  background: rgba(200,200,200,.6);
}

.phui-workboard-view-shadow::-webkit-scrollbar-thumb {
  background: {$lightbluetext};
}

.device-desktop .project-board-wrapper .phui-workboard-view-shadow {
  left: 253px;
}

.device-desktop .phui-profile-menu-collapsed .project-board-wrapper
  .phui-workboard-view-shadow {
  left: 101px;
}

.device-desktop .phui-workboard-view .aphront-multi-column-fixed
  .aphront-multi-column-inner {
    margin-left: 0;
}

.device-tablet .project-board-wrapper {
  margin-left: 8px;
  margin-right: 8px;
}

.project-board-header {
  background-color: #fff;
  border-bottom: 1px solid {$lightblueborder};
  padding: 12px;
  position: relative;
}

.device .project-board-header {
  padding: 0;
}

.project-board-header .phui-header-shell {
  padding: 0;
  border-bottom: 1px solid {$blueborder};
}

.device .project-board-header .phui-header-shell {
  padding: 8px;
}

.project-board-header .phui-header-header {
  font-size: 18px;
}

.project-board-header .phui-header-subheader {
  display: inline-block;
  margin: 0;
  padding: 0 8px;
}

.device-phone .project-board-header .phui-header-subheader {
  display: block;
  margin: 8px 0 2px 0;
  padding: 0;
}
