/**
 * @provides config-options-css
 */

.config-option-table {
  width: 100%;
  border-collapse: collapse;
  border: 1px solid {$lightgreyborder};
  background: #fff;
}

.config-option-table th,
.config-option-table td {
  padding: 4px 12px;
  border: 1px solid {$lightgreyborder};
}

.config-option-table th {
  background: {$lightgreybackground};
  color: {$bluetext};
  text-align: right;
  white-space: nowrap;
}

.config-option-table th em,
.config-option-table td em {
  font-weight: normal;
  color: {$greytext};
}

.config-option-table td {
  color: {$darkgreytext};
  width: 100%;
  white-space: pre-wrap;
}

.config-option-table .column-labels th {
  font-weight: bold;
  color: {$bluetext};
  text-align: center;
  background: {$greybackground};
}

.config-options-current-value {
  padding: 0 8px 6px;
  white-space: pre-wrap;
}

.config-options-current-value span {
  color: {$greytext};
}

.config-options-effective-value,
.config-option-table .config-options-effective-value th {
  background: {$lightyellow};
}
