/**
 * @provides phui-list-view-css
 */

.phui-list-item-header,
.phui-list-item-header a {
  color: {$bluetext};
  font-weight: bold;
  -webkit-font-smoothing: antialiased;
}

/* - Sidenav and Actions -------------------------------------------------------

  Sidebar and Action Menus

*/

.phui-list-sidenav {
  padding: 4px 0;
}

.phui-list-sidenav .phui-list-item-type-label .phui-list-item-name {
  font-weight: bold;
  color: {$bluetext};
  padding: 4px 8px 6px 8px;
  display: block;
  -webkit-font-smoothing: antialiased;
}

.phui-list-sidenav .phui-list-item-type-divider {
  margin: 8px 8px 12px 8px;
  border-bottom: 1px solid {$thinblueborder};
}

.phui-list-sidenav .phui-list-item-icon {
  height: 14px;
  width: 14px;
  display: inline-block;
  position: absolute;
  top: 6px;
  text-align: center;
}

.phui-list-sidenav .phui-list-item-icon + .phui-list-item-name {
  padding-left: 20px;
}

.phui-list-sidenav .phui-list-item-has-icon {
  margin: 0;
  position: relative;
}

.phui-list-sidenav .phui-list-item-view {
  overflow: hidden;
}

.phui-list-sidenav .phui-list-item-href {
  display: block;
  padding: 4px 16px;
  clear: both;
  color: {$darkgreytext};
  line-height: 18px;
}

.phabricator-side-menu .phui-list-item-disabled .phui-list-item-href,
.phui-list-sidenav .phui-list-item-disabled .phui-list-item-href {
  color: {$lightgreytext};
}

.phui-list-sidenav .phui-list-item-has-icon .phui-list-item-href {
  padding: 4px 10px;
}

.phui-list-sidenav .phui-list-item-has-icon .phui-list-item-indented {
  padding-left: 18px;
}


.device-desktop .phui-list-sidenav .phui-list-item-href:hover {
  background: {$sky};
  color: white;
  cursor: pointer;
  text-decoration: none;
}

.device-desktop .phui-list-sidenav .phui-list-item-href:hover .phui-icon-view {
  color: #fff;
}

/* - Top, Full Width Navigations -----------------------------------------------

  Sets a page or box with a top navbar

*/

.phui-list-view.phui-list-navbar {
  list-style: none;
  overflow: hidden;
  border-bottom: 1px solid {$thinblueborder};
}

.phui-list-view.phui-list-navbar > li {
  list-style: none;
  float: left;
  display: block;
  border-right: 1px solid {$thinblueborder};
}

.phui-list-view.phui-list-navbar > li > * {
  display: block;
}

.phui-list-navbar .phui-list-item-href {
  color: {$bluetext};
  padding: 8px 16px;
  line-height: 16px;
}

.phui-list-navbar .phui-list-item-selected .phui-list-item-href {
  background: {$lightbluebackground};
  color: {$darkbluetext};
  font-weight: bold;
}

.phui-list-navbar .phui-list-item-href:hover {
  background: rgba(100,100,100,.1);
  color: {$darkgreytext};
  text-decoration: none;
}

.phui-list-navbar .phui-list-item-icon {
  height: 14px;
  width: 14px;
  display: block;
  font-size: 14px;
}

.device-phone .phui-list-view.phui-list-navbar > li {
  float: none;
  border: none;
}

/* - Status Colors -------------------------------------------------------------

  Colors for navbars

*/

.phui-list-item-warn .phui-list-item-href {
  color: #bc7837;
}

.phui-list-item-fail .phui-list-item-href {
  color: {$red};
}

.phui-list-item-warn.phui-list-item-selected .phui-list-item-href,
.phui-list-item-warn .phui-list-item-href:hover {
  background: {$lightyellow};
  color: #bc7837;
}

.phui-list-item-fail.phui-list-item-selected .phui-list-item-href,
.phui-list-item-fail .phui-list-item-href:hover {
  background: {$lightred};
  color: {$red};
}

.phui-list-item-warn.phui-list-item-selected .phui-list-item-href:hover {
  background: #fcf0bd;
}

.phui-list-item-fail.phui-list-item-selected .phui-list-item-href:hover {
  background: #f5d3d0;
}

/* - Dashboards ------------------------------------------------------------ */

.dashboard-panel .phui-list-view.phui-list-navbar {
  border-left: 1px solid {$lightblueborder};
  border-right: 1px solid {$lightblueborder};
  border-bottom: 1px solid {$thinblueborder};
}

/* - Info Stack ------------------------------------------------------------ */

.phui-info-view + .phui-list-view {
  margin-top: 16px;
  border-top: 1px solid {$thinblueborder};
}
