/**
 * @provides phabricator-feed-css
 */

.phabricator-feed-newer-link {
  float: left;
  font-weight: bold;
}

.phabricator-feed-older-link {
  float: right;
  font-weight: bold;
}

.phabricator-public-feed-frame .phui-feed-story-foot {
  background: none;
}

.phabricator-public-feed-frame .phui-feed-story {
  box-shadow: none;
}

.phabricator-public-feed-frame .phui-feed-wrap {
  border: none;
}
