/**
 * @provides phui-pinboard-view-css
 */

.phui-pinboard-view {
  padding: 16px 8px;
  overflow: hidden;
  text-align: center;
}

.phui-pinboard-list-item {
  display: inline;
}

.phui-pinboard-item-view {
  margin: 0 12px 16px 0;
  text-align: left;
  width: 280px;
  background: #fff;
  border: 1px solid {$lightblueborder};
  border-radius: 3px;
}

.phui-pinboard-view .phui-pinboard-item-view {
  display: inline-block;
}

.device-desktop .phui-pinboard-item-view:hover {
  border-color: {$blue};
}

.device-desktop .phui-pinboard-item-view:hover .phui-pinboard-item-header a {
  color: {$blue};
}

/* On phones, show a single column of items. */
.device-phone .phui-pinboard-item-view {
  float: none;
  margin: 0 auto 16px;
}

.phui-pinboard-item-header {
  padding: 6px 8px;
  display: block;
  font-weight: bold;
  white-space: nowrap;
  overflow: hidden;
  border-top: 1px solid {$thinblueborder};
}

.phui-pinboard-item-header a {
  color: {$darkbluetext};
}

.phui-pinboard-item-image-link {
  display: block;
}

.phui-pinboard-item-image-link img {
  border-top-left-radius: 3px;
  border-top-right-radius: 3px;
}

.phui-pinboard-item-content {
  padding: 0 8px 8px;
  overflow: hidden;
  color: {$bluetext};
  font-size: {$smallerfontsize};
}

.phui-pinboard-item-count {
  float: left;
  margin-left: 8px;
}

.phui-pinboard-icons {
  padding: 4px 8px 4px 0;
  color: {$darkbluetext};
  overflow: hidden;
}

.phui-pinboard-icon {
  margin-right: 4px;
}

.phui-pinboard-item-disabled .phui-pinboard-item-image-link img {
  opacity: 0.25;
}

.phui-pinboard-item-disabled .phui-pinboard-item-header a {
  color: {$greytext};
  text-decoration: line-through;
}

.dashboard-panel .phui-pinboard-view {
  margin: 0;
  padding: 16px 12px 0 12px;
  background: #fff;
  border-left: 1px solid {$lightblueborder};
  border-right: 1px solid {$lightblueborder};
  border-bottom: 1px solid {$blueborder};
}
