/**
 * @javelin
 */

/**
 * Register a callback for invocation after DOMContentReady.
 *
 * NOTE: Although it isn't private, use of this function is heavily discouraged.
 * See @{article:Concepts: Behaviors} for information on using behaviors to
 * structure and invoke glue code.
 *
 * This function is defined as a side effect of init.js.
 *
 * @param function Callback function to invoke after DOMContentReady.
 * @return void
 */
JX.onload = function(callback) {
  // This isn't the real function definition, it's only defined here to let the
  // documentation generator find it. The actual definition is in init.js.
};
