CREATE TABLE {$NAMESPACE}_worker.worker_trigger (
  id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
  phid VARBINARY(64) NOT NULL,
  triggerVersion INT UNSIGNED NOT NULL,
  clockClass VARCHAR(64) NOT NULL COLLATE {$COLLATE_TEXT},
  clockProperties LONGTEXT NOT NULL COLLATE {$COLLATE_TEXT},
  actionClass VARCHAR(64) NOT NULL COLLATE {$COLLATE_TEXT},
  actionProperties LONGTEXT NOT NULL COLLATE {$COLLATE_TEXT},
  UNIQUE KEY `key_phid` (phid),
  UNIQUE KEY `key_trigger` (triggerVersion)
) ENGINE=InnoDB, COLLATE {$COLLATE_TEXT};
