<?php

/**
 * This file is automatically generated. Use 'bin/celerity map' to rebuild it.
 *
 * @generated
 */
return array(
  'names' => array(
    'core.pkg.css' => '4dd8bb4d',
    'core.pkg.js' => '573e6664',
    'darkconsole.pkg.js' => 'e7393ebb',
    'differential.pkg.css' => '2de124c9',
    'differential.pkg.js' => 'f83532f8',
    'diffusion.pkg.css' => 'f45955ed',
    'diffusion.pkg.js' => '3a9a8bfa',
    'maniphest.pkg.css' => '4845691a',
    'maniphest.pkg.js' => '949a7498',
    'rsrc/css/aphront/aphront-bars.css' => '231ac33c',
    'rsrc/css/aphront/dark-console.css' => '6378ef3d',
    'rsrc/css/aphront/dialog-view.css' => 'be0e3a46',
    'rsrc/css/aphront/lightbox-attachment.css' => '7acac05d',
    'rsrc/css/aphront/list-filter-view.css' => '5d6f0526',
    'rsrc/css/aphront/multi-column.css' => 'fd18389d',
    'rsrc/css/aphront/notification.css' => '9c279160',
    'rsrc/css/aphront/panel-view.css' => '8427b78d',
    'rsrc/css/aphront/phabricator-nav-view.css' => 'a24cb589',
    'rsrc/css/aphront/table-view.css' => '6d01d468',
    'rsrc/css/aphront/tokenizer.css' => '056da01b',
    'rsrc/css/aphront/tooltip.css' => '7672b60f',
    'rsrc/css/aphront/typeahead-browse.css' => 'd8581d2c',
    'rsrc/css/aphront/typeahead.css' => '0e403212',
    'rsrc/css/application/almanac/almanac.css' => 'dbb9b3af',
    'rsrc/css/application/auth/auth.css' => '0877ed6e',
    'rsrc/css/application/base/main-menu-view.css' => 'd00a795a',
    'rsrc/css/application/base/notification-menu.css' => 'f31c0bde',
    'rsrc/css/application/base/phabricator-application-launch-view.css' => '95351601',
    'rsrc/css/application/base/phui-theme.css' => '981a58f8',
    'rsrc/css/application/base/standard-page-view.css' => '7b0d68d8',
    'rsrc/css/application/chatlog/chatlog.css' => 'd295b020',
    'rsrc/css/application/conduit/conduit-api.css' => '7bc725c4',
    'rsrc/css/application/config/config-options.css' => '0ede4c9b',
    'rsrc/css/application/config/config-template.css' => '8e6c6fcd',
    'rsrc/css/application/config/config-welcome.css' => '6abd79be',
    'rsrc/css/application/config/setup-issue.css' => 'db7e9c40',
    'rsrc/css/application/config/unhandled-exception.css' => '4c96257a',
    'rsrc/css/application/conpherence/durable-column.css' => '86396117',
    'rsrc/css/application/conpherence/menu.css' => 'f99fee4c',
    'rsrc/css/application/conpherence/message-pane.css' => '5897d3ac',
    'rsrc/css/application/conpherence/notification.css' => '6cdcc253',
    'rsrc/css/application/conpherence/transaction.css' => '85d0974c',
    'rsrc/css/application/conpherence/update.css' => 'faf6be09',
    'rsrc/css/application/conpherence/widget-pane.css' => '775eaaba',
    'rsrc/css/application/contentsource/content-source-view.css' => '4b8b05d4',
    'rsrc/css/application/countdown/timer.css' => 'e7544472',
    'rsrc/css/application/daemon/bulk-job.css' => 'df9c1d4a',
    'rsrc/css/application/dashboard/dashboard.css' => 'eb458607',
    'rsrc/css/application/diff/inline-comment-summary.css' => '51efda3a',
    'rsrc/css/application/differential/add-comment.css' => 'c47f8c40',
    'rsrc/css/application/differential/changeset-view.css' => 'b6b0d1bb',
    'rsrc/css/application/differential/core.css' => '7ac3cabc',
    'rsrc/css/application/differential/phui-inline-comment.css' => '0fdb3667',
    'rsrc/css/application/differential/revision-comment.css' => '14b8565a',
    'rsrc/css/application/differential/revision-history.css' => '0e8eb855',
    'rsrc/css/application/differential/revision-list.css' => 'f3c47d33',
    'rsrc/css/application/differential/table-of-contents.css' => 'ae4b7a55',
    'rsrc/css/application/diffusion/diffusion-icons.css' => '2941baf1',
    'rsrc/css/application/diffusion/diffusion-readme.css' => '2106ea08',
    'rsrc/css/application/diffusion/diffusion-source.css' => '075ba788',
    'rsrc/css/application/feed/feed.css' => 'ecd4ec57',
    'rsrc/css/application/files/global-drag-and-drop.css' => '697324ad',
    'rsrc/css/application/flag/flag.css' => '5337623f',
    'rsrc/css/application/harbormaster/harbormaster.css' => 'b0758ca5',
    'rsrc/css/application/herald/herald-test.css' => 'a52e323e',
    'rsrc/css/application/herald/herald.css' => '826075fa',
    'rsrc/css/application/maniphest/batch-editor.css' => 'b0f0b6d5',
    'rsrc/css/application/maniphest/report.css' => 'f6931fdf',
    'rsrc/css/application/maniphest/task-edit.css' => 'fda62a9b',
    'rsrc/css/application/maniphest/task-summary.css' => '11cc5344',
    'rsrc/css/application/objectselector/object-selector.css' => '85ee8ce6',
    'rsrc/css/application/owners/owners-path-editor.css' => '2f00933b',
    'rsrc/css/application/paste/paste.css' => 'a5157c48',
    'rsrc/css/application/people/people-profile.css' => 'fa2069ec',
    'rsrc/css/application/phame/phame.css' => '6d5b3682',
    'rsrc/css/application/pholio/pholio-edit.css' => '3ad9d1ee',
    'rsrc/css/application/pholio/pholio-inline-comments.css' => '8e545e49',
    'rsrc/css/application/pholio/pholio.css' => '95174bdd',
    'rsrc/css/application/phortune/phortune-credit-card-form.css' => '8391eb02',
    'rsrc/css/application/phortune/phortune.css' => '9149f103',
    'rsrc/css/application/phrequent/phrequent.css' => 'ffc185ad',
    'rsrc/css/application/phriction/phriction-document-css.css' => 'd1861e06',
    'rsrc/css/application/policy/policy-edit.css' => '815c66f7',
    'rsrc/css/application/policy/policy-transaction-detail.css' => '82100a43',
    'rsrc/css/application/policy/policy.css' => '957ea14c',
    'rsrc/css/application/ponder/ponder-view.css' => '7b0df4da',
    'rsrc/css/application/releeph/releeph-core.css' => '9b3c5733',
    'rsrc/css/application/releeph/releeph-preview-branch.css' => 'b7a6f4a5',
    'rsrc/css/application/releeph/releeph-request-differential-create-dialog.css' => '8d8b92cd',
    'rsrc/css/application/releeph/releeph-request-typeahead.css' => '667a48ae',
    'rsrc/css/application/search/search-results.css' => '7dea472c',
    'rsrc/css/application/slowvote/slowvote.css' => 'da0afb1b',
    'rsrc/css/application/tokens/tokens.css' => '3d0f239e',
    'rsrc/css/application/uiexample/example.css' => '528b19de',
    'rsrc/css/core/core.css' => 'a76cefc9',
    'rsrc/css/core/remarkup.css' => '64277c02',
    'rsrc/css/core/syntax.css' => '9fd11da8',
    'rsrc/css/core/z-index.css' => 'a36a45da',
    'rsrc/css/diviner/diviner-shared.css' => 'aa3656aa',
    'rsrc/css/font/font-aleo.css' => '8bdb2835',
    'rsrc/css/font/font-awesome.css' => 'c43323c5',
    'rsrc/css/font/font-lato.css' => 'c7ccd872',
    'rsrc/css/font/phui-font-icon-base.css' => 'ecbbb4c2',
    'rsrc/css/layout/phabricator-filetree-view.css' => 'fccf9f82',
    'rsrc/css/layout/phabricator-hovercard-view.css' => '1239cd52',
    'rsrc/css/layout/phabricator-side-menu-view.css' => '3a3d9f41',
    'rsrc/css/layout/phabricator-source-code-view.css' => 'cbeef983',
    'rsrc/css/phui/calendar/phui-calendar-day.css' => 'd1cf6f93',
    'rsrc/css/phui/calendar/phui-calendar-list.css' => 'c1c7f338',
    'rsrc/css/phui/calendar/phui-calendar-month.css' => '476be7e0',
    'rsrc/css/phui/calendar/phui-calendar.css' => 'ccabe893',
    'rsrc/css/phui/phui-action-list.css' => 'c5eba19d',
    'rsrc/css/phui/phui-action-panel.css' => '91c7b835',
    'rsrc/css/phui/phui-badge.css' => 'f25c3476',
    'rsrc/css/phui/phui-big-info-view.css' => 'bd903741',
    'rsrc/css/phui/phui-box.css' => '10939564',
    'rsrc/css/phui/phui-button.css' => '16020a60',
    'rsrc/css/phui/phui-crumbs-view.css' => '414406b5',
    'rsrc/css/phui/phui-document-pro.css' => '8799acf7',
    'rsrc/css/phui/phui-document-summary.css' => '9ca48bdf',
    'rsrc/css/phui/phui-document.css' => 'a4a1c3b9',
    'rsrc/css/phui/phui-feed-story.css' => 'b7b26d23',
    'rsrc/css/phui/phui-fontkit.css' => '9cda225e',
    'rsrc/css/phui/phui-form-view.css' => '4a1a0f5e',
    'rsrc/css/phui/phui-form.css' => '0b98e572',
    'rsrc/css/phui/phui-header-view.css' => '55bb32dd',
    'rsrc/css/phui/phui-icon-set-selector.css' => '1ab67aad',
    'rsrc/css/phui/phui-icon.css' => '3f33ab57',
    'rsrc/css/phui/phui-image-mask.css' => '5a8b09c8',
    'rsrc/css/phui/phui-info-panel.css' => '27ea50a1',
    'rsrc/css/phui/phui-info-view.css' => '6d7c3509',
    'rsrc/css/phui/phui-list.css' => '9da2aa00',
    'rsrc/css/phui/phui-object-box.css' => '407eaf5a',
    'rsrc/css/phui/phui-object-item-list-view.css' => '26c30d3f',
    'rsrc/css/phui/phui-pager.css' => 'bea33d23',
    'rsrc/css/phui/phui-pinboard-view.css' => '2495140e',
    'rsrc/css/phui/phui-profile-menu.css' => '0734510a',
    'rsrc/css/phui/phui-property-list-view.css' => '27b2849e',
    'rsrc/css/phui/phui-remarkup-preview.css' => '1a8f2591',
    'rsrc/css/phui/phui-spacing.css' => '042804d6',
    'rsrc/css/phui/phui-status.css' => '888cedb8',
    'rsrc/css/phui/phui-tag-view.css' => 'e60e227b',
    'rsrc/css/phui/phui-text.css' => 'cf019f54',
    'rsrc/css/phui/phui-timeline-view.css' => '2efceff8',
    'rsrc/css/phui/phui-two-column-view.css' => '39ecafb1',
    'rsrc/css/phui/phui-workboard-view.css' => 'f488d036',
    'rsrc/css/phui/phui-workpanel-view.css' => 'adec7699',
    'rsrc/css/sprite-login.css' => '60e8560e',
    'rsrc/css/sprite-menu.css' => '9dd65b92',
    'rsrc/css/sprite-tokens.css' => '4f399012',
    'rsrc/externals/font/aleo/aleo-bold.eot' => 'd3d3bed7',
    'rsrc/externals/font/aleo/aleo-bold.svg' => '45899c8e',
    'rsrc/externals/font/aleo/aleo-bold.ttf' => '4b08bef0',
    'rsrc/externals/font/aleo/aleo-bold.woff' => '93b513a1',
    'rsrc/externals/font/aleo/aleo-bold.woff2' => '75fbf322',
    'rsrc/externals/font/aleo/aleo-regular.eot' => 'a4e29e2f',
    'rsrc/externals/font/aleo/aleo-regular.svg' => '42a86f7a',
    'rsrc/externals/font/aleo/aleo-regular.ttf' => '751e7479',
    'rsrc/externals/font/aleo/aleo-regular.woff' => 'c3744be9',
    'rsrc/externals/font/aleo/aleo-regular.woff2' => '851aa0ee',
    'rsrc/externals/font/fontawesome/fontawesome-webfont.eot' => '346fbcc5',
    'rsrc/externals/font/fontawesome/fontawesome-webfont.ttf' => '510fccb2',
    'rsrc/externals/font/fontawesome/fontawesome-webfont.woff' => '0334f580',
    'rsrc/externals/font/fontawesome/fontawesome-webfont.woff2' => '45dca585',
    'rsrc/externals/font/lato/lato-bold.eot' => '99fbcf8c',
    'rsrc/externals/font/lato/lato-bold.svg' => '2aa83045',
    'rsrc/externals/font/lato/lato-bold.ttf' => '0a7141f7',
    'rsrc/externals/font/lato/lato-bold.woff' => 'f5db2061',
    'rsrc/externals/font/lato/lato-bold.woff2' => '37a94ecd',
    'rsrc/externals/font/lato/lato-bolditalic.eot' => 'b93389d0',
    'rsrc/externals/font/lato/lato-bolditalic.svg' => '5442e1ef',
    'rsrc/externals/font/lato/lato-bolditalic.ttf' => 'dad31252',
    'rsrc/externals/font/lato/lato-bolditalic.woff' => 'e53bcf47',
    'rsrc/externals/font/lato/lato-bolditalic.woff2' => 'd035007f',
    'rsrc/externals/font/lato/lato-italic.eot' => '6a903f5d',
    'rsrc/externals/font/lato/lato-italic.svg' => '0dc7cf2f',
    'rsrc/externals/font/lato/lato-italic.ttf' => '629f64f0',
    'rsrc/externals/font/lato/lato-italic.woff' => '678dc4bb',
    'rsrc/externals/font/lato/lato-italic.woff2' => '7c8dd650',
    'rsrc/externals/font/lato/lato-regular.eot' => '848dfb1e',
    'rsrc/externals/font/lato/lato-regular.svg' => 'cbd5fd6b',
    'rsrc/externals/font/lato/lato-regular.ttf' => 'e270165b',
    'rsrc/externals/font/lato/lato-regular.woff' => '13d39fe2',
    'rsrc/externals/font/lato/lato-regular.woff2' => '57a9f742',
    'rsrc/externals/javelin/core/Event.js' => '85ea0626',
    'rsrc/externals/javelin/core/Stratcom.js' => '6c53634d',
    'rsrc/externals/javelin/core/__tests__/event-stop-and-kill.js' => '717554e4',
    'rsrc/externals/javelin/core/__tests__/install.js' => 'c432ee85',
    'rsrc/externals/javelin/core/__tests__/stratcom.js' => '88bf7313',
    'rsrc/externals/javelin/core/__tests__/util.js' => 'e251703d',
    'rsrc/externals/javelin/core/init.js' => '3010e992',
    'rsrc/externals/javelin/core/init_node.js' => 'c234aded',
    'rsrc/externals/javelin/core/install.js' => '05270951',
    'rsrc/externals/javelin/core/util.js' => '93cc50d6',
    'rsrc/externals/javelin/docs/Base.js' => '74676256',
    'rsrc/externals/javelin/docs/onload.js' => 'e819c479',
    'rsrc/externals/javelin/ext/fx/Color.js' => '7e41274a',
    'rsrc/externals/javelin/ext/fx/FX.js' => '54b612ba',
    'rsrc/externals/javelin/ext/reactor/core/DynVal.js' => 'f6555212',
    'rsrc/externals/javelin/ext/reactor/core/Reactor.js' => '2b8de964',
    'rsrc/externals/javelin/ext/reactor/core/ReactorNode.js' => '1ad0a787',
    'rsrc/externals/javelin/ext/reactor/core/ReactorNodeCalmer.js' => '76f4ebed',
    'rsrc/externals/javelin/ext/reactor/dom/RDOM.js' => 'c90a04fc',
    'rsrc/externals/javelin/ext/view/HTMLView.js' => 'fe287620',
    'rsrc/externals/javelin/ext/view/View.js' => '0f764c35',
    'rsrc/externals/javelin/ext/view/ViewInterpreter.js' => 'f829edb3',
    'rsrc/externals/javelin/ext/view/ViewPlaceholder.js' => '47830651',
    'rsrc/externals/javelin/ext/view/ViewRenderer.js' => '6c2b09a2',
    'rsrc/externals/javelin/ext/view/ViewVisitor.js' => 'efe49472',
    'rsrc/externals/javelin/ext/view/__tests__/HTMLView.js' => 'f92d7bcb',
    'rsrc/externals/javelin/ext/view/__tests__/View.js' => '6450b38b',
    'rsrc/externals/javelin/ext/view/__tests__/ViewInterpreter.js' => '7a94d6a5',
    'rsrc/externals/javelin/ext/view/__tests__/ViewRenderer.js' => '6ea96ac9',
    'rsrc/externals/javelin/lib/Cookie.js' => '62dfea03',
    'rsrc/externals/javelin/lib/DOM.js' => '805b806a',
    'rsrc/externals/javelin/lib/History.js' => 'd4505101',
    'rsrc/externals/javelin/lib/JSON.js' => '69adf288',
    'rsrc/externals/javelin/lib/Leader.js' => '331b1611',
    'rsrc/externals/javelin/lib/Mask.js' => '8a41885b',
    'rsrc/externals/javelin/lib/Quicksand.js' => '6b8ef10b',
    'rsrc/externals/javelin/lib/Request.js' => '94b750d2',
    'rsrc/externals/javelin/lib/Resource.js' => '44959b73',
    'rsrc/externals/javelin/lib/Routable.js' => 'b3e7d692',
    'rsrc/externals/javelin/lib/Router.js' => '29274e2b',
    'rsrc/externals/javelin/lib/Scrollbar.js' => '087e919c',
    'rsrc/externals/javelin/lib/Sound.js' => '949c0fe5',
    'rsrc/externals/javelin/lib/URI.js' => 'c989ade3',
    'rsrc/externals/javelin/lib/Vector.js' => '2caa8fb8',
    'rsrc/externals/javelin/lib/WebSocket.js' => 'e292eaf4',
    'rsrc/externals/javelin/lib/Workflow.js' => '5b2e3e2b',
    'rsrc/externals/javelin/lib/__tests__/Cookie.js' => '5ed109e8',
    'rsrc/externals/javelin/lib/__tests__/DOM.js' => 'c984504b',
    'rsrc/externals/javelin/lib/__tests__/JSON.js' => '837a7d68',
    'rsrc/externals/javelin/lib/__tests__/URI.js' => '1e45fda9',
    'rsrc/externals/javelin/lib/__tests__/behavior.js' => '1ea62783',
    'rsrc/externals/javelin/lib/behavior.js' => '61cbc29a',
    'rsrc/externals/javelin/lib/control/tokenizer/Tokenizer.js' => '8d3bc1b2',
    'rsrc/externals/javelin/lib/control/typeahead/Typeahead.js' => '70baed2f',
    'rsrc/externals/javelin/lib/control/typeahead/normalizer/TypeaheadNormalizer.js' => 'e6e25838',
    'rsrc/externals/javelin/lib/control/typeahead/source/TypeaheadCompositeSource.js' => '503e17fd',
    'rsrc/externals/javelin/lib/control/typeahead/source/TypeaheadOnDemandSource.js' => '013ffff9',
    'rsrc/externals/javelin/lib/control/typeahead/source/TypeaheadPreloadedSource.js' => '54f314a0',
    'rsrc/externals/javelin/lib/control/typeahead/source/TypeaheadSource.js' => '1bc11c4a',
    'rsrc/externals/javelin/lib/control/typeahead/source/TypeaheadStaticSource.js' => '6c0e62fa',
    'rsrc/externals/raphael/g.raphael.js' => '40dde778',
    'rsrc/externals/raphael/g.raphael.line.js' => '40da039e',
    'rsrc/externals/raphael/raphael.js' => '51ee6b43',
    'rsrc/favicons/apple-touch-icon-120x120.png' => '43742962',
    'rsrc/favicons/apple-touch-icon-152x152.png' => '669eaec3',
    'rsrc/favicons/apple-touch-icon-76x76.png' => 'ecdef672',
    'rsrc/favicons/favicon-128.png' => '47cdff03',
    'rsrc/favicons/favicon-16x16.png' => 'ee2523ac',
    'rsrc/favicons/favicon-32x32.png' => 'b6a8150e',
    'rsrc/favicons/favicon-96x96.png' => '8f7ea177',
    'rsrc/favicons/mask-icon.svg' => '0460cb1f',
    'rsrc/image/BFCFDA.png' => 'd5ec91f4',
    'rsrc/image/actions/edit.png' => '2fc41442',
    'rsrc/image/avatar.png' => 'e132bb6a',
    'rsrc/image/checker_dark.png' => 'd8e65881',
    'rsrc/image/checker_light.png' => 'a0155918',
    'rsrc/image/checker_lighter.png' => 'd5da91b6',
    'rsrc/image/darkload.gif' => '1ffd3ec6',
    'rsrc/image/divot.png' => '94dded62',
    'rsrc/image/examples/hero.png' => '979a86ae',
    'rsrc/image/grippy_texture.png' => 'aca81e2f',
    'rsrc/image/icon/fatcow/arrow_branch.png' => '2537c01c',
    'rsrc/image/icon/fatcow/arrow_merge.png' => '21b660e0',
    'rsrc/image/icon/fatcow/bullet_black.png' => 'ff190031',
    'rsrc/image/icon/fatcow/bullet_orange.png' => 'e273e5bb',
    'rsrc/image/icon/fatcow/bullet_red.png' => 'c0b75434',
    'rsrc/image/icon/fatcow/calendar_edit.png' => '24632275',
    'rsrc/image/icon/fatcow/document_black.png' => '45fe1c60',
    'rsrc/image/icon/fatcow/flag_blue.png' => 'a01abb1d',
    'rsrc/image/icon/fatcow/flag_finish.png' => '67825cee',
    'rsrc/image/icon/fatcow/flag_ghost.png' => '20ca8783',
    'rsrc/image/icon/fatcow/flag_green.png' => '7e0eaa7a',
    'rsrc/image/icon/fatcow/flag_orange.png' => '9e73df66',
    'rsrc/image/icon/fatcow/flag_pink.png' => '7e92f3b2',
    'rsrc/image/icon/fatcow/flag_purple.png' => 'cc517522',
    'rsrc/image/icon/fatcow/flag_red.png' => '04ec726f',
    'rsrc/image/icon/fatcow/flag_yellow.png' => '73946fd4',
    'rsrc/image/icon/fatcow/key_question.png' => '52a0c26a',
    'rsrc/image/icon/fatcow/link.png' => '7afd4d5e',
    'rsrc/image/icon/fatcow/page_white_edit.png' => '39a2eed8',
    'rsrc/image/icon/fatcow/page_white_put.png' => '08c95a0c',
    'rsrc/image/icon/fatcow/source/conduit.png' => '4ea01d2f',
    'rsrc/image/icon/fatcow/source/email.png' => '9bab3239',
    'rsrc/image/icon/fatcow/source/fax.png' => '04195e68',
    'rsrc/image/icon/fatcow/source/mobile.png' => 'f1321264',
    'rsrc/image/icon/fatcow/source/tablet.png' => '49396799',
    'rsrc/image/icon/fatcow/source/web.png' => '136ccb5d',
    'rsrc/image/icon/lightbox/close-2.png' => 'cc40e7c8',
    'rsrc/image/icon/lightbox/close-hover-2.png' => 'fb5d6d9e',
    'rsrc/image/icon/lightbox/left-arrow-2.png' => '8426133b',
    'rsrc/image/icon/lightbox/left-arrow-hover-2.png' => '701e5ee3',
    'rsrc/image/icon/lightbox/right-arrow-2.png' => '6d5519a0',
    'rsrc/image/icon/lightbox/right-arrow-hover-2.png' => '3a04aa21',
    'rsrc/image/icon/subscribe.png' => 'd03ed5a5',
    'rsrc/image/icon/tango/attachment.png' => 'ecc8022e',
    'rsrc/image/icon/tango/edit.png' => '929a1363',
    'rsrc/image/icon/tango/go-down.png' => '96d95e43',
    'rsrc/image/icon/tango/log.png' => 'b08cc63a',
    'rsrc/image/icon/tango/upload.png' => '7bbb7984',
    'rsrc/image/icon/unsubscribe.png' => '25725013',
    'rsrc/image/lightblue-header.png' => '5c168b6d',
    'rsrc/image/main_texture.png' => '29a2c5ad',
    'rsrc/image/menu_texture.png' => '5a17580d',
    'rsrc/image/people/harding.png' => '45aa614e',
    'rsrc/image/people/jefferson.png' => 'afca0e53',
    'rsrc/image/people/lincoln.png' => '9369126d',
    'rsrc/image/people/mckinley.png' => 'fb8f16ce',
    'rsrc/image/people/taft.png' => 'd7bc402c',
    'rsrc/image/people/user0.png' => '03dacaea',
    'rsrc/image/people/user1.png' => '4a4e7702',
    'rsrc/image/people/user2.png' => '47a0ee40',
    'rsrc/image/people/user3.png' => '835ff627',
    'rsrc/image/people/user4.png' => 'b0e830f1',
    'rsrc/image/people/user5.png' => '9c95b369',
    'rsrc/image/people/user6.png' => 'ba3fbfb0',
    'rsrc/image/people/user7.png' => 'da613924',
    'rsrc/image/people/user8.png' => 'f1035edf',
    'rsrc/image/people/user9.png' => '66730be3',
    'rsrc/image/people/washington.png' => '40dd301c',
    'rsrc/image/phrequent_active.png' => 'a466a8ed',
    'rsrc/image/phrequent_inactive.png' => 'bfc15a69',
    'rsrc/image/sprite-login-X2.png' => 'e3991e37',
    'rsrc/image/sprite-login.png' => '03d5af29',
    'rsrc/image/sprite-menu-X2.png' => 'cfd8fca5',
    'rsrc/image/sprite-menu.png' => 'd7a99faa',
    'rsrc/image/sprite-tokens-X2.png' => '348f1745',
    'rsrc/image/sprite-tokens.png' => 'ce0b62be',
    'rsrc/image/texture/card-gradient.png' => '815f26e8',
    'rsrc/image/texture/dark-menu-hover.png' => '5fa7ece8',
    'rsrc/image/texture/dark-menu.png' => '7e22296e',
    'rsrc/image/texture/grip.png' => '719404f3',
    'rsrc/image/texture/panel-header-gradient.png' => 'e3b8dcfe',
    'rsrc/image/texture/phlnx-bg.png' => '8d819209',
    'rsrc/image/texture/pholio-background.gif' => 'ba29239c',
    'rsrc/image/texture/table_header.png' => '5c433037',
    'rsrc/image/texture/table_header_hover.png' => '038ec3b9',
    'rsrc/image/texture/table_header_tall.png' => 'd56b434f',
    'rsrc/js/application/aphlict/Aphlict.js' => '5359e785',
    'rsrc/js/application/aphlict/behavior-aphlict-dropdown.js' => '031cee25',
    'rsrc/js/application/aphlict/behavior-aphlict-listen.js' => 'fb20ac8d',
    'rsrc/js/application/aphlict/behavior-aphlict-status.js' => 'ea681761',
    'rsrc/js/application/aphlict/behavior-desktop-notifications-control.js' => 'edd1ba66',
    'rsrc/js/application/auth/behavior-persona-login.js' => '9414ff18',
    'rsrc/js/application/calendar/behavior-day-view.js' => '5c46cff2',
    'rsrc/js/application/calendar/behavior-event-all-day.js' => '38dcf3c8',
    'rsrc/js/application/calendar/behavior-recurring-edit.js' => '5f1c4d5f',
    'rsrc/js/application/config/behavior-reorder-fields.js' => 'b6993408',
    'rsrc/js/application/conpherence/ConpherenceThreadManager.js' => '01774ab2',
    'rsrc/js/application/conpherence/behavior-drag-and-drop-photo.js' => 'cf86d16a',
    'rsrc/js/application/conpherence/behavior-durable-column.js' => 'c72aa091',
    'rsrc/js/application/conpherence/behavior-menu.js' => '1d45c74d',
    'rsrc/js/application/conpherence/behavior-pontificate.js' => '21ba5861',
    'rsrc/js/application/conpherence/behavior-quicksand-blacklist.js' => '7927a7d3',
    'rsrc/js/application/conpherence/behavior-widget-pane.js' => 'a8458711',
    'rsrc/js/application/countdown/timer.js' => 'e4cc26b3',
    'rsrc/js/application/daemon/behavior-bulk-job-reload.js' => 'edf8a145',
    'rsrc/js/application/dashboard/behavior-dashboard-async-panel.js' => '469c0d9e',
    'rsrc/js/application/dashboard/behavior-dashboard-move-panels.js' => '82439934',
    'rsrc/js/application/dashboard/behavior-dashboard-query-panel-select.js' => '453c5375',
    'rsrc/js/application/dashboard/behavior-dashboard-tab-panel.js' => 'd4eecc63',
    'rsrc/js/application/differential/ChangesetViewManager.js' => 'a2828756',
    'rsrc/js/application/differential/DifferentialInlineCommentEditor.js' => '64a5550f',
    'rsrc/js/application/differential/behavior-add-reviewers-and-ccs.js' => 'e10f8e18',
    'rsrc/js/application/differential/behavior-comment-jump.js' => '4fdb476d',
    'rsrc/js/application/differential/behavior-comment-preview.js' => 'b064af76',
    'rsrc/js/application/differential/behavior-diff-radios.js' => 'e1ff79b1',
    'rsrc/js/application/differential/behavior-dropdown-menus.js' => '9a6b9324',
    'rsrc/js/application/differential/behavior-edit-inline-comments.js' => '65ef6074',
    'rsrc/js/application/differential/behavior-keyboard-nav.js' => '2c426492',
    'rsrc/js/application/differential/behavior-populate.js' => '8694b1df',
    'rsrc/js/application/differential/behavior-toggle-files.js' => 'ca3f91eb',
    'rsrc/js/application/differential/behavior-user-select.js' => 'a8d8459d',
    'rsrc/js/application/diffusion/DiffusionLocateFileSource.js' => 'b42eddc7',
    'rsrc/js/application/diffusion/behavior-audit-preview.js' => 'd835b03a',
    'rsrc/js/application/diffusion/behavior-commit-branches.js' => 'bdaf4d04',
    'rsrc/js/application/diffusion/behavior-commit-graph.js' => '5a0b1a64',
    'rsrc/js/application/diffusion/behavior-jump-to.js' => '73d09eef',
    'rsrc/js/application/diffusion/behavior-load-blame.js' => '42126667',
    'rsrc/js/application/diffusion/behavior-locate-file.js' => '6d3e1947',
    'rsrc/js/application/diffusion/behavior-pull-lastmodified.js' => 'f01586dc',
    'rsrc/js/application/doorkeeper/behavior-doorkeeper-tag.js' => 'e5822781',
    'rsrc/js/application/drydock/drydock-live-operation-status.js' => '901935ef',
    'rsrc/js/application/files/behavior-icon-composer.js' => '8499b6ab',
    'rsrc/js/application/files/behavior-launch-icon-composer.js' => '48086888',
    'rsrc/js/application/herald/HeraldRuleEditor.js' => '746ca158',
    'rsrc/js/application/herald/PathTypeahead.js' => 'f7fc67ec',
    'rsrc/js/application/herald/herald-rule-editor.js' => '7ebaeed3',
    'rsrc/js/application/maniphest/behavior-batch-editor.js' => '782ab6e7',
    'rsrc/js/application/maniphest/behavior-batch-selector.js' => '7b98d7c5',
    'rsrc/js/application/maniphest/behavior-line-chart.js' => '88f0c5b3',
    'rsrc/js/application/maniphest/behavior-list-edit.js' => 'a9f88de2',
    'rsrc/js/application/maniphest/behavior-subpriorityeditor.js' => '71237763',
    'rsrc/js/application/owners/OwnersPathEditor.js' => 'aa1733d0',
    'rsrc/js/application/owners/owners-path-editor.js' => '7a68dda3',
    'rsrc/js/application/passphrase/passphrase-credential-control.js' => '3cb0b2fc',
    'rsrc/js/application/pholio/behavior-pholio-mock-edit.js' => '246dc085',
    'rsrc/js/application/pholio/behavior-pholio-mock-view.js' => 'fbe497e7',
    'rsrc/js/application/phortune/behavior-stripe-payment-form.js' => '3f5d6dbf',
    'rsrc/js/application/phortune/behavior-test-payment-form.js' => 'fc91ab6c',
    'rsrc/js/application/phortune/phortune-credit-card-form.js' => '2290aeef',
    'rsrc/js/application/policy/behavior-policy-control.js' => 'ae45872f',
    'rsrc/js/application/policy/behavior-policy-rule-editor.js' => '5e9f347c',
    'rsrc/js/application/projects/behavior-project-boards.js' => 'ba4fa35c',
    'rsrc/js/application/projects/behavior-project-create.js' => '065227cc',
    'rsrc/js/application/projects/behavior-reorder-columns.js' => 'e1d25dfb',
    'rsrc/js/application/releeph/releeph-preview-branch.js' => 'b2b4fbaf',
    'rsrc/js/application/releeph/releeph-request-state-change.js' => 'a0b57eb8',
    'rsrc/js/application/releeph/releeph-request-typeahead.js' => 'de2e896f',
    'rsrc/js/application/repository/repository-crossreference.js' => 'e5339c43',
    'rsrc/js/application/search/behavior-reorder-profile-menu-items.js' => 'e2e0a072',
    'rsrc/js/application/search/behavior-reorder-queries.js' => 'e9581f08',
    'rsrc/js/application/slowvote/behavior-slowvote-embed.js' => '887ad43f',
    'rsrc/js/application/transactions/behavior-comment-actions.js' => '1f2fcaf8',
    'rsrc/js/application/transactions/behavior-reorder-configs.js' => 'd7a74243',
    'rsrc/js/application/transactions/behavior-reorder-fields.js' => 'b59e1e96',
    'rsrc/js/application/transactions/behavior-show-older-transactions.js' => 'dbbf48b6',
    'rsrc/js/application/transactions/behavior-transaction-comment-form.js' => 'b23b49e6',
    'rsrc/js/application/transactions/behavior-transaction-list.js' => '13c739ea',
    'rsrc/js/application/typeahead/behavior-typeahead-browse.js' => '635de1ec',
    'rsrc/js/application/typeahead/behavior-typeahead-search.js' => '93d0c9e3',
    'rsrc/js/application/uiexample/JavelinViewExample.js' => 'd4a14807',
    'rsrc/js/application/uiexample/ReactorButtonExample.js' => 'd19198c8',
    'rsrc/js/application/uiexample/ReactorCheckboxExample.js' => '519705ea',
    'rsrc/js/application/uiexample/ReactorFocusExample.js' => '40a6a403',
    'rsrc/js/application/uiexample/ReactorInputExample.js' => '886fd850',
    'rsrc/js/application/uiexample/ReactorMouseoverExample.js' => '47c794d8',
    'rsrc/js/application/uiexample/ReactorRadioExample.js' => '988040b4',
    'rsrc/js/application/uiexample/ReactorSelectExample.js' => 'a155550f',
    'rsrc/js/application/uiexample/ReactorSendClassExample.js' => '1def2711',
    'rsrc/js/application/uiexample/ReactorSendPropertiesExample.js' => 'b1f0ccee',
    'rsrc/js/application/uiexample/busy-example.js' => '60479091',
    'rsrc/js/application/uiexample/gesture-example.js' => '558829c2',
    'rsrc/js/application/uiexample/notification-example.js' => '8ce821c5',
    'rsrc/js/core/Busy.js' => '59a7976a',
    'rsrc/js/core/DragAndDropFileUpload.js' => 'ad10aeac',
    'rsrc/js/core/DraggableList.js' => 'a16ec1c6',
    'rsrc/js/core/FileUpload.js' => '477359c8',
    'rsrc/js/core/Hovercard.js' => 'c6f720ff',
    'rsrc/js/core/KeyboardShortcut.js' => '1ae869f2',
    'rsrc/js/core/KeyboardShortcutManager.js' => 'c1700f6f',
    'rsrc/js/core/MultirowRowManager.js' => 'b5d57730',
    'rsrc/js/core/Notification.js' => 'ccf1cbf8',
    'rsrc/js/core/Prefab.js' => 'e67df814',
    'rsrc/js/core/ShapedRequest.js' => '7cbe244b',
    'rsrc/js/core/TextAreaUtils.js' => '9e54692d',
    'rsrc/js/core/Title.js' => 'df5e11d2',
    'rsrc/js/core/ToolTip.js' => '1d298e3a',
    'rsrc/js/core/behavior-active-nav.js' => 'e379b58e',
    'rsrc/js/core/behavior-audio-source.js' => '59b251eb',
    'rsrc/js/core/behavior-autofocus.js' => '7319e029',
    'rsrc/js/core/behavior-choose-control.js' => '327a00d1',
    'rsrc/js/core/behavior-crop.js' => 'fa0f4fc2',
    'rsrc/js/core/behavior-dark-console.js' => 'f411b6ae',
    'rsrc/js/core/behavior-device.js' => 'a205cf28',
    'rsrc/js/core/behavior-drag-and-drop-textarea.js' => '4f6a4b4e',
    'rsrc/js/core/behavior-error-log.js' => '6882e80a',
    'rsrc/js/core/behavior-fancy-datepicker.js' => '8ae55229',
    'rsrc/js/core/behavior-file-tree.js' => '88236f00',
    'rsrc/js/core/behavior-form.js' => '5c54cbf3',
    'rsrc/js/core/behavior-gesture.js' => '3ab51e2c',
    'rsrc/js/core/behavior-global-drag-and-drop.js' => 'c8e57404',
    'rsrc/js/core/behavior-high-security-warning.js' => 'a464fe03',
    'rsrc/js/core/behavior-history-install.js' => '7ee2b591',
    'rsrc/js/core/behavior-hovercard.js' => '66dd6e9e',
    'rsrc/js/core/behavior-keyboard-pager.js' => 'a8da01f0',
    'rsrc/js/core/behavior-keyboard-shortcuts.js' => 'd75709e6',
    'rsrc/js/core/behavior-lightbox-attachments.js' => 'f8ba29d7',
    'rsrc/js/core/behavior-line-linker.js' => '1499a8cb',
    'rsrc/js/core/behavior-more.js' => 'a80d0378',
    'rsrc/js/core/behavior-object-selector.js' => '49b73b36',
    'rsrc/js/core/behavior-oncopy.js' => '2926fff2',
    'rsrc/js/core/behavior-phabricator-nav.js' => '56a1ca03',
    'rsrc/js/core/behavior-phabricator-remarkup-assist.js' => '340c8eff',
    'rsrc/js/core/behavior-refresh-csrf.js' => 'ab2f381b',
    'rsrc/js/core/behavior-remarkup-preview.js' => '4b700e9e',
    'rsrc/js/core/behavior-reorder-applications.js' => '76b9fc3e',
    'rsrc/js/core/behavior-reveal-content.js' => '60821bc7',
    'rsrc/js/core/behavior-scrollbar.js' => '834a1173',
    'rsrc/js/core/behavior-search-typeahead.js' => '0b7a4f6e',
    'rsrc/js/core/behavior-select-on-click.js' => '4e3e79a6',
    'rsrc/js/core/behavior-time-typeahead.js' => 'f80d6bf0',
    'rsrc/js/core/behavior-toggle-class.js' => '5d7c9f33',
    'rsrc/js/core/behavior-tokenizer.js' => 'b3a4b884',
    'rsrc/js/core/behavior-tooltip.js' => '3ee3408b',
    'rsrc/js/core/behavior-watch-anchor.js' => '9f36c42d',
    'rsrc/js/core/behavior-workflow.js' => '0a3f3021',
    'rsrc/js/core/phtize.js' => 'd254d646',
    'rsrc/js/phui/behavior-phui-dropdown-menu.js' => '54733475',
    'rsrc/js/phui/behavior-phui-object-box-tabs.js' => '2bfa2836',
    'rsrc/js/phui/behavior-phui-profile-menu.js' => '12884df9',
    'rsrc/js/phuix/PHUIXActionListView.js' => 'b5c256b8',
    'rsrc/js/phuix/PHUIXActionView.js' => '8cf6d262',
    'rsrc/js/phuix/PHUIXAutocomplete.js' => '21dc9144',
    'rsrc/js/phuix/PHUIXDropdownMenu.js' => 'bd4c8dca',
    'rsrc/js/phuix/PHUIXFormControl.js' => '8fba1997',
    'rsrc/js/phuix/PHUIXIconView.js' => 'bff6884b',
  ),
  'symbols' => array(
    'almanac-css' => 'dbb9b3af',
    'aphront-bars' => '231ac33c',
    'aphront-dark-console-css' => '6378ef3d',
    'aphront-dialog-view-css' => 'be0e3a46',
    'aphront-list-filter-view-css' => '5d6f0526',
    'aphront-multi-column-view-css' => 'fd18389d',
    'aphront-panel-view-css' => '8427b78d',
    'aphront-table-view-css' => '6d01d468',
    'aphront-tokenizer-control-css' => '056da01b',
    'aphront-tooltip-css' => '7672b60f',
    'aphront-typeahead-control-css' => '0e403212',
    'auth-css' => '0877ed6e',
    'bulk-job-css' => 'df9c1d4a',
    'changeset-view-manager' => 'a2828756',
    'conduit-api-css' => '7bc725c4',
    'config-options-css' => '0ede4c9b',
    'config-welcome-css' => '6abd79be',
    'conpherence-durable-column-view' => '86396117',
    'conpherence-menu-css' => 'f99fee4c',
    'conpherence-message-pane-css' => '5897d3ac',
    'conpherence-notification-css' => '6cdcc253',
    'conpherence-thread-manager' => '01774ab2',
    'conpherence-transaction-css' => '85d0974c',
    'conpherence-update-css' => 'faf6be09',
    'conpherence-widget-pane-css' => '775eaaba',
    'differential-changeset-view-css' => 'b6b0d1bb',
    'differential-core-view-css' => '7ac3cabc',
    'differential-inline-comment-editor' => '64a5550f',
    'differential-revision-add-comment-css' => 'c47f8c40',
    'differential-revision-comment-css' => '14b8565a',
    'differential-revision-history-css' => '0e8eb855',
    'differential-revision-list-css' => 'f3c47d33',
    'differential-table-of-contents-css' => 'ae4b7a55',
    'diffusion-icons-css' => '2941baf1',
    'diffusion-readme-css' => '2106ea08',
    'diffusion-source-css' => '075ba788',
    'diviner-shared-css' => 'aa3656aa',
    'font-aleo' => '8bdb2835',
    'font-fontawesome' => 'c43323c5',
    'font-lato' => 'c7ccd872',
    'global-drag-and-drop-css' => '697324ad',
    'harbormaster-css' => 'b0758ca5',
    'herald-css' => '826075fa',
    'herald-rule-editor' => '746ca158',
    'herald-test-css' => 'a52e323e',
    'inline-comment-summary-css' => '51efda3a',
    'javelin-aphlict' => '5359e785',
    'javelin-behavior' => '61cbc29a',
    'javelin-behavior-aphlict-dropdown' => '031cee25',
    'javelin-behavior-aphlict-listen' => 'fb20ac8d',
    'javelin-behavior-aphlict-status' => 'ea681761',
    'javelin-behavior-aphront-basic-tokenizer' => 'b3a4b884',
    'javelin-behavior-aphront-crop' => 'fa0f4fc2',
    'javelin-behavior-aphront-drag-and-drop-textarea' => '4f6a4b4e',
    'javelin-behavior-aphront-form-disable-on-submit' => '5c54cbf3',
    'javelin-behavior-aphront-more' => 'a80d0378',
    'javelin-behavior-audio-source' => '59b251eb',
    'javelin-behavior-audit-preview' => 'd835b03a',
    'javelin-behavior-bulk-job-reload' => 'edf8a145',
    'javelin-behavior-choose-control' => '327a00d1',
    'javelin-behavior-comment-actions' => '1f2fcaf8',
    'javelin-behavior-config-reorder-fields' => 'b6993408',
    'javelin-behavior-conpherence-drag-and-drop-photo' => 'cf86d16a',
    'javelin-behavior-conpherence-menu' => '1d45c74d',
    'javelin-behavior-conpherence-pontificate' => '21ba5861',
    'javelin-behavior-conpherence-widget-pane' => 'a8458711',
    'javelin-behavior-countdown-timer' => 'e4cc26b3',
    'javelin-behavior-dark-console' => 'f411b6ae',
    'javelin-behavior-dashboard-async-panel' => '469c0d9e',
    'javelin-behavior-dashboard-move-panels' => '82439934',
    'javelin-behavior-dashboard-query-panel-select' => '453c5375',
    'javelin-behavior-dashboard-tab-panel' => 'd4eecc63',
    'javelin-behavior-day-view' => '5c46cff2',
    'javelin-behavior-desktop-notifications-control' => 'edd1ba66',
    'javelin-behavior-device' => 'a205cf28',
    'javelin-behavior-differential-add-reviewers-and-ccs' => 'e10f8e18',
    'javelin-behavior-differential-comment-jump' => '4fdb476d',
    'javelin-behavior-differential-diff-radios' => 'e1ff79b1',
    'javelin-behavior-differential-dropdown-menus' => '9a6b9324',
    'javelin-behavior-differential-edit-inline-comments' => '65ef6074',
    'javelin-behavior-differential-feedback-preview' => 'b064af76',
    'javelin-behavior-differential-keyboard-navigation' => '2c426492',
    'javelin-behavior-differential-populate' => '8694b1df',
    'javelin-behavior-differential-toggle-files' => 'ca3f91eb',
    'javelin-behavior-differential-user-select' => 'a8d8459d',
    'javelin-behavior-diffusion-commit-branches' => 'bdaf4d04',
    'javelin-behavior-diffusion-commit-graph' => '5a0b1a64',
    'javelin-behavior-diffusion-jump-to' => '73d09eef',
    'javelin-behavior-diffusion-locate-file' => '6d3e1947',
    'javelin-behavior-diffusion-pull-lastmodified' => 'f01586dc',
    'javelin-behavior-doorkeeper-tag' => 'e5822781',
    'javelin-behavior-drydock-live-operation-status' => '901935ef',
    'javelin-behavior-durable-column' => 'c72aa091',
    'javelin-behavior-editengine-reorder-configs' => 'd7a74243',
    'javelin-behavior-editengine-reorder-fields' => 'b59e1e96',
    'javelin-behavior-error-log' => '6882e80a',
    'javelin-behavior-event-all-day' => '38dcf3c8',
    'javelin-behavior-fancy-datepicker' => '8ae55229',
    'javelin-behavior-global-drag-and-drop' => 'c8e57404',
    'javelin-behavior-herald-rule-editor' => '7ebaeed3',
    'javelin-behavior-high-security-warning' => 'a464fe03',
    'javelin-behavior-history-install' => '7ee2b591',
    'javelin-behavior-icon-composer' => '8499b6ab',
    'javelin-behavior-launch-icon-composer' => '48086888',
    'javelin-behavior-lightbox-attachments' => 'f8ba29d7',
    'javelin-behavior-line-chart' => '88f0c5b3',
    'javelin-behavior-load-blame' => '42126667',
    'javelin-behavior-maniphest-batch-editor' => '782ab6e7',
    'javelin-behavior-maniphest-batch-selector' => '7b98d7c5',
    'javelin-behavior-maniphest-list-editor' => 'a9f88de2',
    'javelin-behavior-maniphest-subpriority-editor' => '71237763',
    'javelin-behavior-owners-path-editor' => '7a68dda3',
    'javelin-behavior-passphrase-credential-control' => '3cb0b2fc',
    'javelin-behavior-persona-login' => '9414ff18',
    'javelin-behavior-phabricator-active-nav' => 'e379b58e',
    'javelin-behavior-phabricator-autofocus' => '7319e029',
    'javelin-behavior-phabricator-busy-example' => '60479091',
    'javelin-behavior-phabricator-file-tree' => '88236f00',
    'javelin-behavior-phabricator-gesture' => '3ab51e2c',
    'javelin-behavior-phabricator-gesture-example' => '558829c2',
    'javelin-behavior-phabricator-hovercards' => '66dd6e9e',
    'javelin-behavior-phabricator-keyboard-pager' => 'a8da01f0',
    'javelin-behavior-phabricator-keyboard-shortcuts' => 'd75709e6',
    'javelin-behavior-phabricator-line-linker' => '1499a8cb',
    'javelin-behavior-phabricator-nav' => '56a1ca03',
    'javelin-behavior-phabricator-notification-example' => '8ce821c5',
    'javelin-behavior-phabricator-object-selector' => '49b73b36',
    'javelin-behavior-phabricator-oncopy' => '2926fff2',
    'javelin-behavior-phabricator-remarkup-assist' => '340c8eff',
    'javelin-behavior-phabricator-reveal-content' => '60821bc7',
    'javelin-behavior-phabricator-search-typeahead' => '0b7a4f6e',
    'javelin-behavior-phabricator-show-older-transactions' => 'dbbf48b6',
    'javelin-behavior-phabricator-tooltips' => '3ee3408b',
    'javelin-behavior-phabricator-transaction-comment-form' => 'b23b49e6',
    'javelin-behavior-phabricator-transaction-list' => '13c739ea',
    'javelin-behavior-phabricator-watch-anchor' => '9f36c42d',
    'javelin-behavior-pholio-mock-edit' => '246dc085',
    'javelin-behavior-pholio-mock-view' => 'fbe497e7',
    'javelin-behavior-phui-dropdown-menu' => '54733475',
    'javelin-behavior-phui-object-box-tabs' => '2bfa2836',
    'javelin-behavior-phui-profile-menu' => '12884df9',
    'javelin-behavior-policy-control' => 'ae45872f',
    'javelin-behavior-policy-rule-editor' => '5e9f347c',
    'javelin-behavior-project-boards' => 'ba4fa35c',
    'javelin-behavior-project-create' => '065227cc',
    'javelin-behavior-quicksand-blacklist' => '7927a7d3',
    'javelin-behavior-recurring-edit' => '5f1c4d5f',
    'javelin-behavior-refresh-csrf' => 'ab2f381b',
    'javelin-behavior-releeph-preview-branch' => 'b2b4fbaf',
    'javelin-behavior-releeph-request-state-change' => 'a0b57eb8',
    'javelin-behavior-releeph-request-typeahead' => 'de2e896f',
    'javelin-behavior-remarkup-preview' => '4b700e9e',
    'javelin-behavior-reorder-applications' => '76b9fc3e',
    'javelin-behavior-reorder-columns' => 'e1d25dfb',
    'javelin-behavior-reorder-profile-menu-items' => 'e2e0a072',
    'javelin-behavior-repository-crossreference' => 'e5339c43',
    'javelin-behavior-scrollbar' => '834a1173',
    'javelin-behavior-search-reorder-queries' => 'e9581f08',
    'javelin-behavior-select-on-click' => '4e3e79a6',
    'javelin-behavior-slowvote-embed' => '887ad43f',
    'javelin-behavior-stripe-payment-form' => '3f5d6dbf',
    'javelin-behavior-test-payment-form' => 'fc91ab6c',
    'javelin-behavior-time-typeahead' => 'f80d6bf0',
    'javelin-behavior-toggle-class' => '5d7c9f33',
    'javelin-behavior-typeahead-browse' => '635de1ec',
    'javelin-behavior-typeahead-search' => '93d0c9e3',
    'javelin-behavior-view-placeholder' => '47830651',
    'javelin-behavior-workflow' => '0a3f3021',
    'javelin-color' => '7e41274a',
    'javelin-cookie' => '62dfea03',
    'javelin-diffusion-locate-file-source' => 'b42eddc7',
    'javelin-dom' => '805b806a',
    'javelin-dynval' => 'f6555212',
    'javelin-event' => '85ea0626',
    'javelin-fx' => '54b612ba',
    'javelin-history' => 'd4505101',
    'javelin-install' => '05270951',
    'javelin-json' => '69adf288',
    'javelin-leader' => '331b1611',
    'javelin-magical-init' => '3010e992',
    'javelin-mask' => '8a41885b',
    'javelin-quicksand' => '6b8ef10b',
    'javelin-reactor' => '2b8de964',
    'javelin-reactor-dom' => 'c90a04fc',
    'javelin-reactor-node-calmer' => '76f4ebed',
    'javelin-reactornode' => '1ad0a787',
    'javelin-request' => '94b750d2',
    'javelin-resource' => '44959b73',
    'javelin-routable' => 'b3e7d692',
    'javelin-router' => '29274e2b',
    'javelin-scrollbar' => '087e919c',
    'javelin-sound' => '949c0fe5',
    'javelin-stratcom' => '6c53634d',
    'javelin-tokenizer' => '8d3bc1b2',
    'javelin-typeahead' => '70baed2f',
    'javelin-typeahead-composite-source' => '503e17fd',
    'javelin-typeahead-normalizer' => 'e6e25838',
    'javelin-typeahead-ondemand-source' => '013ffff9',
    'javelin-typeahead-preloaded-source' => '54f314a0',
    'javelin-typeahead-source' => '1bc11c4a',
    'javelin-typeahead-static-source' => '6c0e62fa',
    'javelin-uri' => 'c989ade3',
    'javelin-util' => '93cc50d6',
    'javelin-vector' => '2caa8fb8',
    'javelin-view' => '0f764c35',
    'javelin-view-html' => 'fe287620',
    'javelin-view-interpreter' => 'f829edb3',
    'javelin-view-renderer' => '6c2b09a2',
    'javelin-view-visitor' => 'efe49472',
    'javelin-websocket' => 'e292eaf4',
    'javelin-workflow' => '5b2e3e2b',
    'lightbox-attachment-css' => '7acac05d',
    'maniphest-batch-editor' => 'b0f0b6d5',
    'maniphest-report-css' => 'f6931fdf',
    'maniphest-task-edit-css' => 'fda62a9b',
    'maniphest-task-summary-css' => '11cc5344',
    'multirow-row-manager' => 'b5d57730',
    'owners-path-editor' => 'aa1733d0',
    'owners-path-editor-css' => '2f00933b',
    'paste-css' => 'a5157c48',
    'path-typeahead' => 'f7fc67ec',
    'people-profile-css' => 'fa2069ec',
    'phabricator-action-list-view-css' => 'c5eba19d',
    'phabricator-application-launch-view-css' => '95351601',
    'phabricator-busy' => '59a7976a',
    'phabricator-chatlog-css' => 'd295b020',
    'phabricator-content-source-view-css' => '4b8b05d4',
    'phabricator-core-css' => 'a76cefc9',
    'phabricator-countdown-css' => 'e7544472',
    'phabricator-dashboard-css' => 'eb458607',
    'phabricator-drag-and-drop-file-upload' => 'ad10aeac',
    'phabricator-draggable-list' => 'a16ec1c6',
    'phabricator-fatal-config-template-css' => '8e6c6fcd',
    'phabricator-feed-css' => 'ecd4ec57',
    'phabricator-file-upload' => '477359c8',
    'phabricator-filetree-view-css' => 'fccf9f82',
    'phabricator-flag-css' => '5337623f',
    'phabricator-hovercard' => 'c6f720ff',
    'phabricator-hovercard-view-css' => '1239cd52',
    'phabricator-keyboard-shortcut' => '1ae869f2',
    'phabricator-keyboard-shortcut-manager' => 'c1700f6f',
    'phabricator-main-menu-view' => 'd00a795a',
    'phabricator-nav-view-css' => 'a24cb589',
    'phabricator-notification' => 'ccf1cbf8',
    'phabricator-notification-css' => '9c279160',
    'phabricator-notification-menu-css' => 'f31c0bde',
    'phabricator-object-selector-css' => '85ee8ce6',
    'phabricator-phtize' => 'd254d646',
    'phabricator-prefab' => 'e67df814',
    'phabricator-remarkup-css' => '64277c02',
    'phabricator-search-results-css' => '7dea472c',
    'phabricator-shaped-request' => '7cbe244b',
    'phabricator-side-menu-view-css' => '3a3d9f41',
    'phabricator-slowvote-css' => 'da0afb1b',
    'phabricator-source-code-view-css' => 'cbeef983',
    'phabricator-standard-page-view' => '7b0d68d8',
    'phabricator-textareautils' => '9e54692d',
    'phabricator-title' => 'df5e11d2',
    'phabricator-tooltip' => '1d298e3a',
    'phabricator-ui-example-css' => '528b19de',
    'phabricator-uiexample-javelin-view' => 'd4a14807',
    'phabricator-uiexample-reactor-button' => 'd19198c8',
    'phabricator-uiexample-reactor-checkbox' => '519705ea',
    'phabricator-uiexample-reactor-focus' => '40a6a403',
    'phabricator-uiexample-reactor-input' => '886fd850',
    'phabricator-uiexample-reactor-mouseover' => '47c794d8',
    'phabricator-uiexample-reactor-radio' => '988040b4',
    'phabricator-uiexample-reactor-select' => 'a155550f',
    'phabricator-uiexample-reactor-sendclass' => '1def2711',
    'phabricator-uiexample-reactor-sendproperties' => 'b1f0ccee',
    'phabricator-zindex-css' => 'a36a45da',
    'phame-css' => '6d5b3682',
    'pholio-css' => '95174bdd',
    'pholio-edit-css' => '3ad9d1ee',
    'pholio-inline-comments-css' => '8e545e49',
    'phortune-credit-card-form' => '2290aeef',
    'phortune-credit-card-form-css' => '8391eb02',
    'phortune-css' => '9149f103',
    'phrequent-css' => 'ffc185ad',
    'phriction-document-css' => 'd1861e06',
    'phui-action-panel-css' => '91c7b835',
    'phui-badge-view-css' => 'f25c3476',
    'phui-big-info-view-css' => 'bd903741',
    'phui-box-css' => '10939564',
    'phui-button-css' => '16020a60',
    'phui-calendar-css' => 'ccabe893',
    'phui-calendar-day-css' => 'd1cf6f93',
    'phui-calendar-list-css' => 'c1c7f338',
    'phui-calendar-month-css' => '476be7e0',
    'phui-crumbs-view-css' => '414406b5',
    'phui-document-summary-view-css' => '9ca48bdf',
    'phui-document-view-css' => 'a4a1c3b9',
    'phui-document-view-pro-css' => '8799acf7',
    'phui-feed-story-css' => 'b7b26d23',
    'phui-font-icon-base-css' => 'ecbbb4c2',
    'phui-fontkit-css' => '9cda225e',
    'phui-form-css' => '0b98e572',
    'phui-form-view-css' => '4a1a0f5e',
    'phui-header-view-css' => '55bb32dd',
    'phui-icon-set-selector-css' => '1ab67aad',
    'phui-icon-view-css' => '3f33ab57',
    'phui-image-mask-css' => '5a8b09c8',
    'phui-info-panel-css' => '27ea50a1',
    'phui-info-view-css' => '6d7c3509',
    'phui-inline-comment-view-css' => '0fdb3667',
    'phui-list-view-css' => '9da2aa00',
    'phui-object-box-css' => '407eaf5a',
    'phui-object-item-list-view-css' => '26c30d3f',
    'phui-pager-css' => 'bea33d23',
    'phui-pinboard-view-css' => '2495140e',
    'phui-profile-menu-css' => '0734510a',
    'phui-property-list-view-css' => '27b2849e',
    'phui-remarkup-preview-css' => '1a8f2591',
    'phui-spacing-css' => '042804d6',
    'phui-status-list-view-css' => '888cedb8',
    'phui-tag-view-css' => 'e60e227b',
    'phui-text-css' => 'cf019f54',
    'phui-theme-css' => '981a58f8',
    'phui-timeline-view-css' => '2efceff8',
    'phui-two-column-view-css' => '39ecafb1',
    'phui-workboard-view-css' => 'f488d036',
    'phui-workpanel-view-css' => 'adec7699',
    'phuix-action-list-view' => 'b5c256b8',
    'phuix-action-view' => '8cf6d262',
    'phuix-autocomplete' => '21dc9144',
    'phuix-dropdown-menu' => 'bd4c8dca',
    'phuix-form-control-view' => '8fba1997',
    'phuix-icon-view' => 'bff6884b',
    'policy-css' => '957ea14c',
    'policy-edit-css' => '815c66f7',
    'policy-transaction-detail-css' => '82100a43',
    'ponder-view-css' => '7b0df4da',
    'raphael-core' => '51ee6b43',
    'raphael-g' => '40dde778',
    'raphael-g-line' => '40da039e',
    'releeph-core' => '9b3c5733',
    'releeph-preview-branch' => 'b7a6f4a5',
    'releeph-request-differential-create-dialog' => '8d8b92cd',
    'releeph-request-typeahead-css' => '667a48ae',
    'setup-issue-css' => 'db7e9c40',
    'sprite-login-css' => '60e8560e',
    'sprite-menu-css' => '9dd65b92',
    'sprite-tokens-css' => '4f399012',
    'syntax-highlighting-css' => '9fd11da8',
    'tokens-css' => '3d0f239e',
    'typeahead-browse-css' => 'd8581d2c',
    'unhandled-exception-css' => '4c96257a',
  ),
  'requires' => array(
    '013ffff9' => array(
      'javelin-install',
      'javelin-util',
      'javelin-request',
      'javelin-typeahead-source',
    ),
    '01774ab2' => array(
      'javelin-dom',
      'javelin-util',
      'javelin-stratcom',
      'javelin-install',
      'javelin-aphlict',
      'javelin-workflow',
      'javelin-router',
      'javelin-behavior-device',
      'javelin-vector',
    ),
    '031cee25' => array(
      'javelin-behavior',
      'javelin-request',
      'javelin-stratcom',
      'javelin-vector',
      'javelin-dom',
      'javelin-uri',
      'javelin-behavior-device',
      'phabricator-title',
    ),
    '05270951' => array(
      'javelin-util',
      'javelin-magical-init',
    ),
    '056da01b' => array(
      'aphront-typeahead-control-css',
      'phui-tag-view-css',
    ),
    '065227cc' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-stratcom',
      'javelin-workflow',
    ),
    '087e919c' => array(
      'javelin-install',
      'javelin-dom',
      'javelin-stratcom',
      'javelin-vector',
    ),
    '0a3f3021' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-workflow',
      'javelin-dom',
      'javelin-router',
    ),
    '0b7a4f6e' => array(
      'javelin-behavior',
      'javelin-typeahead-ondemand-source',
      'javelin-typeahead',
      'javelin-dom',
      'javelin-uri',
      'javelin-util',
      'javelin-stratcom',
      'phabricator-prefab',
    ),
    '0f764c35' => array(
      'javelin-install',
      'javelin-util',
    ),
    '12884df9' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-dom',
    ),
    '13c739ea' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-workflow',
      'javelin-dom',
      'javelin-uri',
      'phabricator-textareautils',
    ),
    '1499a8cb' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-dom',
      'javelin-history',
    ),
    '1ad0a787' => array(
      'javelin-install',
      'javelin-reactor',
      'javelin-util',
      'javelin-reactor-node-calmer',
    ),
    '1ae869f2' => array(
      'javelin-install',
      'javelin-util',
      'phabricator-keyboard-shortcut-manager',
    ),
    '1bc11c4a' => array(
      'javelin-install',
      'javelin-util',
      'javelin-dom',
      'javelin-typeahead-normalizer',
    ),
    '1d298e3a' => array(
      'javelin-install',
      'javelin-util',
      'javelin-dom',
      'javelin-vector',
    ),
    '1d45c74d' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-util',
      'javelin-stratcom',
      'javelin-workflow',
      'javelin-behavior-device',
      'javelin-history',
      'javelin-vector',
      'javelin-scrollbar',
      'phabricator-title',
      'phabricator-shaped-request',
      'conpherence-thread-manager',
    ),
    '1def2711' => array(
      'javelin-install',
      'javelin-dom',
      'javelin-reactor-dom',
    ),
    '1f2fcaf8' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-workflow',
      'javelin-dom',
      'phuix-form-control-view',
      'phuix-icon-view',
      'javelin-behavior-phabricator-gesture',
    ),
    '21ba5861' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-util',
      'javelin-workflow',
      'javelin-stratcom',
      'conpherence-thread-manager',
    ),
    '21dc9144' => array(
      'javelin-install',
      'javelin-dom',
      'phuix-icon-view',
      'phabricator-prefab',
    ),
    '2290aeef' => array(
      'javelin-install',
      'javelin-dom',
      'javelin-json',
      'javelin-workflow',
      'javelin-util',
    ),
    '246dc085' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-dom',
      'javelin-workflow',
      'javelin-quicksand',
      'phabricator-phtize',
      'phabricator-drag-and-drop-file-upload',
      'phabricator-draggable-list',
    ),
    '2926fff2' => array(
      'javelin-behavior',
      'javelin-dom',
    ),
    '29274e2b' => array(
      'javelin-install',
      'javelin-util',
    ),
    '2b8de964' => array(
      'javelin-install',
      'javelin-util',
    ),
    '2bfa2836' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-dom',
    ),
    '2c426492' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-stratcom',
      'phabricator-keyboard-shortcut',
    ),
    '2caa8fb8' => array(
      'javelin-install',
      'javelin-event',
    ),
    '327a00d1' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-dom',
      'javelin-workflow',
    ),
    '331b1611' => array(
      'javelin-install',
    ),
    '340c8eff' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-dom',
      'phabricator-phtize',
      'phabricator-textareautils',
      'javelin-workflow',
      'javelin-vector',
      'phuix-autocomplete',
    ),
    '3ab51e2c' => array(
      'javelin-behavior',
      'javelin-behavior-device',
      'javelin-stratcom',
      'javelin-vector',
      'javelin-dom',
      'javelin-magical-init',
    ),
    '3cb0b2fc' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-stratcom',
      'javelin-workflow',
      'javelin-util',
      'javelin-uri',
    ),
    '3ee3408b' => array(
      'javelin-behavior',
      'javelin-behavior-device',
      'javelin-stratcom',
      'phabricator-tooltip',
    ),
    '3f5d6dbf' => array(
      'javelin-behavior',
      'javelin-dom',
      'phortune-credit-card-form',
    ),
    '40a6a403' => array(
      'javelin-install',
      'javelin-dom',
      'javelin-reactor-dom',
    ),
    42126667 => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-request',
    ),
    '44959b73' => array(
      'javelin-util',
      'javelin-uri',
      'javelin-install',
    ),
    '453c5375' => array(
      'javelin-behavior',
      'javelin-dom',
    ),
    '469c0d9e' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-workflow',
    ),
    '477359c8' => array(
      'javelin-install',
      'javelin-dom',
      'phabricator-notification',
    ),
    47830651 => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-view-renderer',
      'javelin-install',
    ),
    '47c794d8' => array(
      'javelin-install',
      'javelin-dom',
      'javelin-reactor-dom',
    ),
    48086888 => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-workflow',
    ),
    '49b73b36' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-request',
      'javelin-util',
    ),
    '4b700e9e' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-util',
      'phabricator-shaped-request',
    ),
    '4e3e79a6' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-dom',
    ),
    '4f6a4b4e' => array(
      'javelin-behavior',
      'javelin-dom',
      'phabricator-drag-and-drop-file-upload',
      'phabricator-textareautils',
    ),
    '4fdb476d' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-dom',
    ),
    '503e17fd' => array(
      'javelin-install',
      'javelin-typeahead-source',
      'javelin-util',
    ),
    '519705ea' => array(
      'javelin-install',
      'javelin-dom',
      'javelin-reactor-dom',
    ),
    '5359e785' => array(
      'javelin-install',
      'javelin-util',
      'javelin-websocket',
      'javelin-leader',
      'javelin-json',
    ),
    54733475 => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-dom',
      'phuix-dropdown-menu',
    ),
    '54b612ba' => array(
      'javelin-color',
      'javelin-install',
      'javelin-util',
    ),
    '54f314a0' => array(
      'javelin-install',
      'javelin-util',
      'javelin-request',
      'javelin-typeahead-source',
    ),
    '558829c2' => array(
      'javelin-stratcom',
      'javelin-behavior',
      'javelin-vector',
      'javelin-dom',
    ),
    '56a1ca03' => array(
      'javelin-behavior',
      'javelin-behavior-device',
      'javelin-stratcom',
      'javelin-dom',
      'javelin-magical-init',
      'javelin-vector',
      'javelin-request',
      'javelin-util',
    ),
    '59a7976a' => array(
      'javelin-install',
      'javelin-dom',
      'javelin-fx',
    ),
    '59b251eb' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-vector',
      'javelin-dom',
    ),
    '5a0b1a64' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-stratcom',
    ),
    '5b2e3e2b' => array(
      'javelin-stratcom',
      'javelin-request',
      'javelin-dom',
      'javelin-vector',
      'javelin-install',
      'javelin-util',
      'javelin-mask',
      'javelin-uri',
      'javelin-routable',
    ),
    '5c54cbf3' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-dom',
    ),
    '5d7c9f33' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-dom',
    ),
    '5e9f347c' => array(
      'javelin-behavior',
      'multirow-row-manager',
      'javelin-dom',
      'javelin-util',
      'phabricator-prefab',
      'javelin-json',
    ),
    60479091 => array(
      'phabricator-busy',
      'javelin-behavior',
    ),
    '60821bc7' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-dom',
    ),
    '61cbc29a' => array(
      'javelin-magical-init',
      'javelin-util',
    ),
    '62dfea03' => array(
      'javelin-install',
      'javelin-util',
    ),
    '635de1ec' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-workflow',
      'javelin-dom',
    ),
    '64a5550f' => array(
      'javelin-dom',
      'javelin-util',
      'javelin-stratcom',
      'javelin-install',
      'javelin-request',
      'javelin-workflow',
    ),
    '65ef6074' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-dom',
      'javelin-util',
      'javelin-vector',
      'differential-inline-comment-editor',
    ),
    '66dd6e9e' => array(
      'javelin-behavior',
      'javelin-behavior-device',
      'javelin-stratcom',
      'javelin-vector',
      'phabricator-hovercard',
    ),
    '6882e80a' => array(
      'javelin-dom',
    ),
    '69adf288' => array(
      'javelin-install',
    ),
    '6b8ef10b' => array(
      'javelin-install',
    ),
    '6c0e62fa' => array(
      'javelin-install',
      'javelin-typeahead-source',
    ),
    '6c2b09a2' => array(
      'javelin-install',
      'javelin-util',
    ),
    '6c53634d' => array(
      'javelin-install',
      'javelin-event',
      'javelin-util',
      'javelin-magical-init',
    ),
    '6d3e1947' => array(
      'javelin-behavior',
      'javelin-diffusion-locate-file-source',
      'javelin-dom',
      'javelin-typeahead',
      'javelin-uri',
    ),
    '70baed2f' => array(
      'javelin-install',
      'javelin-dom',
      'javelin-vector',
      'javelin-util',
    ),
    71237763 => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-stratcom',
      'javelin-workflow',
      'phabricator-draggable-list',
    ),
    '7319e029' => array(
      'javelin-behavior',
      'javelin-dom',
    ),
    '73d09eef' => array(
      'javelin-behavior',
      'javelin-vector',
      'javelin-dom',
    ),
    '746ca158' => array(
      'multirow-row-manager',
      'javelin-install',
      'javelin-util',
      'javelin-dom',
      'javelin-stratcom',
      'javelin-json',
      'phabricator-prefab',
    ),
    '76b9fc3e' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-workflow',
      'javelin-dom',
      'phabricator-draggable-list',
    ),
    '76f4ebed' => array(
      'javelin-install',
      'javelin-reactor',
      'javelin-util',
    ),
    '782ab6e7' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-util',
      'phabricator-prefab',
      'multirow-row-manager',
      'javelin-json',
    ),
    '7927a7d3' => array(
      'javelin-behavior',
      'javelin-quicksand',
    ),
    '7a68dda3' => array(
      'owners-path-editor',
      'javelin-behavior',
    ),
    '7b98d7c5' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-stratcom',
      'javelin-util',
    ),
    '7cbe244b' => array(
      'javelin-install',
      'javelin-util',
      'javelin-request',
      'javelin-router',
    ),
    '7e41274a' => array(
      'javelin-install',
    ),
    '7ebaeed3' => array(
      'herald-rule-editor',
      'javelin-behavior',
    ),
    '7ee2b591' => array(
      'javelin-behavior',
      'javelin-history',
    ),
    '805b806a' => array(
      'javelin-magical-init',
      'javelin-install',
      'javelin-util',
      'javelin-vector',
      'javelin-stratcom',
    ),
    82439934 => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-util',
      'javelin-stratcom',
      'javelin-workflow',
      'phabricator-draggable-list',
    ),
    '834a1173' => array(
      'javelin-behavior',
      'javelin-scrollbar',
    ),
    '8499b6ab' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-stratcom',
    ),
    '85ea0626' => array(
      'javelin-install',
    ),
    '85ee8ce6' => array(
      'aphront-dialog-view-css',
    ),
    '8694b1df' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-stratcom',
      'phabricator-tooltip',
      'changeset-view-manager',
    ),
    '88236f00' => array(
      'javelin-behavior',
      'phabricator-keyboard-shortcut',
      'javelin-stratcom',
    ),
    '886fd850' => array(
      'javelin-install',
      'javelin-reactor-dom',
      'javelin-view-html',
      'javelin-view-interpreter',
      'javelin-view-renderer',
    ),
    '887ad43f' => array(
      'javelin-behavior',
      'javelin-request',
      'javelin-stratcom',
      'javelin-dom',
    ),
    '88f0c5b3' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-vector',
    ),
    '8a41885b' => array(
      'javelin-install',
      'javelin-dom',
    ),
    '8ae55229' => array(
      'javelin-behavior',
      'javelin-util',
      'javelin-dom',
      'javelin-stratcom',
      'javelin-vector',
    ),
    '8bdb2835' => array(
      'phui-fontkit-css',
    ),
    '8ce821c5' => array(
      'phabricator-notification',
      'javelin-stratcom',
      'javelin-behavior',
    ),
    '8cf6d262' => array(
      'javelin-install',
      'javelin-dom',
      'javelin-util',
    ),
    '8d3bc1b2' => array(
      'javelin-dom',
      'javelin-util',
      'javelin-stratcom',
      'javelin-install',
    ),
    '8fba1997' => array(
      'javelin-install',
      'javelin-dom',
    ),
    '901935ef' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-request',
    ),
    '93d0c9e3' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-workflow',
      'javelin-dom',
    ),
    '9414ff18' => array(
      'javelin-behavior',
      'javelin-resource',
      'javelin-stratcom',
      'javelin-workflow',
      'javelin-util',
    ),
    '949c0fe5' => array(
      'javelin-install',
    ),
    '94b750d2' => array(
      'javelin-install',
      'javelin-stratcom',
      'javelin-util',
      'javelin-behavior',
      'javelin-json',
      'javelin-dom',
      'javelin-resource',
      'javelin-routable',
    ),
    '988040b4' => array(
      'javelin-install',
      'javelin-dom',
      'javelin-reactor-dom',
    ),
    '9a6b9324' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-util',
      'javelin-stratcom',
      'javelin-workflow',
      'phuix-dropdown-menu',
      'phuix-action-list-view',
      'phuix-action-view',
      'phabricator-phtize',
      'changeset-view-manager',
    ),
    '9e54692d' => array(
      'javelin-install',
      'javelin-dom',
      'javelin-vector',
    ),
    '9f36c42d' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-dom',
      'javelin-vector',
    ),
    'a0b57eb8' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-stratcom',
      'javelin-workflow',
      'javelin-util',
      'phabricator-keyboard-shortcut',
    ),
    'a155550f' => array(
      'javelin-install',
      'javelin-dom',
      'javelin-reactor-dom',
    ),
    'a16ec1c6' => array(
      'javelin-install',
      'javelin-dom',
      'javelin-stratcom',
      'javelin-util',
      'javelin-vector',
      'javelin-magical-init',
    ),
    'a205cf28' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-dom',
      'javelin-vector',
      'javelin-install',
    ),
    'a2828756' => array(
      'javelin-dom',
      'javelin-util',
      'javelin-stratcom',
      'javelin-install',
      'javelin-workflow',
      'javelin-router',
      'javelin-behavior-device',
      'javelin-vector',
    ),
    'a464fe03' => array(
      'javelin-behavior',
      'javelin-uri',
      'phabricator-notification',
    ),
    'a80d0378' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-dom',
    ),
    'a8458711' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-stratcom',
      'javelin-workflow',
      'javelin-util',
      'phabricator-notification',
      'javelin-behavior-device',
      'phuix-dropdown-menu',
      'phuix-action-list-view',
      'phuix-action-view',
      'conpherence-thread-manager',
    ),
    'a8d8459d' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-stratcom',
    ),
    'a8da01f0' => array(
      'javelin-behavior',
      'javelin-uri',
      'phabricator-keyboard-shortcut',
    ),
    'a9f88de2' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-stratcom',
      'javelin-workflow',
      'javelin-fx',
      'javelin-util',
    ),
    'aa1733d0' => array(
      'multirow-row-manager',
      'javelin-install',
      'path-typeahead',
      'javelin-dom',
      'javelin-util',
      'phabricator-prefab',
    ),
    'ab2f381b' => array(
      'javelin-request',
      'javelin-behavior',
      'javelin-dom',
      'javelin-router',
      'javelin-util',
      'phabricator-busy',
    ),
    'ad10aeac' => array(
      'javelin-install',
      'javelin-util',
      'javelin-request',
      'javelin-dom',
      'javelin-uri',
      'phabricator-file-upload',
    ),
    'ae45872f' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-util',
      'phuix-dropdown-menu',
      'phuix-action-list-view',
      'phuix-action-view',
      'javelin-workflow',
    ),
    'b064af76' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-dom',
      'javelin-request',
      'javelin-util',
      'phabricator-shaped-request',
    ),
    'b1f0ccee' => array(
      'javelin-install',
      'javelin-dom',
      'javelin-reactor-dom',
    ),
    'b23b49e6' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-util',
      'javelin-request',
      'phabricator-shaped-request',
    ),
    'b2b4fbaf' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-uri',
      'javelin-request',
    ),
    'b3a4b884' => array(
      'javelin-behavior',
      'phabricator-prefab',
    ),
    'b3e7d692' => array(
      'javelin-install',
    ),
    'b42eddc7' => array(
      'javelin-install',
      'javelin-dom',
      'javelin-typeahead-preloaded-source',
      'javelin-util',
    ),
    'b59e1e96' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-workflow',
      'javelin-dom',
      'phabricator-draggable-list',
    ),
    'b5c256b8' => array(
      'javelin-install',
      'javelin-dom',
    ),
    'b5d57730' => array(
      'javelin-install',
      'javelin-stratcom',
      'javelin-dom',
      'javelin-util',
    ),
    'b6993408' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-dom',
      'javelin-json',
      'phabricator-draggable-list',
    ),
    'b6b0d1bb' => array(
      'phui-inline-comment-view-css',
    ),
    'ba4fa35c' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-util',
      'javelin-vector',
      'javelin-stratcom',
      'javelin-workflow',
      'phabricator-draggable-list',
    ),
    'bd4c8dca' => array(
      'javelin-install',
      'javelin-util',
      'javelin-dom',
      'javelin-vector',
      'javelin-stratcom',
    ),
    'bdaf4d04' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-util',
      'javelin-request',
    ),
    'bff6884b' => array(
      'javelin-install',
      'javelin-dom',
    ),
    'c1700f6f' => array(
      'javelin-install',
      'javelin-util',
      'javelin-stratcom',
      'javelin-dom',
      'javelin-vector',
    ),
    'c6f720ff' => array(
      'javelin-install',
      'javelin-dom',
      'javelin-vector',
      'javelin-request',
      'javelin-uri',
    ),
    'c72aa091' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-stratcom',
      'javelin-behavior-device',
      'javelin-scrollbar',
      'javelin-quicksand',
      'phabricator-keyboard-shortcut',
      'conpherence-thread-manager',
    ),
    'c7ccd872' => array(
      'phui-fontkit-css',
    ),
    'c8e57404' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-uri',
      'javelin-mask',
      'phabricator-drag-and-drop-file-upload',
    ),
    'c90a04fc' => array(
      'javelin-dom',
      'javelin-dynval',
      'javelin-reactor',
      'javelin-reactornode',
      'javelin-install',
      'javelin-util',
    ),
    'c989ade3' => array(
      'javelin-install',
      'javelin-util',
      'javelin-stratcom',
    ),
    'ca3f91eb' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-stratcom',
      'phabricator-phtize',
    ),
    'ccf1cbf8' => array(
      'javelin-install',
      'javelin-dom',
      'javelin-stratcom',
      'javelin-util',
      'phabricator-notification-css',
    ),
    'cf86d16a' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-workflow',
      'phabricator-drag-and-drop-file-upload',
    ),
    'd00a795a' => array(
      'phui-theme-css',
    ),
    'd19198c8' => array(
      'javelin-install',
      'javelin-dom',
      'javelin-util',
      'javelin-dynval',
      'javelin-reactor-dom',
    ),
    'd254d646' => array(
      'javelin-util',
    ),
    'd4505101' => array(
      'javelin-stratcom',
      'javelin-install',
      'javelin-uri',
      'javelin-util',
    ),
    'd4a14807' => array(
      'javelin-install',
      'javelin-dom',
      'javelin-view',
    ),
    'd4eecc63' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-stratcom',
    ),
    'd75709e6' => array(
      'javelin-behavior',
      'javelin-workflow',
      'javelin-json',
      'javelin-dom',
      'phabricator-keyboard-shortcut',
    ),
    'd7a74243' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-workflow',
      'javelin-dom',
      'phabricator-draggable-list',
    ),
    'd835b03a' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-util',
      'phabricator-shaped-request',
    ),
    'dbbf48b6' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-dom',
      'phabricator-busy',
    ),
    'de2e896f' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-typeahead',
      'javelin-typeahead-ondemand-source',
      'javelin-dom',
    ),
    'df5e11d2' => array(
      'javelin-install',
    ),
    'e10f8e18' => array(
      'javelin-behavior',
      'javelin-dom',
      'phabricator-prefab',
    ),
    'e1d25dfb' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-workflow',
      'javelin-dom',
      'phabricator-draggable-list',
    ),
    'e1ff79b1' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-dom',
    ),
    'e292eaf4' => array(
      'javelin-install',
    ),
    'e2e0a072' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-workflow',
      'javelin-dom',
      'phabricator-draggable-list',
    ),
    'e379b58e' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-vector',
      'javelin-dom',
      'javelin-uri',
    ),
    'e4cc26b3' => array(
      'javelin-behavior',
      'javelin-dom',
    ),
    'e5339c43' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-stratcom',
      'javelin-uri',
    ),
    'e5822781' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-json',
      'javelin-workflow',
      'javelin-magical-init',
    ),
    'e67df814' => array(
      'javelin-install',
      'javelin-util',
      'javelin-dom',
      'javelin-typeahead',
      'javelin-tokenizer',
      'javelin-typeahead-preloaded-source',
      'javelin-typeahead-ondemand-source',
      'javelin-dom',
      'javelin-stratcom',
      'javelin-util',
    ),
    'e6e25838' => array(
      'javelin-install',
    ),
    'e9581f08' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-workflow',
      'javelin-dom',
      'phabricator-draggable-list',
    ),
    'ea681761' => array(
      'javelin-behavior',
      'javelin-aphlict',
      'phabricator-phtize',
      'javelin-dom',
    ),
    'edd1ba66' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-dom',
      'javelin-uri',
      'phabricator-notification',
    ),
    'edf8a145' => array(
      'javelin-behavior',
      'javelin-uri',
    ),
    'efe49472' => array(
      'javelin-install',
      'javelin-util',
    ),
    'f01586dc' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-util',
      'javelin-workflow',
      'javelin-json',
    ),
    'f411b6ae' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-util',
      'javelin-dom',
      'javelin-request',
      'phabricator-keyboard-shortcut',
    ),
    'f6555212' => array(
      'javelin-install',
      'javelin-reactornode',
      'javelin-util',
      'javelin-reactor',
    ),
    'f7fc67ec' => array(
      'javelin-install',
      'javelin-typeahead',
      'javelin-dom',
      'javelin-request',
      'javelin-typeahead-ondemand-source',
      'javelin-util',
    ),
    'f80d6bf0' => array(
      'javelin-behavior',
      'javelin-util',
      'javelin-dom',
      'javelin-stratcom',
      'javelin-vector',
      'javelin-typeahead-static-source',
    ),
    'f829edb3' => array(
      'javelin-view',
      'javelin-install',
      'javelin-dom',
    ),
    'f8ba29d7' => array(
      'javelin-behavior',
      'javelin-stratcom',
      'javelin-dom',
      'javelin-mask',
      'javelin-util',
      'phabricator-busy',
    ),
    'fa0f4fc2' => array(
      'javelin-behavior',
      'javelin-dom',
      'javelin-vector',
      'javelin-magical-init',
    ),
    'fb20ac8d' => array(
      'javelin-behavior',
      'javelin-aphlict',
      'javelin-stratcom',
      'javelin-request',
      'javelin-uri',
      'javelin-dom',
      'javelin-json',
      'javelin-router',
      'javelin-util',
      'javelin-leader',
      'javelin-sound',
      'phabricator-notification',
    ),
    'fbe497e7' => array(
      'javelin-behavior',
      'javelin-util',
      'javelin-stratcom',
      'javelin-dom',
      'javelin-vector',
      'javelin-magical-init',
      'javelin-request',
      'javelin-history',
      'javelin-workflow',
      'javelin-mask',
      'javelin-behavior-device',
      'phabricator-keyboard-shortcut',
    ),
    'fc91ab6c' => array(
      'javelin-behavior',
      'javelin-dom',
      'phortune-credit-card-form',
    ),
    'fe287620' => array(
      'javelin-install',
      'javelin-dom',
      'javelin-view-visitor',
      'javelin-util',
    ),
  ),
  'packages' => array(
    'core.pkg.css' => array(
      'phabricator-core-css',
      'phabricator-zindex-css',
      'phui-button-css',
      'phabricator-standard-page-view',
      'aphront-dialog-view-css',
      'phui-form-view-css',
      'aphront-panel-view-css',
      'aphront-table-view-css',
      'aphront-tokenizer-control-css',
      'aphront-typeahead-control-css',
      'aphront-list-filter-view-css',
      'phabricator-remarkup-css',
      'syntax-highlighting-css',
      'phui-pager-css',
      'aphront-tooltip-css',
      'phabricator-flag-css',
      'phui-info-view-css',
      'sprite-menu-css',
      'phabricator-main-menu-view',
      'phabricator-notification-css',
      'phabricator-notification-menu-css',
      'lightbox-attachment-css',
      'phui-header-view-css',
      'phabricator-filetree-view-css',
      'phabricator-nav-view-css',
      'phabricator-side-menu-view-css',
      'phui-crumbs-view-css',
      'phui-object-item-list-view-css',
      'global-drag-and-drop-css',
      'phui-spacing-css',
      'phui-form-css',
      'phui-icon-view-css',
      'phui-profile-menu-css',
      'phabricator-application-launch-view-css',
      'phabricator-action-list-view-css',
      'phui-property-list-view-css',
      'phui-tag-view-css',
      'phui-list-view-css',
      'font-fontawesome',
      'phui-font-icon-base-css',
      'phui-box-css',
      'phui-object-box-css',
      'phui-timeline-view-css',
      'sprite-tokens-css',
      'tokens-css',
      'phui-status-list-view-css',
      'phui-feed-story-css',
      'phabricator-feed-css',
      'phabricator-dashboard-css',
      'aphront-multi-column-view-css',
      'conpherence-durable-column-view',
    ),
    'core.pkg.js' => array(
      'javelin-util',
      'javelin-install',
      'javelin-event',
      'javelin-stratcom',
      'javelin-behavior',
      'javelin-resource',
      'javelin-request',
      'javelin-vector',
      'javelin-dom',
      'javelin-json',
      'javelin-uri',
      'javelin-workflow',
      'javelin-mask',
      'javelin-typeahead',
      'javelin-typeahead-normalizer',
      'javelin-typeahead-source',
      'javelin-typeahead-preloaded-source',
      'javelin-typeahead-ondemand-source',
      'javelin-tokenizer',
      'javelin-history',
      'javelin-router',
      'javelin-routable',
      'javelin-behavior-aphront-basic-tokenizer',
      'javelin-behavior-workflow',
      'javelin-behavior-aphront-form-disable-on-submit',
      'phabricator-keyboard-shortcut-manager',
      'phabricator-keyboard-shortcut',
      'javelin-behavior-phabricator-keyboard-shortcuts',
      'javelin-behavior-refresh-csrf',
      'javelin-behavior-phabricator-watch-anchor',
      'javelin-behavior-phabricator-autofocus',
      'phuix-dropdown-menu',
      'phuix-action-list-view',
      'phuix-action-view',
      'phabricator-phtize',
      'javelin-behavior-phabricator-oncopy',
      'phabricator-tooltip',
      'javelin-behavior-phabricator-tooltips',
      'phabricator-prefab',
      'javelin-behavior-device',
      'javelin-behavior-toggle-class',
      'javelin-behavior-lightbox-attachments',
      'phabricator-busy',
      'javelin-aphlict',
      'phabricator-notification',
      'javelin-behavior-aphlict-listen',
      'javelin-behavior-phabricator-search-typeahead',
      'javelin-behavior-aphlict-dropdown',
      'javelin-behavior-history-install',
      'javelin-behavior-phabricator-gesture',
      'javelin-behavior-phabricator-active-nav',
      'javelin-behavior-phabricator-nav',
      'javelin-behavior-phabricator-remarkup-assist',
      'phabricator-textareautils',
      'phabricator-file-upload',
      'javelin-behavior-global-drag-and-drop',
      'javelin-behavior-phabricator-reveal-content',
      'phabricator-hovercard',
      'javelin-behavior-phabricator-hovercards',
      'javelin-color',
      'javelin-fx',
      'phabricator-draggable-list',
      'javelin-behavior-phabricator-transaction-list',
      'javelin-behavior-phabricator-show-older-transactions',
      'javelin-behavior-phui-dropdown-menu',
      'javelin-behavior-doorkeeper-tag',
      'phabricator-title',
      'javelin-leader',
      'javelin-websocket',
      'javelin-behavior-dashboard-async-panel',
      'javelin-behavior-dashboard-tab-panel',
      'javelin-quicksand',
      'javelin-behavior-quicksand-blacklist',
      'javelin-behavior-high-security-warning',
      'javelin-scrollbar',
      'javelin-behavior-scrollbar',
      'javelin-behavior-durable-column',
      'conpherence-thread-manager',
    ),
    'darkconsole.pkg.js' => array(
      'javelin-behavior-dark-console',
      'javelin-behavior-error-log',
    ),
    'differential.pkg.css' => array(
      'differential-core-view-css',
      'differential-changeset-view-css',
      'differential-revision-history-css',
      'differential-revision-list-css',
      'differential-table-of-contents-css',
      'differential-revision-comment-css',
      'differential-revision-add-comment-css',
      'phabricator-object-selector-css',
      'phabricator-content-source-view-css',
      'inline-comment-summary-css',
      'phui-inline-comment-view-css',
    ),
    'differential.pkg.js' => array(
      'phabricator-drag-and-drop-file-upload',
      'phabricator-shaped-request',
      'javelin-behavior-differential-feedback-preview',
      'javelin-behavior-differential-edit-inline-comments',
      'javelin-behavior-differential-populate',
      'javelin-behavior-differential-diff-radios',
      'javelin-behavior-differential-comment-jump',
      'javelin-behavior-differential-add-reviewers-and-ccs',
      'javelin-behavior-differential-keyboard-navigation',
      'javelin-behavior-aphront-drag-and-drop-textarea',
      'javelin-behavior-phabricator-object-selector',
      'javelin-behavior-repository-crossreference',
      'javelin-behavior-load-blame',
      'differential-inline-comment-editor',
      'javelin-behavior-differential-dropdown-menus',
      'javelin-behavior-differential-toggle-files',
      'javelin-behavior-differential-user-select',
      'javelin-behavior-aphront-more',
      'changeset-view-manager',
    ),
    'diffusion.pkg.css' => array(
      'diffusion-icons-css',
    ),
    'diffusion.pkg.js' => array(
      'javelin-behavior-diffusion-pull-lastmodified',
      'javelin-behavior-diffusion-commit-graph',
      'javelin-behavior-audit-preview',
    ),
    'maniphest.pkg.css' => array(
      'maniphest-task-summary-css',
    ),
    'maniphest.pkg.js' => array(
      'javelin-behavior-maniphest-batch-selector',
      'javelin-behavior-maniphest-subpriority-editor',
      'javelin-behavior-maniphest-list-editor',
    ),
  ),
);
