/**
 * @provides conpherence-menu-css
 */

.conpherence-layout {
  position: fixed;
  bottom: 0;
  left: 0;
  right: 0;
  top: 44px;
  background: #fff;
}

.page-has-warning .conpherence-layout {
  top: 76px;
}

.conpherence-layout .conpherence-no-threads {
  text-align: center;
  position: fixed;
  left: 240px;
  right: 241px;
  top: 76px;
  bottom: 0px;
  min-width: 300px;
  width: auto;
}

.device .conpherence-layout .conpherence-no-threads {
  left: 0;
  right: 0;
  width: 100%;
}

.conpherence-layout .conpherence-no-threads .text {
  margin: 0px 0px 16px 0px;
}

.conpherence-menu-pane {
  width: 100%;
  position: absolute;
  overflow-x: hidden;
  overflow-y: auto;
  top: 0;
  bottom: 0;
  background: #f7f7f7;
  border-right: 1px solid {$lightblueborder}
}

.conpherence-menu-pane .phui-list-item-view.hidden {
  display: none;
}

.conpherence-menu-pane.phabricator-side-menu .phui-list-item-type-label {
  padding: 10px 0 9px 8px;
}

.conpherence-menu-pane .conpherence-room-list-header
.phui-icon-view {
  font-weight: bold;
  float: right;
  text-transform: none;
  margin: 0px 8px 0px 0px;
}

.conpherence-menu-pane .conpherence-room-list-header
.phui-icon-view:hover {
  color: {$sky};
}

.conpherence-menu-pane .conpherence-message-list-header {
  margin-top: 12px;
}

.device-desktop .conpherence-layout .conpherence-menu-pane {
  width: 240px;
}

.device .conpherence-menu-pane {
  top: 41px;
}

.device .conpherence-role-list .conpherence-menu-pane {
  top: 0px;
}

.device-phone .conpherence-menu-pane {
  -webkit-overflow-scrolling: touch;
}

.device .conpherence-role-thread .conpherence-menu-pane {
  display: none;
}

.device-desktop .conpherence-content-pane {
  margin-left: 241px;
}

.conpherence-content-pane {
  display: none;
  margin-left: 0px;
  position: relative;
}

.device-desktop .conpherence-content-pane,
.device .conpherence-role-thread .conpherence-content-pane {
  display: block;
}

.conpherence-menu .conpherence-menu-item-view {
  display: block;
  overflow: hidden;
  position: relative;
  text-decoration: none;
  border-left: 4px solid transparent;
  padding: 4px;
}

.conpherence-menu .conpherence-selected {
  background: rgba(0,0,0,0.05);
  border-left: 4px solid {$sky};
}

.conpherence-menu .phui-list-item-type-link .phui-list-item-href {
  padding: 8px 0 8px 8px;
}

.device-desktop .conpherence-menu
 .conpherence-selected.conpherence-menu-item-view:hover {
  background-color: rgba(0,0,0,0.07);
}

.conpherence-menu .loading {
  font-style: italic;
}

.device-desktop .conpherence-menu .conpherence-menu-item-view:hover {
  background-color: rgba(0,0,0,0.05);
}

.conpherence-menu .conpherence-menu-item-view .conpherence-menu-item-image {
  width: 26px;
  height: 26px;
  background-size: 26px;
  box-shadow: {$borderinset};
  border-radius: 3px;
  float: left;
  margin-left: 4px;
}

.conpherence-menu .conpherence-menu-item-view .conpherence-menu-item-title {
  display: block;
  padding: 4px 0 4px 8px;
  float: left;
  text-align: left;
  font-weight: bold;
  font-size: {$normalfontsize};
  color: {$darkbluetext};
  text-shadow: 0px 1px 1px #fff;
  overflow: hidden;
  width: 175px;
  text-overflow: ellipsis;
}

.conpherence-menu .conpherence-menu-item-view .conpherence-menu-item-subtitle {
  display: none;
}

.conpherence-menu .conpherence-menu-item-view
.conpherence-menu-item-unread-count {
  position: absolute;
  right: 4px;
  top: 10px;
  background: {$blue};
  border-radius: 2px;
  color: #fff;
  font-weight: bold;
  padding: 0 5px 1px;
  font-size: {$smallestfontsize};
}

.conpherence-menu .hide-unread-count .conpherence-menu-item-unread-count,
.conpherence-menu .conpherence-selected .conpherence-menu-item-unread-count {
  display: none;
}

.conpherence-menu .conpherence-menu-item-view .conpherence-menu-item-date {
  position: absolute;
  top: 15px;
  right: 16px;
  color: {$bluetext};
  font-size: {$smallestfontsize};
}

.conpherence-menu-item-header {
  font-weight: bold;
  text-transform: uppercase;
  color: {$bluetext};
}

.no-conpherences-menu-item {
  color: {$bluetext};
  padding: 4px 12px;
}
