/**
 * @provides owners-path-editor-css
 */

.owners-path-editor-table {
  margin: 10px;
}

.owners-path-editor-table td {
  padding: 2px 4px;
  vertical-align: middle;
}

.owners-path-editor-table select.owners-repo {
  width: 150px;
}

.owners-path-editor-table input {
  width: 400px;
}

.owners-path-editor-table div.error-display {
  padding: 4px 12px 0;
}

.owners-path-editor-table div.validating {
  color: {$greytext};
}

.owners-path-editor-table div.invalid {
  color: #aa0000;
}

.owners-path-editor-table div.valid {
  color: #00aa00;
  font-weight: bold;
}
