ALTER TABLE {$NAMESPACE}_phurl.phurl_url
  ADD authorPHID VARBINARY(64) NOT NULL;

ALTER TABLE {$NAMESPACE}_phurl.phurl_url
  CHANGE description description LONGTEXT NOT NULL COLLATE {$COLLATE_TEXT},
  CHANGE longURL longURL LONGTEXT NOT NULL COLLATE {$COLLATE_TEXT};
