<?php

final class PhabricatorPirateEnglishTranslation
  extends PhutilTranslation {

  public function getLocaleCode() {
    return 'en_P*';
  }

  protected function getTranslations() {
    return array(
      'Search' => 'Scour',
      'Review Code' => 'Inspect Riggins',
      'Tasks and Bugs' => 'Bilge rats',
      'Cancel' => 'Belay',
      'Advanced Search' => 'Scour Hard',
      'No search results.' => 'We be finding nothin.',
      'Send' => 'Aye!',
      'Partial' => 'Parrtial',
      'Upload' => 'Hoist',
      'Partial Upload' => 'Parrtial Hoist',
      'Submit' => 'Aye!',
      'Create' => 'Make Sail',
      'Okay' => 'Ahoy!',
      'Edit Query' => 'Overhaul Query',
      'Hide Query' => 'Furl Query',
      'Execute Query' => 'Quarter Query',
      'Wiki' => 'Sea Log',
      'Blog' => 'Capn\'s Tales',
      'Add Action...' => 'Be Addin\' an Action...',
      'Change Subscribers' => 'Change Spies',
      'Change Projects' => 'Change Prrojects',
      'Change Priority' => 'Change Priarrrity',
      'Change Status' => 'Change Ye Status',
      'Assign / Claim' => 'Arrsign / Stake Claim',
      'Prototype' => 'Ramshackle',
      'Continue' => 'Set Sail',
      'Recent Activity' => 'Recent Plunderin\'s',
      'Browse and Audit Commits' => 'Inspect ye Work of Yore',
      'Upcoming Events' => 'Upcoming Pillages',
      'Get Organized' => 'Straighten yer gig',
      'Host and Browse Repositories' => 'Hide ye Treasures',
      'Chat with Others' => 'Parley with yer Mates',
      'Review Recent Activity' => 'Spy ye Freshest Bottles',
      'Comment' => 'Scrawl',
      'Actions' => 'Actions! Arrr!',
      'Title' => 'Ye Olde Title',
      'Assigned To' => 'Matey Assigned',
      'Status' => 'Ye Status',
      'Priority' => 'Priarrrity',
      'Description' => 'Splainin\'',
      'Visible To' => 'Bein\' Spied By',
      'Editable By' => 'Plunderable By',
      'Subscribers' => 'Spies',
      'Projects' => 'Prrojects',
      '%s added a comment.' => '%s scrawled.',
      '%s edited the task description.' =>
        'Cap\'n %s be updatin\' ye task\'s splainin\'.',
      '%s claimed this task.' =>
        'Cap\'n %s be stakin\' a claim on this here task.',
      '%s created this task.' =>
        'Cap\'n %s be the one creatin\' this here task.',
    );
  }
}
