/**
 * @provides differential-revision-comment-css
 */

/*  Spooky haunted panel which floats on the bottom of the screen.
    Haunt modes are:

      - Mode 1: Just the comment box.
      - Mode 2: Comment box, comment preview, and inline comment previews.
*/
.differential-haunt-mode-1 .differential-add-comment-panel,
.differential-haunt-mode-2 .differential-add-comment-panel {
  background-color: #fff;
  position: fixed;
  width: 100%;
  bottom: 0;
  right: 0;
  left: 0;
  overflow: auto;
  max-height: 375px;
  margin: 0;

  box-shadow: 0 0 4px #000;
  -webkit-box-shadow: 0 0 4px #000;
  -moz-box-shadow: 0 0 4px #000;
}

.differential-haunt-mode-1 .differential-add-comment-panel .phui-box,
.differential-haunt-mode-2 .differential-add-comment-panel .phui-box {
  margin: 0;
  border-width: 0;
}

.differential-haunt-mode-2 .differential-add-comment-panel {
  max-height: 75%;
}

.differential-haunt-mode-1 .differential-add-comment-panel .phui-header-shell,
.differential-haunt-mode-2 .differential-add-comment-panel .phui-header-shell {
  display: none;
}

.differential-haunt-mode-1 .aphront-panel-preview {
  display: none;
}

.differential-haunt-mode-1 {
  padding-bottom: 250px;
}
