/**
 * @provides phriction-document-css
 */

.phriction-document-crumbs a {
  font-weight: bold;
}

.phriction-revert-table {
  width: 100%;
}

.phriction-revert-table td {
  text-align: center;
  width: 50%;
  padding: 8px 0;
  margin: 0 16px;
  border-top: 1px solid {$lightblueborder};
}

.phriction-history-nav-table {
  width: 100%;
}

.phriction-history-nav-table td {
  width: 50%;
  color: {$darkgreytext};
  padding: 12px;
  background-color: {$lightgreybackground};
}

.phriction-history-nav-table td.nav-next {
  text-align: right;
}

.phui-document-content .phriction-link {
  font-weight: bold;
}
