/**
 * @provides phabricator-action-list-view-css
 */

.device .phabricator-action-list-view {
  padding: 4px 0;
  display: none;
}

.device .phuix-dropdown-menu .phabricator-action-list-view {
  /* When an action list view appears inside a dropdown menu, don't hide it
     by default. */
  display: block;
}

.device .phabricator-action-list-view.phabricator-action-list-toggle,
.device-desktop .phui-document-content
  .phabricator-action-list-view.phabricator-action-list-toggle {
  display: block;
  width: 200px;
  border: 1px solid {$lightblueborder};
  border-radius: 3px;
  position: absolute;
  right: 16px;
  top: 42px;
  background: #fff;
  box-shadow: {$dropshadow};
  padding: 4px 0;
}

.device-phone .phabricator-action-list-view.phabricator-action-list-toggle {
  right: 8px;
  top: 38px;
}

.phabricator-action-view {
  position: relative;
}

.phabricator-action-view button.phabricator-action-view-item {
  border: none;
  background: transparent;
  box-shadow: none;
  outline: 0;
  padding: 0;
  margin: 0;
  font-weight: normal;
  width: 100%;
  text-align: left;
  text-shadow: none;

  border-radius: 0;
  color: {$anchor};
  font: inherit;
  display: inline;
  min-width: 0;
}

.phabricator-action-view button.phabricator-action-view-item,
.phabricator-action-view-item {
  padding: 4px 4px 4px 28px;
  display: block;
  text-decoration: none;
  color: {$darkbluetext};
}

.phabricator-action-view-label .phabricator-action-view-item {
  font-size: {$normalfontsize};
  font-weight: bold;
  color: {$bluetext};
  padding: 4px 8px 6px 8px;
  display: block;
  -webkit-font-smoothing: antialiased;
}

.device-desktop li.phabricator-action-view-label:hover
  .phabricator-action-view-item {
    background-color: #fff;
    color: {$bluetext};
}

.phabricator-action-view + .phabricator-action-view-label {
  padding-top: 8px;
}

.phabricator-action-view-icon {
  width: 14px;
  height: 14px;
  position: absolute;
  top: 6px;
  left: 9px;
  text-align: center;
}

.device-desktop .phabricator-action-view:hover .phabricator-action-view-item {
  text-decoration: none;
  background-color: {$sky};
  color: #fff;
}

.device-desktop .phabricator-action-view:hover .phabricator-action-view-icon {
  color: #fff;
}

.phabricator-action-view-disabled .phabricator-action-view-item,
.phabricator-action-view-disabled button.phabricator-action-view-item {
  color: {$lightgreytext};
}

.phabricator-action-view-selected {
  border-left: 3px solid {$sky};
  background: {$hoverblue};
}

.phabricator-action-view-selected .phabricator-action-view-item {
  padding-left: 25px;
}

.phabricator-action-view-selected .phabricator-action-view-icon {
  left: 6px;
}

.phabricator-action-view button[disabled] {
  opacity: 1.0;
}

.device-desktop .phabricator-action-view-disabled:hover
  .phabricator-action-view-item,
.device-desktop .phabricator-action-view-disabled:hover
  button.phabricator-action-view-item,
.device-desktop .phabricator-action-view-disabled:hover
  .phabricator-action-view-icon,
.device-desktop .phabricator-action-view-disabled:hover
  button.phabricator-action-view-icon {
  background-color: {$greybackground};
  color: {$lightgreytext};
}
