/**
 * @provides phabricator-chatlog-css
 */

.device-phone .phabricator-chat-log-wrap {
  padding: 0;
}

.phabricator-chat-log-pager-bottom {
  padding: 8px 4px 16px;
  font-weight: bold;
  float: right;
}

.phabricator-chat-log-pager-bottom a {
  padding: 2px 3px;
}

.phabricator-chat-log-panel {
  clear: both;
}

.phabricator-chat-log {
  width: 100%;
}

.phabricator-chat-log td {
  padding: 8px;
  line-height: 18px;
}

.phabricator-chat-log tr {
  background: #fff;
}

.phabricator-chat-log tr td.author {
  background: {$greybackground};
}

.phabricator-chat-log tr.alternate {
  border-top: 1px solid {$thinblueborder};
  border-bottom: 1px solid {$thinblueborder};
}

.phabricator-chat-log tr.alternate td.author {
  background: {$lightgreybackground};
}

.phabricator-chat-log tr.highlight td {
  background: {$lightyellow};
}

.phabricator-chat-log td.timestamp {
  white-space: nowrap;
  text-align: right;
  width: 12em;
}

.phabricator-chat-log td.message .timestamp {
  color: {$bluetext};
  font-size: {$smallestfontsize};
  float: right;
  margin-left: 5px;
}

.phabricator-chat-log td.author {
  white-space: nowrap;
  text-align: right;
  font-weight: bold;
  width: 140px;
  color: {$darkbluetext};
}

.device-phone .phabricator-chat-log td.author {
  width: 80px;
}

.phabricator-chat-log td.message {
  white-space: pre-wrap;
  word-break: break-word;
}
