/**
 * @provides aphront-typeahead-control-css
 */

div.jx-typeahead-hardpoint {
  position: relative;
  _zoom: 1; /* Some kind of IE6 fix? */ /* yes */ /* why? */
}

div.jx-typeahead-results {
  position: absolute;
  border: 1px solid {$hoverborder};
  border-top: 0px;
  padding: 0;
  background: #fefefe;
  width: 98%;
  box-shadow: 0px 1px 2px rgba(0, 0, 0, 0.2);
  margin: -1px 1% 0;
}

.aphront-form-control-typeahead div.jx-typeahead-results {
  width: 100%;
  margin: 0;
  box-sizing: border-box;
}

div.jx-typeahead-results a.jx-result {
  color: {$darkgreytext};
  display: block;
  font-size: {$normalfontsize};
  border-top: 1px solid {$hoverborder};
}

div.jx-typeahead-results a.jx-result:hover,
div.jx-typeahead-results a.focused {
  display: block;
  background: {$hoverblue};
  text-decoration: none;
}

div.jx-typeahead-results .phui-icon-view {
  margin-top: 2px;
}

table.jx-typeahead button {
  margin-left: 3px;
}

table.jx-typeahead input {
  font-size: {$normalfontsize};
  padding: 2px;
}

input.jx-typeahead-placeholder {
  color: {$lightgreytext};
}

div.jx-tokenizer-container-focused.jx-typeahead-waiting {
  border-color: {$lightblueborder};
}

div.jx-typeahead-results a.diffusion-locate-file {
  padding: 4px 8px;
  color: {$darkgreytext}
}

.diffusion-locate-file strong {
  color: {$blue};
}

.diffusion-locate-file .phui-icon-view {
  padding-right: 8px;
}
