/**
 * @provides differential-revision-history-css
 */

.differential-update-history-footer {
  text-align: right;
  vertical-align: middle;
  padding: 4px 8px;
  border-top: 1px solid {$lightblueborder};
  background: {$lightbluebackground};
}

.differential-update-history-footer label {
  margin-right: 12px;
  color: {$greytext};
}

.differential-update-history-footer select {
  margin-left: 8px;
}

.device .differential-update-history-footer label {
  display: block;
  margin-right: 0;
  margin-bottom: 4px;
}

.differential-update-history-radio {
  padding: 0 8px;
}

.aphront-table-view td.differential-update-history-old {
  background: #f9d0d0;
}

.aphront-table-view td.differential-update-history-new {
  background: #d0ffd0;
}

.aphront-table-view
  .differential-update-history-old-now
  td.differential-update-history-old,
.aphront-table-view
  .alt-differential-update-history-old-now
  td.differential-update-history-old {
  background: #ffaaaa;
}

.aphront-table-view
  .differential-update-history-new-now
  td.differential-update-history-new,
.aphront-table-view
  .alt-differential-update-history-new-now
  td.differential-update-history-new {
  background: #aaffaa;
}

.lintunit-star {
  text-align: center;
  padding: 0 16px;
}
