/**
 * @provides font-lato
 * @requires phui-fontkit-css
 */

@font-face {
    font-family: 'Lato';
    src: url(/rsrc/externals/font/lato/lato-regular.eot);
    src: url(/rsrc/externals/font/lato/lato-regular.eot?#iefix)
          format('embedded-opentype'),
         url(/rsrc/externals/font/lato/lato-regular.woff2)
          format('woff2'),
         url(/rsrc/externals/font/lato/lato-regular.woff)
          format('woff'),
         url(/rsrc/externals/font/lato/lato-regular.ttf)
          format('truetype'),
         url(/rsrc/externals/font/lato/lato-regular.svg#latoregular)
          format('svg');
    font-weight: normal;
    font-style: normal;

}

@font-face {
    font-family: 'Lato';
    src: url(/rsrc/externals/font/lato/lato-italic.eot);
    src: url(/rsrc/externals/font/lato/lato-italic.eot?#iefix)
          format('embedded-opentype'),
         url(/rsrc/externals/font/lato/lato-italic.woff2)
          format('woff2'),
         url(/rsrc/externals/font/lato/lato-italic.woff)
          format('woff'),
         url(/rsrc/externals/font/lato/lato-italic.ttf)
          format('truetype'),
         url(/rsrc/externals/font/lato/lato-italic.svg#latoitalic)
          format('svg');
    font-weight: normal;
    font-style: italic;

}

@font-face {
    font-family: 'Lato';
    src: url(/rsrc/externals/font/lato/lato-bold.eot);
    src: url(/rsrc/externals/font/lato/lato-bold.eot?#iefix)
          format('embedded-opentype'),
         url(/rsrc/externals/font/lato/lato-bold.woff2)
          format('woff2'),
         url(/rsrc/externals/font/lato/lato-bold.woff)
          format('woff'),
         url(/rsrc/externals/font/lato/lato-bold.ttf)
          format('truetype'),
         url(/rsrc/externals/font/lato/lato-bold.svg#latobold)
          format('svg');
    font-weight: bold;
    font-style: normal;

}

@font-face {
    font-family: 'Lato';
    src: url(/rsrc/externals/font/lato/lato-bolditalic.eot);
    src: url(/rsrc/externals/font/lato/lato-bolditalic.eot?#iefix)
          format('embedded-opentype'),
         url(/rsrc/externals/font/lato/lato-bolditalic.woff2)
          format('woff2'),
         url(/rsrc/externals/font/lato/lato-bolditalic.woff)
          format('woff'),
         url(/rsrc/externals/font/lato/lato-bolditalic.ttf)
          format('truetype'),
         url(/rsrc/externals/font/lato/lato-bolditalic.svg#latobold_italic)
          format('svg');
    font-weight: bold;
    font-style: italic;

}
