/**
 * @provides phabricator-standard-page-view
 */

.phabricator-anchor-view,
.phabricator-anchor-navigation-marker {
  position: absolute;
  margin-top: -15px;
}

.phabricator-chromeless-page .phabricator-standard-page {
  background: transparent;
  border-width: 0px;
}

.phabricator-standard-page-body {
  clear: both;
}

.phabricator-standard-page-footer {
  text-align: right;
  margin: 32px 16px 16px;
  padding: 12px 0;
  border-top: 1px solid rgba(71, 87, 120, 0.20);
  color: {$greytext};
}

.device .phabricator-standard-page-footer {
  margin: 4px 8px;
}

!print .phabricator-standard-page-footer {
  display: none;
}

.device-desktop .has-local-nav + .phabricator-standard-page-footer {
  margin-left: 221px;
}

.device .phabricator-side-menu-home + .phabricator-standard-page-footer {
  display: none;
}

.keyboard-shortcut-help td,
.keyboard-shortcut-help th {
  padding: 8px;
  vertical-align: middle;
}

.keyboard-shortcut-help th {
  white-space: nowrap;
  color: {$greytext};
}

.keyboard-shortcut-help kbd {
  background: #222222;
  padding: 6px;
  color: #ffffff;
  font-weight: bold;
  border: 1px solid #555555;
}

.keyboard-focus-focus-reticle {
  background: #ffffd3;
  position: absolute;
  border: 1px solid #999900;
}

a.handle-status-closed {
  text-decoration: line-through;
  color: #676767;
}

a.handle-status-closed:hover {
  text-decoration: line-through;
  color: #19558D;
}

a.handle-availability-disabled,
a.handle-availability-none,
a.handle-availability-partial {
  padding-left: 11px;
  background-repeat: no-repeat;
  background-position: -4px center;
}

a.handle-availability-none {
  background-image: url(/rsrc/image/icon/fatcow/bullet_red.png);
}

a.handle-availability-partial {
  background-image: url(/rsrc/image/icon/fatcow/bullet_orange.png);
}

a.handle-availability-disabled {
  background-image: url(/rsrc/image/icon/fatcow/bullet_black.png);
}

.aphront-developer-error-callout {
  position: relative;
  padding: 2em;
  background: #aa0000;
  color: white;
  text-align: center;
  font-size: {$smallerfontsize};
}

.setup-warning-callout {
  padding: 8px 16px;
  background: {$lightred};
  border-bottom: 1px solid {$sh-redborder};
  position: relative;
}

.setup-warning-callout a {
  color: {$red};
}

.phui-handle .phui-icon-view {
  display: inline-block;
  margin: 2px 2px -2px 0;
}

.jx-scrollbar-frame {
  position: relative;
  overflow: hidden;
}

.jx-scrollbar-viewport {
  position: absolute;
  overflow-x: hidden;
  overflow-y: scroll;
  top: 0;
  bottom: 0;
  left: 0;
  right: 0;
}

/* Fixes so pages actually print when magic scrollbar is present */
!print .main-page-frame {
  position: static;
  overflow: visible;
}

!print .jx-scrollbar-viewport {
  position: static;
  width: auto !important;
  height: auto !important;
}

.jx-scrollbar-test {
  position: absolute;
  left: -300px;
}

.jx-scrollbar-bar {
  position: absolute;
  top: 0;
  right: 0;
  bottom: 7px;
  width: 11px;
}

.jx-scrollbar-bar .jx-scrollbar-handle {
  position: absolute;
  right: 2px;
  -webkit-border-radius: 7px;
  -moz-border-radius: 7px;
  border-radius: 7px;
  min-height: 10px;
  width: 7px;
  opacity: 0;
  -webkit-transition: opacity 0.2s linear;
  -moz-transition: opacity 0.2s linear;
  -o-transition: opacity 0.2s linear;
  -ms-transition: opacity 0.2s linear;
  transition: opacity 0.2s linear;
  background: #6c6e71;
  -webkit-background-clip: padding-box;
  -moz-background-clip: padding;
}

.jx-scrollbar-bar:hover .jx-scrollbar-handle {
  opacity: 0.7;
  -webkit-transition: opacity 0 linear;
  -moz-transition: opacity 0 linear;
  -o-transition: opacity 0 linear;
  -ms-transition: opacity 0 linear;
  transition: opacity 0 linear;
}

.jx-scrollbar-bar .jx-scrollbar-visible {
  opacity: 0.7;
}

.jx-scrollbar-link {
  position: absolute;
  left: -50px;
}
