/**
 * @provides paste-css
 */

.container-of-paste {
  margin: 16px 16px 0 16px;
}

.device .container-of-paste {
  margin: 8px 8px 0 8px;
}

.paste-embed {
  background: {$sh-yellowbackground};
  border: 1px solid {$sh-lightyellowborder};
  border-radius: 3px;
}

.paste-embed .phabricator-source-code-container {
  border: none;
}

.paste-embed-head {
  border-bottom: 1px solid {$sh-lightyellowborder};
  padding: 8px 12px;
}

.paste-embed-head a {
  color: {$darkbluetext};
  font-weight: bold;
}

.paste-embed-body {
  overflow-y: auto;
}
