/**
 * @provides aphront-multi-column-view-css
 */

.aphront-multi-column-view {
  width: 100%;
}

.device .aphront-multi-column-outer {
  padding: 0;
  margin: 0 auto;
  width: 100%;
}

.aphront-multi-column-inner .aphront-multi-column-column-last,
.device .aphront-multi-column-column {
  margin: 0;
}

.device-desktop .phui-workboard-view-shadow .aphront-multi-column-inner
  .aphront-multi-column-column-last {
  margin-right: 8px;
}

.device .aphront-multi-column-outer
  div.aphront-multi-column-column-outer {
  width: auto;
  margin-bottom: 16px;
}

.device .aphront-multi-column-fixed .aphront-multi-column-inner {
   width: 100%;
}

.aphront-multi-column-fixed .aphront-multi-column-column-outer {
  width: 300px;
}

/* flexible, but with a minimum */
.device-desktop .aphront-multi-column-fluidish .aphront-multi-column-1-up {
  min-width: 200px;
}

.device-desktop .aphront-multi-column-fluidish .aphront-multi-column-2-up {
  min-width: 400px;
}

.device-desktop .aphront-multi-column-fluidish .aphront-multi-column-3-up {
  min-width: 600px;
}

.device-desktop .aphront-multi-column-fluidish .aphront-multi-column-4-up {
  min-width: 800px;
}

.device-desktop .aphront-multi-column-fluidish .aphront-multi-column-5-up {
  min-width: 1000px;
}

.device-desktop .aphront-multi-column-fluidish .aphront-multi-column-6-up {
  min-width: 1200px;
}

.device-desktop .aphront-multi-column-fluidish .aphront-multi-column-7-up {
  min-width: 1400px;
}

/* table division */
.aphront-multi-column-fluid .aphront-multi-column-1-up
  .aphront-multi-column-column-outer {
  width: 100%;
}

.aphront-multi-column-fluid .aphront-multi-column-2-up
  .aphront-multi-column-column-outer {
  width: 50%;
}

.aphront-multi-column-fluid .aphront-multi-column-3-up
  .aphront-multi-column-column-outer {
  width: 33.3333%;
}

.aphront-multi-column-fluid .aphront-multi-column-4-up
  .aphront-multi-column-column-outer {
  width: 25%;
}

.aphront-multi-column-fluid .aphront-multi-column-5-up
  .aphront-multi-column-column-outer {
  width: 20%;
}

.aphront-multi-column-fluid .aphront-multi-column-6-up
  .aphront-multi-column-column-outer {
  width: 16.6666%;
}

.aphront-multi-column-fluid .aphront-multi-column-7-up
  .aphront-multi-column-column-outer {
  width: 14.2857%;
}

/* Shadow Panels */

.aphront-multi-column-inner {
  display: table;
  width: 100%;
}

.device-desktop .aphront-multi-column-fixed .aphront-multi-column-inner {
  margin: 0 0 12px 12px;
}

/* magic sauce */
.aphront-multi-column-fluid .aphront-multi-column-inner {
  table-layout: fixed;
}

.device .aphront-multi-column-inner {
  display: block;
  width: auto;
}

.aphront-multi-column-column-outer {
  display: table-cell;
  height: 100%;
  vertical-align: top;
}

.phui-box-border .aphront-multi-column-column-outer {
  border-right: 1px solid {$lightblueborder};
  padding-left: 8px;
  padding-right: 8px;
}

.phui-box-border .aphront-multi-colum-column-outer-last {
  border: none;
}

.device .aphront-multi-column-column-outer {
  display: block;
  border: none;
}

.device .phui-box-border .aphront-multi-column-column-outer {
  padding-bottom: 16px;
  border-bottom: 1px solid #e7e7e7;
}

.device .phui-box-border .aphront-multi-colum-column-outer-last {
  border: none;
  padding-bottom: 0;
}

/* Make Dashboards reasonably display on homepage narrow widths */
@media (max-width: 1000px) {
  .device-desktop .dashboard-view .aphront-multi-column-inner {
    display: block;
    width: auto;
  }
  .device-desktop .dashboard-view .aphront-multi-column-column-outer {
    display: block;
    border: none;
  }
  .device-desktop .dashboard-view .aphront-multi-column-column.mlr {
    margin: 0;
  }
  .device-desktop .dashboard-view .aphront-multi-column-fluid
    .aphront-multi-column-2-up .aphront-multi-column-column-outer.half {
     width: auto;
     margin: 0 0 16px;
  }
  .device-desktop .dashboard-view .aphront-multi-column-fluid
    .aphront-multi-column-2-up .aphront-multi-column-column-outer.thirds {
     width: auto;
     margin: 0 0 16px;
  }
  .device-desktop .dashboard-view .aphront-multi-column-fluid
    .aphront-multi-column-2-up .aphront-multi-column-column-outer.third {
    width: auto;
    margin: 0 0 16px;
  }
}

/* Make Dashboards with Durable reasonably display on homepage narrow widths */
@media (max-width: 1300px) {
  .device-desktop.with-durable-column .dashboard-view
    .aphront-multi-column-inner {
      display: block;
      width: auto;
  }
  .device-desktop.with-durable-column .dashboard-view
    .aphront-multi-column-column-outer {
      display: block;
      border: none;
  }
  .device-desktop.with-durable-column .dashboard-view
    .aphront-multi-column-column.mlr {
      margin: 0;
  }
  .device-desktop.with-durable-column .dashboard-view
    .aphront-multi-column-fluid .aphront-multi-column-2-up
    .aphront-multi-column-column-outer.half {
      width: auto;
      margin: 0 0 16px;
  }
  .device-desktop.with-durable-column .dashboard-view
    .aphront-multi-column-fluid .aphront-multi-column-2-up
    .aphront-multi-column-column-outer.thirds {
      width: auto;
      margin: 0 0 16px;
  }
  .device-desktop.with-durable-column .dashboard-view
    .aphront-multi-column-fluid .aphront-multi-column-2-up
    .aphront-multi-column-column-outer.third {
      width: auto;
      margin: 0 0 16px;
  }
}
