/**
 * @provides diffusion-source-css
 */

.diffusion-source {
  width: 100%;
  font: 10px/13px "Menlo", "Consolas", "Monaco", monospace;
  background: #fff;
}

.diffusion-source tr.phabricator-source-highlight {
  background: {$sh-yellowbackground};
}

.diffusion-source th {
  text-align: right;
  vertical-align: top;
  background: {$lightgreybackground};
  color: {$bluetext};
  border-right: 1px solid {$thinblueborder};
}

.diffusion-source td {
  letter-spacing:     0.0083334px;
  vertical-align:     top;
  white-space:        pre-wrap;
  padding-bottom:     1px;
  padding-left:       8px;
  line-height:        16px;
  width:              100%;
}

.diffusion-browse-type-form {
  float: right;
}

.diffusion-blame-link,
.diffusion-rev-link {
  white-space: nowrap;
}

.diffusion-blame-link {
  min-width: 28px;
}

.diffusion-source th.diffusion-rev-link {
  text-align: left;
  min-width: 130px;
}

.diffusion-blame-link a,
.diffusion-rev-link a,
.diffusion-line-link a {
  color: {$darkbluetext};
}

.diffusion-rev-link a,
.diffusion-rev-link span {
  margin: 2px 8px 0;
  display: inline-block;
}

.diffusion-rev-link span {
  margin-right: -4px;
  margin-left: -4px;
  color: {$lightgreytext};
}

.diffusion-blame-link a,
.diffusion-line-link a {
  /* Give the user a larger click target. */
  display: block;
  padding: 2px 8px;
}

.diffusion-line-link {
  -moz-user-select: -moz-none;
  -khtml-user-select: none;
  -webkit-user-select: none;
  -ms-user-select: none;
  user-select: none;
}
