/**
 * @provides phui-workpanel-view-css
 */

.phui-workpanel-view .phui-header-shell {
  padding: 16px 0 8px;
  border-color: {$lightgreyborder};
  width: 300px;
  background-color: transparent;
}

.phui-workpanel-view .phui-header-shell .phui-header-header {
  font-size: {$biggerfontsize};
  line-height: 18px;
  color: {$darkbluetext};
}

.phui-workpanel-view .phui-header-shell .phui-header-subheader {
  padding: 0 4px;
  margin: 0;
  display: inline-block;
  color: {$lightgreytext};
}

.device-phone .phui-workboard-view {
  width: auto;
  margin: 0 8px;
}

.phui-workboard-view {
  -webkit-touch-callout: none;
  -webkit-user-select: none;
  -khtml-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none;
}

.phui-workboard-view .phui-object-item .phui-object-item-objname {
  -webkit-touch-callout: text;
  -webkit-user-select: text;
  -khtml-user-select: text;
  -moz-user-select: text;
  -ms-user-select: text;
  user-select: text;
}

.phui-workboard-view .phui-object-item-link {
  white-space: normal;
  display: inline;
  -webkit-font-smoothing: antialiased;
}

.phui-workboard-view .phui-object-item-objname {
  vertical-align: top;
}

.phui-workpanel-view .phui-workpanel-header-action {
  float: right;
  width: 24px;
  border-left: 1px solid #b3b5b6;
}

.phui-workpanel-view .phui-workpanel-body {
  padding-top: 8px;
}

.phui-workpanel-view .phui-workpanel-footer-action a {
  color: {$darkbluetext};
  font-weight: bold;
}

.device-desktop .phui-workpanel-view .phui-workpanel-footer-action:hover {
  background-color: rgba(100,100,100,.1);
  border-radius: 3px;
}

.phui-workpanel-view .phui-list-item-icon {
  height: 14px;
  width: 14px;
  display: inline-block;
}

.phui-workpanel-view .phui-list-item-name {
  padding-left: 5px;
  display: inline-block;
}

.aphront-multi-column-fixed .phui-workpanel-view {
  width: 300px;
}

.device-phone .aphront-multi-column-fixed .phui-workpanel-view,
.device-phone .phui-workpanel-view .phui-header-shell {
  width: auto;
}

.phui-workpanel-body .phui-object-item-list-view {
  min-height: 54px;
}

.device .aphront-multi-column-outer
  div.aphront-multi-column-column-outer .phui-workpanel-body {
  width: auto;
}

.project-panel-hidden {
  opacity: 0.75;
}

.project-panel-empty .phui-object-item-list-view {
  background: {$sh-indigobackground};
  border-radius: 3px;
  margin-bottom: 4px;
  border: 1px dashed {$sh-indigoborder};
}

.project-panel-empty .phui-object-item-list-view .drag-ghost {
  display: none;
}

.project-panel-empty .phui-object-item-list-view.drag-target-list {
  background: rgba(255,255,255,.7);
}

.phui-workpanel-view.project-panel-over-limit .phui-header-header {
  color: {$red};
}

.phui-workpanel-view.project-panel-over-limit .phui-header-shell {
  border-color: {$red};
}

/* - Workpanel Cards -----------------------------------------------------------

  Slight display changes for how cards work in tight spaces

*/

.phui-workpanel-view .phui-object-item-grippable .phui-object-item-frame {
  padding-left: 0;
}

.phui-workpanel-view .phui-object-item-grip {
  display: none;
}

.phui-workpanel-view .phui-object-item-attribute a {
  color: {$bluetext};
}
