CREATE TABLE {$NAMESPACE}_spaces.spaces_namespacetransaction (
  id INT UNSIGNED NOT NULL PRIMARY KEY AUTO_INCREMENT,
  phid VARBINARY(64) NOT NULL,
  authorPHID VARBINARY(64) NOT NULL,
  objectPHID VARBINARY(64) NOT NULL,
  viewPolicy VARBINARY(64) NOT NULL,
  editPolicy VARBINARY(64) NOT NULL,
  commentPHID VARBINARY(64),
  commentVersion INT UNSIGNED NOT NULL,
  transactionType VARCHAR(32) NOT NULL COLLATE {$COLLATE_TEXT},
  oldValue LONGTEXT NOT NULL COLLATE {$COLLATE_TEXT},
  newValue LONGTEXT NOT NULL COLLATE {$COLLATE_TEXT},
  contentSource LONGTEXT NOT NULL COLLATE {$COLLATE_TEXT},
  metadata LONGTEXT NOT NULL COLLATE {$COLLATE_TEXT},
  dateCreated INT UNSIGNED NOT NULL,
  dateModified INT UNSIGNED NOT NULL,
  UNIQUE KEY `key_phid` (phid),
  KEY `key_object` (objectPHID)
) ENGINE=InnoDB, COLLATE {$COLLATE_TEXT};
