<?php

final class DifferentialUnitStatus extends Phobject {

  const UNIT_NONE       = 0;
  const UNIT_OKAY       = 1;
  const UNIT_WARN       = 2;
  const UNIT_FAIL       = 3;
  const UNIT_SKIP       = 4;
  const UNIT_AUTO_SKIP  = 6;

}
