<?php

interface PhabricatorSpacesInterface extends PhabricatorPHIDInterface {

  public function getSpacePHID();

}

// TEMPLATE IMPLEMENTATION /////////////////////////////////////////////////////

/* -(  PhabricatorSpacesInterface  )----------------------------------------- */
/*

  public function getSpacePHID() {
    return $this->spacePHID;
  }

*/
