CREATE TABLE {$NAMESPACE}_harbormaster.harbormaster_buildunitmessage (
  id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
  buildTargetPHID VARBINARY(64) NOT NULL,
  engine VARCHAR(255) COLLATE {$COLLATE_TEXT} NOT NULL,
  namespace VARCHAR(255) COLLATE {$COLLATE_TEXT} NOT NULL,
  name VARCHAR(255) COLLATE {$COLLATE_TEXT} NOT NULL,
  result VARCHAR(32) COLLATE {$COLLATE_TEXT} NOT NULL,
  duration DOUBLE,
  properties LONGTEXT COLLATE {$COLLATE_TEXT} NOT NULL,
  dateCreated INT UNSIGNED NOT NULL,
  dateModified INT UNSIGNED NOT NULL,
  KEY `key_target` (buildTargetPHID)
) ENGINE=INNODB, COLLATE {$COLLATE_TEXT};
