<?php

abstract class PonderController extends PhabricatorController {

  protected function buildSideNavView($for_app = false) {
    $user = $this->getRequest()->getUser();

    $nav = new AphrontSideNavFilterView();
    $nav->setBaseURI(new PhutilURI($this->getApplicationURI()));

    if ($for_app) {
      $nav->addFilter('question/create/', pht('Ask Question'));
    }

    id(new PonderQuestionSearchEngine())
      ->setViewer($user)
      ->addNavigationItems($nav->getMenu());

    $nav->selectFilter(null);

    return $nav;
  }

  public function buildApplicationMenu() {
    return $this->buildSideNavView($for_app = true)->getMenu();
  }

  protected function buildApplicationCrumbs() {
    $crumbs = parent::buildApplicationCrumbs();
    $href = $this->getApplicationURI('question/create/');
    $crumbs
      ->addAction(
        id(new PHUIListItemView())
          ->setName(pht('Ask Question'))
          ->setHref($href)
          ->setIcon('fa-plus-square'));

    return $crumbs;
  }

}
