<?php

/**
 * This file is automatically generated. Use 'arc liberate' to rebuild it.
 *
 * @generated
 * @phutil-library-version 2
 */
phutil_register_library_map(array(
  '__library_version__' => 2,
  'class' => array(
    'AlmanacAddress' => 'applications/almanac/util/AlmanacAddress.php',
    'AlmanacBinding' => 'applications/almanac/storage/AlmanacBinding.php',
    'AlmanacBindingEditController' => 'applications/almanac/controller/AlmanacBindingEditController.php',
    'AlmanacBindingEditor' => 'applications/almanac/editor/AlmanacBindingEditor.php',
    'AlmanacBindingPHIDType' => 'applications/almanac/phid/AlmanacBindingPHIDType.php',
    'AlmanacBindingQuery' => 'applications/almanac/query/AlmanacBindingQuery.php',
    'AlmanacBindingTableView' => 'applications/almanac/view/AlmanacBindingTableView.php',
    'AlmanacBindingTransaction' => 'applications/almanac/storage/AlmanacBindingTransaction.php',
    'AlmanacBindingTransactionQuery' => 'applications/almanac/query/AlmanacBindingTransactionQuery.php',
    'AlmanacBindingViewController' => 'applications/almanac/controller/AlmanacBindingViewController.php',
    'AlmanacClusterDatabaseServiceType' => 'applications/almanac/servicetype/AlmanacClusterDatabaseServiceType.php',
    'AlmanacClusterRepositoryServiceType' => 'applications/almanac/servicetype/AlmanacClusterRepositoryServiceType.php',
    'AlmanacClusterServiceType' => 'applications/almanac/servicetype/AlmanacClusterServiceType.php',
    'AlmanacConduitAPIMethod' => 'applications/almanac/conduit/AlmanacConduitAPIMethod.php',
    'AlmanacConsoleController' => 'applications/almanac/controller/AlmanacConsoleController.php',
    'AlmanacController' => 'applications/almanac/controller/AlmanacController.php',
    'AlmanacCoreCustomField' => 'applications/almanac/customfield/AlmanacCoreCustomField.php',
    'AlmanacCreateClusterServicesCapability' => 'applications/almanac/capability/AlmanacCreateClusterServicesCapability.php',
    'AlmanacCreateDevicesCapability' => 'applications/almanac/capability/AlmanacCreateDevicesCapability.php',
    'AlmanacCreateNetworksCapability' => 'applications/almanac/capability/AlmanacCreateNetworksCapability.php',
    'AlmanacCreateServicesCapability' => 'applications/almanac/capability/AlmanacCreateServicesCapability.php',
    'AlmanacCustomField' => 'applications/almanac/customfield/AlmanacCustomField.php',
    'AlmanacCustomServiceType' => 'applications/almanac/servicetype/AlmanacCustomServiceType.php',
    'AlmanacDAO' => 'applications/almanac/storage/AlmanacDAO.php',
    'AlmanacDevice' => 'applications/almanac/storage/AlmanacDevice.php',
    'AlmanacDeviceController' => 'applications/almanac/controller/AlmanacDeviceController.php',
    'AlmanacDeviceEditController' => 'applications/almanac/controller/AlmanacDeviceEditController.php',
    'AlmanacDeviceEditor' => 'applications/almanac/editor/AlmanacDeviceEditor.php',
    'AlmanacDeviceListController' => 'applications/almanac/controller/AlmanacDeviceListController.php',
    'AlmanacDevicePHIDType' => 'applications/almanac/phid/AlmanacDevicePHIDType.php',
    'AlmanacDeviceQuery' => 'applications/almanac/query/AlmanacDeviceQuery.php',
    'AlmanacDeviceSearchEngine' => 'applications/almanac/query/AlmanacDeviceSearchEngine.php',
    'AlmanacDeviceTransaction' => 'applications/almanac/storage/AlmanacDeviceTransaction.php',
    'AlmanacDeviceTransactionQuery' => 'applications/almanac/query/AlmanacDeviceTransactionQuery.php',
    'AlmanacDeviceViewController' => 'applications/almanac/controller/AlmanacDeviceViewController.php',
    'AlmanacDrydockPoolServiceType' => 'applications/almanac/servicetype/AlmanacDrydockPoolServiceType.php',
    'AlmanacInterface' => 'applications/almanac/storage/AlmanacInterface.php',
    'AlmanacInterfaceDatasource' => 'applications/almanac/typeahead/AlmanacInterfaceDatasource.php',
    'AlmanacInterfaceEditController' => 'applications/almanac/controller/AlmanacInterfaceEditController.php',
    'AlmanacInterfacePHIDType' => 'applications/almanac/phid/AlmanacInterfacePHIDType.php',
    'AlmanacInterfaceQuery' => 'applications/almanac/query/AlmanacInterfaceQuery.php',
    'AlmanacInterfaceTableView' => 'applications/almanac/view/AlmanacInterfaceTableView.php',
    'AlmanacKeys' => 'applications/almanac/util/AlmanacKeys.php',
    'AlmanacManagementLockWorkflow' => 'applications/almanac/management/AlmanacManagementLockWorkflow.php',
    'AlmanacManagementRegisterWorkflow' => 'applications/almanac/management/AlmanacManagementRegisterWorkflow.php',
    'AlmanacManagementTrustKeyWorkflow' => 'applications/almanac/management/AlmanacManagementTrustKeyWorkflow.php',
    'AlmanacManagementUnlockWorkflow' => 'applications/almanac/management/AlmanacManagementUnlockWorkflow.php',
    'AlmanacManagementUntrustKeyWorkflow' => 'applications/almanac/management/AlmanacManagementUntrustKeyWorkflow.php',
    'AlmanacManagementWorkflow' => 'applications/almanac/management/AlmanacManagementWorkflow.php',
    'AlmanacNames' => 'applications/almanac/util/AlmanacNames.php',
    'AlmanacNamesTestCase' => 'applications/almanac/util/__tests__/AlmanacNamesTestCase.php',
    'AlmanacNetwork' => 'applications/almanac/storage/AlmanacNetwork.php',
    'AlmanacNetworkController' => 'applications/almanac/controller/AlmanacNetworkController.php',
    'AlmanacNetworkEditController' => 'applications/almanac/controller/AlmanacNetworkEditController.php',
    'AlmanacNetworkEditor' => 'applications/almanac/editor/AlmanacNetworkEditor.php',
    'AlmanacNetworkListController' => 'applications/almanac/controller/AlmanacNetworkListController.php',
    'AlmanacNetworkPHIDType' => 'applications/almanac/phid/AlmanacNetworkPHIDType.php',
    'AlmanacNetworkQuery' => 'applications/almanac/query/AlmanacNetworkQuery.php',
    'AlmanacNetworkSearchEngine' => 'applications/almanac/query/AlmanacNetworkSearchEngine.php',
    'AlmanacNetworkTransaction' => 'applications/almanac/storage/AlmanacNetworkTransaction.php',
    'AlmanacNetworkTransactionQuery' => 'applications/almanac/query/AlmanacNetworkTransactionQuery.php',
    'AlmanacNetworkViewController' => 'applications/almanac/controller/AlmanacNetworkViewController.php',
    'AlmanacPropertiesDestructionEngineExtension' => 'applications/almanac/engineextension/AlmanacPropertiesDestructionEngineExtension.php',
    'AlmanacProperty' => 'applications/almanac/storage/AlmanacProperty.php',
    'AlmanacPropertyController' => 'applications/almanac/controller/AlmanacPropertyController.php',
    'AlmanacPropertyDeleteController' => 'applications/almanac/controller/AlmanacPropertyDeleteController.php',
    'AlmanacPropertyEditController' => 'applications/almanac/controller/AlmanacPropertyEditController.php',
    'AlmanacPropertyInterface' => 'applications/almanac/property/AlmanacPropertyInterface.php',
    'AlmanacPropertyQuery' => 'applications/almanac/query/AlmanacPropertyQuery.php',
    'AlmanacQuery' => 'applications/almanac/query/AlmanacQuery.php',
    'AlmanacQueryDevicesConduitAPIMethod' => 'applications/almanac/conduit/AlmanacQueryDevicesConduitAPIMethod.php',
    'AlmanacQueryServicesConduitAPIMethod' => 'applications/almanac/conduit/AlmanacQueryServicesConduitAPIMethod.php',
    'AlmanacSchemaSpec' => 'applications/almanac/storage/AlmanacSchemaSpec.php',
    'AlmanacService' => 'applications/almanac/storage/AlmanacService.php',
    'AlmanacServiceController' => 'applications/almanac/controller/AlmanacServiceController.php',
    'AlmanacServiceDatasource' => 'applications/almanac/typeahead/AlmanacServiceDatasource.php',
    'AlmanacServiceEditController' => 'applications/almanac/controller/AlmanacServiceEditController.php',
    'AlmanacServiceEditor' => 'applications/almanac/editor/AlmanacServiceEditor.php',
    'AlmanacServiceListController' => 'applications/almanac/controller/AlmanacServiceListController.php',
    'AlmanacServicePHIDType' => 'applications/almanac/phid/AlmanacServicePHIDType.php',
    'AlmanacServiceQuery' => 'applications/almanac/query/AlmanacServiceQuery.php',
    'AlmanacServiceSearchEngine' => 'applications/almanac/query/AlmanacServiceSearchEngine.php',
    'AlmanacServiceTransaction' => 'applications/almanac/storage/AlmanacServiceTransaction.php',
    'AlmanacServiceTransactionQuery' => 'applications/almanac/query/AlmanacServiceTransactionQuery.php',
    'AlmanacServiceType' => 'applications/almanac/servicetype/AlmanacServiceType.php',
    'AlmanacServiceTypeTestCase' => 'applications/almanac/servicetype/__tests__/AlmanacServiceTypeTestCase.php',
    'AlmanacServiceViewController' => 'applications/almanac/controller/AlmanacServiceViewController.php',
    'AphlictDropdownDataQuery' => 'applications/aphlict/query/AphlictDropdownDataQuery.php',
    'Aphront304Response' => 'aphront/response/Aphront304Response.php',
    'Aphront400Response' => 'aphront/response/Aphront400Response.php',
    'Aphront403Response' => 'aphront/response/Aphront403Response.php',
    'Aphront404Response' => 'aphront/response/Aphront404Response.php',
    'AphrontAjaxResponse' => 'aphront/response/AphrontAjaxResponse.php',
    'AphrontApplicationConfiguration' => 'aphront/configuration/AphrontApplicationConfiguration.php',
    'AphrontBarView' => 'view/widget/bars/AphrontBarView.php',
    'AphrontBoolHTTPParameterType' => 'aphront/httpparametertype/AphrontBoolHTTPParameterType.php',
    'AphrontCSRFException' => 'aphront/exception/AphrontCSRFException.php',
    'AphrontCalendarEventView' => 'applications/calendar/view/AphrontCalendarEventView.php',
    'AphrontController' => 'aphront/AphrontController.php',
    'AphrontCursorPagerView' => 'view/control/AphrontCursorPagerView.php',
    'AphrontDefaultApplicationConfiguration' => 'aphront/configuration/AphrontDefaultApplicationConfiguration.php',
    'AphrontDialogResponse' => 'aphront/response/AphrontDialogResponse.php',
    'AphrontDialogView' => 'view/AphrontDialogView.php',
    'AphrontException' => 'aphront/exception/AphrontException.php',
    'AphrontFileResponse' => 'aphront/response/AphrontFileResponse.php',
    'AphrontFormCheckboxControl' => 'view/form/control/AphrontFormCheckboxControl.php',
    'AphrontFormControl' => 'view/form/control/AphrontFormControl.php',
    'AphrontFormDateControl' => 'view/form/control/AphrontFormDateControl.php',
    'AphrontFormDateControlValue' => 'view/form/control/AphrontFormDateControlValue.php',
    'AphrontFormDividerControl' => 'view/form/control/AphrontFormDividerControl.php',
    'AphrontFormFileControl' => 'view/form/control/AphrontFormFileControl.php',
    'AphrontFormHandlesControl' => 'view/form/control/AphrontFormHandlesControl.php',
    'AphrontFormMarkupControl' => 'view/form/control/AphrontFormMarkupControl.php',
    'AphrontFormPasswordControl' => 'view/form/control/AphrontFormPasswordControl.php',
    'AphrontFormPolicyControl' => 'view/form/control/AphrontFormPolicyControl.php',
    'AphrontFormRadioButtonControl' => 'view/form/control/AphrontFormRadioButtonControl.php',
    'AphrontFormRecaptchaControl' => 'view/form/control/AphrontFormRecaptchaControl.php',
    'AphrontFormSelectControl' => 'view/form/control/AphrontFormSelectControl.php',
    'AphrontFormStaticControl' => 'view/form/control/AphrontFormStaticControl.php',
    'AphrontFormSubmitControl' => 'view/form/control/AphrontFormSubmitControl.php',
    'AphrontFormTextAreaControl' => 'view/form/control/AphrontFormTextAreaControl.php',
    'AphrontFormTextControl' => 'view/form/control/AphrontFormTextControl.php',
    'AphrontFormTextWithSubmitControl' => 'view/form/control/AphrontFormTextWithSubmitControl.php',
    'AphrontFormTokenizerControl' => 'view/form/control/AphrontFormTokenizerControl.php',
    'AphrontFormTypeaheadControl' => 'view/form/control/AphrontFormTypeaheadControl.php',
    'AphrontFormView' => 'view/form/AphrontFormView.php',
    'AphrontGlyphBarView' => 'view/widget/bars/AphrontGlyphBarView.php',
    'AphrontHTMLResponse' => 'aphront/response/AphrontHTMLResponse.php',
    'AphrontHTTPParameterType' => 'aphront/httpparametertype/AphrontHTTPParameterType.php',
    'AphrontHTTPProxyResponse' => 'aphront/response/AphrontHTTPProxyResponse.php',
    'AphrontHTTPSink' => 'aphront/sink/AphrontHTTPSink.php',
    'AphrontHTTPSinkTestCase' => 'aphront/sink/__tests__/AphrontHTTPSinkTestCase.php',
    'AphrontIntHTTPParameterType' => 'aphront/httpparametertype/AphrontIntHTTPParameterType.php',
    'AphrontIsolatedDatabaseConnectionTestCase' => 'infrastructure/storage/__tests__/AphrontIsolatedDatabaseConnectionTestCase.php',
    'AphrontIsolatedHTTPSink' => 'aphront/sink/AphrontIsolatedHTTPSink.php',
    'AphrontJSONResponse' => 'aphront/response/AphrontJSONResponse.php',
    'AphrontJavelinView' => 'view/AphrontJavelinView.php',
    'AphrontKeyboardShortcutsAvailableView' => 'view/widget/AphrontKeyboardShortcutsAvailableView.php',
    'AphrontListFilterView' => 'view/layout/AphrontListFilterView.php',
    'AphrontListHTTPParameterType' => 'aphront/httpparametertype/AphrontListHTTPParameterType.php',
    'AphrontMalformedRequestException' => 'aphront/exception/AphrontMalformedRequestException.php',
    'AphrontMoreView' => 'view/layout/AphrontMoreView.php',
    'AphrontMultiColumnView' => 'view/layout/AphrontMultiColumnView.php',
    'AphrontMySQLDatabaseConnectionTestCase' => 'infrastructure/storage/__tests__/AphrontMySQLDatabaseConnectionTestCase.php',
    'AphrontNullView' => 'view/AphrontNullView.php',
    'AphrontPHIDHTTPParameterType' => 'aphront/httpparametertype/AphrontPHIDHTTPParameterType.php',
    'AphrontPHIDListHTTPParameterType' => 'aphront/httpparametertype/AphrontPHIDListHTTPParameterType.php',
    'AphrontPHPHTTPSink' => 'aphront/sink/AphrontPHPHTTPSink.php',
    'AphrontPageView' => 'view/page/AphrontPageView.php',
    'AphrontPlainTextResponse' => 'aphront/response/AphrontPlainTextResponse.php',
    'AphrontProgressBarView' => 'view/widget/bars/AphrontProgressBarView.php',
    'AphrontProjectListHTTPParameterType' => 'aphront/httpparametertype/AphrontProjectListHTTPParameterType.php',
    'AphrontProxyResponse' => 'aphront/response/AphrontProxyResponse.php',
    'AphrontRedirectResponse' => 'aphront/response/AphrontRedirectResponse.php',
    'AphrontRedirectResponseTestCase' => 'aphront/response/__tests__/AphrontRedirectResponseTestCase.php',
    'AphrontReloadResponse' => 'aphront/response/AphrontReloadResponse.php',
    'AphrontRequest' => 'aphront/AphrontRequest.php',
    'AphrontRequestExceptionHandler' => 'aphront/handler/AphrontRequestExceptionHandler.php',
    'AphrontRequestTestCase' => 'aphront/__tests__/AphrontRequestTestCase.php',
    'AphrontResponse' => 'aphront/response/AphrontResponse.php',
    'AphrontResponseProducerInterface' => 'aphront/interface/AphrontResponseProducerInterface.php',
    'AphrontRoutingMap' => 'aphront/site/AphrontRoutingMap.php',
    'AphrontRoutingResult' => 'aphront/site/AphrontRoutingResult.php',
    'AphrontSelectHTTPParameterType' => 'aphront/httpparametertype/AphrontSelectHTTPParameterType.php',
    'AphrontSideNavFilterView' => 'view/layout/AphrontSideNavFilterView.php',
    'AphrontSite' => 'aphront/site/AphrontSite.php',
    'AphrontStackTraceView' => 'view/widget/AphrontStackTraceView.php',
    'AphrontStandaloneHTMLResponse' => 'aphront/response/AphrontStandaloneHTMLResponse.php',
    'AphrontStringHTTPParameterType' => 'aphront/httpparametertype/AphrontStringHTTPParameterType.php',
    'AphrontStringListHTTPParameterType' => 'aphront/httpparametertype/AphrontStringListHTTPParameterType.php',
    'AphrontTableView' => 'view/control/AphrontTableView.php',
    'AphrontTagView' => 'view/AphrontTagView.php',
    'AphrontTokenizerTemplateView' => 'view/control/AphrontTokenizerTemplateView.php',
    'AphrontTypeaheadTemplateView' => 'view/control/AphrontTypeaheadTemplateView.php',
    'AphrontUnhandledExceptionResponse' => 'aphront/response/AphrontUnhandledExceptionResponse.php',
    'AphrontUserListHTTPParameterType' => 'aphront/httpparametertype/AphrontUserListHTTPParameterType.php',
    'AphrontView' => 'view/AphrontView.php',
    'AphrontWebpageResponse' => 'aphront/response/AphrontWebpageResponse.php',
    'ArcanistConduitAPIMethod' => 'applications/arcanist/conduit/ArcanistConduitAPIMethod.php',
    'AuditConduitAPIMethod' => 'applications/audit/conduit/AuditConduitAPIMethod.php',
    'AuditQueryConduitAPIMethod' => 'applications/audit/conduit/AuditQueryConduitAPIMethod.php',
    'AuthManageProvidersCapability' => 'applications/auth/capability/AuthManageProvidersCapability.php',
    'CalendarTimeUtil' => 'applications/calendar/util/CalendarTimeUtil.php',
    'CalendarTimeUtilTestCase' => 'applications/calendar/__tests__/CalendarTimeUtilTestCase.php',
    'CelerityAPI' => 'applications/celerity/CelerityAPI.php',
    'CelerityDefaultPostprocessor' => 'applications/celerity/postprocessor/CelerityDefaultPostprocessor.php',
    'CelerityHighContrastPostprocessor' => 'applications/celerity/postprocessor/CelerityHighContrastPostprocessor.php',
    'CelerityLargeFontPostprocessor' => 'applications/celerity/postprocessor/CelerityLargeFontPostprocessor.php',
    'CelerityManagementMapWorkflow' => 'applications/celerity/management/CelerityManagementMapWorkflow.php',
    'CelerityManagementWorkflow' => 'applications/celerity/management/CelerityManagementWorkflow.php',
    'CelerityPhabricatorResourceController' => 'applications/celerity/controller/CelerityPhabricatorResourceController.php',
    'CelerityPhabricatorResources' => 'applications/celerity/resources/CelerityPhabricatorResources.php',
    'CelerityPhysicalResources' => 'applications/celerity/resources/CelerityPhysicalResources.php',
    'CelerityPhysicalResourcesTestCase' => 'applications/celerity/resources/__tests__/CelerityPhysicalResourcesTestCase.php',
    'CelerityPostprocessor' => 'applications/celerity/postprocessor/CelerityPostprocessor.php',
    'CelerityPostprocessorTestCase' => 'applications/celerity/__tests__/CelerityPostprocessorTestCase.php',
    'CelerityResourceController' => 'applications/celerity/controller/CelerityResourceController.php',
    'CelerityResourceGraph' => 'applications/celerity/CelerityResourceGraph.php',
    'CelerityResourceMap' => 'applications/celerity/CelerityResourceMap.php',
    'CelerityResourceMapGenerator' => 'applications/celerity/CelerityResourceMapGenerator.php',
    'CelerityResourceTransformer' => 'applications/celerity/CelerityResourceTransformer.php',
    'CelerityResourceTransformerTestCase' => 'applications/celerity/__tests__/CelerityResourceTransformerTestCase.php',
    'CelerityResources' => 'applications/celerity/resources/CelerityResources.php',
    'CelerityResourcesOnDisk' => 'applications/celerity/resources/CelerityResourcesOnDisk.php',
    'CeleritySpriteGenerator' => 'applications/celerity/CeleritySpriteGenerator.php',
    'CelerityStaticResourceResponse' => 'applications/celerity/CelerityStaticResourceResponse.php',
    'ChatLogConduitAPIMethod' => 'applications/chatlog/conduit/ChatLogConduitAPIMethod.php',
    'ChatLogQueryConduitAPIMethod' => 'applications/chatlog/conduit/ChatLogQueryConduitAPIMethod.php',
    'ChatLogRecordConduitAPIMethod' => 'applications/chatlog/conduit/ChatLogRecordConduitAPIMethod.php',
    'ConduitAPIMethod' => 'applications/conduit/method/ConduitAPIMethod.php',
    'ConduitAPIMethodTestCase' => 'applications/conduit/method/__tests__/ConduitAPIMethodTestCase.php',
    'ConduitAPIRequest' => 'applications/conduit/protocol/ConduitAPIRequest.php',
    'ConduitAPIResponse' => 'applications/conduit/protocol/ConduitAPIResponse.php',
    'ConduitApplicationNotInstalledException' => 'applications/conduit/protocol/exception/ConduitApplicationNotInstalledException.php',
    'ConduitBoolParameterType' => 'applications/conduit/parametertype/ConduitBoolParameterType.php',
    'ConduitCall' => 'applications/conduit/call/ConduitCall.php',
    'ConduitCallTestCase' => 'applications/conduit/call/__tests__/ConduitCallTestCase.php',
    'ConduitConnectConduitAPIMethod' => 'applications/conduit/method/ConduitConnectConduitAPIMethod.php',
    'ConduitEpochParameterType' => 'applications/conduit/parametertype/ConduitEpochParameterType.php',
    'ConduitException' => 'applications/conduit/protocol/exception/ConduitException.php',
    'ConduitGetCapabilitiesConduitAPIMethod' => 'applications/conduit/method/ConduitGetCapabilitiesConduitAPIMethod.php',
    'ConduitGetCertificateConduitAPIMethod' => 'applications/conduit/method/ConduitGetCertificateConduitAPIMethod.php',
    'ConduitIntListParameterType' => 'applications/conduit/parametertype/ConduitIntListParameterType.php',
    'ConduitIntParameterType' => 'applications/conduit/parametertype/ConduitIntParameterType.php',
    'ConduitListParameterType' => 'applications/conduit/parametertype/ConduitListParameterType.php',
    'ConduitLogGarbageCollector' => 'applications/conduit/garbagecollector/ConduitLogGarbageCollector.php',
    'ConduitMethodDoesNotExistException' => 'applications/conduit/protocol/exception/ConduitMethodDoesNotExistException.php',
    'ConduitMethodNotFoundException' => 'applications/conduit/protocol/exception/ConduitMethodNotFoundException.php',
    'ConduitPHIDListParameterType' => 'applications/conduit/parametertype/ConduitPHIDListParameterType.php',
    'ConduitPHIDParameterType' => 'applications/conduit/parametertype/ConduitPHIDParameterType.php',
    'ConduitParameterType' => 'applications/conduit/parametertype/ConduitParameterType.php',
    'ConduitPingConduitAPIMethod' => 'applications/conduit/method/ConduitPingConduitAPIMethod.php',
    'ConduitProjectListParameterType' => 'applications/conduit/parametertype/ConduitProjectListParameterType.php',
    'ConduitQueryConduitAPIMethod' => 'applications/conduit/method/ConduitQueryConduitAPIMethod.php',
    'ConduitResultSearchEngineExtension' => 'applications/conduit/query/ConduitResultSearchEngineExtension.php',
    'ConduitSSHWorkflow' => 'applications/conduit/ssh/ConduitSSHWorkflow.php',
    'ConduitStringListParameterType' => 'applications/conduit/parametertype/ConduitStringListParameterType.php',
    'ConduitStringParameterType' => 'applications/conduit/parametertype/ConduitStringParameterType.php',
    'ConduitTokenGarbageCollector' => 'applications/conduit/garbagecollector/ConduitTokenGarbageCollector.php',
    'ConduitUserListParameterType' => 'applications/conduit/parametertype/ConduitUserListParameterType.php',
    'ConduitUserParameterType' => 'applications/conduit/parametertype/ConduitUserParameterType.php',
    'ConduitWildParameterType' => 'applications/conduit/parametertype/ConduitWildParameterType.php',
    'ConpherenceColumnViewController' => 'applications/conpherence/controller/ConpherenceColumnViewController.php',
    'ConpherenceConduitAPIMethod' => 'applications/conpherence/conduit/ConpherenceConduitAPIMethod.php',
    'ConpherenceConfigOptions' => 'applications/conpherence/config/ConpherenceConfigOptions.php',
    'ConpherenceConstants' => 'applications/conpherence/constants/ConpherenceConstants.php',
    'ConpherenceController' => 'applications/conpherence/controller/ConpherenceController.php',
    'ConpherenceCreateThreadConduitAPIMethod' => 'applications/conpherence/conduit/ConpherenceCreateThreadConduitAPIMethod.php',
    'ConpherenceDAO' => 'applications/conpherence/storage/ConpherenceDAO.php',
    'ConpherenceDurableColumnView' => 'applications/conpherence/view/ConpherenceDurableColumnView.php',
    'ConpherenceEditor' => 'applications/conpherence/editor/ConpherenceEditor.php',
    'ConpherenceFormDragAndDropUploadControl' => 'applications/conpherence/view/ConpherenceFormDragAndDropUploadControl.php',
    'ConpherenceFulltextQuery' => 'applications/conpherence/query/ConpherenceFulltextQuery.php',
    'ConpherenceImageData' => 'applications/conpherence/constants/ConpherenceImageData.php',
    'ConpherenceIndex' => 'applications/conpherence/storage/ConpherenceIndex.php',
    'ConpherenceLayoutView' => 'applications/conpherence/view/ConpherenceLayoutView.php',
    'ConpherenceListController' => 'applications/conpherence/controller/ConpherenceListController.php',
    'ConpherenceMenuItemView' => 'applications/conpherence/view/ConpherenceMenuItemView.php',
    'ConpherenceNewRoomController' => 'applications/conpherence/controller/ConpherenceNewRoomController.php',
    'ConpherenceNotificationPanelController' => 'applications/conpherence/controller/ConpherenceNotificationPanelController.php',
    'ConpherenceParticipant' => 'applications/conpherence/storage/ConpherenceParticipant.php',
    'ConpherenceParticipantCountQuery' => 'applications/conpherence/query/ConpherenceParticipantCountQuery.php',
    'ConpherenceParticipantQuery' => 'applications/conpherence/query/ConpherenceParticipantQuery.php',
    'ConpherenceParticipationStatus' => 'applications/conpherence/constants/ConpherenceParticipationStatus.php',
    'ConpherencePeopleWidgetView' => 'applications/conpherence/view/ConpherencePeopleWidgetView.php',
    'ConpherencePicCropControl' => 'applications/conpherence/view/ConpherencePicCropControl.php',
    'ConpherenceQueryThreadConduitAPIMethod' => 'applications/conpherence/conduit/ConpherenceQueryThreadConduitAPIMethod.php',
    'ConpherenceQueryTransactionConduitAPIMethod' => 'applications/conpherence/conduit/ConpherenceQueryTransactionConduitAPIMethod.php',
    'ConpherenceReplyHandler' => 'applications/conpherence/mail/ConpherenceReplyHandler.php',
    'ConpherenceRoomListController' => 'applications/conpherence/controller/ConpherenceRoomListController.php',
    'ConpherenceRoomTestCase' => 'applications/conpherence/__tests__/ConpherenceRoomTestCase.php',
    'ConpherenceSchemaSpec' => 'applications/conpherence/storage/ConpherenceSchemaSpec.php',
    'ConpherenceSettings' => 'applications/conpherence/constants/ConpherenceSettings.php',
    'ConpherenceTestCase' => 'applications/conpherence/__tests__/ConpherenceTestCase.php',
    'ConpherenceThread' => 'applications/conpherence/storage/ConpherenceThread.php',
    'ConpherenceThreadIndexEngineExtension' => 'applications/conpherence/engineextension/ConpherenceThreadIndexEngineExtension.php',
    'ConpherenceThreadListView' => 'applications/conpherence/view/ConpherenceThreadListView.php',
    'ConpherenceThreadMailReceiver' => 'applications/conpherence/mail/ConpherenceThreadMailReceiver.php',
    'ConpherenceThreadMembersPolicyRule' => 'applications/conpherence/policyrule/ConpherenceThreadMembersPolicyRule.php',
    'ConpherenceThreadQuery' => 'applications/conpherence/query/ConpherenceThreadQuery.php',
    'ConpherenceThreadRemarkupRule' => 'applications/conpherence/remarkup/ConpherenceThreadRemarkupRule.php',
    'ConpherenceThreadSearchEngine' => 'applications/conpherence/query/ConpherenceThreadSearchEngine.php',
    'ConpherenceTransaction' => 'applications/conpherence/storage/ConpherenceTransaction.php',
    'ConpherenceTransactionComment' => 'applications/conpherence/storage/ConpherenceTransactionComment.php',
    'ConpherenceTransactionQuery' => 'applications/conpherence/query/ConpherenceTransactionQuery.php',
    'ConpherenceTransactionRenderer' => 'applications/conpherence/ConpherenceTransactionRenderer.php',
    'ConpherenceTransactionView' => 'applications/conpherence/view/ConpherenceTransactionView.php',
    'ConpherenceUpdateActions' => 'applications/conpherence/constants/ConpherenceUpdateActions.php',
    'ConpherenceUpdateController' => 'applications/conpherence/controller/ConpherenceUpdateController.php',
    'ConpherenceUpdateThreadConduitAPIMethod' => 'applications/conpherence/conduit/ConpherenceUpdateThreadConduitAPIMethod.php',
    'ConpherenceViewController' => 'applications/conpherence/controller/ConpherenceViewController.php',
    'ConpherenceWidgetConfigConstants' => 'applications/conpherence/constants/ConpherenceWidgetConfigConstants.php',
    'ConpherenceWidgetController' => 'applications/conpherence/controller/ConpherenceWidgetController.php',
    'ConpherenceWidgetView' => 'applications/conpherence/view/ConpherenceWidgetView.php',
    'DarkConsoleController' => 'applications/console/controller/DarkConsoleController.php',
    'DarkConsoleCore' => 'applications/console/core/DarkConsoleCore.php',
    'DarkConsoleDataController' => 'applications/console/controller/DarkConsoleDataController.php',
    'DarkConsoleErrorLogPlugin' => 'applications/console/plugin/DarkConsoleErrorLogPlugin.php',
    'DarkConsoleErrorLogPluginAPI' => 'applications/console/plugin/errorlog/DarkConsoleErrorLogPluginAPI.php',
    'DarkConsoleEventPlugin' => 'applications/console/plugin/DarkConsoleEventPlugin.php',
    'DarkConsoleEventPluginAPI' => 'applications/console/plugin/event/DarkConsoleEventPluginAPI.php',
    'DarkConsolePlugin' => 'applications/console/plugin/DarkConsolePlugin.php',
    'DarkConsoleRequestPlugin' => 'applications/console/plugin/DarkConsoleRequestPlugin.php',
    'DarkConsoleServicesPlugin' => 'applications/console/plugin/DarkConsoleServicesPlugin.php',
    'DarkConsoleStartupPlugin' => 'applications/console/plugin/DarkConsoleStartupPlugin.php',
    'DarkConsoleXHProfPlugin' => 'applications/console/plugin/DarkConsoleXHProfPlugin.php',
    'DarkConsoleXHProfPluginAPI' => 'applications/console/plugin/xhprof/DarkConsoleXHProfPluginAPI.php',
    'DatabaseConfigurationProvider' => 'infrastructure/storage/configuration/DatabaseConfigurationProvider.php',
    'DefaultDatabaseConfigurationProvider' => 'infrastructure/storage/configuration/DefaultDatabaseConfigurationProvider.php',
    'DifferentialAction' => 'applications/differential/constants/DifferentialAction.php',
    'DifferentialActionEmailCommand' => 'applications/differential/command/DifferentialActionEmailCommand.php',
    'DifferentialActionMenuEventListener' => 'applications/differential/event/DifferentialActionMenuEventListener.php',
    'DifferentialAddCommentView' => 'applications/differential/view/DifferentialAddCommentView.php',
    'DifferentialAdjustmentMapTestCase' => 'applications/differential/storage/__tests__/DifferentialAdjustmentMapTestCase.php',
    'DifferentialAffectedPath' => 'applications/differential/storage/DifferentialAffectedPath.php',
    'DifferentialApplyPatchField' => 'applications/differential/customfield/DifferentialApplyPatchField.php',
    'DifferentialAsanaRepresentationField' => 'applications/differential/customfield/DifferentialAsanaRepresentationField.php',
    'DifferentialAuditorsField' => 'applications/differential/customfield/DifferentialAuditorsField.php',
    'DifferentialAuthorField' => 'applications/differential/customfield/DifferentialAuthorField.php',
    'DifferentialBlameRevisionField' => 'applications/differential/customfield/DifferentialBlameRevisionField.php',
    'DifferentialBlockHeraldAction' => 'applications/differential/herald/DifferentialBlockHeraldAction.php',
    'DifferentialBranchField' => 'applications/differential/customfield/DifferentialBranchField.php',
    'DifferentialChangeHeraldFieldGroup' => 'applications/differential/herald/DifferentialChangeHeraldFieldGroup.php',
    'DifferentialChangeType' => 'applications/differential/constants/DifferentialChangeType.php',
    'DifferentialChangesSinceLastUpdateField' => 'applications/differential/customfield/DifferentialChangesSinceLastUpdateField.php',
    'DifferentialChangeset' => 'applications/differential/storage/DifferentialChangeset.php',
    'DifferentialChangesetDetailView' => 'applications/differential/view/DifferentialChangesetDetailView.php',
    'DifferentialChangesetFileTreeSideNavBuilder' => 'applications/differential/view/DifferentialChangesetFileTreeSideNavBuilder.php',
    'DifferentialChangesetHTMLRenderer' => 'applications/differential/render/DifferentialChangesetHTMLRenderer.php',
    'DifferentialChangesetListView' => 'applications/differential/view/DifferentialChangesetListView.php',
    'DifferentialChangesetOneUpRenderer' => 'applications/differential/render/DifferentialChangesetOneUpRenderer.php',
    'DifferentialChangesetOneUpTestRenderer' => 'applications/differential/render/DifferentialChangesetOneUpTestRenderer.php',
    'DifferentialChangesetParser' => 'applications/differential/parser/DifferentialChangesetParser.php',
    'DifferentialChangesetParserTestCase' => 'applications/differential/parser/__tests__/DifferentialChangesetParserTestCase.php',
    'DifferentialChangesetQuery' => 'applications/differential/query/DifferentialChangesetQuery.php',
    'DifferentialChangesetRenderer' => 'applications/differential/render/DifferentialChangesetRenderer.php',
    'DifferentialChangesetTestRenderer' => 'applications/differential/render/DifferentialChangesetTestRenderer.php',
    'DifferentialChangesetTwoUpRenderer' => 'applications/differential/render/DifferentialChangesetTwoUpRenderer.php',
    'DifferentialChangesetTwoUpTestRenderer' => 'applications/differential/render/DifferentialChangesetTwoUpTestRenderer.php',
    'DifferentialChangesetViewController' => 'applications/differential/controller/DifferentialChangesetViewController.php',
    'DifferentialCloseConduitAPIMethod' => 'applications/differential/conduit/DifferentialCloseConduitAPIMethod.php',
    'DifferentialCommentPreviewController' => 'applications/differential/controller/DifferentialCommentPreviewController.php',
    'DifferentialCommentSaveController' => 'applications/differential/controller/DifferentialCommentSaveController.php',
    'DifferentialCommitMessageParser' => 'applications/differential/parser/DifferentialCommitMessageParser.php',
    'DifferentialCommitMessageParserTestCase' => 'applications/differential/parser/__tests__/DifferentialCommitMessageParserTestCase.php',
    'DifferentialCommitsField' => 'applications/differential/customfield/DifferentialCommitsField.php',
    'DifferentialConduitAPIMethod' => 'applications/differential/conduit/DifferentialConduitAPIMethod.php',
    'DifferentialConflictsField' => 'applications/differential/customfield/DifferentialConflictsField.php',
    'DifferentialController' => 'applications/differential/controller/DifferentialController.php',
    'DifferentialCoreCustomField' => 'applications/differential/customfield/DifferentialCoreCustomField.php',
    'DifferentialCreateCommentConduitAPIMethod' => 'applications/differential/conduit/DifferentialCreateCommentConduitAPIMethod.php',
    'DifferentialCreateDiffConduitAPIMethod' => 'applications/differential/conduit/DifferentialCreateDiffConduitAPIMethod.php',
    'DifferentialCreateInlineConduitAPIMethod' => 'applications/differential/conduit/DifferentialCreateInlineConduitAPIMethod.php',
    'DifferentialCreateMailReceiver' => 'applications/differential/mail/DifferentialCreateMailReceiver.php',
    'DifferentialCreateRawDiffConduitAPIMethod' => 'applications/differential/conduit/DifferentialCreateRawDiffConduitAPIMethod.php',
    'DifferentialCreateRevisionConduitAPIMethod' => 'applications/differential/conduit/DifferentialCreateRevisionConduitAPIMethod.php',
    'DifferentialCustomField' => 'applications/differential/customfield/DifferentialCustomField.php',
    'DifferentialCustomFieldDependsOnParser' => 'applications/differential/parser/DifferentialCustomFieldDependsOnParser.php',
    'DifferentialCustomFieldDependsOnParserTestCase' => 'applications/differential/parser/__tests__/DifferentialCustomFieldDependsOnParserTestCase.php',
    'DifferentialCustomFieldNumericIndex' => 'applications/differential/storage/DifferentialCustomFieldNumericIndex.php',
    'DifferentialCustomFieldRevertsParser' => 'applications/differential/parser/DifferentialCustomFieldRevertsParser.php',
    'DifferentialCustomFieldRevertsParserTestCase' => 'applications/differential/parser/__tests__/DifferentialCustomFieldRevertsParserTestCase.php',
    'DifferentialCustomFieldStorage' => 'applications/differential/storage/DifferentialCustomFieldStorage.php',
    'DifferentialCustomFieldStringIndex' => 'applications/differential/storage/DifferentialCustomFieldStringIndex.php',
    'DifferentialDAO' => 'applications/differential/storage/DifferentialDAO.php',
    'DifferentialDefaultViewCapability' => 'applications/differential/capability/DifferentialDefaultViewCapability.php',
    'DifferentialDependenciesField' => 'applications/differential/customfield/DifferentialDependenciesField.php',
    'DifferentialDependsOnField' => 'applications/differential/customfield/DifferentialDependsOnField.php',
    'DifferentialDiff' => 'applications/differential/storage/DifferentialDiff.php',
    'DifferentialDiffAffectedFilesHeraldField' => 'applications/differential/herald/DifferentialDiffAffectedFilesHeraldField.php',
    'DifferentialDiffAuthorHeraldField' => 'applications/differential/herald/DifferentialDiffAuthorHeraldField.php',
    'DifferentialDiffAuthorProjectsHeraldField' => 'applications/differential/herald/DifferentialDiffAuthorProjectsHeraldField.php',
    'DifferentialDiffContentAddedHeraldField' => 'applications/differential/herald/DifferentialDiffContentAddedHeraldField.php',
    'DifferentialDiffContentHeraldField' => 'applications/differential/herald/DifferentialDiffContentHeraldField.php',
    'DifferentialDiffContentRemovedHeraldField' => 'applications/differential/herald/DifferentialDiffContentRemovedHeraldField.php',
    'DifferentialDiffCreateController' => 'applications/differential/controller/DifferentialDiffCreateController.php',
    'DifferentialDiffEditor' => 'applications/differential/editor/DifferentialDiffEditor.php',
    'DifferentialDiffExtractionEngine' => 'applications/differential/engine/DifferentialDiffExtractionEngine.php',
    'DifferentialDiffHeraldField' => 'applications/differential/herald/DifferentialDiffHeraldField.php',
    'DifferentialDiffHeraldFieldGroup' => 'applications/differential/herald/DifferentialDiffHeraldFieldGroup.php',
    'DifferentialDiffInlineCommentQuery' => 'applications/differential/query/DifferentialDiffInlineCommentQuery.php',
    'DifferentialDiffPHIDType' => 'applications/differential/phid/DifferentialDiffPHIDType.php',
    'DifferentialDiffProperty' => 'applications/differential/storage/DifferentialDiffProperty.php',
    'DifferentialDiffQuery' => 'applications/differential/query/DifferentialDiffQuery.php',
    'DifferentialDiffRepositoryHeraldField' => 'applications/differential/herald/DifferentialDiffRepositoryHeraldField.php',
    'DifferentialDiffRepositoryProjectsHeraldField' => 'applications/differential/herald/DifferentialDiffRepositoryProjectsHeraldField.php',
    'DifferentialDiffTestCase' => 'applications/differential/storage/__tests__/DifferentialDiffTestCase.php',
    'DifferentialDiffTransaction' => 'applications/differential/storage/DifferentialDiffTransaction.php',
    'DifferentialDiffTransactionQuery' => 'applications/differential/query/DifferentialDiffTransactionQuery.php',
    'DifferentialDiffViewController' => 'applications/differential/controller/DifferentialDiffViewController.php',
    'DifferentialDoorkeeperRevisionFeedStoryPublisher' => 'applications/differential/doorkeeper/DifferentialDoorkeeperRevisionFeedStoryPublisher.php',
    'DifferentialDraft' => 'applications/differential/storage/DifferentialDraft.php',
    'DifferentialEditPolicyField' => 'applications/differential/customfield/DifferentialEditPolicyField.php',
    'DifferentialFieldParseException' => 'applications/differential/exception/DifferentialFieldParseException.php',
    'DifferentialFieldValidationException' => 'applications/differential/exception/DifferentialFieldValidationException.php',
    'DifferentialFindConduitAPIMethod' => 'applications/differential/conduit/DifferentialFindConduitAPIMethod.php',
    'DifferentialGetAllDiffsConduitAPIMethod' => 'applications/differential/conduit/DifferentialGetAllDiffsConduitAPIMethod.php',
    'DifferentialGetCommitMessageConduitAPIMethod' => 'applications/differential/conduit/DifferentialGetCommitMessageConduitAPIMethod.php',
    'DifferentialGetCommitPathsConduitAPIMethod' => 'applications/differential/conduit/DifferentialGetCommitPathsConduitAPIMethod.php',
    'DifferentialGetDiffConduitAPIMethod' => 'applications/differential/conduit/DifferentialGetDiffConduitAPIMethod.php',
    'DifferentialGetRawDiffConduitAPIMethod' => 'applications/differential/conduit/DifferentialGetRawDiffConduitAPIMethod.php',
    'DifferentialGetRevisionCommentsConduitAPIMethod' => 'applications/differential/conduit/DifferentialGetRevisionCommentsConduitAPIMethod.php',
    'DifferentialGetRevisionConduitAPIMethod' => 'applications/differential/conduit/DifferentialGetRevisionConduitAPIMethod.php',
    'DifferentialGetWorkingCopy' => 'applications/differential/DifferentialGetWorkingCopy.php',
    'DifferentialGitHubLandingStrategy' => 'applications/differential/landing/DifferentialGitHubLandingStrategy.php',
    'DifferentialGitSVNIDField' => 'applications/differential/customfield/DifferentialGitSVNIDField.php',
    'DifferentialHarbormasterField' => 'applications/differential/customfield/DifferentialHarbormasterField.php',
    'DifferentialHiddenComment' => 'applications/differential/storage/DifferentialHiddenComment.php',
    'DifferentialHostField' => 'applications/differential/customfield/DifferentialHostField.php',
    'DifferentialHostedGitLandingStrategy' => 'applications/differential/landing/DifferentialHostedGitLandingStrategy.php',
    'DifferentialHostedMercurialLandingStrategy' => 'applications/differential/landing/DifferentialHostedMercurialLandingStrategy.php',
    'DifferentialHovercardEngineExtension' => 'applications/differential/engineextension/DifferentialHovercardEngineExtension.php',
    'DifferentialHunk' => 'applications/differential/storage/DifferentialHunk.php',
    'DifferentialHunkParser' => 'applications/differential/parser/DifferentialHunkParser.php',
    'DifferentialHunkParserTestCase' => 'applications/differential/parser/__tests__/DifferentialHunkParserTestCase.php',
    'DifferentialHunkQuery' => 'applications/differential/query/DifferentialHunkQuery.php',
    'DifferentialHunkTestCase' => 'applications/differential/storage/__tests__/DifferentialHunkTestCase.php',
    'DifferentialInlineComment' => 'applications/differential/storage/DifferentialInlineComment.php',
    'DifferentialInlineCommentEditController' => 'applications/differential/controller/DifferentialInlineCommentEditController.php',
    'DifferentialInlineCommentPreviewController' => 'applications/differential/controller/DifferentialInlineCommentPreviewController.php',
    'DifferentialInlineCommentQuery' => 'applications/differential/query/DifferentialInlineCommentQuery.php',
    'DifferentialJIRAIssuesField' => 'applications/differential/customfield/DifferentialJIRAIssuesField.php',
    'DifferentialLandingActionMenuEventListener' => 'applications/differential/landing/DifferentialLandingActionMenuEventListener.php',
    'DifferentialLandingStrategy' => 'applications/differential/landing/DifferentialLandingStrategy.php',
    'DifferentialLegacyHunk' => 'applications/differential/storage/DifferentialLegacyHunk.php',
    'DifferentialLineAdjustmentMap' => 'applications/differential/parser/DifferentialLineAdjustmentMap.php',
    'DifferentialLintField' => 'applications/differential/customfield/DifferentialLintField.php',
    'DifferentialLintStatus' => 'applications/differential/constants/DifferentialLintStatus.php',
    'DifferentialLocalCommitsView' => 'applications/differential/view/DifferentialLocalCommitsView.php',
    'DifferentialManiphestTasksField' => 'applications/differential/customfield/DifferentialManiphestTasksField.php',
    'DifferentialModernHunk' => 'applications/differential/storage/DifferentialModernHunk.php',
    'DifferentialNextStepField' => 'applications/differential/customfield/DifferentialNextStepField.php',
    'DifferentialParseCacheGarbageCollector' => 'applications/differential/garbagecollector/DifferentialParseCacheGarbageCollector.php',
    'DifferentialParseCommitMessageConduitAPIMethod' => 'applications/differential/conduit/DifferentialParseCommitMessageConduitAPIMethod.php',
    'DifferentialParseRenderTestCase' => 'applications/differential/__tests__/DifferentialParseRenderTestCase.php',
    'DifferentialPathField' => 'applications/differential/customfield/DifferentialPathField.php',
    'DifferentialPrimaryPaneView' => 'applications/differential/view/DifferentialPrimaryPaneView.php',
    'DifferentialProjectReviewersField' => 'applications/differential/customfield/DifferentialProjectReviewersField.php',
    'DifferentialProjectsField' => 'applications/differential/customfield/DifferentialProjectsField.php',
    'DifferentialQueryConduitAPIMethod' => 'applications/differential/conduit/DifferentialQueryConduitAPIMethod.php',
    'DifferentialQueryDiffsConduitAPIMethod' => 'applications/differential/conduit/DifferentialQueryDiffsConduitAPIMethod.php',
    'DifferentialRawDiffRenderer' => 'applications/differential/render/DifferentialRawDiffRenderer.php',
    'DifferentialReleephRequestFieldSpecification' => 'applications/releeph/differential/DifferentialReleephRequestFieldSpecification.php',
    'DifferentialRemarkupRule' => 'applications/differential/remarkup/DifferentialRemarkupRule.php',
    'DifferentialReplyHandler' => 'applications/differential/mail/DifferentialReplyHandler.php',
    'DifferentialRepositoryField' => 'applications/differential/customfield/DifferentialRepositoryField.php',
    'DifferentialRepositoryLookup' => 'applications/differential/query/DifferentialRepositoryLookup.php',
    'DifferentialRequiredSignaturesField' => 'applications/differential/customfield/DifferentialRequiredSignaturesField.php',
    'DifferentialRevertPlanField' => 'applications/differential/customfield/DifferentialRevertPlanField.php',
    'DifferentialReviewedByField' => 'applications/differential/customfield/DifferentialReviewedByField.php',
    'DifferentialReviewer' => 'applications/differential/storage/DifferentialReviewer.php',
    'DifferentialReviewerForRevisionEdgeType' => 'applications/differential/edge/DifferentialReviewerForRevisionEdgeType.php',
    'DifferentialReviewerStatus' => 'applications/differential/constants/DifferentialReviewerStatus.php',
    'DifferentialReviewersAddBlockingReviewersHeraldAction' => 'applications/differential/herald/DifferentialReviewersAddBlockingReviewersHeraldAction.php',
    'DifferentialReviewersAddBlockingSelfHeraldAction' => 'applications/differential/herald/DifferentialReviewersAddBlockingSelfHeraldAction.php',
    'DifferentialReviewersAddReviewersHeraldAction' => 'applications/differential/herald/DifferentialReviewersAddReviewersHeraldAction.php',
    'DifferentialReviewersAddSelfHeraldAction' => 'applications/differential/herald/DifferentialReviewersAddSelfHeraldAction.php',
    'DifferentialReviewersField' => 'applications/differential/customfield/DifferentialReviewersField.php',
    'DifferentialReviewersHeraldAction' => 'applications/differential/herald/DifferentialReviewersHeraldAction.php',
    'DifferentialReviewersView' => 'applications/differential/view/DifferentialReviewersView.php',
    'DifferentialRevision' => 'applications/differential/storage/DifferentialRevision.php',
    'DifferentialRevisionAffectedFilesHeraldField' => 'applications/differential/herald/DifferentialRevisionAffectedFilesHeraldField.php',
    'DifferentialRevisionAuthorHeraldField' => 'applications/differential/herald/DifferentialRevisionAuthorHeraldField.php',
    'DifferentialRevisionAuthorProjectsHeraldField' => 'applications/differential/herald/DifferentialRevisionAuthorProjectsHeraldField.php',
    'DifferentialRevisionCloseDetailsController' => 'applications/differential/controller/DifferentialRevisionCloseDetailsController.php',
    'DifferentialRevisionContentAddedHeraldField' => 'applications/differential/herald/DifferentialRevisionContentAddedHeraldField.php',
    'DifferentialRevisionContentHeraldField' => 'applications/differential/herald/DifferentialRevisionContentHeraldField.php',
    'DifferentialRevisionContentRemovedHeraldField' => 'applications/differential/herald/DifferentialRevisionContentRemovedHeraldField.php',
    'DifferentialRevisionControlSystem' => 'applications/differential/constants/DifferentialRevisionControlSystem.php',
    'DifferentialRevisionDependedOnByRevisionEdgeType' => 'applications/differential/edge/DifferentialRevisionDependedOnByRevisionEdgeType.php',
    'DifferentialRevisionDependsOnRevisionEdgeType' => 'applications/differential/edge/DifferentialRevisionDependsOnRevisionEdgeType.php',
    'DifferentialRevisionDetailView' => 'applications/differential/view/DifferentialRevisionDetailView.php',
    'DifferentialRevisionEditController' => 'applications/differential/controller/DifferentialRevisionEditController.php',
    'DifferentialRevisionFulltextEngine' => 'applications/differential/search/DifferentialRevisionFulltextEngine.php',
    'DifferentialRevisionHasCommitEdgeType' => 'applications/differential/edge/DifferentialRevisionHasCommitEdgeType.php',
    'DifferentialRevisionHasReviewerEdgeType' => 'applications/differential/edge/DifferentialRevisionHasReviewerEdgeType.php',
    'DifferentialRevisionHasTaskEdgeType' => 'applications/differential/edge/DifferentialRevisionHasTaskEdgeType.php',
    'DifferentialRevisionHeraldField' => 'applications/differential/herald/DifferentialRevisionHeraldField.php',
    'DifferentialRevisionHeraldFieldGroup' => 'applications/differential/herald/DifferentialRevisionHeraldFieldGroup.php',
    'DifferentialRevisionIDField' => 'applications/differential/customfield/DifferentialRevisionIDField.php',
    'DifferentialRevisionLandController' => 'applications/differential/controller/DifferentialRevisionLandController.php',
    'DifferentialRevisionListController' => 'applications/differential/controller/DifferentialRevisionListController.php',
    'DifferentialRevisionListView' => 'applications/differential/view/DifferentialRevisionListView.php',
    'DifferentialRevisionMailReceiver' => 'applications/differential/mail/DifferentialRevisionMailReceiver.php',
    'DifferentialRevisionOperationController' => 'applications/differential/controller/DifferentialRevisionOperationController.php',
    'DifferentialRevisionPHIDType' => 'applications/differential/phid/DifferentialRevisionPHIDType.php',
    'DifferentialRevisionPackageHeraldField' => 'applications/differential/herald/DifferentialRevisionPackageHeraldField.php',
    'DifferentialRevisionPackageOwnerHeraldField' => 'applications/differential/herald/DifferentialRevisionPackageOwnerHeraldField.php',
    'DifferentialRevisionQuery' => 'applications/differential/query/DifferentialRevisionQuery.php',
    'DifferentialRevisionRepositoryHeraldField' => 'applications/differential/herald/DifferentialRevisionRepositoryHeraldField.php',
    'DifferentialRevisionRepositoryProjectsHeraldField' => 'applications/differential/herald/DifferentialRevisionRepositoryProjectsHeraldField.php',
    'DifferentialRevisionReviewersHeraldField' => 'applications/differential/herald/DifferentialRevisionReviewersHeraldField.php',
    'DifferentialRevisionSearchEngine' => 'applications/differential/query/DifferentialRevisionSearchEngine.php',
    'DifferentialRevisionStatus' => 'applications/differential/constants/DifferentialRevisionStatus.php',
    'DifferentialRevisionSummaryHeraldField' => 'applications/differential/herald/DifferentialRevisionSummaryHeraldField.php',
    'DifferentialRevisionTitleHeraldField' => 'applications/differential/herald/DifferentialRevisionTitleHeraldField.php',
    'DifferentialRevisionUpdateHistoryView' => 'applications/differential/view/DifferentialRevisionUpdateHistoryView.php',
    'DifferentialRevisionViewController' => 'applications/differential/controller/DifferentialRevisionViewController.php',
    'DifferentialSchemaSpec' => 'applications/differential/storage/DifferentialSchemaSpec.php',
    'DifferentialSetDiffPropertyConduitAPIMethod' => 'applications/differential/conduit/DifferentialSetDiffPropertyConduitAPIMethod.php',
    'DifferentialStoredCustomField' => 'applications/differential/customfield/DifferentialStoredCustomField.php',
    'DifferentialSubscribersField' => 'applications/differential/customfield/DifferentialSubscribersField.php',
    'DifferentialSummaryField' => 'applications/differential/customfield/DifferentialSummaryField.php',
    'DifferentialTestPlanField' => 'applications/differential/customfield/DifferentialTestPlanField.php',
    'DifferentialTitleField' => 'applications/differential/customfield/DifferentialTitleField.php',
    'DifferentialTransaction' => 'applications/differential/storage/DifferentialTransaction.php',
    'DifferentialTransactionComment' => 'applications/differential/storage/DifferentialTransactionComment.php',
    'DifferentialTransactionEditor' => 'applications/differential/editor/DifferentialTransactionEditor.php',
    'DifferentialTransactionQuery' => 'applications/differential/query/DifferentialTransactionQuery.php',
    'DifferentialTransactionView' => 'applications/differential/view/DifferentialTransactionView.php',
    'DifferentialUnitField' => 'applications/differential/customfield/DifferentialUnitField.php',
    'DifferentialUnitStatus' => 'applications/differential/constants/DifferentialUnitStatus.php',
    'DifferentialUnitTestResult' => 'applications/differential/constants/DifferentialUnitTestResult.php',
    'DifferentialUpdateRevisionConduitAPIMethod' => 'applications/differential/conduit/DifferentialUpdateRevisionConduitAPIMethod.php',
    'DifferentialViewPolicyField' => 'applications/differential/customfield/DifferentialViewPolicyField.php',
    'DiffusionAuditorDatasource' => 'applications/diffusion/typeahead/DiffusionAuditorDatasource.php',
    'DiffusionAuditorFunctionDatasource' => 'applications/diffusion/typeahead/DiffusionAuditorFunctionDatasource.php',
    'DiffusionAuditorsAddAuditorsHeraldAction' => 'applications/diffusion/herald/DiffusionAuditorsAddAuditorsHeraldAction.php',
    'DiffusionAuditorsAddSelfHeraldAction' => 'applications/diffusion/herald/DiffusionAuditorsAddSelfHeraldAction.php',
    'DiffusionAuditorsHeraldAction' => 'applications/diffusion/herald/DiffusionAuditorsHeraldAction.php',
    'DiffusionBlameConduitAPIMethod' => 'applications/diffusion/conduit/DiffusionBlameConduitAPIMethod.php',
    'DiffusionBlameQuery' => 'applications/diffusion/query/blame/DiffusionBlameQuery.php',
    'DiffusionBlockHeraldAction' => 'applications/diffusion/herald/DiffusionBlockHeraldAction.php',
    'DiffusionBranchQueryConduitAPIMethod' => 'applications/diffusion/conduit/DiffusionBranchQueryConduitAPIMethod.php',
    'DiffusionBranchTableController' => 'applications/diffusion/controller/DiffusionBranchTableController.php',
    'DiffusionBranchTableView' => 'applications/diffusion/view/DiffusionBranchTableView.php',
    'DiffusionBrowseController' => 'applications/diffusion/controller/DiffusionBrowseController.php',
    'DiffusionBrowseQueryConduitAPIMethod' => 'applications/diffusion/conduit/DiffusionBrowseQueryConduitAPIMethod.php',
    'DiffusionBrowseResultSet' => 'applications/diffusion/data/DiffusionBrowseResultSet.php',
    'DiffusionBrowseTableView' => 'applications/diffusion/view/DiffusionBrowseTableView.php',
    'DiffusionCachedResolveRefsQuery' => 'applications/diffusion/query/DiffusionCachedResolveRefsQuery.php',
    'DiffusionChangeController' => 'applications/diffusion/controller/DiffusionChangeController.php',
    'DiffusionChangeHeraldFieldGroup' => 'applications/diffusion/herald/DiffusionChangeHeraldFieldGroup.php',
    'DiffusionCommitAffectedFilesHeraldField' => 'applications/diffusion/herald/DiffusionCommitAffectedFilesHeraldField.php',
    'DiffusionCommitAuthorHeraldField' => 'applications/diffusion/herald/DiffusionCommitAuthorHeraldField.php',
    'DiffusionCommitAutocloseHeraldField' => 'applications/diffusion/herald/DiffusionCommitAutocloseHeraldField.php',
    'DiffusionCommitBranchesController' => 'applications/diffusion/controller/DiffusionCommitBranchesController.php',
    'DiffusionCommitBranchesHeraldField' => 'applications/diffusion/herald/DiffusionCommitBranchesHeraldField.php',
    'DiffusionCommitCommitterHeraldField' => 'applications/diffusion/herald/DiffusionCommitCommitterHeraldField.php',
    'DiffusionCommitController' => 'applications/diffusion/controller/DiffusionCommitController.php',
    'DiffusionCommitDiffContentAddedHeraldField' => 'applications/diffusion/herald/DiffusionCommitDiffContentAddedHeraldField.php',
    'DiffusionCommitDiffContentHeraldField' => 'applications/diffusion/herald/DiffusionCommitDiffContentHeraldField.php',
    'DiffusionCommitDiffContentRemovedHeraldField' => 'applications/diffusion/herald/DiffusionCommitDiffContentRemovedHeraldField.php',
    'DiffusionCommitDiffEnormousHeraldField' => 'applications/diffusion/herald/DiffusionCommitDiffEnormousHeraldField.php',
    'DiffusionCommitEditController' => 'applications/diffusion/controller/DiffusionCommitEditController.php',
    'DiffusionCommitFulltextEngine' => 'applications/repository/search/DiffusionCommitFulltextEngine.php',
    'DiffusionCommitHasRevisionEdgeType' => 'applications/diffusion/edge/DiffusionCommitHasRevisionEdgeType.php',
    'DiffusionCommitHasTaskEdgeType' => 'applications/diffusion/edge/DiffusionCommitHasTaskEdgeType.php',
    'DiffusionCommitHash' => 'applications/diffusion/data/DiffusionCommitHash.php',
    'DiffusionCommitHeraldField' => 'applications/diffusion/herald/DiffusionCommitHeraldField.php',
    'DiffusionCommitHeraldFieldGroup' => 'applications/diffusion/herald/DiffusionCommitHeraldFieldGroup.php',
    'DiffusionCommitHookEngine' => 'applications/diffusion/engine/DiffusionCommitHookEngine.php',
    'DiffusionCommitHookRejectException' => 'applications/diffusion/exception/DiffusionCommitHookRejectException.php',
    'DiffusionCommitMergeHeraldField' => 'applications/diffusion/herald/DiffusionCommitMergeHeraldField.php',
    'DiffusionCommitMessageHeraldField' => 'applications/diffusion/herald/DiffusionCommitMessageHeraldField.php',
    'DiffusionCommitPackageAuditHeraldField' => 'applications/diffusion/herald/DiffusionCommitPackageAuditHeraldField.php',
    'DiffusionCommitPackageHeraldField' => 'applications/diffusion/herald/DiffusionCommitPackageHeraldField.php',
    'DiffusionCommitPackageOwnerHeraldField' => 'applications/diffusion/herald/DiffusionCommitPackageOwnerHeraldField.php',
    'DiffusionCommitParentsQueryConduitAPIMethod' => 'applications/diffusion/conduit/DiffusionCommitParentsQueryConduitAPIMethod.php',
    'DiffusionCommitQuery' => 'applications/diffusion/query/DiffusionCommitQuery.php',
    'DiffusionCommitRef' => 'applications/diffusion/data/DiffusionCommitRef.php',
    'DiffusionCommitRemarkupRule' => 'applications/diffusion/remarkup/DiffusionCommitRemarkupRule.php',
    'DiffusionCommitRemarkupRuleTestCase' => 'applications/diffusion/remarkup/__tests__/DiffusionCommitRemarkupRuleTestCase.php',
    'DiffusionCommitRepositoryHeraldField' => 'applications/diffusion/herald/DiffusionCommitRepositoryHeraldField.php',
    'DiffusionCommitRepositoryProjectsHeraldField' => 'applications/diffusion/herald/DiffusionCommitRepositoryProjectsHeraldField.php',
    'DiffusionCommitRevertedByCommitEdgeType' => 'applications/diffusion/edge/DiffusionCommitRevertedByCommitEdgeType.php',
    'DiffusionCommitRevertsCommitEdgeType' => 'applications/diffusion/edge/DiffusionCommitRevertsCommitEdgeType.php',
    'DiffusionCommitReviewerHeraldField' => 'applications/diffusion/herald/DiffusionCommitReviewerHeraldField.php',
    'DiffusionCommitRevisionAcceptedHeraldField' => 'applications/diffusion/herald/DiffusionCommitRevisionAcceptedHeraldField.php',
    'DiffusionCommitRevisionHeraldField' => 'applications/diffusion/herald/DiffusionCommitRevisionHeraldField.php',
    'DiffusionCommitRevisionReviewersHeraldField' => 'applications/diffusion/herald/DiffusionCommitRevisionReviewersHeraldField.php',
    'DiffusionCommitRevisionSubscribersHeraldField' => 'applications/diffusion/herald/DiffusionCommitRevisionSubscribersHeraldField.php',
    'DiffusionCommitTagsController' => 'applications/diffusion/controller/DiffusionCommitTagsController.php',
    'DiffusionConduitAPIMethod' => 'applications/diffusion/conduit/DiffusionConduitAPIMethod.php',
    'DiffusionController' => 'applications/diffusion/controller/DiffusionController.php',
    'DiffusionCreateCommentConduitAPIMethod' => 'applications/diffusion/conduit/DiffusionCreateCommentConduitAPIMethod.php',
    'DiffusionCreateRepositoriesCapability' => 'applications/diffusion/capability/DiffusionCreateRepositoriesCapability.php',
    'DiffusionDefaultEditCapability' => 'applications/diffusion/capability/DiffusionDefaultEditCapability.php',
    'DiffusionDefaultPushCapability' => 'applications/diffusion/capability/DiffusionDefaultPushCapability.php',
    'DiffusionDefaultViewCapability' => 'applications/diffusion/capability/DiffusionDefaultViewCapability.php',
    'DiffusionDiffController' => 'applications/diffusion/controller/DiffusionDiffController.php',
    'DiffusionDiffInlineCommentQuery' => 'applications/diffusion/query/DiffusionDiffInlineCommentQuery.php',
    'DiffusionDiffQueryConduitAPIMethod' => 'applications/diffusion/conduit/DiffusionDiffQueryConduitAPIMethod.php',
    'DiffusionDoorkeeperCommitFeedStoryPublisher' => 'applications/diffusion/doorkeeper/DiffusionDoorkeeperCommitFeedStoryPublisher.php',
    'DiffusionEmptyResultView' => 'applications/diffusion/view/DiffusionEmptyResultView.php',
    'DiffusionExistsQueryConduitAPIMethod' => 'applications/diffusion/conduit/DiffusionExistsQueryConduitAPIMethod.php',
    'DiffusionExternalController' => 'applications/diffusion/controller/DiffusionExternalController.php',
    'DiffusionExternalSymbolQuery' => 'applications/diffusion/symbol/DiffusionExternalSymbolQuery.php',
    'DiffusionExternalSymbolsSource' => 'applications/diffusion/symbol/DiffusionExternalSymbolsSource.php',
    'DiffusionFileContentQuery' => 'applications/diffusion/query/filecontent/DiffusionFileContentQuery.php',
    'DiffusionFileContentQueryConduitAPIMethod' => 'applications/diffusion/conduit/DiffusionFileContentQueryConduitAPIMethod.php',
    'DiffusionFindSymbolsConduitAPIMethod' => 'applications/diffusion/conduit/DiffusionFindSymbolsConduitAPIMethod.php',
    'DiffusionGetLintMessagesConduitAPIMethod' => 'applications/diffusion/conduit/DiffusionGetLintMessagesConduitAPIMethod.php',
    'DiffusionGetRecentCommitsByPathConduitAPIMethod' => 'applications/diffusion/conduit/DiffusionGetRecentCommitsByPathConduitAPIMethod.php',
    'DiffusionGitBlameQuery' => 'applications/diffusion/query/blame/DiffusionGitBlameQuery.php',
    'DiffusionGitBranch' => 'applications/diffusion/data/DiffusionGitBranch.php',
    'DiffusionGitBranchTestCase' => 'applications/diffusion/data/__tests__/DiffusionGitBranchTestCase.php',
    'DiffusionGitFileContentQuery' => 'applications/diffusion/query/filecontent/DiffusionGitFileContentQuery.php',
    'DiffusionGitRawDiffQuery' => 'applications/diffusion/query/rawdiff/DiffusionGitRawDiffQuery.php',
    'DiffusionGitReceivePackSSHWorkflow' => 'applications/diffusion/ssh/DiffusionGitReceivePackSSHWorkflow.php',
    'DiffusionGitRequest' => 'applications/diffusion/request/DiffusionGitRequest.php',
    'DiffusionGitResponse' => 'applications/diffusion/response/DiffusionGitResponse.php',
    'DiffusionGitSSHWorkflow' => 'applications/diffusion/ssh/DiffusionGitSSHWorkflow.php',
    'DiffusionGitUploadPackSSHWorkflow' => 'applications/diffusion/ssh/DiffusionGitUploadPackSSHWorkflow.php',
    'DiffusionHistoryController' => 'applications/diffusion/controller/DiffusionHistoryController.php',
    'DiffusionHistoryQueryConduitAPIMethod' => 'applications/diffusion/conduit/DiffusionHistoryQueryConduitAPIMethod.php',
    'DiffusionHistoryTableView' => 'applications/diffusion/view/DiffusionHistoryTableView.php',
    'DiffusionHovercardEngineExtension' => 'applications/diffusion/engineextension/DiffusionHovercardEngineExtension.php',
    'DiffusionInlineCommentController' => 'applications/diffusion/controller/DiffusionInlineCommentController.php',
    'DiffusionInlineCommentPreviewController' => 'applications/diffusion/controller/DiffusionInlineCommentPreviewController.php',
    'DiffusionLastModifiedController' => 'applications/diffusion/controller/DiffusionLastModifiedController.php',
    'DiffusionLastModifiedQueryConduitAPIMethod' => 'applications/diffusion/conduit/DiffusionLastModifiedQueryConduitAPIMethod.php',
    'DiffusionLintController' => 'applications/diffusion/controller/DiffusionLintController.php',
    'DiffusionLintCountQuery' => 'applications/diffusion/query/DiffusionLintCountQuery.php',
    'DiffusionLintSaveRunner' => 'applications/diffusion/DiffusionLintSaveRunner.php',
    'DiffusionLookSoonConduitAPIMethod' => 'applications/diffusion/conduit/DiffusionLookSoonConduitAPIMethod.php',
    'DiffusionLowLevelCommitFieldsQuery' => 'applications/diffusion/query/lowlevel/DiffusionLowLevelCommitFieldsQuery.php',
    'DiffusionLowLevelCommitQuery' => 'applications/diffusion/query/lowlevel/DiffusionLowLevelCommitQuery.php',
    'DiffusionLowLevelGitRefQuery' => 'applications/diffusion/query/lowlevel/DiffusionLowLevelGitRefQuery.php',
    'DiffusionLowLevelMercurialBranchesQuery' => 'applications/diffusion/query/lowlevel/DiffusionLowLevelMercurialBranchesQuery.php',
    'DiffusionLowLevelMercurialPathsQuery' => 'applications/diffusion/query/lowlevel/DiffusionLowLevelMercurialPathsQuery.php',
    'DiffusionLowLevelMercurialPathsQueryTests' => 'applications/diffusion/query/lowlevel/__tests__/DiffusionLowLevelMercurialPathsQueryTests.php',
    'DiffusionLowLevelParentsQuery' => 'applications/diffusion/query/lowlevel/DiffusionLowLevelParentsQuery.php',
    'DiffusionLowLevelQuery' => 'applications/diffusion/query/lowlevel/DiffusionLowLevelQuery.php',
    'DiffusionLowLevelResolveRefsQuery' => 'applications/diffusion/query/lowlevel/DiffusionLowLevelResolveRefsQuery.php',
    'DiffusionMercurialBlameQuery' => 'applications/diffusion/query/blame/DiffusionMercurialBlameQuery.php',
    'DiffusionMercurialFileContentQuery' => 'applications/diffusion/query/filecontent/DiffusionMercurialFileContentQuery.php',
    'DiffusionMercurialRawDiffQuery' => 'applications/diffusion/query/rawdiff/DiffusionMercurialRawDiffQuery.php',
    'DiffusionMercurialRequest' => 'applications/diffusion/request/DiffusionMercurialRequest.php',
    'DiffusionMercurialResponse' => 'applications/diffusion/response/DiffusionMercurialResponse.php',
    'DiffusionMercurialSSHWorkflow' => 'applications/diffusion/ssh/DiffusionMercurialSSHWorkflow.php',
    'DiffusionMercurialServeSSHWorkflow' => 'applications/diffusion/ssh/DiffusionMercurialServeSSHWorkflow.php',
    'DiffusionMercurialWireClientSSHProtocolChannel' => 'applications/diffusion/ssh/DiffusionMercurialWireClientSSHProtocolChannel.php',
    'DiffusionMercurialWireProtocol' => 'applications/diffusion/protocol/DiffusionMercurialWireProtocol.php',
    'DiffusionMercurialWireProtocolTests' => 'applications/diffusion/protocol/__tests__/DiffusionMercurialWireProtocolTests.php',
    'DiffusionMercurialWireSSHTestCase' => 'applications/diffusion/ssh/__tests__/DiffusionMercurialWireSSHTestCase.php',
    'DiffusionMergedCommitsQueryConduitAPIMethod' => 'applications/diffusion/conduit/DiffusionMergedCommitsQueryConduitAPIMethod.php',
    'DiffusionMirrorDeleteController' => 'applications/diffusion/controller/DiffusionMirrorDeleteController.php',
    'DiffusionMirrorEditController' => 'applications/diffusion/controller/DiffusionMirrorEditController.php',
    'DiffusionPathChange' => 'applications/diffusion/data/DiffusionPathChange.php',
    'DiffusionPathChangeQuery' => 'applications/diffusion/query/pathchange/DiffusionPathChangeQuery.php',
    'DiffusionPathCompleteController' => 'applications/diffusion/controller/DiffusionPathCompleteController.php',
    'DiffusionPathIDQuery' => 'applications/diffusion/query/pathid/DiffusionPathIDQuery.php',
    'DiffusionPathQuery' => 'applications/diffusion/query/DiffusionPathQuery.php',
    'DiffusionPathQueryTestCase' => 'applications/diffusion/query/pathid/__tests__/DiffusionPathQueryTestCase.php',
    'DiffusionPathTreeController' => 'applications/diffusion/controller/DiffusionPathTreeController.php',
    'DiffusionPathValidateController' => 'applications/diffusion/controller/DiffusionPathValidateController.php',
    'DiffusionPhpExternalSymbolsSource' => 'applications/diffusion/symbol/DiffusionPhpExternalSymbolsSource.php',
    'DiffusionPreCommitContentAffectedFilesHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitContentAffectedFilesHeraldField.php',
    'DiffusionPreCommitContentAuthorHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitContentAuthorHeraldField.php',
    'DiffusionPreCommitContentAuthorRawHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitContentAuthorRawHeraldField.php',
    'DiffusionPreCommitContentBranchesHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitContentBranchesHeraldField.php',
    'DiffusionPreCommitContentCommitterHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitContentCommitterHeraldField.php',
    'DiffusionPreCommitContentCommitterRawHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitContentCommitterRawHeraldField.php',
    'DiffusionPreCommitContentDiffContentAddedHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitContentDiffContentAddedHeraldField.php',
    'DiffusionPreCommitContentDiffContentHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitContentDiffContentHeraldField.php',
    'DiffusionPreCommitContentDiffContentRemovedHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitContentDiffContentRemovedHeraldField.php',
    'DiffusionPreCommitContentDiffEnormousHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitContentDiffEnormousHeraldField.php',
    'DiffusionPreCommitContentHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitContentHeraldField.php',
    'DiffusionPreCommitContentMergeHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitContentMergeHeraldField.php',
    'DiffusionPreCommitContentMessageHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitContentMessageHeraldField.php',
    'DiffusionPreCommitContentPusherHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitContentPusherHeraldField.php',
    'DiffusionPreCommitContentPusherIsCommitterHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitContentPusherIsCommitterHeraldField.php',
    'DiffusionPreCommitContentPusherProjectsHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitContentPusherProjectsHeraldField.php',
    'DiffusionPreCommitContentRepositoryHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitContentRepositoryHeraldField.php',
    'DiffusionPreCommitContentRepositoryProjectsHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitContentRepositoryProjectsHeraldField.php',
    'DiffusionPreCommitContentRevisionAcceptedHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitContentRevisionAcceptedHeraldField.php',
    'DiffusionPreCommitContentRevisionHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitContentRevisionHeraldField.php',
    'DiffusionPreCommitContentRevisionReviewersHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitContentRevisionReviewersHeraldField.php',
    'DiffusionPreCommitContentRevisionSubscribersHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitContentRevisionSubscribersHeraldField.php',
    'DiffusionPreCommitRefChangeHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitRefChangeHeraldField.php',
    'DiffusionPreCommitRefHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitRefHeraldField.php',
    'DiffusionPreCommitRefHeraldFieldGroup' => 'applications/diffusion/herald/DiffusionPreCommitRefHeraldFieldGroup.php',
    'DiffusionPreCommitRefNameHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitRefNameHeraldField.php',
    'DiffusionPreCommitRefPusherHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitRefPusherHeraldField.php',
    'DiffusionPreCommitRefPusherProjectsHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitRefPusherProjectsHeraldField.php',
    'DiffusionPreCommitRefRepositoryHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitRefRepositoryHeraldField.php',
    'DiffusionPreCommitRefRepositoryProjectsHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitRefRepositoryProjectsHeraldField.php',
    'DiffusionPreCommitRefTypeHeraldField' => 'applications/diffusion/herald/DiffusionPreCommitRefTypeHeraldField.php',
    'DiffusionPushCapability' => 'applications/diffusion/capability/DiffusionPushCapability.php',
    'DiffusionPushEventViewController' => 'applications/diffusion/controller/DiffusionPushEventViewController.php',
    'DiffusionPushLogController' => 'applications/diffusion/controller/DiffusionPushLogController.php',
    'DiffusionPushLogListController' => 'applications/diffusion/controller/DiffusionPushLogListController.php',
    'DiffusionPushLogListView' => 'applications/diffusion/view/DiffusionPushLogListView.php',
    'DiffusionPythonExternalSymbolsSource' => 'applications/diffusion/symbol/DiffusionPythonExternalSymbolsSource.php',
    'DiffusionQuery' => 'applications/diffusion/query/DiffusionQuery.php',
    'DiffusionQueryCommitsConduitAPIMethod' => 'applications/diffusion/conduit/DiffusionQueryCommitsConduitAPIMethod.php',
    'DiffusionQueryConduitAPIMethod' => 'applications/diffusion/conduit/DiffusionQueryConduitAPIMethod.php',
    'DiffusionQueryPathsConduitAPIMethod' => 'applications/diffusion/conduit/DiffusionQueryPathsConduitAPIMethod.php',
    'DiffusionRawDiffQuery' => 'applications/diffusion/query/rawdiff/DiffusionRawDiffQuery.php',
    'DiffusionRawDiffQueryConduitAPIMethod' => 'applications/diffusion/conduit/DiffusionRawDiffQueryConduitAPIMethod.php',
    'DiffusionReadmeView' => 'applications/diffusion/view/DiffusionReadmeView.php',
    'DiffusionRefDatasource' => 'applications/diffusion/typeahead/DiffusionRefDatasource.php',
    'DiffusionRefNotFoundException' => 'applications/diffusion/exception/DiffusionRefNotFoundException.php',
    'DiffusionRefTableController' => 'applications/diffusion/controller/DiffusionRefTableController.php',
    'DiffusionRefsQueryConduitAPIMethod' => 'applications/diffusion/conduit/DiffusionRefsQueryConduitAPIMethod.php',
    'DiffusionRenameHistoryQuery' => 'applications/diffusion/query/DiffusionRenameHistoryQuery.php',
    'DiffusionRepositoryByIDRemarkupRule' => 'applications/diffusion/remarkup/DiffusionRepositoryByIDRemarkupRule.php',
    'DiffusionRepositoryController' => 'applications/diffusion/controller/DiffusionRepositoryController.php',
    'DiffusionRepositoryCreateController' => 'applications/diffusion/controller/DiffusionRepositoryCreateController.php',
    'DiffusionRepositoryDatasource' => 'applications/diffusion/typeahead/DiffusionRepositoryDatasource.php',
    'DiffusionRepositoryDefaultController' => 'applications/diffusion/controller/DiffusionRepositoryDefaultController.php',
    'DiffusionRepositoryEditActionsController' => 'applications/diffusion/controller/DiffusionRepositoryEditActionsController.php',
    'DiffusionRepositoryEditActivateController' => 'applications/diffusion/controller/DiffusionRepositoryEditActivateController.php',
    'DiffusionRepositoryEditAutomationController' => 'applications/diffusion/controller/DiffusionRepositoryEditAutomationController.php',
    'DiffusionRepositoryEditBasicController' => 'applications/diffusion/controller/DiffusionRepositoryEditBasicController.php',
    'DiffusionRepositoryEditBranchesController' => 'applications/diffusion/controller/DiffusionRepositoryEditBranchesController.php',
    'DiffusionRepositoryEditController' => 'applications/diffusion/controller/DiffusionRepositoryEditController.php',
    'DiffusionRepositoryEditDangerousController' => 'applications/diffusion/controller/DiffusionRepositoryEditDangerousController.php',
    'DiffusionRepositoryEditDeleteController' => 'applications/diffusion/controller/DiffusionRepositoryEditDeleteController.php',
    'DiffusionRepositoryEditEncodingController' => 'applications/diffusion/controller/DiffusionRepositoryEditEncodingController.php',
    'DiffusionRepositoryEditHostingController' => 'applications/diffusion/controller/DiffusionRepositoryEditHostingController.php',
    'DiffusionRepositoryEditMainController' => 'applications/diffusion/controller/DiffusionRepositoryEditMainController.php',
    'DiffusionRepositoryEditStagingController' => 'applications/diffusion/controller/DiffusionRepositoryEditStagingController.php',
    'DiffusionRepositoryEditStorageController' => 'applications/diffusion/controller/DiffusionRepositoryEditStorageController.php',
    'DiffusionRepositoryEditSubversionController' => 'applications/diffusion/controller/DiffusionRepositoryEditSubversionController.php',
    'DiffusionRepositoryEditUpdateController' => 'applications/diffusion/controller/DiffusionRepositoryEditUpdateController.php',
    'DiffusionRepositoryListController' => 'applications/diffusion/controller/DiffusionRepositoryListController.php',
    'DiffusionRepositoryNewController' => 'applications/diffusion/controller/DiffusionRepositoryNewController.php',
    'DiffusionRepositoryPath' => 'applications/diffusion/data/DiffusionRepositoryPath.php',
    'DiffusionRepositoryRef' => 'applications/diffusion/data/DiffusionRepositoryRef.php',
    'DiffusionRepositoryRemarkupRule' => 'applications/diffusion/remarkup/DiffusionRepositoryRemarkupRule.php',
    'DiffusionRepositorySymbolsController' => 'applications/diffusion/controller/DiffusionRepositorySymbolsController.php',
    'DiffusionRepositoryTag' => 'applications/diffusion/data/DiffusionRepositoryTag.php',
    'DiffusionRepositoryTestAutomationController' => 'applications/diffusion/controller/DiffusionRepositoryTestAutomationController.php',
    'DiffusionRepositoryURIsIndexEngineExtension' => 'applications/diffusion/engineextension/DiffusionRepositoryURIsIndexEngineExtension.php',
    'DiffusionRequest' => 'applications/diffusion/request/DiffusionRequest.php',
    'DiffusionResolveRefsConduitAPIMethod' => 'applications/diffusion/conduit/DiffusionResolveRefsConduitAPIMethod.php',
    'DiffusionResolveUserQuery' => 'applications/diffusion/query/DiffusionResolveUserQuery.php',
    'DiffusionSSHWorkflow' => 'applications/diffusion/ssh/DiffusionSSHWorkflow.php',
    'DiffusionSearchQueryConduitAPIMethod' => 'applications/diffusion/conduit/DiffusionSearchQueryConduitAPIMethod.php',
    'DiffusionServeController' => 'applications/diffusion/controller/DiffusionServeController.php',
    'DiffusionSetPasswordSettingsPanel' => 'applications/diffusion/panel/DiffusionSetPasswordSettingsPanel.php',
    'DiffusionSetupException' => 'applications/diffusion/exception/DiffusionSetupException.php',
    'DiffusionSubversionSSHWorkflow' => 'applications/diffusion/ssh/DiffusionSubversionSSHWorkflow.php',
    'DiffusionSubversionServeSSHWorkflow' => 'applications/diffusion/ssh/DiffusionSubversionServeSSHWorkflow.php',
    'DiffusionSubversionWireProtocol' => 'applications/diffusion/protocol/DiffusionSubversionWireProtocol.php',
    'DiffusionSubversionWireProtocolTestCase' => 'applications/diffusion/protocol/__tests__/DiffusionSubversionWireProtocolTestCase.php',
    'DiffusionSvnBlameQuery' => 'applications/diffusion/query/blame/DiffusionSvnBlameQuery.php',
    'DiffusionSvnFileContentQuery' => 'applications/diffusion/query/filecontent/DiffusionSvnFileContentQuery.php',
    'DiffusionSvnRawDiffQuery' => 'applications/diffusion/query/rawdiff/DiffusionSvnRawDiffQuery.php',
    'DiffusionSvnRequest' => 'applications/diffusion/request/DiffusionSvnRequest.php',
    'DiffusionSymbolController' => 'applications/diffusion/controller/DiffusionSymbolController.php',
    'DiffusionSymbolDatasource' => 'applications/diffusion/typeahead/DiffusionSymbolDatasource.php',
    'DiffusionSymbolQuery' => 'applications/diffusion/query/DiffusionSymbolQuery.php',
    'DiffusionTagListController' => 'applications/diffusion/controller/DiffusionTagListController.php',
    'DiffusionTagListView' => 'applications/diffusion/view/DiffusionTagListView.php',
    'DiffusionTagsQueryConduitAPIMethod' => 'applications/diffusion/conduit/DiffusionTagsQueryConduitAPIMethod.php',
    'DiffusionURITestCase' => 'applications/diffusion/request/__tests__/DiffusionURITestCase.php',
    'DiffusionUpdateCoverageConduitAPIMethod' => 'applications/diffusion/conduit/DiffusionUpdateCoverageConduitAPIMethod.php',
    'DiffusionView' => 'applications/diffusion/view/DiffusionView.php',
    'DivinerArticleAtomizer' => 'applications/diviner/atomizer/DivinerArticleAtomizer.php',
    'DivinerAtom' => 'applications/diviner/atom/DivinerAtom.php',
    'DivinerAtomCache' => 'applications/diviner/cache/DivinerAtomCache.php',
    'DivinerAtomController' => 'applications/diviner/controller/DivinerAtomController.php',
    'DivinerAtomListController' => 'applications/diviner/controller/DivinerAtomListController.php',
    'DivinerAtomPHIDType' => 'applications/diviner/phid/DivinerAtomPHIDType.php',
    'DivinerAtomQuery' => 'applications/diviner/query/DivinerAtomQuery.php',
    'DivinerAtomRef' => 'applications/diviner/atom/DivinerAtomRef.php',
    'DivinerAtomSearchEngine' => 'applications/diviner/query/DivinerAtomSearchEngine.php',
    'DivinerAtomizeWorkflow' => 'applications/diviner/workflow/DivinerAtomizeWorkflow.php',
    'DivinerAtomizer' => 'applications/diviner/atomizer/DivinerAtomizer.php',
    'DivinerBookController' => 'applications/diviner/controller/DivinerBookController.php',
    'DivinerBookDatasource' => 'applications/diviner/typeahead/DivinerBookDatasource.php',
    'DivinerBookEditController' => 'applications/diviner/controller/DivinerBookEditController.php',
    'DivinerBookItemView' => 'applications/diviner/view/DivinerBookItemView.php',
    'DivinerBookPHIDType' => 'applications/diviner/phid/DivinerBookPHIDType.php',
    'DivinerBookQuery' => 'applications/diviner/query/DivinerBookQuery.php',
    'DivinerController' => 'applications/diviner/controller/DivinerController.php',
    'DivinerDAO' => 'applications/diviner/storage/DivinerDAO.php',
    'DivinerDefaultEditCapability' => 'applications/diviner/capability/DivinerDefaultEditCapability.php',
    'DivinerDefaultRenderer' => 'applications/diviner/renderer/DivinerDefaultRenderer.php',
    'DivinerDefaultViewCapability' => 'applications/diviner/capability/DivinerDefaultViewCapability.php',
    'DivinerDiskCache' => 'applications/diviner/cache/DivinerDiskCache.php',
    'DivinerFileAtomizer' => 'applications/diviner/atomizer/DivinerFileAtomizer.php',
    'DivinerFindController' => 'applications/diviner/controller/DivinerFindController.php',
    'DivinerGenerateWorkflow' => 'applications/diviner/workflow/DivinerGenerateWorkflow.php',
    'DivinerLiveAtom' => 'applications/diviner/storage/DivinerLiveAtom.php',
    'DivinerLiveBook' => 'applications/diviner/storage/DivinerLiveBook.php',
    'DivinerLiveBookEditor' => 'applications/diviner/editor/DivinerLiveBookEditor.php',
    'DivinerLiveBookFulltextEngine' => 'applications/diviner/search/DivinerLiveBookFulltextEngine.php',
    'DivinerLiveBookTransaction' => 'applications/diviner/storage/DivinerLiveBookTransaction.php',
    'DivinerLiveBookTransactionQuery' => 'applications/diviner/query/DivinerLiveBookTransactionQuery.php',
    'DivinerLivePublisher' => 'applications/diviner/publisher/DivinerLivePublisher.php',
    'DivinerLiveSymbol' => 'applications/diviner/storage/DivinerLiveSymbol.php',
    'DivinerLiveSymbolFulltextEngine' => 'applications/diviner/search/DivinerLiveSymbolFulltextEngine.php',
    'DivinerMainController' => 'applications/diviner/controller/DivinerMainController.php',
    'DivinerPHPAtomizer' => 'applications/diviner/atomizer/DivinerPHPAtomizer.php',
    'DivinerParameterTableView' => 'applications/diviner/view/DivinerParameterTableView.php',
    'DivinerPublishCache' => 'applications/diviner/cache/DivinerPublishCache.php',
    'DivinerPublisher' => 'applications/diviner/publisher/DivinerPublisher.php',
    'DivinerRenderer' => 'applications/diviner/renderer/DivinerRenderer.php',
    'DivinerReturnTableView' => 'applications/diviner/view/DivinerReturnTableView.php',
    'DivinerSchemaSpec' => 'applications/diviner/storage/DivinerSchemaSpec.php',
    'DivinerSectionView' => 'applications/diviner/view/DivinerSectionView.php',
    'DivinerStaticPublisher' => 'applications/diviner/publisher/DivinerStaticPublisher.php',
    'DivinerSymbolRemarkupRule' => 'applications/diviner/markup/DivinerSymbolRemarkupRule.php',
    'DivinerWorkflow' => 'applications/diviner/workflow/DivinerWorkflow.php',
    'DoorkeeperAsanaFeedWorker' => 'applications/doorkeeper/worker/DoorkeeperAsanaFeedWorker.php',
    'DoorkeeperAsanaRemarkupRule' => 'applications/doorkeeper/remarkup/DoorkeeperAsanaRemarkupRule.php',
    'DoorkeeperBridge' => 'applications/doorkeeper/bridge/DoorkeeperBridge.php',
    'DoorkeeperBridgeAsana' => 'applications/doorkeeper/bridge/DoorkeeperBridgeAsana.php',
    'DoorkeeperBridgeJIRA' => 'applications/doorkeeper/bridge/DoorkeeperBridgeJIRA.php',
    'DoorkeeperBridgeJIRATestCase' => 'applications/doorkeeper/bridge/__tests__/DoorkeeperBridgeJIRATestCase.php',
    'DoorkeeperDAO' => 'applications/doorkeeper/storage/DoorkeeperDAO.php',
    'DoorkeeperExternalObject' => 'applications/doorkeeper/storage/DoorkeeperExternalObject.php',
    'DoorkeeperExternalObjectQuery' => 'applications/doorkeeper/query/DoorkeeperExternalObjectQuery.php',
    'DoorkeeperFeedStoryPublisher' => 'applications/doorkeeper/engine/DoorkeeperFeedStoryPublisher.php',
    'DoorkeeperFeedWorker' => 'applications/doorkeeper/worker/DoorkeeperFeedWorker.php',
    'DoorkeeperImportEngine' => 'applications/doorkeeper/engine/DoorkeeperImportEngine.php',
    'DoorkeeperJIRAFeedWorker' => 'applications/doorkeeper/worker/DoorkeeperJIRAFeedWorker.php',
    'DoorkeeperJIRARemarkupRule' => 'applications/doorkeeper/remarkup/DoorkeeperJIRARemarkupRule.php',
    'DoorkeeperMissingLinkException' => 'applications/doorkeeper/exception/DoorkeeperMissingLinkException.php',
    'DoorkeeperObjectRef' => 'applications/doorkeeper/engine/DoorkeeperObjectRef.php',
    'DoorkeeperRemarkupRule' => 'applications/doorkeeper/remarkup/DoorkeeperRemarkupRule.php',
    'DoorkeeperSchemaSpec' => 'applications/doorkeeper/storage/DoorkeeperSchemaSpec.php',
    'DoorkeeperTagView' => 'applications/doorkeeper/view/DoorkeeperTagView.php',
    'DoorkeeperTagsController' => 'applications/doorkeeper/controller/DoorkeeperTagsController.php',
    'DrydockAlmanacServiceHostBlueprintImplementation' => 'applications/drydock/blueprint/DrydockAlmanacServiceHostBlueprintImplementation.php',
    'DrydockApacheWebrootInterface' => 'applications/drydock/interface/webroot/DrydockApacheWebrootInterface.php',
    'DrydockAuthorization' => 'applications/drydock/storage/DrydockAuthorization.php',
    'DrydockAuthorizationAuthorizeController' => 'applications/drydock/controller/DrydockAuthorizationAuthorizeController.php',
    'DrydockAuthorizationListController' => 'applications/drydock/controller/DrydockAuthorizationListController.php',
    'DrydockAuthorizationListView' => 'applications/drydock/view/DrydockAuthorizationListView.php',
    'DrydockAuthorizationPHIDType' => 'applications/drydock/phid/DrydockAuthorizationPHIDType.php',
    'DrydockAuthorizationQuery' => 'applications/drydock/query/DrydockAuthorizationQuery.php',
    'DrydockAuthorizationSearchEngine' => 'applications/drydock/query/DrydockAuthorizationSearchEngine.php',
    'DrydockAuthorizationViewController' => 'applications/drydock/controller/DrydockAuthorizationViewController.php',
    'DrydockBlueprint' => 'applications/drydock/storage/DrydockBlueprint.php',
    'DrydockBlueprintController' => 'applications/drydock/controller/DrydockBlueprintController.php',
    'DrydockBlueprintCoreCustomField' => 'applications/drydock/customfield/DrydockBlueprintCoreCustomField.php',
    'DrydockBlueprintCreateController' => 'applications/drydock/controller/DrydockBlueprintCreateController.php',
    'DrydockBlueprintCustomField' => 'applications/drydock/customfield/DrydockBlueprintCustomField.php',
    'DrydockBlueprintDatasource' => 'applications/drydock/typeahead/DrydockBlueprintDatasource.php',
    'DrydockBlueprintDisableController' => 'applications/drydock/controller/DrydockBlueprintDisableController.php',
    'DrydockBlueprintEditController' => 'applications/drydock/controller/DrydockBlueprintEditController.php',
    'DrydockBlueprintEditor' => 'applications/drydock/editor/DrydockBlueprintEditor.php',
    'DrydockBlueprintImplementation' => 'applications/drydock/blueprint/DrydockBlueprintImplementation.php',
    'DrydockBlueprintImplementationTestCase' => 'applications/drydock/blueprint/__tests__/DrydockBlueprintImplementationTestCase.php',
    'DrydockBlueprintListController' => 'applications/drydock/controller/DrydockBlueprintListController.php',
    'DrydockBlueprintPHIDType' => 'applications/drydock/phid/DrydockBlueprintPHIDType.php',
    'DrydockBlueprintQuery' => 'applications/drydock/query/DrydockBlueprintQuery.php',
    'DrydockBlueprintSearchEngine' => 'applications/drydock/query/DrydockBlueprintSearchEngine.php',
    'DrydockBlueprintTransaction' => 'applications/drydock/storage/DrydockBlueprintTransaction.php',
    'DrydockBlueprintTransactionQuery' => 'applications/drydock/query/DrydockBlueprintTransactionQuery.php',
    'DrydockBlueprintViewController' => 'applications/drydock/controller/DrydockBlueprintViewController.php',
    'DrydockCommand' => 'applications/drydock/storage/DrydockCommand.php',
    'DrydockCommandInterface' => 'applications/drydock/interface/command/DrydockCommandInterface.php',
    'DrydockCommandQuery' => 'applications/drydock/query/DrydockCommandQuery.php',
    'DrydockConsoleController' => 'applications/drydock/controller/DrydockConsoleController.php',
    'DrydockConstants' => 'applications/drydock/constants/DrydockConstants.php',
    'DrydockController' => 'applications/drydock/controller/DrydockController.php',
    'DrydockCreateBlueprintsCapability' => 'applications/drydock/capability/DrydockCreateBlueprintsCapability.php',
    'DrydockDAO' => 'applications/drydock/storage/DrydockDAO.php',
    'DrydockDefaultEditCapability' => 'applications/drydock/capability/DrydockDefaultEditCapability.php',
    'DrydockDefaultViewCapability' => 'applications/drydock/capability/DrydockDefaultViewCapability.php',
    'DrydockFilesystemInterface' => 'applications/drydock/interface/filesystem/DrydockFilesystemInterface.php',
    'DrydockInterface' => 'applications/drydock/interface/DrydockInterface.php',
    'DrydockLandRepositoryOperation' => 'applications/drydock/operation/DrydockLandRepositoryOperation.php',
    'DrydockLease' => 'applications/drydock/storage/DrydockLease.php',
    'DrydockLeaseAcquiredLogType' => 'applications/drydock/logtype/DrydockLeaseAcquiredLogType.php',
    'DrydockLeaseActivatedLogType' => 'applications/drydock/logtype/DrydockLeaseActivatedLogType.php',
    'DrydockLeaseActivationFailureLogType' => 'applications/drydock/logtype/DrydockLeaseActivationFailureLogType.php',
    'DrydockLeaseActivationYieldLogType' => 'applications/drydock/logtype/DrydockLeaseActivationYieldLogType.php',
    'DrydockLeaseController' => 'applications/drydock/controller/DrydockLeaseController.php',
    'DrydockLeaseDatasource' => 'applications/drydock/typeahead/DrydockLeaseDatasource.php',
    'DrydockLeaseDestroyedLogType' => 'applications/drydock/logtype/DrydockLeaseDestroyedLogType.php',
    'DrydockLeaseListController' => 'applications/drydock/controller/DrydockLeaseListController.php',
    'DrydockLeaseListView' => 'applications/drydock/view/DrydockLeaseListView.php',
    'DrydockLeaseNoAuthorizationsLogType' => 'applications/drydock/logtype/DrydockLeaseNoAuthorizationsLogType.php',
    'DrydockLeaseNoBlueprintsLogType' => 'applications/drydock/logtype/DrydockLeaseNoBlueprintsLogType.php',
    'DrydockLeasePHIDType' => 'applications/drydock/phid/DrydockLeasePHIDType.php',
    'DrydockLeaseQuery' => 'applications/drydock/query/DrydockLeaseQuery.php',
    'DrydockLeaseQueuedLogType' => 'applications/drydock/logtype/DrydockLeaseQueuedLogType.php',
    'DrydockLeaseReclaimLogType' => 'applications/drydock/logtype/DrydockLeaseReclaimLogType.php',
    'DrydockLeaseReleaseController' => 'applications/drydock/controller/DrydockLeaseReleaseController.php',
    'DrydockLeaseReleasedLogType' => 'applications/drydock/logtype/DrydockLeaseReleasedLogType.php',
    'DrydockLeaseSearchEngine' => 'applications/drydock/query/DrydockLeaseSearchEngine.php',
    'DrydockLeaseStatus' => 'applications/drydock/constants/DrydockLeaseStatus.php',
    'DrydockLeaseUpdateWorker' => 'applications/drydock/worker/DrydockLeaseUpdateWorker.php',
    'DrydockLeaseViewController' => 'applications/drydock/controller/DrydockLeaseViewController.php',
    'DrydockLeaseWaitingForResourcesLogType' => 'applications/drydock/logtype/DrydockLeaseWaitingForResourcesLogType.php',
    'DrydockLog' => 'applications/drydock/storage/DrydockLog.php',
    'DrydockLogController' => 'applications/drydock/controller/DrydockLogController.php',
    'DrydockLogGarbageCollector' => 'applications/drydock/garbagecollector/DrydockLogGarbageCollector.php',
    'DrydockLogListController' => 'applications/drydock/controller/DrydockLogListController.php',
    'DrydockLogListView' => 'applications/drydock/view/DrydockLogListView.php',
    'DrydockLogQuery' => 'applications/drydock/query/DrydockLogQuery.php',
    'DrydockLogSearchEngine' => 'applications/drydock/query/DrydockLogSearchEngine.php',
    'DrydockLogType' => 'applications/drydock/logtype/DrydockLogType.php',
    'DrydockManagementCommandWorkflow' => 'applications/drydock/management/DrydockManagementCommandWorkflow.php',
    'DrydockManagementLeaseWorkflow' => 'applications/drydock/management/DrydockManagementLeaseWorkflow.php',
    'DrydockManagementReclaimWorkflow' => 'applications/drydock/management/DrydockManagementReclaimWorkflow.php',
    'DrydockManagementReleaseLeaseWorkflow' => 'applications/drydock/management/DrydockManagementReleaseLeaseWorkflow.php',
    'DrydockManagementReleaseResourceWorkflow' => 'applications/drydock/management/DrydockManagementReleaseResourceWorkflow.php',
    'DrydockManagementUpdateLeaseWorkflow' => 'applications/drydock/management/DrydockManagementUpdateLeaseWorkflow.php',
    'DrydockManagementUpdateResourceWorkflow' => 'applications/drydock/management/DrydockManagementUpdateResourceWorkflow.php',
    'DrydockManagementWorkflow' => 'applications/drydock/management/DrydockManagementWorkflow.php',
    'DrydockObjectAuthorizationView' => 'applications/drydock/view/DrydockObjectAuthorizationView.php',
    'DrydockQuery' => 'applications/drydock/query/DrydockQuery.php',
    'DrydockRepositoryOperation' => 'applications/drydock/storage/DrydockRepositoryOperation.php',
    'DrydockRepositoryOperationDismissController' => 'applications/drydock/controller/DrydockRepositoryOperationDismissController.php',
    'DrydockRepositoryOperationListController' => 'applications/drydock/controller/DrydockRepositoryOperationListController.php',
    'DrydockRepositoryOperationPHIDType' => 'applications/drydock/phid/DrydockRepositoryOperationPHIDType.php',
    'DrydockRepositoryOperationQuery' => 'applications/drydock/query/DrydockRepositoryOperationQuery.php',
    'DrydockRepositoryOperationSearchEngine' => 'applications/drydock/query/DrydockRepositoryOperationSearchEngine.php',
    'DrydockRepositoryOperationStatusController' => 'applications/drydock/controller/DrydockRepositoryOperationStatusController.php',
    'DrydockRepositoryOperationStatusView' => 'applications/drydock/view/DrydockRepositoryOperationStatusView.php',
    'DrydockRepositoryOperationType' => 'applications/drydock/operation/DrydockRepositoryOperationType.php',
    'DrydockRepositoryOperationUpdateWorker' => 'applications/drydock/worker/DrydockRepositoryOperationUpdateWorker.php',
    'DrydockRepositoryOperationViewController' => 'applications/drydock/controller/DrydockRepositoryOperationViewController.php',
    'DrydockResource' => 'applications/drydock/storage/DrydockResource.php',
    'DrydockResourceActivationFailureLogType' => 'applications/drydock/logtype/DrydockResourceActivationFailureLogType.php',
    'DrydockResourceActivationYieldLogType' => 'applications/drydock/logtype/DrydockResourceActivationYieldLogType.php',
    'DrydockResourceController' => 'applications/drydock/controller/DrydockResourceController.php',
    'DrydockResourceDatasource' => 'applications/drydock/typeahead/DrydockResourceDatasource.php',
    'DrydockResourceListController' => 'applications/drydock/controller/DrydockResourceListController.php',
    'DrydockResourceListView' => 'applications/drydock/view/DrydockResourceListView.php',
    'DrydockResourcePHIDType' => 'applications/drydock/phid/DrydockResourcePHIDType.php',
    'DrydockResourceQuery' => 'applications/drydock/query/DrydockResourceQuery.php',
    'DrydockResourceReclaimLogType' => 'applications/drydock/logtype/DrydockResourceReclaimLogType.php',
    'DrydockResourceReleaseController' => 'applications/drydock/controller/DrydockResourceReleaseController.php',
    'DrydockResourceSearchEngine' => 'applications/drydock/query/DrydockResourceSearchEngine.php',
    'DrydockResourceStatus' => 'applications/drydock/constants/DrydockResourceStatus.php',
    'DrydockResourceUpdateWorker' => 'applications/drydock/worker/DrydockResourceUpdateWorker.php',
    'DrydockResourceViewController' => 'applications/drydock/controller/DrydockResourceViewController.php',
    'DrydockSFTPFilesystemInterface' => 'applications/drydock/interface/filesystem/DrydockSFTPFilesystemInterface.php',
    'DrydockSSHCommandInterface' => 'applications/drydock/interface/command/DrydockSSHCommandInterface.php',
    'DrydockSlotLock' => 'applications/drydock/storage/DrydockSlotLock.php',
    'DrydockSlotLockException' => 'applications/drydock/exception/DrydockSlotLockException.php',
    'DrydockSlotLockFailureLogType' => 'applications/drydock/logtype/DrydockSlotLockFailureLogType.php',
    'DrydockTestRepositoryOperation' => 'applications/drydock/operation/DrydockTestRepositoryOperation.php',
    'DrydockWebrootInterface' => 'applications/drydock/interface/webroot/DrydockWebrootInterface.php',
    'DrydockWorker' => 'applications/drydock/worker/DrydockWorker.php',
    'DrydockWorkingCopyBlueprintImplementation' => 'applications/drydock/blueprint/DrydockWorkingCopyBlueprintImplementation.php',
    'FeedConduitAPIMethod' => 'applications/feed/conduit/FeedConduitAPIMethod.php',
    'FeedPublishConduitAPIMethod' => 'applications/feed/conduit/FeedPublishConduitAPIMethod.php',
    'FeedPublisherHTTPWorker' => 'applications/feed/worker/FeedPublisherHTTPWorker.php',
    'FeedPublisherWorker' => 'applications/feed/worker/FeedPublisherWorker.php',
    'FeedPushWorker' => 'applications/feed/worker/FeedPushWorker.php',
    'FeedQueryConduitAPIMethod' => 'applications/feed/conduit/FeedQueryConduitAPIMethod.php',
    'FeedStoryNotificationGarbageCollector' => 'applications/notification/garbagecollector/FeedStoryNotificationGarbageCollector.php',
    'FileAllocateConduitAPIMethod' => 'applications/files/conduit/FileAllocateConduitAPIMethod.php',
    'FileConduitAPIMethod' => 'applications/files/conduit/FileConduitAPIMethod.php',
    'FileCreateMailReceiver' => 'applications/files/mail/FileCreateMailReceiver.php',
    'FileDownloadConduitAPIMethod' => 'applications/files/conduit/FileDownloadConduitAPIMethod.php',
    'FileInfoConduitAPIMethod' => 'applications/files/conduit/FileInfoConduitAPIMethod.php',
    'FileMailReceiver' => 'applications/files/mail/FileMailReceiver.php',
    'FileQueryChunksConduitAPIMethod' => 'applications/files/conduit/FileQueryChunksConduitAPIMethod.php',
    'FileReplyHandler' => 'applications/files/mail/FileReplyHandler.php',
    'FileUploadChunkConduitAPIMethod' => 'applications/files/conduit/FileUploadChunkConduitAPIMethod.php',
    'FileUploadConduitAPIMethod' => 'applications/files/conduit/FileUploadConduitAPIMethod.php',
    'FileUploadHashConduitAPIMethod' => 'applications/files/conduit/FileUploadHashConduitAPIMethod.php',
    'FilesDefaultViewCapability' => 'applications/files/capability/FilesDefaultViewCapability.php',
    'FlagConduitAPIMethod' => 'applications/flag/conduit/FlagConduitAPIMethod.php',
    'FlagDeleteConduitAPIMethod' => 'applications/flag/conduit/FlagDeleteConduitAPIMethod.php',
    'FlagEditConduitAPIMethod' => 'applications/flag/conduit/FlagEditConduitAPIMethod.php',
    'FlagQueryConduitAPIMethod' => 'applications/flag/conduit/FlagQueryConduitAPIMethod.php',
    'FundBacker' => 'applications/fund/storage/FundBacker.php',
    'FundBackerCart' => 'applications/fund/phortune/FundBackerCart.php',
    'FundBackerEditor' => 'applications/fund/editor/FundBackerEditor.php',
    'FundBackerListController' => 'applications/fund/controller/FundBackerListController.php',
    'FundBackerPHIDType' => 'applications/fund/phid/FundBackerPHIDType.php',
    'FundBackerProduct' => 'applications/fund/phortune/FundBackerProduct.php',
    'FundBackerQuery' => 'applications/fund/query/FundBackerQuery.php',
    'FundBackerSearchEngine' => 'applications/fund/query/FundBackerSearchEngine.php',
    'FundBackerTransaction' => 'applications/fund/storage/FundBackerTransaction.php',
    'FundBackerTransactionQuery' => 'applications/fund/query/FundBackerTransactionQuery.php',
    'FundController' => 'applications/fund/controller/FundController.php',
    'FundCreateInitiativesCapability' => 'applications/fund/capability/FundCreateInitiativesCapability.php',
    'FundDAO' => 'applications/fund/storage/FundDAO.php',
    'FundDefaultViewCapability' => 'applications/fund/capability/FundDefaultViewCapability.php',
    'FundInitiative' => 'applications/fund/storage/FundInitiative.php',
    'FundInitiativeBackController' => 'applications/fund/controller/FundInitiativeBackController.php',
    'FundInitiativeCloseController' => 'applications/fund/controller/FundInitiativeCloseController.php',
    'FundInitiativeEditController' => 'applications/fund/controller/FundInitiativeEditController.php',
    'FundInitiativeEditor' => 'applications/fund/editor/FundInitiativeEditor.php',
    'FundInitiativeFulltextEngine' => 'applications/fund/search/FundInitiativeFulltextEngine.php',
    'FundInitiativeListController' => 'applications/fund/controller/FundInitiativeListController.php',
    'FundInitiativePHIDType' => 'applications/fund/phid/FundInitiativePHIDType.php',
    'FundInitiativeQuery' => 'applications/fund/query/FundInitiativeQuery.php',
    'FundInitiativeRemarkupRule' => 'applications/fund/remarkup/FundInitiativeRemarkupRule.php',
    'FundInitiativeReplyHandler' => 'applications/fund/mail/FundInitiativeReplyHandler.php',
    'FundInitiativeSearchEngine' => 'applications/fund/query/FundInitiativeSearchEngine.php',
    'FundInitiativeTransaction' => 'applications/fund/storage/FundInitiativeTransaction.php',
    'FundInitiativeTransactionQuery' => 'applications/fund/query/FundInitiativeTransactionQuery.php',
    'FundInitiativeViewController' => 'applications/fund/controller/FundInitiativeViewController.php',
    'FundSchemaSpec' => 'applications/fund/storage/FundSchemaSpec.php',
    'HarbormasterArcLintBuildStepImplementation' => 'applications/harbormaster/step/HarbormasterArcLintBuildStepImplementation.php',
    'HarbormasterArcUnitBuildStepImplementation' => 'applications/harbormaster/step/HarbormasterArcUnitBuildStepImplementation.php',
    'HarbormasterArtifact' => 'applications/harbormaster/artifact/HarbormasterArtifact.php',
    'HarbormasterAutotargetsTestCase' => 'applications/harbormaster/__tests__/HarbormasterAutotargetsTestCase.php',
    'HarbormasterBuild' => 'applications/harbormaster/storage/build/HarbormasterBuild.php',
    'HarbormasterBuildAbortedException' => 'applications/harbormaster/exception/HarbormasterBuildAbortedException.php',
    'HarbormasterBuildActionController' => 'applications/harbormaster/controller/HarbormasterBuildActionController.php',
    'HarbormasterBuildArcanistAutoplan' => 'applications/harbormaster/autoplan/HarbormasterBuildArcanistAutoplan.php',
    'HarbormasterBuildArtifact' => 'applications/harbormaster/storage/build/HarbormasterBuildArtifact.php',
    'HarbormasterBuildArtifactPHIDType' => 'applications/harbormaster/phid/HarbormasterBuildArtifactPHIDType.php',
    'HarbormasterBuildArtifactQuery' => 'applications/harbormaster/query/HarbormasterBuildArtifactQuery.php',
    'HarbormasterBuildAutoplan' => 'applications/harbormaster/autoplan/HarbormasterBuildAutoplan.php',
    'HarbormasterBuildCommand' => 'applications/harbormaster/storage/HarbormasterBuildCommand.php',
    'HarbormasterBuildDependencyDatasource' => 'applications/harbormaster/typeahead/HarbormasterBuildDependencyDatasource.php',
    'HarbormasterBuildEngine' => 'applications/harbormaster/engine/HarbormasterBuildEngine.php',
    'HarbormasterBuildFailureException' => 'applications/harbormaster/exception/HarbormasterBuildFailureException.php',
    'HarbormasterBuildGraph' => 'applications/harbormaster/engine/HarbormasterBuildGraph.php',
    'HarbormasterBuildLintMessage' => 'applications/harbormaster/storage/build/HarbormasterBuildLintMessage.php',
    'HarbormasterBuildLog' => 'applications/harbormaster/storage/build/HarbormasterBuildLog.php',
    'HarbormasterBuildLogPHIDType' => 'applications/harbormaster/phid/HarbormasterBuildLogPHIDType.php',
    'HarbormasterBuildLogQuery' => 'applications/harbormaster/query/HarbormasterBuildLogQuery.php',
    'HarbormasterBuildMessage' => 'applications/harbormaster/storage/HarbormasterBuildMessage.php',
    'HarbormasterBuildMessageQuery' => 'applications/harbormaster/query/HarbormasterBuildMessageQuery.php',
    'HarbormasterBuildPHIDType' => 'applications/harbormaster/phid/HarbormasterBuildPHIDType.php',
    'HarbormasterBuildPlan' => 'applications/harbormaster/storage/configuration/HarbormasterBuildPlan.php',
    'HarbormasterBuildPlanDatasource' => 'applications/harbormaster/typeahead/HarbormasterBuildPlanDatasource.php',
    'HarbormasterBuildPlanDefaultEditCapability' => 'applications/harbormaster/capability/HarbormasterBuildPlanDefaultEditCapability.php',
    'HarbormasterBuildPlanDefaultViewCapability' => 'applications/harbormaster/capability/HarbormasterBuildPlanDefaultViewCapability.php',
    'HarbormasterBuildPlanEditor' => 'applications/harbormaster/editor/HarbormasterBuildPlanEditor.php',
    'HarbormasterBuildPlanPHIDType' => 'applications/harbormaster/phid/HarbormasterBuildPlanPHIDType.php',
    'HarbormasterBuildPlanQuery' => 'applications/harbormaster/query/HarbormasterBuildPlanQuery.php',
    'HarbormasterBuildPlanSearchEngine' => 'applications/harbormaster/query/HarbormasterBuildPlanSearchEngine.php',
    'HarbormasterBuildPlanTransaction' => 'applications/harbormaster/storage/configuration/HarbormasterBuildPlanTransaction.php',
    'HarbormasterBuildPlanTransactionQuery' => 'applications/harbormaster/query/HarbormasterBuildPlanTransactionQuery.php',
    'HarbormasterBuildQuery' => 'applications/harbormaster/query/HarbormasterBuildQuery.php',
    'HarbormasterBuildRequest' => 'applications/harbormaster/engine/HarbormasterBuildRequest.php',
    'HarbormasterBuildStep' => 'applications/harbormaster/storage/configuration/HarbormasterBuildStep.php',
    'HarbormasterBuildStepCoreCustomField' => 'applications/harbormaster/customfield/HarbormasterBuildStepCoreCustomField.php',
    'HarbormasterBuildStepCustomField' => 'applications/harbormaster/customfield/HarbormasterBuildStepCustomField.php',
    'HarbormasterBuildStepEditor' => 'applications/harbormaster/editor/HarbormasterBuildStepEditor.php',
    'HarbormasterBuildStepGroup' => 'applications/harbormaster/stepgroup/HarbormasterBuildStepGroup.php',
    'HarbormasterBuildStepImplementation' => 'applications/harbormaster/step/HarbormasterBuildStepImplementation.php',
    'HarbormasterBuildStepImplementationTestCase' => 'applications/harbormaster/step/__tests__/HarbormasterBuildStepImplementationTestCase.php',
    'HarbormasterBuildStepPHIDType' => 'applications/harbormaster/phid/HarbormasterBuildStepPHIDType.php',
    'HarbormasterBuildStepQuery' => 'applications/harbormaster/query/HarbormasterBuildStepQuery.php',
    'HarbormasterBuildStepTransaction' => 'applications/harbormaster/storage/configuration/HarbormasterBuildStepTransaction.php',
    'HarbormasterBuildStepTransactionQuery' => 'applications/harbormaster/query/HarbormasterBuildStepTransactionQuery.php',
    'HarbormasterBuildTarget' => 'applications/harbormaster/storage/build/HarbormasterBuildTarget.php',
    'HarbormasterBuildTargetPHIDType' => 'applications/harbormaster/phid/HarbormasterBuildTargetPHIDType.php',
    'HarbormasterBuildTargetQuery' => 'applications/harbormaster/query/HarbormasterBuildTargetQuery.php',
    'HarbormasterBuildTransaction' => 'applications/harbormaster/storage/HarbormasterBuildTransaction.php',
    'HarbormasterBuildTransactionEditor' => 'applications/harbormaster/editor/HarbormasterBuildTransactionEditor.php',
    'HarbormasterBuildTransactionQuery' => 'applications/harbormaster/query/HarbormasterBuildTransactionQuery.php',
    'HarbormasterBuildUnitMessage' => 'applications/harbormaster/storage/build/HarbormasterBuildUnitMessage.php',
    'HarbormasterBuildViewController' => 'applications/harbormaster/controller/HarbormasterBuildViewController.php',
    'HarbormasterBuildWorker' => 'applications/harbormaster/worker/HarbormasterBuildWorker.php',
    'HarbormasterBuildable' => 'applications/harbormaster/storage/HarbormasterBuildable.php',
    'HarbormasterBuildableActionController' => 'applications/harbormaster/controller/HarbormasterBuildableActionController.php',
    'HarbormasterBuildableAdapterInterface' => 'applications/harbormaster/herald/HarbormasterBuildableAdapterInterface.php',
    'HarbormasterBuildableInterface' => 'applications/harbormaster/interface/HarbormasterBuildableInterface.php',
    'HarbormasterBuildableListController' => 'applications/harbormaster/controller/HarbormasterBuildableListController.php',
    'HarbormasterBuildablePHIDType' => 'applications/harbormaster/phid/HarbormasterBuildablePHIDType.php',
    'HarbormasterBuildableQuery' => 'applications/harbormaster/query/HarbormasterBuildableQuery.php',
    'HarbormasterBuildableSearchEngine' => 'applications/harbormaster/query/HarbormasterBuildableSearchEngine.php',
    'HarbormasterBuildableTransaction' => 'applications/harbormaster/storage/HarbormasterBuildableTransaction.php',
    'HarbormasterBuildableTransactionEditor' => 'applications/harbormaster/editor/HarbormasterBuildableTransactionEditor.php',
    'HarbormasterBuildableTransactionQuery' => 'applications/harbormaster/query/HarbormasterBuildableTransactionQuery.php',
    'HarbormasterBuildableViewController' => 'applications/harbormaster/controller/HarbormasterBuildableViewController.php',
    'HarbormasterBuiltinBuildStepGroup' => 'applications/harbormaster/stepgroup/HarbormasterBuiltinBuildStepGroup.php',
    'HarbormasterCommandBuildStepImplementation' => 'applications/harbormaster/step/HarbormasterCommandBuildStepImplementation.php',
    'HarbormasterConduitAPIMethod' => 'applications/harbormaster/conduit/HarbormasterConduitAPIMethod.php',
    'HarbormasterController' => 'applications/harbormaster/controller/HarbormasterController.php',
    'HarbormasterCreateArtifactConduitAPIMethod' => 'applications/harbormaster/conduit/HarbormasterCreateArtifactConduitAPIMethod.php',
    'HarbormasterCreatePlansCapability' => 'applications/harbormaster/capability/HarbormasterCreatePlansCapability.php',
    'HarbormasterDAO' => 'applications/harbormaster/storage/HarbormasterDAO.php',
    'HarbormasterDrydockBuildStepGroup' => 'applications/harbormaster/stepgroup/HarbormasterDrydockBuildStepGroup.php',
    'HarbormasterDrydockCommandBuildStepImplementation' => 'applications/harbormaster/step/HarbormasterDrydockCommandBuildStepImplementation.php',
    'HarbormasterDrydockLeaseArtifact' => 'applications/harbormaster/artifact/HarbormasterDrydockLeaseArtifact.php',
    'HarbormasterExecFuture' => 'applications/harbormaster/future/HarbormasterExecFuture.php',
    'HarbormasterExternalBuildStepGroup' => 'applications/harbormaster/stepgroup/HarbormasterExternalBuildStepGroup.php',
    'HarbormasterFileArtifact' => 'applications/harbormaster/artifact/HarbormasterFileArtifact.php',
    'HarbormasterHTTPRequestBuildStepImplementation' => 'applications/harbormaster/step/HarbormasterHTTPRequestBuildStepImplementation.php',
    'HarbormasterHostArtifact' => 'applications/harbormaster/artifact/HarbormasterHostArtifact.php',
    'HarbormasterLeaseHostBuildStepImplementation' => 'applications/harbormaster/step/HarbormasterLeaseHostBuildStepImplementation.php',
    'HarbormasterLeaseWorkingCopyBuildStepImplementation' => 'applications/harbormaster/step/HarbormasterLeaseWorkingCopyBuildStepImplementation.php',
    'HarbormasterLintMessagesController' => 'applications/harbormaster/controller/HarbormasterLintMessagesController.php',
    'HarbormasterLintPropertyView' => 'applications/harbormaster/view/HarbormasterLintPropertyView.php',
    'HarbormasterManagementBuildWorkflow' => 'applications/harbormaster/management/HarbormasterManagementBuildWorkflow.php',
    'HarbormasterManagementUpdateWorkflow' => 'applications/harbormaster/management/HarbormasterManagementUpdateWorkflow.php',
    'HarbormasterManagementWorkflow' => 'applications/harbormaster/management/HarbormasterManagementWorkflow.php',
    'HarbormasterMessageType' => 'applications/harbormaster/engine/HarbormasterMessageType.php',
    'HarbormasterObject' => 'applications/harbormaster/storage/HarbormasterObject.php',
    'HarbormasterOtherBuildStepGroup' => 'applications/harbormaster/stepgroup/HarbormasterOtherBuildStepGroup.php',
    'HarbormasterPlanController' => 'applications/harbormaster/controller/HarbormasterPlanController.php',
    'HarbormasterPlanDisableController' => 'applications/harbormaster/controller/HarbormasterPlanDisableController.php',
    'HarbormasterPlanEditController' => 'applications/harbormaster/controller/HarbormasterPlanEditController.php',
    'HarbormasterPlanListController' => 'applications/harbormaster/controller/HarbormasterPlanListController.php',
    'HarbormasterPlanRunController' => 'applications/harbormaster/controller/HarbormasterPlanRunController.php',
    'HarbormasterPlanViewController' => 'applications/harbormaster/controller/HarbormasterPlanViewController.php',
    'HarbormasterPrototypeBuildStepGroup' => 'applications/harbormaster/stepgroup/HarbormasterPrototypeBuildStepGroup.php',
    'HarbormasterPublishFragmentBuildStepImplementation' => 'applications/harbormaster/step/HarbormasterPublishFragmentBuildStepImplementation.php',
    'HarbormasterQueryAutotargetsConduitAPIMethod' => 'applications/harbormaster/conduit/HarbormasterQueryAutotargetsConduitAPIMethod.php',
    'HarbormasterQueryBuildablesConduitAPIMethod' => 'applications/harbormaster/conduit/HarbormasterQueryBuildablesConduitAPIMethod.php',
    'HarbormasterQueryBuildsConduitAPIMethod' => 'applications/harbormaster/conduit/HarbormasterQueryBuildsConduitAPIMethod.php',
    'HarbormasterRemarkupRule' => 'applications/harbormaster/remarkup/HarbormasterRemarkupRule.php',
    'HarbormasterRunBuildPlansHeraldAction' => 'applications/harbormaster/herald/HarbormasterRunBuildPlansHeraldAction.php',
    'HarbormasterSchemaSpec' => 'applications/harbormaster/storage/HarbormasterSchemaSpec.php',
    'HarbormasterScratchTable' => 'applications/harbormaster/storage/HarbormasterScratchTable.php',
    'HarbormasterSendMessageConduitAPIMethod' => 'applications/harbormaster/conduit/HarbormasterSendMessageConduitAPIMethod.php',
    'HarbormasterSleepBuildStepImplementation' => 'applications/harbormaster/step/HarbormasterSleepBuildStepImplementation.php',
    'HarbormasterStepAddController' => 'applications/harbormaster/controller/HarbormasterStepAddController.php',
    'HarbormasterStepDeleteController' => 'applications/harbormaster/controller/HarbormasterStepDeleteController.php',
    'HarbormasterStepEditController' => 'applications/harbormaster/controller/HarbormasterStepEditController.php',
    'HarbormasterStepViewController' => 'applications/harbormaster/controller/HarbormasterStepViewController.php',
    'HarbormasterTargetEngine' => 'applications/harbormaster/engine/HarbormasterTargetEngine.php',
    'HarbormasterTargetWorker' => 'applications/harbormaster/worker/HarbormasterTargetWorker.php',
    'HarbormasterTestBuildStepGroup' => 'applications/harbormaster/stepgroup/HarbormasterTestBuildStepGroup.php',
    'HarbormasterThrowExceptionBuildStep' => 'applications/harbormaster/step/HarbormasterThrowExceptionBuildStep.php',
    'HarbormasterUIEventListener' => 'applications/harbormaster/event/HarbormasterUIEventListener.php',
    'HarbormasterURIArtifact' => 'applications/harbormaster/artifact/HarbormasterURIArtifact.php',
    'HarbormasterUnitMessagesController' => 'applications/harbormaster/controller/HarbormasterUnitMessagesController.php',
    'HarbormasterUnitPropertyView' => 'applications/harbormaster/view/HarbormasterUnitPropertyView.php',
    'HarbormasterUploadArtifactBuildStepImplementation' => 'applications/harbormaster/step/HarbormasterUploadArtifactBuildStepImplementation.php',
    'HarbormasterWaitForPreviousBuildStepImplementation' => 'applications/harbormaster/step/HarbormasterWaitForPreviousBuildStepImplementation.php',
    'HarbormasterWorker' => 'applications/harbormaster/worker/HarbormasterWorker.php',
    'HarbormasterWorkingCopyArtifact' => 'applications/harbormaster/artifact/HarbormasterWorkingCopyArtifact.php',
    'HeraldAction' => 'applications/herald/action/HeraldAction.php',
    'HeraldActionGroup' => 'applications/herald/action/HeraldActionGroup.php',
    'HeraldActionRecord' => 'applications/herald/storage/HeraldActionRecord.php',
    'HeraldAdapter' => 'applications/herald/adapter/HeraldAdapter.php',
    'HeraldAlwaysField' => 'applications/herald/field/HeraldAlwaysField.php',
    'HeraldAnotherRuleField' => 'applications/herald/field/HeraldAnotherRuleField.php',
    'HeraldApplicationActionGroup' => 'applications/herald/action/HeraldApplicationActionGroup.php',
    'HeraldApplyTranscript' => 'applications/herald/storage/transcript/HeraldApplyTranscript.php',
    'HeraldBasicFieldGroup' => 'applications/herald/field/HeraldBasicFieldGroup.php',
    'HeraldCommitAdapter' => 'applications/diffusion/herald/HeraldCommitAdapter.php',
    'HeraldCondition' => 'applications/herald/storage/HeraldCondition.php',
    'HeraldConditionTranscript' => 'applications/herald/storage/transcript/HeraldConditionTranscript.php',
    'HeraldContentSourceField' => 'applications/herald/field/HeraldContentSourceField.php',
    'HeraldController' => 'applications/herald/controller/HeraldController.php',
    'HeraldDAO' => 'applications/herald/storage/HeraldDAO.php',
    'HeraldDifferentialAdapter' => 'applications/differential/herald/HeraldDifferentialAdapter.php',
    'HeraldDifferentialDiffAdapter' => 'applications/differential/herald/HeraldDifferentialDiffAdapter.php',
    'HeraldDifferentialRevisionAdapter' => 'applications/differential/herald/HeraldDifferentialRevisionAdapter.php',
    'HeraldDisableController' => 'applications/herald/controller/HeraldDisableController.php',
    'HeraldDoNothingAction' => 'applications/herald/action/HeraldDoNothingAction.php',
    'HeraldEditFieldGroup' => 'applications/herald/field/HeraldEditFieldGroup.php',
    'HeraldEffect' => 'applications/herald/engine/HeraldEffect.php',
    'HeraldEmptyFieldValue' => 'applications/herald/value/HeraldEmptyFieldValue.php',
    'HeraldEngine' => 'applications/herald/engine/HeraldEngine.php',
    'HeraldExactProjectsField' => 'applications/project/herald/HeraldExactProjectsField.php',
    'HeraldField' => 'applications/herald/field/HeraldField.php',
    'HeraldFieldGroup' => 'applications/herald/field/HeraldFieldGroup.php',
    'HeraldFieldTestCase' => 'applications/herald/field/__tests__/HeraldFieldTestCase.php',
    'HeraldFieldValue' => 'applications/herald/value/HeraldFieldValue.php',
    'HeraldGroup' => 'applications/herald/group/HeraldGroup.php',
    'HeraldInvalidActionException' => 'applications/herald/engine/exception/HeraldInvalidActionException.php',
    'HeraldInvalidConditionException' => 'applications/herald/engine/exception/HeraldInvalidConditionException.php',
    'HeraldManageGlobalRulesCapability' => 'applications/herald/capability/HeraldManageGlobalRulesCapability.php',
    'HeraldManiphestTaskAdapter' => 'applications/maniphest/herald/HeraldManiphestTaskAdapter.php',
    'HeraldNewController' => 'applications/herald/controller/HeraldNewController.php',
    'HeraldNewObjectField' => 'applications/herald/field/HeraldNewObjectField.php',
    'HeraldNotifyActionGroup' => 'applications/herald/action/HeraldNotifyActionGroup.php',
    'HeraldObjectTranscript' => 'applications/herald/storage/transcript/HeraldObjectTranscript.php',
    'HeraldPhameBlogAdapter' => 'applications/phame/herald/HeraldPhameBlogAdapter.php',
    'HeraldPhamePostAdapter' => 'applications/phame/herald/HeraldPhamePostAdapter.php',
    'HeraldPholioMockAdapter' => 'applications/pholio/herald/HeraldPholioMockAdapter.php',
    'HeraldPonderQuestionAdapter' => 'applications/ponder/herald/HeraldPonderQuestionAdapter.php',
    'HeraldPreCommitAdapter' => 'applications/diffusion/herald/HeraldPreCommitAdapter.php',
    'HeraldPreCommitContentAdapter' => 'applications/diffusion/herald/HeraldPreCommitContentAdapter.php',
    'HeraldPreCommitRefAdapter' => 'applications/diffusion/herald/HeraldPreCommitRefAdapter.php',
    'HeraldPreventActionGroup' => 'applications/herald/action/HeraldPreventActionGroup.php',
    'HeraldProjectsField' => 'applications/project/herald/HeraldProjectsField.php',
    'HeraldRecursiveConditionsException' => 'applications/herald/engine/exception/HeraldRecursiveConditionsException.php',
    'HeraldRelatedFieldGroup' => 'applications/herald/field/HeraldRelatedFieldGroup.php',
    'HeraldRemarkupRule' => 'applications/herald/remarkup/HeraldRemarkupRule.php',
    'HeraldRepetitionPolicyConfig' => 'applications/herald/config/HeraldRepetitionPolicyConfig.php',
    'HeraldRule' => 'applications/herald/storage/HeraldRule.php',
    'HeraldRuleController' => 'applications/herald/controller/HeraldRuleController.php',
    'HeraldRuleEditor' => 'applications/herald/editor/HeraldRuleEditor.php',
    'HeraldRuleListController' => 'applications/herald/controller/HeraldRuleListController.php',
    'HeraldRulePHIDType' => 'applications/herald/phid/HeraldRulePHIDType.php',
    'HeraldRuleQuery' => 'applications/herald/query/HeraldRuleQuery.php',
    'HeraldRuleSearchEngine' => 'applications/herald/query/HeraldRuleSearchEngine.php',
    'HeraldRuleTestCase' => 'applications/herald/storage/__tests__/HeraldRuleTestCase.php',
    'HeraldRuleTransaction' => 'applications/herald/storage/HeraldRuleTransaction.php',
    'HeraldRuleTransactionComment' => 'applications/herald/storage/HeraldRuleTransactionComment.php',
    'HeraldRuleTranscript' => 'applications/herald/storage/transcript/HeraldRuleTranscript.php',
    'HeraldRuleTypeConfig' => 'applications/herald/config/HeraldRuleTypeConfig.php',
    'HeraldRuleViewController' => 'applications/herald/controller/HeraldRuleViewController.php',
    'HeraldSchemaSpec' => 'applications/herald/storage/HeraldSchemaSpec.php',
    'HeraldSelectFieldValue' => 'applications/herald/value/HeraldSelectFieldValue.php',
    'HeraldSpaceField' => 'applications/spaces/herald/HeraldSpaceField.php',
    'HeraldSubscribersField' => 'applications/subscriptions/herald/HeraldSubscribersField.php',
    'HeraldSupportActionGroup' => 'applications/herald/action/HeraldSupportActionGroup.php',
    'HeraldSupportFieldGroup' => 'applications/herald/field/HeraldSupportFieldGroup.php',
    'HeraldTestConsoleController' => 'applications/herald/controller/HeraldTestConsoleController.php',
    'HeraldTextFieldValue' => 'applications/herald/value/HeraldTextFieldValue.php',
    'HeraldTokenizerFieldValue' => 'applications/herald/value/HeraldTokenizerFieldValue.php',
    'HeraldTransactionQuery' => 'applications/herald/query/HeraldTransactionQuery.php',
    'HeraldTranscript' => 'applications/herald/storage/transcript/HeraldTranscript.php',
    'HeraldTranscriptController' => 'applications/herald/controller/HeraldTranscriptController.php',
    'HeraldTranscriptDestructionEngineExtension' => 'applications/herald/engineextension/HeraldTranscriptDestructionEngineExtension.php',
    'HeraldTranscriptGarbageCollector' => 'applications/herald/garbagecollector/HeraldTranscriptGarbageCollector.php',
    'HeraldTranscriptListController' => 'applications/herald/controller/HeraldTranscriptListController.php',
    'HeraldTranscriptPHIDType' => 'applications/herald/phid/HeraldTranscriptPHIDType.php',
    'HeraldTranscriptQuery' => 'applications/herald/query/HeraldTranscriptQuery.php',
    'HeraldTranscriptSearchEngine' => 'applications/herald/query/HeraldTranscriptSearchEngine.php',
    'HeraldTranscriptTestCase' => 'applications/herald/storage/__tests__/HeraldTranscriptTestCase.php',
    'HeraldUtilityActionGroup' => 'applications/herald/action/HeraldUtilityActionGroup.php',
    'Javelin' => 'infrastructure/javelin/Javelin.php',
    'JavelinReactorUIExample' => 'applications/uiexample/examples/JavelinReactorUIExample.php',
    'JavelinUIExample' => 'applications/uiexample/examples/JavelinUIExample.php',
    'JavelinViewExampleServerView' => 'applications/uiexample/examples/JavelinViewExampleServerView.php',
    'JavelinViewUIExample' => 'applications/uiexample/examples/JavelinViewUIExample.php',
    'LegalpadController' => 'applications/legalpad/controller/LegalpadController.php',
    'LegalpadCreateDocumentsCapability' => 'applications/legalpad/capability/LegalpadCreateDocumentsCapability.php',
    'LegalpadDAO' => 'applications/legalpad/storage/LegalpadDAO.php',
    'LegalpadDefaultEditCapability' => 'applications/legalpad/capability/LegalpadDefaultEditCapability.php',
    'LegalpadDefaultViewCapability' => 'applications/legalpad/capability/LegalpadDefaultViewCapability.php',
    'LegalpadDocument' => 'applications/legalpad/storage/LegalpadDocument.php',
    'LegalpadDocumentBody' => 'applications/legalpad/storage/LegalpadDocumentBody.php',
    'LegalpadDocumentCommentController' => 'applications/legalpad/controller/LegalpadDocumentCommentController.php',
    'LegalpadDocumentDatasource' => 'applications/legalpad/typeahead/LegalpadDocumentDatasource.php',
    'LegalpadDocumentDoneController' => 'applications/legalpad/controller/LegalpadDocumentDoneController.php',
    'LegalpadDocumentEditController' => 'applications/legalpad/controller/LegalpadDocumentEditController.php',
    'LegalpadDocumentEditor' => 'applications/legalpad/editor/LegalpadDocumentEditor.php',
    'LegalpadDocumentListController' => 'applications/legalpad/controller/LegalpadDocumentListController.php',
    'LegalpadDocumentManageController' => 'applications/legalpad/controller/LegalpadDocumentManageController.php',
    'LegalpadDocumentQuery' => 'applications/legalpad/query/LegalpadDocumentQuery.php',
    'LegalpadDocumentRemarkupRule' => 'applications/legalpad/remarkup/LegalpadDocumentRemarkupRule.php',
    'LegalpadDocumentSearchEngine' => 'applications/legalpad/query/LegalpadDocumentSearchEngine.php',
    'LegalpadDocumentSignController' => 'applications/legalpad/controller/LegalpadDocumentSignController.php',
    'LegalpadDocumentSignature' => 'applications/legalpad/storage/LegalpadDocumentSignature.php',
    'LegalpadDocumentSignatureAddController' => 'applications/legalpad/controller/LegalpadDocumentSignatureAddController.php',
    'LegalpadDocumentSignatureListController' => 'applications/legalpad/controller/LegalpadDocumentSignatureListController.php',
    'LegalpadDocumentSignatureQuery' => 'applications/legalpad/query/LegalpadDocumentSignatureQuery.php',
    'LegalpadDocumentSignatureSearchEngine' => 'applications/legalpad/query/LegalpadDocumentSignatureSearchEngine.php',
    'LegalpadDocumentSignatureVerificationController' => 'applications/legalpad/controller/LegalpadDocumentSignatureVerificationController.php',
    'LegalpadDocumentSignatureViewController' => 'applications/legalpad/controller/LegalpadDocumentSignatureViewController.php',
    'LegalpadMailReceiver' => 'applications/legalpad/mail/LegalpadMailReceiver.php',
    'LegalpadObjectNeedsSignatureEdgeType' => 'applications/legalpad/edge/LegalpadObjectNeedsSignatureEdgeType.php',
    'LegalpadReplyHandler' => 'applications/legalpad/mail/LegalpadReplyHandler.php',
    'LegalpadRequireSignatureHeraldAction' => 'applications/legalpad/herald/LegalpadRequireSignatureHeraldAction.php',
    'LegalpadSchemaSpec' => 'applications/legalpad/storage/LegalpadSchemaSpec.php',
    'LegalpadSignatureNeededByObjectEdgeType' => 'applications/legalpad/edge/LegalpadSignatureNeededByObjectEdgeType.php',
    'LegalpadTransaction' => 'applications/legalpad/storage/LegalpadTransaction.php',
    'LegalpadTransactionComment' => 'applications/legalpad/storage/LegalpadTransactionComment.php',
    'LegalpadTransactionQuery' => 'applications/legalpad/query/LegalpadTransactionQuery.php',
    'LegalpadTransactionView' => 'applications/legalpad/view/LegalpadTransactionView.php',
    'LiskChunkTestCase' => 'infrastructure/storage/lisk/__tests__/LiskChunkTestCase.php',
    'LiskDAO' => 'infrastructure/storage/lisk/LiskDAO.php',
    'LiskDAOSet' => 'infrastructure/storage/lisk/LiskDAOSet.php',
    'LiskDAOTestCase' => 'infrastructure/storage/lisk/__tests__/LiskDAOTestCase.php',
    'LiskEphemeralObjectException' => 'infrastructure/storage/lisk/LiskEphemeralObjectException.php',
    'LiskFixtureTestCase' => 'infrastructure/storage/lisk/__tests__/LiskFixtureTestCase.php',
    'LiskIsolationTestCase' => 'infrastructure/storage/lisk/__tests__/LiskIsolationTestCase.php',
    'LiskIsolationTestDAO' => 'infrastructure/storage/lisk/__tests__/LiskIsolationTestDAO.php',
    'LiskIsolationTestDAOException' => 'infrastructure/storage/lisk/__tests__/LiskIsolationTestDAOException.php',
    'LiskMigrationIterator' => 'infrastructure/storage/lisk/LiskMigrationIterator.php',
    'LiskRawMigrationIterator' => 'infrastructure/storage/lisk/LiskRawMigrationIterator.php',
    'MacroConduitAPIMethod' => 'applications/macro/conduit/MacroConduitAPIMethod.php',
    'MacroCreateMemeConduitAPIMethod' => 'applications/macro/conduit/MacroCreateMemeConduitAPIMethod.php',
    'MacroQueryConduitAPIMethod' => 'applications/macro/conduit/MacroQueryConduitAPIMethod.php',
    'ManiphestAssignEmailCommand' => 'applications/maniphest/command/ManiphestAssignEmailCommand.php',
    'ManiphestAssigneeDatasource' => 'applications/maniphest/typeahead/ManiphestAssigneeDatasource.php',
    'ManiphestBatchEditController' => 'applications/maniphest/controller/ManiphestBatchEditController.php',
    'ManiphestBulkEditCapability' => 'applications/maniphest/capability/ManiphestBulkEditCapability.php',
    'ManiphestClaimEmailCommand' => 'applications/maniphest/command/ManiphestClaimEmailCommand.php',
    'ManiphestCloseEmailCommand' => 'applications/maniphest/command/ManiphestCloseEmailCommand.php',
    'ManiphestConduitAPIMethod' => 'applications/maniphest/conduit/ManiphestConduitAPIMethod.php',
    'ManiphestConfiguredCustomField' => 'applications/maniphest/field/ManiphestConfiguredCustomField.php',
    'ManiphestConstants' => 'applications/maniphest/constants/ManiphestConstants.php',
    'ManiphestController' => 'applications/maniphest/controller/ManiphestController.php',
    'ManiphestCreateMailReceiver' => 'applications/maniphest/mail/ManiphestCreateMailReceiver.php',
    'ManiphestCreateTaskConduitAPIMethod' => 'applications/maniphest/conduit/ManiphestCreateTaskConduitAPIMethod.php',
    'ManiphestCustomField' => 'applications/maniphest/field/ManiphestCustomField.php',
    'ManiphestCustomFieldNumericIndex' => 'applications/maniphest/storage/ManiphestCustomFieldNumericIndex.php',
    'ManiphestCustomFieldStatusParser' => 'applications/maniphest/field/parser/ManiphestCustomFieldStatusParser.php',
    'ManiphestCustomFieldStatusParserTestCase' => 'applications/maniphest/field/parser/__tests__/ManiphestCustomFieldStatusParserTestCase.php',
    'ManiphestCustomFieldStorage' => 'applications/maniphest/storage/ManiphestCustomFieldStorage.php',
    'ManiphestCustomFieldStringIndex' => 'applications/maniphest/storage/ManiphestCustomFieldStringIndex.php',
    'ManiphestDAO' => 'applications/maniphest/storage/ManiphestDAO.php',
    'ManiphestDefaultEditCapability' => 'applications/maniphest/capability/ManiphestDefaultEditCapability.php',
    'ManiphestDefaultViewCapability' => 'applications/maniphest/capability/ManiphestDefaultViewCapability.php',
    'ManiphestEditAssignCapability' => 'applications/maniphest/capability/ManiphestEditAssignCapability.php',
    'ManiphestEditConduitAPIMethod' => 'applications/maniphest/conduit/ManiphestEditConduitAPIMethod.php',
    'ManiphestEditEngine' => 'applications/maniphest/editor/ManiphestEditEngine.php',
    'ManiphestEditPoliciesCapability' => 'applications/maniphest/capability/ManiphestEditPoliciesCapability.php',
    'ManiphestEditPriorityCapability' => 'applications/maniphest/capability/ManiphestEditPriorityCapability.php',
    'ManiphestEditProjectsCapability' => 'applications/maniphest/capability/ManiphestEditProjectsCapability.php',
    'ManiphestEditStatusCapability' => 'applications/maniphest/capability/ManiphestEditStatusCapability.php',
    'ManiphestEmailCommand' => 'applications/maniphest/command/ManiphestEmailCommand.php',
    'ManiphestExcelDefaultFormat' => 'applications/maniphest/export/ManiphestExcelDefaultFormat.php',
    'ManiphestExcelFormat' => 'applications/maniphest/export/ManiphestExcelFormat.php',
    'ManiphestExcelFormatTestCase' => 'applications/maniphest/export/__tests__/ManiphestExcelFormatTestCase.php',
    'ManiphestExportController' => 'applications/maniphest/controller/ManiphestExportController.php',
    'ManiphestGetTaskTransactionsConduitAPIMethod' => 'applications/maniphest/conduit/ManiphestGetTaskTransactionsConduitAPIMethod.php',
    'ManiphestHovercardEngineExtension' => 'applications/maniphest/engineextension/ManiphestHovercardEngineExtension.php',
    'ManiphestInfoConduitAPIMethod' => 'applications/maniphest/conduit/ManiphestInfoConduitAPIMethod.php',
    'ManiphestNameIndex' => 'applications/maniphest/storage/ManiphestNameIndex.php',
    'ManiphestPriorityConfigOptionType' => 'applications/maniphest/config/ManiphestPriorityConfigOptionType.php',
    'ManiphestPriorityEmailCommand' => 'applications/maniphest/command/ManiphestPriorityEmailCommand.php',
    'ManiphestProjectNameFulltextEngineExtension' => 'applications/maniphest/engineextension/ManiphestProjectNameFulltextEngineExtension.php',
    'ManiphestQueryConduitAPIMethod' => 'applications/maniphest/conduit/ManiphestQueryConduitAPIMethod.php',
    'ManiphestQueryStatusesConduitAPIMethod' => 'applications/maniphest/conduit/ManiphestQueryStatusesConduitAPIMethod.php',
    'ManiphestRemarkupRule' => 'applications/maniphest/remarkup/ManiphestRemarkupRule.php',
    'ManiphestReplyHandler' => 'applications/maniphest/mail/ManiphestReplyHandler.php',
    'ManiphestReportController' => 'applications/maniphest/controller/ManiphestReportController.php',
    'ManiphestSchemaSpec' => 'applications/maniphest/storage/ManiphestSchemaSpec.php',
    'ManiphestSearchConduitAPIMethod' => 'applications/maniphest/conduit/ManiphestSearchConduitAPIMethod.php',
    'ManiphestStatusConfigOptionType' => 'applications/maniphest/config/ManiphestStatusConfigOptionType.php',
    'ManiphestStatusEmailCommand' => 'applications/maniphest/command/ManiphestStatusEmailCommand.php',
    'ManiphestSubpriorityController' => 'applications/maniphest/controller/ManiphestSubpriorityController.php',
    'ManiphestTask' => 'applications/maniphest/storage/ManiphestTask.php',
    'ManiphestTaskAssignHeraldAction' => 'applications/maniphest/herald/ManiphestTaskAssignHeraldAction.php',
    'ManiphestTaskAssignOtherHeraldAction' => 'applications/maniphest/herald/ManiphestTaskAssignOtherHeraldAction.php',
    'ManiphestTaskAssignSelfHeraldAction' => 'applications/maniphest/herald/ManiphestTaskAssignSelfHeraldAction.php',
    'ManiphestTaskAssigneeHeraldField' => 'applications/maniphest/herald/ManiphestTaskAssigneeHeraldField.php',
    'ManiphestTaskAuthorHeraldField' => 'applications/maniphest/herald/ManiphestTaskAuthorHeraldField.php',
    'ManiphestTaskAuthorPolicyRule' => 'applications/maniphest/policyrule/ManiphestTaskAuthorPolicyRule.php',
    'ManiphestTaskClosedStatusDatasource' => 'applications/maniphest/typeahead/ManiphestTaskClosedStatusDatasource.php',
    'ManiphestTaskDependedOnByTaskEdgeType' => 'applications/maniphest/edge/ManiphestTaskDependedOnByTaskEdgeType.php',
    'ManiphestTaskDependsOnTaskEdgeType' => 'applications/maniphest/edge/ManiphestTaskDependsOnTaskEdgeType.php',
    'ManiphestTaskDescriptionHeraldField' => 'applications/maniphest/herald/ManiphestTaskDescriptionHeraldField.php',
    'ManiphestTaskDetailController' => 'applications/maniphest/controller/ManiphestTaskDetailController.php',
    'ManiphestTaskEditBulkJobType' => 'applications/maniphest/bulk/ManiphestTaskEditBulkJobType.php',
    'ManiphestTaskEditController' => 'applications/maniphest/controller/ManiphestTaskEditController.php',
    'ManiphestTaskFulltextEngine' => 'applications/maniphest/search/ManiphestTaskFulltextEngine.php',
    'ManiphestTaskHasCommitEdgeType' => 'applications/maniphest/edge/ManiphestTaskHasCommitEdgeType.php',
    'ManiphestTaskHasMockEdgeType' => 'applications/maniphest/edge/ManiphestTaskHasMockEdgeType.php',
    'ManiphestTaskHasRevisionEdgeType' => 'applications/maniphest/edge/ManiphestTaskHasRevisionEdgeType.php',
    'ManiphestTaskHeraldField' => 'applications/maniphest/herald/ManiphestTaskHeraldField.php',
    'ManiphestTaskHeraldFieldGroup' => 'applications/maniphest/herald/ManiphestTaskHeraldFieldGroup.php',
    'ManiphestTaskListController' => 'applications/maniphest/controller/ManiphestTaskListController.php',
    'ManiphestTaskListHTTPParameterType' => 'applications/maniphest/httpparametertype/ManiphestTaskListHTTPParameterType.php',
    'ManiphestTaskListView' => 'applications/maniphest/view/ManiphestTaskListView.php',
    'ManiphestTaskMailReceiver' => 'applications/maniphest/mail/ManiphestTaskMailReceiver.php',
    'ManiphestTaskOpenStatusDatasource' => 'applications/maniphest/typeahead/ManiphestTaskOpenStatusDatasource.php',
    'ManiphestTaskPHIDResolver' => 'applications/maniphest/httpparametertype/ManiphestTaskPHIDResolver.php',
    'ManiphestTaskPHIDType' => 'applications/maniphest/phid/ManiphestTaskPHIDType.php',
    'ManiphestTaskPriority' => 'applications/maniphest/constants/ManiphestTaskPriority.php',
    'ManiphestTaskPriorityDatasource' => 'applications/maniphest/typeahead/ManiphestTaskPriorityDatasource.php',
    'ManiphestTaskPriorityHeraldAction' => 'applications/maniphest/herald/ManiphestTaskPriorityHeraldAction.php',
    'ManiphestTaskPriorityHeraldField' => 'applications/maniphest/herald/ManiphestTaskPriorityHeraldField.php',
    'ManiphestTaskQuery' => 'applications/maniphest/query/ManiphestTaskQuery.php',
    'ManiphestTaskResultListView' => 'applications/maniphest/view/ManiphestTaskResultListView.php',
    'ManiphestTaskSearchEngine' => 'applications/maniphest/query/ManiphestTaskSearchEngine.php',
    'ManiphestTaskStatus' => 'applications/maniphest/constants/ManiphestTaskStatus.php',
    'ManiphestTaskStatusDatasource' => 'applications/maniphest/typeahead/ManiphestTaskStatusDatasource.php',
    'ManiphestTaskStatusFunctionDatasource' => 'applications/maniphest/typeahead/ManiphestTaskStatusFunctionDatasource.php',
    'ManiphestTaskStatusHeraldAction' => 'applications/maniphest/herald/ManiphestTaskStatusHeraldAction.php',
    'ManiphestTaskStatusHeraldField' => 'applications/maniphest/herald/ManiphestTaskStatusHeraldField.php',
    'ManiphestTaskStatusTestCase' => 'applications/maniphest/constants/__tests__/ManiphestTaskStatusTestCase.php',
    'ManiphestTaskTestCase' => 'applications/maniphest/__tests__/ManiphestTaskTestCase.php',
    'ManiphestTaskTitleHeraldField' => 'applications/maniphest/herald/ManiphestTaskTitleHeraldField.php',
    'ManiphestTransaction' => 'applications/maniphest/storage/ManiphestTransaction.php',
    'ManiphestTransactionComment' => 'applications/maniphest/storage/ManiphestTransactionComment.php',
    'ManiphestTransactionEditor' => 'applications/maniphest/editor/ManiphestTransactionEditor.php',
    'ManiphestTransactionQuery' => 'applications/maniphest/query/ManiphestTransactionQuery.php',
    'ManiphestUpdateConduitAPIMethod' => 'applications/maniphest/conduit/ManiphestUpdateConduitAPIMethod.php',
    'ManiphestView' => 'applications/maniphest/view/ManiphestView.php',
    'MetaMTAEmailTransactionCommand' => 'applications/metamta/command/MetaMTAEmailTransactionCommand.php',
    'MetaMTAEmailTransactionCommandTestCase' => 'applications/metamta/command/__tests__/MetaMTAEmailTransactionCommandTestCase.php',
    'MetaMTAMailReceivedGarbageCollector' => 'applications/metamta/garbagecollector/MetaMTAMailReceivedGarbageCollector.php',
    'MetaMTAMailSentGarbageCollector' => 'applications/metamta/garbagecollector/MetaMTAMailSentGarbageCollector.php',
    'MetaMTAReceivedMailStatus' => 'applications/metamta/constants/MetaMTAReceivedMailStatus.php',
    'MultimeterContext' => 'applications/multimeter/storage/MultimeterContext.php',
    'MultimeterControl' => 'applications/multimeter/data/MultimeterControl.php',
    'MultimeterController' => 'applications/multimeter/controller/MultimeterController.php',
    'MultimeterDAO' => 'applications/multimeter/storage/MultimeterDAO.php',
    'MultimeterDimension' => 'applications/multimeter/storage/MultimeterDimension.php',
    'MultimeterEvent' => 'applications/multimeter/storage/MultimeterEvent.php',
    'MultimeterEventGarbageCollector' => 'applications/multimeter/garbagecollector/MultimeterEventGarbageCollector.php',
    'MultimeterHost' => 'applications/multimeter/storage/MultimeterHost.php',
    'MultimeterLabel' => 'applications/multimeter/storage/MultimeterLabel.php',
    'MultimeterSampleController' => 'applications/multimeter/controller/MultimeterSampleController.php',
    'MultimeterViewer' => 'applications/multimeter/storage/MultimeterViewer.php',
    'NuanceConduitAPIMethod' => 'applications/nuance/conduit/NuanceConduitAPIMethod.php',
    'NuanceConsoleController' => 'applications/nuance/controller/NuanceConsoleController.php',
    'NuanceController' => 'applications/nuance/controller/NuanceController.php',
    'NuanceCreateItemConduitAPIMethod' => 'applications/nuance/conduit/NuanceCreateItemConduitAPIMethod.php',
    'NuanceDAO' => 'applications/nuance/storage/NuanceDAO.php',
    'NuanceItem' => 'applications/nuance/storage/NuanceItem.php',
    'NuanceItemEditController' => 'applications/nuance/controller/NuanceItemEditController.php',
    'NuanceItemEditor' => 'applications/nuance/editor/NuanceItemEditor.php',
    'NuanceItemPHIDType' => 'applications/nuance/phid/NuanceItemPHIDType.php',
    'NuanceItemQuery' => 'applications/nuance/query/NuanceItemQuery.php',
    'NuanceItemTransaction' => 'applications/nuance/storage/NuanceItemTransaction.php',
    'NuanceItemTransactionComment' => 'applications/nuance/storage/NuanceItemTransactionComment.php',
    'NuanceItemTransactionQuery' => 'applications/nuance/query/NuanceItemTransactionQuery.php',
    'NuanceItemViewController' => 'applications/nuance/controller/NuanceItemViewController.php',
    'NuancePhabricatorFormSourceDefinition' => 'applications/nuance/source/NuancePhabricatorFormSourceDefinition.php',
    'NuanceQuery' => 'applications/nuance/query/NuanceQuery.php',
    'NuanceQueue' => 'applications/nuance/storage/NuanceQueue.php',
    'NuanceQueueDatasource' => 'applications/nuance/typeahead/NuanceQueueDatasource.php',
    'NuanceQueueEditController' => 'applications/nuance/controller/NuanceQueueEditController.php',
    'NuanceQueueEditor' => 'applications/nuance/editor/NuanceQueueEditor.php',
    'NuanceQueueListController' => 'applications/nuance/controller/NuanceQueueListController.php',
    'NuanceQueuePHIDType' => 'applications/nuance/phid/NuanceQueuePHIDType.php',
    'NuanceQueueQuery' => 'applications/nuance/query/NuanceQueueQuery.php',
    'NuanceQueueSearchEngine' => 'applications/nuance/query/NuanceQueueSearchEngine.php',
    'NuanceQueueTransaction' => 'applications/nuance/storage/NuanceQueueTransaction.php',
    'NuanceQueueTransactionComment' => 'applications/nuance/storage/NuanceQueueTransactionComment.php',
    'NuanceQueueTransactionQuery' => 'applications/nuance/query/NuanceQueueTransactionQuery.php',
    'NuanceQueueViewController' => 'applications/nuance/controller/NuanceQueueViewController.php',
    'NuanceRequestor' => 'applications/nuance/storage/NuanceRequestor.php',
    'NuanceRequestorEditController' => 'applications/nuance/controller/NuanceRequestorEditController.php',
    'NuanceRequestorEditor' => 'applications/nuance/editor/NuanceRequestorEditor.php',
    'NuanceRequestorPHIDType' => 'applications/nuance/phid/NuanceRequestorPHIDType.php',
    'NuanceRequestorQuery' => 'applications/nuance/query/NuanceRequestorQuery.php',
    'NuanceRequestorSource' => 'applications/nuance/storage/NuanceRequestorSource.php',
    'NuanceRequestorTransaction' => 'applications/nuance/storage/NuanceRequestorTransaction.php',
    'NuanceRequestorTransactionComment' => 'applications/nuance/storage/NuanceRequestorTransactionComment.php',
    'NuanceRequestorTransactionQuery' => 'applications/nuance/query/NuanceRequestorTransactionQuery.php',
    'NuanceRequestorViewController' => 'applications/nuance/controller/NuanceRequestorViewController.php',
    'NuanceSchemaSpec' => 'applications/nuance/storage/NuanceSchemaSpec.php',
    'NuanceSource' => 'applications/nuance/storage/NuanceSource.php',
    'NuanceSourceActionController' => 'applications/nuance/controller/NuanceSourceActionController.php',
    'NuanceSourceCreateController' => 'applications/nuance/controller/NuanceSourceCreateController.php',
    'NuanceSourceDefaultEditCapability' => 'applications/nuance/capability/NuanceSourceDefaultEditCapability.php',
    'NuanceSourceDefaultViewCapability' => 'applications/nuance/capability/NuanceSourceDefaultViewCapability.php',
    'NuanceSourceDefinition' => 'applications/nuance/source/NuanceSourceDefinition.php',
    'NuanceSourceDefinitionTestCase' => 'applications/nuance/source/__tests__/NuanceSourceDefinitionTestCase.php',
    'NuanceSourceEditController' => 'applications/nuance/controller/NuanceSourceEditController.php',
    'NuanceSourceEditor' => 'applications/nuance/editor/NuanceSourceEditor.php',
    'NuanceSourceListController' => 'applications/nuance/controller/NuanceSourceListController.php',
    'NuanceSourceManageCapability' => 'applications/nuance/capability/NuanceSourceManageCapability.php',
    'NuanceSourcePHIDType' => 'applications/nuance/phid/NuanceSourcePHIDType.php',
    'NuanceSourceQuery' => 'applications/nuance/query/NuanceSourceQuery.php',
    'NuanceSourceSearchEngine' => 'applications/nuance/query/NuanceSourceSearchEngine.php',
    'NuanceSourceTransaction' => 'applications/nuance/storage/NuanceSourceTransaction.php',
    'NuanceSourceTransactionComment' => 'applications/nuance/storage/NuanceSourceTransactionComment.php',
    'NuanceSourceTransactionQuery' => 'applications/nuance/query/NuanceSourceTransactionQuery.php',
    'NuanceSourceViewController' => 'applications/nuance/controller/NuanceSourceViewController.php',
    'NuanceTransaction' => 'applications/nuance/storage/NuanceTransaction.php',
    'OwnersConduitAPIMethod' => 'applications/owners/conduit/OwnersConduitAPIMethod.php',
    'OwnersEditConduitAPIMethod' => 'applications/owners/conduit/OwnersEditConduitAPIMethod.php',
    'OwnersPackageReplyHandler' => 'applications/owners/mail/OwnersPackageReplyHandler.php',
    'OwnersQueryConduitAPIMethod' => 'applications/owners/conduit/OwnersQueryConduitAPIMethod.php',
    'OwnersSearchConduitAPIMethod' => 'applications/owners/conduit/OwnersSearchConduitAPIMethod.php',
    'PHIDConduitAPIMethod' => 'applications/phid/conduit/PHIDConduitAPIMethod.php',
    'PHIDInfoConduitAPIMethod' => 'applications/phid/conduit/PHIDInfoConduitAPIMethod.php',
    'PHIDLookupConduitAPIMethod' => 'applications/phid/conduit/PHIDLookupConduitAPIMethod.php',
    'PHIDQueryConduitAPIMethod' => 'applications/phid/conduit/PHIDQueryConduitAPIMethod.php',
    'PHUI' => 'view/phui/PHUI.php',
    'PHUIActionPanelExample' => 'applications/uiexample/examples/PHUIActionPanelExample.php',
    'PHUIActionPanelView' => 'view/phui/PHUIActionPanelView.php',
    'PHUIApplicationMenuView' => 'view/layout/PHUIApplicationMenuView.php',
    'PHUIBadgeBoxView' => 'view/phui/PHUIBadgeBoxView.php',
    'PHUIBadgeExample' => 'applications/uiexample/examples/PHUIBadgeExample.php',
    'PHUIBadgeMiniView' => 'view/phui/PHUIBadgeMiniView.php',
    'PHUIBadgeView' => 'view/phui/PHUIBadgeView.php',
    'PHUIBigInfoView' => 'view/phui/PHUIBigInfoView.php',
    'PHUIBoxExample' => 'applications/uiexample/examples/PHUIBoxExample.php',
    'PHUIBoxView' => 'view/phui/PHUIBoxView.php',
    'PHUIButtonBarExample' => 'applications/uiexample/examples/PHUIButtonBarExample.php',
    'PHUIButtonBarView' => 'view/phui/PHUIButtonBarView.php',
    'PHUIButtonExample' => 'applications/uiexample/examples/PHUIButtonExample.php',
    'PHUIButtonView' => 'view/phui/PHUIButtonView.php',
    'PHUICalendarDayView' => 'view/phui/calendar/PHUICalendarDayView.php',
    'PHUICalendarListView' => 'view/phui/calendar/PHUICalendarListView.php',
    'PHUICalendarMonthView' => 'view/phui/calendar/PHUICalendarMonthView.php',
    'PHUICalendarWidgetView' => 'view/phui/calendar/PHUICalendarWidgetView.php',
    'PHUIColorPalletteExample' => 'applications/uiexample/examples/PHUIColorPalletteExample.php',
    'PHUICrumbView' => 'view/phui/PHUICrumbView.php',
    'PHUICrumbsView' => 'view/phui/PHUICrumbsView.php',
    'PHUIDiffInlineCommentDetailView' => 'infrastructure/diff/view/PHUIDiffInlineCommentDetailView.php',
    'PHUIDiffInlineCommentEditView' => 'infrastructure/diff/view/PHUIDiffInlineCommentEditView.php',
    'PHUIDiffInlineCommentRowScaffold' => 'infrastructure/diff/view/PHUIDiffInlineCommentRowScaffold.php',
    'PHUIDiffInlineCommentTableScaffold' => 'infrastructure/diff/view/PHUIDiffInlineCommentTableScaffold.php',
    'PHUIDiffInlineCommentUndoView' => 'infrastructure/diff/view/PHUIDiffInlineCommentUndoView.php',
    'PHUIDiffInlineCommentView' => 'infrastructure/diff/view/PHUIDiffInlineCommentView.php',
    'PHUIDiffOneUpInlineCommentRowScaffold' => 'infrastructure/diff/view/PHUIDiffOneUpInlineCommentRowScaffold.php',
    'PHUIDiffRevealIconView' => 'infrastructure/diff/view/PHUIDiffRevealIconView.php',
    'PHUIDiffTableOfContentsItemView' => 'infrastructure/diff/view/PHUIDiffTableOfContentsItemView.php',
    'PHUIDiffTableOfContentsListView' => 'infrastructure/diff/view/PHUIDiffTableOfContentsListView.php',
    'PHUIDiffTwoUpInlineCommentRowScaffold' => 'infrastructure/diff/view/PHUIDiffTwoUpInlineCommentRowScaffold.php',
    'PHUIDocumentExample' => 'applications/uiexample/examples/PHUIDocumentExample.php',
    'PHUIDocumentSummaryView' => 'view/phui/PHUIDocumentSummaryView.php',
    'PHUIDocumentView' => 'view/phui/PHUIDocumentView.php',
    'PHUIDocumentViewPro' => 'view/phui/PHUIDocumentViewPro.php',
    'PHUIFeedStoryExample' => 'applications/uiexample/examples/PHUIFeedStoryExample.php',
    'PHUIFeedStoryView' => 'view/phui/PHUIFeedStoryView.php',
    'PHUIFormDividerControl' => 'view/form/control/PHUIFormDividerControl.php',
    'PHUIFormFreeformDateControl' => 'view/form/control/PHUIFormFreeformDateControl.php',
    'PHUIFormIconSetControl' => 'view/form/control/PHUIFormIconSetControl.php',
    'PHUIFormInsetView' => 'view/form/PHUIFormInsetView.php',
    'PHUIFormLayoutView' => 'view/form/PHUIFormLayoutView.php',
    'PHUIFormMultiSubmitControl' => 'view/form/control/PHUIFormMultiSubmitControl.php',
    'PHUIFormPageView' => 'view/form/PHUIFormPageView.php',
    'PHUIHandleListView' => 'applications/phid/view/PHUIHandleListView.php',
    'PHUIHandleTagListView' => 'applications/phid/view/PHUIHandleTagListView.php',
    'PHUIHandleView' => 'applications/phid/view/PHUIHandleView.php',
    'PHUIHeaderView' => 'view/phui/PHUIHeaderView.php',
    'PHUIIconCircleView' => 'view/phui/PHUIIconCircleView.php',
    'PHUIIconExample' => 'applications/uiexample/examples/PHUIIconExample.php',
    'PHUIIconView' => 'view/phui/PHUIIconView.php',
    'PHUIImageMaskExample' => 'applications/uiexample/examples/PHUIImageMaskExample.php',
    'PHUIImageMaskView' => 'view/phui/PHUIImageMaskView.php',
    'PHUIInfoExample' => 'applications/uiexample/examples/PHUIInfoExample.php',
    'PHUIInfoPanelExample' => 'applications/uiexample/examples/PHUIInfoPanelExample.php',
    'PHUIInfoPanelView' => 'view/phui/PHUIInfoPanelView.php',
    'PHUIInfoView' => 'view/form/PHUIInfoView.php',
    'PHUIListExample' => 'applications/uiexample/examples/PHUIListExample.php',
    'PHUIListItemView' => 'view/phui/PHUIListItemView.php',
    'PHUIListView' => 'view/phui/PHUIListView.php',
    'PHUIListViewTestCase' => 'view/layout/__tests__/PHUIListViewTestCase.php',
    'PHUIMainMenuView' => 'view/phui/PHUIMainMenuView.php',
    'PHUIObjectBoxView' => 'view/phui/PHUIObjectBoxView.php',
    'PHUIObjectItemListExample' => 'applications/uiexample/examples/PHUIObjectItemListExample.php',
    'PHUIObjectItemListView' => 'view/phui/PHUIObjectItemListView.php',
    'PHUIObjectItemView' => 'view/phui/PHUIObjectItemView.php',
    'PHUIPagedFormView' => 'view/form/PHUIPagedFormView.php',
    'PHUIPagerView' => 'view/phui/PHUIPagerView.php',
    'PHUIPinboardItemView' => 'view/phui/PHUIPinboardItemView.php',
    'PHUIPinboardView' => 'view/phui/PHUIPinboardView.php',
    'PHUIPropertyGroupView' => 'view/phui/PHUIPropertyGroupView.php',
    'PHUIPropertyListExample' => 'applications/uiexample/examples/PHUIPropertyListExample.php',
    'PHUIPropertyListView' => 'view/phui/PHUIPropertyListView.php',
    'PHUIRemarkupPreviewPanel' => 'view/phui/PHUIRemarkupPreviewPanel.php',
    'PHUIRemarkupView' => 'infrastructure/markup/view/PHUIRemarkupView.php',
    'PHUISpacesNamespaceContextView' => 'applications/spaces/view/PHUISpacesNamespaceContextView.php',
    'PHUIStatusItemView' => 'view/phui/PHUIStatusItemView.php',
    'PHUIStatusListView' => 'view/phui/PHUIStatusListView.php',
    'PHUITagExample' => 'applications/uiexample/examples/PHUITagExample.php',
    'PHUITagView' => 'view/phui/PHUITagView.php',
    'PHUITextExample' => 'applications/uiexample/examples/PHUITextExample.php',
    'PHUITextView' => 'view/phui/PHUITextView.php',
    'PHUITimelineEventView' => 'view/phui/PHUITimelineEventView.php',
    'PHUITimelineExample' => 'applications/uiexample/examples/PHUITimelineExample.php',
    'PHUITimelineView' => 'view/phui/PHUITimelineView.php',
    'PHUITwoColumnView' => 'view/phui/PHUITwoColumnView.php',
    'PHUITypeaheadExample' => 'applications/uiexample/examples/PHUITypeaheadExample.php',
    'PHUIWorkboardView' => 'view/phui/PHUIWorkboardView.php',
    'PHUIWorkpanelView' => 'view/phui/PHUIWorkpanelView.php',
    'PassphraseAbstractKey' => 'applications/passphrase/keys/PassphraseAbstractKey.php',
    'PassphraseConduitAPIMethod' => 'applications/passphrase/conduit/PassphraseConduitAPIMethod.php',
    'PassphraseController' => 'applications/passphrase/controller/PassphraseController.php',
    'PassphraseCredential' => 'applications/passphrase/storage/PassphraseCredential.php',
    'PassphraseCredentialAuthorPolicyRule' => 'applications/passphrase/policyrule/PassphraseCredentialAuthorPolicyRule.php',
    'PassphraseCredentialConduitController' => 'applications/passphrase/controller/PassphraseCredentialConduitController.php',
    'PassphraseCredentialControl' => 'applications/passphrase/view/PassphraseCredentialControl.php',
    'PassphraseCredentialCreateController' => 'applications/passphrase/controller/PassphraseCredentialCreateController.php',
    'PassphraseCredentialDestroyController' => 'applications/passphrase/controller/PassphraseCredentialDestroyController.php',
    'PassphraseCredentialEditController' => 'applications/passphrase/controller/PassphraseCredentialEditController.php',
    'PassphraseCredentialFulltextEngine' => 'applications/passphrase/search/PassphraseCredentialFulltextEngine.php',
    'PassphraseCredentialListController' => 'applications/passphrase/controller/PassphraseCredentialListController.php',
    'PassphraseCredentialLockController' => 'applications/passphrase/controller/PassphraseCredentialLockController.php',
    'PassphraseCredentialPHIDType' => 'applications/passphrase/phid/PassphraseCredentialPHIDType.php',
    'PassphraseCredentialPublicController' => 'applications/passphrase/controller/PassphraseCredentialPublicController.php',
    'PassphraseCredentialQuery' => 'applications/passphrase/query/PassphraseCredentialQuery.php',
    'PassphraseCredentialRevealController' => 'applications/passphrase/controller/PassphraseCredentialRevealController.php',
    'PassphraseCredentialSearchEngine' => 'applications/passphrase/query/PassphraseCredentialSearchEngine.php',
    'PassphraseCredentialTransaction' => 'applications/passphrase/storage/PassphraseCredentialTransaction.php',
    'PassphraseCredentialTransactionEditor' => 'applications/passphrase/editor/PassphraseCredentialTransactionEditor.php',
    'PassphraseCredentialTransactionQuery' => 'applications/passphrase/query/PassphraseCredentialTransactionQuery.php',
    'PassphraseCredentialType' => 'applications/passphrase/credentialtype/PassphraseCredentialType.php',
    'PassphraseCredentialTypeTestCase' => 'applications/passphrase/credentialtype/__tests__/PassphraseCredentialTypeTestCase.php',
    'PassphraseCredentialViewController' => 'applications/passphrase/controller/PassphraseCredentialViewController.php',
    'PassphraseDAO' => 'applications/passphrase/storage/PassphraseDAO.php',
    'PassphraseDefaultEditCapability' => 'applications/passphrase/capability/PassphraseDefaultEditCapability.php',
    'PassphraseDefaultViewCapability' => 'applications/passphrase/capability/PassphraseDefaultViewCapability.php',
    'PassphraseNoteCredentialType' => 'applications/passphrase/credentialtype/PassphraseNoteCredentialType.php',
    'PassphrasePasswordCredentialType' => 'applications/passphrase/credentialtype/PassphrasePasswordCredentialType.php',
    'PassphrasePasswordKey' => 'applications/passphrase/keys/PassphrasePasswordKey.php',
    'PassphraseQueryConduitAPIMethod' => 'applications/passphrase/conduit/PassphraseQueryConduitAPIMethod.php',
    'PassphraseRemarkupRule' => 'applications/passphrase/remarkup/PassphraseRemarkupRule.php',
    'PassphraseSSHGeneratedKeyCredentialType' => 'applications/passphrase/credentialtype/PassphraseSSHGeneratedKeyCredentialType.php',
    'PassphraseSSHKey' => 'applications/passphrase/keys/PassphraseSSHKey.php',
    'PassphraseSSHPrivateKeyCredentialType' => 'applications/passphrase/credentialtype/PassphraseSSHPrivateKeyCredentialType.php',
    'PassphraseSSHPrivateKeyFileCredentialType' => 'applications/passphrase/credentialtype/PassphraseSSHPrivateKeyFileCredentialType.php',
    'PassphraseSSHPrivateKeyTextCredentialType' => 'applications/passphrase/credentialtype/PassphraseSSHPrivateKeyTextCredentialType.php',
    'PassphraseSchemaSpec' => 'applications/passphrase/storage/PassphraseSchemaSpec.php',
    'PassphraseSecret' => 'applications/passphrase/storage/PassphraseSecret.php',
    'PasteConduitAPIMethod' => 'applications/paste/conduit/PasteConduitAPIMethod.php',
    'PasteCreateConduitAPIMethod' => 'applications/paste/conduit/PasteCreateConduitAPIMethod.php',
    'PasteCreateMailReceiver' => 'applications/paste/mail/PasteCreateMailReceiver.php',
    'PasteDefaultEditCapability' => 'applications/paste/capability/PasteDefaultEditCapability.php',
    'PasteDefaultViewCapability' => 'applications/paste/capability/PasteDefaultViewCapability.php',
    'PasteEditConduitAPIMethod' => 'applications/paste/conduit/PasteEditConduitAPIMethod.php',
    'PasteEmbedView' => 'applications/paste/view/PasteEmbedView.php',
    'PasteInfoConduitAPIMethod' => 'applications/paste/conduit/PasteInfoConduitAPIMethod.php',
    'PasteMailReceiver' => 'applications/paste/mail/PasteMailReceiver.php',
    'PasteQueryConduitAPIMethod' => 'applications/paste/conduit/PasteQueryConduitAPIMethod.php',
    'PasteReplyHandler' => 'applications/paste/mail/PasteReplyHandler.php',
    'PasteSearchConduitAPIMethod' => 'applications/paste/conduit/PasteSearchConduitAPIMethod.php',
    'PeopleBrowseUserDirectoryCapability' => 'applications/people/capability/PeopleBrowseUserDirectoryCapability.php',
    'PeopleCreateUsersCapability' => 'applications/people/capability/PeopleCreateUsersCapability.php',
    'PeopleUserLogGarbageCollector' => 'applications/people/garbagecollector/PeopleUserLogGarbageCollector.php',
    'Phabricator404Controller' => 'applications/base/controller/Phabricator404Controller.php',
    'PhabricatorAWSConfigOptions' => 'applications/config/option/PhabricatorAWSConfigOptions.php',
    'PhabricatorAccessControlTestCase' => 'applications/base/controller/__tests__/PhabricatorAccessControlTestCase.php',
    'PhabricatorAccessLog' => 'infrastructure/log/PhabricatorAccessLog.php',
    'PhabricatorAccessLogConfigOptions' => 'applications/config/option/PhabricatorAccessLogConfigOptions.php',
    'PhabricatorAccountSettingsPanel' => 'applications/settings/panel/PhabricatorAccountSettingsPanel.php',
    'PhabricatorActionListView' => 'view/layout/PhabricatorActionListView.php',
    'PhabricatorActionView' => 'view/layout/PhabricatorActionView.php',
    'PhabricatorActivitySettingsPanel' => 'applications/settings/panel/PhabricatorActivitySettingsPanel.php',
    'PhabricatorAdministratorsPolicyRule' => 'applications/people/policyrule/PhabricatorAdministratorsPolicyRule.php',
    'PhabricatorAjaxRequestExceptionHandler' => 'aphront/handler/PhabricatorAjaxRequestExceptionHandler.php',
    'PhabricatorAlmanacApplication' => 'applications/almanac/application/PhabricatorAlmanacApplication.php',
    'PhabricatorAmazonAuthProvider' => 'applications/auth/provider/PhabricatorAmazonAuthProvider.php',
    'PhabricatorAnchorView' => 'view/layout/PhabricatorAnchorView.php',
    'PhabricatorAphlictManagementDebugWorkflow' => 'applications/aphlict/management/PhabricatorAphlictManagementDebugWorkflow.php',
    'PhabricatorAphlictManagementRestartWorkflow' => 'applications/aphlict/management/PhabricatorAphlictManagementRestartWorkflow.php',
    'PhabricatorAphlictManagementStartWorkflow' => 'applications/aphlict/management/PhabricatorAphlictManagementStartWorkflow.php',
    'PhabricatorAphlictManagementStatusWorkflow' => 'applications/aphlict/management/PhabricatorAphlictManagementStatusWorkflow.php',
    'PhabricatorAphlictManagementStopWorkflow' => 'applications/aphlict/management/PhabricatorAphlictManagementStopWorkflow.php',
    'PhabricatorAphlictManagementWorkflow' => 'applications/aphlict/management/PhabricatorAphlictManagementWorkflow.php',
    'PhabricatorAphlictSetupCheck' => 'applications/notification/setup/PhabricatorAphlictSetupCheck.php',
    'PhabricatorAphrontBarUIExample' => 'applications/uiexample/examples/PhabricatorAphrontBarUIExample.php',
    'PhabricatorAphrontViewTestCase' => 'view/__tests__/PhabricatorAphrontViewTestCase.php',
    'PhabricatorAppSearchEngine' => 'applications/meta/query/PhabricatorAppSearchEngine.php',
    'PhabricatorApplication' => 'applications/base/PhabricatorApplication.php',
    'PhabricatorApplicationApplicationPHIDType' => 'applications/meta/phid/PhabricatorApplicationApplicationPHIDType.php',
    'PhabricatorApplicationConfigOptions' => 'applications/config/option/PhabricatorApplicationConfigOptions.php',
    'PhabricatorApplicationConfigurationPanel' => 'applications/meta/panel/PhabricatorApplicationConfigurationPanel.php',
    'PhabricatorApplicationConfigurationPanelTestCase' => 'applications/meta/panel/__tests__/PhabricatorApplicationConfigurationPanelTestCase.php',
    'PhabricatorApplicationDatasource' => 'applications/meta/typeahead/PhabricatorApplicationDatasource.php',
    'PhabricatorApplicationDetailViewController' => 'applications/meta/controller/PhabricatorApplicationDetailViewController.php',
    'PhabricatorApplicationEditController' => 'applications/meta/controller/PhabricatorApplicationEditController.php',
    'PhabricatorApplicationEditHTTPParameterHelpView' => 'applications/transactions/view/PhabricatorApplicationEditHTTPParameterHelpView.php',
    'PhabricatorApplicationEmailCommandsController' => 'applications/meta/controller/PhabricatorApplicationEmailCommandsController.php',
    'PhabricatorApplicationLaunchView' => 'applications/meta/view/PhabricatorApplicationLaunchView.php',
    'PhabricatorApplicationPanelController' => 'applications/meta/controller/PhabricatorApplicationPanelController.php',
    'PhabricatorApplicationQuery' => 'applications/meta/query/PhabricatorApplicationQuery.php',
    'PhabricatorApplicationSearchController' => 'applications/search/controller/PhabricatorApplicationSearchController.php',
    'PhabricatorApplicationSearchEngine' => 'applications/search/engine/PhabricatorApplicationSearchEngine.php',
    'PhabricatorApplicationSearchEngineTestCase' => 'applications/search/engine/__tests__/PhabricatorApplicationSearchEngineTestCase.php',
    'PhabricatorApplicationSearchResultView' => 'applications/search/view/PhabricatorApplicationSearchResultView.php',
    'PhabricatorApplicationStatusView' => 'applications/meta/view/PhabricatorApplicationStatusView.php',
    'PhabricatorApplicationTestCase' => 'applications/base/__tests__/PhabricatorApplicationTestCase.php',
    'PhabricatorApplicationTransaction' => 'applications/transactions/storage/PhabricatorApplicationTransaction.php',
    'PhabricatorApplicationTransactionComment' => 'applications/transactions/storage/PhabricatorApplicationTransactionComment.php',
    'PhabricatorApplicationTransactionCommentEditController' => 'applications/transactions/controller/PhabricatorApplicationTransactionCommentEditController.php',
    'PhabricatorApplicationTransactionCommentEditor' => 'applications/transactions/editor/PhabricatorApplicationTransactionCommentEditor.php',
    'PhabricatorApplicationTransactionCommentHistoryController' => 'applications/transactions/controller/PhabricatorApplicationTransactionCommentHistoryController.php',
    'PhabricatorApplicationTransactionCommentQuery' => 'applications/transactions/query/PhabricatorApplicationTransactionCommentQuery.php',
    'PhabricatorApplicationTransactionCommentQuoteController' => 'applications/transactions/controller/PhabricatorApplicationTransactionCommentQuoteController.php',
    'PhabricatorApplicationTransactionCommentRawController' => 'applications/transactions/controller/PhabricatorApplicationTransactionCommentRawController.php',
    'PhabricatorApplicationTransactionCommentRemoveController' => 'applications/transactions/controller/PhabricatorApplicationTransactionCommentRemoveController.php',
    'PhabricatorApplicationTransactionCommentView' => 'applications/transactions/view/PhabricatorApplicationTransactionCommentView.php',
    'PhabricatorApplicationTransactionController' => 'applications/transactions/controller/PhabricatorApplicationTransactionController.php',
    'PhabricatorApplicationTransactionDetailController' => 'applications/transactions/controller/PhabricatorApplicationTransactionDetailController.php',
    'PhabricatorApplicationTransactionEditor' => 'applications/transactions/editor/PhabricatorApplicationTransactionEditor.php',
    'PhabricatorApplicationTransactionFeedStory' => 'applications/transactions/feed/PhabricatorApplicationTransactionFeedStory.php',
    'PhabricatorApplicationTransactionInterface' => 'applications/transactions/interface/PhabricatorApplicationTransactionInterface.php',
    'PhabricatorApplicationTransactionNoEffectException' => 'applications/transactions/exception/PhabricatorApplicationTransactionNoEffectException.php',
    'PhabricatorApplicationTransactionNoEffectResponse' => 'applications/transactions/response/PhabricatorApplicationTransactionNoEffectResponse.php',
    'PhabricatorApplicationTransactionPublishWorker' => 'applications/transactions/worker/PhabricatorApplicationTransactionPublishWorker.php',
    'PhabricatorApplicationTransactionQuery' => 'applications/transactions/query/PhabricatorApplicationTransactionQuery.php',
    'PhabricatorApplicationTransactionRemarkupPreviewController' => 'applications/transactions/controller/PhabricatorApplicationTransactionRemarkupPreviewController.php',
    'PhabricatorApplicationTransactionReplyHandler' => 'applications/transactions/replyhandler/PhabricatorApplicationTransactionReplyHandler.php',
    'PhabricatorApplicationTransactionResponse' => 'applications/transactions/response/PhabricatorApplicationTransactionResponse.php',
    'PhabricatorApplicationTransactionShowOlderController' => 'applications/transactions/controller/PhabricatorApplicationTransactionShowOlderController.php',
    'PhabricatorApplicationTransactionStructureException' => 'applications/transactions/exception/PhabricatorApplicationTransactionStructureException.php',
    'PhabricatorApplicationTransactionTemplatedCommentQuery' => 'applications/transactions/query/PhabricatorApplicationTransactionTemplatedCommentQuery.php',
    'PhabricatorApplicationTransactionTextDiffDetailView' => 'applications/transactions/view/PhabricatorApplicationTransactionTextDiffDetailView.php',
    'PhabricatorApplicationTransactionTransactionPHIDType' => 'applications/transactions/phid/PhabricatorApplicationTransactionTransactionPHIDType.php',
    'PhabricatorApplicationTransactionValidationError' => 'applications/transactions/error/PhabricatorApplicationTransactionValidationError.php',
    'PhabricatorApplicationTransactionValidationException' => 'applications/transactions/exception/PhabricatorApplicationTransactionValidationException.php',
    'PhabricatorApplicationTransactionValidationResponse' => 'applications/transactions/response/PhabricatorApplicationTransactionValidationResponse.php',
    'PhabricatorApplicationTransactionValueController' => 'applications/transactions/controller/PhabricatorApplicationTransactionValueController.php',
    'PhabricatorApplicationTransactionView' => 'applications/transactions/view/PhabricatorApplicationTransactionView.php',
    'PhabricatorApplicationUninstallController' => 'applications/meta/controller/PhabricatorApplicationUninstallController.php',
    'PhabricatorApplicationsApplication' => 'applications/meta/application/PhabricatorApplicationsApplication.php',
    'PhabricatorApplicationsController' => 'applications/meta/controller/PhabricatorApplicationsController.php',
    'PhabricatorApplicationsListController' => 'applications/meta/controller/PhabricatorApplicationsListController.php',
    'PhabricatorAsanaAuthProvider' => 'applications/auth/provider/PhabricatorAsanaAuthProvider.php',
    'PhabricatorAsanaConfigOptions' => 'applications/doorkeeper/option/PhabricatorAsanaConfigOptions.php',
    'PhabricatorAsanaSubtaskHasObjectEdgeType' => 'applications/doorkeeper/edge/PhabricatorAsanaSubtaskHasObjectEdgeType.php',
    'PhabricatorAsanaTaskHasObjectEdgeType' => 'applications/doorkeeper/edge/PhabricatorAsanaTaskHasObjectEdgeType.php',
    'PhabricatorAuditActionConstants' => 'applications/audit/constants/PhabricatorAuditActionConstants.php',
    'PhabricatorAuditAddCommentController' => 'applications/audit/controller/PhabricatorAuditAddCommentController.php',
    'PhabricatorAuditApplication' => 'applications/audit/application/PhabricatorAuditApplication.php',
    'PhabricatorAuditCommentEditor' => 'applications/audit/editor/PhabricatorAuditCommentEditor.php',
    'PhabricatorAuditCommitStatusConstants' => 'applications/audit/constants/PhabricatorAuditCommitStatusConstants.php',
    'PhabricatorAuditController' => 'applications/audit/controller/PhabricatorAuditController.php',
    'PhabricatorAuditEditor' => 'applications/audit/editor/PhabricatorAuditEditor.php',
    'PhabricatorAuditInlineComment' => 'applications/audit/storage/PhabricatorAuditInlineComment.php',
    'PhabricatorAuditListController' => 'applications/audit/controller/PhabricatorAuditListController.php',
    'PhabricatorAuditListView' => 'applications/audit/view/PhabricatorAuditListView.php',
    'PhabricatorAuditMailReceiver' => 'applications/audit/mail/PhabricatorAuditMailReceiver.php',
    'PhabricatorAuditManagementDeleteWorkflow' => 'applications/audit/management/PhabricatorAuditManagementDeleteWorkflow.php',
    'PhabricatorAuditManagementWorkflow' => 'applications/audit/management/PhabricatorAuditManagementWorkflow.php',
    'PhabricatorAuditPreviewController' => 'applications/audit/controller/PhabricatorAuditPreviewController.php',
    'PhabricatorAuditReplyHandler' => 'applications/audit/mail/PhabricatorAuditReplyHandler.php',
    'PhabricatorAuditStatusConstants' => 'applications/audit/constants/PhabricatorAuditStatusConstants.php',
    'PhabricatorAuditTransaction' => 'applications/audit/storage/PhabricatorAuditTransaction.php',
    'PhabricatorAuditTransactionComment' => 'applications/audit/storage/PhabricatorAuditTransactionComment.php',
    'PhabricatorAuditTransactionQuery' => 'applications/audit/query/PhabricatorAuditTransactionQuery.php',
    'PhabricatorAuditTransactionView' => 'applications/audit/view/PhabricatorAuditTransactionView.php',
    'PhabricatorAuthAccountView' => 'applications/auth/view/PhabricatorAuthAccountView.php',
    'PhabricatorAuthApplication' => 'applications/auth/application/PhabricatorAuthApplication.php',
    'PhabricatorAuthAuthFactorPHIDType' => 'applications/auth/phid/PhabricatorAuthAuthFactorPHIDType.php',
    'PhabricatorAuthAuthProviderPHIDType' => 'applications/auth/phid/PhabricatorAuthAuthProviderPHIDType.php',
    'PhabricatorAuthConduitAPIMethod' => 'applications/auth/conduit/PhabricatorAuthConduitAPIMethod.php',
    'PhabricatorAuthConfirmLinkController' => 'applications/auth/controller/PhabricatorAuthConfirmLinkController.php',
    'PhabricatorAuthController' => 'applications/auth/controller/PhabricatorAuthController.php',
    'PhabricatorAuthDAO' => 'applications/auth/storage/PhabricatorAuthDAO.php',
    'PhabricatorAuthDisableController' => 'applications/auth/controller/config/PhabricatorAuthDisableController.php',
    'PhabricatorAuthDowngradeSessionController' => 'applications/auth/controller/PhabricatorAuthDowngradeSessionController.php',
    'PhabricatorAuthEditController' => 'applications/auth/controller/config/PhabricatorAuthEditController.php',
    'PhabricatorAuthFactor' => 'applications/auth/factor/PhabricatorAuthFactor.php',
    'PhabricatorAuthFactorConfig' => 'applications/auth/storage/PhabricatorAuthFactorConfig.php',
    'PhabricatorAuthFactorTestCase' => 'applications/auth/factor/__tests__/PhabricatorAuthFactorTestCase.php',
    'PhabricatorAuthFinishController' => 'applications/auth/controller/PhabricatorAuthFinishController.php',
    'PhabricatorAuthHighSecurityRequiredException' => 'applications/auth/exception/PhabricatorAuthHighSecurityRequiredException.php',
    'PhabricatorAuthHighSecurityToken' => 'applications/auth/data/PhabricatorAuthHighSecurityToken.php',
    'PhabricatorAuthInvite' => 'applications/auth/storage/PhabricatorAuthInvite.php',
    'PhabricatorAuthInviteAccountException' => 'applications/auth/exception/PhabricatorAuthInviteAccountException.php',
    'PhabricatorAuthInviteAction' => 'applications/auth/data/PhabricatorAuthInviteAction.php',
    'PhabricatorAuthInviteActionTableView' => 'applications/auth/view/PhabricatorAuthInviteActionTableView.php',
    'PhabricatorAuthInviteController' => 'applications/auth/controller/PhabricatorAuthInviteController.php',
    'PhabricatorAuthInviteDialogException' => 'applications/auth/exception/PhabricatorAuthInviteDialogException.php',
    'PhabricatorAuthInviteEngine' => 'applications/auth/engine/PhabricatorAuthInviteEngine.php',
    'PhabricatorAuthInviteException' => 'applications/auth/exception/PhabricatorAuthInviteException.php',
    'PhabricatorAuthInviteInvalidException' => 'applications/auth/exception/PhabricatorAuthInviteInvalidException.php',
    'PhabricatorAuthInviteLoginException' => 'applications/auth/exception/PhabricatorAuthInviteLoginException.php',
    'PhabricatorAuthInvitePHIDType' => 'applications/auth/phid/PhabricatorAuthInvitePHIDType.php',
    'PhabricatorAuthInviteQuery' => 'applications/auth/query/PhabricatorAuthInviteQuery.php',
    'PhabricatorAuthInviteRegisteredException' => 'applications/auth/exception/PhabricatorAuthInviteRegisteredException.php',
    'PhabricatorAuthInviteSearchEngine' => 'applications/auth/query/PhabricatorAuthInviteSearchEngine.php',
    'PhabricatorAuthInviteTestCase' => 'applications/auth/factor/__tests__/PhabricatorAuthInviteTestCase.php',
    'PhabricatorAuthInviteVerifyException' => 'applications/auth/exception/PhabricatorAuthInviteVerifyException.php',
    'PhabricatorAuthInviteWorker' => 'applications/auth/worker/PhabricatorAuthInviteWorker.php',
    'PhabricatorAuthLinkController' => 'applications/auth/controller/PhabricatorAuthLinkController.php',
    'PhabricatorAuthListController' => 'applications/auth/controller/config/PhabricatorAuthListController.php',
    'PhabricatorAuthLoginController' => 'applications/auth/controller/PhabricatorAuthLoginController.php',
    'PhabricatorAuthLoginHandler' => 'applications/auth/handler/PhabricatorAuthLoginHandler.php',
    'PhabricatorAuthMainMenuBarExtension' => 'applications/auth/extension/PhabricatorAuthMainMenuBarExtension.php',
    'PhabricatorAuthManagementCachePKCS8Workflow' => 'applications/auth/management/PhabricatorAuthManagementCachePKCS8Workflow.php',
    'PhabricatorAuthManagementLDAPWorkflow' => 'applications/auth/management/PhabricatorAuthManagementLDAPWorkflow.php',
    'PhabricatorAuthManagementListFactorsWorkflow' => 'applications/auth/management/PhabricatorAuthManagementListFactorsWorkflow.php',
    'PhabricatorAuthManagementRecoverWorkflow' => 'applications/auth/management/PhabricatorAuthManagementRecoverWorkflow.php',
    'PhabricatorAuthManagementRefreshWorkflow' => 'applications/auth/management/PhabricatorAuthManagementRefreshWorkflow.php',
    'PhabricatorAuthManagementStripWorkflow' => 'applications/auth/management/PhabricatorAuthManagementStripWorkflow.php',
    'PhabricatorAuthManagementTrustOAuthClientWorkflow' => 'applications/auth/management/PhabricatorAuthManagementTrustOAuthClientWorkflow.php',
    'PhabricatorAuthManagementUnlimitWorkflow' => 'applications/auth/management/PhabricatorAuthManagementUnlimitWorkflow.php',
    'PhabricatorAuthManagementUntrustOAuthClientWorkflow' => 'applications/auth/management/PhabricatorAuthManagementUntrustOAuthClientWorkflow.php',
    'PhabricatorAuthManagementVerifyWorkflow' => 'applications/auth/management/PhabricatorAuthManagementVerifyWorkflow.php',
    'PhabricatorAuthManagementWorkflow' => 'applications/auth/management/PhabricatorAuthManagementWorkflow.php',
    'PhabricatorAuthNeedsApprovalController' => 'applications/auth/controller/PhabricatorAuthNeedsApprovalController.php',
    'PhabricatorAuthNeedsMultiFactorController' => 'applications/auth/controller/PhabricatorAuthNeedsMultiFactorController.php',
    'PhabricatorAuthNewController' => 'applications/auth/controller/config/PhabricatorAuthNewController.php',
    'PhabricatorAuthOldOAuthRedirectController' => 'applications/auth/controller/PhabricatorAuthOldOAuthRedirectController.php',
    'PhabricatorAuthOneTimeLoginController' => 'applications/auth/controller/PhabricatorAuthOneTimeLoginController.php',
    'PhabricatorAuthProvider' => 'applications/auth/provider/PhabricatorAuthProvider.php',
    'PhabricatorAuthProviderConfig' => 'applications/auth/storage/PhabricatorAuthProviderConfig.php',
    'PhabricatorAuthProviderConfigController' => 'applications/auth/controller/config/PhabricatorAuthProviderConfigController.php',
    'PhabricatorAuthProviderConfigEditor' => 'applications/auth/editor/PhabricatorAuthProviderConfigEditor.php',
    'PhabricatorAuthProviderConfigQuery' => 'applications/auth/query/PhabricatorAuthProviderConfigQuery.php',
    'PhabricatorAuthProviderConfigTransaction' => 'applications/auth/storage/PhabricatorAuthProviderConfigTransaction.php',
    'PhabricatorAuthProviderConfigTransactionQuery' => 'applications/auth/query/PhabricatorAuthProviderConfigTransactionQuery.php',
    'PhabricatorAuthQueryPublicKeysConduitAPIMethod' => 'applications/auth/conduit/PhabricatorAuthQueryPublicKeysConduitAPIMethod.php',
    'PhabricatorAuthRegisterController' => 'applications/auth/controller/PhabricatorAuthRegisterController.php',
    'PhabricatorAuthRevokeTokenController' => 'applications/auth/controller/PhabricatorAuthRevokeTokenController.php',
    'PhabricatorAuthSSHKey' => 'applications/auth/storage/PhabricatorAuthSSHKey.php',
    'PhabricatorAuthSSHKeyController' => 'applications/auth/controller/PhabricatorAuthSSHKeyController.php',
    'PhabricatorAuthSSHKeyDeleteController' => 'applications/auth/controller/PhabricatorAuthSSHKeyDeleteController.php',
    'PhabricatorAuthSSHKeyEditController' => 'applications/auth/controller/PhabricatorAuthSSHKeyEditController.php',
    'PhabricatorAuthSSHKeyGenerateController' => 'applications/auth/controller/PhabricatorAuthSSHKeyGenerateController.php',
    'PhabricatorAuthSSHKeyPHIDType' => 'applications/auth/phid/PhabricatorAuthSSHKeyPHIDType.php',
    'PhabricatorAuthSSHKeyQuery' => 'applications/auth/query/PhabricatorAuthSSHKeyQuery.php',
    'PhabricatorAuthSSHKeyTableView' => 'applications/auth/view/PhabricatorAuthSSHKeyTableView.php',
    'PhabricatorAuthSSHPublicKey' => 'applications/auth/sshkey/PhabricatorAuthSSHPublicKey.php',
    'PhabricatorAuthSession' => 'applications/auth/storage/PhabricatorAuthSession.php',
    'PhabricatorAuthSessionEngine' => 'applications/auth/engine/PhabricatorAuthSessionEngine.php',
    'PhabricatorAuthSessionGarbageCollector' => 'applications/auth/garbagecollector/PhabricatorAuthSessionGarbageCollector.php',
    'PhabricatorAuthSessionQuery' => 'applications/auth/query/PhabricatorAuthSessionQuery.php',
    'PhabricatorAuthSetupCheck' => 'applications/config/check/PhabricatorAuthSetupCheck.php',
    'PhabricatorAuthStartController' => 'applications/auth/controller/PhabricatorAuthStartController.php',
    'PhabricatorAuthTemporaryToken' => 'applications/auth/storage/PhabricatorAuthTemporaryToken.php',
    'PhabricatorAuthTemporaryTokenGarbageCollector' => 'applications/auth/garbagecollector/PhabricatorAuthTemporaryTokenGarbageCollector.php',
    'PhabricatorAuthTemporaryTokenQuery' => 'applications/auth/query/PhabricatorAuthTemporaryTokenQuery.php',
    'PhabricatorAuthTerminateSessionController' => 'applications/auth/controller/PhabricatorAuthTerminateSessionController.php',
    'PhabricatorAuthTryFactorAction' => 'applications/auth/action/PhabricatorAuthTryFactorAction.php',
    'PhabricatorAuthUnlinkController' => 'applications/auth/controller/PhabricatorAuthUnlinkController.php',
    'PhabricatorAuthValidateController' => 'applications/auth/controller/PhabricatorAuthValidateController.php',
    'PhabricatorAuthenticationConfigOptions' => 'applications/config/option/PhabricatorAuthenticationConfigOptions.php',
    'PhabricatorAutoEventListener' => 'infrastructure/events/PhabricatorAutoEventListener.php',
    'PhabricatorBadgeHasRecipientEdgeType' => 'applications/badges/edge/PhabricatorBadgeHasRecipientEdgeType.php',
    'PhabricatorBadgesApplication' => 'applications/badges/application/PhabricatorBadgesApplication.php',
    'PhabricatorBadgesArchiveController' => 'applications/badges/controller/PhabricatorBadgesArchiveController.php',
    'PhabricatorBadgesBadge' => 'applications/badges/storage/PhabricatorBadgesBadge.php',
    'PhabricatorBadgesCommentController' => 'applications/badges/controller/PhabricatorBadgesCommentController.php',
    'PhabricatorBadgesController' => 'applications/badges/controller/PhabricatorBadgesController.php',
    'PhabricatorBadgesCreateCapability' => 'applications/badges/capability/PhabricatorBadgesCreateCapability.php',
    'PhabricatorBadgesDAO' => 'applications/badges/storage/PhabricatorBadgesDAO.php',
    'PhabricatorBadgesDefaultEditCapability' => 'applications/badges/capability/PhabricatorBadgesDefaultEditCapability.php',
    'PhabricatorBadgesEditController' => 'applications/badges/controller/PhabricatorBadgesEditController.php',
    'PhabricatorBadgesEditEngine' => 'applications/badges/editor/PhabricatorBadgesEditEngine.php',
    'PhabricatorBadgesEditRecipientsController' => 'applications/badges/controller/PhabricatorBadgesEditRecipientsController.php',
    'PhabricatorBadgesEditor' => 'applications/badges/editor/PhabricatorBadgesEditor.php',
    'PhabricatorBadgesIconSet' => 'applications/badges/icon/PhabricatorBadgesIconSet.php',
    'PhabricatorBadgesListController' => 'applications/badges/controller/PhabricatorBadgesListController.php',
    'PhabricatorBadgesMailReceiver' => 'applications/badges/mail/PhabricatorBadgesMailReceiver.php',
    'PhabricatorBadgesPHIDType' => 'applications/badges/phid/PhabricatorBadgesPHIDType.php',
    'PhabricatorBadgesQuery' => 'applications/badges/query/PhabricatorBadgesQuery.php',
    'PhabricatorBadgesRecipientsListView' => 'applications/badges/view/PhabricatorBadgesRecipientsListView.php',
    'PhabricatorBadgesRemoveRecipientsController' => 'applications/badges/controller/PhabricatorBadgesRemoveRecipientsController.php',
    'PhabricatorBadgesReplyHandler' => 'applications/badges/mail/PhabricatorBadgesReplyHandler.php',
    'PhabricatorBadgesSchemaSpec' => 'applications/badges/storage/PhabricatorBadgesSchemaSpec.php',
    'PhabricatorBadgesSearchEngine' => 'applications/badges/query/PhabricatorBadgesSearchEngine.php',
    'PhabricatorBadgesTransaction' => 'applications/badges/storage/PhabricatorBadgesTransaction.php',
    'PhabricatorBadgesTransactionComment' => 'applications/badges/storage/PhabricatorBadgesTransactionComment.php',
    'PhabricatorBadgesTransactionQuery' => 'applications/badges/query/PhabricatorBadgesTransactionQuery.php',
    'PhabricatorBadgesViewController' => 'applications/badges/controller/PhabricatorBadgesViewController.php',
    'PhabricatorBarePageUIExample' => 'applications/uiexample/examples/PhabricatorBarePageUIExample.php',
    'PhabricatorBarePageView' => 'view/page/PhabricatorBarePageView.php',
    'PhabricatorBaseURISetupCheck' => 'applications/config/check/PhabricatorBaseURISetupCheck.php',
    'PhabricatorBcryptPasswordHasher' => 'infrastructure/util/password/PhabricatorBcryptPasswordHasher.php',
    'PhabricatorBinariesSetupCheck' => 'applications/config/check/PhabricatorBinariesSetupCheck.php',
    'PhabricatorBitbucketAuthProvider' => 'applications/auth/provider/PhabricatorBitbucketAuthProvider.php',
    'PhabricatorBot' => 'infrastructure/daemon/bot/PhabricatorBot.php',
    'PhabricatorBotChannel' => 'infrastructure/daemon/bot/target/PhabricatorBotChannel.php',
    'PhabricatorBotDebugLogHandler' => 'infrastructure/daemon/bot/handler/PhabricatorBotDebugLogHandler.php',
    'PhabricatorBotFeedNotificationHandler' => 'infrastructure/daemon/bot/handler/PhabricatorBotFeedNotificationHandler.php',
    'PhabricatorBotFlowdockProtocolAdapter' => 'infrastructure/daemon/bot/adapter/PhabricatorBotFlowdockProtocolAdapter.php',
    'PhabricatorBotHandler' => 'infrastructure/daemon/bot/handler/PhabricatorBotHandler.php',
    'PhabricatorBotLogHandler' => 'infrastructure/daemon/bot/handler/PhabricatorBotLogHandler.php',
    'PhabricatorBotMacroHandler' => 'infrastructure/daemon/bot/handler/PhabricatorBotMacroHandler.php',
    'PhabricatorBotMessage' => 'infrastructure/daemon/bot/PhabricatorBotMessage.php',
    'PhabricatorBotObjectNameHandler' => 'infrastructure/daemon/bot/handler/PhabricatorBotObjectNameHandler.php',
    'PhabricatorBotSymbolHandler' => 'infrastructure/daemon/bot/handler/PhabricatorBotSymbolHandler.php',
    'PhabricatorBotTarget' => 'infrastructure/daemon/bot/target/PhabricatorBotTarget.php',
    'PhabricatorBotUser' => 'infrastructure/daemon/bot/target/PhabricatorBotUser.php',
    'PhabricatorBotWhatsNewHandler' => 'infrastructure/daemon/bot/handler/PhabricatorBotWhatsNewHandler.php',
    'PhabricatorBritishEnglishTranslation' => 'infrastructure/internationalization/translation/PhabricatorBritishEnglishTranslation.php',
    'PhabricatorBuiltinPatchList' => 'infrastructure/storage/patch/PhabricatorBuiltinPatchList.php',
    'PhabricatorBusyUIExample' => 'applications/uiexample/examples/PhabricatorBusyUIExample.php',
    'PhabricatorCacheDAO' => 'applications/cache/storage/PhabricatorCacheDAO.php',
    'PhabricatorCacheGeneralGarbageCollector' => 'applications/cache/garbagecollector/PhabricatorCacheGeneralGarbageCollector.php',
    'PhabricatorCacheManagementPurgeWorkflow' => 'applications/cache/management/PhabricatorCacheManagementPurgeWorkflow.php',
    'PhabricatorCacheManagementWorkflow' => 'applications/cache/management/PhabricatorCacheManagementWorkflow.php',
    'PhabricatorCacheMarkupGarbageCollector' => 'applications/cache/garbagecollector/PhabricatorCacheMarkupGarbageCollector.php',
    'PhabricatorCacheSchemaSpec' => 'applications/cache/storage/PhabricatorCacheSchemaSpec.php',
    'PhabricatorCacheSetupCheck' => 'applications/config/check/PhabricatorCacheSetupCheck.php',
    'PhabricatorCacheSpec' => 'applications/cache/spec/PhabricatorCacheSpec.php',
    'PhabricatorCacheTTLGarbageCollector' => 'applications/cache/garbagecollector/PhabricatorCacheTTLGarbageCollector.php',
    'PhabricatorCaches' => 'applications/cache/PhabricatorCaches.php',
    'PhabricatorCachesTestCase' => 'applications/cache/__tests__/PhabricatorCachesTestCase.php',
    'PhabricatorCalendarApplication' => 'applications/calendar/application/PhabricatorCalendarApplication.php',
    'PhabricatorCalendarController' => 'applications/calendar/controller/PhabricatorCalendarController.php',
    'PhabricatorCalendarDAO' => 'applications/calendar/storage/PhabricatorCalendarDAO.php',
    'PhabricatorCalendarEvent' => 'applications/calendar/storage/PhabricatorCalendarEvent.php',
    'PhabricatorCalendarEventCancelController' => 'applications/calendar/controller/PhabricatorCalendarEventCancelController.php',
    'PhabricatorCalendarEventCommentController' => 'applications/calendar/controller/PhabricatorCalendarEventCommentController.php',
    'PhabricatorCalendarEventDragController' => 'applications/calendar/controller/PhabricatorCalendarEventDragController.php',
    'PhabricatorCalendarEventEditController' => 'applications/calendar/controller/PhabricatorCalendarEventEditController.php',
    'PhabricatorCalendarEventEditor' => 'applications/calendar/editor/PhabricatorCalendarEventEditor.php',
    'PhabricatorCalendarEventEmailCommand' => 'applications/calendar/command/PhabricatorCalendarEventEmailCommand.php',
    'PhabricatorCalendarEventFulltextEngine' => 'applications/calendar/search/PhabricatorCalendarEventFulltextEngine.php',
    'PhabricatorCalendarEventInvitee' => 'applications/calendar/storage/PhabricatorCalendarEventInvitee.php',
    'PhabricatorCalendarEventInviteeQuery' => 'applications/calendar/query/PhabricatorCalendarEventInviteeQuery.php',
    'PhabricatorCalendarEventJoinController' => 'applications/calendar/controller/PhabricatorCalendarEventJoinController.php',
    'PhabricatorCalendarEventListController' => 'applications/calendar/controller/PhabricatorCalendarEventListController.php',
    'PhabricatorCalendarEventMailReceiver' => 'applications/calendar/mail/PhabricatorCalendarEventMailReceiver.php',
    'PhabricatorCalendarEventPHIDType' => 'applications/calendar/phid/PhabricatorCalendarEventPHIDType.php',
    'PhabricatorCalendarEventQuery' => 'applications/calendar/query/PhabricatorCalendarEventQuery.php',
    'PhabricatorCalendarEventRSVPEmailCommand' => 'applications/calendar/command/PhabricatorCalendarEventRSVPEmailCommand.php',
    'PhabricatorCalendarEventSearchEngine' => 'applications/calendar/query/PhabricatorCalendarEventSearchEngine.php',
    'PhabricatorCalendarEventTransaction' => 'applications/calendar/storage/PhabricatorCalendarEventTransaction.php',
    'PhabricatorCalendarEventTransactionComment' => 'applications/calendar/storage/PhabricatorCalendarEventTransactionComment.php',
    'PhabricatorCalendarEventTransactionQuery' => 'applications/calendar/query/PhabricatorCalendarEventTransactionQuery.php',
    'PhabricatorCalendarEventViewController' => 'applications/calendar/controller/PhabricatorCalendarEventViewController.php',
    'PhabricatorCalendarHoliday' => 'applications/calendar/storage/PhabricatorCalendarHoliday.php',
    'PhabricatorCalendarHolidayTestCase' => 'applications/calendar/storage/__tests__/PhabricatorCalendarHolidayTestCase.php',
    'PhabricatorCalendarIconSet' => 'applications/calendar/icon/PhabricatorCalendarIconSet.php',
    'PhabricatorCalendarRemarkupRule' => 'applications/calendar/remarkup/PhabricatorCalendarRemarkupRule.php',
    'PhabricatorCalendarReplyHandler' => 'applications/calendar/mail/PhabricatorCalendarReplyHandler.php',
    'PhabricatorCalendarSchemaSpec' => 'applications/calendar/storage/PhabricatorCalendarSchemaSpec.php',
    'PhabricatorCampfireProtocolAdapter' => 'infrastructure/daemon/bot/adapter/PhabricatorCampfireProtocolAdapter.php',
    'PhabricatorCelerityApplication' => 'applications/celerity/application/PhabricatorCelerityApplication.php',
    'PhabricatorCelerityTestCase' => '__tests__/PhabricatorCelerityTestCase.php',
    'PhabricatorChangeParserTestCase' => 'applications/repository/worker/__tests__/PhabricatorChangeParserTestCase.php',
    'PhabricatorChangesetResponse' => 'infrastructure/diff/PhabricatorChangesetResponse.php',
    'PhabricatorChatLogApplication' => 'applications/chatlog/application/PhabricatorChatLogApplication.php',
    'PhabricatorChatLogChannel' => 'applications/chatlog/storage/PhabricatorChatLogChannel.php',
    'PhabricatorChatLogChannelListController' => 'applications/chatlog/controller/PhabricatorChatLogChannelListController.php',
    'PhabricatorChatLogChannelLogController' => 'applications/chatlog/controller/PhabricatorChatLogChannelLogController.php',
    'PhabricatorChatLogChannelQuery' => 'applications/chatlog/query/PhabricatorChatLogChannelQuery.php',
    'PhabricatorChatLogController' => 'applications/chatlog/controller/PhabricatorChatLogController.php',
    'PhabricatorChatLogDAO' => 'applications/chatlog/storage/PhabricatorChatLogDAO.php',
    'PhabricatorChatLogEvent' => 'applications/chatlog/storage/PhabricatorChatLogEvent.php',
    'PhabricatorChatLogQuery' => 'applications/chatlog/query/PhabricatorChatLogQuery.php',
    'PhabricatorChunkedFileStorageEngine' => 'applications/files/engine/PhabricatorChunkedFileStorageEngine.php',
    'PhabricatorClusterConfigOptions' => 'applications/config/option/PhabricatorClusterConfigOptions.php',
    'PhabricatorCommentEditEngineExtension' => 'applications/transactions/engineextension/PhabricatorCommentEditEngineExtension.php',
    'PhabricatorCommentEditField' => 'applications/transactions/editfield/PhabricatorCommentEditField.php',
    'PhabricatorCommentEditType' => 'applications/transactions/edittype/PhabricatorCommentEditType.php',
    'PhabricatorCommitBranchesField' => 'applications/repository/customfield/PhabricatorCommitBranchesField.php',
    'PhabricatorCommitCustomField' => 'applications/repository/customfield/PhabricatorCommitCustomField.php',
    'PhabricatorCommitMergedCommitsField' => 'applications/repository/customfield/PhabricatorCommitMergedCommitsField.php',
    'PhabricatorCommitRepositoryField' => 'applications/repository/customfield/PhabricatorCommitRepositoryField.php',
    'PhabricatorCommitSearchEngine' => 'applications/audit/query/PhabricatorCommitSearchEngine.php',
    'PhabricatorCommitTagsField' => 'applications/repository/customfield/PhabricatorCommitTagsField.php',
    'PhabricatorCommonPasswords' => 'applications/auth/constants/PhabricatorCommonPasswords.php',
    'PhabricatorConduitAPIController' => 'applications/conduit/controller/PhabricatorConduitAPIController.php',
    'PhabricatorConduitApplication' => 'applications/conduit/application/PhabricatorConduitApplication.php',
    'PhabricatorConduitCertificateToken' => 'applications/conduit/storage/PhabricatorConduitCertificateToken.php',
    'PhabricatorConduitConnectionLog' => 'applications/conduit/storage/PhabricatorConduitConnectionLog.php',
    'PhabricatorConduitConsoleController' => 'applications/conduit/controller/PhabricatorConduitConsoleController.php',
    'PhabricatorConduitController' => 'applications/conduit/controller/PhabricatorConduitController.php',
    'PhabricatorConduitDAO' => 'applications/conduit/storage/PhabricatorConduitDAO.php',
    'PhabricatorConduitEditField' => 'applications/transactions/editfield/PhabricatorConduitEditField.php',
    'PhabricatorConduitListController' => 'applications/conduit/controller/PhabricatorConduitListController.php',
    'PhabricatorConduitLogController' => 'applications/conduit/controller/PhabricatorConduitLogController.php',
    'PhabricatorConduitLogQuery' => 'applications/conduit/query/PhabricatorConduitLogQuery.php',
    'PhabricatorConduitLogSearchEngine' => 'applications/conduit/query/PhabricatorConduitLogSearchEngine.php',
    'PhabricatorConduitMethodCallLog' => 'applications/conduit/storage/PhabricatorConduitMethodCallLog.php',
    'PhabricatorConduitMethodQuery' => 'applications/conduit/query/PhabricatorConduitMethodQuery.php',
    'PhabricatorConduitRequestExceptionHandler' => 'aphront/handler/PhabricatorConduitRequestExceptionHandler.php',
    'PhabricatorConduitResultInterface' => 'applications/conduit/interface/PhabricatorConduitResultInterface.php',
    'PhabricatorConduitSearchEngine' => 'applications/conduit/query/PhabricatorConduitSearchEngine.php',
    'PhabricatorConduitSearchFieldSpecification' => 'applications/conduit/interface/PhabricatorConduitSearchFieldSpecification.php',
    'PhabricatorConduitTestCase' => '__tests__/PhabricatorConduitTestCase.php',
    'PhabricatorConduitToken' => 'applications/conduit/storage/PhabricatorConduitToken.php',
    'PhabricatorConduitTokenController' => 'applications/conduit/controller/PhabricatorConduitTokenController.php',
    'PhabricatorConduitTokenEditController' => 'applications/conduit/controller/PhabricatorConduitTokenEditController.php',
    'PhabricatorConduitTokenHandshakeController' => 'applications/conduit/controller/PhabricatorConduitTokenHandshakeController.php',
    'PhabricatorConduitTokenQuery' => 'applications/conduit/query/PhabricatorConduitTokenQuery.php',
    'PhabricatorConduitTokenTerminateController' => 'applications/conduit/controller/PhabricatorConduitTokenTerminateController.php',
    'PhabricatorConduitTokensSettingsPanel' => 'applications/conduit/settings/PhabricatorConduitTokensSettingsPanel.php',
    'PhabricatorConfigAllController' => 'applications/config/controller/PhabricatorConfigAllController.php',
    'PhabricatorConfigApplication' => 'applications/config/application/PhabricatorConfigApplication.php',
    'PhabricatorConfigCacheController' => 'applications/config/controller/PhabricatorConfigCacheController.php',
    'PhabricatorConfigCollectorsModule' => 'applications/config/module/PhabricatorConfigCollectorsModule.php',
    'PhabricatorConfigColumnSchema' => 'applications/config/schema/PhabricatorConfigColumnSchema.php',
    'PhabricatorConfigConfigPHIDType' => 'applications/config/phid/PhabricatorConfigConfigPHIDType.php',
    'PhabricatorConfigController' => 'applications/config/controller/PhabricatorConfigController.php',
    'PhabricatorConfigCoreSchemaSpec' => 'applications/config/schema/PhabricatorConfigCoreSchemaSpec.php',
    'PhabricatorConfigDatabaseController' => 'applications/config/controller/PhabricatorConfigDatabaseController.php',
    'PhabricatorConfigDatabaseIssueController' => 'applications/config/controller/PhabricatorConfigDatabaseIssueController.php',
    'PhabricatorConfigDatabaseSchema' => 'applications/config/schema/PhabricatorConfigDatabaseSchema.php',
    'PhabricatorConfigDatabaseSource' => 'infrastructure/env/PhabricatorConfigDatabaseSource.php',
    'PhabricatorConfigDatabaseStatusController' => 'applications/config/controller/PhabricatorConfigDatabaseStatusController.php',
    'PhabricatorConfigDefaultSource' => 'infrastructure/env/PhabricatorConfigDefaultSource.php',
    'PhabricatorConfigDictionarySource' => 'infrastructure/env/PhabricatorConfigDictionarySource.php',
    'PhabricatorConfigEdgeModule' => 'applications/config/module/PhabricatorConfigEdgeModule.php',
    'PhabricatorConfigEditController' => 'applications/config/controller/PhabricatorConfigEditController.php',
    'PhabricatorConfigEditor' => 'applications/config/editor/PhabricatorConfigEditor.php',
    'PhabricatorConfigEntry' => 'applications/config/storage/PhabricatorConfigEntry.php',
    'PhabricatorConfigEntryDAO' => 'applications/config/storage/PhabricatorConfigEntryDAO.php',
    'PhabricatorConfigEntryQuery' => 'applications/config/query/PhabricatorConfigEntryQuery.php',
    'PhabricatorConfigFileSource' => 'infrastructure/env/PhabricatorConfigFileSource.php',
    'PhabricatorConfigGroupController' => 'applications/config/controller/PhabricatorConfigGroupController.php',
    'PhabricatorConfigHTTPParameterTypesModule' => 'applications/config/module/PhabricatorConfigHTTPParameterTypesModule.php',
    'PhabricatorConfigHistoryController' => 'applications/config/controller/PhabricatorConfigHistoryController.php',
    'PhabricatorConfigIgnoreController' => 'applications/config/controller/PhabricatorConfigIgnoreController.php',
    'PhabricatorConfigIssueListController' => 'applications/config/controller/PhabricatorConfigIssueListController.php',
    'PhabricatorConfigIssueViewController' => 'applications/config/controller/PhabricatorConfigIssueViewController.php',
    'PhabricatorConfigJSON' => 'applications/config/json/PhabricatorConfigJSON.php',
    'PhabricatorConfigJSONOptionType' => 'applications/config/custom/PhabricatorConfigJSONOptionType.php',
    'PhabricatorConfigKeySchema' => 'applications/config/schema/PhabricatorConfigKeySchema.php',
    'PhabricatorConfigListController' => 'applications/config/controller/PhabricatorConfigListController.php',
    'PhabricatorConfigLocalSource' => 'infrastructure/env/PhabricatorConfigLocalSource.php',
    'PhabricatorConfigManagementDeleteWorkflow' => 'applications/config/management/PhabricatorConfigManagementDeleteWorkflow.php',
    'PhabricatorConfigManagementGetWorkflow' => 'applications/config/management/PhabricatorConfigManagementGetWorkflow.php',
    'PhabricatorConfigManagementListWorkflow' => 'applications/config/management/PhabricatorConfigManagementListWorkflow.php',
    'PhabricatorConfigManagementMigrateWorkflow' => 'applications/config/management/PhabricatorConfigManagementMigrateWorkflow.php',
    'PhabricatorConfigManagementSetWorkflow' => 'applications/config/management/PhabricatorConfigManagementSetWorkflow.php',
    'PhabricatorConfigManagementWorkflow' => 'applications/config/management/PhabricatorConfigManagementWorkflow.php',
    'PhabricatorConfigModule' => 'applications/config/module/PhabricatorConfigModule.php',
    'PhabricatorConfigModuleController' => 'applications/config/controller/PhabricatorConfigModuleController.php',
    'PhabricatorConfigOption' => 'applications/config/option/PhabricatorConfigOption.php',
    'PhabricatorConfigOptionType' => 'applications/config/custom/PhabricatorConfigOptionType.php',
    'PhabricatorConfigPHIDModule' => 'applications/config/module/PhabricatorConfigPHIDModule.php',
    'PhabricatorConfigProxySource' => 'infrastructure/env/PhabricatorConfigProxySource.php',
    'PhabricatorConfigPurgeCacheController' => 'applications/config/controller/PhabricatorConfigPurgeCacheController.php',
    'PhabricatorConfigRequestExceptionHandlerModule' => 'applications/config/module/PhabricatorConfigRequestExceptionHandlerModule.php',
    'PhabricatorConfigResponse' => 'applications/config/response/PhabricatorConfigResponse.php',
    'PhabricatorConfigSchemaQuery' => 'applications/config/schema/PhabricatorConfigSchemaQuery.php',
    'PhabricatorConfigSchemaSpec' => 'applications/config/schema/PhabricatorConfigSchemaSpec.php',
    'PhabricatorConfigServerSchema' => 'applications/config/schema/PhabricatorConfigServerSchema.php',
    'PhabricatorConfigSiteModule' => 'applications/config/module/PhabricatorConfigSiteModule.php',
    'PhabricatorConfigSiteSource' => 'infrastructure/env/PhabricatorConfigSiteSource.php',
    'PhabricatorConfigSource' => 'infrastructure/env/PhabricatorConfigSource.php',
    'PhabricatorConfigStackSource' => 'infrastructure/env/PhabricatorConfigStackSource.php',
    'PhabricatorConfigStorageSchema' => 'applications/config/schema/PhabricatorConfigStorageSchema.php',
    'PhabricatorConfigTableSchema' => 'applications/config/schema/PhabricatorConfigTableSchema.php',
    'PhabricatorConfigTransaction' => 'applications/config/storage/PhabricatorConfigTransaction.php',
    'PhabricatorConfigTransactionQuery' => 'applications/config/query/PhabricatorConfigTransactionQuery.php',
    'PhabricatorConfigValidationException' => 'applications/config/exception/PhabricatorConfigValidationException.php',
    'PhabricatorConfigVersionsModule' => 'applications/config/module/PhabricatorConfigVersionsModule.php',
    'PhabricatorConfigWelcomeController' => 'applications/config/controller/PhabricatorConfigWelcomeController.php',
    'PhabricatorConpherenceApplication' => 'applications/conpherence/application/PhabricatorConpherenceApplication.php',
    'PhabricatorConpherencePreferencesSettingsPanel' => 'applications/settings/panel/PhabricatorConpherencePreferencesSettingsPanel.php',
    'PhabricatorConpherenceThreadPHIDType' => 'applications/conpherence/phid/PhabricatorConpherenceThreadPHIDType.php',
    'PhabricatorConsoleApplication' => 'applications/console/application/PhabricatorConsoleApplication.php',
    'PhabricatorContentSource' => 'applications/metamta/contentsource/PhabricatorContentSource.php',
    'PhabricatorContentSourceView' => 'applications/metamta/contentsource/PhabricatorContentSourceView.php',
    'PhabricatorContributedToObjectEdgeType' => 'applications/transactions/edges/PhabricatorContributedToObjectEdgeType.php',
    'PhabricatorController' => 'applications/base/controller/PhabricatorController.php',
    'PhabricatorCookies' => 'applications/auth/constants/PhabricatorCookies.php',
    'PhabricatorCoreConfigOptions' => 'applications/config/option/PhabricatorCoreConfigOptions.php',
    'PhabricatorCountdown' => 'applications/countdown/storage/PhabricatorCountdown.php',
    'PhabricatorCountdownApplication' => 'applications/countdown/application/PhabricatorCountdownApplication.php',
    'PhabricatorCountdownCommentController' => 'applications/countdown/controller/PhabricatorCountdownCommentController.php',
    'PhabricatorCountdownController' => 'applications/countdown/controller/PhabricatorCountdownController.php',
    'PhabricatorCountdownCountdownPHIDType' => 'applications/countdown/phid/PhabricatorCountdownCountdownPHIDType.php',
    'PhabricatorCountdownDAO' => 'applications/countdown/storage/PhabricatorCountdownDAO.php',
    'PhabricatorCountdownDefaultEditCapability' => 'applications/countdown/capability/PhabricatorCountdownDefaultEditCapability.php',
    'PhabricatorCountdownDefaultViewCapability' => 'applications/countdown/capability/PhabricatorCountdownDefaultViewCapability.php',
    'PhabricatorCountdownDeleteController' => 'applications/countdown/controller/PhabricatorCountdownDeleteController.php',
    'PhabricatorCountdownEditController' => 'applications/countdown/controller/PhabricatorCountdownEditController.php',
    'PhabricatorCountdownEditor' => 'applications/countdown/editor/PhabricatorCountdownEditor.php',
    'PhabricatorCountdownListController' => 'applications/countdown/controller/PhabricatorCountdownListController.php',
    'PhabricatorCountdownMailReceiver' => 'applications/countdown/mail/PhabricatorCountdownMailReceiver.php',
    'PhabricatorCountdownQuery' => 'applications/countdown/query/PhabricatorCountdownQuery.php',
    'PhabricatorCountdownRemarkupRule' => 'applications/countdown/remarkup/PhabricatorCountdownRemarkupRule.php',
    'PhabricatorCountdownReplyHandler' => 'applications/countdown/mail/PhabricatorCountdownReplyHandler.php',
    'PhabricatorCountdownSchemaSpec' => 'applications/countdown/storage/PhabricatorCountdownSchemaSpec.php',
    'PhabricatorCountdownSearchEngine' => 'applications/countdown/query/PhabricatorCountdownSearchEngine.php',
    'PhabricatorCountdownTransaction' => 'applications/countdown/storage/PhabricatorCountdownTransaction.php',
    'PhabricatorCountdownTransactionComment' => 'applications/countdown/storage/PhabricatorCountdownTransactionComment.php',
    'PhabricatorCountdownTransactionQuery' => 'applications/countdown/query/PhabricatorCountdownTransactionQuery.php',
    'PhabricatorCountdownView' => 'applications/countdown/view/PhabricatorCountdownView.php',
    'PhabricatorCountdownViewController' => 'applications/countdown/controller/PhabricatorCountdownViewController.php',
    'PhabricatorCredentialsUsedByObjectEdgeType' => 'applications/passphrase/edge/PhabricatorCredentialsUsedByObjectEdgeType.php',
    'PhabricatorCursorPagedPolicyAwareQuery' => 'infrastructure/query/policy/PhabricatorCursorPagedPolicyAwareQuery.php',
    'PhabricatorCustomField' => 'infrastructure/customfield/field/PhabricatorCustomField.php',
    'PhabricatorCustomFieldAttachment' => 'infrastructure/customfield/field/PhabricatorCustomFieldAttachment.php',
    'PhabricatorCustomFieldConfigOptionType' => 'infrastructure/customfield/config/PhabricatorCustomFieldConfigOptionType.php',
    'PhabricatorCustomFieldDataNotAvailableException' => 'infrastructure/customfield/exception/PhabricatorCustomFieldDataNotAvailableException.php',
    'PhabricatorCustomFieldEditEngineExtension' => 'infrastructure/customfield/engineextension/PhabricatorCustomFieldEditEngineExtension.php',
    'PhabricatorCustomFieldEditField' => 'infrastructure/customfield/editor/PhabricatorCustomFieldEditField.php',
    'PhabricatorCustomFieldEditType' => 'infrastructure/customfield/editor/PhabricatorCustomFieldEditType.php',
    'PhabricatorCustomFieldFulltextEngineExtension' => 'infrastructure/customfield/engineextension/PhabricatorCustomFieldFulltextEngineExtension.php',
    'PhabricatorCustomFieldHeraldField' => 'infrastructure/customfield/herald/PhabricatorCustomFieldHeraldField.php',
    'PhabricatorCustomFieldHeraldFieldGroup' => 'infrastructure/customfield/herald/PhabricatorCustomFieldHeraldFieldGroup.php',
    'PhabricatorCustomFieldImplementationIncompleteException' => 'infrastructure/customfield/exception/PhabricatorCustomFieldImplementationIncompleteException.php',
    'PhabricatorCustomFieldIndexStorage' => 'infrastructure/customfield/storage/PhabricatorCustomFieldIndexStorage.php',
    'PhabricatorCustomFieldInterface' => 'infrastructure/customfield/interface/PhabricatorCustomFieldInterface.php',
    'PhabricatorCustomFieldList' => 'infrastructure/customfield/field/PhabricatorCustomFieldList.php',
    'PhabricatorCustomFieldMonogramParser' => 'infrastructure/customfield/parser/PhabricatorCustomFieldMonogramParser.php',
    'PhabricatorCustomFieldNotAttachedException' => 'infrastructure/customfield/exception/PhabricatorCustomFieldNotAttachedException.php',
    'PhabricatorCustomFieldNotProxyException' => 'infrastructure/customfield/exception/PhabricatorCustomFieldNotProxyException.php',
    'PhabricatorCustomFieldNumericIndexStorage' => 'infrastructure/customfield/storage/PhabricatorCustomFieldNumericIndexStorage.php',
    'PhabricatorCustomFieldSearchEngineExtension' => 'infrastructure/customfield/engineextension/PhabricatorCustomFieldSearchEngineExtension.php',
    'PhabricatorCustomFieldStorage' => 'infrastructure/customfield/storage/PhabricatorCustomFieldStorage.php',
    'PhabricatorCustomFieldStringIndexStorage' => 'infrastructure/customfield/storage/PhabricatorCustomFieldStringIndexStorage.php',
    'PhabricatorCustomHeaderConfigType' => 'applications/config/custom/PhabricatorCustomHeaderConfigType.php',
    'PhabricatorDaemon' => 'infrastructure/daemon/PhabricatorDaemon.php',
    'PhabricatorDaemonBulkJobListController' => 'applications/daemon/controller/PhabricatorDaemonBulkJobListController.php',
    'PhabricatorDaemonBulkJobMonitorController' => 'applications/daemon/controller/PhabricatorDaemonBulkJobMonitorController.php',
    'PhabricatorDaemonBulkJobViewController' => 'applications/daemon/controller/PhabricatorDaemonBulkJobViewController.php',
    'PhabricatorDaemonConsoleController' => 'applications/daemon/controller/PhabricatorDaemonConsoleController.php',
    'PhabricatorDaemonController' => 'applications/daemon/controller/PhabricatorDaemonController.php',
    'PhabricatorDaemonDAO' => 'applications/daemon/storage/PhabricatorDaemonDAO.php',
    'PhabricatorDaemonEventListener' => 'applications/daemon/event/PhabricatorDaemonEventListener.php',
    'PhabricatorDaemonLog' => 'applications/daemon/storage/PhabricatorDaemonLog.php',
    'PhabricatorDaemonLogEvent' => 'applications/daemon/storage/PhabricatorDaemonLogEvent.php',
    'PhabricatorDaemonLogEventGarbageCollector' => 'applications/daemon/garbagecollector/PhabricatorDaemonLogEventGarbageCollector.php',
    'PhabricatorDaemonLogEventViewController' => 'applications/daemon/controller/PhabricatorDaemonLogEventViewController.php',
    'PhabricatorDaemonLogEventsView' => 'applications/daemon/view/PhabricatorDaemonLogEventsView.php',
    'PhabricatorDaemonLogGarbageCollector' => 'applications/daemon/garbagecollector/PhabricatorDaemonLogGarbageCollector.php',
    'PhabricatorDaemonLogListController' => 'applications/daemon/controller/PhabricatorDaemonLogListController.php',
    'PhabricatorDaemonLogListView' => 'applications/daemon/view/PhabricatorDaemonLogListView.php',
    'PhabricatorDaemonLogQuery' => 'applications/daemon/query/PhabricatorDaemonLogQuery.php',
    'PhabricatorDaemonLogViewController' => 'applications/daemon/controller/PhabricatorDaemonLogViewController.php',
    'PhabricatorDaemonManagementDebugWorkflow' => 'applications/daemon/management/PhabricatorDaemonManagementDebugWorkflow.php',
    'PhabricatorDaemonManagementLaunchWorkflow' => 'applications/daemon/management/PhabricatorDaemonManagementLaunchWorkflow.php',
    'PhabricatorDaemonManagementListWorkflow' => 'applications/daemon/management/PhabricatorDaemonManagementListWorkflow.php',
    'PhabricatorDaemonManagementLogWorkflow' => 'applications/daemon/management/PhabricatorDaemonManagementLogWorkflow.php',
    'PhabricatorDaemonManagementReloadWorkflow' => 'applications/daemon/management/PhabricatorDaemonManagementReloadWorkflow.php',
    'PhabricatorDaemonManagementRestartWorkflow' => 'applications/daemon/management/PhabricatorDaemonManagementRestartWorkflow.php',
    'PhabricatorDaemonManagementStartWorkflow' => 'applications/daemon/management/PhabricatorDaemonManagementStartWorkflow.php',
    'PhabricatorDaemonManagementStatusWorkflow' => 'applications/daemon/management/PhabricatorDaemonManagementStatusWorkflow.php',
    'PhabricatorDaemonManagementStopWorkflow' => 'applications/daemon/management/PhabricatorDaemonManagementStopWorkflow.php',
    'PhabricatorDaemonManagementWorkflow' => 'applications/daemon/management/PhabricatorDaemonManagementWorkflow.php',
    'PhabricatorDaemonOverseerModule' => 'infrastructure/daemon/overseer/PhabricatorDaemonOverseerModule.php',
    'PhabricatorDaemonReference' => 'infrastructure/daemon/control/PhabricatorDaemonReference.php',
    'PhabricatorDaemonTaskGarbageCollector' => 'applications/daemon/garbagecollector/PhabricatorDaemonTaskGarbageCollector.php',
    'PhabricatorDaemonTasksTableView' => 'applications/daemon/view/PhabricatorDaemonTasksTableView.php',
    'PhabricatorDaemonsApplication' => 'applications/daemon/application/PhabricatorDaemonsApplication.php',
    'PhabricatorDaemonsSetupCheck' => 'applications/config/check/PhabricatorDaemonsSetupCheck.php',
    'PhabricatorDailyRoutineTriggerClock' => 'infrastructure/daemon/workers/clock/PhabricatorDailyRoutineTriggerClock.php',
    'PhabricatorDashboard' => 'applications/dashboard/storage/PhabricatorDashboard.php',
    'PhabricatorDashboardAddPanelController' => 'applications/dashboard/controller/PhabricatorDashboardAddPanelController.php',
    'PhabricatorDashboardApplication' => 'applications/dashboard/application/PhabricatorDashboardApplication.php',
    'PhabricatorDashboardArchiveController' => 'applications/dashboard/controller/PhabricatorDashboardArchiveController.php',
    'PhabricatorDashboardController' => 'applications/dashboard/controller/PhabricatorDashboardController.php',
    'PhabricatorDashboardCopyController' => 'applications/dashboard/controller/PhabricatorDashboardCopyController.php',
    'PhabricatorDashboardDAO' => 'applications/dashboard/storage/PhabricatorDashboardDAO.php',
    'PhabricatorDashboardDashboardHasPanelEdgeType' => 'applications/dashboard/edge/PhabricatorDashboardDashboardHasPanelEdgeType.php',
    'PhabricatorDashboardDashboardPHIDType' => 'applications/dashboard/phid/PhabricatorDashboardDashboardPHIDType.php',
    'PhabricatorDashboardEditController' => 'applications/dashboard/controller/PhabricatorDashboardEditController.php',
    'PhabricatorDashboardHistoryController' => 'applications/dashboard/controller/PhabricatorDashboardHistoryController.php',
    'PhabricatorDashboardInstall' => 'applications/dashboard/storage/PhabricatorDashboardInstall.php',
    'PhabricatorDashboardInstallController' => 'applications/dashboard/controller/PhabricatorDashboardInstallController.php',
    'PhabricatorDashboardLayoutConfig' => 'applications/dashboard/layoutconfig/PhabricatorDashboardLayoutConfig.php',
    'PhabricatorDashboardListController' => 'applications/dashboard/controller/PhabricatorDashboardListController.php',
    'PhabricatorDashboardManageController' => 'applications/dashboard/controller/PhabricatorDashboardManageController.php',
    'PhabricatorDashboardMovePanelController' => 'applications/dashboard/controller/PhabricatorDashboardMovePanelController.php',
    'PhabricatorDashboardPanel' => 'applications/dashboard/storage/PhabricatorDashboardPanel.php',
    'PhabricatorDashboardPanelArchiveController' => 'applications/dashboard/controller/PhabricatorDashboardPanelArchiveController.php',
    'PhabricatorDashboardPanelCoreCustomField' => 'applications/dashboard/customfield/PhabricatorDashboardPanelCoreCustomField.php',
    'PhabricatorDashboardPanelCustomField' => 'applications/dashboard/customfield/PhabricatorDashboardPanelCustomField.php',
    'PhabricatorDashboardPanelEditController' => 'applications/dashboard/controller/PhabricatorDashboardPanelEditController.php',
    'PhabricatorDashboardPanelHasDashboardEdgeType' => 'applications/dashboard/edge/PhabricatorDashboardPanelHasDashboardEdgeType.php',
    'PhabricatorDashboardPanelListController' => 'applications/dashboard/controller/PhabricatorDashboardPanelListController.php',
    'PhabricatorDashboardPanelPHIDType' => 'applications/dashboard/phid/PhabricatorDashboardPanelPHIDType.php',
    'PhabricatorDashboardPanelQuery' => 'applications/dashboard/query/PhabricatorDashboardPanelQuery.php',
    'PhabricatorDashboardPanelRenderController' => 'applications/dashboard/controller/PhabricatorDashboardPanelRenderController.php',
    'PhabricatorDashboardPanelRenderingEngine' => 'applications/dashboard/engine/PhabricatorDashboardPanelRenderingEngine.php',
    'PhabricatorDashboardPanelSearchApplicationCustomField' => 'applications/dashboard/customfield/PhabricatorDashboardPanelSearchApplicationCustomField.php',
    'PhabricatorDashboardPanelSearchEngine' => 'applications/dashboard/query/PhabricatorDashboardPanelSearchEngine.php',
    'PhabricatorDashboardPanelSearchQueryCustomField' => 'applications/dashboard/customfield/PhabricatorDashboardPanelSearchQueryCustomField.php',
    'PhabricatorDashboardPanelTabsCustomField' => 'applications/dashboard/customfield/PhabricatorDashboardPanelTabsCustomField.php',
    'PhabricatorDashboardPanelTransaction' => 'applications/dashboard/storage/PhabricatorDashboardPanelTransaction.php',
    'PhabricatorDashboardPanelTransactionEditor' => 'applications/dashboard/editor/PhabricatorDashboardPanelTransactionEditor.php',
    'PhabricatorDashboardPanelTransactionQuery' => 'applications/dashboard/query/PhabricatorDashboardPanelTransactionQuery.php',
    'PhabricatorDashboardPanelType' => 'applications/dashboard/paneltype/PhabricatorDashboardPanelType.php',
    'PhabricatorDashboardPanelViewController' => 'applications/dashboard/controller/PhabricatorDashboardPanelViewController.php',
    'PhabricatorDashboardQuery' => 'applications/dashboard/query/PhabricatorDashboardQuery.php',
    'PhabricatorDashboardQueryPanelType' => 'applications/dashboard/paneltype/PhabricatorDashboardQueryPanelType.php',
    'PhabricatorDashboardRemarkupRule' => 'applications/dashboard/remarkup/PhabricatorDashboardRemarkupRule.php',
    'PhabricatorDashboardRemovePanelController' => 'applications/dashboard/controller/PhabricatorDashboardRemovePanelController.php',
    'PhabricatorDashboardRenderingEngine' => 'applications/dashboard/engine/PhabricatorDashboardRenderingEngine.php',
    'PhabricatorDashboardSchemaSpec' => 'applications/dashboard/storage/PhabricatorDashboardSchemaSpec.php',
    'PhabricatorDashboardSearchEngine' => 'applications/dashboard/query/PhabricatorDashboardSearchEngine.php',
    'PhabricatorDashboardTabsPanelType' => 'applications/dashboard/paneltype/PhabricatorDashboardTabsPanelType.php',
    'PhabricatorDashboardTextPanelType' => 'applications/dashboard/paneltype/PhabricatorDashboardTextPanelType.php',
    'PhabricatorDashboardTransaction' => 'applications/dashboard/storage/PhabricatorDashboardTransaction.php',
    'PhabricatorDashboardTransactionEditor' => 'applications/dashboard/editor/PhabricatorDashboardTransactionEditor.php',
    'PhabricatorDashboardTransactionQuery' => 'applications/dashboard/query/PhabricatorDashboardTransactionQuery.php',
    'PhabricatorDashboardUninstallController' => 'applications/dashboard/controller/PhabricatorDashboardUninstallController.php',
    'PhabricatorDashboardViewController' => 'applications/dashboard/controller/PhabricatorDashboardViewController.php',
    'PhabricatorDataCacheSpec' => 'applications/cache/spec/PhabricatorDataCacheSpec.php',
    'PhabricatorDataNotAttachedException' => 'infrastructure/storage/lisk/PhabricatorDataNotAttachedException.php',
    'PhabricatorDatabaseSetupCheck' => 'applications/config/check/PhabricatorDatabaseSetupCheck.php',
    'PhabricatorDatasourceEditField' => 'applications/transactions/editfield/PhabricatorDatasourceEditField.php',
    'PhabricatorDatasourceEditType' => 'applications/transactions/edittype/PhabricatorDatasourceEditType.php',
    'PhabricatorDateTimeSettingsPanel' => 'applications/settings/panel/PhabricatorDateTimeSettingsPanel.php',
    'PhabricatorDebugController' => 'applications/system/controller/PhabricatorDebugController.php',
    'PhabricatorDefaultRequestExceptionHandler' => 'aphront/handler/PhabricatorDefaultRequestExceptionHandler.php',
    'PhabricatorDesktopNotificationsSettingsPanel' => 'applications/settings/panel/PhabricatorDesktopNotificationsSettingsPanel.php',
    'PhabricatorDestructibleInterface' => 'applications/system/interface/PhabricatorDestructibleInterface.php',
    'PhabricatorDestructionEngine' => 'applications/system/engine/PhabricatorDestructionEngine.php',
    'PhabricatorDestructionEngineExtension' => 'applications/system/engine/PhabricatorDestructionEngineExtension.php',
    'PhabricatorDestructionEngineExtensionModule' => 'applications/system/engine/PhabricatorDestructionEngineExtensionModule.php',
    'PhabricatorDeveloperConfigOptions' => 'applications/config/option/PhabricatorDeveloperConfigOptions.php',
    'PhabricatorDeveloperPreferencesSettingsPanel' => 'applications/settings/panel/PhabricatorDeveloperPreferencesSettingsPanel.php',
    'PhabricatorDiffInlineCommentQuery' => 'infrastructure/diff/query/PhabricatorDiffInlineCommentQuery.php',
    'PhabricatorDiffPreferencesSettingsPanel' => 'applications/settings/panel/PhabricatorDiffPreferencesSettingsPanel.php',
    'PhabricatorDifferenceEngine' => 'infrastructure/diff/PhabricatorDifferenceEngine.php',
    'PhabricatorDifferentialApplication' => 'applications/differential/application/PhabricatorDifferentialApplication.php',
    'PhabricatorDifferentialAttachCommitWorkflow' => 'applications/differential/management/PhabricatorDifferentialAttachCommitWorkflow.php',
    'PhabricatorDifferentialConfigOptions' => 'applications/differential/config/PhabricatorDifferentialConfigOptions.php',
    'PhabricatorDifferentialExtractWorkflow' => 'applications/differential/management/PhabricatorDifferentialExtractWorkflow.php',
    'PhabricatorDifferentialManagementWorkflow' => 'applications/differential/management/PhabricatorDifferentialManagementWorkflow.php',
    'PhabricatorDifferentialRevisionTestDataGenerator' => 'applications/differential/lipsum/PhabricatorDifferentialRevisionTestDataGenerator.php',
    'PhabricatorDiffusionApplication' => 'applications/diffusion/application/PhabricatorDiffusionApplication.php',
    'PhabricatorDiffusionConfigOptions' => 'applications/diffusion/config/PhabricatorDiffusionConfigOptions.php',
    'PhabricatorDisabledUserController' => 'applications/auth/controller/PhabricatorDisabledUserController.php',
    'PhabricatorDisplayPreferencesSettingsPanel' => 'applications/settings/panel/PhabricatorDisplayPreferencesSettingsPanel.php',
    'PhabricatorDisqusAuthProvider' => 'applications/auth/provider/PhabricatorDisqusAuthProvider.php',
    'PhabricatorDividerProfilePanel' => 'applications/search/profilepanel/PhabricatorDividerProfilePanel.php',
    'PhabricatorDivinerApplication' => 'applications/diviner/application/PhabricatorDivinerApplication.php',
    'PhabricatorDoorkeeperApplication' => 'applications/doorkeeper/application/PhabricatorDoorkeeperApplication.php',
    'PhabricatorDraft' => 'applications/draft/storage/PhabricatorDraft.php',
    'PhabricatorDraftDAO' => 'applications/draft/storage/PhabricatorDraftDAO.php',
    'PhabricatorDrydockApplication' => 'applications/drydock/application/PhabricatorDrydockApplication.php',
    'PhabricatorEdgeConfig' => 'infrastructure/edges/constants/PhabricatorEdgeConfig.php',
    'PhabricatorEdgeConstants' => 'infrastructure/edges/constants/PhabricatorEdgeConstants.php',
    'PhabricatorEdgeCycleException' => 'infrastructure/edges/exception/PhabricatorEdgeCycleException.php',
    'PhabricatorEdgeEditType' => 'applications/transactions/edittype/PhabricatorEdgeEditType.php',
    'PhabricatorEdgeEditor' => 'infrastructure/edges/editor/PhabricatorEdgeEditor.php',
    'PhabricatorEdgeGraph' => 'infrastructure/edges/util/PhabricatorEdgeGraph.php',
    'PhabricatorEdgeQuery' => 'infrastructure/edges/query/PhabricatorEdgeQuery.php',
    'PhabricatorEdgeTestCase' => 'infrastructure/edges/__tests__/PhabricatorEdgeTestCase.php',
    'PhabricatorEdgeType' => 'infrastructure/edges/type/PhabricatorEdgeType.php',
    'PhabricatorEdgeTypeTestCase' => 'infrastructure/edges/type/__tests__/PhabricatorEdgeTypeTestCase.php',
    'PhabricatorEdgesDestructionEngineExtension' => 'infrastructure/edges/engineextension/PhabricatorEdgesDestructionEngineExtension.php',
    'PhabricatorEditEngine' => 'applications/transactions/editengine/PhabricatorEditEngine.php',
    'PhabricatorEditEngineAPIMethod' => 'applications/transactions/editengine/PhabricatorEditEngineAPIMethod.php',
    'PhabricatorEditEngineCommentAction' => 'applications/transactions/commentaction/PhabricatorEditEngineCommentAction.php',
    'PhabricatorEditEngineConfiguration' => 'applications/transactions/storage/PhabricatorEditEngineConfiguration.php',
    'PhabricatorEditEngineConfigurationDefaultCreateController' => 'applications/transactions/controller/PhabricatorEditEngineConfigurationDefaultCreateController.php',
    'PhabricatorEditEngineConfigurationDefaultsController' => 'applications/transactions/controller/PhabricatorEditEngineConfigurationDefaultsController.php',
    'PhabricatorEditEngineConfigurationDisableController' => 'applications/transactions/controller/PhabricatorEditEngineConfigurationDisableController.php',
    'PhabricatorEditEngineConfigurationEditController' => 'applications/transactions/controller/PhabricatorEditEngineConfigurationEditController.php',
    'PhabricatorEditEngineConfigurationEditEngine' => 'applications/transactions/editor/PhabricatorEditEngineConfigurationEditEngine.php',
    'PhabricatorEditEngineConfigurationEditor' => 'applications/transactions/editor/PhabricatorEditEngineConfigurationEditor.php',
    'PhabricatorEditEngineConfigurationIsEditController' => 'applications/transactions/controller/PhabricatorEditEngineConfigurationIsEditController.php',
    'PhabricatorEditEngineConfigurationListController' => 'applications/transactions/controller/PhabricatorEditEngineConfigurationListController.php',
    'PhabricatorEditEngineConfigurationLockController' => 'applications/transactions/controller/PhabricatorEditEngineConfigurationLockController.php',
    'PhabricatorEditEngineConfigurationPHIDType' => 'applications/transactions/phid/PhabricatorEditEngineConfigurationPHIDType.php',
    'PhabricatorEditEngineConfigurationQuery' => 'applications/transactions/query/PhabricatorEditEngineConfigurationQuery.php',
    'PhabricatorEditEngineConfigurationReorderController' => 'applications/transactions/controller/PhabricatorEditEngineConfigurationReorderController.php',
    'PhabricatorEditEngineConfigurationSaveController' => 'applications/transactions/controller/PhabricatorEditEngineConfigurationSaveController.php',
    'PhabricatorEditEngineConfigurationSearchEngine' => 'applications/transactions/query/PhabricatorEditEngineConfigurationSearchEngine.php',
    'PhabricatorEditEngineConfigurationSortController' => 'applications/transactions/controller/PhabricatorEditEngineConfigurationSortController.php',
    'PhabricatorEditEngineConfigurationTransaction' => 'applications/transactions/storage/PhabricatorEditEngineConfigurationTransaction.php',
    'PhabricatorEditEngineConfigurationTransactionQuery' => 'applications/transactions/query/PhabricatorEditEngineConfigurationTransactionQuery.php',
    'PhabricatorEditEngineConfigurationViewController' => 'applications/transactions/controller/PhabricatorEditEngineConfigurationViewController.php',
    'PhabricatorEditEngineController' => 'applications/transactions/controller/PhabricatorEditEngineController.php',
    'PhabricatorEditEngineExtension' => 'applications/transactions/engineextension/PhabricatorEditEngineExtension.php',
    'PhabricatorEditEngineExtensionModule' => 'applications/transactions/engineextension/PhabricatorEditEngineExtensionModule.php',
    'PhabricatorEditEngineListController' => 'applications/transactions/controller/PhabricatorEditEngineListController.php',
    'PhabricatorEditEngineQuery' => 'applications/transactions/query/PhabricatorEditEngineQuery.php',
    'PhabricatorEditEngineSearchEngine' => 'applications/transactions/query/PhabricatorEditEngineSearchEngine.php',
    'PhabricatorEditEngineSelectCommentAction' => 'applications/transactions/commentaction/PhabricatorEditEngineSelectCommentAction.php',
    'PhabricatorEditEngineTokenizerCommentAction' => 'applications/transactions/commentaction/PhabricatorEditEngineTokenizerCommentAction.php',
    'PhabricatorEditField' => 'applications/transactions/editfield/PhabricatorEditField.php',
    'PhabricatorEditType' => 'applications/transactions/edittype/PhabricatorEditType.php',
    'PhabricatorEditor' => 'infrastructure/PhabricatorEditor.php',
    'PhabricatorElasticFulltextStorageEngine' => 'applications/search/fulltextstorage/PhabricatorElasticFulltextStorageEngine.php',
    'PhabricatorElasticSearchSetupCheck' => 'applications/config/check/PhabricatorElasticSearchSetupCheck.php',
    'PhabricatorEmailAddressesSettingsPanel' => 'applications/settings/panel/PhabricatorEmailAddressesSettingsPanel.php',
    'PhabricatorEmailFormatSettingsPanel' => 'applications/settings/panel/PhabricatorEmailFormatSettingsPanel.php',
    'PhabricatorEmailLoginController' => 'applications/auth/controller/PhabricatorEmailLoginController.php',
    'PhabricatorEmailPreferencesSettingsPanel' => 'applications/settings/panel/PhabricatorEmailPreferencesSettingsPanel.php',
    'PhabricatorEmailVerificationController' => 'applications/auth/controller/PhabricatorEmailVerificationController.php',
    'PhabricatorEmbedFileRemarkupRule' => 'applications/files/markup/PhabricatorEmbedFileRemarkupRule.php',
    'PhabricatorEmojiRemarkupRule' => 'applications/macro/markup/PhabricatorEmojiRemarkupRule.php',
    'PhabricatorEmptyQueryException' => 'infrastructure/query/PhabricatorEmptyQueryException.php',
    'PhabricatorEnv' => 'infrastructure/env/PhabricatorEnv.php',
    'PhabricatorEnvTestCase' => 'infrastructure/env/__tests__/PhabricatorEnvTestCase.php',
    'PhabricatorEvent' => 'infrastructure/events/PhabricatorEvent.php',
    'PhabricatorEventEngine' => 'infrastructure/events/PhabricatorEventEngine.php',
    'PhabricatorEventListener' => 'infrastructure/events/PhabricatorEventListener.php',
    'PhabricatorEventType' => 'infrastructure/events/constant/PhabricatorEventType.php',
    'PhabricatorExampleEventListener' => 'infrastructure/events/PhabricatorExampleEventListener.php',
    'PhabricatorExecFutureFileUploadSource' => 'applications/files/uploadsource/PhabricatorExecFutureFileUploadSource.php',
    'PhabricatorExtendedPolicyInterface' => 'applications/policy/interface/PhabricatorExtendedPolicyInterface.php',
    'PhabricatorExtendingPhabricatorConfigOptions' => 'applications/config/option/PhabricatorExtendingPhabricatorConfigOptions.php',
    'PhabricatorExtensionsSetupCheck' => 'applications/config/check/PhabricatorExtensionsSetupCheck.php',
    'PhabricatorExternalAccount' => 'applications/people/storage/PhabricatorExternalAccount.php',
    'PhabricatorExternalAccountQuery' => 'applications/auth/query/PhabricatorExternalAccountQuery.php',
    'PhabricatorExternalAccountsSettingsPanel' => 'applications/settings/panel/PhabricatorExternalAccountsSettingsPanel.php',
    'PhabricatorExtraConfigSetupCheck' => 'applications/config/check/PhabricatorExtraConfigSetupCheck.php',
    'PhabricatorFacebookAuthProvider' => 'applications/auth/provider/PhabricatorFacebookAuthProvider.php',
    'PhabricatorFactAggregate' => 'applications/fact/storage/PhabricatorFactAggregate.php',
    'PhabricatorFactApplication' => 'applications/fact/application/PhabricatorFactApplication.php',
    'PhabricatorFactChartController' => 'applications/fact/controller/PhabricatorFactChartController.php',
    'PhabricatorFactController' => 'applications/fact/controller/PhabricatorFactController.php',
    'PhabricatorFactCountEngine' => 'applications/fact/engine/PhabricatorFactCountEngine.php',
    'PhabricatorFactCursor' => 'applications/fact/storage/PhabricatorFactCursor.php',
    'PhabricatorFactDAO' => 'applications/fact/storage/PhabricatorFactDAO.php',
    'PhabricatorFactDaemon' => 'applications/fact/daemon/PhabricatorFactDaemon.php',
    'PhabricatorFactEngine' => 'applications/fact/engine/PhabricatorFactEngine.php',
    'PhabricatorFactEngineTestCase' => 'applications/fact/engine/__tests__/PhabricatorFactEngineTestCase.php',
    'PhabricatorFactHomeController' => 'applications/fact/controller/PhabricatorFactHomeController.php',
    'PhabricatorFactLastUpdatedEngine' => 'applications/fact/engine/PhabricatorFactLastUpdatedEngine.php',
    'PhabricatorFactManagementAnalyzeWorkflow' => 'applications/fact/management/PhabricatorFactManagementAnalyzeWorkflow.php',
    'PhabricatorFactManagementCursorsWorkflow' => 'applications/fact/management/PhabricatorFactManagementCursorsWorkflow.php',
    'PhabricatorFactManagementDestroyWorkflow' => 'applications/fact/management/PhabricatorFactManagementDestroyWorkflow.php',
    'PhabricatorFactManagementListWorkflow' => 'applications/fact/management/PhabricatorFactManagementListWorkflow.php',
    'PhabricatorFactManagementStatusWorkflow' => 'applications/fact/management/PhabricatorFactManagementStatusWorkflow.php',
    'PhabricatorFactManagementWorkflow' => 'applications/fact/management/PhabricatorFactManagementWorkflow.php',
    'PhabricatorFactRaw' => 'applications/fact/storage/PhabricatorFactRaw.php',
    'PhabricatorFactSimpleSpec' => 'applications/fact/spec/PhabricatorFactSimpleSpec.php',
    'PhabricatorFactSpec' => 'applications/fact/spec/PhabricatorFactSpec.php',
    'PhabricatorFactUpdateIterator' => 'applications/fact/extract/PhabricatorFactUpdateIterator.php',
    'PhabricatorFeedApplication' => 'applications/feed/application/PhabricatorFeedApplication.php',
    'PhabricatorFeedBuilder' => 'applications/feed/builder/PhabricatorFeedBuilder.php',
    'PhabricatorFeedConfigOptions' => 'applications/feed/config/PhabricatorFeedConfigOptions.php',
    'PhabricatorFeedController' => 'applications/feed/controller/PhabricatorFeedController.php',
    'PhabricatorFeedDAO' => 'applications/feed/storage/PhabricatorFeedDAO.php',
    'PhabricatorFeedDetailController' => 'applications/feed/controller/PhabricatorFeedDetailController.php',
    'PhabricatorFeedListController' => 'applications/feed/controller/PhabricatorFeedListController.php',
    'PhabricatorFeedManagementRepublishWorkflow' => 'applications/feed/management/PhabricatorFeedManagementRepublishWorkflow.php',
    'PhabricatorFeedManagementWorkflow' => 'applications/feed/management/PhabricatorFeedManagementWorkflow.php',
    'PhabricatorFeedQuery' => 'applications/feed/query/PhabricatorFeedQuery.php',
    'PhabricatorFeedSearchEngine' => 'applications/feed/query/PhabricatorFeedSearchEngine.php',
    'PhabricatorFeedStory' => 'applications/feed/story/PhabricatorFeedStory.php',
    'PhabricatorFeedStoryData' => 'applications/feed/storage/PhabricatorFeedStoryData.php',
    'PhabricatorFeedStoryNotification' => 'applications/notification/storage/PhabricatorFeedStoryNotification.php',
    'PhabricatorFeedStoryPublisher' => 'applications/feed/PhabricatorFeedStoryPublisher.php',
    'PhabricatorFeedStoryReference' => 'applications/feed/storage/PhabricatorFeedStoryReference.php',
    'PhabricatorFile' => 'applications/files/storage/PhabricatorFile.php',
    'PhabricatorFileBundleLoader' => 'applications/files/query/PhabricatorFileBundleLoader.php',
    'PhabricatorFileChunk' => 'applications/files/storage/PhabricatorFileChunk.php',
    'PhabricatorFileChunkIterator' => 'applications/files/engine/PhabricatorFileChunkIterator.php',
    'PhabricatorFileChunkQuery' => 'applications/files/query/PhabricatorFileChunkQuery.php',
    'PhabricatorFileCommentController' => 'applications/files/controller/PhabricatorFileCommentController.php',
    'PhabricatorFileComposeController' => 'applications/files/controller/PhabricatorFileComposeController.php',
    'PhabricatorFileController' => 'applications/files/controller/PhabricatorFileController.php',
    'PhabricatorFileDAO' => 'applications/files/storage/PhabricatorFileDAO.php',
    'PhabricatorFileDataController' => 'applications/files/controller/PhabricatorFileDataController.php',
    'PhabricatorFileDeleteController' => 'applications/files/controller/PhabricatorFileDeleteController.php',
    'PhabricatorFileDropUploadController' => 'applications/files/controller/PhabricatorFileDropUploadController.php',
    'PhabricatorFileEditController' => 'applications/files/controller/PhabricatorFileEditController.php',
    'PhabricatorFileEditor' => 'applications/files/editor/PhabricatorFileEditor.php',
    'PhabricatorFileFilePHIDType' => 'applications/files/phid/PhabricatorFileFilePHIDType.php',
    'PhabricatorFileHasObjectEdgeType' => 'applications/files/edge/PhabricatorFileHasObjectEdgeType.php',
    'PhabricatorFileIconSetSelectController' => 'applications/files/controller/PhabricatorFileIconSetSelectController.php',
    'PhabricatorFileImageMacro' => 'applications/macro/storage/PhabricatorFileImageMacro.php',
    'PhabricatorFileImageTransform' => 'applications/files/transform/PhabricatorFileImageTransform.php',
    'PhabricatorFileInfoController' => 'applications/files/controller/PhabricatorFileInfoController.php',
    'PhabricatorFileLinkView' => 'view/layout/PhabricatorFileLinkView.php',
    'PhabricatorFileListController' => 'applications/files/controller/PhabricatorFileListController.php',
    'PhabricatorFileQuery' => 'applications/files/query/PhabricatorFileQuery.php',
    'PhabricatorFileSchemaSpec' => 'applications/files/storage/PhabricatorFileSchemaSpec.php',
    'PhabricatorFileSearchEngine' => 'applications/files/query/PhabricatorFileSearchEngine.php',
    'PhabricatorFileStorageBlob' => 'applications/files/storage/PhabricatorFileStorageBlob.php',
    'PhabricatorFileStorageConfigurationException' => 'applications/files/exception/PhabricatorFileStorageConfigurationException.php',
    'PhabricatorFileStorageEngine' => 'applications/files/engine/PhabricatorFileStorageEngine.php',
    'PhabricatorFileStorageEngineTestCase' => 'applications/files/engine/__tests__/PhabricatorFileStorageEngineTestCase.php',
    'PhabricatorFileTemporaryGarbageCollector' => 'applications/files/garbagecollector/PhabricatorFileTemporaryGarbageCollector.php',
    'PhabricatorFileTestCase' => 'applications/files/storage/__tests__/PhabricatorFileTestCase.php',
    'PhabricatorFileTestDataGenerator' => 'applications/files/lipsum/PhabricatorFileTestDataGenerator.php',
    'PhabricatorFileThumbnailTransform' => 'applications/files/transform/PhabricatorFileThumbnailTransform.php',
    'PhabricatorFileTransaction' => 'applications/files/storage/PhabricatorFileTransaction.php',
    'PhabricatorFileTransactionComment' => 'applications/files/storage/PhabricatorFileTransactionComment.php',
    'PhabricatorFileTransactionQuery' => 'applications/files/query/PhabricatorFileTransactionQuery.php',
    'PhabricatorFileTransform' => 'applications/files/transform/PhabricatorFileTransform.php',
    'PhabricatorFileTransformController' => 'applications/files/controller/PhabricatorFileTransformController.php',
    'PhabricatorFileTransformListController' => 'applications/files/controller/PhabricatorFileTransformListController.php',
    'PhabricatorFileTransformTestCase' => 'applications/files/transform/__tests__/PhabricatorFileTransformTestCase.php',
    'PhabricatorFileUploadController' => 'applications/files/controller/PhabricatorFileUploadController.php',
    'PhabricatorFileUploadDialogController' => 'applications/files/controller/PhabricatorFileUploadDialogController.php',
    'PhabricatorFileUploadException' => 'applications/files/exception/PhabricatorFileUploadException.php',
    'PhabricatorFileUploadSource' => 'applications/files/uploadsource/PhabricatorFileUploadSource.php',
    'PhabricatorFileinfoSetupCheck' => 'applications/config/check/PhabricatorFileinfoSetupCheck.php',
    'PhabricatorFilesApplication' => 'applications/files/application/PhabricatorFilesApplication.php',
    'PhabricatorFilesApplicationStorageEnginePanel' => 'applications/files/applicationpanel/PhabricatorFilesApplicationStorageEnginePanel.php',
    'PhabricatorFilesBuiltinFile' => 'applications/files/builtin/PhabricatorFilesBuiltinFile.php',
    'PhabricatorFilesComposeIconBuiltinFile' => 'applications/files/builtin/PhabricatorFilesComposeIconBuiltinFile.php',
    'PhabricatorFilesConfigOptions' => 'applications/files/config/PhabricatorFilesConfigOptions.php',
    'PhabricatorFilesManagementCatWorkflow' => 'applications/files/management/PhabricatorFilesManagementCatWorkflow.php',
    'PhabricatorFilesManagementCompactWorkflow' => 'applications/files/management/PhabricatorFilesManagementCompactWorkflow.php',
    'PhabricatorFilesManagementEnginesWorkflow' => 'applications/files/management/PhabricatorFilesManagementEnginesWorkflow.php',
    'PhabricatorFilesManagementMigrateWorkflow' => 'applications/files/management/PhabricatorFilesManagementMigrateWorkflow.php',
    'PhabricatorFilesManagementPurgeWorkflow' => 'applications/files/management/PhabricatorFilesManagementPurgeWorkflow.php',
    'PhabricatorFilesManagementRebuildWorkflow' => 'applications/files/management/PhabricatorFilesManagementRebuildWorkflow.php',
    'PhabricatorFilesManagementWorkflow' => 'applications/files/management/PhabricatorFilesManagementWorkflow.php',
    'PhabricatorFilesOnDiskBuiltinFile' => 'applications/files/builtin/PhabricatorFilesOnDiskBuiltinFile.php',
    'PhabricatorFilesOutboundRequestAction' => 'applications/files/action/PhabricatorFilesOutboundRequestAction.php',
    'PhabricatorFlag' => 'applications/flag/storage/PhabricatorFlag.php',
    'PhabricatorFlagAddFlagHeraldAction' => 'applications/flag/herald/PhabricatorFlagAddFlagHeraldAction.php',
    'PhabricatorFlagColor' => 'applications/flag/constants/PhabricatorFlagColor.php',
    'PhabricatorFlagConstants' => 'applications/flag/constants/PhabricatorFlagConstants.php',
    'PhabricatorFlagController' => 'applications/flag/controller/PhabricatorFlagController.php',
    'PhabricatorFlagDAO' => 'applications/flag/storage/PhabricatorFlagDAO.php',
    'PhabricatorFlagDeleteController' => 'applications/flag/controller/PhabricatorFlagDeleteController.php',
    'PhabricatorFlagDestructionEngineExtension' => 'applications/flag/engineextension/PhabricatorFlagDestructionEngineExtension.php',
    'PhabricatorFlagEditController' => 'applications/flag/controller/PhabricatorFlagEditController.php',
    'PhabricatorFlagListController' => 'applications/flag/controller/PhabricatorFlagListController.php',
    'PhabricatorFlagQuery' => 'applications/flag/query/PhabricatorFlagQuery.php',
    'PhabricatorFlagSearchEngine' => 'applications/flag/query/PhabricatorFlagSearchEngine.php',
    'PhabricatorFlagSelectControl' => 'applications/flag/view/PhabricatorFlagSelectControl.php',
    'PhabricatorFlaggableInterface' => 'applications/flag/interface/PhabricatorFlaggableInterface.php',
    'PhabricatorFlagsApplication' => 'applications/flag/application/PhabricatorFlagsApplication.php',
    'PhabricatorFlagsUIEventListener' => 'applications/flag/events/PhabricatorFlagsUIEventListener.php',
    'PhabricatorFulltextEngine' => 'applications/search/index/PhabricatorFulltextEngine.php',
    'PhabricatorFulltextEngineExtension' => 'applications/search/index/PhabricatorFulltextEngineExtension.php',
    'PhabricatorFulltextEngineExtensionModule' => 'applications/search/index/PhabricatorFulltextEngineExtensionModule.php',
    'PhabricatorFulltextIndexEngineExtension' => 'applications/search/engineextension/PhabricatorFulltextIndexEngineExtension.php',
    'PhabricatorFulltextInterface' => 'applications/search/interface/PhabricatorFulltextInterface.php',
    'PhabricatorFulltextStorageEngine' => 'applications/search/fulltextstorage/PhabricatorFulltextStorageEngine.php',
    'PhabricatorFundApplication' => 'applications/fund/application/PhabricatorFundApplication.php',
    'PhabricatorGDSetupCheck' => 'applications/config/check/PhabricatorGDSetupCheck.php',
    'PhabricatorGarbageCollector' => 'infrastructure/daemon/garbagecollector/PhabricatorGarbageCollector.php',
    'PhabricatorGarbageCollectorManagementCollectWorkflow' => 'infrastructure/daemon/garbagecollector/management/PhabricatorGarbageCollectorManagementCollectWorkflow.php',
    'PhabricatorGarbageCollectorManagementSetPolicyWorkflow' => 'infrastructure/daemon/garbagecollector/management/PhabricatorGarbageCollectorManagementSetPolicyWorkflow.php',
    'PhabricatorGarbageCollectorManagementWorkflow' => 'infrastructure/daemon/garbagecollector/management/PhabricatorGarbageCollectorManagementWorkflow.php',
    'PhabricatorGestureUIExample' => 'applications/uiexample/examples/PhabricatorGestureUIExample.php',
    'PhabricatorGitGraphStream' => 'applications/repository/daemon/PhabricatorGitGraphStream.php',
    'PhabricatorGitHubAuthProvider' => 'applications/auth/provider/PhabricatorGitHubAuthProvider.php',
    'PhabricatorGlobalLock' => 'infrastructure/util/PhabricatorGlobalLock.php',
    'PhabricatorGlobalUploadTargetView' => 'applications/files/view/PhabricatorGlobalUploadTargetView.php',
    'PhabricatorGoogleAuthProvider' => 'applications/auth/provider/PhabricatorGoogleAuthProvider.php',
    'PhabricatorHTTPParameterTypeTableView' => 'applications/config/view/PhabricatorHTTPParameterTypeTableView.php',
    'PhabricatorHandleList' => 'applications/phid/handle/pool/PhabricatorHandleList.php',
    'PhabricatorHandleObjectSelectorDataView' => 'applications/phid/handle/view/PhabricatorHandleObjectSelectorDataView.php',
    'PhabricatorHandlePool' => 'applications/phid/handle/pool/PhabricatorHandlePool.php',
    'PhabricatorHandlePoolTestCase' => 'applications/phid/handle/pool/__tests__/PhabricatorHandlePoolTestCase.php',
    'PhabricatorHandleQuery' => 'applications/phid/query/PhabricatorHandleQuery.php',
    'PhabricatorHandlesEditField' => 'applications/transactions/editfield/PhabricatorHandlesEditField.php',
    'PhabricatorHarbormasterApplication' => 'applications/harbormaster/application/PhabricatorHarbormasterApplication.php',
    'PhabricatorHarbormasterConfigOptions' => 'applications/harbormaster/config/PhabricatorHarbormasterConfigOptions.php',
    'PhabricatorHash' => 'infrastructure/util/PhabricatorHash.php',
    'PhabricatorHashTestCase' => 'infrastructure/util/__tests__/PhabricatorHashTestCase.php',
    'PhabricatorHelpApplication' => 'applications/help/application/PhabricatorHelpApplication.php',
    'PhabricatorHelpController' => 'applications/help/controller/PhabricatorHelpController.php',
    'PhabricatorHelpDocumentationController' => 'applications/help/controller/PhabricatorHelpDocumentationController.php',
    'PhabricatorHelpEditorProtocolController' => 'applications/help/controller/PhabricatorHelpEditorProtocolController.php',
    'PhabricatorHelpKeyboardShortcutController' => 'applications/help/controller/PhabricatorHelpKeyboardShortcutController.php',
    'PhabricatorHelpMainMenuBarExtension' => 'applications/help/extension/PhabricatorHelpMainMenuBarExtension.php',
    'PhabricatorHeraldApplication' => 'applications/herald/application/PhabricatorHeraldApplication.php',
    'PhabricatorHighSecurityRequestExceptionHandler' => 'aphront/handler/PhabricatorHighSecurityRequestExceptionHandler.php',
    'PhabricatorHomeApplication' => 'applications/home/application/PhabricatorHomeApplication.php',
    'PhabricatorHomeController' => 'applications/home/controller/PhabricatorHomeController.php',
    'PhabricatorHomeMainController' => 'applications/home/controller/PhabricatorHomeMainController.php',
    'PhabricatorHomePreferencesSettingsPanel' => 'applications/settings/panel/PhabricatorHomePreferencesSettingsPanel.php',
    'PhabricatorHomeQuickCreateController' => 'applications/home/controller/PhabricatorHomeQuickCreateController.php',
    'PhabricatorHovercardEngineExtension' => 'applications/search/engineextension/PhabricatorHovercardEngineExtension.php',
    'PhabricatorHovercardEngineExtensionModule' => 'applications/search/engineextension/PhabricatorHovercardEngineExtensionModule.php',
    'PhabricatorHovercardUIExample' => 'applications/uiexample/examples/PhabricatorHovercardUIExample.php',
    'PhabricatorHovercardView' => 'view/widget/hovercard/PhabricatorHovercardView.php',
    'PhabricatorHunksManagementMigrateWorkflow' => 'applications/differential/management/PhabricatorHunksManagementMigrateWorkflow.php',
    'PhabricatorHunksManagementWorkflow' => 'applications/differential/management/PhabricatorHunksManagementWorkflow.php',
    'PhabricatorIDsSearchEngineExtension' => 'applications/search/engineextension/PhabricatorIDsSearchEngineExtension.php',
    'PhabricatorIDsSearchField' => 'applications/search/field/PhabricatorIDsSearchField.php',
    'PhabricatorIRCProtocolAdapter' => 'infrastructure/daemon/bot/adapter/PhabricatorIRCProtocolAdapter.php',
    'PhabricatorIconRemarkupRule' => 'applications/macro/markup/PhabricatorIconRemarkupRule.php',
    'PhabricatorIconSet' => 'applications/files/iconset/PhabricatorIconSet.php',
    'PhabricatorIconSetEditField' => 'applications/transactions/editfield/PhabricatorIconSetEditField.php',
    'PhabricatorIconSetIcon' => 'applications/files/iconset/PhabricatorIconSetIcon.php',
    'PhabricatorImageMacroRemarkupRule' => 'applications/macro/markup/PhabricatorImageMacroRemarkupRule.php',
    'PhabricatorImageTransformer' => 'applications/files/PhabricatorImageTransformer.php',
    'PhabricatorImagemagickSetupCheck' => 'applications/config/check/PhabricatorImagemagickSetupCheck.php',
    'PhabricatorIndexEngine' => 'applications/search/index/PhabricatorIndexEngine.php',
    'PhabricatorIndexEngineExtension' => 'applications/search/index/PhabricatorIndexEngineExtension.php',
    'PhabricatorIndexEngineExtensionModule' => 'applications/search/index/PhabricatorIndexEngineExtensionModule.php',
    'PhabricatorInfrastructureTestCase' => '__tests__/PhabricatorInfrastructureTestCase.php',
    'PhabricatorInlineCommentController' => 'infrastructure/diff/PhabricatorInlineCommentController.php',
    'PhabricatorInlineCommentInterface' => 'infrastructure/diff/interface/PhabricatorInlineCommentInterface.php',
    'PhabricatorInlineCommentPreviewController' => 'infrastructure/diff/PhabricatorInlineCommentPreviewController.php',
    'PhabricatorInlineSummaryView' => 'infrastructure/diff/view/PhabricatorInlineSummaryView.php',
    'PhabricatorInstructionsEditField' => 'applications/transactions/editfield/PhabricatorInstructionsEditField.php',
    'PhabricatorInternationalizationManagementExtractWorkflow' => 'infrastructure/internationalization/management/PhabricatorInternationalizationManagementExtractWorkflow.php',
    'PhabricatorInternationalizationManagementWorkflow' => 'infrastructure/internationalization/management/PhabricatorInternationalizationManagementWorkflow.php',
    'PhabricatorInvalidConfigSetupCheck' => 'applications/config/check/PhabricatorInvalidConfigSetupCheck.php',
    'PhabricatorIteratedMD5PasswordHasher' => 'infrastructure/util/password/PhabricatorIteratedMD5PasswordHasher.php',
    'PhabricatorIteratedMD5PasswordHasherTestCase' => 'infrastructure/util/password/__tests__/PhabricatorIteratedMD5PasswordHasherTestCase.php',
    'PhabricatorJIRAAuthProvider' => 'applications/auth/provider/PhabricatorJIRAAuthProvider.php',
    'PhabricatorJavelinLinter' => 'infrastructure/lint/linter/PhabricatorJavelinLinter.php',
    'PhabricatorJiraIssueHasObjectEdgeType' => 'applications/doorkeeper/edge/PhabricatorJiraIssueHasObjectEdgeType.php',
    'PhabricatorJumpNavHandler' => 'applications/search/engine/PhabricatorJumpNavHandler.php',
    'PhabricatorKeyValueDatabaseCache' => 'applications/cache/PhabricatorKeyValueDatabaseCache.php',
    'PhabricatorLDAPAuthProvider' => 'applications/auth/provider/PhabricatorLDAPAuthProvider.php',
    'PhabricatorLegalpadApplication' => 'applications/legalpad/application/PhabricatorLegalpadApplication.php',
    'PhabricatorLegalpadConfigOptions' => 'applications/legalpad/config/PhabricatorLegalpadConfigOptions.php',
    'PhabricatorLegalpadDocumentPHIDType' => 'applications/legalpad/phid/PhabricatorLegalpadDocumentPHIDType.php',
    'PhabricatorLegalpadSignaturePolicyRule' => 'applications/legalpad/policyrule/PhabricatorLegalpadSignaturePolicyRule.php',
    'PhabricatorLibraryTestCase' => '__tests__/PhabricatorLibraryTestCase.php',
    'PhabricatorLinkProfilePanel' => 'applications/search/profilepanel/PhabricatorLinkProfilePanel.php',
    'PhabricatorLipsumArtist' => 'applications/lipsum/image/PhabricatorLipsumArtist.php',
    'PhabricatorLipsumGenerateWorkflow' => 'applications/lipsum/management/PhabricatorLipsumGenerateWorkflow.php',
    'PhabricatorLipsumManagementWorkflow' => 'applications/lipsum/management/PhabricatorLipsumManagementWorkflow.php',
    'PhabricatorLipsumMondrianArtist' => 'applications/lipsum/image/PhabricatorLipsumMondrianArtist.php',
    'PhabricatorLiskDAO' => 'infrastructure/storage/lisk/PhabricatorLiskDAO.php',
    'PhabricatorLiskFulltextEngineExtension' => 'applications/search/engineextension/PhabricatorLiskFulltextEngineExtension.php',
    'PhabricatorLiskSearchEngineExtension' => 'applications/search/engineextension/PhabricatorLiskSearchEngineExtension.php',
    'PhabricatorLiskSerializer' => 'infrastructure/storage/lisk/PhabricatorLiskSerializer.php',
    'PhabricatorListFilterUIExample' => 'applications/uiexample/examples/PhabricatorListFilterUIExample.php',
    'PhabricatorLocalDiskFileStorageEngine' => 'applications/files/engine/PhabricatorLocalDiskFileStorageEngine.php',
    'PhabricatorLocalTimeTestCase' => 'view/__tests__/PhabricatorLocalTimeTestCase.php',
    'PhabricatorLocaleScopeGuard' => 'infrastructure/internationalization/scope/PhabricatorLocaleScopeGuard.php',
    'PhabricatorLocaleScopeGuardTestCase' => 'infrastructure/internationalization/scope/__tests__/PhabricatorLocaleScopeGuardTestCase.php',
    'PhabricatorLogTriggerAction' => 'infrastructure/daemon/workers/action/PhabricatorLogTriggerAction.php',
    'PhabricatorLogoutController' => 'applications/auth/controller/PhabricatorLogoutController.php',
    'PhabricatorLunarPhasePolicyRule' => 'applications/policy/rule/PhabricatorLunarPhasePolicyRule.php',
    'PhabricatorMacroApplication' => 'applications/macro/application/PhabricatorMacroApplication.php',
    'PhabricatorMacroAudioController' => 'applications/macro/controller/PhabricatorMacroAudioController.php',
    'PhabricatorMacroCommentController' => 'applications/macro/controller/PhabricatorMacroCommentController.php',
    'PhabricatorMacroConfigOptions' => 'applications/macro/config/PhabricatorMacroConfigOptions.php',
    'PhabricatorMacroController' => 'applications/macro/controller/PhabricatorMacroController.php',
    'PhabricatorMacroDatasource' => 'applications/macro/typeahead/PhabricatorMacroDatasource.php',
    'PhabricatorMacroDisableController' => 'applications/macro/controller/PhabricatorMacroDisableController.php',
    'PhabricatorMacroEditController' => 'applications/macro/controller/PhabricatorMacroEditController.php',
    'PhabricatorMacroEditor' => 'applications/macro/editor/PhabricatorMacroEditor.php',
    'PhabricatorMacroListController' => 'applications/macro/controller/PhabricatorMacroListController.php',
    'PhabricatorMacroMacroPHIDType' => 'applications/macro/phid/PhabricatorMacroMacroPHIDType.php',
    'PhabricatorMacroMailReceiver' => 'applications/macro/mail/PhabricatorMacroMailReceiver.php',
    'PhabricatorMacroManageCapability' => 'applications/macro/capability/PhabricatorMacroManageCapability.php',
    'PhabricatorMacroMemeController' => 'applications/macro/controller/PhabricatorMacroMemeController.php',
    'PhabricatorMacroMemeDialogController' => 'applications/macro/controller/PhabricatorMacroMemeDialogController.php',
    'PhabricatorMacroQuery' => 'applications/macro/query/PhabricatorMacroQuery.php',
    'PhabricatorMacroReplyHandler' => 'applications/macro/mail/PhabricatorMacroReplyHandler.php',
    'PhabricatorMacroSearchEngine' => 'applications/macro/query/PhabricatorMacroSearchEngine.php',
    'PhabricatorMacroTransaction' => 'applications/macro/storage/PhabricatorMacroTransaction.php',
    'PhabricatorMacroTransactionComment' => 'applications/macro/storage/PhabricatorMacroTransactionComment.php',
    'PhabricatorMacroTransactionQuery' => 'applications/macro/query/PhabricatorMacroTransactionQuery.php',
    'PhabricatorMacroViewController' => 'applications/macro/controller/PhabricatorMacroViewController.php',
    'PhabricatorMailEmailHeraldField' => 'applications/metamta/herald/PhabricatorMailEmailHeraldField.php',
    'PhabricatorMailEmailHeraldFieldGroup' => 'applications/metamta/herald/PhabricatorMailEmailHeraldFieldGroup.php',
    'PhabricatorMailEmailSubjectHeraldField' => 'applications/metamta/herald/PhabricatorMailEmailSubjectHeraldField.php',
    'PhabricatorMailImplementationAdapter' => 'applications/metamta/adapter/PhabricatorMailImplementationAdapter.php',
    'PhabricatorMailImplementationAmazonSESAdapter' => 'applications/metamta/adapter/PhabricatorMailImplementationAmazonSESAdapter.php',
    'PhabricatorMailImplementationMailgunAdapter' => 'applications/metamta/adapter/PhabricatorMailImplementationMailgunAdapter.php',
    'PhabricatorMailImplementationPHPMailerAdapter' => 'applications/metamta/adapter/PhabricatorMailImplementationPHPMailerAdapter.php',
    'PhabricatorMailImplementationPHPMailerLiteAdapter' => 'applications/metamta/adapter/PhabricatorMailImplementationPHPMailerLiteAdapter.php',
    'PhabricatorMailImplementationSendGridAdapter' => 'applications/metamta/adapter/PhabricatorMailImplementationSendGridAdapter.php',
    'PhabricatorMailImplementationTestAdapter' => 'applications/metamta/adapter/PhabricatorMailImplementationTestAdapter.php',
    'PhabricatorMailManagementListInboundWorkflow' => 'applications/metamta/management/PhabricatorMailManagementListInboundWorkflow.php',
    'PhabricatorMailManagementListOutboundWorkflow' => 'applications/metamta/management/PhabricatorMailManagementListOutboundWorkflow.php',
    'PhabricatorMailManagementReceiveTestWorkflow' => 'applications/metamta/management/PhabricatorMailManagementReceiveTestWorkflow.php',
    'PhabricatorMailManagementResendWorkflow' => 'applications/metamta/management/PhabricatorMailManagementResendWorkflow.php',
    'PhabricatorMailManagementSendTestWorkflow' => 'applications/metamta/management/PhabricatorMailManagementSendTestWorkflow.php',
    'PhabricatorMailManagementShowInboundWorkflow' => 'applications/metamta/management/PhabricatorMailManagementShowInboundWorkflow.php',
    'PhabricatorMailManagementShowOutboundWorkflow' => 'applications/metamta/management/PhabricatorMailManagementShowOutboundWorkflow.php',
    'PhabricatorMailManagementVolumeWorkflow' => 'applications/metamta/management/PhabricatorMailManagementVolumeWorkflow.php',
    'PhabricatorMailManagementWorkflow' => 'applications/metamta/management/PhabricatorMailManagementWorkflow.php',
    'PhabricatorMailOutboundMailHeraldAdapter' => 'applications/metamta/herald/PhabricatorMailOutboundMailHeraldAdapter.php',
    'PhabricatorMailOutboundRoutingHeraldAction' => 'applications/metamta/herald/PhabricatorMailOutboundRoutingHeraldAction.php',
    'PhabricatorMailOutboundRoutingSelfEmailHeraldAction' => 'applications/metamta/herald/PhabricatorMailOutboundRoutingSelfEmailHeraldAction.php',
    'PhabricatorMailOutboundRoutingSelfNotificationHeraldAction' => 'applications/metamta/herald/PhabricatorMailOutboundRoutingSelfNotificationHeraldAction.php',
    'PhabricatorMailOutboundStatus' => 'applications/metamta/constants/PhabricatorMailOutboundStatus.php',
    'PhabricatorMailReceiver' => 'applications/metamta/receiver/PhabricatorMailReceiver.php',
    'PhabricatorMailReceiverTestCase' => 'applications/metamta/receiver/__tests__/PhabricatorMailReceiverTestCase.php',
    'PhabricatorMailReplyHandler' => 'applications/metamta/replyhandler/PhabricatorMailReplyHandler.php',
    'PhabricatorMailRoutingRule' => 'applications/metamta/constants/PhabricatorMailRoutingRule.php',
    'PhabricatorMailSetupCheck' => 'applications/config/check/PhabricatorMailSetupCheck.php',
    'PhabricatorMailTarget' => 'applications/metamta/replyhandler/PhabricatorMailTarget.php',
    'PhabricatorMailgunConfigOptions' => 'applications/config/option/PhabricatorMailgunConfigOptions.php',
    'PhabricatorMainMenuBarExtension' => 'view/page/menu/PhabricatorMainMenuBarExtension.php',
    'PhabricatorMainMenuSearchView' => 'view/page/menu/PhabricatorMainMenuSearchView.php',
    'PhabricatorMainMenuView' => 'view/page/menu/PhabricatorMainMenuView.php',
    'PhabricatorManagementWorkflow' => 'infrastructure/management/PhabricatorManagementWorkflow.php',
    'PhabricatorManiphestApplication' => 'applications/maniphest/application/PhabricatorManiphestApplication.php',
    'PhabricatorManiphestConfigOptions' => 'applications/maniphest/config/PhabricatorManiphestConfigOptions.php',
    'PhabricatorManiphestTaskTestDataGenerator' => 'applications/maniphest/lipsum/PhabricatorManiphestTaskTestDataGenerator.php',
    'PhabricatorMarkupCache' => 'applications/cache/storage/PhabricatorMarkupCache.php',
    'PhabricatorMarkupEngine' => 'infrastructure/markup/PhabricatorMarkupEngine.php',
    'PhabricatorMarkupInterface' => 'infrastructure/markup/PhabricatorMarkupInterface.php',
    'PhabricatorMarkupOneOff' => 'infrastructure/markup/PhabricatorMarkupOneOff.php',
    'PhabricatorMarkupPreviewController' => 'infrastructure/markup/PhabricatorMarkupPreviewController.php',
    'PhabricatorMemeRemarkupRule' => 'applications/macro/markup/PhabricatorMemeRemarkupRule.php',
    'PhabricatorMentionRemarkupRule' => 'applications/people/markup/PhabricatorMentionRemarkupRule.php',
    'PhabricatorMentionableInterface' => 'applications/transactions/interface/PhabricatorMentionableInterface.php',
    'PhabricatorMercurialGraphStream' => 'applications/repository/daemon/PhabricatorMercurialGraphStream.php',
    'PhabricatorMetaMTAActor' => 'applications/metamta/query/PhabricatorMetaMTAActor.php',
    'PhabricatorMetaMTAActorQuery' => 'applications/metamta/query/PhabricatorMetaMTAActorQuery.php',
    'PhabricatorMetaMTAApplication' => 'applications/metamta/application/PhabricatorMetaMTAApplication.php',
    'PhabricatorMetaMTAApplicationEmail' => 'applications/metamta/storage/PhabricatorMetaMTAApplicationEmail.php',
    'PhabricatorMetaMTAApplicationEmailDatasource' => 'applications/metamta/typeahead/PhabricatorMetaMTAApplicationEmailDatasource.php',
    'PhabricatorMetaMTAApplicationEmailEditor' => 'applications/metamta/editor/PhabricatorMetaMTAApplicationEmailEditor.php',
    'PhabricatorMetaMTAApplicationEmailHeraldField' => 'applications/metamta/herald/PhabricatorMetaMTAApplicationEmailHeraldField.php',
    'PhabricatorMetaMTAApplicationEmailPHIDType' => 'applications/phid/PhabricatorMetaMTAApplicationEmailPHIDType.php',
    'PhabricatorMetaMTAApplicationEmailPanel' => 'applications/metamta/applicationpanel/PhabricatorMetaMTAApplicationEmailPanel.php',
    'PhabricatorMetaMTAApplicationEmailQuery' => 'applications/metamta/query/PhabricatorMetaMTAApplicationEmailQuery.php',
    'PhabricatorMetaMTAApplicationEmailTransaction' => 'applications/metamta/storage/PhabricatorMetaMTAApplicationEmailTransaction.php',
    'PhabricatorMetaMTAApplicationEmailTransactionQuery' => 'applications/metamta/query/PhabricatorMetaMTAApplicationEmailTransactionQuery.php',
    'PhabricatorMetaMTAAttachment' => 'applications/metamta/storage/PhabricatorMetaMTAAttachment.php',
    'PhabricatorMetaMTAConfigOptions' => 'applications/config/option/PhabricatorMetaMTAConfigOptions.php',
    'PhabricatorMetaMTAController' => 'applications/metamta/controller/PhabricatorMetaMTAController.php',
    'PhabricatorMetaMTADAO' => 'applications/metamta/storage/PhabricatorMetaMTADAO.php',
    'PhabricatorMetaMTAEmailBodyParser' => 'applications/metamta/parser/PhabricatorMetaMTAEmailBodyParser.php',
    'PhabricatorMetaMTAEmailBodyParserTestCase' => 'applications/metamta/parser/__tests__/PhabricatorMetaMTAEmailBodyParserTestCase.php',
    'PhabricatorMetaMTAEmailHeraldAction' => 'applications/metamta/herald/PhabricatorMetaMTAEmailHeraldAction.php',
    'PhabricatorMetaMTAEmailOthersHeraldAction' => 'applications/metamta/herald/PhabricatorMetaMTAEmailOthersHeraldAction.php',
    'PhabricatorMetaMTAEmailSelfHeraldAction' => 'applications/metamta/herald/PhabricatorMetaMTAEmailSelfHeraldAction.php',
    'PhabricatorMetaMTAErrorMailAction' => 'applications/metamta/action/PhabricatorMetaMTAErrorMailAction.php',
    'PhabricatorMetaMTAMail' => 'applications/metamta/storage/PhabricatorMetaMTAMail.php',
    'PhabricatorMetaMTAMailBody' => 'applications/metamta/view/PhabricatorMetaMTAMailBody.php',
    'PhabricatorMetaMTAMailBodyTestCase' => 'applications/metamta/view/__tests__/PhabricatorMetaMTAMailBodyTestCase.php',
    'PhabricatorMetaMTAMailHasRecipientEdgeType' => 'applications/metamta/edge/PhabricatorMetaMTAMailHasRecipientEdgeType.php',
    'PhabricatorMetaMTAMailListController' => 'applications/metamta/controller/PhabricatorMetaMTAMailListController.php',
    'PhabricatorMetaMTAMailPHIDType' => 'applications/metamta/phid/PhabricatorMetaMTAMailPHIDType.php',
    'PhabricatorMetaMTAMailQuery' => 'applications/metamta/query/PhabricatorMetaMTAMailQuery.php',
    'PhabricatorMetaMTAMailSearchEngine' => 'applications/metamta/query/PhabricatorMetaMTAMailSearchEngine.php',
    'PhabricatorMetaMTAMailSection' => 'applications/metamta/view/PhabricatorMetaMTAMailSection.php',
    'PhabricatorMetaMTAMailTestCase' => 'applications/metamta/storage/__tests__/PhabricatorMetaMTAMailTestCase.php',
    'PhabricatorMetaMTAMailViewController' => 'applications/metamta/controller/PhabricatorMetaMTAMailViewController.php',
    'PhabricatorMetaMTAMailableDatasource' => 'applications/metamta/typeahead/PhabricatorMetaMTAMailableDatasource.php',
    'PhabricatorMetaMTAMailableFunctionDatasource' => 'applications/metamta/typeahead/PhabricatorMetaMTAMailableFunctionDatasource.php',
    'PhabricatorMetaMTAMailgunReceiveController' => 'applications/metamta/controller/PhabricatorMetaMTAMailgunReceiveController.php',
    'PhabricatorMetaMTAMemberQuery' => 'applications/metamta/query/PhabricatorMetaMTAMemberQuery.php',
    'PhabricatorMetaMTAPermanentFailureException' => 'applications/metamta/exception/PhabricatorMetaMTAPermanentFailureException.php',
    'PhabricatorMetaMTAReceivedMail' => 'applications/metamta/storage/PhabricatorMetaMTAReceivedMail.php',
    'PhabricatorMetaMTAReceivedMailProcessingException' => 'applications/metamta/exception/PhabricatorMetaMTAReceivedMailProcessingException.php',
    'PhabricatorMetaMTAReceivedMailTestCase' => 'applications/metamta/storage/__tests__/PhabricatorMetaMTAReceivedMailTestCase.php',
    'PhabricatorMetaMTASchemaSpec' => 'applications/metamta/storage/PhabricatorMetaMTASchemaSpec.php',
    'PhabricatorMetaMTASendGridReceiveController' => 'applications/metamta/controller/PhabricatorMetaMTASendGridReceiveController.php',
    'PhabricatorMetaMTAWorker' => 'applications/metamta/PhabricatorMetaMTAWorker.php',
    'PhabricatorMetronomicTriggerClock' => 'infrastructure/daemon/workers/clock/PhabricatorMetronomicTriggerClock.php',
    'PhabricatorMotivatorProfilePanel' => 'applications/search/profilepanel/PhabricatorMotivatorProfilePanel.php',
    'PhabricatorMultiColumnUIExample' => 'applications/uiexample/examples/PhabricatorMultiColumnUIExample.php',
    'PhabricatorMultiFactorSettingsPanel' => 'applications/settings/panel/PhabricatorMultiFactorSettingsPanel.php',
    'PhabricatorMultimeterApplication' => 'applications/multimeter/application/PhabricatorMultimeterApplication.php',
    'PhabricatorMustVerifyEmailController' => 'applications/auth/controller/PhabricatorMustVerifyEmailController.php',
    'PhabricatorMySQLConfigOptions' => 'applications/config/option/PhabricatorMySQLConfigOptions.php',
    'PhabricatorMySQLFileStorageEngine' => 'applications/files/engine/PhabricatorMySQLFileStorageEngine.php',
    'PhabricatorMySQLFulltextStorageEngine' => 'applications/search/fulltextstorage/PhabricatorMySQLFulltextStorageEngine.php',
    'PhabricatorMySQLSetupCheck' => 'applications/config/check/PhabricatorMySQLSetupCheck.php',
    'PhabricatorNamedQuery' => 'applications/search/storage/PhabricatorNamedQuery.php',
    'PhabricatorNamedQueryQuery' => 'applications/search/query/PhabricatorNamedQueryQuery.php',
    'PhabricatorNavigationRemarkupRule' => 'infrastructure/markup/rule/PhabricatorNavigationRemarkupRule.php',
    'PhabricatorNeverTriggerClock' => 'infrastructure/daemon/workers/clock/PhabricatorNeverTriggerClock.php',
    'PhabricatorNgramsIndexEngineExtension' => 'applications/search/engineextension/PhabricatorNgramsIndexEngineExtension.php',
    'PhabricatorNgramsInterface' => 'applications/search/interface/PhabricatorNgramsInterface.php',
    'PhabricatorNotificationBuilder' => 'applications/notification/builder/PhabricatorNotificationBuilder.php',
    'PhabricatorNotificationClearController' => 'applications/notification/controller/PhabricatorNotificationClearController.php',
    'PhabricatorNotificationClient' => 'applications/notification/client/PhabricatorNotificationClient.php',
    'PhabricatorNotificationConfigOptions' => 'applications/config/option/PhabricatorNotificationConfigOptions.php',
    'PhabricatorNotificationController' => 'applications/notification/controller/PhabricatorNotificationController.php',
    'PhabricatorNotificationDestructionEngineExtension' => 'applications/notification/engineextension/PhabricatorNotificationDestructionEngineExtension.php',
    'PhabricatorNotificationIndividualController' => 'applications/notification/controller/PhabricatorNotificationIndividualController.php',
    'PhabricatorNotificationListController' => 'applications/notification/controller/PhabricatorNotificationListController.php',
    'PhabricatorNotificationPanelController' => 'applications/notification/controller/PhabricatorNotificationPanelController.php',
    'PhabricatorNotificationQuery' => 'applications/notification/query/PhabricatorNotificationQuery.php',
    'PhabricatorNotificationSearchEngine' => 'applications/notification/query/PhabricatorNotificationSearchEngine.php',
    'PhabricatorNotificationStatusController' => 'applications/notification/controller/PhabricatorNotificationStatusController.php',
    'PhabricatorNotificationStatusView' => 'applications/notification/view/PhabricatorNotificationStatusView.php',
    'PhabricatorNotificationTestController' => 'applications/notification/controller/PhabricatorNotificationTestController.php',
    'PhabricatorNotificationTestFeedStory' => 'applications/notification/feed/PhabricatorNotificationTestFeedStory.php',
    'PhabricatorNotificationUIExample' => 'applications/uiexample/examples/PhabricatorNotificationUIExample.php',
    'PhabricatorNotificationsApplication' => 'applications/notification/application/PhabricatorNotificationsApplication.php',
    'PhabricatorNuanceApplication' => 'applications/nuance/application/PhabricatorNuanceApplication.php',
    'PhabricatorOAuth1AuthProvider' => 'applications/auth/provider/PhabricatorOAuth1AuthProvider.php',
    'PhabricatorOAuth2AuthProvider' => 'applications/auth/provider/PhabricatorOAuth2AuthProvider.php',
    'PhabricatorOAuthAuthProvider' => 'applications/auth/provider/PhabricatorOAuthAuthProvider.php',
    'PhabricatorOAuthClientAuthorization' => 'applications/oauthserver/storage/PhabricatorOAuthClientAuthorization.php',
    'PhabricatorOAuthClientAuthorizationQuery' => 'applications/oauthserver/query/PhabricatorOAuthClientAuthorizationQuery.php',
    'PhabricatorOAuthClientController' => 'applications/oauthserver/controller/client/PhabricatorOAuthClientController.php',
    'PhabricatorOAuthClientDeleteController' => 'applications/oauthserver/controller/client/PhabricatorOAuthClientDeleteController.php',
    'PhabricatorOAuthClientEditController' => 'applications/oauthserver/controller/client/PhabricatorOAuthClientEditController.php',
    'PhabricatorOAuthClientListController' => 'applications/oauthserver/controller/client/PhabricatorOAuthClientListController.php',
    'PhabricatorOAuthClientSecretController' => 'applications/oauthserver/controller/client/PhabricatorOAuthClientSecretController.php',
    'PhabricatorOAuthClientViewController' => 'applications/oauthserver/controller/client/PhabricatorOAuthClientViewController.php',
    'PhabricatorOAuthResponse' => 'applications/oauthserver/PhabricatorOAuthResponse.php',
    'PhabricatorOAuthServer' => 'applications/oauthserver/PhabricatorOAuthServer.php',
    'PhabricatorOAuthServerAccessToken' => 'applications/oauthserver/storage/PhabricatorOAuthServerAccessToken.php',
    'PhabricatorOAuthServerApplication' => 'applications/oauthserver/application/PhabricatorOAuthServerApplication.php',
    'PhabricatorOAuthServerAuthController' => 'applications/oauthserver/controller/PhabricatorOAuthServerAuthController.php',
    'PhabricatorOAuthServerAuthorizationCode' => 'applications/oauthserver/storage/PhabricatorOAuthServerAuthorizationCode.php',
    'PhabricatorOAuthServerAuthorizationsSettingsPanel' => 'applications/oauthserver/panel/PhabricatorOAuthServerAuthorizationsSettingsPanel.php',
    'PhabricatorOAuthServerClient' => 'applications/oauthserver/storage/PhabricatorOAuthServerClient.php',
    'PhabricatorOAuthServerClientAuthorizationPHIDType' => 'applications/oauthserver/phid/PhabricatorOAuthServerClientAuthorizationPHIDType.php',
    'PhabricatorOAuthServerClientPHIDType' => 'applications/oauthserver/phid/PhabricatorOAuthServerClientPHIDType.php',
    'PhabricatorOAuthServerClientQuery' => 'applications/oauthserver/query/PhabricatorOAuthServerClientQuery.php',
    'PhabricatorOAuthServerClientSearchEngine' => 'applications/oauthserver/query/PhabricatorOAuthServerClientSearchEngine.php',
    'PhabricatorOAuthServerController' => 'applications/oauthserver/controller/PhabricatorOAuthServerController.php',
    'PhabricatorOAuthServerCreateClientsCapability' => 'applications/oauthserver/capability/PhabricatorOAuthServerCreateClientsCapability.php',
    'PhabricatorOAuthServerDAO' => 'applications/oauthserver/storage/PhabricatorOAuthServerDAO.php',
    'PhabricatorOAuthServerScope' => 'applications/oauthserver/PhabricatorOAuthServerScope.php',
    'PhabricatorOAuthServerTestCase' => 'applications/oauthserver/__tests__/PhabricatorOAuthServerTestCase.php',
    'PhabricatorOAuthServerTestController' => 'applications/oauthserver/controller/PhabricatorOAuthServerTestController.php',
    'PhabricatorOAuthServerTokenController' => 'applications/oauthserver/controller/PhabricatorOAuthServerTokenController.php',
    'PhabricatorObjectHandle' => 'applications/phid/PhabricatorObjectHandle.php',
    'PhabricatorObjectHasAsanaSubtaskEdgeType' => 'applications/doorkeeper/edge/PhabricatorObjectHasAsanaSubtaskEdgeType.php',
    'PhabricatorObjectHasAsanaTaskEdgeType' => 'applications/doorkeeper/edge/PhabricatorObjectHasAsanaTaskEdgeType.php',
    'PhabricatorObjectHasContributorEdgeType' => 'applications/transactions/edges/PhabricatorObjectHasContributorEdgeType.php',
    'PhabricatorObjectHasFileEdgeType' => 'applications/transactions/edges/PhabricatorObjectHasFileEdgeType.php',
    'PhabricatorObjectHasJiraIssueEdgeType' => 'applications/doorkeeper/edge/PhabricatorObjectHasJiraIssueEdgeType.php',
    'PhabricatorObjectHasSubscriberEdgeType' => 'applications/transactions/edges/PhabricatorObjectHasSubscriberEdgeType.php',
    'PhabricatorObjectHasUnsubscriberEdgeType' => 'applications/transactions/edges/PhabricatorObjectHasUnsubscriberEdgeType.php',
    'PhabricatorObjectHasWatcherEdgeType' => 'applications/transactions/edges/PhabricatorObjectHasWatcherEdgeType.php',
    'PhabricatorObjectListQuery' => 'applications/phid/query/PhabricatorObjectListQuery.php',
    'PhabricatorObjectListQueryTestCase' => 'applications/phid/query/__tests__/PhabricatorObjectListQueryTestCase.php',
    'PhabricatorObjectMailReceiver' => 'applications/metamta/receiver/PhabricatorObjectMailReceiver.php',
    'PhabricatorObjectMailReceiverTestCase' => 'applications/metamta/receiver/__tests__/PhabricatorObjectMailReceiverTestCase.php',
    'PhabricatorObjectMentionedByObjectEdgeType' => 'applications/transactions/edges/PhabricatorObjectMentionedByObjectEdgeType.php',
    'PhabricatorObjectMentionsObjectEdgeType' => 'applications/transactions/edges/PhabricatorObjectMentionsObjectEdgeType.php',
    'PhabricatorObjectQuery' => 'applications/phid/query/PhabricatorObjectQuery.php',
    'PhabricatorObjectRemarkupRule' => 'infrastructure/markup/rule/PhabricatorObjectRemarkupRule.php',
    'PhabricatorObjectSelectorDialog' => 'view/control/PhabricatorObjectSelectorDialog.php',
    'PhabricatorObjectUsesCredentialsEdgeType' => 'applications/transactions/edges/PhabricatorObjectUsesCredentialsEdgeType.php',
    'PhabricatorOffsetPagedQuery' => 'infrastructure/query/PhabricatorOffsetPagedQuery.php',
    'PhabricatorOneTimeTriggerClock' => 'infrastructure/daemon/workers/clock/PhabricatorOneTimeTriggerClock.php',
    'PhabricatorOpcodeCacheSpec' => 'applications/cache/spec/PhabricatorOpcodeCacheSpec.php',
    'PhabricatorOwnerPathQuery' => 'applications/owners/query/PhabricatorOwnerPathQuery.php',
    'PhabricatorOwnersApplication' => 'applications/owners/application/PhabricatorOwnersApplication.php',
    'PhabricatorOwnersArchiveController' => 'applications/owners/controller/PhabricatorOwnersArchiveController.php',
    'PhabricatorOwnersConfigOptions' => 'applications/owners/config/PhabricatorOwnersConfigOptions.php',
    'PhabricatorOwnersConfiguredCustomField' => 'applications/owners/customfield/PhabricatorOwnersConfiguredCustomField.php',
    'PhabricatorOwnersController' => 'applications/owners/controller/PhabricatorOwnersController.php',
    'PhabricatorOwnersCustomField' => 'applications/owners/customfield/PhabricatorOwnersCustomField.php',
    'PhabricatorOwnersCustomFieldNumericIndex' => 'applications/owners/storage/PhabricatorOwnersCustomFieldNumericIndex.php',
    'PhabricatorOwnersCustomFieldStorage' => 'applications/owners/storage/PhabricatorOwnersCustomFieldStorage.php',
    'PhabricatorOwnersCustomFieldStringIndex' => 'applications/owners/storage/PhabricatorOwnersCustomFieldStringIndex.php',
    'PhabricatorOwnersDAO' => 'applications/owners/storage/PhabricatorOwnersDAO.php',
    'PhabricatorOwnersDetailController' => 'applications/owners/controller/PhabricatorOwnersDetailController.php',
    'PhabricatorOwnersEditController' => 'applications/owners/controller/PhabricatorOwnersEditController.php',
    'PhabricatorOwnersListController' => 'applications/owners/controller/PhabricatorOwnersListController.php',
    'PhabricatorOwnersOwner' => 'applications/owners/storage/PhabricatorOwnersOwner.php',
    'PhabricatorOwnersPackage' => 'applications/owners/storage/PhabricatorOwnersPackage.php',
    'PhabricatorOwnersPackageDatasource' => 'applications/owners/typeahead/PhabricatorOwnersPackageDatasource.php',
    'PhabricatorOwnersPackageEditEngine' => 'applications/owners/editor/PhabricatorOwnersPackageEditEngine.php',
    'PhabricatorOwnersPackageFulltextEngine' => 'applications/owners/query/PhabricatorOwnersPackageFulltextEngine.php',
    'PhabricatorOwnersPackageFunctionDatasource' => 'applications/owners/typeahead/PhabricatorOwnersPackageFunctionDatasource.php',
    'PhabricatorOwnersPackageNameNgrams' => 'applications/owners/storage/PhabricatorOwnersPackageNameNgrams.php',
    'PhabricatorOwnersPackageOwnerDatasource' => 'applications/owners/typeahead/PhabricatorOwnersPackageOwnerDatasource.php',
    'PhabricatorOwnersPackagePHIDType' => 'applications/owners/phid/PhabricatorOwnersPackagePHIDType.php',
    'PhabricatorOwnersPackageQuery' => 'applications/owners/query/PhabricatorOwnersPackageQuery.php',
    'PhabricatorOwnersPackageSearchEngine' => 'applications/owners/query/PhabricatorOwnersPackageSearchEngine.php',
    'PhabricatorOwnersPackageTestCase' => 'applications/owners/storage/__tests__/PhabricatorOwnersPackageTestCase.php',
    'PhabricatorOwnersPackageTransaction' => 'applications/owners/storage/PhabricatorOwnersPackageTransaction.php',
    'PhabricatorOwnersPackageTransactionEditor' => 'applications/owners/editor/PhabricatorOwnersPackageTransactionEditor.php',
    'PhabricatorOwnersPackageTransactionQuery' => 'applications/owners/query/PhabricatorOwnersPackageTransactionQuery.php',
    'PhabricatorOwnersPath' => 'applications/owners/storage/PhabricatorOwnersPath.php',
    'PhabricatorOwnersPathsController' => 'applications/owners/controller/PhabricatorOwnersPathsController.php',
    'PhabricatorOwnersPathsSearchEngineAttachment' => 'applications/owners/engineextension/PhabricatorOwnersPathsSearchEngineAttachment.php',
    'PhabricatorOwnersSchemaSpec' => 'applications/owners/storage/PhabricatorOwnersSchemaSpec.php',
    'PhabricatorOwnersSearchField' => 'applications/owners/searchfield/PhabricatorOwnersSearchField.php',
    'PhabricatorPHDConfigOptions' => 'applications/config/option/PhabricatorPHDConfigOptions.php',
    'PhabricatorPHID' => 'applications/phid/storage/PhabricatorPHID.php',
    'PhabricatorPHIDConstants' => 'applications/phid/PhabricatorPHIDConstants.php',
    'PhabricatorPHIDInterface' => 'applications/phid/interface/PhabricatorPHIDInterface.php',
    'PhabricatorPHIDListEditField' => 'applications/transactions/editfield/PhabricatorPHIDListEditField.php',
    'PhabricatorPHIDListEditType' => 'applications/transactions/edittype/PhabricatorPHIDListEditType.php',
    'PhabricatorPHIDResolver' => 'applications/phid/resolver/PhabricatorPHIDResolver.php',
    'PhabricatorPHIDType' => 'applications/phid/type/PhabricatorPHIDType.php',
    'PhabricatorPHIDTypeTestCase' => 'applications/phid/type/__tests__/PhabricatorPHIDTypeTestCase.php',
    'PhabricatorPHIDsSearchField' => 'applications/search/field/PhabricatorPHIDsSearchField.php',
    'PhabricatorPHPASTApplication' => 'applications/phpast/application/PhabricatorPHPASTApplication.php',
    'PhabricatorPHPConfigSetupCheck' => 'applications/config/check/PhabricatorPHPConfigSetupCheck.php',
    'PhabricatorPHPMailerConfigOptions' => 'applications/config/option/PhabricatorPHPMailerConfigOptions.php',
    'PhabricatorPagedFormUIExample' => 'applications/uiexample/examples/PhabricatorPagedFormUIExample.php',
    'PhabricatorPagerUIExample' => 'applications/uiexample/examples/PhabricatorPagerUIExample.php',
    'PhabricatorPassphraseApplication' => 'applications/passphrase/application/PhabricatorPassphraseApplication.php',
    'PhabricatorPasswordAuthProvider' => 'applications/auth/provider/PhabricatorPasswordAuthProvider.php',
    'PhabricatorPasswordHasher' => 'infrastructure/util/password/PhabricatorPasswordHasher.php',
    'PhabricatorPasswordHasherTestCase' => 'infrastructure/util/password/__tests__/PhabricatorPasswordHasherTestCase.php',
    'PhabricatorPasswordHasherUnavailableException' => 'infrastructure/util/password/PhabricatorPasswordHasherUnavailableException.php',
    'PhabricatorPasswordSettingsPanel' => 'applications/settings/panel/PhabricatorPasswordSettingsPanel.php',
    'PhabricatorPaste' => 'applications/paste/storage/PhabricatorPaste.php',
    'PhabricatorPasteApplication' => 'applications/paste/application/PhabricatorPasteApplication.php',
    'PhabricatorPasteArchiveController' => 'applications/paste/controller/PhabricatorPasteArchiveController.php',
    'PhabricatorPasteConfigOptions' => 'applications/paste/config/PhabricatorPasteConfigOptions.php',
    'PhabricatorPasteContentSearchEngineAttachment' => 'applications/paste/engineextension/PhabricatorPasteContentSearchEngineAttachment.php',
    'PhabricatorPasteController' => 'applications/paste/controller/PhabricatorPasteController.php',
    'PhabricatorPasteDAO' => 'applications/paste/storage/PhabricatorPasteDAO.php',
    'PhabricatorPasteEditController' => 'applications/paste/controller/PhabricatorPasteEditController.php',
    'PhabricatorPasteEditEngine' => 'applications/paste/editor/PhabricatorPasteEditEngine.php',
    'PhabricatorPasteEditor' => 'applications/paste/editor/PhabricatorPasteEditor.php',
    'PhabricatorPasteFilenameContextFreeGrammar' => 'applications/paste/lipsum/PhabricatorPasteFilenameContextFreeGrammar.php',
    'PhabricatorPasteListController' => 'applications/paste/controller/PhabricatorPasteListController.php',
    'PhabricatorPastePastePHIDType' => 'applications/paste/phid/PhabricatorPastePastePHIDType.php',
    'PhabricatorPasteQuery' => 'applications/paste/query/PhabricatorPasteQuery.php',
    'PhabricatorPasteRawController' => 'applications/paste/controller/PhabricatorPasteRawController.php',
    'PhabricatorPasteRemarkupRule' => 'applications/paste/remarkup/PhabricatorPasteRemarkupRule.php',
    'PhabricatorPasteSchemaSpec' => 'applications/paste/storage/PhabricatorPasteSchemaSpec.php',
    'PhabricatorPasteSearchEngine' => 'applications/paste/query/PhabricatorPasteSearchEngine.php',
    'PhabricatorPasteSnippet' => 'applications/paste/snippet/PhabricatorPasteSnippet.php',
    'PhabricatorPasteTestDataGenerator' => 'applications/paste/lipsum/PhabricatorPasteTestDataGenerator.php',
    'PhabricatorPasteTransaction' => 'applications/paste/storage/PhabricatorPasteTransaction.php',
    'PhabricatorPasteTransactionComment' => 'applications/paste/storage/PhabricatorPasteTransactionComment.php',
    'PhabricatorPasteTransactionQuery' => 'applications/paste/query/PhabricatorPasteTransactionQuery.php',
    'PhabricatorPasteViewController' => 'applications/paste/controller/PhabricatorPasteViewController.php',
    'PhabricatorPathSetupCheck' => 'applications/config/check/PhabricatorPathSetupCheck.php',
    'PhabricatorPeopleAnyOwnerDatasource' => 'applications/people/typeahead/PhabricatorPeopleAnyOwnerDatasource.php',
    'PhabricatorPeopleApplication' => 'applications/people/application/PhabricatorPeopleApplication.php',
    'PhabricatorPeopleApproveController' => 'applications/people/controller/PhabricatorPeopleApproveController.php',
    'PhabricatorPeopleCalendarController' => 'applications/people/controller/PhabricatorPeopleCalendarController.php',
    'PhabricatorPeopleController' => 'applications/people/controller/PhabricatorPeopleController.php',
    'PhabricatorPeopleCreateController' => 'applications/people/controller/PhabricatorPeopleCreateController.php',
    'PhabricatorPeopleDatasource' => 'applications/people/typeahead/PhabricatorPeopleDatasource.php',
    'PhabricatorPeopleDeleteController' => 'applications/people/controller/PhabricatorPeopleDeleteController.php',
    'PhabricatorPeopleDetailsProfilePanel' => 'applications/people/profilepanel/PhabricatorPeopleDetailsProfilePanel.php',
    'PhabricatorPeopleDisableController' => 'applications/people/controller/PhabricatorPeopleDisableController.php',
    'PhabricatorPeopleEmpowerController' => 'applications/people/controller/PhabricatorPeopleEmpowerController.php',
    'PhabricatorPeopleExternalPHIDType' => 'applications/people/phid/PhabricatorPeopleExternalPHIDType.php',
    'PhabricatorPeopleHovercardEngineExtension' => 'applications/people/engineextension/PhabricatorPeopleHovercardEngineExtension.php',
    'PhabricatorPeopleInviteController' => 'applications/people/controller/PhabricatorPeopleInviteController.php',
    'PhabricatorPeopleInviteListController' => 'applications/people/controller/PhabricatorPeopleInviteListController.php',
    'PhabricatorPeopleInviteSendController' => 'applications/people/controller/PhabricatorPeopleInviteSendController.php',
    'PhabricatorPeopleLdapController' => 'applications/people/controller/PhabricatorPeopleLdapController.php',
    'PhabricatorPeopleListController' => 'applications/people/controller/PhabricatorPeopleListController.php',
    'PhabricatorPeopleLogQuery' => 'applications/people/query/PhabricatorPeopleLogQuery.php',
    'PhabricatorPeopleLogSearchEngine' => 'applications/people/query/PhabricatorPeopleLogSearchEngine.php',
    'PhabricatorPeopleLogsController' => 'applications/people/controller/PhabricatorPeopleLogsController.php',
    'PhabricatorPeopleMainMenuBarExtension' => 'applications/people/extension/PhabricatorPeopleMainMenuBarExtension.php',
    'PhabricatorPeopleNewController' => 'applications/people/controller/PhabricatorPeopleNewController.php',
    'PhabricatorPeopleNoOwnerDatasource' => 'applications/people/typeahead/PhabricatorPeopleNoOwnerDatasource.php',
    'PhabricatorPeopleOwnerDatasource' => 'applications/people/typeahead/PhabricatorPeopleOwnerDatasource.php',
    'PhabricatorPeopleProfileController' => 'applications/people/controller/PhabricatorPeopleProfileController.php',
    'PhabricatorPeopleProfileEditController' => 'applications/people/controller/PhabricatorPeopleProfileEditController.php',
    'PhabricatorPeopleProfilePanelEngine' => 'applications/people/engine/PhabricatorPeopleProfilePanelEngine.php',
    'PhabricatorPeopleProfilePictureController' => 'applications/people/controller/PhabricatorPeopleProfilePictureController.php',
    'PhabricatorPeopleProfileViewController' => 'applications/people/controller/PhabricatorPeopleProfileViewController.php',
    'PhabricatorPeopleQuery' => 'applications/people/query/PhabricatorPeopleQuery.php',
    'PhabricatorPeopleRenameController' => 'applications/people/controller/PhabricatorPeopleRenameController.php',
    'PhabricatorPeopleSearchEngine' => 'applications/people/query/PhabricatorPeopleSearchEngine.php',
    'PhabricatorPeopleTestDataGenerator' => 'applications/people/lipsum/PhabricatorPeopleTestDataGenerator.php',
    'PhabricatorPeopleTransactionQuery' => 'applications/people/query/PhabricatorPeopleTransactionQuery.php',
    'PhabricatorPeopleUserFunctionDatasource' => 'applications/people/typeahead/PhabricatorPeopleUserFunctionDatasource.php',
    'PhabricatorPeopleUserPHIDType' => 'applications/people/phid/PhabricatorPeopleUserPHIDType.php',
    'PhabricatorPeopleWelcomeController' => 'applications/people/controller/PhabricatorPeopleWelcomeController.php',
    'PhabricatorPersonaAuthProvider' => 'applications/auth/provider/PhabricatorPersonaAuthProvider.php',
    'PhabricatorPhabricatorAuthProvider' => 'applications/auth/provider/PhabricatorPhabricatorAuthProvider.php',
    'PhabricatorPhameApplication' => 'applications/phame/application/PhabricatorPhameApplication.php',
    'PhabricatorPhameBlogPHIDType' => 'applications/phame/phid/PhabricatorPhameBlogPHIDType.php',
    'PhabricatorPhamePostPHIDType' => 'applications/phame/phid/PhabricatorPhamePostPHIDType.php',
    'PhabricatorPhluxApplication' => 'applications/phlux/application/PhabricatorPhluxApplication.php',
    'PhabricatorPholioApplication' => 'applications/pholio/application/PhabricatorPholioApplication.php',
    'PhabricatorPholioConfigOptions' => 'applications/pholio/config/PhabricatorPholioConfigOptions.php',
    'PhabricatorPholioMockTestDataGenerator' => 'applications/pholio/lipsum/PhabricatorPholioMockTestDataGenerator.php',
    'PhabricatorPhortuneApplication' => 'applications/phortune/application/PhabricatorPhortuneApplication.php',
    'PhabricatorPhortuneManagementInvoiceWorkflow' => 'applications/phortune/management/PhabricatorPhortuneManagementInvoiceWorkflow.php',
    'PhabricatorPhortuneManagementWorkflow' => 'applications/phortune/management/PhabricatorPhortuneManagementWorkflow.php',
    'PhabricatorPhragmentApplication' => 'applications/phragment/application/PhabricatorPhragmentApplication.php',
    'PhabricatorPhrequentApplication' => 'applications/phrequent/application/PhabricatorPhrequentApplication.php',
    'PhabricatorPhrequentConfigOptions' => 'applications/phrequent/config/PhabricatorPhrequentConfigOptions.php',
    'PhabricatorPhrictionApplication' => 'applications/phriction/application/PhabricatorPhrictionApplication.php',
    'PhabricatorPhrictionConfigOptions' => 'applications/phriction/config/PhabricatorPhrictionConfigOptions.php',
    'PhabricatorPhurlApplication' => 'applications/phurl/application/PhabricatorPhurlApplication.php',
    'PhabricatorPhurlConfigOptions' => 'applications/config/option/PhabricatorPhurlConfigOptions.php',
    'PhabricatorPhurlController' => 'applications/phurl/controller/PhabricatorPhurlController.php',
    'PhabricatorPhurlDAO' => 'applications/phurl/storage/PhabricatorPhurlDAO.php',
    'PhabricatorPhurlLinkRemarkupRule' => 'applications/phurl/remarkup/PhabricatorPhurlLinkRemarkupRule.php',
    'PhabricatorPhurlRemarkupRule' => 'applications/phurl/remarkup/PhabricatorPhurlRemarkupRule.php',
    'PhabricatorPhurlSchemaSpec' => 'applications/phurl/storage/PhabricatorPhurlSchemaSpec.php',
    'PhabricatorPhurlShortURLController' => 'applications/phurl/controller/PhabricatorPhurlShortURLController.php',
    'PhabricatorPhurlShortURLDefaultController' => 'applications/phurl/controller/PhabricatorPhurlShortURLDefaultController.php',
    'PhabricatorPhurlURL' => 'applications/phurl/storage/PhabricatorPhurlURL.php',
    'PhabricatorPhurlURLAccessController' => 'applications/phurl/controller/PhabricatorPhurlURLAccessController.php',
    'PhabricatorPhurlURLCommentController' => 'applications/phurl/controller/PhabricatorPhurlURLCommentController.php',
    'PhabricatorPhurlURLCreateCapability' => 'applications/phurl/capability/PhabricatorPhurlURLCreateCapability.php',
    'PhabricatorPhurlURLEditController' => 'applications/phurl/controller/PhabricatorPhurlURLEditController.php',
    'PhabricatorPhurlURLEditor' => 'applications/phurl/editor/PhabricatorPhurlURLEditor.php',
    'PhabricatorPhurlURLListController' => 'applications/phurl/controller/PhabricatorPhurlURLListController.php',
    'PhabricatorPhurlURLMailReceiver' => 'applications/phurl/mail/PhabricatorPhurlURLMailReceiver.php',
    'PhabricatorPhurlURLPHIDType' => 'applications/phurl/phid/PhabricatorPhurlURLPHIDType.php',
    'PhabricatorPhurlURLQuery' => 'applications/phurl/query/PhabricatorPhurlURLQuery.php',
    'PhabricatorPhurlURLReplyHandler' => 'applications/phurl/mail/PhabricatorPhurlURLReplyHandler.php',
    'PhabricatorPhurlURLSearchEngine' => 'applications/phurl/query/PhabricatorPhurlURLSearchEngine.php',
    'PhabricatorPhurlURLTransaction' => 'applications/phurl/storage/PhabricatorPhurlURLTransaction.php',
    'PhabricatorPhurlURLTransactionComment' => 'applications/phurl/storage/PhabricatorPhurlURLTransactionComment.php',
    'PhabricatorPhurlURLTransactionQuery' => 'applications/phurl/query/PhabricatorPhurlURLTransactionQuery.php',
    'PhabricatorPhurlURLViewController' => 'applications/phurl/controller/PhabricatorPhurlURLViewController.php',
    'PhabricatorPirateEnglishTranslation' => 'infrastructure/internationalization/translation/PhabricatorPirateEnglishTranslation.php',
    'PhabricatorPlatformSite' => 'aphront/site/PhabricatorPlatformSite.php',
    'PhabricatorPolicies' => 'applications/policy/constants/PhabricatorPolicies.php',
    'PhabricatorPolicy' => 'applications/policy/storage/PhabricatorPolicy.php',
    'PhabricatorPolicyApplication' => 'applications/policy/application/PhabricatorPolicyApplication.php',
    'PhabricatorPolicyAwareQuery' => 'infrastructure/query/policy/PhabricatorPolicyAwareQuery.php',
    'PhabricatorPolicyAwareTestQuery' => 'applications/policy/__tests__/PhabricatorPolicyAwareTestQuery.php',
    'PhabricatorPolicyCanEditCapability' => 'applications/policy/capability/PhabricatorPolicyCanEditCapability.php',
    'PhabricatorPolicyCanJoinCapability' => 'applications/policy/capability/PhabricatorPolicyCanJoinCapability.php',
    'PhabricatorPolicyCanViewCapability' => 'applications/policy/capability/PhabricatorPolicyCanViewCapability.php',
    'PhabricatorPolicyCapability' => 'applications/policy/capability/PhabricatorPolicyCapability.php',
    'PhabricatorPolicyCapabilityTestCase' => 'applications/policy/capability/__tests__/PhabricatorPolicyCapabilityTestCase.php',
    'PhabricatorPolicyConfigOptions' => 'applications/policy/config/PhabricatorPolicyConfigOptions.php',
    'PhabricatorPolicyConstants' => 'applications/policy/constants/PhabricatorPolicyConstants.php',
    'PhabricatorPolicyController' => 'applications/policy/controller/PhabricatorPolicyController.php',
    'PhabricatorPolicyDAO' => 'applications/policy/storage/PhabricatorPolicyDAO.php',
    'PhabricatorPolicyDataTestCase' => 'applications/policy/__tests__/PhabricatorPolicyDataTestCase.php',
    'PhabricatorPolicyEditController' => 'applications/policy/controller/PhabricatorPolicyEditController.php',
    'PhabricatorPolicyEditEngineExtension' => 'applications/policy/editor/PhabricatorPolicyEditEngineExtension.php',
    'PhabricatorPolicyEditField' => 'applications/transactions/editfield/PhabricatorPolicyEditField.php',
    'PhabricatorPolicyException' => 'applications/policy/exception/PhabricatorPolicyException.php',
    'PhabricatorPolicyExplainController' => 'applications/policy/controller/PhabricatorPolicyExplainController.php',
    'PhabricatorPolicyFilter' => 'applications/policy/filter/PhabricatorPolicyFilter.php',
    'PhabricatorPolicyInterface' => 'applications/policy/interface/PhabricatorPolicyInterface.php',
    'PhabricatorPolicyManagementShowWorkflow' => 'applications/policy/management/PhabricatorPolicyManagementShowWorkflow.php',
    'PhabricatorPolicyManagementUnlockWorkflow' => 'applications/policy/management/PhabricatorPolicyManagementUnlockWorkflow.php',
    'PhabricatorPolicyManagementWorkflow' => 'applications/policy/management/PhabricatorPolicyManagementWorkflow.php',
    'PhabricatorPolicyPHIDTypePolicy' => 'applications/policy/phid/PhabricatorPolicyPHIDTypePolicy.php',
    'PhabricatorPolicyQuery' => 'applications/policy/query/PhabricatorPolicyQuery.php',
    'PhabricatorPolicyRequestExceptionHandler' => 'aphront/handler/PhabricatorPolicyRequestExceptionHandler.php',
    'PhabricatorPolicyRule' => 'applications/policy/rule/PhabricatorPolicyRule.php',
    'PhabricatorPolicySearchEngineExtension' => 'applications/policy/engineextension/PhabricatorPolicySearchEngineExtension.php',
    'PhabricatorPolicyTestCase' => 'applications/policy/__tests__/PhabricatorPolicyTestCase.php',
    'PhabricatorPolicyTestObject' => 'applications/policy/__tests__/PhabricatorPolicyTestObject.php',
    'PhabricatorPolicyType' => 'applications/policy/constants/PhabricatorPolicyType.php',
    'PhabricatorPonderApplication' => 'applications/ponder/application/PhabricatorPonderApplication.php',
    'PhabricatorProfilePanel' => 'applications/search/profilepanel/PhabricatorProfilePanel.php',
    'PhabricatorProfilePanelConfiguration' => 'applications/search/storage/PhabricatorProfilePanelConfiguration.php',
    'PhabricatorProfilePanelConfigurationQuery' => 'applications/search/query/PhabricatorProfilePanelConfigurationQuery.php',
    'PhabricatorProfilePanelConfigurationTransaction' => 'applications/search/storage/PhabricatorProfilePanelConfigurationTransaction.php',
    'PhabricatorProfilePanelConfigurationTransactionQuery' => 'applications/search/query/PhabricatorProfilePanelConfigurationTransactionQuery.php',
    'PhabricatorProfilePanelEditEngine' => 'applications/search/editor/PhabricatorProfilePanelEditEngine.php',
    'PhabricatorProfilePanelEditor' => 'applications/search/editor/PhabricatorProfilePanelEditor.php',
    'PhabricatorProfilePanelEngine' => 'applications/search/engine/PhabricatorProfilePanelEngine.php',
    'PhabricatorProfilePanelIconSet' => 'applications/search/profilepanel/PhabricatorProfilePanelIconSet.php',
    'PhabricatorProfilePanelPHIDType' => 'applications/search/phidtype/PhabricatorProfilePanelPHIDType.php',
    'PhabricatorProject' => 'applications/project/storage/PhabricatorProject.php',
    'PhabricatorProjectAddHeraldAction' => 'applications/project/herald/PhabricatorProjectAddHeraldAction.php',
    'PhabricatorProjectApplication' => 'applications/project/application/PhabricatorProjectApplication.php',
    'PhabricatorProjectArchiveController' => 'applications/project/controller/PhabricatorProjectArchiveController.php',
    'PhabricatorProjectBoardController' => 'applications/project/controller/PhabricatorProjectBoardController.php',
    'PhabricatorProjectBoardImportController' => 'applications/project/controller/PhabricatorProjectBoardImportController.php',
    'PhabricatorProjectBoardReorderController' => 'applications/project/controller/PhabricatorProjectBoardReorderController.php',
    'PhabricatorProjectBoardViewController' => 'applications/project/controller/PhabricatorProjectBoardViewController.php',
    'PhabricatorProjectColorsConfigOptionType' => 'applications/project/config/PhabricatorProjectColorsConfigOptionType.php',
    'PhabricatorProjectColumn' => 'applications/project/storage/PhabricatorProjectColumn.php',
    'PhabricatorProjectColumnDetailController' => 'applications/project/controller/PhabricatorProjectColumnDetailController.php',
    'PhabricatorProjectColumnEditController' => 'applications/project/controller/PhabricatorProjectColumnEditController.php',
    'PhabricatorProjectColumnHideController' => 'applications/project/controller/PhabricatorProjectColumnHideController.php',
    'PhabricatorProjectColumnPHIDType' => 'applications/project/phid/PhabricatorProjectColumnPHIDType.php',
    'PhabricatorProjectColumnPosition' => 'applications/project/storage/PhabricatorProjectColumnPosition.php',
    'PhabricatorProjectColumnPositionQuery' => 'applications/project/query/PhabricatorProjectColumnPositionQuery.php',
    'PhabricatorProjectColumnQuery' => 'applications/project/query/PhabricatorProjectColumnQuery.php',
    'PhabricatorProjectColumnTransaction' => 'applications/project/storage/PhabricatorProjectColumnTransaction.php',
    'PhabricatorProjectColumnTransactionEditor' => 'applications/project/editor/PhabricatorProjectColumnTransactionEditor.php',
    'PhabricatorProjectColumnTransactionQuery' => 'applications/project/query/PhabricatorProjectColumnTransactionQuery.php',
    'PhabricatorProjectConfigOptions' => 'applications/project/config/PhabricatorProjectConfigOptions.php',
    'PhabricatorProjectConfiguredCustomField' => 'applications/project/customfield/PhabricatorProjectConfiguredCustomField.php',
    'PhabricatorProjectController' => 'applications/project/controller/PhabricatorProjectController.php',
    'PhabricatorProjectCoreTestCase' => 'applications/project/__tests__/PhabricatorProjectCoreTestCase.php',
    'PhabricatorProjectCustomField' => 'applications/project/customfield/PhabricatorProjectCustomField.php',
    'PhabricatorProjectCustomFieldNumericIndex' => 'applications/project/storage/PhabricatorProjectCustomFieldNumericIndex.php',
    'PhabricatorProjectCustomFieldStorage' => 'applications/project/storage/PhabricatorProjectCustomFieldStorage.php',
    'PhabricatorProjectCustomFieldStringIndex' => 'applications/project/storage/PhabricatorProjectCustomFieldStringIndex.php',
    'PhabricatorProjectDAO' => 'applications/project/storage/PhabricatorProjectDAO.php',
    'PhabricatorProjectDatasource' => 'applications/project/typeahead/PhabricatorProjectDatasource.php',
    'PhabricatorProjectDescriptionField' => 'applications/project/customfield/PhabricatorProjectDescriptionField.php',
    'PhabricatorProjectDetailsProfilePanel' => 'applications/project/profilepanel/PhabricatorProjectDetailsProfilePanel.php',
    'PhabricatorProjectEditController' => 'applications/project/controller/PhabricatorProjectEditController.php',
    'PhabricatorProjectEditEngine' => 'applications/project/engine/PhabricatorProjectEditEngine.php',
    'PhabricatorProjectEditPictureController' => 'applications/project/controller/PhabricatorProjectEditPictureController.php',
    'PhabricatorProjectFulltextEngine' => 'applications/project/search/PhabricatorProjectFulltextEngine.php',
    'PhabricatorProjectHeraldAction' => 'applications/project/herald/PhabricatorProjectHeraldAction.php',
    'PhabricatorProjectHeraldAdapter' => 'applications/project/herald/PhabricatorProjectHeraldAdapter.php',
    'PhabricatorProjectHeraldFieldGroup' => 'applications/project/herald/PhabricatorProjectHeraldFieldGroup.php',
    'PhabricatorProjectHistoryController' => 'applications/project/controller/PhabricatorProjectHistoryController.php',
    'PhabricatorProjectIconSet' => 'applications/project/icon/PhabricatorProjectIconSet.php',
    'PhabricatorProjectIconsConfigOptionType' => 'applications/project/config/PhabricatorProjectIconsConfigOptionType.php',
    'PhabricatorProjectInterface' => 'applications/project/interface/PhabricatorProjectInterface.php',
    'PhabricatorProjectListController' => 'applications/project/controller/PhabricatorProjectListController.php',
    'PhabricatorProjectListView' => 'applications/project/view/PhabricatorProjectListView.php',
    'PhabricatorProjectLockController' => 'applications/project/controller/PhabricatorProjectLockController.php',
    'PhabricatorProjectLogicalAncestorDatasource' => 'applications/project/typeahead/PhabricatorProjectLogicalAncestorDatasource.php',
    'PhabricatorProjectLogicalDatasource' => 'applications/project/typeahead/PhabricatorProjectLogicalDatasource.php',
    'PhabricatorProjectLogicalOrNotDatasource' => 'applications/project/typeahead/PhabricatorProjectLogicalOrNotDatasource.php',
    'PhabricatorProjectLogicalUserDatasource' => 'applications/project/typeahead/PhabricatorProjectLogicalUserDatasource.php',
    'PhabricatorProjectLogicalViewerDatasource' => 'applications/project/typeahead/PhabricatorProjectLogicalViewerDatasource.php',
    'PhabricatorProjectMaterializedMemberEdgeType' => 'applications/project/edge/PhabricatorProjectMaterializedMemberEdgeType.php',
    'PhabricatorProjectMemberListView' => 'applications/project/view/PhabricatorProjectMemberListView.php',
    'PhabricatorProjectMemberOfProjectEdgeType' => 'applications/project/edge/PhabricatorProjectMemberOfProjectEdgeType.php',
    'PhabricatorProjectMembersAddController' => 'applications/project/controller/PhabricatorProjectMembersAddController.php',
    'PhabricatorProjectMembersDatasource' => 'applications/project/typeahead/PhabricatorProjectMembersDatasource.php',
    'PhabricatorProjectMembersPolicyRule' => 'applications/project/policyrule/PhabricatorProjectMembersPolicyRule.php',
    'PhabricatorProjectMembersProfilePanel' => 'applications/project/profilepanel/PhabricatorProjectMembersProfilePanel.php',
    'PhabricatorProjectMembersRemoveController' => 'applications/project/controller/PhabricatorProjectMembersRemoveController.php',
    'PhabricatorProjectMembersViewController' => 'applications/project/controller/PhabricatorProjectMembersViewController.php',
    'PhabricatorProjectMilestonesController' => 'applications/project/controller/PhabricatorProjectMilestonesController.php',
    'PhabricatorProjectMoveController' => 'applications/project/controller/PhabricatorProjectMoveController.php',
    'PhabricatorProjectNameContextFreeGrammar' => 'applications/project/lipsum/PhabricatorProjectNameContextFreeGrammar.php',
    'PhabricatorProjectNoProjectsDatasource' => 'applications/project/typeahead/PhabricatorProjectNoProjectsDatasource.php',
    'PhabricatorProjectObjectHasProjectEdgeType' => 'applications/project/edge/PhabricatorProjectObjectHasProjectEdgeType.php',
    'PhabricatorProjectOrUserDatasource' => 'applications/project/typeahead/PhabricatorProjectOrUserDatasource.php',
    'PhabricatorProjectOrUserFunctionDatasource' => 'applications/project/typeahead/PhabricatorProjectOrUserFunctionDatasource.php',
    'PhabricatorProjectPHIDResolver' => 'applications/phid/resolver/PhabricatorProjectPHIDResolver.php',
    'PhabricatorProjectPanelController' => 'applications/project/controller/PhabricatorProjectPanelController.php',
    'PhabricatorProjectProfileController' => 'applications/project/controller/PhabricatorProjectProfileController.php',
    'PhabricatorProjectProfilePanelEngine' => 'applications/project/engine/PhabricatorProjectProfilePanelEngine.php',
    'PhabricatorProjectProjectHasMemberEdgeType' => 'applications/project/edge/PhabricatorProjectProjectHasMemberEdgeType.php',
    'PhabricatorProjectProjectHasObjectEdgeType' => 'applications/project/edge/PhabricatorProjectProjectHasObjectEdgeType.php',
    'PhabricatorProjectProjectPHIDType' => 'applications/project/phid/PhabricatorProjectProjectPHIDType.php',
    'PhabricatorProjectQuery' => 'applications/project/query/PhabricatorProjectQuery.php',
    'PhabricatorProjectRemoveHeraldAction' => 'applications/project/herald/PhabricatorProjectRemoveHeraldAction.php',
    'PhabricatorProjectSchemaSpec' => 'applications/project/storage/PhabricatorProjectSchemaSpec.php',
    'PhabricatorProjectSearchEngine' => 'applications/project/query/PhabricatorProjectSearchEngine.php',
    'PhabricatorProjectSearchField' => 'applications/project/searchfield/PhabricatorProjectSearchField.php',
    'PhabricatorProjectSilenceController' => 'applications/project/controller/PhabricatorProjectSilenceController.php',
    'PhabricatorProjectSilencedEdgeType' => 'applications/project/edge/PhabricatorProjectSilencedEdgeType.php',
    'PhabricatorProjectSlug' => 'applications/project/storage/PhabricatorProjectSlug.php',
    'PhabricatorProjectStandardCustomField' => 'applications/project/customfield/PhabricatorProjectStandardCustomField.php',
    'PhabricatorProjectStatus' => 'applications/project/constants/PhabricatorProjectStatus.php',
    'PhabricatorProjectSubprojectsController' => 'applications/project/controller/PhabricatorProjectSubprojectsController.php',
    'PhabricatorProjectTestDataGenerator' => 'applications/project/lipsum/PhabricatorProjectTestDataGenerator.php',
    'PhabricatorProjectTransaction' => 'applications/project/storage/PhabricatorProjectTransaction.php',
    'PhabricatorProjectTransactionEditor' => 'applications/project/editor/PhabricatorProjectTransactionEditor.php',
    'PhabricatorProjectTransactionQuery' => 'applications/project/query/PhabricatorProjectTransactionQuery.php',
    'PhabricatorProjectUIEventListener' => 'applications/project/events/PhabricatorProjectUIEventListener.php',
    'PhabricatorProjectUpdateController' => 'applications/project/controller/PhabricatorProjectUpdateController.php',
    'PhabricatorProjectUserFunctionDatasource' => 'applications/project/typeahead/PhabricatorProjectUserFunctionDatasource.php',
    'PhabricatorProjectUserListView' => 'applications/project/view/PhabricatorProjectUserListView.php',
    'PhabricatorProjectViewController' => 'applications/project/controller/PhabricatorProjectViewController.php',
    'PhabricatorProjectWatchController' => 'applications/project/controller/PhabricatorProjectWatchController.php',
    'PhabricatorProjectWatcherListView' => 'applications/project/view/PhabricatorProjectWatcherListView.php',
    'PhabricatorProjectWorkboardProfilePanel' => 'applications/project/profilepanel/PhabricatorProjectWorkboardProfilePanel.php',
    'PhabricatorProjectsEditEngineExtension' => 'applications/project/engineextension/PhabricatorProjectsEditEngineExtension.php',
    'PhabricatorProjectsEditField' => 'applications/transactions/editfield/PhabricatorProjectsEditField.php',
    'PhabricatorProjectsFulltextEngineExtension' => 'applications/project/engineextension/PhabricatorProjectsFulltextEngineExtension.php',
    'PhabricatorProjectsMembershipIndexEngineExtension' => 'applications/project/engineextension/PhabricatorProjectsMembershipIndexEngineExtension.php',
    'PhabricatorProjectsPolicyRule' => 'applications/project/policyrule/PhabricatorProjectsPolicyRule.php',
    'PhabricatorProjectsSearchEngineAttachment' => 'applications/project/engineextension/PhabricatorProjectsSearchEngineAttachment.php',
    'PhabricatorProjectsSearchEngineExtension' => 'applications/project/engineextension/PhabricatorProjectsSearchEngineExtension.php',
    'PhabricatorProtocolAdapter' => 'infrastructure/daemon/bot/adapter/PhabricatorProtocolAdapter.php',
    'PhabricatorPygmentSetupCheck' => 'applications/config/check/PhabricatorPygmentSetupCheck.php',
    'PhabricatorQuery' => 'infrastructure/query/PhabricatorQuery.php',
    'PhabricatorQueryConstraint' => 'infrastructure/query/constraint/PhabricatorQueryConstraint.php',
    'PhabricatorQueryOrderItem' => 'infrastructure/query/order/PhabricatorQueryOrderItem.php',
    'PhabricatorQueryOrderTestCase' => 'infrastructure/query/order/__tests__/PhabricatorQueryOrderTestCase.php',
    'PhabricatorQueryOrderVector' => 'infrastructure/query/order/PhabricatorQueryOrderVector.php',
    'PhabricatorRateLimitRequestExceptionHandler' => 'aphront/handler/PhabricatorRateLimitRequestExceptionHandler.php',
    'PhabricatorRecaptchaConfigOptions' => 'applications/config/option/PhabricatorRecaptchaConfigOptions.php',
    'PhabricatorRecipientHasBadgeEdgeType' => 'applications/badges/edge/PhabricatorRecipientHasBadgeEdgeType.php',
    'PhabricatorRedirectController' => 'applications/base/controller/PhabricatorRedirectController.php',
    'PhabricatorRefreshCSRFController' => 'applications/auth/controller/PhabricatorRefreshCSRFController.php',
    'PhabricatorRegistrationProfile' => 'applications/people/storage/PhabricatorRegistrationProfile.php',
    'PhabricatorReleephApplication' => 'applications/releeph/application/PhabricatorReleephApplication.php',
    'PhabricatorReleephApplicationConfigOptions' => 'applications/releeph/config/PhabricatorReleephApplicationConfigOptions.php',
    'PhabricatorRemarkupControl' => 'view/form/control/PhabricatorRemarkupControl.php',
    'PhabricatorRemarkupCowsayBlockInterpreter' => 'infrastructure/markup/interpreter/PhabricatorRemarkupCowsayBlockInterpreter.php',
    'PhabricatorRemarkupCustomBlockRule' => 'infrastructure/markup/rule/PhabricatorRemarkupCustomBlockRule.php',
    'PhabricatorRemarkupCustomInlineRule' => 'infrastructure/markup/rule/PhabricatorRemarkupCustomInlineRule.php',
    'PhabricatorRemarkupEditField' => 'applications/transactions/editfield/PhabricatorRemarkupEditField.php',
    'PhabricatorRemarkupFigletBlockInterpreter' => 'infrastructure/markup/interpreter/PhabricatorRemarkupFigletBlockInterpreter.php',
    'PhabricatorRemarkupUIExample' => 'applications/uiexample/examples/PhabricatorRemarkupUIExample.php',
    'PhabricatorRepositoriesSetupCheck' => 'applications/config/check/PhabricatorRepositoriesSetupCheck.php',
    'PhabricatorRepository' => 'applications/repository/storage/PhabricatorRepository.php',
    'PhabricatorRepositoryAuditRequest' => 'applications/repository/storage/PhabricatorRepositoryAuditRequest.php',
    'PhabricatorRepositoryBranch' => 'applications/repository/storage/PhabricatorRepositoryBranch.php',
    'PhabricatorRepositoryCommit' => 'applications/repository/storage/PhabricatorRepositoryCommit.php',
    'PhabricatorRepositoryCommitChangeParserWorker' => 'applications/repository/worker/commitchangeparser/PhabricatorRepositoryCommitChangeParserWorker.php',
    'PhabricatorRepositoryCommitData' => 'applications/repository/storage/PhabricatorRepositoryCommitData.php',
    'PhabricatorRepositoryCommitHeraldWorker' => 'applications/repository/worker/PhabricatorRepositoryCommitHeraldWorker.php',
    'PhabricatorRepositoryCommitMessageParserWorker' => 'applications/repository/worker/commitmessageparser/PhabricatorRepositoryCommitMessageParserWorker.php',
    'PhabricatorRepositoryCommitOwnersWorker' => 'applications/repository/worker/PhabricatorRepositoryCommitOwnersWorker.php',
    'PhabricatorRepositoryCommitPHIDType' => 'applications/repository/phid/PhabricatorRepositoryCommitPHIDType.php',
    'PhabricatorRepositoryCommitParserWorker' => 'applications/repository/worker/PhabricatorRepositoryCommitParserWorker.php',
    'PhabricatorRepositoryCommitRef' => 'applications/repository/engine/PhabricatorRepositoryCommitRef.php',
    'PhabricatorRepositoryConfigOptions' => 'applications/repository/config/PhabricatorRepositoryConfigOptions.php',
    'PhabricatorRepositoryDAO' => 'applications/repository/storage/PhabricatorRepositoryDAO.php',
    'PhabricatorRepositoryDiscoveryEngine' => 'applications/repository/engine/PhabricatorRepositoryDiscoveryEngine.php',
    'PhabricatorRepositoryEditor' => 'applications/repository/editor/PhabricatorRepositoryEditor.php',
    'PhabricatorRepositoryEngine' => 'applications/repository/engine/PhabricatorRepositoryEngine.php',
    'PhabricatorRepositoryGitCommitChangeParserWorker' => 'applications/repository/worker/commitchangeparser/PhabricatorRepositoryGitCommitChangeParserWorker.php',
    'PhabricatorRepositoryGitCommitMessageParserWorker' => 'applications/repository/worker/commitmessageparser/PhabricatorRepositoryGitCommitMessageParserWorker.php',
    'PhabricatorRepositoryGraphCache' => 'applications/repository/graphcache/PhabricatorRepositoryGraphCache.php',
    'PhabricatorRepositoryGraphStream' => 'applications/repository/daemon/PhabricatorRepositoryGraphStream.php',
    'PhabricatorRepositoryManagementCacheWorkflow' => 'applications/repository/management/PhabricatorRepositoryManagementCacheWorkflow.php',
    'PhabricatorRepositoryManagementDiscoverWorkflow' => 'applications/repository/management/PhabricatorRepositoryManagementDiscoverWorkflow.php',
    'PhabricatorRepositoryManagementEditWorkflow' => 'applications/repository/management/PhabricatorRepositoryManagementEditWorkflow.php',
    'PhabricatorRepositoryManagementImportingWorkflow' => 'applications/repository/management/PhabricatorRepositoryManagementImportingWorkflow.php',
    'PhabricatorRepositoryManagementListPathsWorkflow' => 'applications/repository/management/PhabricatorRepositoryManagementListPathsWorkflow.php',
    'PhabricatorRepositoryManagementListWorkflow' => 'applications/repository/management/PhabricatorRepositoryManagementListWorkflow.php',
    'PhabricatorRepositoryManagementLookupUsersWorkflow' => 'applications/repository/management/PhabricatorRepositoryManagementLookupUsersWorkflow.php',
    'PhabricatorRepositoryManagementMarkImportedWorkflow' => 'applications/repository/management/PhabricatorRepositoryManagementMarkImportedWorkflow.php',
    'PhabricatorRepositoryManagementMirrorWorkflow' => 'applications/repository/management/PhabricatorRepositoryManagementMirrorWorkflow.php',
    'PhabricatorRepositoryManagementMovePathsWorkflow' => 'applications/repository/management/PhabricatorRepositoryManagementMovePathsWorkflow.php',
    'PhabricatorRepositoryManagementParentsWorkflow' => 'applications/repository/management/PhabricatorRepositoryManagementParentsWorkflow.php',
    'PhabricatorRepositoryManagementPullWorkflow' => 'applications/repository/management/PhabricatorRepositoryManagementPullWorkflow.php',
    'PhabricatorRepositoryManagementRefsWorkflow' => 'applications/repository/management/PhabricatorRepositoryManagementRefsWorkflow.php',
    'PhabricatorRepositoryManagementReparseWorkflow' => 'applications/repository/management/PhabricatorRepositoryManagementReparseWorkflow.php',
    'PhabricatorRepositoryManagementUpdateWorkflow' => 'applications/repository/management/PhabricatorRepositoryManagementUpdateWorkflow.php',
    'PhabricatorRepositoryManagementWorkflow' => 'applications/repository/management/PhabricatorRepositoryManagementWorkflow.php',
    'PhabricatorRepositoryMercurialCommitChangeParserWorker' => 'applications/repository/worker/commitchangeparser/PhabricatorRepositoryMercurialCommitChangeParserWorker.php',
    'PhabricatorRepositoryMercurialCommitMessageParserWorker' => 'applications/repository/worker/commitmessageparser/PhabricatorRepositoryMercurialCommitMessageParserWorker.php',
    'PhabricatorRepositoryMirror' => 'applications/repository/storage/PhabricatorRepositoryMirror.php',
    'PhabricatorRepositoryMirrorEngine' => 'applications/repository/engine/PhabricatorRepositoryMirrorEngine.php',
    'PhabricatorRepositoryMirrorPHIDType' => 'applications/repository/phid/PhabricatorRepositoryMirrorPHIDType.php',
    'PhabricatorRepositoryMirrorQuery' => 'applications/repository/query/PhabricatorRepositoryMirrorQuery.php',
    'PhabricatorRepositoryParsedChange' => 'applications/repository/data/PhabricatorRepositoryParsedChange.php',
    'PhabricatorRepositoryPullEngine' => 'applications/repository/engine/PhabricatorRepositoryPullEngine.php',
    'PhabricatorRepositoryPullLocalDaemon' => 'applications/repository/daemon/PhabricatorRepositoryPullLocalDaemon.php',
    'PhabricatorRepositoryPushEvent' => 'applications/repository/storage/PhabricatorRepositoryPushEvent.php',
    'PhabricatorRepositoryPushEventPHIDType' => 'applications/repository/phid/PhabricatorRepositoryPushEventPHIDType.php',
    'PhabricatorRepositoryPushEventQuery' => 'applications/repository/query/PhabricatorRepositoryPushEventQuery.php',
    'PhabricatorRepositoryPushLog' => 'applications/repository/storage/PhabricatorRepositoryPushLog.php',
    'PhabricatorRepositoryPushLogPHIDType' => 'applications/repository/phid/PhabricatorRepositoryPushLogPHIDType.php',
    'PhabricatorRepositoryPushLogQuery' => 'applications/repository/query/PhabricatorRepositoryPushLogQuery.php',
    'PhabricatorRepositoryPushLogSearchEngine' => 'applications/repository/query/PhabricatorRepositoryPushLogSearchEngine.php',
    'PhabricatorRepositoryPushMailWorker' => 'applications/repository/worker/PhabricatorRepositoryPushMailWorker.php',
    'PhabricatorRepositoryPushReplyHandler' => 'applications/repository/mail/PhabricatorRepositoryPushReplyHandler.php',
    'PhabricatorRepositoryQuery' => 'applications/repository/query/PhabricatorRepositoryQuery.php',
    'PhabricatorRepositoryRefCursor' => 'applications/repository/storage/PhabricatorRepositoryRefCursor.php',
    'PhabricatorRepositoryRefCursorPHIDType' => 'applications/repository/phid/PhabricatorRepositoryRefCursorPHIDType.php',
    'PhabricatorRepositoryRefCursorQuery' => 'applications/repository/query/PhabricatorRepositoryRefCursorQuery.php',
    'PhabricatorRepositoryRefEngine' => 'applications/repository/engine/PhabricatorRepositoryRefEngine.php',
    'PhabricatorRepositoryRepositoryPHIDType' => 'applications/repository/phid/PhabricatorRepositoryRepositoryPHIDType.php',
    'PhabricatorRepositorySchemaSpec' => 'applications/repository/storage/PhabricatorRepositorySchemaSpec.php',
    'PhabricatorRepositorySearchEngine' => 'applications/repository/query/PhabricatorRepositorySearchEngine.php',
    'PhabricatorRepositoryStatusMessage' => 'applications/repository/storage/PhabricatorRepositoryStatusMessage.php',
    'PhabricatorRepositorySvnCommitChangeParserWorker' => 'applications/repository/worker/commitchangeparser/PhabricatorRepositorySvnCommitChangeParserWorker.php',
    'PhabricatorRepositorySvnCommitMessageParserWorker' => 'applications/repository/worker/commitmessageparser/PhabricatorRepositorySvnCommitMessageParserWorker.php',
    'PhabricatorRepositorySymbol' => 'applications/repository/storage/PhabricatorRepositorySymbol.php',
    'PhabricatorRepositoryTestCase' => 'applications/repository/storage/__tests__/PhabricatorRepositoryTestCase.php',
    'PhabricatorRepositoryTransaction' => 'applications/repository/storage/PhabricatorRepositoryTransaction.php',
    'PhabricatorRepositoryTransactionQuery' => 'applications/repository/query/PhabricatorRepositoryTransactionQuery.php',
    'PhabricatorRepositoryType' => 'applications/repository/constants/PhabricatorRepositoryType.php',
    'PhabricatorRepositoryURIIndex' => 'applications/repository/storage/PhabricatorRepositoryURIIndex.php',
    'PhabricatorRepositoryURINormalizer' => 'applications/repository/data/PhabricatorRepositoryURINormalizer.php',
    'PhabricatorRepositoryURINormalizerTestCase' => 'applications/repository/data/__tests__/PhabricatorRepositoryURINormalizerTestCase.php',
    'PhabricatorRepositoryURITestCase' => 'applications/repository/storage/__tests__/PhabricatorRepositoryURITestCase.php',
    'PhabricatorRepositoryVCSPassword' => 'applications/repository/storage/PhabricatorRepositoryVCSPassword.php',
    'PhabricatorRepositoryVersion' => 'applications/repository/constants/PhabricatorRepositoryVersion.php',
    'PhabricatorRequestExceptionHandler' => 'aphront/handler/PhabricatorRequestExceptionHandler.php',
    'PhabricatorResourceSite' => 'aphront/site/PhabricatorResourceSite.php',
    'PhabricatorRobotsController' => 'applications/system/controller/PhabricatorRobotsController.php',
    'PhabricatorS3FileStorageEngine' => 'applications/files/engine/PhabricatorS3FileStorageEngine.php',
    'PhabricatorSMS' => 'infrastructure/sms/storage/PhabricatorSMS.php',
    'PhabricatorSMSConfigOptions' => 'applications/config/option/PhabricatorSMSConfigOptions.php',
    'PhabricatorSMSDAO' => 'infrastructure/sms/storage/PhabricatorSMSDAO.php',
    'PhabricatorSMSDemultiplexWorker' => 'infrastructure/sms/worker/PhabricatorSMSDemultiplexWorker.php',
    'PhabricatorSMSImplementationAdapter' => 'infrastructure/sms/adapter/PhabricatorSMSImplementationAdapter.php',
    'PhabricatorSMSImplementationTestBlackholeAdapter' => 'infrastructure/sms/adapter/PhabricatorSMSImplementationTestBlackholeAdapter.php',
    'PhabricatorSMSImplementationTwilioAdapter' => 'infrastructure/sms/adapter/PhabricatorSMSImplementationTwilioAdapter.php',
    'PhabricatorSMSManagementListOutboundWorkflow' => 'infrastructure/sms/management/PhabricatorSMSManagementListOutboundWorkflow.php',
    'PhabricatorSMSManagementSendTestWorkflow' => 'infrastructure/sms/management/PhabricatorSMSManagementSendTestWorkflow.php',
    'PhabricatorSMSManagementShowOutboundWorkflow' => 'infrastructure/sms/management/PhabricatorSMSManagementShowOutboundWorkflow.php',
    'PhabricatorSMSManagementWorkflow' => 'infrastructure/sms/management/PhabricatorSMSManagementWorkflow.php',
    'PhabricatorSMSSendWorker' => 'infrastructure/sms/worker/PhabricatorSMSSendWorker.php',
    'PhabricatorSMSWorker' => 'infrastructure/sms/worker/PhabricatorSMSWorker.php',
    'PhabricatorSQLPatchList' => 'infrastructure/storage/patch/PhabricatorSQLPatchList.php',
    'PhabricatorSSHKeyGenerator' => 'infrastructure/util/PhabricatorSSHKeyGenerator.php',
    'PhabricatorSSHKeysSettingsPanel' => 'applications/settings/panel/PhabricatorSSHKeysSettingsPanel.php',
    'PhabricatorSSHLog' => 'infrastructure/log/PhabricatorSSHLog.php',
    'PhabricatorSSHPassthruCommand' => 'infrastructure/ssh/PhabricatorSSHPassthruCommand.php',
    'PhabricatorSSHPublicKeyInterface' => 'applications/auth/sshkey/PhabricatorSSHPublicKeyInterface.php',
    'PhabricatorSSHWorkflow' => 'infrastructure/ssh/PhabricatorSSHWorkflow.php',
    'PhabricatorSavedQuery' => 'applications/search/storage/PhabricatorSavedQuery.php',
    'PhabricatorSavedQueryQuery' => 'applications/search/query/PhabricatorSavedQueryQuery.php',
    'PhabricatorScheduleTaskTriggerAction' => 'infrastructure/daemon/workers/action/PhabricatorScheduleTaskTriggerAction.php',
    'PhabricatorScopedEnv' => 'infrastructure/env/PhabricatorScopedEnv.php',
    'PhabricatorSearchAbstractDocument' => 'applications/search/index/PhabricatorSearchAbstractDocument.php',
    'PhabricatorSearchApplication' => 'applications/search/application/PhabricatorSearchApplication.php',
    'PhabricatorSearchApplicationSearchEngine' => 'applications/search/query/PhabricatorSearchApplicationSearchEngine.php',
    'PhabricatorSearchApplicationStorageEnginePanel' => 'applications/search/applicationpanel/PhabricatorSearchApplicationStorageEnginePanel.php',
    'PhabricatorSearchAttachController' => 'applications/search/controller/PhabricatorSearchAttachController.php',
    'PhabricatorSearchBaseController' => 'applications/search/controller/PhabricatorSearchBaseController.php',
    'PhabricatorSearchCheckboxesField' => 'applications/search/field/PhabricatorSearchCheckboxesField.php',
    'PhabricatorSearchConfigOptions' => 'applications/search/config/PhabricatorSearchConfigOptions.php',
    'PhabricatorSearchController' => 'applications/search/controller/PhabricatorSearchController.php',
    'PhabricatorSearchCustomFieldProxyField' => 'applications/search/field/PhabricatorSearchCustomFieldProxyField.php',
    'PhabricatorSearchDAO' => 'applications/search/storage/PhabricatorSearchDAO.php',
    'PhabricatorSearchDatasource' => 'applications/search/typeahead/PhabricatorSearchDatasource.php',
    'PhabricatorSearchDatasourceField' => 'applications/search/field/PhabricatorSearchDatasourceField.php',
    'PhabricatorSearchDateControlField' => 'applications/search/field/PhabricatorSearchDateControlField.php',
    'PhabricatorSearchDateField' => 'applications/search/field/PhabricatorSearchDateField.php',
    'PhabricatorSearchDeleteController' => 'applications/search/controller/PhabricatorSearchDeleteController.php',
    'PhabricatorSearchDocument' => 'applications/search/storage/document/PhabricatorSearchDocument.php',
    'PhabricatorSearchDocumentField' => 'applications/search/storage/document/PhabricatorSearchDocumentField.php',
    'PhabricatorSearchDocumentFieldType' => 'applications/search/constants/PhabricatorSearchDocumentFieldType.php',
    'PhabricatorSearchDocumentQuery' => 'applications/search/query/PhabricatorSearchDocumentQuery.php',
    'PhabricatorSearchDocumentRelationship' => 'applications/search/storage/document/PhabricatorSearchDocumentRelationship.php',
    'PhabricatorSearchDocumentTypeDatasource' => 'applications/search/typeahead/PhabricatorSearchDocumentTypeDatasource.php',
    'PhabricatorSearchEditController' => 'applications/search/controller/PhabricatorSearchEditController.php',
    'PhabricatorSearchEngineAPIMethod' => 'applications/search/engine/PhabricatorSearchEngineAPIMethod.php',
    'PhabricatorSearchEngineAttachment' => 'applications/search/engineextension/PhabricatorSearchEngineAttachment.php',
    'PhabricatorSearchEngineExtension' => 'applications/search/engineextension/PhabricatorSearchEngineExtension.php',
    'PhabricatorSearchEngineExtensionModule' => 'applications/search/engineextension/PhabricatorSearchEngineExtensionModule.php',
    'PhabricatorSearchEngineTestCase' => 'applications/search/engine/__tests__/PhabricatorSearchEngineTestCase.php',
    'PhabricatorSearchField' => 'applications/search/field/PhabricatorSearchField.php',
    'PhabricatorSearchHovercardController' => 'applications/search/controller/PhabricatorSearchHovercardController.php',
    'PhabricatorSearchIndexVersion' => 'applications/search/storage/PhabricatorSearchIndexVersion.php',
    'PhabricatorSearchIndexVersionDestructionEngineExtension' => 'applications/search/engineextension/PhabricatorSearchIndexVersionDestructionEngineExtension.php',
    'PhabricatorSearchManagementIndexWorkflow' => 'applications/search/management/PhabricatorSearchManagementIndexWorkflow.php',
    'PhabricatorSearchManagementInitWorkflow' => 'applications/search/management/PhabricatorSearchManagementInitWorkflow.php',
    'PhabricatorSearchManagementWorkflow' => 'applications/search/management/PhabricatorSearchManagementWorkflow.php',
    'PhabricatorSearchNgrams' => 'applications/search/ngrams/PhabricatorSearchNgrams.php',
    'PhabricatorSearchNgramsDestructionEngineExtension' => 'applications/search/engineextension/PhabricatorSearchNgramsDestructionEngineExtension.php',
    'PhabricatorSearchOrderController' => 'applications/search/controller/PhabricatorSearchOrderController.php',
    'PhabricatorSearchOrderField' => 'applications/search/field/PhabricatorSearchOrderField.php',
    'PhabricatorSearchPreferencesSettingsPanel' => 'applications/settings/panel/PhabricatorSearchPreferencesSettingsPanel.php',
    'PhabricatorSearchRelationship' => 'applications/search/constants/PhabricatorSearchRelationship.php',
    'PhabricatorSearchResultView' => 'applications/search/view/PhabricatorSearchResultView.php',
    'PhabricatorSearchSelectController' => 'applications/search/controller/PhabricatorSearchSelectController.php',
    'PhabricatorSearchSelectField' => 'applications/search/field/PhabricatorSearchSelectField.php',
    'PhabricatorSearchStringListField' => 'applications/search/field/PhabricatorSearchStringListField.php',
    'PhabricatorSearchSubscribersField' => 'applications/search/field/PhabricatorSearchSubscribersField.php',
    'PhabricatorSearchTextField' => 'applications/search/field/PhabricatorSearchTextField.php',
    'PhabricatorSearchThreeStateField' => 'applications/search/field/PhabricatorSearchThreeStateField.php',
    'PhabricatorSearchTokenizerField' => 'applications/search/field/PhabricatorSearchTokenizerField.php',
    'PhabricatorSearchWorker' => 'applications/search/worker/PhabricatorSearchWorker.php',
    'PhabricatorSecurityConfigOptions' => 'applications/config/option/PhabricatorSecurityConfigOptions.php',
    'PhabricatorSecuritySetupCheck' => 'applications/config/check/PhabricatorSecuritySetupCheck.php',
    'PhabricatorSelectEditField' => 'applications/transactions/editfield/PhabricatorSelectEditField.php',
    'PhabricatorSendGridConfigOptions' => 'applications/config/option/PhabricatorSendGridConfigOptions.php',
    'PhabricatorSessionsSettingsPanel' => 'applications/settings/panel/PhabricatorSessionsSettingsPanel.php',
    'PhabricatorSettingsAddEmailAction' => 'applications/settings/action/PhabricatorSettingsAddEmailAction.php',
    'PhabricatorSettingsAdjustController' => 'applications/settings/controller/PhabricatorSettingsAdjustController.php',
    'PhabricatorSettingsApplication' => 'applications/settings/application/PhabricatorSettingsApplication.php',
    'PhabricatorSettingsMainController' => 'applications/settings/controller/PhabricatorSettingsMainController.php',
    'PhabricatorSettingsMainMenuBarExtension' => 'applications/settings/extension/PhabricatorSettingsMainMenuBarExtension.php',
    'PhabricatorSettingsPanel' => 'applications/settings/panel/PhabricatorSettingsPanel.php',
    'PhabricatorSetupCheck' => 'applications/config/check/PhabricatorSetupCheck.php',
    'PhabricatorSetupCheckTestCase' => 'applications/config/check/__tests__/PhabricatorSetupCheckTestCase.php',
    'PhabricatorSetupIssue' => 'applications/config/issue/PhabricatorSetupIssue.php',
    'PhabricatorSetupIssueUIExample' => 'applications/uiexample/examples/PhabricatorSetupIssueUIExample.php',
    'PhabricatorSetupIssueView' => 'applications/config/view/PhabricatorSetupIssueView.php',
    'PhabricatorShortSite' => 'aphront/site/PhabricatorShortSite.php',
    'PhabricatorSimpleEditType' => 'applications/transactions/edittype/PhabricatorSimpleEditType.php',
    'PhabricatorSite' => 'aphront/site/PhabricatorSite.php',
    'PhabricatorSlowvoteApplication' => 'applications/slowvote/application/PhabricatorSlowvoteApplication.php',
    'PhabricatorSlowvoteChoice' => 'applications/slowvote/storage/PhabricatorSlowvoteChoice.php',
    'PhabricatorSlowvoteCloseController' => 'applications/slowvote/controller/PhabricatorSlowvoteCloseController.php',
    'PhabricatorSlowvoteCommentController' => 'applications/slowvote/controller/PhabricatorSlowvoteCommentController.php',
    'PhabricatorSlowvoteController' => 'applications/slowvote/controller/PhabricatorSlowvoteController.php',
    'PhabricatorSlowvoteDAO' => 'applications/slowvote/storage/PhabricatorSlowvoteDAO.php',
    'PhabricatorSlowvoteDefaultViewCapability' => 'applications/slowvote/capability/PhabricatorSlowvoteDefaultViewCapability.php',
    'PhabricatorSlowvoteEditController' => 'applications/slowvote/controller/PhabricatorSlowvoteEditController.php',
    'PhabricatorSlowvoteEditor' => 'applications/slowvote/editor/PhabricatorSlowvoteEditor.php',
    'PhabricatorSlowvoteListController' => 'applications/slowvote/controller/PhabricatorSlowvoteListController.php',
    'PhabricatorSlowvoteMailReceiver' => 'applications/slowvote/mail/PhabricatorSlowvoteMailReceiver.php',
    'PhabricatorSlowvoteOption' => 'applications/slowvote/storage/PhabricatorSlowvoteOption.php',
    'PhabricatorSlowvotePoll' => 'applications/slowvote/storage/PhabricatorSlowvotePoll.php',
    'PhabricatorSlowvotePollController' => 'applications/slowvote/controller/PhabricatorSlowvotePollController.php',
    'PhabricatorSlowvotePollPHIDType' => 'applications/slowvote/phid/PhabricatorSlowvotePollPHIDType.php',
    'PhabricatorSlowvoteQuery' => 'applications/slowvote/query/PhabricatorSlowvoteQuery.php',
    'PhabricatorSlowvoteReplyHandler' => 'applications/slowvote/mail/PhabricatorSlowvoteReplyHandler.php',
    'PhabricatorSlowvoteSchemaSpec' => 'applications/slowvote/storage/PhabricatorSlowvoteSchemaSpec.php',
    'PhabricatorSlowvoteSearchEngine' => 'applications/slowvote/query/PhabricatorSlowvoteSearchEngine.php',
    'PhabricatorSlowvoteTransaction' => 'applications/slowvote/storage/PhabricatorSlowvoteTransaction.php',
    'PhabricatorSlowvoteTransactionComment' => 'applications/slowvote/storage/PhabricatorSlowvoteTransactionComment.php',
    'PhabricatorSlowvoteTransactionQuery' => 'applications/slowvote/query/PhabricatorSlowvoteTransactionQuery.php',
    'PhabricatorSlowvoteVoteController' => 'applications/slowvote/controller/PhabricatorSlowvoteVoteController.php',
    'PhabricatorSlug' => 'infrastructure/util/PhabricatorSlug.php',
    'PhabricatorSlugTestCase' => 'infrastructure/util/__tests__/PhabricatorSlugTestCase.php',
    'PhabricatorSortTableUIExample' => 'applications/uiexample/examples/PhabricatorSortTableUIExample.php',
    'PhabricatorSourceCodeView' => 'view/layout/PhabricatorSourceCodeView.php',
    'PhabricatorSpaceEditField' => 'applications/transactions/editfield/PhabricatorSpaceEditField.php',
    'PhabricatorSpacesApplication' => 'applications/spaces/application/PhabricatorSpacesApplication.php',
    'PhabricatorSpacesArchiveController' => 'applications/spaces/controller/PhabricatorSpacesArchiveController.php',
    'PhabricatorSpacesCapabilityCreateSpaces' => 'applications/spaces/capability/PhabricatorSpacesCapabilityCreateSpaces.php',
    'PhabricatorSpacesCapabilityDefaultEdit' => 'applications/spaces/capability/PhabricatorSpacesCapabilityDefaultEdit.php',
    'PhabricatorSpacesCapabilityDefaultView' => 'applications/spaces/capability/PhabricatorSpacesCapabilityDefaultView.php',
    'PhabricatorSpacesController' => 'applications/spaces/controller/PhabricatorSpacesController.php',
    'PhabricatorSpacesDAO' => 'applications/spaces/storage/PhabricatorSpacesDAO.php',
    'PhabricatorSpacesEditController' => 'applications/spaces/controller/PhabricatorSpacesEditController.php',
    'PhabricatorSpacesInterface' => 'applications/spaces/interface/PhabricatorSpacesInterface.php',
    'PhabricatorSpacesListController' => 'applications/spaces/controller/PhabricatorSpacesListController.php',
    'PhabricatorSpacesNamespace' => 'applications/spaces/storage/PhabricatorSpacesNamespace.php',
    'PhabricatorSpacesNamespaceDatasource' => 'applications/spaces/typeahead/PhabricatorSpacesNamespaceDatasource.php',
    'PhabricatorSpacesNamespaceEditor' => 'applications/spaces/editor/PhabricatorSpacesNamespaceEditor.php',
    'PhabricatorSpacesNamespacePHIDType' => 'applications/spaces/phid/PhabricatorSpacesNamespacePHIDType.php',
    'PhabricatorSpacesNamespaceQuery' => 'applications/spaces/query/PhabricatorSpacesNamespaceQuery.php',
    'PhabricatorSpacesNamespaceSearchEngine' => 'applications/spaces/query/PhabricatorSpacesNamespaceSearchEngine.php',
    'PhabricatorSpacesNamespaceTransaction' => 'applications/spaces/storage/PhabricatorSpacesNamespaceTransaction.php',
    'PhabricatorSpacesNamespaceTransactionQuery' => 'applications/spaces/query/PhabricatorSpacesNamespaceTransactionQuery.php',
    'PhabricatorSpacesNoAccessController' => 'applications/spaces/controller/PhabricatorSpacesNoAccessController.php',
    'PhabricatorSpacesRemarkupRule' => 'applications/spaces/remarkup/PhabricatorSpacesRemarkupRule.php',
    'PhabricatorSpacesSchemaSpec' => 'applications/spaces/storage/PhabricatorSpacesSchemaSpec.php',
    'PhabricatorSpacesSearchEngineExtension' => 'applications/spaces/engineextension/PhabricatorSpacesSearchEngineExtension.php',
    'PhabricatorSpacesSearchField' => 'applications/spaces/searchfield/PhabricatorSpacesSearchField.php',
    'PhabricatorSpacesTestCase' => 'applications/spaces/__tests__/PhabricatorSpacesTestCase.php',
    'PhabricatorSpacesViewController' => 'applications/spaces/controller/PhabricatorSpacesViewController.php',
    'PhabricatorStandardCustomField' => 'infrastructure/customfield/standard/PhabricatorStandardCustomField.php',
    'PhabricatorStandardCustomFieldBlueprints' => 'infrastructure/customfield/standard/PhabricatorStandardCustomFieldBlueprints.php',
    'PhabricatorStandardCustomFieldBool' => 'infrastructure/customfield/standard/PhabricatorStandardCustomFieldBool.php',
    'PhabricatorStandardCustomFieldCredential' => 'infrastructure/customfield/standard/PhabricatorStandardCustomFieldCredential.php',
    'PhabricatorStandardCustomFieldDatasource' => 'infrastructure/customfield/standard/PhabricatorStandardCustomFieldDatasource.php',
    'PhabricatorStandardCustomFieldDate' => 'infrastructure/customfield/standard/PhabricatorStandardCustomFieldDate.php',
    'PhabricatorStandardCustomFieldHeader' => 'infrastructure/customfield/standard/PhabricatorStandardCustomFieldHeader.php',
    'PhabricatorStandardCustomFieldInt' => 'infrastructure/customfield/standard/PhabricatorStandardCustomFieldInt.php',
    'PhabricatorStandardCustomFieldInterface' => 'infrastructure/customfield/interface/PhabricatorStandardCustomFieldInterface.php',
    'PhabricatorStandardCustomFieldLink' => 'infrastructure/customfield/standard/PhabricatorStandardCustomFieldLink.php',
    'PhabricatorStandardCustomFieldPHIDs' => 'infrastructure/customfield/standard/PhabricatorStandardCustomFieldPHIDs.php',
    'PhabricatorStandardCustomFieldRemarkup' => 'infrastructure/customfield/standard/PhabricatorStandardCustomFieldRemarkup.php',
    'PhabricatorStandardCustomFieldSelect' => 'infrastructure/customfield/standard/PhabricatorStandardCustomFieldSelect.php',
    'PhabricatorStandardCustomFieldText' => 'infrastructure/customfield/standard/PhabricatorStandardCustomFieldText.php',
    'PhabricatorStandardCustomFieldTokenizer' => 'infrastructure/customfield/standard/PhabricatorStandardCustomFieldTokenizer.php',
    'PhabricatorStandardCustomFieldUsers' => 'infrastructure/customfield/standard/PhabricatorStandardCustomFieldUsers.php',
    'PhabricatorStandardPageView' => 'view/page/PhabricatorStandardPageView.php',
    'PhabricatorStandardSelectCustomFieldDatasource' => 'infrastructure/customfield/datasource/PhabricatorStandardSelectCustomFieldDatasource.php',
    'PhabricatorStatusController' => 'applications/system/controller/PhabricatorStatusController.php',
    'PhabricatorStatusUIExample' => 'applications/uiexample/examples/PhabricatorStatusUIExample.php',
    'PhabricatorStorageFixtureScopeGuard' => 'infrastructure/testing/fixture/PhabricatorStorageFixtureScopeGuard.php',
    'PhabricatorStorageManagementAPI' => 'infrastructure/storage/management/PhabricatorStorageManagementAPI.php',
    'PhabricatorStorageManagementAdjustWorkflow' => 'infrastructure/storage/management/workflow/PhabricatorStorageManagementAdjustWorkflow.php',
    'PhabricatorStorageManagementDatabasesWorkflow' => 'infrastructure/storage/management/workflow/PhabricatorStorageManagementDatabasesWorkflow.php',
    'PhabricatorStorageManagementDestroyWorkflow' => 'infrastructure/storage/management/workflow/PhabricatorStorageManagementDestroyWorkflow.php',
    'PhabricatorStorageManagementDumpWorkflow' => 'infrastructure/storage/management/workflow/PhabricatorStorageManagementDumpWorkflow.php',
    'PhabricatorStorageManagementProbeWorkflow' => 'infrastructure/storage/management/workflow/PhabricatorStorageManagementProbeWorkflow.php',
    'PhabricatorStorageManagementQuickstartWorkflow' => 'infrastructure/storage/management/workflow/PhabricatorStorageManagementQuickstartWorkflow.php',
    'PhabricatorStorageManagementRenamespaceWorkflow' => 'infrastructure/storage/management/workflow/PhabricatorStorageManagementRenamespaceWorkflow.php',
    'PhabricatorStorageManagementShellWorkflow' => 'infrastructure/storage/management/workflow/PhabricatorStorageManagementShellWorkflow.php',
    'PhabricatorStorageManagementStatusWorkflow' => 'infrastructure/storage/management/workflow/PhabricatorStorageManagementStatusWorkflow.php',
    'PhabricatorStorageManagementUpgradeWorkflow' => 'infrastructure/storage/management/workflow/PhabricatorStorageManagementUpgradeWorkflow.php',
    'PhabricatorStorageManagementWorkflow' => 'infrastructure/storage/management/workflow/PhabricatorStorageManagementWorkflow.php',
    'PhabricatorStoragePatch' => 'infrastructure/storage/management/PhabricatorStoragePatch.php',
    'PhabricatorStorageSchemaSpec' => 'infrastructure/storage/schema/PhabricatorStorageSchemaSpec.php',
    'PhabricatorStorageSetupCheck' => 'applications/config/check/PhabricatorStorageSetupCheck.php',
    'PhabricatorStreamingProtocolAdapter' => 'infrastructure/daemon/bot/adapter/PhabricatorStreamingProtocolAdapter.php',
    'PhabricatorStringListEditField' => 'applications/transactions/editfield/PhabricatorStringListEditField.php',
    'PhabricatorSubscribableInterface' => 'applications/subscriptions/interface/PhabricatorSubscribableInterface.php',
    'PhabricatorSubscribedToObjectEdgeType' => 'applications/transactions/edges/PhabricatorSubscribedToObjectEdgeType.php',
    'PhabricatorSubscribersEditField' => 'applications/transactions/editfield/PhabricatorSubscribersEditField.php',
    'PhabricatorSubscribersQuery' => 'applications/subscriptions/query/PhabricatorSubscribersQuery.php',
    'PhabricatorSubscriptionTriggerClock' => 'infrastructure/daemon/workers/clock/PhabricatorSubscriptionTriggerClock.php',
    'PhabricatorSubscriptionsAddSelfHeraldAction' => 'applications/subscriptions/herald/PhabricatorSubscriptionsAddSelfHeraldAction.php',
    'PhabricatorSubscriptionsAddSubscribersHeraldAction' => 'applications/subscriptions/herald/PhabricatorSubscriptionsAddSubscribersHeraldAction.php',
    'PhabricatorSubscriptionsApplication' => 'applications/subscriptions/application/PhabricatorSubscriptionsApplication.php',
    'PhabricatorSubscriptionsEditController' => 'applications/subscriptions/controller/PhabricatorSubscriptionsEditController.php',
    'PhabricatorSubscriptionsEditEngineExtension' => 'applications/subscriptions/engineextension/PhabricatorSubscriptionsEditEngineExtension.php',
    'PhabricatorSubscriptionsEditor' => 'applications/subscriptions/editor/PhabricatorSubscriptionsEditor.php',
    'PhabricatorSubscriptionsFulltextEngineExtension' => 'applications/subscriptions/engineextension/PhabricatorSubscriptionsFulltextEngineExtension.php',
    'PhabricatorSubscriptionsHeraldAction' => 'applications/subscriptions/herald/PhabricatorSubscriptionsHeraldAction.php',
    'PhabricatorSubscriptionsListController' => 'applications/subscriptions/controller/PhabricatorSubscriptionsListController.php',
    'PhabricatorSubscriptionsRemoveSelfHeraldAction' => 'applications/subscriptions/herald/PhabricatorSubscriptionsRemoveSelfHeraldAction.php',
    'PhabricatorSubscriptionsRemoveSubscribersHeraldAction' => 'applications/subscriptions/herald/PhabricatorSubscriptionsRemoveSubscribersHeraldAction.php',
    'PhabricatorSubscriptionsSearchEngineAttachment' => 'applications/subscriptions/engineextension/PhabricatorSubscriptionsSearchEngineAttachment.php',
    'PhabricatorSubscriptionsSearchEngineExtension' => 'applications/subscriptions/engineextension/PhabricatorSubscriptionsSearchEngineExtension.php',
    'PhabricatorSubscriptionsSubscribeEmailCommand' => 'applications/subscriptions/command/PhabricatorSubscriptionsSubscribeEmailCommand.php',
    'PhabricatorSubscriptionsSubscribersPolicyRule' => 'applications/subscriptions/policyrule/PhabricatorSubscriptionsSubscribersPolicyRule.php',
    'PhabricatorSubscriptionsTransactionController' => 'applications/subscriptions/controller/PhabricatorSubscriptionsTransactionController.php',
    'PhabricatorSubscriptionsUIEventListener' => 'applications/subscriptions/events/PhabricatorSubscriptionsUIEventListener.php',
    'PhabricatorSubscriptionsUnsubscribeEmailCommand' => 'applications/subscriptions/command/PhabricatorSubscriptionsUnsubscribeEmailCommand.php',
    'PhabricatorSupportApplication' => 'applications/support/application/PhabricatorSupportApplication.php',
    'PhabricatorSyntaxHighlighter' => 'infrastructure/markup/PhabricatorSyntaxHighlighter.php',
    'PhabricatorSyntaxHighlightingConfigOptions' => 'applications/config/option/PhabricatorSyntaxHighlightingConfigOptions.php',
    'PhabricatorSystemAction' => 'applications/system/action/PhabricatorSystemAction.php',
    'PhabricatorSystemActionEngine' => 'applications/system/engine/PhabricatorSystemActionEngine.php',
    'PhabricatorSystemActionGarbageCollector' => 'applications/system/garbagecollector/PhabricatorSystemActionGarbageCollector.php',
    'PhabricatorSystemActionLog' => 'applications/system/storage/PhabricatorSystemActionLog.php',
    'PhabricatorSystemActionRateLimitException' => 'applications/system/exception/PhabricatorSystemActionRateLimitException.php',
    'PhabricatorSystemApplication' => 'applications/system/application/PhabricatorSystemApplication.php',
    'PhabricatorSystemDAO' => 'applications/system/storage/PhabricatorSystemDAO.php',
    'PhabricatorSystemDestructionGarbageCollector' => 'applications/system/garbagecollector/PhabricatorSystemDestructionGarbageCollector.php',
    'PhabricatorSystemDestructionLog' => 'applications/system/storage/PhabricatorSystemDestructionLog.php',
    'PhabricatorSystemRemoveDestroyWorkflow' => 'applications/system/management/PhabricatorSystemRemoveDestroyWorkflow.php',
    'PhabricatorSystemRemoveLogWorkflow' => 'applications/system/management/PhabricatorSystemRemoveLogWorkflow.php',
    'PhabricatorSystemRemoveWorkflow' => 'applications/system/management/PhabricatorSystemRemoveWorkflow.php',
    'PhabricatorSystemSelectEncodingController' => 'applications/system/controller/PhabricatorSystemSelectEncodingController.php',
    'PhabricatorSystemSelectHighlightController' => 'applications/system/controller/PhabricatorSystemSelectHighlightController.php',
    'PhabricatorTOTPAuthFactor' => 'applications/auth/factor/PhabricatorTOTPAuthFactor.php',
    'PhabricatorTOTPAuthFactorTestCase' => 'applications/auth/factor/__tests__/PhabricatorTOTPAuthFactorTestCase.php',
    'PhabricatorTaskmasterDaemon' => 'infrastructure/daemon/workers/PhabricatorTaskmasterDaemon.php',
    'PhabricatorTestApplication' => 'applications/base/controller/__tests__/PhabricatorTestApplication.php',
    'PhabricatorTestCase' => 'infrastructure/testing/PhabricatorTestCase.php',
    'PhabricatorTestController' => 'applications/base/controller/__tests__/PhabricatorTestController.php',
    'PhabricatorTestDataGenerator' => 'applications/lipsum/generator/PhabricatorTestDataGenerator.php',
    'PhabricatorTestNoCycleEdgeType' => 'applications/transactions/edges/PhabricatorTestNoCycleEdgeType.php',
    'PhabricatorTestStorageEngine' => 'applications/files/engine/PhabricatorTestStorageEngine.php',
    'PhabricatorTestWorker' => 'infrastructure/daemon/workers/__tests__/PhabricatorTestWorker.php',
    'PhabricatorTextAreaEditField' => 'applications/transactions/editfield/PhabricatorTextAreaEditField.php',
    'PhabricatorTextEditField' => 'applications/transactions/editfield/PhabricatorTextEditField.php',
    'PhabricatorTime' => 'infrastructure/time/PhabricatorTime.php',
    'PhabricatorTimeGuard' => 'infrastructure/time/PhabricatorTimeGuard.php',
    'PhabricatorTimeTestCase' => 'infrastructure/time/__tests__/PhabricatorTimeTestCase.php',
    'PhabricatorTimezoneSetupCheck' => 'applications/config/check/PhabricatorTimezoneSetupCheck.php',
    'PhabricatorToken' => 'applications/tokens/storage/PhabricatorToken.php',
    'PhabricatorTokenController' => 'applications/tokens/controller/PhabricatorTokenController.php',
    'PhabricatorTokenCount' => 'applications/tokens/storage/PhabricatorTokenCount.php',
    'PhabricatorTokenCountQuery' => 'applications/tokens/query/PhabricatorTokenCountQuery.php',
    'PhabricatorTokenDAO' => 'applications/tokens/storage/PhabricatorTokenDAO.php',
    'PhabricatorTokenDestructionEngineExtension' => 'applications/tokens/engineextension/PhabricatorTokenDestructionEngineExtension.php',
    'PhabricatorTokenGiveController' => 'applications/tokens/controller/PhabricatorTokenGiveController.php',
    'PhabricatorTokenGiven' => 'applications/tokens/storage/PhabricatorTokenGiven.php',
    'PhabricatorTokenGivenController' => 'applications/tokens/controller/PhabricatorTokenGivenController.php',
    'PhabricatorTokenGivenEditor' => 'applications/tokens/editor/PhabricatorTokenGivenEditor.php',
    'PhabricatorTokenGivenFeedStory' => 'applications/tokens/feed/PhabricatorTokenGivenFeedStory.php',
    'PhabricatorTokenGivenQuery' => 'applications/tokens/query/PhabricatorTokenGivenQuery.php',
    'PhabricatorTokenLeaderController' => 'applications/tokens/controller/PhabricatorTokenLeaderController.php',
    'PhabricatorTokenQuery' => 'applications/tokens/query/PhabricatorTokenQuery.php',
    'PhabricatorTokenReceiverInterface' => 'applications/tokens/interface/PhabricatorTokenReceiverInterface.php',
    'PhabricatorTokenReceiverQuery' => 'applications/tokens/query/PhabricatorTokenReceiverQuery.php',
    'PhabricatorTokenTokenPHIDType' => 'applications/tokens/phid/PhabricatorTokenTokenPHIDType.php',
    'PhabricatorTokenUIEventListener' => 'applications/tokens/event/PhabricatorTokenUIEventListener.php',
    'PhabricatorTokenizerEditField' => 'applications/transactions/editfield/PhabricatorTokenizerEditField.php',
    'PhabricatorTokensApplication' => 'applications/tokens/application/PhabricatorTokensApplication.php',
    'PhabricatorTokensSettingsPanel' => 'applications/settings/panel/PhabricatorTokensSettingsPanel.php',
    'PhabricatorTooltipUIExample' => 'applications/uiexample/examples/PhabricatorTooltipUIExample.php',
    'PhabricatorTransactions' => 'applications/transactions/constants/PhabricatorTransactions.php',
    'PhabricatorTransactionsApplication' => 'applications/transactions/application/PhabricatorTransactionsApplication.php',
    'PhabricatorTransactionsDestructionEngineExtension' => 'applications/transactions/engineextension/PhabricatorTransactionsDestructionEngineExtension.php',
    'PhabricatorTransactionsFulltextEngineExtension' => 'applications/transactions/engineextension/PhabricatorTransactionsFulltextEngineExtension.php',
    'PhabricatorTransformedFile' => 'applications/files/storage/PhabricatorTransformedFile.php',
    'PhabricatorTranslationsConfigOptions' => 'applications/config/option/PhabricatorTranslationsConfigOptions.php',
    'PhabricatorTriggerAction' => 'infrastructure/daemon/workers/action/PhabricatorTriggerAction.php',
    'PhabricatorTriggerClock' => 'infrastructure/daemon/workers/clock/PhabricatorTriggerClock.php',
    'PhabricatorTriggerClockTestCase' => 'infrastructure/daemon/workers/clock/__tests__/PhabricatorTriggerClockTestCase.php',
    'PhabricatorTriggerDaemon' => 'infrastructure/daemon/workers/PhabricatorTriggerDaemon.php',
    'PhabricatorTrivialTestCase' => 'infrastructure/testing/__tests__/PhabricatorTrivialTestCase.php',
    'PhabricatorTwitchAuthProvider' => 'applications/auth/provider/PhabricatorTwitchAuthProvider.php',
    'PhabricatorTwitterAuthProvider' => 'applications/auth/provider/PhabricatorTwitterAuthProvider.php',
    'PhabricatorTypeaheadApplication' => 'applications/typeahead/application/PhabricatorTypeaheadApplication.php',
    'PhabricatorTypeaheadCompositeDatasource' => 'applications/typeahead/datasource/PhabricatorTypeaheadCompositeDatasource.php',
    'PhabricatorTypeaheadDatasource' => 'applications/typeahead/datasource/PhabricatorTypeaheadDatasource.php',
    'PhabricatorTypeaheadDatasourceController' => 'applications/typeahead/controller/PhabricatorTypeaheadDatasourceController.php',
    'PhabricatorTypeaheadFunctionHelpController' => 'applications/typeahead/controller/PhabricatorTypeaheadFunctionHelpController.php',
    'PhabricatorTypeaheadInvalidTokenException' => 'applications/typeahead/exception/PhabricatorTypeaheadInvalidTokenException.php',
    'PhabricatorTypeaheadModularDatasourceController' => 'applications/typeahead/controller/PhabricatorTypeaheadModularDatasourceController.php',
    'PhabricatorTypeaheadMonogramDatasource' => 'applications/typeahead/datasource/PhabricatorTypeaheadMonogramDatasource.php',
    'PhabricatorTypeaheadResult' => 'applications/typeahead/storage/PhabricatorTypeaheadResult.php',
    'PhabricatorTypeaheadRuntimeCompositeDatasource' => 'applications/typeahead/datasource/PhabricatorTypeaheadRuntimeCompositeDatasource.php',
    'PhabricatorTypeaheadTokenView' => 'applications/typeahead/view/PhabricatorTypeaheadTokenView.php',
    'PhabricatorUIConfigOptions' => 'applications/config/option/PhabricatorUIConfigOptions.php',
    'PhabricatorUIExample' => 'applications/uiexample/examples/PhabricatorUIExample.php',
    'PhabricatorUIExampleRenderController' => 'applications/uiexample/controller/PhabricatorUIExampleRenderController.php',
    'PhabricatorUIExamplesApplication' => 'applications/uiexample/application/PhabricatorUIExamplesApplication.php',
    'PhabricatorUSEnglishTranslation' => 'infrastructure/internationalization/translation/PhabricatorUSEnglishTranslation.php',
    'PhabricatorUnitsTestCase' => 'view/__tests__/PhabricatorUnitsTestCase.php',
    'PhabricatorUnsubscribedFromObjectEdgeType' => 'applications/transactions/edges/PhabricatorUnsubscribedFromObjectEdgeType.php',
    'PhabricatorUser' => 'applications/people/storage/PhabricatorUser.php',
    'PhabricatorUserBlurbField' => 'applications/people/customfield/PhabricatorUserBlurbField.php',
    'PhabricatorUserConfigOptions' => 'applications/people/config/PhabricatorUserConfigOptions.php',
    'PhabricatorUserConfiguredCustomField' => 'applications/people/customfield/PhabricatorUserConfiguredCustomField.php',
    'PhabricatorUserConfiguredCustomFieldStorage' => 'applications/people/storage/PhabricatorUserConfiguredCustomFieldStorage.php',
    'PhabricatorUserCustomField' => 'applications/people/customfield/PhabricatorUserCustomField.php',
    'PhabricatorUserCustomFieldNumericIndex' => 'applications/people/storage/PhabricatorUserCustomFieldNumericIndex.php',
    'PhabricatorUserCustomFieldStringIndex' => 'applications/people/storage/PhabricatorUserCustomFieldStringIndex.php',
    'PhabricatorUserDAO' => 'applications/people/storage/PhabricatorUserDAO.php',
    'PhabricatorUserEditor' => 'applications/people/editor/PhabricatorUserEditor.php',
    'PhabricatorUserEditorTestCase' => 'applications/people/editor/__tests__/PhabricatorUserEditorTestCase.php',
    'PhabricatorUserEmail' => 'applications/people/storage/PhabricatorUserEmail.php',
    'PhabricatorUserEmailTestCase' => 'applications/people/storage/__tests__/PhabricatorUserEmailTestCase.php',
    'PhabricatorUserFulltextEngine' => 'applications/people/search/PhabricatorUserFulltextEngine.php',
    'PhabricatorUserLog' => 'applications/people/storage/PhabricatorUserLog.php',
    'PhabricatorUserLogView' => 'applications/people/view/PhabricatorUserLogView.php',
    'PhabricatorUserPHIDResolver' => 'applications/phid/resolver/PhabricatorUserPHIDResolver.php',
    'PhabricatorUserPreferences' => 'applications/settings/storage/PhabricatorUserPreferences.php',
    'PhabricatorUserProfile' => 'applications/people/storage/PhabricatorUserProfile.php',
    'PhabricatorUserProfileEditor' => 'applications/people/editor/PhabricatorUserProfileEditor.php',
    'PhabricatorUserRealNameField' => 'applications/people/customfield/PhabricatorUserRealNameField.php',
    'PhabricatorUserRolesField' => 'applications/people/customfield/PhabricatorUserRolesField.php',
    'PhabricatorUserSchemaSpec' => 'applications/people/storage/PhabricatorUserSchemaSpec.php',
    'PhabricatorUserSinceField' => 'applications/people/customfield/PhabricatorUserSinceField.php',
    'PhabricatorUserStatusField' => 'applications/people/customfield/PhabricatorUserStatusField.php',
    'PhabricatorUserTestCase' => 'applications/people/storage/__tests__/PhabricatorUserTestCase.php',
    'PhabricatorUserTitleField' => 'applications/people/customfield/PhabricatorUserTitleField.php',
    'PhabricatorUserTransaction' => 'applications/people/storage/PhabricatorUserTransaction.php',
    'PhabricatorUsersEditField' => 'applications/transactions/editfield/PhabricatorUsersEditField.php',
    'PhabricatorUsersPolicyRule' => 'applications/people/policyrule/PhabricatorUsersPolicyRule.php',
    'PhabricatorUsersSearchField' => 'applications/people/searchfield/PhabricatorUsersSearchField.php',
    'PhabricatorVCSResponse' => 'applications/repository/response/PhabricatorVCSResponse.php',
    'PhabricatorVersionedDraft' => 'applications/draft/storage/PhabricatorVersionedDraft.php',
    'PhabricatorVeryWowEnglishTranslation' => 'infrastructure/internationalization/translation/PhabricatorVeryWowEnglishTranslation.php',
    'PhabricatorViewerDatasource' => 'applications/people/typeahead/PhabricatorViewerDatasource.php',
    'PhabricatorWatcherHasObjectEdgeType' => 'applications/transactions/edges/PhabricatorWatcherHasObjectEdgeType.php',
    'PhabricatorWordPressAuthProvider' => 'applications/auth/provider/PhabricatorWordPressAuthProvider.php',
    'PhabricatorWorker' => 'infrastructure/daemon/workers/PhabricatorWorker.php',
    'PhabricatorWorkerActiveTask' => 'infrastructure/daemon/workers/storage/PhabricatorWorkerActiveTask.php',
    'PhabricatorWorkerArchiveTask' => 'infrastructure/daemon/workers/storage/PhabricatorWorkerArchiveTask.php',
    'PhabricatorWorkerArchiveTaskQuery' => 'infrastructure/daemon/workers/query/PhabricatorWorkerArchiveTaskQuery.php',
    'PhabricatorWorkerBulkJob' => 'infrastructure/daemon/workers/storage/PhabricatorWorkerBulkJob.php',
    'PhabricatorWorkerBulkJobCreateWorker' => 'infrastructure/daemon/workers/bulk/PhabricatorWorkerBulkJobCreateWorker.php',
    'PhabricatorWorkerBulkJobEditor' => 'infrastructure/daemon/workers/editor/PhabricatorWorkerBulkJobEditor.php',
    'PhabricatorWorkerBulkJobPHIDType' => 'infrastructure/daemon/workers/phid/PhabricatorWorkerBulkJobPHIDType.php',
    'PhabricatorWorkerBulkJobQuery' => 'infrastructure/daemon/workers/query/PhabricatorWorkerBulkJobQuery.php',
    'PhabricatorWorkerBulkJobSearchEngine' => 'infrastructure/daemon/workers/query/PhabricatorWorkerBulkJobSearchEngine.php',
    'PhabricatorWorkerBulkJobTaskWorker' => 'infrastructure/daemon/workers/bulk/PhabricatorWorkerBulkJobTaskWorker.php',
    'PhabricatorWorkerBulkJobTestCase' => 'infrastructure/daemon/workers/__tests__/PhabricatorWorkerBulkJobTestCase.php',
    'PhabricatorWorkerBulkJobTransaction' => 'infrastructure/daemon/workers/storage/PhabricatorWorkerBulkJobTransaction.php',
    'PhabricatorWorkerBulkJobTransactionQuery' => 'infrastructure/daemon/workers/query/PhabricatorWorkerBulkJobTransactionQuery.php',
    'PhabricatorWorkerBulkJobType' => 'infrastructure/daemon/workers/bulk/PhabricatorWorkerBulkJobType.php',
    'PhabricatorWorkerBulkJobWorker' => 'infrastructure/daemon/workers/bulk/PhabricatorWorkerBulkJobWorker.php',
    'PhabricatorWorkerBulkTask' => 'infrastructure/daemon/workers/storage/PhabricatorWorkerBulkTask.php',
    'PhabricatorWorkerDAO' => 'infrastructure/daemon/workers/storage/PhabricatorWorkerDAO.php',
    'PhabricatorWorkerDestructionEngineExtension' => 'infrastructure/daemon/workers/engineextension/PhabricatorWorkerDestructionEngineExtension.php',
    'PhabricatorWorkerLeaseQuery' => 'infrastructure/daemon/workers/query/PhabricatorWorkerLeaseQuery.php',
    'PhabricatorWorkerManagementCancelWorkflow' => 'infrastructure/daemon/workers/management/PhabricatorWorkerManagementCancelWorkflow.php',
    'PhabricatorWorkerManagementExecuteWorkflow' => 'infrastructure/daemon/workers/management/PhabricatorWorkerManagementExecuteWorkflow.php',
    'PhabricatorWorkerManagementFloodWorkflow' => 'infrastructure/daemon/workers/management/PhabricatorWorkerManagementFloodWorkflow.php',
    'PhabricatorWorkerManagementFreeWorkflow' => 'infrastructure/daemon/workers/management/PhabricatorWorkerManagementFreeWorkflow.php',
    'PhabricatorWorkerManagementRetryWorkflow' => 'infrastructure/daemon/workers/management/PhabricatorWorkerManagementRetryWorkflow.php',
    'PhabricatorWorkerManagementWorkflow' => 'infrastructure/daemon/workers/management/PhabricatorWorkerManagementWorkflow.php',
    'PhabricatorWorkerPermanentFailureException' => 'infrastructure/daemon/workers/exception/PhabricatorWorkerPermanentFailureException.php',
    'PhabricatorWorkerSchemaSpec' => 'infrastructure/daemon/workers/storage/PhabricatorWorkerSchemaSpec.php',
    'PhabricatorWorkerTask' => 'infrastructure/daemon/workers/storage/PhabricatorWorkerTask.php',
    'PhabricatorWorkerTaskData' => 'infrastructure/daemon/workers/storage/PhabricatorWorkerTaskData.php',
    'PhabricatorWorkerTaskDetailController' => 'applications/daemon/controller/PhabricatorWorkerTaskDetailController.php',
    'PhabricatorWorkerTestCase' => 'infrastructure/daemon/workers/__tests__/PhabricatorWorkerTestCase.php',
    'PhabricatorWorkerTrigger' => 'infrastructure/daemon/workers/storage/PhabricatorWorkerTrigger.php',
    'PhabricatorWorkerTriggerEvent' => 'infrastructure/daemon/workers/storage/PhabricatorWorkerTriggerEvent.php',
    'PhabricatorWorkerTriggerManagementFireWorkflow' => 'infrastructure/daemon/workers/management/PhabricatorWorkerTriggerManagementFireWorkflow.php',
    'PhabricatorWorkerTriggerManagementWorkflow' => 'infrastructure/daemon/workers/management/PhabricatorWorkerTriggerManagementWorkflow.php',
    'PhabricatorWorkerTriggerPHIDType' => 'infrastructure/daemon/workers/phid/PhabricatorWorkerTriggerPHIDType.php',
    'PhabricatorWorkerTriggerQuery' => 'infrastructure/daemon/workers/query/PhabricatorWorkerTriggerQuery.php',
    'PhabricatorWorkerYieldException' => 'infrastructure/daemon/workers/exception/PhabricatorWorkerYieldException.php',
    'PhabricatorWorkingCopyDiscoveryTestCase' => 'applications/repository/engine/__tests__/PhabricatorWorkingCopyDiscoveryTestCase.php',
    'PhabricatorWorkingCopyPullTestCase' => 'applications/repository/engine/__tests__/PhabricatorWorkingCopyPullTestCase.php',
    'PhabricatorWorkingCopyTestCase' => 'applications/repository/engine/__tests__/PhabricatorWorkingCopyTestCase.php',
    'PhabricatorXHPASTDAO' => 'applications/phpast/storage/PhabricatorXHPASTDAO.php',
    'PhabricatorXHPASTParseTree' => 'applications/phpast/storage/PhabricatorXHPASTParseTree.php',
    'PhabricatorXHPASTViewController' => 'applications/phpast/controller/PhabricatorXHPASTViewController.php',
    'PhabricatorXHPASTViewFrameController' => 'applications/phpast/controller/PhabricatorXHPASTViewFrameController.php',
    'PhabricatorXHPASTViewFramesetController' => 'applications/phpast/controller/PhabricatorXHPASTViewFramesetController.php',
    'PhabricatorXHPASTViewInputController' => 'applications/phpast/controller/PhabricatorXHPASTViewInputController.php',
    'PhabricatorXHPASTViewPanelController' => 'applications/phpast/controller/PhabricatorXHPASTViewPanelController.php',
    'PhabricatorXHPASTViewRunController' => 'applications/phpast/controller/PhabricatorXHPASTViewRunController.php',
    'PhabricatorXHPASTViewStreamController' => 'applications/phpast/controller/PhabricatorXHPASTViewStreamController.php',
    'PhabricatorXHPASTViewTreeController' => 'applications/phpast/controller/PhabricatorXHPASTViewTreeController.php',
    'PhabricatorXHProfApplication' => 'applications/xhprof/application/PhabricatorXHProfApplication.php',
    'PhabricatorXHProfController' => 'applications/xhprof/controller/PhabricatorXHProfController.php',
    'PhabricatorXHProfDAO' => 'applications/xhprof/storage/PhabricatorXHProfDAO.php',
    'PhabricatorXHProfProfileController' => 'applications/xhprof/controller/PhabricatorXHProfProfileController.php',
    'PhabricatorXHProfProfileSymbolView' => 'applications/xhprof/view/PhabricatorXHProfProfileSymbolView.php',
    'PhabricatorXHProfProfileTopLevelView' => 'applications/xhprof/view/PhabricatorXHProfProfileTopLevelView.php',
    'PhabricatorXHProfProfileView' => 'applications/xhprof/view/PhabricatorXHProfProfileView.php',
    'PhabricatorXHProfSample' => 'applications/xhprof/storage/PhabricatorXHProfSample.php',
    'PhabricatorXHProfSampleListController' => 'applications/xhprof/controller/PhabricatorXHProfSampleListController.php',
    'PhabricatorYoutubeRemarkupRule' => 'infrastructure/markup/rule/PhabricatorYoutubeRemarkupRule.php',
    'PhameBlog' => 'applications/phame/storage/PhameBlog.php',
    'PhameBlogArchiveController' => 'applications/phame/controller/blog/PhameBlogArchiveController.php',
    'PhameBlogController' => 'applications/phame/controller/blog/PhameBlogController.php',
    'PhameBlogCreateCapability' => 'applications/phame/capability/PhameBlogCreateCapability.php',
    'PhameBlogEditConduitAPIMethod' => 'applications/phame/conduit/PhameBlogEditConduitAPIMethod.php',
    'PhameBlogEditController' => 'applications/phame/controller/blog/PhameBlogEditController.php',
    'PhameBlogEditEngine' => 'applications/phame/editor/PhameBlogEditEngine.php',
    'PhameBlogEditor' => 'applications/phame/editor/PhameBlogEditor.php',
    'PhameBlogFeedController' => 'applications/phame/controller/blog/PhameBlogFeedController.php',
    'PhameBlogListController' => 'applications/phame/controller/blog/PhameBlogListController.php',
    'PhameBlogListView' => 'applications/phame/view/PhameBlogListView.php',
    'PhameBlogManageController' => 'applications/phame/controller/blog/PhameBlogManageController.php',
    'PhameBlogProfilePictureController' => 'applications/phame/controller/blog/PhameBlogProfilePictureController.php',
    'PhameBlogQuery' => 'applications/phame/query/PhameBlogQuery.php',
    'PhameBlogReplyHandler' => 'applications/phame/mail/PhameBlogReplyHandler.php',
    'PhameBlogSearchConduitAPIMethod' => 'applications/phame/conduit/PhameBlogSearchConduitAPIMethod.php',
    'PhameBlogSearchEngine' => 'applications/phame/query/PhameBlogSearchEngine.php',
    'PhameBlogSite' => 'applications/phame/site/PhameBlogSite.php',
    'PhameBlogTransaction' => 'applications/phame/storage/PhameBlogTransaction.php',
    'PhameBlogTransactionQuery' => 'applications/phame/query/PhameBlogTransactionQuery.php',
    'PhameBlogViewController' => 'applications/phame/controller/blog/PhameBlogViewController.php',
    'PhameConstants' => 'applications/phame/constants/PhameConstants.php',
    'PhameController' => 'applications/phame/controller/PhameController.php',
    'PhameDAO' => 'applications/phame/storage/PhameDAO.php',
    'PhameDescriptionView' => 'applications/phame/view/PhameDescriptionView.php',
    'PhameDraftListView' => 'applications/phame/view/PhameDraftListView.php',
    'PhameHomeController' => 'applications/phame/controller/PhameHomeController.php',
    'PhameLiveController' => 'applications/phame/controller/PhameLiveController.php',
    'PhameNextPostView' => 'applications/phame/view/PhameNextPostView.php',
    'PhamePost' => 'applications/phame/storage/PhamePost.php',
    'PhamePostCommentController' => 'applications/phame/controller/post/PhamePostCommentController.php',
    'PhamePostController' => 'applications/phame/controller/post/PhamePostController.php',
    'PhamePostEditConduitAPIMethod' => 'applications/phame/conduit/PhamePostEditConduitAPIMethod.php',
    'PhamePostEditController' => 'applications/phame/controller/post/PhamePostEditController.php',
    'PhamePostEditEngine' => 'applications/phame/editor/PhamePostEditEngine.php',
    'PhamePostEditor' => 'applications/phame/editor/PhamePostEditor.php',
    'PhamePostHistoryController' => 'applications/phame/controller/post/PhamePostHistoryController.php',
    'PhamePostListController' => 'applications/phame/controller/post/PhamePostListController.php',
    'PhamePostListView' => 'applications/phame/view/PhamePostListView.php',
    'PhamePostMailReceiver' => 'applications/phame/mail/PhamePostMailReceiver.php',
    'PhamePostMoveController' => 'applications/phame/controller/post/PhamePostMoveController.php',
    'PhamePostPublishController' => 'applications/phame/controller/post/PhamePostPublishController.php',
    'PhamePostQuery' => 'applications/phame/query/PhamePostQuery.php',
    'PhamePostReplyHandler' => 'applications/phame/mail/PhamePostReplyHandler.php',
    'PhamePostSearchConduitAPIMethod' => 'applications/phame/conduit/PhamePostSearchConduitAPIMethod.php',
    'PhamePostSearchEngine' => 'applications/phame/query/PhamePostSearchEngine.php',
    'PhamePostTransaction' => 'applications/phame/storage/PhamePostTransaction.php',
    'PhamePostTransactionComment' => 'applications/phame/storage/PhamePostTransactionComment.php',
    'PhamePostTransactionQuery' => 'applications/phame/query/PhamePostTransactionQuery.php',
    'PhamePostViewController' => 'applications/phame/controller/post/PhamePostViewController.php',
    'PhameSchemaSpec' => 'applications/phame/storage/PhameSchemaSpec.php',
    'PhameSite' => 'applications/phame/site/PhameSite.php',
    'PhluxController' => 'applications/phlux/controller/PhluxController.php',
    'PhluxDAO' => 'applications/phlux/storage/PhluxDAO.php',
    'PhluxEditController' => 'applications/phlux/controller/PhluxEditController.php',
    'PhluxListController' => 'applications/phlux/controller/PhluxListController.php',
    'PhluxTransaction' => 'applications/phlux/storage/PhluxTransaction.php',
    'PhluxTransactionQuery' => 'applications/phlux/query/PhluxTransactionQuery.php',
    'PhluxVariable' => 'applications/phlux/storage/PhluxVariable.php',
    'PhluxVariableEditor' => 'applications/phlux/editor/PhluxVariableEditor.php',
    'PhluxVariablePHIDType' => 'applications/phlux/phid/PhluxVariablePHIDType.php',
    'PhluxVariableQuery' => 'applications/phlux/query/PhluxVariableQuery.php',
    'PhluxViewController' => 'applications/phlux/controller/PhluxViewController.php',
    'PholioActionMenuEventListener' => 'applications/pholio/event/PholioActionMenuEventListener.php',
    'PholioController' => 'applications/pholio/controller/PholioController.php',
    'PholioDAO' => 'applications/pholio/storage/PholioDAO.php',
    'PholioDefaultEditCapability' => 'applications/pholio/capability/PholioDefaultEditCapability.php',
    'PholioDefaultViewCapability' => 'applications/pholio/capability/PholioDefaultViewCapability.php',
    'PholioImage' => 'applications/pholio/storage/PholioImage.php',
    'PholioImagePHIDType' => 'applications/pholio/phid/PholioImagePHIDType.php',
    'PholioImageQuery' => 'applications/pholio/query/PholioImageQuery.php',
    'PholioImageUploadController' => 'applications/pholio/controller/PholioImageUploadController.php',
    'PholioInlineController' => 'applications/pholio/controller/PholioInlineController.php',
    'PholioInlineListController' => 'applications/pholio/controller/PholioInlineListController.php',
    'PholioMock' => 'applications/pholio/storage/PholioMock.php',
    'PholioMockArchiveController' => 'applications/pholio/controller/PholioMockArchiveController.php',
    'PholioMockAuthorHeraldField' => 'applications/pholio/herald/PholioMockAuthorHeraldField.php',
    'PholioMockCommentController' => 'applications/pholio/controller/PholioMockCommentController.php',
    'PholioMockDescriptionHeraldField' => 'applications/pholio/herald/PholioMockDescriptionHeraldField.php',
    'PholioMockEditController' => 'applications/pholio/controller/PholioMockEditController.php',
    'PholioMockEditor' => 'applications/pholio/editor/PholioMockEditor.php',
    'PholioMockEmbedView' => 'applications/pholio/view/PholioMockEmbedView.php',
    'PholioMockFulltextEngine' => 'applications/pholio/search/PholioMockFulltextEngine.php',
    'PholioMockHasTaskEdgeType' => 'applications/pholio/edge/PholioMockHasTaskEdgeType.php',
    'PholioMockHeraldField' => 'applications/pholio/herald/PholioMockHeraldField.php',
    'PholioMockHeraldFieldGroup' => 'applications/pholio/herald/PholioMockHeraldFieldGroup.php',
    'PholioMockImagesView' => 'applications/pholio/view/PholioMockImagesView.php',
    'PholioMockListController' => 'applications/pholio/controller/PholioMockListController.php',
    'PholioMockMailReceiver' => 'applications/pholio/mail/PholioMockMailReceiver.php',
    'PholioMockNameHeraldField' => 'applications/pholio/herald/PholioMockNameHeraldField.php',
    'PholioMockPHIDType' => 'applications/pholio/phid/PholioMockPHIDType.php',
    'PholioMockQuery' => 'applications/pholio/query/PholioMockQuery.php',
    'PholioMockSearchEngine' => 'applications/pholio/query/PholioMockSearchEngine.php',
    'PholioMockThumbGridView' => 'applications/pholio/view/PholioMockThumbGridView.php',
    'PholioMockViewController' => 'applications/pholio/controller/PholioMockViewController.php',
    'PholioRemarkupRule' => 'applications/pholio/remarkup/PholioRemarkupRule.php',
    'PholioReplyHandler' => 'applications/pholio/mail/PholioReplyHandler.php',
    'PholioSchemaSpec' => 'applications/pholio/storage/PholioSchemaSpec.php',
    'PholioTransaction' => 'applications/pholio/storage/PholioTransaction.php',
    'PholioTransactionComment' => 'applications/pholio/storage/PholioTransactionComment.php',
    'PholioTransactionQuery' => 'applications/pholio/query/PholioTransactionQuery.php',
    'PholioTransactionView' => 'applications/pholio/view/PholioTransactionView.php',
    'PholioUploadedImageView' => 'applications/pholio/view/PholioUploadedImageView.php',
    'PhortuneAccount' => 'applications/phortune/storage/PhortuneAccount.php',
    'PhortuneAccountEditController' => 'applications/phortune/controller/PhortuneAccountEditController.php',
    'PhortuneAccountEditor' => 'applications/phortune/editor/PhortuneAccountEditor.php',
    'PhortuneAccountHasMemberEdgeType' => 'applications/phortune/edge/PhortuneAccountHasMemberEdgeType.php',
    'PhortuneAccountListController' => 'applications/phortune/controller/PhortuneAccountListController.php',
    'PhortuneAccountPHIDType' => 'applications/phortune/phid/PhortuneAccountPHIDType.php',
    'PhortuneAccountQuery' => 'applications/phortune/query/PhortuneAccountQuery.php',
    'PhortuneAccountTransaction' => 'applications/phortune/storage/PhortuneAccountTransaction.php',
    'PhortuneAccountTransactionQuery' => 'applications/phortune/query/PhortuneAccountTransactionQuery.php',
    'PhortuneAccountViewController' => 'applications/phortune/controller/PhortuneAccountViewController.php',
    'PhortuneAdHocCart' => 'applications/phortune/cart/PhortuneAdHocCart.php',
    'PhortuneAdHocProduct' => 'applications/phortune/product/PhortuneAdHocProduct.php',
    'PhortuneCart' => 'applications/phortune/storage/PhortuneCart.php',
    'PhortuneCartAcceptController' => 'applications/phortune/controller/PhortuneCartAcceptController.php',
    'PhortuneCartCancelController' => 'applications/phortune/controller/PhortuneCartCancelController.php',
    'PhortuneCartCheckoutController' => 'applications/phortune/controller/PhortuneCartCheckoutController.php',
    'PhortuneCartController' => 'applications/phortune/controller/PhortuneCartController.php',
    'PhortuneCartEditor' => 'applications/phortune/editor/PhortuneCartEditor.php',
    'PhortuneCartImplementation' => 'applications/phortune/cart/PhortuneCartImplementation.php',
    'PhortuneCartListController' => 'applications/phortune/controller/PhortuneCartListController.php',
    'PhortuneCartPHIDType' => 'applications/phortune/phid/PhortuneCartPHIDType.php',
    'PhortuneCartQuery' => 'applications/phortune/query/PhortuneCartQuery.php',
    'PhortuneCartReplyHandler' => 'applications/phortune/mail/PhortuneCartReplyHandler.php',
    'PhortuneCartSearchEngine' => 'applications/phortune/query/PhortuneCartSearchEngine.php',
    'PhortuneCartTransaction' => 'applications/phortune/storage/PhortuneCartTransaction.php',
    'PhortuneCartTransactionQuery' => 'applications/phortune/query/PhortuneCartTransactionQuery.php',
    'PhortuneCartUpdateController' => 'applications/phortune/controller/PhortuneCartUpdateController.php',
    'PhortuneCartViewController' => 'applications/phortune/controller/PhortuneCartViewController.php',
    'PhortuneCharge' => 'applications/phortune/storage/PhortuneCharge.php',
    'PhortuneChargeListController' => 'applications/phortune/controller/PhortuneChargeListController.php',
    'PhortuneChargePHIDType' => 'applications/phortune/phid/PhortuneChargePHIDType.php',
    'PhortuneChargeQuery' => 'applications/phortune/query/PhortuneChargeQuery.php',
    'PhortuneChargeSearchEngine' => 'applications/phortune/query/PhortuneChargeSearchEngine.php',
    'PhortuneChargeTableView' => 'applications/phortune/view/PhortuneChargeTableView.php',
    'PhortuneConstants' => 'applications/phortune/constants/PhortuneConstants.php',
    'PhortuneController' => 'applications/phortune/controller/PhortuneController.php',
    'PhortuneCreditCardForm' => 'applications/phortune/view/PhortuneCreditCardForm.php',
    'PhortuneCurrency' => 'applications/phortune/currency/PhortuneCurrency.php',
    'PhortuneCurrencySerializer' => 'applications/phortune/currency/PhortuneCurrencySerializer.php',
    'PhortuneCurrencyTestCase' => 'applications/phortune/currency/__tests__/PhortuneCurrencyTestCase.php',
    'PhortuneDAO' => 'applications/phortune/storage/PhortuneDAO.php',
    'PhortuneErrCode' => 'applications/phortune/constants/PhortuneErrCode.php',
    'PhortuneLandingController' => 'applications/phortune/controller/PhortuneLandingController.php',
    'PhortuneMemberHasAccountEdgeType' => 'applications/phortune/edge/PhortuneMemberHasAccountEdgeType.php',
    'PhortuneMemberHasMerchantEdgeType' => 'applications/phortune/edge/PhortuneMemberHasMerchantEdgeType.php',
    'PhortuneMerchant' => 'applications/phortune/storage/PhortuneMerchant.php',
    'PhortuneMerchantCapability' => 'applications/phortune/capability/PhortuneMerchantCapability.php',
    'PhortuneMerchantController' => 'applications/phortune/controller/PhortuneMerchantController.php',
    'PhortuneMerchantEditController' => 'applications/phortune/controller/PhortuneMerchantEditController.php',
    'PhortuneMerchantEditor' => 'applications/phortune/editor/PhortuneMerchantEditor.php',
    'PhortuneMerchantHasMemberEdgeType' => 'applications/phortune/edge/PhortuneMerchantHasMemberEdgeType.php',
    'PhortuneMerchantInvoiceCreateController' => 'applications/phortune/controller/PhortuneMerchantInvoiceCreateController.php',
    'PhortuneMerchantListController' => 'applications/phortune/controller/PhortuneMerchantListController.php',
    'PhortuneMerchantPHIDType' => 'applications/phortune/phid/PhortuneMerchantPHIDType.php',
    'PhortuneMerchantQuery' => 'applications/phortune/query/PhortuneMerchantQuery.php',
    'PhortuneMerchantSearchEngine' => 'applications/phortune/query/PhortuneMerchantSearchEngine.php',
    'PhortuneMerchantTransaction' => 'applications/phortune/storage/PhortuneMerchantTransaction.php',
    'PhortuneMerchantTransactionQuery' => 'applications/phortune/query/PhortuneMerchantTransactionQuery.php',
    'PhortuneMerchantViewController' => 'applications/phortune/controller/PhortuneMerchantViewController.php',
    'PhortuneMonthYearExpiryControl' => 'applications/phortune/control/PhortuneMonthYearExpiryControl.php',
    'PhortuneOrderTableView' => 'applications/phortune/view/PhortuneOrderTableView.php',
    'PhortunePayPalPaymentProvider' => 'applications/phortune/provider/PhortunePayPalPaymentProvider.php',
    'PhortunePaymentMethod' => 'applications/phortune/storage/PhortunePaymentMethod.php',
    'PhortunePaymentMethodCreateController' => 'applications/phortune/controller/PhortunePaymentMethodCreateController.php',
    'PhortunePaymentMethodDisableController' => 'applications/phortune/controller/PhortunePaymentMethodDisableController.php',
    'PhortunePaymentMethodEditController' => 'applications/phortune/controller/PhortunePaymentMethodEditController.php',
    'PhortunePaymentMethodPHIDType' => 'applications/phortune/phid/PhortunePaymentMethodPHIDType.php',
    'PhortunePaymentMethodQuery' => 'applications/phortune/query/PhortunePaymentMethodQuery.php',
    'PhortunePaymentProvider' => 'applications/phortune/provider/PhortunePaymentProvider.php',
    'PhortunePaymentProviderConfig' => 'applications/phortune/storage/PhortunePaymentProviderConfig.php',
    'PhortunePaymentProviderConfigEditor' => 'applications/phortune/editor/PhortunePaymentProviderConfigEditor.php',
    'PhortunePaymentProviderConfigQuery' => 'applications/phortune/query/PhortunePaymentProviderConfigQuery.php',
    'PhortunePaymentProviderConfigTransaction' => 'applications/phortune/storage/PhortunePaymentProviderConfigTransaction.php',
    'PhortunePaymentProviderConfigTransactionQuery' => 'applications/phortune/query/PhortunePaymentProviderConfigTransactionQuery.php',
    'PhortunePaymentProviderPHIDType' => 'applications/phortune/phid/PhortunePaymentProviderPHIDType.php',
    'PhortunePaymentProviderTestCase' => 'applications/phortune/provider/__tests__/PhortunePaymentProviderTestCase.php',
    'PhortuneProduct' => 'applications/phortune/storage/PhortuneProduct.php',
    'PhortuneProductImplementation' => 'applications/phortune/product/PhortuneProductImplementation.php',
    'PhortuneProductListController' => 'applications/phortune/controller/PhortuneProductListController.php',
    'PhortuneProductPHIDType' => 'applications/phortune/phid/PhortuneProductPHIDType.php',
    'PhortuneProductQuery' => 'applications/phortune/query/PhortuneProductQuery.php',
    'PhortuneProductViewController' => 'applications/phortune/controller/PhortuneProductViewController.php',
    'PhortuneProviderActionController' => 'applications/phortune/controller/PhortuneProviderActionController.php',
    'PhortuneProviderDisableController' => 'applications/phortune/controller/PhortuneProviderDisableController.php',
    'PhortuneProviderEditController' => 'applications/phortune/controller/PhortuneProviderEditController.php',
    'PhortunePurchase' => 'applications/phortune/storage/PhortunePurchase.php',
    'PhortunePurchasePHIDType' => 'applications/phortune/phid/PhortunePurchasePHIDType.php',
    'PhortunePurchaseQuery' => 'applications/phortune/query/PhortunePurchaseQuery.php',
    'PhortuneSchemaSpec' => 'applications/phortune/storage/PhortuneSchemaSpec.php',
    'PhortuneStripePaymentProvider' => 'applications/phortune/provider/PhortuneStripePaymentProvider.php',
    'PhortuneSubscription' => 'applications/phortune/storage/PhortuneSubscription.php',
    'PhortuneSubscriptionCart' => 'applications/phortune/cart/PhortuneSubscriptionCart.php',
    'PhortuneSubscriptionEditController' => 'applications/phortune/controller/PhortuneSubscriptionEditController.php',
    'PhortuneSubscriptionImplementation' => 'applications/phortune/subscription/PhortuneSubscriptionImplementation.php',
    'PhortuneSubscriptionListController' => 'applications/phortune/controller/PhortuneSubscriptionListController.php',
    'PhortuneSubscriptionPHIDType' => 'applications/phortune/phid/PhortuneSubscriptionPHIDType.php',
    'PhortuneSubscriptionProduct' => 'applications/phortune/product/PhortuneSubscriptionProduct.php',
    'PhortuneSubscriptionQuery' => 'applications/phortune/query/PhortuneSubscriptionQuery.php',
    'PhortuneSubscriptionSearchEngine' => 'applications/phortune/query/PhortuneSubscriptionSearchEngine.php',
    'PhortuneSubscriptionTableView' => 'applications/phortune/view/PhortuneSubscriptionTableView.php',
    'PhortuneSubscriptionViewController' => 'applications/phortune/controller/PhortuneSubscriptionViewController.php',
    'PhortuneSubscriptionWorker' => 'applications/phortune/worker/PhortuneSubscriptionWorker.php',
    'PhortuneTestPaymentProvider' => 'applications/phortune/provider/PhortuneTestPaymentProvider.php',
    'PhortuneWePayPaymentProvider' => 'applications/phortune/provider/PhortuneWePayPaymentProvider.php',
    'PhragmentBrowseController' => 'applications/phragment/controller/PhragmentBrowseController.php',
    'PhragmentCanCreateCapability' => 'applications/phragment/capability/PhragmentCanCreateCapability.php',
    'PhragmentConduitAPIMethod' => 'applications/phragment/conduit/PhragmentConduitAPIMethod.php',
    'PhragmentController' => 'applications/phragment/controller/PhragmentController.php',
    'PhragmentCreateController' => 'applications/phragment/controller/PhragmentCreateController.php',
    'PhragmentDAO' => 'applications/phragment/storage/PhragmentDAO.php',
    'PhragmentFragment' => 'applications/phragment/storage/PhragmentFragment.php',
    'PhragmentFragmentPHIDType' => 'applications/phragment/phid/PhragmentFragmentPHIDType.php',
    'PhragmentFragmentQuery' => 'applications/phragment/query/PhragmentFragmentQuery.php',
    'PhragmentFragmentVersion' => 'applications/phragment/storage/PhragmentFragmentVersion.php',
    'PhragmentFragmentVersionPHIDType' => 'applications/phragment/phid/PhragmentFragmentVersionPHIDType.php',
    'PhragmentFragmentVersionQuery' => 'applications/phragment/query/PhragmentFragmentVersionQuery.php',
    'PhragmentGetPatchConduitAPIMethod' => 'applications/phragment/conduit/PhragmentGetPatchConduitAPIMethod.php',
    'PhragmentHistoryController' => 'applications/phragment/controller/PhragmentHistoryController.php',
    'PhragmentPatchController' => 'applications/phragment/controller/PhragmentPatchController.php',
    'PhragmentPatchUtil' => 'applications/phragment/util/PhragmentPatchUtil.php',
    'PhragmentPolicyController' => 'applications/phragment/controller/PhragmentPolicyController.php',
    'PhragmentQueryFragmentsConduitAPIMethod' => 'applications/phragment/conduit/PhragmentQueryFragmentsConduitAPIMethod.php',
    'PhragmentRevertController' => 'applications/phragment/controller/PhragmentRevertController.php',
    'PhragmentSchemaSpec' => 'applications/phragment/storage/PhragmentSchemaSpec.php',
    'PhragmentSnapshot' => 'applications/phragment/storage/PhragmentSnapshot.php',
    'PhragmentSnapshotChild' => 'applications/phragment/storage/PhragmentSnapshotChild.php',
    'PhragmentSnapshotChildQuery' => 'applications/phragment/query/PhragmentSnapshotChildQuery.php',
    'PhragmentSnapshotCreateController' => 'applications/phragment/controller/PhragmentSnapshotCreateController.php',
    'PhragmentSnapshotDeleteController' => 'applications/phragment/controller/PhragmentSnapshotDeleteController.php',
    'PhragmentSnapshotPHIDType' => 'applications/phragment/phid/PhragmentSnapshotPHIDType.php',
    'PhragmentSnapshotPromoteController' => 'applications/phragment/controller/PhragmentSnapshotPromoteController.php',
    'PhragmentSnapshotQuery' => 'applications/phragment/query/PhragmentSnapshotQuery.php',
    'PhragmentSnapshotViewController' => 'applications/phragment/controller/PhragmentSnapshotViewController.php',
    'PhragmentUpdateController' => 'applications/phragment/controller/PhragmentUpdateController.php',
    'PhragmentVersionController' => 'applications/phragment/controller/PhragmentVersionController.php',
    'PhragmentZIPController' => 'applications/phragment/controller/PhragmentZIPController.php',
    'PhrequentConduitAPIMethod' => 'applications/phrequent/conduit/PhrequentConduitAPIMethod.php',
    'PhrequentController' => 'applications/phrequent/controller/PhrequentController.php',
    'PhrequentDAO' => 'applications/phrequent/storage/PhrequentDAO.php',
    'PhrequentListController' => 'applications/phrequent/controller/PhrequentListController.php',
    'PhrequentPopConduitAPIMethod' => 'applications/phrequent/conduit/PhrequentPopConduitAPIMethod.php',
    'PhrequentPushConduitAPIMethod' => 'applications/phrequent/conduit/PhrequentPushConduitAPIMethod.php',
    'PhrequentSearchEngine' => 'applications/phrequent/query/PhrequentSearchEngine.php',
    'PhrequentTimeBlock' => 'applications/phrequent/storage/PhrequentTimeBlock.php',
    'PhrequentTimeBlockTestCase' => 'applications/phrequent/storage/__tests__/PhrequentTimeBlockTestCase.php',
    'PhrequentTimeSlices' => 'applications/phrequent/storage/PhrequentTimeSlices.php',
    'PhrequentTrackController' => 'applications/phrequent/controller/PhrequentTrackController.php',
    'PhrequentTrackableInterface' => 'applications/phrequent/interface/PhrequentTrackableInterface.php',
    'PhrequentTrackingConduitAPIMethod' => 'applications/phrequent/conduit/PhrequentTrackingConduitAPIMethod.php',
    'PhrequentTrackingEditor' => 'applications/phrequent/editor/PhrequentTrackingEditor.php',
    'PhrequentUIEventListener' => 'applications/phrequent/event/PhrequentUIEventListener.php',
    'PhrequentUserTime' => 'applications/phrequent/storage/PhrequentUserTime.php',
    'PhrequentUserTimeQuery' => 'applications/phrequent/query/PhrequentUserTimeQuery.php',
    'PhrictionChangeType' => 'applications/phriction/constants/PhrictionChangeType.php',
    'PhrictionConduitAPIMethod' => 'applications/phriction/conduit/PhrictionConduitAPIMethod.php',
    'PhrictionConstants' => 'applications/phriction/constants/PhrictionConstants.php',
    'PhrictionContent' => 'applications/phriction/storage/PhrictionContent.php',
    'PhrictionController' => 'applications/phriction/controller/PhrictionController.php',
    'PhrictionCreateConduitAPIMethod' => 'applications/phriction/conduit/PhrictionCreateConduitAPIMethod.php',
    'PhrictionDAO' => 'applications/phriction/storage/PhrictionDAO.php',
    'PhrictionDeleteController' => 'applications/phriction/controller/PhrictionDeleteController.php',
    'PhrictionDiffController' => 'applications/phriction/controller/PhrictionDiffController.php',
    'PhrictionDocument' => 'applications/phriction/storage/PhrictionDocument.php',
    'PhrictionDocumentAuthorHeraldField' => 'applications/phriction/herald/PhrictionDocumentAuthorHeraldField.php',
    'PhrictionDocumentContentHeraldField' => 'applications/phriction/herald/PhrictionDocumentContentHeraldField.php',
    'PhrictionDocumentController' => 'applications/phriction/controller/PhrictionDocumentController.php',
    'PhrictionDocumentFulltextEngine' => 'applications/phriction/search/PhrictionDocumentFulltextEngine.php',
    'PhrictionDocumentHeraldAdapter' => 'applications/phriction/herald/PhrictionDocumentHeraldAdapter.php',
    'PhrictionDocumentHeraldField' => 'applications/phriction/herald/PhrictionDocumentHeraldField.php',
    'PhrictionDocumentHeraldFieldGroup' => 'applications/phriction/herald/PhrictionDocumentHeraldFieldGroup.php',
    'PhrictionDocumentPHIDType' => 'applications/phriction/phid/PhrictionDocumentPHIDType.php',
    'PhrictionDocumentPathHeraldField' => 'applications/phriction/herald/PhrictionDocumentPathHeraldField.php',
    'PhrictionDocumentQuery' => 'applications/phriction/query/PhrictionDocumentQuery.php',
    'PhrictionDocumentStatus' => 'applications/phriction/constants/PhrictionDocumentStatus.php',
    'PhrictionDocumentTitleHeraldField' => 'applications/phriction/herald/PhrictionDocumentTitleHeraldField.php',
    'PhrictionEditConduitAPIMethod' => 'applications/phriction/conduit/PhrictionEditConduitAPIMethod.php',
    'PhrictionEditController' => 'applications/phriction/controller/PhrictionEditController.php',
    'PhrictionHistoryConduitAPIMethod' => 'applications/phriction/conduit/PhrictionHistoryConduitAPIMethod.php',
    'PhrictionHistoryController' => 'applications/phriction/controller/PhrictionHistoryController.php',
    'PhrictionInfoConduitAPIMethod' => 'applications/phriction/conduit/PhrictionInfoConduitAPIMethod.php',
    'PhrictionListController' => 'applications/phriction/controller/PhrictionListController.php',
    'PhrictionMoveController' => 'applications/phriction/controller/PhrictionMoveController.php',
    'PhrictionNewController' => 'applications/phriction/controller/PhrictionNewController.php',
    'PhrictionRemarkupRule' => 'applications/phriction/markup/PhrictionRemarkupRule.php',
    'PhrictionReplyHandler' => 'applications/phriction/mail/PhrictionReplyHandler.php',
    'PhrictionSchemaSpec' => 'applications/phriction/storage/PhrictionSchemaSpec.php',
    'PhrictionSearchEngine' => 'applications/phriction/query/PhrictionSearchEngine.php',
    'PhrictionTransaction' => 'applications/phriction/storage/PhrictionTransaction.php',
    'PhrictionTransactionComment' => 'applications/phriction/storage/PhrictionTransactionComment.php',
    'PhrictionTransactionEditor' => 'applications/phriction/editor/PhrictionTransactionEditor.php',
    'PhrictionTransactionQuery' => 'applications/phriction/query/PhrictionTransactionQuery.php',
    'PolicyLockOptionType' => 'applications/policy/config/PolicyLockOptionType.php',
    'PonderAddAnswerView' => 'applications/ponder/view/PonderAddAnswerView.php',
    'PonderAnswer' => 'applications/ponder/storage/PonderAnswer.php',
    'PonderAnswerCommentController' => 'applications/ponder/controller/PonderAnswerCommentController.php',
    'PonderAnswerEditController' => 'applications/ponder/controller/PonderAnswerEditController.php',
    'PonderAnswerEditor' => 'applications/ponder/editor/PonderAnswerEditor.php',
    'PonderAnswerHasVotingUserEdgeType' => 'applications/ponder/edge/PonderAnswerHasVotingUserEdgeType.php',
    'PonderAnswerHistoryController' => 'applications/ponder/controller/PonderAnswerHistoryController.php',
    'PonderAnswerMailReceiver' => 'applications/ponder/mail/PonderAnswerMailReceiver.php',
    'PonderAnswerPHIDType' => 'applications/ponder/phid/PonderAnswerPHIDType.php',
    'PonderAnswerQuery' => 'applications/ponder/query/PonderAnswerQuery.php',
    'PonderAnswerReplyHandler' => 'applications/ponder/mail/PonderAnswerReplyHandler.php',
    'PonderAnswerSaveController' => 'applications/ponder/controller/PonderAnswerSaveController.php',
    'PonderAnswerStatus' => 'applications/ponder/constants/PonderAnswerStatus.php',
    'PonderAnswerTransaction' => 'applications/ponder/storage/PonderAnswerTransaction.php',
    'PonderAnswerTransactionComment' => 'applications/ponder/storage/PonderAnswerTransactionComment.php',
    'PonderAnswerTransactionQuery' => 'applications/ponder/query/PonderAnswerTransactionQuery.php',
    'PonderAnswerView' => 'applications/ponder/view/PonderAnswerView.php',
    'PonderConstants' => 'applications/ponder/constants/PonderConstants.php',
    'PonderController' => 'applications/ponder/controller/PonderController.php',
    'PonderDAO' => 'applications/ponder/storage/PonderDAO.php',
    'PonderDefaultViewCapability' => 'applications/ponder/capability/PonderDefaultViewCapability.php',
    'PonderEditor' => 'applications/ponder/editor/PonderEditor.php',
    'PonderFooterView' => 'applications/ponder/view/PonderFooterView.php',
    'PonderHelpfulSaveController' => 'applications/ponder/controller/PonderHelpfulSaveController.php',
    'PonderModerateCapability' => 'applications/ponder/capability/PonderModerateCapability.php',
    'PonderQuestion' => 'applications/ponder/storage/PonderQuestion.php',
    'PonderQuestionCommentController' => 'applications/ponder/controller/PonderQuestionCommentController.php',
    'PonderQuestionEditController' => 'applications/ponder/controller/PonderQuestionEditController.php',
    'PonderQuestionEditor' => 'applications/ponder/editor/PonderQuestionEditor.php',
    'PonderQuestionFulltextEngine' => 'applications/ponder/search/PonderQuestionFulltextEngine.php',
    'PonderQuestionHistoryController' => 'applications/ponder/controller/PonderQuestionHistoryController.php',
    'PonderQuestionListController' => 'applications/ponder/controller/PonderQuestionListController.php',
    'PonderQuestionMailReceiver' => 'applications/ponder/mail/PonderQuestionMailReceiver.php',
    'PonderQuestionPHIDType' => 'applications/ponder/phid/PonderQuestionPHIDType.php',
    'PonderQuestionQuery' => 'applications/ponder/query/PonderQuestionQuery.php',
    'PonderQuestionReplyHandler' => 'applications/ponder/mail/PonderQuestionReplyHandler.php',
    'PonderQuestionSearchEngine' => 'applications/ponder/query/PonderQuestionSearchEngine.php',
    'PonderQuestionStatus' => 'applications/ponder/constants/PonderQuestionStatus.php',
    'PonderQuestionStatusController' => 'applications/ponder/controller/PonderQuestionStatusController.php',
    'PonderQuestionTransaction' => 'applications/ponder/storage/PonderQuestionTransaction.php',
    'PonderQuestionTransactionComment' => 'applications/ponder/storage/PonderQuestionTransactionComment.php',
    'PonderQuestionTransactionQuery' => 'applications/ponder/query/PonderQuestionTransactionQuery.php',
    'PonderQuestionViewController' => 'applications/ponder/controller/PonderQuestionViewController.php',
    'PonderRemarkupRule' => 'applications/ponder/remarkup/PonderRemarkupRule.php',
    'PonderSchemaSpec' => 'applications/ponder/storage/PonderSchemaSpec.php',
    'PonderVotableInterface' => 'applications/ponder/storage/PonderVotableInterface.php',
    'PonderVote' => 'applications/ponder/constants/PonderVote.php',
    'PonderVoteEditor' => 'applications/ponder/editor/PonderVoteEditor.php',
    'PonderVotingUserHasAnswerEdgeType' => 'applications/ponder/edge/PonderVotingUserHasAnswerEdgeType.php',
    'ProjectAddProjectsEmailCommand' => 'applications/project/command/ProjectAddProjectsEmailCommand.php',
    'ProjectBoardTaskCard' => 'applications/project/view/ProjectBoardTaskCard.php',
    'ProjectCanLockProjectsCapability' => 'applications/project/capability/ProjectCanLockProjectsCapability.php',
    'ProjectConduitAPIMethod' => 'applications/project/conduit/ProjectConduitAPIMethod.php',
    'ProjectCreateConduitAPIMethod' => 'applications/project/conduit/ProjectCreateConduitAPIMethod.php',
    'ProjectCreateProjectsCapability' => 'applications/project/capability/ProjectCreateProjectsCapability.php',
    'ProjectDefaultEditCapability' => 'applications/project/capability/ProjectDefaultEditCapability.php',
    'ProjectDefaultJoinCapability' => 'applications/project/capability/ProjectDefaultJoinCapability.php',
    'ProjectDefaultViewCapability' => 'applications/project/capability/ProjectDefaultViewCapability.php',
    'ProjectEditConduitAPIMethod' => 'applications/project/conduit/ProjectEditConduitAPIMethod.php',
    'ProjectQueryConduitAPIMethod' => 'applications/project/conduit/ProjectQueryConduitAPIMethod.php',
    'ProjectRemarkupRule' => 'applications/project/remarkup/ProjectRemarkupRule.php',
    'ProjectRemarkupRuleTestCase' => 'applications/project/remarkup/__tests__/ProjectRemarkupRuleTestCase.php',
    'ProjectReplyHandler' => 'applications/project/mail/ProjectReplyHandler.php',
    'ProjectSearchConduitAPIMethod' => 'applications/project/conduit/ProjectSearchConduitAPIMethod.php',
    'QueryFormattingTestCase' => 'infrastructure/storage/__tests__/QueryFormattingTestCase.php',
    'ReleephAuthorFieldSpecification' => 'applications/releeph/field/specification/ReleephAuthorFieldSpecification.php',
    'ReleephBranch' => 'applications/releeph/storage/ReleephBranch.php',
    'ReleephBranchAccessController' => 'applications/releeph/controller/branch/ReleephBranchAccessController.php',
    'ReleephBranchCommitFieldSpecification' => 'applications/releeph/field/specification/ReleephBranchCommitFieldSpecification.php',
    'ReleephBranchController' => 'applications/releeph/controller/branch/ReleephBranchController.php',
    'ReleephBranchCreateController' => 'applications/releeph/controller/branch/ReleephBranchCreateController.php',
    'ReleephBranchEditController' => 'applications/releeph/controller/branch/ReleephBranchEditController.php',
    'ReleephBranchEditor' => 'applications/releeph/editor/ReleephBranchEditor.php',
    'ReleephBranchHistoryController' => 'applications/releeph/controller/branch/ReleephBranchHistoryController.php',
    'ReleephBranchNamePreviewController' => 'applications/releeph/controller/branch/ReleephBranchNamePreviewController.php',
    'ReleephBranchPHIDType' => 'applications/releeph/phid/ReleephBranchPHIDType.php',
    'ReleephBranchPreviewView' => 'applications/releeph/view/branch/ReleephBranchPreviewView.php',
    'ReleephBranchQuery' => 'applications/releeph/query/ReleephBranchQuery.php',
    'ReleephBranchSearchEngine' => 'applications/releeph/query/ReleephBranchSearchEngine.php',
    'ReleephBranchTemplate' => 'applications/releeph/view/branch/ReleephBranchTemplate.php',
    'ReleephBranchTransaction' => 'applications/releeph/storage/ReleephBranchTransaction.php',
    'ReleephBranchTransactionQuery' => 'applications/releeph/query/ReleephBranchTransactionQuery.php',
    'ReleephBranchViewController' => 'applications/releeph/controller/branch/ReleephBranchViewController.php',
    'ReleephCommitFinder' => 'applications/releeph/commitfinder/ReleephCommitFinder.php',
    'ReleephCommitFinderException' => 'applications/releeph/commitfinder/ReleephCommitFinderException.php',
    'ReleephCommitMessageFieldSpecification' => 'applications/releeph/field/specification/ReleephCommitMessageFieldSpecification.php',
    'ReleephConduitAPIMethod' => 'applications/releeph/conduit/ReleephConduitAPIMethod.php',
    'ReleephController' => 'applications/releeph/controller/ReleephController.php',
    'ReleephDAO' => 'applications/releeph/storage/ReleephDAO.php',
    'ReleephDefaultFieldSelector' => 'applications/releeph/field/selector/ReleephDefaultFieldSelector.php',
    'ReleephDependsOnFieldSpecification' => 'applications/releeph/field/specification/ReleephDependsOnFieldSpecification.php',
    'ReleephDiffChurnFieldSpecification' => 'applications/releeph/field/specification/ReleephDiffChurnFieldSpecification.php',
    'ReleephDiffMessageFieldSpecification' => 'applications/releeph/field/specification/ReleephDiffMessageFieldSpecification.php',
    'ReleephDiffSizeFieldSpecification' => 'applications/releeph/field/specification/ReleephDiffSizeFieldSpecification.php',
    'ReleephFieldParseException' => 'applications/releeph/field/exception/ReleephFieldParseException.php',
    'ReleephFieldSelector' => 'applications/releeph/field/selector/ReleephFieldSelector.php',
    'ReleephFieldSpecification' => 'applications/releeph/field/specification/ReleephFieldSpecification.php',
    'ReleephGetBranchesConduitAPIMethod' => 'applications/releeph/conduit/ReleephGetBranchesConduitAPIMethod.php',
    'ReleephIntentFieldSpecification' => 'applications/releeph/field/specification/ReleephIntentFieldSpecification.php',
    'ReleephLevelFieldSpecification' => 'applications/releeph/field/specification/ReleephLevelFieldSpecification.php',
    'ReleephOriginalCommitFieldSpecification' => 'applications/releeph/field/specification/ReleephOriginalCommitFieldSpecification.php',
    'ReleephProductActionController' => 'applications/releeph/controller/product/ReleephProductActionController.php',
    'ReleephProductController' => 'applications/releeph/controller/product/ReleephProductController.php',
    'ReleephProductCreateController' => 'applications/releeph/controller/product/ReleephProductCreateController.php',
    'ReleephProductEditController' => 'applications/releeph/controller/product/ReleephProductEditController.php',
    'ReleephProductEditor' => 'applications/releeph/editor/ReleephProductEditor.php',
    'ReleephProductHistoryController' => 'applications/releeph/controller/product/ReleephProductHistoryController.php',
    'ReleephProductListController' => 'applications/releeph/controller/product/ReleephProductListController.php',
    'ReleephProductPHIDType' => 'applications/releeph/phid/ReleephProductPHIDType.php',
    'ReleephProductQuery' => 'applications/releeph/query/ReleephProductQuery.php',
    'ReleephProductSearchEngine' => 'applications/releeph/query/ReleephProductSearchEngine.php',
    'ReleephProductTransaction' => 'applications/releeph/storage/ReleephProductTransaction.php',
    'ReleephProductTransactionQuery' => 'applications/releeph/query/ReleephProductTransactionQuery.php',
    'ReleephProductViewController' => 'applications/releeph/controller/product/ReleephProductViewController.php',
    'ReleephProject' => 'applications/releeph/storage/ReleephProject.php',
    'ReleephQueryBranchesConduitAPIMethod' => 'applications/releeph/conduit/ReleephQueryBranchesConduitAPIMethod.php',
    'ReleephQueryProductsConduitAPIMethod' => 'applications/releeph/conduit/ReleephQueryProductsConduitAPIMethod.php',
    'ReleephQueryRequestsConduitAPIMethod' => 'applications/releeph/conduit/ReleephQueryRequestsConduitAPIMethod.php',
    'ReleephReasonFieldSpecification' => 'applications/releeph/field/specification/ReleephReasonFieldSpecification.php',
    'ReleephRequest' => 'applications/releeph/storage/ReleephRequest.php',
    'ReleephRequestActionController' => 'applications/releeph/controller/request/ReleephRequestActionController.php',
    'ReleephRequestCommentController' => 'applications/releeph/controller/request/ReleephRequestCommentController.php',
    'ReleephRequestConduitAPIMethod' => 'applications/releeph/conduit/ReleephRequestConduitAPIMethod.php',
    'ReleephRequestController' => 'applications/releeph/controller/request/ReleephRequestController.php',
    'ReleephRequestDifferentialCreateController' => 'applications/releeph/controller/request/ReleephRequestDifferentialCreateController.php',
    'ReleephRequestEditController' => 'applications/releeph/controller/request/ReleephRequestEditController.php',
    'ReleephRequestMailReceiver' => 'applications/releeph/mail/ReleephRequestMailReceiver.php',
    'ReleephRequestPHIDType' => 'applications/releeph/phid/ReleephRequestPHIDType.php',
    'ReleephRequestQuery' => 'applications/releeph/query/ReleephRequestQuery.php',
    'ReleephRequestReplyHandler' => 'applications/releeph/mail/ReleephRequestReplyHandler.php',
    'ReleephRequestSearchEngine' => 'applications/releeph/query/ReleephRequestSearchEngine.php',
    'ReleephRequestStatus' => 'applications/releeph/constants/ReleephRequestStatus.php',
    'ReleephRequestTransaction' => 'applications/releeph/storage/ReleephRequestTransaction.php',
    'ReleephRequestTransactionComment' => 'applications/releeph/storage/ReleephRequestTransactionComment.php',
    'ReleephRequestTransactionQuery' => 'applications/releeph/query/ReleephRequestTransactionQuery.php',
    'ReleephRequestTransactionalEditor' => 'applications/releeph/editor/ReleephRequestTransactionalEditor.php',
    'ReleephRequestTypeaheadControl' => 'applications/releeph/view/request/ReleephRequestTypeaheadControl.php',
    'ReleephRequestTypeaheadController' => 'applications/releeph/controller/request/ReleephRequestTypeaheadController.php',
    'ReleephRequestView' => 'applications/releeph/view/ReleephRequestView.php',
    'ReleephRequestViewController' => 'applications/releeph/controller/request/ReleephRequestViewController.php',
    'ReleephRequestorFieldSpecification' => 'applications/releeph/field/specification/ReleephRequestorFieldSpecification.php',
    'ReleephRevisionFieldSpecification' => 'applications/releeph/field/specification/ReleephRevisionFieldSpecification.php',
    'ReleephSeverityFieldSpecification' => 'applications/releeph/field/specification/ReleephSeverityFieldSpecification.php',
    'ReleephSummaryFieldSpecification' => 'applications/releeph/field/specification/ReleephSummaryFieldSpecification.php',
    'ReleephWorkCanPushConduitAPIMethod' => 'applications/releeph/conduit/work/ReleephWorkCanPushConduitAPIMethod.php',
    'ReleephWorkGetAuthorInfoConduitAPIMethod' => 'applications/releeph/conduit/work/ReleephWorkGetAuthorInfoConduitAPIMethod.php',
    'ReleephWorkGetBranchCommitMessageConduitAPIMethod' => 'applications/releeph/conduit/work/ReleephWorkGetBranchCommitMessageConduitAPIMethod.php',
    'ReleephWorkGetBranchConduitAPIMethod' => 'applications/releeph/conduit/work/ReleephWorkGetBranchConduitAPIMethod.php',
    'ReleephWorkGetCommitMessageConduitAPIMethod' => 'applications/releeph/conduit/work/ReleephWorkGetCommitMessageConduitAPIMethod.php',
    'ReleephWorkNextRequestConduitAPIMethod' => 'applications/releeph/conduit/work/ReleephWorkNextRequestConduitAPIMethod.php',
    'ReleephWorkRecordConduitAPIMethod' => 'applications/releeph/conduit/work/ReleephWorkRecordConduitAPIMethod.php',
    'ReleephWorkRecordPickStatusConduitAPIMethod' => 'applications/releeph/conduit/work/ReleephWorkRecordPickStatusConduitAPIMethod.php',
    'RemarkupProcessConduitAPIMethod' => 'applications/remarkup/conduit/RemarkupProcessConduitAPIMethod.php',
    'RepositoryConduitAPIMethod' => 'applications/repository/conduit/RepositoryConduitAPIMethod.php',
    'RepositoryCreateConduitAPIMethod' => 'applications/repository/conduit/RepositoryCreateConduitAPIMethod.php',
    'RepositoryQueryConduitAPIMethod' => 'applications/repository/conduit/RepositoryQueryConduitAPIMethod.php',
    'ShellLogView' => 'applications/harbormaster/view/ShellLogView.php',
    'SlowvoteConduitAPIMethod' => 'applications/slowvote/conduit/SlowvoteConduitAPIMethod.php',
    'SlowvoteEmbedView' => 'applications/slowvote/view/SlowvoteEmbedView.php',
    'SlowvoteInfoConduitAPIMethod' => 'applications/slowvote/conduit/SlowvoteInfoConduitAPIMethod.php',
    'SlowvoteRemarkupRule' => 'applications/slowvote/remarkup/SlowvoteRemarkupRule.php',
    'SubscriptionListDialogBuilder' => 'applications/subscriptions/view/SubscriptionListDialogBuilder.php',
    'SubscriptionListStringBuilder' => 'applications/subscriptions/view/SubscriptionListStringBuilder.php',
    'TokenConduitAPIMethod' => 'applications/tokens/conduit/TokenConduitAPIMethod.php',
    'TokenGiveConduitAPIMethod' => 'applications/tokens/conduit/TokenGiveConduitAPIMethod.php',
    'TokenGivenConduitAPIMethod' => 'applications/tokens/conduit/TokenGivenConduitAPIMethod.php',
    'TokenQueryConduitAPIMethod' => 'applications/tokens/conduit/TokenQueryConduitAPIMethod.php',
    'UserConduitAPIMethod' => 'applications/people/conduit/UserConduitAPIMethod.php',
    'UserDisableConduitAPIMethod' => 'applications/people/conduit/UserDisableConduitAPIMethod.php',
    'UserEnableConduitAPIMethod' => 'applications/people/conduit/UserEnableConduitAPIMethod.php',
    'UserFindConduitAPIMethod' => 'applications/people/conduit/UserFindConduitAPIMethod.php',
    'UserQueryConduitAPIMethod' => 'applications/people/conduit/UserQueryConduitAPIMethod.php',
    'UserWhoAmIConduitAPIMethod' => 'applications/people/conduit/UserWhoAmIConduitAPIMethod.php',
  ),
  'function' => array(
    'celerity_generate_unique_node_id' => 'applications/celerity/api.php',
    'celerity_get_resource_uri' => 'applications/celerity/api.php',
    'javelin_tag' => 'infrastructure/javelin/markup.php',
    'phabricator_date' => 'view/viewutils.php',
    'phabricator_datetime' => 'view/viewutils.php',
    'phabricator_form' => 'infrastructure/javelin/markup.php',
    'phabricator_format_local_time' => 'view/viewutils.php',
    'phabricator_relative_date' => 'view/viewutils.php',
    'phabricator_time' => 'view/viewutils.php',
    'phid_get_subtype' => 'applications/phid/utils.php',
    'phid_get_type' => 'applications/phid/utils.php',
    'phid_group_by_type' => 'applications/phid/utils.php',
    'require_celerity_resource' => 'applications/celerity/api.php',
  ),
  'xmap' => array(
    'AlmanacAddress' => 'Phobject',
    'AlmanacBinding' => array(
      'AlmanacDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorCustomFieldInterface',
      'PhabricatorApplicationTransactionInterface',
      'AlmanacPropertyInterface',
      'PhabricatorDestructibleInterface',
    ),
    'AlmanacBindingEditController' => 'AlmanacServiceController',
    'AlmanacBindingEditor' => 'PhabricatorApplicationTransactionEditor',
    'AlmanacBindingPHIDType' => 'PhabricatorPHIDType',
    'AlmanacBindingQuery' => 'AlmanacQuery',
    'AlmanacBindingTableView' => 'AphrontView',
    'AlmanacBindingTransaction' => 'PhabricatorApplicationTransaction',
    'AlmanacBindingTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'AlmanacBindingViewController' => 'AlmanacServiceController',
    'AlmanacClusterDatabaseServiceType' => 'AlmanacClusterServiceType',
    'AlmanacClusterRepositoryServiceType' => 'AlmanacClusterServiceType',
    'AlmanacClusterServiceType' => 'AlmanacServiceType',
    'AlmanacConduitAPIMethod' => 'ConduitAPIMethod',
    'AlmanacConsoleController' => 'AlmanacController',
    'AlmanacController' => 'PhabricatorController',
    'AlmanacCoreCustomField' => array(
      'AlmanacCustomField',
      'PhabricatorStandardCustomFieldInterface',
    ),
    'AlmanacCreateClusterServicesCapability' => 'PhabricatorPolicyCapability',
    'AlmanacCreateDevicesCapability' => 'PhabricatorPolicyCapability',
    'AlmanacCreateNetworksCapability' => 'PhabricatorPolicyCapability',
    'AlmanacCreateServicesCapability' => 'PhabricatorPolicyCapability',
    'AlmanacCustomField' => 'PhabricatorCustomField',
    'AlmanacCustomServiceType' => 'AlmanacServiceType',
    'AlmanacDAO' => 'PhabricatorLiskDAO',
    'AlmanacDevice' => array(
      'AlmanacDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorCustomFieldInterface',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorProjectInterface',
      'PhabricatorSSHPublicKeyInterface',
      'AlmanacPropertyInterface',
      'PhabricatorDestructibleInterface',
    ),
    'AlmanacDeviceController' => 'AlmanacController',
    'AlmanacDeviceEditController' => 'AlmanacDeviceController',
    'AlmanacDeviceEditor' => 'PhabricatorApplicationTransactionEditor',
    'AlmanacDeviceListController' => 'AlmanacDeviceController',
    'AlmanacDevicePHIDType' => 'PhabricatorPHIDType',
    'AlmanacDeviceQuery' => 'AlmanacQuery',
    'AlmanacDeviceSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'AlmanacDeviceTransaction' => 'PhabricatorApplicationTransaction',
    'AlmanacDeviceTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'AlmanacDeviceViewController' => 'AlmanacDeviceController',
    'AlmanacDrydockPoolServiceType' => 'AlmanacServiceType',
    'AlmanacInterface' => array(
      'AlmanacDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorDestructibleInterface',
    ),
    'AlmanacInterfaceDatasource' => 'PhabricatorTypeaheadDatasource',
    'AlmanacInterfaceEditController' => 'AlmanacDeviceController',
    'AlmanacInterfacePHIDType' => 'PhabricatorPHIDType',
    'AlmanacInterfaceQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'AlmanacInterfaceTableView' => 'AphrontView',
    'AlmanacKeys' => 'Phobject',
    'AlmanacManagementLockWorkflow' => 'AlmanacManagementWorkflow',
    'AlmanacManagementRegisterWorkflow' => 'AlmanacManagementWorkflow',
    'AlmanacManagementTrustKeyWorkflow' => 'AlmanacManagementWorkflow',
    'AlmanacManagementUnlockWorkflow' => 'AlmanacManagementWorkflow',
    'AlmanacManagementUntrustKeyWorkflow' => 'AlmanacManagementWorkflow',
    'AlmanacManagementWorkflow' => 'PhabricatorManagementWorkflow',
    'AlmanacNames' => 'Phobject',
    'AlmanacNamesTestCase' => 'PhabricatorTestCase',
    'AlmanacNetwork' => array(
      'AlmanacDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorPolicyInterface',
      'PhabricatorDestructibleInterface',
    ),
    'AlmanacNetworkController' => 'AlmanacController',
    'AlmanacNetworkEditController' => 'AlmanacNetworkController',
    'AlmanacNetworkEditor' => 'PhabricatorApplicationTransactionEditor',
    'AlmanacNetworkListController' => 'AlmanacNetworkController',
    'AlmanacNetworkPHIDType' => 'PhabricatorPHIDType',
    'AlmanacNetworkQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'AlmanacNetworkSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'AlmanacNetworkTransaction' => 'PhabricatorApplicationTransaction',
    'AlmanacNetworkTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'AlmanacNetworkViewController' => 'AlmanacNetworkController',
    'AlmanacPropertiesDestructionEngineExtension' => 'PhabricatorDestructionEngineExtension',
    'AlmanacProperty' => array(
      'PhabricatorCustomFieldStorage',
      'PhabricatorPolicyInterface',
    ),
    'AlmanacPropertyController' => 'AlmanacController',
    'AlmanacPropertyDeleteController' => 'AlmanacDeviceController',
    'AlmanacPropertyEditController' => 'AlmanacDeviceController',
    'AlmanacPropertyQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'AlmanacQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'AlmanacQueryDevicesConduitAPIMethod' => 'AlmanacConduitAPIMethod',
    'AlmanacQueryServicesConduitAPIMethod' => 'AlmanacConduitAPIMethod',
    'AlmanacSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'AlmanacService' => array(
      'AlmanacDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorCustomFieldInterface',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorProjectInterface',
      'AlmanacPropertyInterface',
      'PhabricatorDestructibleInterface',
    ),
    'AlmanacServiceController' => 'AlmanacController',
    'AlmanacServiceDatasource' => 'PhabricatorTypeaheadDatasource',
    'AlmanacServiceEditController' => 'AlmanacServiceController',
    'AlmanacServiceEditor' => 'PhabricatorApplicationTransactionEditor',
    'AlmanacServiceListController' => 'AlmanacServiceController',
    'AlmanacServicePHIDType' => 'PhabricatorPHIDType',
    'AlmanacServiceQuery' => 'AlmanacQuery',
    'AlmanacServiceSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'AlmanacServiceTransaction' => 'PhabricatorApplicationTransaction',
    'AlmanacServiceTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'AlmanacServiceType' => 'Phobject',
    'AlmanacServiceTypeTestCase' => 'PhabricatorTestCase',
    'AlmanacServiceViewController' => 'AlmanacServiceController',
    'AphlictDropdownDataQuery' => 'Phobject',
    'Aphront304Response' => 'AphrontResponse',
    'Aphront400Response' => 'AphrontResponse',
    'Aphront403Response' => 'AphrontHTMLResponse',
    'Aphront404Response' => 'AphrontHTMLResponse',
    'AphrontAjaxResponse' => 'AphrontResponse',
    'AphrontApplicationConfiguration' => 'Phobject',
    'AphrontBarView' => 'AphrontView',
    'AphrontBoolHTTPParameterType' => 'AphrontHTTPParameterType',
    'AphrontCSRFException' => 'AphrontException',
    'AphrontCalendarEventView' => 'AphrontView',
    'AphrontController' => 'Phobject',
    'AphrontCursorPagerView' => 'AphrontView',
    'AphrontDefaultApplicationConfiguration' => 'AphrontApplicationConfiguration',
    'AphrontDialogResponse' => 'AphrontResponse',
    'AphrontDialogView' => array(
      'AphrontView',
      'AphrontResponseProducerInterface',
    ),
    'AphrontException' => 'Exception',
    'AphrontFileResponse' => 'AphrontResponse',
    'AphrontFormCheckboxControl' => 'AphrontFormControl',
    'AphrontFormControl' => 'AphrontView',
    'AphrontFormDateControl' => 'AphrontFormControl',
    'AphrontFormDateControlValue' => 'Phobject',
    'AphrontFormDividerControl' => 'AphrontFormControl',
    'AphrontFormFileControl' => 'AphrontFormControl',
    'AphrontFormHandlesControl' => 'AphrontFormControl',
    'AphrontFormMarkupControl' => 'AphrontFormControl',
    'AphrontFormPasswordControl' => 'AphrontFormControl',
    'AphrontFormPolicyControl' => 'AphrontFormControl',
    'AphrontFormRadioButtonControl' => 'AphrontFormControl',
    'AphrontFormRecaptchaControl' => 'AphrontFormControl',
    'AphrontFormSelectControl' => 'AphrontFormControl',
    'AphrontFormStaticControl' => 'AphrontFormControl',
    'AphrontFormSubmitControl' => 'AphrontFormControl',
    'AphrontFormTextAreaControl' => 'AphrontFormControl',
    'AphrontFormTextControl' => 'AphrontFormControl',
    'AphrontFormTextWithSubmitControl' => 'AphrontFormControl',
    'AphrontFormTokenizerControl' => 'AphrontFormControl',
    'AphrontFormTypeaheadControl' => 'AphrontFormControl',
    'AphrontFormView' => 'AphrontView',
    'AphrontGlyphBarView' => 'AphrontBarView',
    'AphrontHTMLResponse' => 'AphrontResponse',
    'AphrontHTTPParameterType' => 'Phobject',
    'AphrontHTTPProxyResponse' => 'AphrontResponse',
    'AphrontHTTPSink' => 'Phobject',
    'AphrontHTTPSinkTestCase' => 'PhabricatorTestCase',
    'AphrontIntHTTPParameterType' => 'AphrontHTTPParameterType',
    'AphrontIsolatedDatabaseConnectionTestCase' => 'PhabricatorTestCase',
    'AphrontIsolatedHTTPSink' => 'AphrontHTTPSink',
    'AphrontJSONResponse' => 'AphrontResponse',
    'AphrontJavelinView' => 'AphrontView',
    'AphrontKeyboardShortcutsAvailableView' => 'AphrontView',
    'AphrontListFilterView' => 'AphrontView',
    'AphrontListHTTPParameterType' => 'AphrontHTTPParameterType',
    'AphrontMalformedRequestException' => 'AphrontException',
    'AphrontMoreView' => 'AphrontView',
    'AphrontMultiColumnView' => 'AphrontView',
    'AphrontMySQLDatabaseConnectionTestCase' => 'PhabricatorTestCase',
    'AphrontNullView' => 'AphrontView',
    'AphrontPHIDHTTPParameterType' => 'AphrontHTTPParameterType',
    'AphrontPHIDListHTTPParameterType' => 'AphrontListHTTPParameterType',
    'AphrontPHPHTTPSink' => 'AphrontHTTPSink',
    'AphrontPageView' => 'AphrontView',
    'AphrontPlainTextResponse' => 'AphrontResponse',
    'AphrontProgressBarView' => 'AphrontBarView',
    'AphrontProjectListHTTPParameterType' => 'AphrontListHTTPParameterType',
    'AphrontProxyResponse' => array(
      'AphrontResponse',
      'AphrontResponseProducerInterface',
    ),
    'AphrontRedirectResponse' => 'AphrontResponse',
    'AphrontRedirectResponseTestCase' => 'PhabricatorTestCase',
    'AphrontReloadResponse' => 'AphrontRedirectResponse',
    'AphrontRequest' => 'Phobject',
    'AphrontRequestExceptionHandler' => 'Phobject',
    'AphrontRequestTestCase' => 'PhabricatorTestCase',
    'AphrontResponse' => 'Phobject',
    'AphrontRoutingMap' => 'Phobject',
    'AphrontRoutingResult' => 'Phobject',
    'AphrontSelectHTTPParameterType' => 'AphrontHTTPParameterType',
    'AphrontSideNavFilterView' => 'AphrontView',
    'AphrontSite' => 'Phobject',
    'AphrontStackTraceView' => 'AphrontView',
    'AphrontStandaloneHTMLResponse' => 'AphrontHTMLResponse',
    'AphrontStringHTTPParameterType' => 'AphrontHTTPParameterType',
    'AphrontStringListHTTPParameterType' => 'AphrontListHTTPParameterType',
    'AphrontTableView' => 'AphrontView',
    'AphrontTagView' => 'AphrontView',
    'AphrontTokenizerTemplateView' => 'AphrontView',
    'AphrontTypeaheadTemplateView' => 'AphrontView',
    'AphrontUnhandledExceptionResponse' => 'AphrontStandaloneHTMLResponse',
    'AphrontUserListHTTPParameterType' => 'AphrontListHTTPParameterType',
    'AphrontView' => array(
      'Phobject',
      'PhutilSafeHTMLProducerInterface',
    ),
    'AphrontWebpageResponse' => 'AphrontHTMLResponse',
    'ArcanistConduitAPIMethod' => 'ConduitAPIMethod',
    'AuditConduitAPIMethod' => 'ConduitAPIMethod',
    'AuditQueryConduitAPIMethod' => 'AuditConduitAPIMethod',
    'AuthManageProvidersCapability' => 'PhabricatorPolicyCapability',
    'CalendarTimeUtil' => 'Phobject',
    'CalendarTimeUtilTestCase' => 'PhabricatorTestCase',
    'CelerityAPI' => 'Phobject',
    'CelerityDefaultPostprocessor' => 'CelerityPostprocessor',
    'CelerityHighContrastPostprocessor' => 'CelerityPostprocessor',
    'CelerityLargeFontPostprocessor' => 'CelerityPostprocessor',
    'CelerityManagementMapWorkflow' => 'CelerityManagementWorkflow',
    'CelerityManagementWorkflow' => 'PhabricatorManagementWorkflow',
    'CelerityPhabricatorResourceController' => 'CelerityResourceController',
    'CelerityPhabricatorResources' => 'CelerityResourcesOnDisk',
    'CelerityPhysicalResources' => 'CelerityResources',
    'CelerityPhysicalResourcesTestCase' => 'PhabricatorTestCase',
    'CelerityPostprocessor' => 'Phobject',
    'CelerityPostprocessorTestCase' => 'PhabricatorTestCase',
    'CelerityResourceController' => 'PhabricatorController',
    'CelerityResourceGraph' => 'AbstractDirectedGraph',
    'CelerityResourceMap' => 'Phobject',
    'CelerityResourceMapGenerator' => 'Phobject',
    'CelerityResourceTransformer' => 'Phobject',
    'CelerityResourceTransformerTestCase' => 'PhabricatorTestCase',
    'CelerityResources' => 'Phobject',
    'CelerityResourcesOnDisk' => 'CelerityPhysicalResources',
    'CeleritySpriteGenerator' => 'Phobject',
    'CelerityStaticResourceResponse' => 'Phobject',
    'ChatLogConduitAPIMethod' => 'ConduitAPIMethod',
    'ChatLogQueryConduitAPIMethod' => 'ChatLogConduitAPIMethod',
    'ChatLogRecordConduitAPIMethod' => 'ChatLogConduitAPIMethod',
    'ConduitAPIMethod' => array(
      'Phobject',
      'PhabricatorPolicyInterface',
    ),
    'ConduitAPIMethodTestCase' => 'PhabricatorTestCase',
    'ConduitAPIRequest' => 'Phobject',
    'ConduitAPIResponse' => 'Phobject',
    'ConduitApplicationNotInstalledException' => 'ConduitMethodNotFoundException',
    'ConduitBoolParameterType' => 'ConduitListParameterType',
    'ConduitCall' => 'Phobject',
    'ConduitCallTestCase' => 'PhabricatorTestCase',
    'ConduitConnectConduitAPIMethod' => 'ConduitAPIMethod',
    'ConduitEpochParameterType' => 'ConduitListParameterType',
    'ConduitException' => 'Exception',
    'ConduitGetCapabilitiesConduitAPIMethod' => 'ConduitAPIMethod',
    'ConduitGetCertificateConduitAPIMethod' => 'ConduitAPIMethod',
    'ConduitIntListParameterType' => 'ConduitListParameterType',
    'ConduitIntParameterType' => 'ConduitListParameterType',
    'ConduitListParameterType' => 'ConduitParameterType',
    'ConduitLogGarbageCollector' => 'PhabricatorGarbageCollector',
    'ConduitMethodDoesNotExistException' => 'ConduitMethodNotFoundException',
    'ConduitMethodNotFoundException' => 'ConduitException',
    'ConduitPHIDListParameterType' => 'ConduitListParameterType',
    'ConduitPHIDParameterType' => 'ConduitParameterType',
    'ConduitParameterType' => 'Phobject',
    'ConduitPingConduitAPIMethod' => 'ConduitAPIMethod',
    'ConduitProjectListParameterType' => 'ConduitListParameterType',
    'ConduitQueryConduitAPIMethod' => 'ConduitAPIMethod',
    'ConduitResultSearchEngineExtension' => 'PhabricatorSearchEngineExtension',
    'ConduitSSHWorkflow' => 'PhabricatorSSHWorkflow',
    'ConduitStringListParameterType' => 'ConduitListParameterType',
    'ConduitStringParameterType' => 'ConduitParameterType',
    'ConduitTokenGarbageCollector' => 'PhabricatorGarbageCollector',
    'ConduitUserListParameterType' => 'ConduitListParameterType',
    'ConduitUserParameterType' => 'ConduitParameterType',
    'ConduitWildParameterType' => 'ConduitListParameterType',
    'ConpherenceColumnViewController' => 'ConpherenceController',
    'ConpherenceConduitAPIMethod' => 'ConduitAPIMethod',
    'ConpherenceConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'ConpherenceConstants' => 'Phobject',
    'ConpherenceController' => 'PhabricatorController',
    'ConpherenceCreateThreadConduitAPIMethod' => 'ConpherenceConduitAPIMethod',
    'ConpherenceDAO' => 'PhabricatorLiskDAO',
    'ConpherenceDurableColumnView' => 'AphrontTagView',
    'ConpherenceEditor' => 'PhabricatorApplicationTransactionEditor',
    'ConpherenceFormDragAndDropUploadControl' => 'AphrontFormControl',
    'ConpherenceFulltextQuery' => 'PhabricatorOffsetPagedQuery',
    'ConpherenceImageData' => 'ConpherenceConstants',
    'ConpherenceIndex' => 'ConpherenceDAO',
    'ConpherenceLayoutView' => 'AphrontView',
    'ConpherenceListController' => 'ConpherenceController',
    'ConpherenceMenuItemView' => 'AphrontTagView',
    'ConpherenceNewRoomController' => 'ConpherenceController',
    'ConpherenceNotificationPanelController' => 'ConpherenceController',
    'ConpherenceParticipant' => 'ConpherenceDAO',
    'ConpherenceParticipantCountQuery' => 'PhabricatorOffsetPagedQuery',
    'ConpherenceParticipantQuery' => 'PhabricatorOffsetPagedQuery',
    'ConpherenceParticipationStatus' => 'ConpherenceConstants',
    'ConpherencePeopleWidgetView' => 'ConpherenceWidgetView',
    'ConpherencePicCropControl' => 'AphrontFormControl',
    'ConpherenceQueryThreadConduitAPIMethod' => 'ConpherenceConduitAPIMethod',
    'ConpherenceQueryTransactionConduitAPIMethod' => 'ConpherenceConduitAPIMethod',
    'ConpherenceReplyHandler' => 'PhabricatorMailReplyHandler',
    'ConpherenceRoomListController' => 'ConpherenceController',
    'ConpherenceRoomTestCase' => 'ConpherenceTestCase',
    'ConpherenceSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'ConpherenceSettings' => 'ConpherenceConstants',
    'ConpherenceTestCase' => 'PhabricatorTestCase',
    'ConpherenceThread' => array(
      'ConpherenceDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorMentionableInterface',
      'PhabricatorDestructibleInterface',
    ),
    'ConpherenceThreadIndexEngineExtension' => 'PhabricatorIndexEngineExtension',
    'ConpherenceThreadListView' => 'AphrontView',
    'ConpherenceThreadMailReceiver' => 'PhabricatorObjectMailReceiver',
    'ConpherenceThreadMembersPolicyRule' => 'PhabricatorPolicyRule',
    'ConpherenceThreadQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'ConpherenceThreadRemarkupRule' => 'PhabricatorObjectRemarkupRule',
    'ConpherenceThreadSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'ConpherenceTransaction' => 'PhabricatorApplicationTransaction',
    'ConpherenceTransactionComment' => 'PhabricatorApplicationTransactionComment',
    'ConpherenceTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'ConpherenceTransactionRenderer' => 'Phobject',
    'ConpherenceTransactionView' => 'AphrontView',
    'ConpherenceUpdateActions' => 'ConpherenceConstants',
    'ConpherenceUpdateController' => 'ConpherenceController',
    'ConpherenceUpdateThreadConduitAPIMethod' => 'ConpherenceConduitAPIMethod',
    'ConpherenceViewController' => 'ConpherenceController',
    'ConpherenceWidgetConfigConstants' => 'ConpherenceConstants',
    'ConpherenceWidgetController' => 'ConpherenceController',
    'ConpherenceWidgetView' => 'AphrontView',
    'DarkConsoleController' => 'PhabricatorController',
    'DarkConsoleCore' => 'Phobject',
    'DarkConsoleDataController' => 'PhabricatorController',
    'DarkConsoleErrorLogPlugin' => 'DarkConsolePlugin',
    'DarkConsoleErrorLogPluginAPI' => 'Phobject',
    'DarkConsoleEventPlugin' => 'DarkConsolePlugin',
    'DarkConsoleEventPluginAPI' => 'PhabricatorEventListener',
    'DarkConsolePlugin' => 'Phobject',
    'DarkConsoleRequestPlugin' => 'DarkConsolePlugin',
    'DarkConsoleServicesPlugin' => 'DarkConsolePlugin',
    'DarkConsoleStartupPlugin' => 'DarkConsolePlugin',
    'DarkConsoleXHProfPlugin' => 'DarkConsolePlugin',
    'DarkConsoleXHProfPluginAPI' => 'Phobject',
    'DefaultDatabaseConfigurationProvider' => array(
      'Phobject',
      'DatabaseConfigurationProvider',
    ),
    'DifferentialAction' => 'Phobject',
    'DifferentialActionEmailCommand' => 'MetaMTAEmailTransactionCommand',
    'DifferentialActionMenuEventListener' => 'PhabricatorEventListener',
    'DifferentialAddCommentView' => 'AphrontView',
    'DifferentialAdjustmentMapTestCase' => 'PhutilTestCase',
    'DifferentialAffectedPath' => 'DifferentialDAO',
    'DifferentialApplyPatchField' => 'DifferentialCustomField',
    'DifferentialAsanaRepresentationField' => 'DifferentialCustomField',
    'DifferentialAuditorsField' => 'DifferentialStoredCustomField',
    'DifferentialAuthorField' => 'DifferentialCustomField',
    'DifferentialBlameRevisionField' => 'DifferentialStoredCustomField',
    'DifferentialBlockHeraldAction' => 'HeraldAction',
    'DifferentialBranchField' => 'DifferentialCustomField',
    'DifferentialChangeHeraldFieldGroup' => 'HeraldFieldGroup',
    'DifferentialChangeType' => 'Phobject',
    'DifferentialChangesSinceLastUpdateField' => 'DifferentialCustomField',
    'DifferentialChangeset' => array(
      'DifferentialDAO',
      'PhabricatorPolicyInterface',
    ),
    'DifferentialChangesetDetailView' => 'AphrontView',
    'DifferentialChangesetFileTreeSideNavBuilder' => 'Phobject',
    'DifferentialChangesetHTMLRenderer' => 'DifferentialChangesetRenderer',
    'DifferentialChangesetListView' => 'AphrontView',
    'DifferentialChangesetOneUpRenderer' => 'DifferentialChangesetHTMLRenderer',
    'DifferentialChangesetOneUpTestRenderer' => 'DifferentialChangesetTestRenderer',
    'DifferentialChangesetParser' => 'Phobject',
    'DifferentialChangesetParserTestCase' => 'PhabricatorTestCase',
    'DifferentialChangesetQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'DifferentialChangesetRenderer' => 'Phobject',
    'DifferentialChangesetTestRenderer' => 'DifferentialChangesetRenderer',
    'DifferentialChangesetTwoUpRenderer' => 'DifferentialChangesetHTMLRenderer',
    'DifferentialChangesetTwoUpTestRenderer' => 'DifferentialChangesetTestRenderer',
    'DifferentialChangesetViewController' => 'DifferentialController',
    'DifferentialCloseConduitAPIMethod' => 'DifferentialConduitAPIMethod',
    'DifferentialCommentPreviewController' => 'DifferentialController',
    'DifferentialCommentSaveController' => 'DifferentialController',
    'DifferentialCommitMessageParser' => 'Phobject',
    'DifferentialCommitMessageParserTestCase' => 'PhabricatorTestCase',
    'DifferentialCommitsField' => 'DifferentialCustomField',
    'DifferentialConduitAPIMethod' => 'ConduitAPIMethod',
    'DifferentialConflictsField' => 'DifferentialCustomField',
    'DifferentialController' => 'PhabricatorController',
    'DifferentialCoreCustomField' => 'DifferentialCustomField',
    'DifferentialCreateCommentConduitAPIMethod' => 'DifferentialConduitAPIMethod',
    'DifferentialCreateDiffConduitAPIMethod' => 'DifferentialConduitAPIMethod',
    'DifferentialCreateInlineConduitAPIMethod' => 'DifferentialConduitAPIMethod',
    'DifferentialCreateMailReceiver' => 'PhabricatorMailReceiver',
    'DifferentialCreateRawDiffConduitAPIMethod' => 'DifferentialConduitAPIMethod',
    'DifferentialCreateRevisionConduitAPIMethod' => 'DifferentialConduitAPIMethod',
    'DifferentialCustomField' => 'PhabricatorCustomField',
    'DifferentialCustomFieldDependsOnParser' => 'PhabricatorCustomFieldMonogramParser',
    'DifferentialCustomFieldDependsOnParserTestCase' => 'PhabricatorTestCase',
    'DifferentialCustomFieldNumericIndex' => 'PhabricatorCustomFieldNumericIndexStorage',
    'DifferentialCustomFieldRevertsParser' => 'PhabricatorCustomFieldMonogramParser',
    'DifferentialCustomFieldRevertsParserTestCase' => 'PhabricatorTestCase',
    'DifferentialCustomFieldStorage' => 'PhabricatorCustomFieldStorage',
    'DifferentialCustomFieldStringIndex' => 'PhabricatorCustomFieldStringIndexStorage',
    'DifferentialDAO' => 'PhabricatorLiskDAO',
    'DifferentialDefaultViewCapability' => 'PhabricatorPolicyCapability',
    'DifferentialDependenciesField' => 'DifferentialCustomField',
    'DifferentialDependsOnField' => 'DifferentialCustomField',
    'DifferentialDiff' => array(
      'DifferentialDAO',
      'PhabricatorPolicyInterface',
      'HarbormasterBuildableInterface',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorDestructibleInterface',
    ),
    'DifferentialDiffAffectedFilesHeraldField' => 'DifferentialDiffHeraldField',
    'DifferentialDiffAuthorHeraldField' => 'DifferentialDiffHeraldField',
    'DifferentialDiffAuthorProjectsHeraldField' => 'DifferentialDiffHeraldField',
    'DifferentialDiffContentAddedHeraldField' => 'DifferentialDiffHeraldField',
    'DifferentialDiffContentHeraldField' => 'DifferentialDiffHeraldField',
    'DifferentialDiffContentRemovedHeraldField' => 'DifferentialDiffHeraldField',
    'DifferentialDiffCreateController' => 'DifferentialController',
    'DifferentialDiffEditor' => 'PhabricatorApplicationTransactionEditor',
    'DifferentialDiffExtractionEngine' => 'Phobject',
    'DifferentialDiffHeraldField' => 'HeraldField',
    'DifferentialDiffHeraldFieldGroup' => 'HeraldFieldGroup',
    'DifferentialDiffInlineCommentQuery' => 'PhabricatorDiffInlineCommentQuery',
    'DifferentialDiffPHIDType' => 'PhabricatorPHIDType',
    'DifferentialDiffProperty' => 'DifferentialDAO',
    'DifferentialDiffQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'DifferentialDiffRepositoryHeraldField' => 'DifferentialDiffHeraldField',
    'DifferentialDiffRepositoryProjectsHeraldField' => 'DifferentialDiffHeraldField',
    'DifferentialDiffTestCase' => 'PhutilTestCase',
    'DifferentialDiffTransaction' => 'PhabricatorApplicationTransaction',
    'DifferentialDiffTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'DifferentialDiffViewController' => 'DifferentialController',
    'DifferentialDoorkeeperRevisionFeedStoryPublisher' => 'DoorkeeperFeedStoryPublisher',
    'DifferentialDraft' => 'DifferentialDAO',
    'DifferentialEditPolicyField' => 'DifferentialCoreCustomField',
    'DifferentialFieldParseException' => 'Exception',
    'DifferentialFieldValidationException' => 'Exception',
    'DifferentialFindConduitAPIMethod' => 'DifferentialConduitAPIMethod',
    'DifferentialGetAllDiffsConduitAPIMethod' => 'DifferentialConduitAPIMethod',
    'DifferentialGetCommitMessageConduitAPIMethod' => 'DifferentialConduitAPIMethod',
    'DifferentialGetCommitPathsConduitAPIMethod' => 'DifferentialConduitAPIMethod',
    'DifferentialGetDiffConduitAPIMethod' => 'DifferentialConduitAPIMethod',
    'DifferentialGetRawDiffConduitAPIMethod' => 'DifferentialConduitAPIMethod',
    'DifferentialGetRevisionCommentsConduitAPIMethod' => 'DifferentialConduitAPIMethod',
    'DifferentialGetRevisionConduitAPIMethod' => 'DifferentialConduitAPIMethod',
    'DifferentialGetWorkingCopy' => 'Phobject',
    'DifferentialGitHubLandingStrategy' => 'DifferentialLandingStrategy',
    'DifferentialGitSVNIDField' => 'DifferentialCustomField',
    'DifferentialHarbormasterField' => 'DifferentialCustomField',
    'DifferentialHiddenComment' => 'DifferentialDAO',
    'DifferentialHostField' => 'DifferentialCustomField',
    'DifferentialHostedGitLandingStrategy' => 'DifferentialLandingStrategy',
    'DifferentialHostedMercurialLandingStrategy' => 'DifferentialLandingStrategy',
    'DifferentialHovercardEngineExtension' => 'PhabricatorHovercardEngineExtension',
    'DifferentialHunk' => array(
      'DifferentialDAO',
      'PhabricatorPolicyInterface',
    ),
    'DifferentialHunkParser' => 'Phobject',
    'DifferentialHunkParserTestCase' => 'PhabricatorTestCase',
    'DifferentialHunkQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'DifferentialHunkTestCase' => 'PhutilTestCase',
    'DifferentialInlineComment' => array(
      'Phobject',
      'PhabricatorInlineCommentInterface',
    ),
    'DifferentialInlineCommentEditController' => 'PhabricatorInlineCommentController',
    'DifferentialInlineCommentPreviewController' => 'PhabricatorInlineCommentPreviewController',
    'DifferentialInlineCommentQuery' => 'PhabricatorOffsetPagedQuery',
    'DifferentialJIRAIssuesField' => 'DifferentialStoredCustomField',
    'DifferentialLandingActionMenuEventListener' => 'PhabricatorEventListener',
    'DifferentialLandingStrategy' => 'Phobject',
    'DifferentialLegacyHunk' => 'DifferentialHunk',
    'DifferentialLineAdjustmentMap' => 'Phobject',
    'DifferentialLintField' => 'DifferentialHarbormasterField',
    'DifferentialLintStatus' => 'Phobject',
    'DifferentialLocalCommitsView' => 'AphrontView',
    'DifferentialManiphestTasksField' => 'DifferentialCoreCustomField',
    'DifferentialModernHunk' => 'DifferentialHunk',
    'DifferentialNextStepField' => 'DifferentialCustomField',
    'DifferentialParseCacheGarbageCollector' => 'PhabricatorGarbageCollector',
    'DifferentialParseCommitMessageConduitAPIMethod' => 'DifferentialConduitAPIMethod',
    'DifferentialParseRenderTestCase' => 'PhabricatorTestCase',
    'DifferentialPathField' => 'DifferentialCustomField',
    'DifferentialPrimaryPaneView' => 'AphrontView',
    'DifferentialProjectReviewersField' => 'DifferentialCustomField',
    'DifferentialProjectsField' => 'DifferentialCoreCustomField',
    'DifferentialQueryConduitAPIMethod' => 'DifferentialConduitAPIMethod',
    'DifferentialQueryDiffsConduitAPIMethod' => 'DifferentialConduitAPIMethod',
    'DifferentialRawDiffRenderer' => 'Phobject',
    'DifferentialReleephRequestFieldSpecification' => 'Phobject',
    'DifferentialRemarkupRule' => 'PhabricatorObjectRemarkupRule',
    'DifferentialReplyHandler' => 'PhabricatorApplicationTransactionReplyHandler',
    'DifferentialRepositoryField' => 'DifferentialCoreCustomField',
    'DifferentialRepositoryLookup' => 'Phobject',
    'DifferentialRequiredSignaturesField' => 'DifferentialCoreCustomField',
    'DifferentialRevertPlanField' => 'DifferentialStoredCustomField',
    'DifferentialReviewedByField' => 'DifferentialCoreCustomField',
    'DifferentialReviewer' => 'Phobject',
    'DifferentialReviewerForRevisionEdgeType' => 'PhabricatorEdgeType',
    'DifferentialReviewerStatus' => 'Phobject',
    'DifferentialReviewersAddBlockingReviewersHeraldAction' => 'DifferentialReviewersHeraldAction',
    'DifferentialReviewersAddBlockingSelfHeraldAction' => 'DifferentialReviewersHeraldAction',
    'DifferentialReviewersAddReviewersHeraldAction' => 'DifferentialReviewersHeraldAction',
    'DifferentialReviewersAddSelfHeraldAction' => 'DifferentialReviewersHeraldAction',
    'DifferentialReviewersField' => 'DifferentialCoreCustomField',
    'DifferentialReviewersHeraldAction' => 'HeraldAction',
    'DifferentialReviewersView' => 'AphrontView',
    'DifferentialRevision' => array(
      'DifferentialDAO',
      'PhabricatorTokenReceiverInterface',
      'PhabricatorPolicyInterface',
      'PhabricatorExtendedPolicyInterface',
      'PhabricatorFlaggableInterface',
      'PhrequentTrackableInterface',
      'HarbormasterBuildableInterface',
      'PhabricatorSubscribableInterface',
      'PhabricatorCustomFieldInterface',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorMentionableInterface',
      'PhabricatorDestructibleInterface',
      'PhabricatorProjectInterface',
      'PhabricatorFulltextInterface',
    ),
    'DifferentialRevisionAffectedFilesHeraldField' => 'DifferentialRevisionHeraldField',
    'DifferentialRevisionAuthorHeraldField' => 'DifferentialRevisionHeraldField',
    'DifferentialRevisionAuthorProjectsHeraldField' => 'DifferentialRevisionHeraldField',
    'DifferentialRevisionCloseDetailsController' => 'DifferentialController',
    'DifferentialRevisionContentAddedHeraldField' => 'DifferentialRevisionHeraldField',
    'DifferentialRevisionContentHeraldField' => 'DifferentialRevisionHeraldField',
    'DifferentialRevisionContentRemovedHeraldField' => 'DifferentialRevisionHeraldField',
    'DifferentialRevisionControlSystem' => 'Phobject',
    'DifferentialRevisionDependedOnByRevisionEdgeType' => 'PhabricatorEdgeType',
    'DifferentialRevisionDependsOnRevisionEdgeType' => 'PhabricatorEdgeType',
    'DifferentialRevisionDetailView' => 'AphrontView',
    'DifferentialRevisionEditController' => 'DifferentialController',
    'DifferentialRevisionFulltextEngine' => 'PhabricatorFulltextEngine',
    'DifferentialRevisionHasCommitEdgeType' => 'PhabricatorEdgeType',
    'DifferentialRevisionHasReviewerEdgeType' => 'PhabricatorEdgeType',
    'DifferentialRevisionHasTaskEdgeType' => 'PhabricatorEdgeType',
    'DifferentialRevisionHeraldField' => 'HeraldField',
    'DifferentialRevisionHeraldFieldGroup' => 'HeraldFieldGroup',
    'DifferentialRevisionIDField' => 'DifferentialCustomField',
    'DifferentialRevisionLandController' => 'DifferentialController',
    'DifferentialRevisionListController' => 'DifferentialController',
    'DifferentialRevisionListView' => 'AphrontView',
    'DifferentialRevisionMailReceiver' => 'PhabricatorObjectMailReceiver',
    'DifferentialRevisionOperationController' => 'DifferentialController',
    'DifferentialRevisionPHIDType' => 'PhabricatorPHIDType',
    'DifferentialRevisionPackageHeraldField' => 'DifferentialRevisionHeraldField',
    'DifferentialRevisionPackageOwnerHeraldField' => 'DifferentialRevisionHeraldField',
    'DifferentialRevisionQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'DifferentialRevisionRepositoryHeraldField' => 'DifferentialRevisionHeraldField',
    'DifferentialRevisionRepositoryProjectsHeraldField' => 'DifferentialRevisionHeraldField',
    'DifferentialRevisionReviewersHeraldField' => 'DifferentialRevisionHeraldField',
    'DifferentialRevisionSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'DifferentialRevisionStatus' => 'Phobject',
    'DifferentialRevisionSummaryHeraldField' => 'DifferentialRevisionHeraldField',
    'DifferentialRevisionTitleHeraldField' => 'DifferentialRevisionHeraldField',
    'DifferentialRevisionUpdateHistoryView' => 'AphrontView',
    'DifferentialRevisionViewController' => 'DifferentialController',
    'DifferentialSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'DifferentialSetDiffPropertyConduitAPIMethod' => 'DifferentialConduitAPIMethod',
    'DifferentialStoredCustomField' => 'DifferentialCustomField',
    'DifferentialSubscribersField' => 'DifferentialCoreCustomField',
    'DifferentialSummaryField' => 'DifferentialCoreCustomField',
    'DifferentialTestPlanField' => 'DifferentialCoreCustomField',
    'DifferentialTitleField' => 'DifferentialCoreCustomField',
    'DifferentialTransaction' => 'PhabricatorApplicationTransaction',
    'DifferentialTransactionComment' => 'PhabricatorApplicationTransactionComment',
    'DifferentialTransactionEditor' => 'PhabricatorApplicationTransactionEditor',
    'DifferentialTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'DifferentialTransactionView' => 'PhabricatorApplicationTransactionView',
    'DifferentialUnitField' => 'DifferentialHarbormasterField',
    'DifferentialUnitStatus' => 'Phobject',
    'DifferentialUnitTestResult' => 'Phobject',
    'DifferentialUpdateRevisionConduitAPIMethod' => 'DifferentialConduitAPIMethod',
    'DifferentialViewPolicyField' => 'DifferentialCoreCustomField',
    'DiffusionAuditorDatasource' => 'PhabricatorTypeaheadCompositeDatasource',
    'DiffusionAuditorFunctionDatasource' => 'PhabricatorTypeaheadCompositeDatasource',
    'DiffusionAuditorsAddAuditorsHeraldAction' => 'DiffusionAuditorsHeraldAction',
    'DiffusionAuditorsAddSelfHeraldAction' => 'DiffusionAuditorsHeraldAction',
    'DiffusionAuditorsHeraldAction' => 'HeraldAction',
    'DiffusionBlameConduitAPIMethod' => 'DiffusionQueryConduitAPIMethod',
    'DiffusionBlameQuery' => 'DiffusionQuery',
    'DiffusionBlockHeraldAction' => 'HeraldAction',
    'DiffusionBranchQueryConduitAPIMethod' => 'DiffusionQueryConduitAPIMethod',
    'DiffusionBranchTableController' => 'DiffusionController',
    'DiffusionBranchTableView' => 'DiffusionView',
    'DiffusionBrowseController' => 'DiffusionController',
    'DiffusionBrowseQueryConduitAPIMethod' => 'DiffusionQueryConduitAPIMethod',
    'DiffusionBrowseResultSet' => 'Phobject',
    'DiffusionBrowseTableView' => 'DiffusionView',
    'DiffusionCachedResolveRefsQuery' => 'DiffusionLowLevelQuery',
    'DiffusionChangeController' => 'DiffusionController',
    'DiffusionChangeHeraldFieldGroup' => 'HeraldFieldGroup',
    'DiffusionCommitAffectedFilesHeraldField' => 'DiffusionCommitHeraldField',
    'DiffusionCommitAuthorHeraldField' => 'DiffusionCommitHeraldField',
    'DiffusionCommitAutocloseHeraldField' => 'DiffusionCommitHeraldField',
    'DiffusionCommitBranchesController' => 'DiffusionController',
    'DiffusionCommitBranchesHeraldField' => 'DiffusionCommitHeraldField',
    'DiffusionCommitCommitterHeraldField' => 'DiffusionCommitHeraldField',
    'DiffusionCommitController' => 'DiffusionController',
    'DiffusionCommitDiffContentAddedHeraldField' => 'DiffusionCommitHeraldField',
    'DiffusionCommitDiffContentHeraldField' => 'DiffusionCommitHeraldField',
    'DiffusionCommitDiffContentRemovedHeraldField' => 'DiffusionCommitHeraldField',
    'DiffusionCommitDiffEnormousHeraldField' => 'DiffusionCommitHeraldField',
    'DiffusionCommitEditController' => 'DiffusionController',
    'DiffusionCommitFulltextEngine' => 'PhabricatorFulltextEngine',
    'DiffusionCommitHasRevisionEdgeType' => 'PhabricatorEdgeType',
    'DiffusionCommitHasTaskEdgeType' => 'PhabricatorEdgeType',
    'DiffusionCommitHash' => 'Phobject',
    'DiffusionCommitHeraldField' => 'HeraldField',
    'DiffusionCommitHeraldFieldGroup' => 'HeraldFieldGroup',
    'DiffusionCommitHookEngine' => 'Phobject',
    'DiffusionCommitHookRejectException' => 'Exception',
    'DiffusionCommitMergeHeraldField' => 'DiffusionCommitHeraldField',
    'DiffusionCommitMessageHeraldField' => 'DiffusionCommitHeraldField',
    'DiffusionCommitPackageAuditHeraldField' => 'DiffusionCommitHeraldField',
    'DiffusionCommitPackageHeraldField' => 'DiffusionCommitHeraldField',
    'DiffusionCommitPackageOwnerHeraldField' => 'DiffusionCommitHeraldField',
    'DiffusionCommitParentsQueryConduitAPIMethod' => 'DiffusionQueryConduitAPIMethod',
    'DiffusionCommitQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'DiffusionCommitRef' => 'Phobject',
    'DiffusionCommitRemarkupRule' => 'PhabricatorObjectRemarkupRule',
    'DiffusionCommitRemarkupRuleTestCase' => 'PhabricatorTestCase',
    'DiffusionCommitRepositoryHeraldField' => 'DiffusionCommitHeraldField',
    'DiffusionCommitRepositoryProjectsHeraldField' => 'DiffusionCommitHeraldField',
    'DiffusionCommitRevertedByCommitEdgeType' => 'PhabricatorEdgeType',
    'DiffusionCommitRevertsCommitEdgeType' => 'PhabricatorEdgeType',
    'DiffusionCommitReviewerHeraldField' => 'DiffusionCommitHeraldField',
    'DiffusionCommitRevisionAcceptedHeraldField' => 'DiffusionCommitHeraldField',
    'DiffusionCommitRevisionHeraldField' => 'DiffusionCommitHeraldField',
    'DiffusionCommitRevisionReviewersHeraldField' => 'DiffusionCommitHeraldField',
    'DiffusionCommitRevisionSubscribersHeraldField' => 'DiffusionCommitHeraldField',
    'DiffusionCommitTagsController' => 'DiffusionController',
    'DiffusionConduitAPIMethod' => 'ConduitAPIMethod',
    'DiffusionController' => 'PhabricatorController',
    'DiffusionCreateCommentConduitAPIMethod' => 'DiffusionConduitAPIMethod',
    'DiffusionCreateRepositoriesCapability' => 'PhabricatorPolicyCapability',
    'DiffusionDefaultEditCapability' => 'PhabricatorPolicyCapability',
    'DiffusionDefaultPushCapability' => 'PhabricatorPolicyCapability',
    'DiffusionDefaultViewCapability' => 'PhabricatorPolicyCapability',
    'DiffusionDiffController' => 'DiffusionController',
    'DiffusionDiffInlineCommentQuery' => 'PhabricatorDiffInlineCommentQuery',
    'DiffusionDiffQueryConduitAPIMethod' => 'DiffusionQueryConduitAPIMethod',
    'DiffusionDoorkeeperCommitFeedStoryPublisher' => 'DoorkeeperFeedStoryPublisher',
    'DiffusionEmptyResultView' => 'DiffusionView',
    'DiffusionExistsQueryConduitAPIMethod' => 'DiffusionQueryConduitAPIMethod',
    'DiffusionExternalController' => 'DiffusionController',
    'DiffusionExternalSymbolQuery' => 'Phobject',
    'DiffusionExternalSymbolsSource' => 'Phobject',
    'DiffusionFileContentQuery' => 'DiffusionQuery',
    'DiffusionFileContentQueryConduitAPIMethod' => 'DiffusionQueryConduitAPIMethod',
    'DiffusionFindSymbolsConduitAPIMethod' => 'DiffusionConduitAPIMethod',
    'DiffusionGetLintMessagesConduitAPIMethod' => 'DiffusionConduitAPIMethod',
    'DiffusionGetRecentCommitsByPathConduitAPIMethod' => 'DiffusionConduitAPIMethod',
    'DiffusionGitBlameQuery' => 'DiffusionBlameQuery',
    'DiffusionGitBranch' => 'Phobject',
    'DiffusionGitBranchTestCase' => 'PhabricatorTestCase',
    'DiffusionGitFileContentQuery' => 'DiffusionFileContentQuery',
    'DiffusionGitRawDiffQuery' => 'DiffusionRawDiffQuery',
    'DiffusionGitReceivePackSSHWorkflow' => 'DiffusionGitSSHWorkflow',
    'DiffusionGitRequest' => 'DiffusionRequest',
    'DiffusionGitResponse' => 'AphrontResponse',
    'DiffusionGitSSHWorkflow' => 'DiffusionSSHWorkflow',
    'DiffusionGitUploadPackSSHWorkflow' => 'DiffusionGitSSHWorkflow',
    'DiffusionHistoryController' => 'DiffusionController',
    'DiffusionHistoryQueryConduitAPIMethod' => 'DiffusionQueryConduitAPIMethod',
    'DiffusionHistoryTableView' => 'DiffusionView',
    'DiffusionHovercardEngineExtension' => 'PhabricatorHovercardEngineExtension',
    'DiffusionInlineCommentController' => 'PhabricatorInlineCommentController',
    'DiffusionInlineCommentPreviewController' => 'PhabricatorInlineCommentPreviewController',
    'DiffusionLastModifiedController' => 'DiffusionController',
    'DiffusionLastModifiedQueryConduitAPIMethod' => 'DiffusionQueryConduitAPIMethod',
    'DiffusionLintController' => 'DiffusionController',
    'DiffusionLintCountQuery' => 'PhabricatorQuery',
    'DiffusionLintSaveRunner' => 'Phobject',
    'DiffusionLookSoonConduitAPIMethod' => 'DiffusionConduitAPIMethod',
    'DiffusionLowLevelCommitFieldsQuery' => 'DiffusionLowLevelQuery',
    'DiffusionLowLevelCommitQuery' => 'DiffusionLowLevelQuery',
    'DiffusionLowLevelGitRefQuery' => 'DiffusionLowLevelQuery',
    'DiffusionLowLevelMercurialBranchesQuery' => 'DiffusionLowLevelQuery',
    'DiffusionLowLevelMercurialPathsQuery' => 'DiffusionLowLevelQuery',
    'DiffusionLowLevelMercurialPathsQueryTests' => 'PhabricatorTestCase',
    'DiffusionLowLevelParentsQuery' => 'DiffusionLowLevelQuery',
    'DiffusionLowLevelQuery' => 'Phobject',
    'DiffusionLowLevelResolveRefsQuery' => 'DiffusionLowLevelQuery',
    'DiffusionMercurialBlameQuery' => 'DiffusionBlameQuery',
    'DiffusionMercurialFileContentQuery' => 'DiffusionFileContentQuery',
    'DiffusionMercurialRawDiffQuery' => 'DiffusionRawDiffQuery',
    'DiffusionMercurialRequest' => 'DiffusionRequest',
    'DiffusionMercurialResponse' => 'AphrontResponse',
    'DiffusionMercurialSSHWorkflow' => 'DiffusionSSHWorkflow',
    'DiffusionMercurialServeSSHWorkflow' => 'DiffusionMercurialSSHWorkflow',
    'DiffusionMercurialWireClientSSHProtocolChannel' => 'PhutilProtocolChannel',
    'DiffusionMercurialWireProtocol' => 'Phobject',
    'DiffusionMercurialWireProtocolTests' => 'PhabricatorTestCase',
    'DiffusionMercurialWireSSHTestCase' => 'PhabricatorTestCase',
    'DiffusionMergedCommitsQueryConduitAPIMethod' => 'DiffusionQueryConduitAPIMethod',
    'DiffusionMirrorDeleteController' => 'DiffusionController',
    'DiffusionMirrorEditController' => 'DiffusionController',
    'DiffusionPathChange' => 'Phobject',
    'DiffusionPathChangeQuery' => 'Phobject',
    'DiffusionPathCompleteController' => 'DiffusionController',
    'DiffusionPathIDQuery' => 'Phobject',
    'DiffusionPathQuery' => 'Phobject',
    'DiffusionPathQueryTestCase' => 'PhabricatorTestCase',
    'DiffusionPathTreeController' => 'DiffusionController',
    'DiffusionPathValidateController' => 'DiffusionController',
    'DiffusionPhpExternalSymbolsSource' => 'DiffusionExternalSymbolsSource',
    'DiffusionPreCommitContentAffectedFilesHeraldField' => 'DiffusionPreCommitContentHeraldField',
    'DiffusionPreCommitContentAuthorHeraldField' => 'DiffusionPreCommitContentHeraldField',
    'DiffusionPreCommitContentAuthorRawHeraldField' => 'DiffusionPreCommitContentHeraldField',
    'DiffusionPreCommitContentBranchesHeraldField' => 'DiffusionPreCommitContentHeraldField',
    'DiffusionPreCommitContentCommitterHeraldField' => 'DiffusionPreCommitContentHeraldField',
    'DiffusionPreCommitContentCommitterRawHeraldField' => 'DiffusionPreCommitContentHeraldField',
    'DiffusionPreCommitContentDiffContentAddedHeraldField' => 'DiffusionPreCommitContentHeraldField',
    'DiffusionPreCommitContentDiffContentHeraldField' => 'DiffusionPreCommitContentHeraldField',
    'DiffusionPreCommitContentDiffContentRemovedHeraldField' => 'DiffusionPreCommitContentHeraldField',
    'DiffusionPreCommitContentDiffEnormousHeraldField' => 'DiffusionPreCommitContentHeraldField',
    'DiffusionPreCommitContentHeraldField' => 'HeraldField',
    'DiffusionPreCommitContentMergeHeraldField' => 'DiffusionPreCommitContentHeraldField',
    'DiffusionPreCommitContentMessageHeraldField' => 'DiffusionPreCommitContentHeraldField',
    'DiffusionPreCommitContentPusherHeraldField' => 'DiffusionPreCommitContentHeraldField',
    'DiffusionPreCommitContentPusherIsCommitterHeraldField' => 'DiffusionPreCommitContentHeraldField',
    'DiffusionPreCommitContentPusherProjectsHeraldField' => 'DiffusionPreCommitContentHeraldField',
    'DiffusionPreCommitContentRepositoryHeraldField' => 'DiffusionPreCommitContentHeraldField',
    'DiffusionPreCommitContentRepositoryProjectsHeraldField' => 'DiffusionPreCommitContentHeraldField',
    'DiffusionPreCommitContentRevisionAcceptedHeraldField' => 'DiffusionPreCommitContentHeraldField',
    'DiffusionPreCommitContentRevisionHeraldField' => 'DiffusionPreCommitContentHeraldField',
    'DiffusionPreCommitContentRevisionReviewersHeraldField' => 'DiffusionPreCommitContentHeraldField',
    'DiffusionPreCommitContentRevisionSubscribersHeraldField' => 'DiffusionPreCommitContentHeraldField',
    'DiffusionPreCommitRefChangeHeraldField' => 'DiffusionPreCommitRefHeraldField',
    'DiffusionPreCommitRefHeraldField' => 'HeraldField',
    'DiffusionPreCommitRefHeraldFieldGroup' => 'HeraldFieldGroup',
    'DiffusionPreCommitRefNameHeraldField' => 'DiffusionPreCommitRefHeraldField',
    'DiffusionPreCommitRefPusherHeraldField' => 'DiffusionPreCommitRefHeraldField',
    'DiffusionPreCommitRefPusherProjectsHeraldField' => 'DiffusionPreCommitRefHeraldField',
    'DiffusionPreCommitRefRepositoryHeraldField' => 'DiffusionPreCommitRefHeraldField',
    'DiffusionPreCommitRefRepositoryProjectsHeraldField' => 'DiffusionPreCommitRefHeraldField',
    'DiffusionPreCommitRefTypeHeraldField' => 'DiffusionPreCommitRefHeraldField',
    'DiffusionPushCapability' => 'PhabricatorPolicyCapability',
    'DiffusionPushEventViewController' => 'DiffusionPushLogController',
    'DiffusionPushLogController' => 'DiffusionController',
    'DiffusionPushLogListController' => 'DiffusionPushLogController',
    'DiffusionPushLogListView' => 'AphrontView',
    'DiffusionPythonExternalSymbolsSource' => 'DiffusionExternalSymbolsSource',
    'DiffusionQuery' => 'PhabricatorQuery',
    'DiffusionQueryCommitsConduitAPIMethod' => 'DiffusionConduitAPIMethod',
    'DiffusionQueryConduitAPIMethod' => 'DiffusionConduitAPIMethod',
    'DiffusionQueryPathsConduitAPIMethod' => 'DiffusionQueryConduitAPIMethod',
    'DiffusionRawDiffQuery' => 'DiffusionQuery',
    'DiffusionRawDiffQueryConduitAPIMethod' => 'DiffusionQueryConduitAPIMethod',
    'DiffusionReadmeView' => 'DiffusionView',
    'DiffusionRefDatasource' => 'PhabricatorTypeaheadDatasource',
    'DiffusionRefNotFoundException' => 'Exception',
    'DiffusionRefTableController' => 'DiffusionController',
    'DiffusionRefsQueryConduitAPIMethod' => 'DiffusionQueryConduitAPIMethod',
    'DiffusionRenameHistoryQuery' => 'Phobject',
    'DiffusionRepositoryByIDRemarkupRule' => 'PhabricatorObjectRemarkupRule',
    'DiffusionRepositoryController' => 'DiffusionController',
    'DiffusionRepositoryCreateController' => 'DiffusionRepositoryEditController',
    'DiffusionRepositoryDatasource' => 'PhabricatorTypeaheadDatasource',
    'DiffusionRepositoryDefaultController' => 'DiffusionController',
    'DiffusionRepositoryEditActionsController' => 'DiffusionRepositoryEditController',
    'DiffusionRepositoryEditActivateController' => 'DiffusionRepositoryEditController',
    'DiffusionRepositoryEditAutomationController' => 'DiffusionRepositoryEditController',
    'DiffusionRepositoryEditBasicController' => 'DiffusionRepositoryEditController',
    'DiffusionRepositoryEditBranchesController' => 'DiffusionRepositoryEditController',
    'DiffusionRepositoryEditController' => 'DiffusionController',
    'DiffusionRepositoryEditDangerousController' => 'DiffusionRepositoryEditController',
    'DiffusionRepositoryEditDeleteController' => 'DiffusionRepositoryEditController',
    'DiffusionRepositoryEditEncodingController' => 'DiffusionRepositoryEditController',
    'DiffusionRepositoryEditHostingController' => 'DiffusionRepositoryEditController',
    'DiffusionRepositoryEditMainController' => 'DiffusionRepositoryEditController',
    'DiffusionRepositoryEditStagingController' => 'DiffusionRepositoryEditController',
    'DiffusionRepositoryEditStorageController' => 'DiffusionRepositoryEditController',
    'DiffusionRepositoryEditSubversionController' => 'DiffusionRepositoryEditController',
    'DiffusionRepositoryEditUpdateController' => 'DiffusionRepositoryEditController',
    'DiffusionRepositoryListController' => 'DiffusionController',
    'DiffusionRepositoryNewController' => 'DiffusionController',
    'DiffusionRepositoryPath' => 'Phobject',
    'DiffusionRepositoryRef' => 'Phobject',
    'DiffusionRepositoryRemarkupRule' => 'PhabricatorObjectRemarkupRule',
    'DiffusionRepositorySymbolsController' => 'DiffusionRepositoryEditController',
    'DiffusionRepositoryTag' => 'Phobject',
    'DiffusionRepositoryTestAutomationController' => 'DiffusionRepositoryEditController',
    'DiffusionRepositoryURIsIndexEngineExtension' => 'PhabricatorIndexEngineExtension',
    'DiffusionRequest' => 'Phobject',
    'DiffusionResolveRefsConduitAPIMethod' => 'DiffusionQueryConduitAPIMethod',
    'DiffusionResolveUserQuery' => 'Phobject',
    'DiffusionSSHWorkflow' => 'PhabricatorSSHWorkflow',
    'DiffusionSearchQueryConduitAPIMethod' => 'DiffusionQueryConduitAPIMethod',
    'DiffusionServeController' => 'DiffusionController',
    'DiffusionSetPasswordSettingsPanel' => 'PhabricatorSettingsPanel',
    'DiffusionSetupException' => 'Exception',
    'DiffusionSubversionSSHWorkflow' => 'DiffusionSSHWorkflow',
    'DiffusionSubversionServeSSHWorkflow' => 'DiffusionSubversionSSHWorkflow',
    'DiffusionSubversionWireProtocol' => 'Phobject',
    'DiffusionSubversionWireProtocolTestCase' => 'PhabricatorTestCase',
    'DiffusionSvnBlameQuery' => 'DiffusionBlameQuery',
    'DiffusionSvnFileContentQuery' => 'DiffusionFileContentQuery',
    'DiffusionSvnRawDiffQuery' => 'DiffusionRawDiffQuery',
    'DiffusionSvnRequest' => 'DiffusionRequest',
    'DiffusionSymbolController' => 'DiffusionController',
    'DiffusionSymbolDatasource' => 'PhabricatorTypeaheadDatasource',
    'DiffusionSymbolQuery' => 'PhabricatorOffsetPagedQuery',
    'DiffusionTagListController' => 'DiffusionController',
    'DiffusionTagListView' => 'DiffusionView',
    'DiffusionTagsQueryConduitAPIMethod' => 'DiffusionQueryConduitAPIMethod',
    'DiffusionURITestCase' => 'PhutilTestCase',
    'DiffusionUpdateCoverageConduitAPIMethod' => 'DiffusionConduitAPIMethod',
    'DiffusionView' => 'AphrontView',
    'DivinerArticleAtomizer' => 'DivinerAtomizer',
    'DivinerAtom' => 'Phobject',
    'DivinerAtomCache' => 'DivinerDiskCache',
    'DivinerAtomController' => 'DivinerController',
    'DivinerAtomListController' => 'DivinerController',
    'DivinerAtomPHIDType' => 'PhabricatorPHIDType',
    'DivinerAtomQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'DivinerAtomRef' => 'Phobject',
    'DivinerAtomSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'DivinerAtomizeWorkflow' => 'DivinerWorkflow',
    'DivinerAtomizer' => 'Phobject',
    'DivinerBookController' => 'DivinerController',
    'DivinerBookDatasource' => 'PhabricatorTypeaheadDatasource',
    'DivinerBookEditController' => 'DivinerController',
    'DivinerBookItemView' => 'AphrontTagView',
    'DivinerBookPHIDType' => 'PhabricatorPHIDType',
    'DivinerBookQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'DivinerController' => 'PhabricatorController',
    'DivinerDAO' => 'PhabricatorLiskDAO',
    'DivinerDefaultEditCapability' => 'PhabricatorPolicyCapability',
    'DivinerDefaultRenderer' => 'DivinerRenderer',
    'DivinerDefaultViewCapability' => 'PhabricatorPolicyCapability',
    'DivinerDiskCache' => 'Phobject',
    'DivinerFileAtomizer' => 'DivinerAtomizer',
    'DivinerFindController' => 'DivinerController',
    'DivinerGenerateWorkflow' => 'DivinerWorkflow',
    'DivinerLiveAtom' => 'DivinerDAO',
    'DivinerLiveBook' => array(
      'DivinerDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorProjectInterface',
      'PhabricatorDestructibleInterface',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorFulltextInterface',
    ),
    'DivinerLiveBookEditor' => 'PhabricatorApplicationTransactionEditor',
    'DivinerLiveBookFulltextEngine' => 'PhabricatorFulltextEngine',
    'DivinerLiveBookTransaction' => 'PhabricatorApplicationTransaction',
    'DivinerLiveBookTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'DivinerLivePublisher' => 'DivinerPublisher',
    'DivinerLiveSymbol' => array(
      'DivinerDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorMarkupInterface',
      'PhabricatorDestructibleInterface',
      'PhabricatorFulltextInterface',
    ),
    'DivinerLiveSymbolFulltextEngine' => 'PhabricatorFulltextEngine',
    'DivinerMainController' => 'DivinerController',
    'DivinerPHPAtomizer' => 'DivinerAtomizer',
    'DivinerParameterTableView' => 'AphrontTagView',
    'DivinerPublishCache' => 'DivinerDiskCache',
    'DivinerPublisher' => 'Phobject',
    'DivinerRenderer' => 'Phobject',
    'DivinerReturnTableView' => 'AphrontTagView',
    'DivinerSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'DivinerSectionView' => 'AphrontTagView',
    'DivinerStaticPublisher' => 'DivinerPublisher',
    'DivinerSymbolRemarkupRule' => 'PhutilRemarkupRule',
    'DivinerWorkflow' => 'PhabricatorManagementWorkflow',
    'DoorkeeperAsanaFeedWorker' => 'DoorkeeperFeedWorker',
    'DoorkeeperAsanaRemarkupRule' => 'DoorkeeperRemarkupRule',
    'DoorkeeperBridge' => 'Phobject',
    'DoorkeeperBridgeAsana' => 'DoorkeeperBridge',
    'DoorkeeperBridgeJIRA' => 'DoorkeeperBridge',
    'DoorkeeperBridgeJIRATestCase' => 'PhabricatorTestCase',
    'DoorkeeperDAO' => 'PhabricatorLiskDAO',
    'DoorkeeperExternalObject' => array(
      'DoorkeeperDAO',
      'PhabricatorPolicyInterface',
    ),
    'DoorkeeperExternalObjectQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'DoorkeeperFeedStoryPublisher' => 'Phobject',
    'DoorkeeperFeedWorker' => 'FeedPushWorker',
    'DoorkeeperImportEngine' => 'Phobject',
    'DoorkeeperJIRAFeedWorker' => 'DoorkeeperFeedWorker',
    'DoorkeeperJIRARemarkupRule' => 'DoorkeeperRemarkupRule',
    'DoorkeeperMissingLinkException' => 'Exception',
    'DoorkeeperObjectRef' => 'Phobject',
    'DoorkeeperRemarkupRule' => 'PhutilRemarkupRule',
    'DoorkeeperSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'DoorkeeperTagView' => 'AphrontView',
    'DoorkeeperTagsController' => 'PhabricatorController',
    'DrydockAlmanacServiceHostBlueprintImplementation' => 'DrydockBlueprintImplementation',
    'DrydockApacheWebrootInterface' => 'DrydockWebrootInterface',
    'DrydockAuthorization' => array(
      'DrydockDAO',
      'PhabricatorPolicyInterface',
    ),
    'DrydockAuthorizationAuthorizeController' => 'DrydockController',
    'DrydockAuthorizationListController' => 'DrydockController',
    'DrydockAuthorizationListView' => 'AphrontView',
    'DrydockAuthorizationPHIDType' => 'PhabricatorPHIDType',
    'DrydockAuthorizationQuery' => 'DrydockQuery',
    'DrydockAuthorizationSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'DrydockAuthorizationViewController' => 'DrydockController',
    'DrydockBlueprint' => array(
      'DrydockDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorPolicyInterface',
      'PhabricatorCustomFieldInterface',
    ),
    'DrydockBlueprintController' => 'DrydockController',
    'DrydockBlueprintCoreCustomField' => array(
      'DrydockBlueprintCustomField',
      'PhabricatorStandardCustomFieldInterface',
    ),
    'DrydockBlueprintCreateController' => 'DrydockBlueprintController',
    'DrydockBlueprintCustomField' => 'PhabricatorCustomField',
    'DrydockBlueprintDatasource' => 'PhabricatorTypeaheadDatasource',
    'DrydockBlueprintDisableController' => 'DrydockBlueprintController',
    'DrydockBlueprintEditController' => 'DrydockBlueprintController',
    'DrydockBlueprintEditor' => 'PhabricatorApplicationTransactionEditor',
    'DrydockBlueprintImplementation' => 'Phobject',
    'DrydockBlueprintImplementationTestCase' => 'PhabricatorTestCase',
    'DrydockBlueprintListController' => 'DrydockBlueprintController',
    'DrydockBlueprintPHIDType' => 'PhabricatorPHIDType',
    'DrydockBlueprintQuery' => 'DrydockQuery',
    'DrydockBlueprintSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'DrydockBlueprintTransaction' => 'PhabricatorApplicationTransaction',
    'DrydockBlueprintTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'DrydockBlueprintViewController' => 'DrydockBlueprintController',
    'DrydockCommand' => array(
      'DrydockDAO',
      'PhabricatorPolicyInterface',
    ),
    'DrydockCommandInterface' => 'DrydockInterface',
    'DrydockCommandQuery' => 'DrydockQuery',
    'DrydockConsoleController' => 'DrydockController',
    'DrydockConstants' => 'Phobject',
    'DrydockController' => 'PhabricatorController',
    'DrydockCreateBlueprintsCapability' => 'PhabricatorPolicyCapability',
    'DrydockDAO' => 'PhabricatorLiskDAO',
    'DrydockDefaultEditCapability' => 'PhabricatorPolicyCapability',
    'DrydockDefaultViewCapability' => 'PhabricatorPolicyCapability',
    'DrydockFilesystemInterface' => 'DrydockInterface',
    'DrydockInterface' => 'Phobject',
    'DrydockLandRepositoryOperation' => 'DrydockRepositoryOperationType',
    'DrydockLease' => array(
      'DrydockDAO',
      'PhabricatorPolicyInterface',
    ),
    'DrydockLeaseAcquiredLogType' => 'DrydockLogType',
    'DrydockLeaseActivatedLogType' => 'DrydockLogType',
    'DrydockLeaseActivationFailureLogType' => 'DrydockLogType',
    'DrydockLeaseActivationYieldLogType' => 'DrydockLogType',
    'DrydockLeaseController' => 'DrydockController',
    'DrydockLeaseDatasource' => 'PhabricatorTypeaheadDatasource',
    'DrydockLeaseDestroyedLogType' => 'DrydockLogType',
    'DrydockLeaseListController' => 'DrydockLeaseController',
    'DrydockLeaseListView' => 'AphrontView',
    'DrydockLeaseNoAuthorizationsLogType' => 'DrydockLogType',
    'DrydockLeaseNoBlueprintsLogType' => 'DrydockLogType',
    'DrydockLeasePHIDType' => 'PhabricatorPHIDType',
    'DrydockLeaseQuery' => 'DrydockQuery',
    'DrydockLeaseQueuedLogType' => 'DrydockLogType',
    'DrydockLeaseReclaimLogType' => 'DrydockLogType',
    'DrydockLeaseReleaseController' => 'DrydockLeaseController',
    'DrydockLeaseReleasedLogType' => 'DrydockLogType',
    'DrydockLeaseSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'DrydockLeaseStatus' => 'DrydockConstants',
    'DrydockLeaseUpdateWorker' => 'DrydockWorker',
    'DrydockLeaseViewController' => 'DrydockLeaseController',
    'DrydockLeaseWaitingForResourcesLogType' => 'DrydockLogType',
    'DrydockLog' => array(
      'DrydockDAO',
      'PhabricatorPolicyInterface',
    ),
    'DrydockLogController' => 'DrydockController',
    'DrydockLogGarbageCollector' => 'PhabricatorGarbageCollector',
    'DrydockLogListController' => 'DrydockLogController',
    'DrydockLogListView' => 'AphrontView',
    'DrydockLogQuery' => 'DrydockQuery',
    'DrydockLogSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'DrydockLogType' => 'Phobject',
    'DrydockManagementCommandWorkflow' => 'DrydockManagementWorkflow',
    'DrydockManagementLeaseWorkflow' => 'DrydockManagementWorkflow',
    'DrydockManagementReclaimWorkflow' => 'DrydockManagementWorkflow',
    'DrydockManagementReleaseLeaseWorkflow' => 'DrydockManagementWorkflow',
    'DrydockManagementReleaseResourceWorkflow' => 'DrydockManagementWorkflow',
    'DrydockManagementUpdateLeaseWorkflow' => 'DrydockManagementWorkflow',
    'DrydockManagementUpdateResourceWorkflow' => 'DrydockManagementWorkflow',
    'DrydockManagementWorkflow' => 'PhabricatorManagementWorkflow',
    'DrydockObjectAuthorizationView' => 'AphrontView',
    'DrydockQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'DrydockRepositoryOperation' => array(
      'DrydockDAO',
      'PhabricatorPolicyInterface',
    ),
    'DrydockRepositoryOperationDismissController' => 'DrydockController',
    'DrydockRepositoryOperationListController' => 'DrydockController',
    'DrydockRepositoryOperationPHIDType' => 'PhabricatorPHIDType',
    'DrydockRepositoryOperationQuery' => 'DrydockQuery',
    'DrydockRepositoryOperationSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'DrydockRepositoryOperationStatusController' => 'DrydockController',
    'DrydockRepositoryOperationStatusView' => 'AphrontView',
    'DrydockRepositoryOperationType' => 'Phobject',
    'DrydockRepositoryOperationUpdateWorker' => 'DrydockWorker',
    'DrydockRepositoryOperationViewController' => 'DrydockController',
    'DrydockResource' => array(
      'DrydockDAO',
      'PhabricatorPolicyInterface',
    ),
    'DrydockResourceActivationFailureLogType' => 'DrydockLogType',
    'DrydockResourceActivationYieldLogType' => 'DrydockLogType',
    'DrydockResourceController' => 'DrydockController',
    'DrydockResourceDatasource' => 'PhabricatorTypeaheadDatasource',
    'DrydockResourceListController' => 'DrydockResourceController',
    'DrydockResourceListView' => 'AphrontView',
    'DrydockResourcePHIDType' => 'PhabricatorPHIDType',
    'DrydockResourceQuery' => 'DrydockQuery',
    'DrydockResourceReclaimLogType' => 'DrydockLogType',
    'DrydockResourceReleaseController' => 'DrydockResourceController',
    'DrydockResourceSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'DrydockResourceStatus' => 'DrydockConstants',
    'DrydockResourceUpdateWorker' => 'DrydockWorker',
    'DrydockResourceViewController' => 'DrydockResourceController',
    'DrydockSFTPFilesystemInterface' => 'DrydockFilesystemInterface',
    'DrydockSSHCommandInterface' => 'DrydockCommandInterface',
    'DrydockSlotLock' => 'DrydockDAO',
    'DrydockSlotLockException' => 'Exception',
    'DrydockSlotLockFailureLogType' => 'DrydockLogType',
    'DrydockTestRepositoryOperation' => 'DrydockRepositoryOperationType',
    'DrydockWebrootInterface' => 'DrydockInterface',
    'DrydockWorker' => 'PhabricatorWorker',
    'DrydockWorkingCopyBlueprintImplementation' => 'DrydockBlueprintImplementation',
    'FeedConduitAPIMethod' => 'ConduitAPIMethod',
    'FeedPublishConduitAPIMethod' => 'FeedConduitAPIMethod',
    'FeedPublisherHTTPWorker' => 'FeedPushWorker',
    'FeedPublisherWorker' => 'FeedPushWorker',
    'FeedPushWorker' => 'PhabricatorWorker',
    'FeedQueryConduitAPIMethod' => 'FeedConduitAPIMethod',
    'FeedStoryNotificationGarbageCollector' => 'PhabricatorGarbageCollector',
    'FileAllocateConduitAPIMethod' => 'FileConduitAPIMethod',
    'FileConduitAPIMethod' => 'ConduitAPIMethod',
    'FileCreateMailReceiver' => 'PhabricatorMailReceiver',
    'FileDownloadConduitAPIMethod' => 'FileConduitAPIMethod',
    'FileInfoConduitAPIMethod' => 'FileConduitAPIMethod',
    'FileMailReceiver' => 'PhabricatorObjectMailReceiver',
    'FileQueryChunksConduitAPIMethod' => 'FileConduitAPIMethod',
    'FileReplyHandler' => 'PhabricatorApplicationTransactionReplyHandler',
    'FileUploadChunkConduitAPIMethod' => 'FileConduitAPIMethod',
    'FileUploadConduitAPIMethod' => 'FileConduitAPIMethod',
    'FileUploadHashConduitAPIMethod' => 'FileConduitAPIMethod',
    'FilesDefaultViewCapability' => 'PhabricatorPolicyCapability',
    'FlagConduitAPIMethod' => 'ConduitAPIMethod',
    'FlagDeleteConduitAPIMethod' => 'FlagConduitAPIMethod',
    'FlagEditConduitAPIMethod' => 'FlagConduitAPIMethod',
    'FlagQueryConduitAPIMethod' => 'FlagConduitAPIMethod',
    'FundBacker' => array(
      'FundDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorApplicationTransactionInterface',
    ),
    'FundBackerCart' => 'PhortuneCartImplementation',
    'FundBackerEditor' => 'PhabricatorApplicationTransactionEditor',
    'FundBackerListController' => 'FundController',
    'FundBackerPHIDType' => 'PhabricatorPHIDType',
    'FundBackerProduct' => 'PhortuneProductImplementation',
    'FundBackerQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'FundBackerSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'FundBackerTransaction' => 'PhabricatorApplicationTransaction',
    'FundBackerTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'FundController' => 'PhabricatorController',
    'FundCreateInitiativesCapability' => 'PhabricatorPolicyCapability',
    'FundDAO' => 'PhabricatorLiskDAO',
    'FundDefaultViewCapability' => 'PhabricatorPolicyCapability',
    'FundInitiative' => array(
      'FundDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorProjectInterface',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorSubscribableInterface',
      'PhabricatorMentionableInterface',
      'PhabricatorFlaggableInterface',
      'PhabricatorTokenReceiverInterface',
      'PhabricatorDestructibleInterface',
      'PhabricatorFulltextInterface',
    ),
    'FundInitiativeBackController' => 'FundController',
    'FundInitiativeCloseController' => 'FundController',
    'FundInitiativeEditController' => 'FundController',
    'FundInitiativeEditor' => 'PhabricatorApplicationTransactionEditor',
    'FundInitiativeFulltextEngine' => 'PhabricatorFulltextEngine',
    'FundInitiativeListController' => 'FundController',
    'FundInitiativePHIDType' => 'PhabricatorPHIDType',
    'FundInitiativeQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'FundInitiativeRemarkupRule' => 'PhabricatorObjectRemarkupRule',
    'FundInitiativeReplyHandler' => 'PhabricatorApplicationTransactionReplyHandler',
    'FundInitiativeSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'FundInitiativeTransaction' => 'PhabricatorApplicationTransaction',
    'FundInitiativeTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'FundInitiativeViewController' => 'FundController',
    'FundSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'HarbormasterArcLintBuildStepImplementation' => 'HarbormasterBuildStepImplementation',
    'HarbormasterArcUnitBuildStepImplementation' => 'HarbormasterBuildStepImplementation',
    'HarbormasterArtifact' => 'Phobject',
    'HarbormasterAutotargetsTestCase' => 'PhabricatorTestCase',
    'HarbormasterBuild' => array(
      'HarbormasterDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorPolicyInterface',
    ),
    'HarbormasterBuildAbortedException' => 'Exception',
    'HarbormasterBuildActionController' => 'HarbormasterController',
    'HarbormasterBuildArcanistAutoplan' => 'HarbormasterBuildAutoplan',
    'HarbormasterBuildArtifact' => array(
      'HarbormasterDAO',
      'PhabricatorPolicyInterface',
    ),
    'HarbormasterBuildArtifactPHIDType' => 'PhabricatorPHIDType',
    'HarbormasterBuildArtifactQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'HarbormasterBuildAutoplan' => 'Phobject',
    'HarbormasterBuildCommand' => 'HarbormasterDAO',
    'HarbormasterBuildDependencyDatasource' => 'PhabricatorTypeaheadDatasource',
    'HarbormasterBuildEngine' => 'Phobject',
    'HarbormasterBuildFailureException' => 'Exception',
    'HarbormasterBuildGraph' => 'AbstractDirectedGraph',
    'HarbormasterBuildLintMessage' => 'HarbormasterDAO',
    'HarbormasterBuildLog' => array(
      'HarbormasterDAO',
      'PhabricatorPolicyInterface',
    ),
    'HarbormasterBuildLogPHIDType' => 'PhabricatorPHIDType',
    'HarbormasterBuildLogQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'HarbormasterBuildMessage' => array(
      'HarbormasterDAO',
      'PhabricatorPolicyInterface',
    ),
    'HarbormasterBuildMessageQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'HarbormasterBuildPHIDType' => 'PhabricatorPHIDType',
    'HarbormasterBuildPlan' => array(
      'HarbormasterDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorPolicyInterface',
      'PhabricatorSubscribableInterface',
    ),
    'HarbormasterBuildPlanDatasource' => 'PhabricatorTypeaheadDatasource',
    'HarbormasterBuildPlanDefaultEditCapability' => 'PhabricatorPolicyCapability',
    'HarbormasterBuildPlanDefaultViewCapability' => 'PhabricatorPolicyCapability',
    'HarbormasterBuildPlanEditor' => 'PhabricatorApplicationTransactionEditor',
    'HarbormasterBuildPlanPHIDType' => 'PhabricatorPHIDType',
    'HarbormasterBuildPlanQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'HarbormasterBuildPlanSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'HarbormasterBuildPlanTransaction' => 'PhabricatorApplicationTransaction',
    'HarbormasterBuildPlanTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'HarbormasterBuildQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'HarbormasterBuildRequest' => 'Phobject',
    'HarbormasterBuildStep' => array(
      'HarbormasterDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorPolicyInterface',
      'PhabricatorCustomFieldInterface',
    ),
    'HarbormasterBuildStepCoreCustomField' => array(
      'HarbormasterBuildStepCustomField',
      'PhabricatorStandardCustomFieldInterface',
    ),
    'HarbormasterBuildStepCustomField' => 'PhabricatorCustomField',
    'HarbormasterBuildStepEditor' => 'PhabricatorApplicationTransactionEditor',
    'HarbormasterBuildStepGroup' => 'Phobject',
    'HarbormasterBuildStepImplementation' => 'Phobject',
    'HarbormasterBuildStepImplementationTestCase' => 'PhabricatorTestCase',
    'HarbormasterBuildStepPHIDType' => 'PhabricatorPHIDType',
    'HarbormasterBuildStepQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'HarbormasterBuildStepTransaction' => 'PhabricatorApplicationTransaction',
    'HarbormasterBuildStepTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'HarbormasterBuildTarget' => array(
      'HarbormasterDAO',
      'PhabricatorPolicyInterface',
    ),
    'HarbormasterBuildTargetPHIDType' => 'PhabricatorPHIDType',
    'HarbormasterBuildTargetQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'HarbormasterBuildTransaction' => 'PhabricatorApplicationTransaction',
    'HarbormasterBuildTransactionEditor' => 'PhabricatorApplicationTransactionEditor',
    'HarbormasterBuildTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'HarbormasterBuildUnitMessage' => 'HarbormasterDAO',
    'HarbormasterBuildViewController' => 'HarbormasterController',
    'HarbormasterBuildWorker' => 'HarbormasterWorker',
    'HarbormasterBuildable' => array(
      'HarbormasterDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorPolicyInterface',
      'HarbormasterBuildableInterface',
    ),
    'HarbormasterBuildableActionController' => 'HarbormasterController',
    'HarbormasterBuildableListController' => 'HarbormasterController',
    'HarbormasterBuildablePHIDType' => 'PhabricatorPHIDType',
    'HarbormasterBuildableQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'HarbormasterBuildableSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'HarbormasterBuildableTransaction' => 'PhabricatorApplicationTransaction',
    'HarbormasterBuildableTransactionEditor' => 'PhabricatorApplicationTransactionEditor',
    'HarbormasterBuildableTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'HarbormasterBuildableViewController' => 'HarbormasterController',
    'HarbormasterBuiltinBuildStepGroup' => 'HarbormasterBuildStepGroup',
    'HarbormasterCommandBuildStepImplementation' => 'HarbormasterBuildStepImplementation',
    'HarbormasterConduitAPIMethod' => 'ConduitAPIMethod',
    'HarbormasterController' => 'PhabricatorController',
    'HarbormasterCreateArtifactConduitAPIMethod' => 'HarbormasterConduitAPIMethod',
    'HarbormasterCreatePlansCapability' => 'PhabricatorPolicyCapability',
    'HarbormasterDAO' => 'PhabricatorLiskDAO',
    'HarbormasterDrydockBuildStepGroup' => 'HarbormasterBuildStepGroup',
    'HarbormasterDrydockCommandBuildStepImplementation' => 'HarbormasterBuildStepImplementation',
    'HarbormasterDrydockLeaseArtifact' => 'HarbormasterArtifact',
    'HarbormasterExecFuture' => 'Future',
    'HarbormasterExternalBuildStepGroup' => 'HarbormasterBuildStepGroup',
    'HarbormasterFileArtifact' => 'HarbormasterArtifact',
    'HarbormasterHTTPRequestBuildStepImplementation' => 'HarbormasterBuildStepImplementation',
    'HarbormasterHostArtifact' => 'HarbormasterDrydockLeaseArtifact',
    'HarbormasterLeaseHostBuildStepImplementation' => 'HarbormasterBuildStepImplementation',
    'HarbormasterLeaseWorkingCopyBuildStepImplementation' => 'HarbormasterBuildStepImplementation',
    'HarbormasterLintMessagesController' => 'HarbormasterController',
    'HarbormasterLintPropertyView' => 'AphrontView',
    'HarbormasterManagementBuildWorkflow' => 'HarbormasterManagementWorkflow',
    'HarbormasterManagementUpdateWorkflow' => 'HarbormasterManagementWorkflow',
    'HarbormasterManagementWorkflow' => 'PhabricatorManagementWorkflow',
    'HarbormasterMessageType' => 'Phobject',
    'HarbormasterObject' => 'HarbormasterDAO',
    'HarbormasterOtherBuildStepGroup' => 'HarbormasterBuildStepGroup',
    'HarbormasterPlanController' => 'HarbormasterController',
    'HarbormasterPlanDisableController' => 'HarbormasterPlanController',
    'HarbormasterPlanEditController' => 'HarbormasterPlanController',
    'HarbormasterPlanListController' => 'HarbormasterPlanController',
    'HarbormasterPlanRunController' => 'HarbormasterController',
    'HarbormasterPlanViewController' => 'HarbormasterPlanController',
    'HarbormasterPrototypeBuildStepGroup' => 'HarbormasterBuildStepGroup',
    'HarbormasterPublishFragmentBuildStepImplementation' => 'HarbormasterBuildStepImplementation',
    'HarbormasterQueryAutotargetsConduitAPIMethod' => 'HarbormasterConduitAPIMethod',
    'HarbormasterQueryBuildablesConduitAPIMethod' => 'HarbormasterConduitAPIMethod',
    'HarbormasterQueryBuildsConduitAPIMethod' => 'HarbormasterConduitAPIMethod',
    'HarbormasterRemarkupRule' => 'PhabricatorObjectRemarkupRule',
    'HarbormasterRunBuildPlansHeraldAction' => 'HeraldAction',
    'HarbormasterSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'HarbormasterScratchTable' => 'HarbormasterDAO',
    'HarbormasterSendMessageConduitAPIMethod' => 'HarbormasterConduitAPIMethod',
    'HarbormasterSleepBuildStepImplementation' => 'HarbormasterBuildStepImplementation',
    'HarbormasterStepAddController' => 'HarbormasterController',
    'HarbormasterStepDeleteController' => 'HarbormasterController',
    'HarbormasterStepEditController' => 'HarbormasterController',
    'HarbormasterStepViewController' => 'HarbormasterController',
    'HarbormasterTargetEngine' => 'Phobject',
    'HarbormasterTargetWorker' => 'HarbormasterWorker',
    'HarbormasterTestBuildStepGroup' => 'HarbormasterBuildStepGroup',
    'HarbormasterThrowExceptionBuildStep' => 'HarbormasterBuildStepImplementation',
    'HarbormasterUIEventListener' => 'PhabricatorEventListener',
    'HarbormasterURIArtifact' => 'HarbormasterArtifact',
    'HarbormasterUnitMessagesController' => 'HarbormasterController',
    'HarbormasterUnitPropertyView' => 'AphrontView',
    'HarbormasterUploadArtifactBuildStepImplementation' => 'HarbormasterBuildStepImplementation',
    'HarbormasterWaitForPreviousBuildStepImplementation' => 'HarbormasterBuildStepImplementation',
    'HarbormasterWorker' => 'PhabricatorWorker',
    'HarbormasterWorkingCopyArtifact' => 'HarbormasterDrydockLeaseArtifact',
    'HeraldAction' => 'Phobject',
    'HeraldActionGroup' => 'HeraldGroup',
    'HeraldActionRecord' => 'HeraldDAO',
    'HeraldAdapter' => 'Phobject',
    'HeraldAlwaysField' => 'HeraldField',
    'HeraldAnotherRuleField' => 'HeraldField',
    'HeraldApplicationActionGroup' => 'HeraldActionGroup',
    'HeraldApplyTranscript' => 'Phobject',
    'HeraldBasicFieldGroup' => 'HeraldFieldGroup',
    'HeraldCommitAdapter' => array(
      'HeraldAdapter',
      'HarbormasterBuildableAdapterInterface',
    ),
    'HeraldCondition' => 'HeraldDAO',
    'HeraldConditionTranscript' => 'Phobject',
    'HeraldContentSourceField' => 'HeraldField',
    'HeraldController' => 'PhabricatorController',
    'HeraldDAO' => 'PhabricatorLiskDAO',
    'HeraldDifferentialAdapter' => 'HeraldAdapter',
    'HeraldDifferentialDiffAdapter' => 'HeraldDifferentialAdapter',
    'HeraldDifferentialRevisionAdapter' => array(
      'HeraldDifferentialAdapter',
      'HarbormasterBuildableAdapterInterface',
    ),
    'HeraldDisableController' => 'HeraldController',
    'HeraldDoNothingAction' => 'HeraldAction',
    'HeraldEditFieldGroup' => 'HeraldFieldGroup',
    'HeraldEffect' => 'Phobject',
    'HeraldEmptyFieldValue' => 'HeraldFieldValue',
    'HeraldEngine' => 'Phobject',
    'HeraldExactProjectsField' => 'HeraldField',
    'HeraldField' => 'Phobject',
    'HeraldFieldGroup' => 'HeraldGroup',
    'HeraldFieldTestCase' => 'PhutilTestCase',
    'HeraldFieldValue' => 'Phobject',
    'HeraldGroup' => 'Phobject',
    'HeraldInvalidActionException' => 'Exception',
    'HeraldInvalidConditionException' => 'Exception',
    'HeraldManageGlobalRulesCapability' => 'PhabricatorPolicyCapability',
    'HeraldManiphestTaskAdapter' => 'HeraldAdapter',
    'HeraldNewController' => 'HeraldController',
    'HeraldNewObjectField' => 'HeraldField',
    'HeraldNotifyActionGroup' => 'HeraldActionGroup',
    'HeraldObjectTranscript' => 'Phobject',
    'HeraldPhameBlogAdapter' => 'HeraldAdapter',
    'HeraldPhamePostAdapter' => 'HeraldAdapter',
    'HeraldPholioMockAdapter' => 'HeraldAdapter',
    'HeraldPonderQuestionAdapter' => 'HeraldAdapter',
    'HeraldPreCommitAdapter' => 'HeraldAdapter',
    'HeraldPreCommitContentAdapter' => 'HeraldPreCommitAdapter',
    'HeraldPreCommitRefAdapter' => 'HeraldPreCommitAdapter',
    'HeraldPreventActionGroup' => 'HeraldActionGroup',
    'HeraldProjectsField' => 'HeraldField',
    'HeraldRecursiveConditionsException' => 'Exception',
    'HeraldRelatedFieldGroup' => 'HeraldFieldGroup',
    'HeraldRemarkupRule' => 'PhabricatorObjectRemarkupRule',
    'HeraldRepetitionPolicyConfig' => 'Phobject',
    'HeraldRule' => array(
      'HeraldDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorFlaggableInterface',
      'PhabricatorPolicyInterface',
      'PhabricatorDestructibleInterface',
      'PhabricatorSubscribableInterface',
    ),
    'HeraldRuleController' => 'HeraldController',
    'HeraldRuleEditor' => 'PhabricatorApplicationTransactionEditor',
    'HeraldRuleListController' => 'HeraldController',
    'HeraldRulePHIDType' => 'PhabricatorPHIDType',
    'HeraldRuleQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'HeraldRuleSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'HeraldRuleTestCase' => 'PhabricatorTestCase',
    'HeraldRuleTransaction' => 'PhabricatorApplicationTransaction',
    'HeraldRuleTransactionComment' => 'PhabricatorApplicationTransactionComment',
    'HeraldRuleTranscript' => 'Phobject',
    'HeraldRuleTypeConfig' => 'Phobject',
    'HeraldRuleViewController' => 'HeraldController',
    'HeraldSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'HeraldSelectFieldValue' => 'HeraldFieldValue',
    'HeraldSpaceField' => 'HeraldField',
    'HeraldSubscribersField' => 'HeraldField',
    'HeraldSupportActionGroup' => 'HeraldActionGroup',
    'HeraldSupportFieldGroup' => 'HeraldFieldGroup',
    'HeraldTestConsoleController' => 'HeraldController',
    'HeraldTextFieldValue' => 'HeraldFieldValue',
    'HeraldTokenizerFieldValue' => 'HeraldFieldValue',
    'HeraldTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'HeraldTranscript' => array(
      'HeraldDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorDestructibleInterface',
    ),
    'HeraldTranscriptController' => 'HeraldController',
    'HeraldTranscriptDestructionEngineExtension' => 'PhabricatorDestructionEngineExtension',
    'HeraldTranscriptGarbageCollector' => 'PhabricatorGarbageCollector',
    'HeraldTranscriptListController' => 'HeraldController',
    'HeraldTranscriptPHIDType' => 'PhabricatorPHIDType',
    'HeraldTranscriptQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'HeraldTranscriptSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'HeraldTranscriptTestCase' => 'PhabricatorTestCase',
    'HeraldUtilityActionGroup' => 'HeraldActionGroup',
    'Javelin' => 'Phobject',
    'JavelinReactorUIExample' => 'PhabricatorUIExample',
    'JavelinUIExample' => 'PhabricatorUIExample',
    'JavelinViewExampleServerView' => 'AphrontView',
    'JavelinViewUIExample' => 'PhabricatorUIExample',
    'LegalpadController' => 'PhabricatorController',
    'LegalpadCreateDocumentsCapability' => 'PhabricatorPolicyCapability',
    'LegalpadDAO' => 'PhabricatorLiskDAO',
    'LegalpadDefaultEditCapability' => 'PhabricatorPolicyCapability',
    'LegalpadDefaultViewCapability' => 'PhabricatorPolicyCapability',
    'LegalpadDocument' => array(
      'LegalpadDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorSubscribableInterface',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorDestructibleInterface',
    ),
    'LegalpadDocumentBody' => array(
      'LegalpadDAO',
      'PhabricatorMarkupInterface',
    ),
    'LegalpadDocumentCommentController' => 'LegalpadController',
    'LegalpadDocumentDatasource' => 'PhabricatorTypeaheadDatasource',
    'LegalpadDocumentDoneController' => 'LegalpadController',
    'LegalpadDocumentEditController' => 'LegalpadController',
    'LegalpadDocumentEditor' => 'PhabricatorApplicationTransactionEditor',
    'LegalpadDocumentListController' => 'LegalpadController',
    'LegalpadDocumentManageController' => 'LegalpadController',
    'LegalpadDocumentQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'LegalpadDocumentRemarkupRule' => 'PhabricatorObjectRemarkupRule',
    'LegalpadDocumentSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'LegalpadDocumentSignController' => 'LegalpadController',
    'LegalpadDocumentSignature' => array(
      'LegalpadDAO',
      'PhabricatorPolicyInterface',
    ),
    'LegalpadDocumentSignatureAddController' => 'LegalpadController',
    'LegalpadDocumentSignatureListController' => 'LegalpadController',
    'LegalpadDocumentSignatureQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'LegalpadDocumentSignatureSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'LegalpadDocumentSignatureVerificationController' => 'LegalpadController',
    'LegalpadDocumentSignatureViewController' => 'LegalpadController',
    'LegalpadMailReceiver' => 'PhabricatorObjectMailReceiver',
    'LegalpadObjectNeedsSignatureEdgeType' => 'PhabricatorEdgeType',
    'LegalpadReplyHandler' => 'PhabricatorApplicationTransactionReplyHandler',
    'LegalpadRequireSignatureHeraldAction' => 'HeraldAction',
    'LegalpadSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'LegalpadSignatureNeededByObjectEdgeType' => 'PhabricatorEdgeType',
    'LegalpadTransaction' => 'PhabricatorApplicationTransaction',
    'LegalpadTransactionComment' => 'PhabricatorApplicationTransactionComment',
    'LegalpadTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'LegalpadTransactionView' => 'PhabricatorApplicationTransactionView',
    'LiskChunkTestCase' => 'PhabricatorTestCase',
    'LiskDAO' => 'Phobject',
    'LiskDAOSet' => 'Phobject',
    'LiskDAOTestCase' => 'PhabricatorTestCase',
    'LiskEphemeralObjectException' => 'Exception',
    'LiskFixtureTestCase' => 'PhabricatorTestCase',
    'LiskIsolationTestCase' => 'PhabricatorTestCase',
    'LiskIsolationTestDAO' => 'LiskDAO',
    'LiskIsolationTestDAOException' => 'Exception',
    'LiskMigrationIterator' => 'PhutilBufferedIterator',
    'LiskRawMigrationIterator' => 'PhutilBufferedIterator',
    'MacroConduitAPIMethod' => 'ConduitAPIMethod',
    'MacroCreateMemeConduitAPIMethod' => 'MacroConduitAPIMethod',
    'MacroQueryConduitAPIMethod' => 'MacroConduitAPIMethod',
    'ManiphestAssignEmailCommand' => 'ManiphestEmailCommand',
    'ManiphestAssigneeDatasource' => 'PhabricatorTypeaheadCompositeDatasource',
    'ManiphestBatchEditController' => 'ManiphestController',
    'ManiphestBulkEditCapability' => 'PhabricatorPolicyCapability',
    'ManiphestClaimEmailCommand' => 'ManiphestEmailCommand',
    'ManiphestCloseEmailCommand' => 'ManiphestEmailCommand',
    'ManiphestConduitAPIMethod' => 'ConduitAPIMethod',
    'ManiphestConfiguredCustomField' => array(
      'ManiphestCustomField',
      'PhabricatorStandardCustomFieldInterface',
    ),
    'ManiphestConstants' => 'Phobject',
    'ManiphestController' => 'PhabricatorController',
    'ManiphestCreateMailReceiver' => 'PhabricatorMailReceiver',
    'ManiphestCreateTaskConduitAPIMethod' => 'ManiphestConduitAPIMethod',
    'ManiphestCustomField' => 'PhabricatorCustomField',
    'ManiphestCustomFieldNumericIndex' => 'PhabricatorCustomFieldNumericIndexStorage',
    'ManiphestCustomFieldStatusParser' => 'PhabricatorCustomFieldMonogramParser',
    'ManiphestCustomFieldStatusParserTestCase' => 'PhabricatorTestCase',
    'ManiphestCustomFieldStorage' => 'PhabricatorCustomFieldStorage',
    'ManiphestCustomFieldStringIndex' => 'PhabricatorCustomFieldStringIndexStorage',
    'ManiphestDAO' => 'PhabricatorLiskDAO',
    'ManiphestDefaultEditCapability' => 'PhabricatorPolicyCapability',
    'ManiphestDefaultViewCapability' => 'PhabricatorPolicyCapability',
    'ManiphestEditAssignCapability' => 'PhabricatorPolicyCapability',
    'ManiphestEditConduitAPIMethod' => 'PhabricatorEditEngineAPIMethod',
    'ManiphestEditEngine' => 'PhabricatorEditEngine',
    'ManiphestEditPoliciesCapability' => 'PhabricatorPolicyCapability',
    'ManiphestEditPriorityCapability' => 'PhabricatorPolicyCapability',
    'ManiphestEditProjectsCapability' => 'PhabricatorPolicyCapability',
    'ManiphestEditStatusCapability' => 'PhabricatorPolicyCapability',
    'ManiphestEmailCommand' => 'MetaMTAEmailTransactionCommand',
    'ManiphestExcelDefaultFormat' => 'ManiphestExcelFormat',
    'ManiphestExcelFormat' => 'Phobject',
    'ManiphestExcelFormatTestCase' => 'PhabricatorTestCase',
    'ManiphestExportController' => 'ManiphestController',
    'ManiphestGetTaskTransactionsConduitAPIMethod' => 'ManiphestConduitAPIMethod',
    'ManiphestHovercardEngineExtension' => 'PhabricatorHovercardEngineExtension',
    'ManiphestInfoConduitAPIMethod' => 'ManiphestConduitAPIMethod',
    'ManiphestNameIndex' => 'ManiphestDAO',
    'ManiphestPriorityConfigOptionType' => 'PhabricatorConfigJSONOptionType',
    'ManiphestPriorityEmailCommand' => 'ManiphestEmailCommand',
    'ManiphestProjectNameFulltextEngineExtension' => 'PhabricatorFulltextEngineExtension',
    'ManiphestQueryConduitAPIMethod' => 'ManiphestConduitAPIMethod',
    'ManiphestQueryStatusesConduitAPIMethod' => 'ManiphestConduitAPIMethod',
    'ManiphestRemarkupRule' => 'PhabricatorObjectRemarkupRule',
    'ManiphestReplyHandler' => 'PhabricatorApplicationTransactionReplyHandler',
    'ManiphestReportController' => 'ManiphestController',
    'ManiphestSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'ManiphestSearchConduitAPIMethod' => 'PhabricatorSearchEngineAPIMethod',
    'ManiphestStatusConfigOptionType' => 'PhabricatorConfigJSONOptionType',
    'ManiphestStatusEmailCommand' => 'ManiphestEmailCommand',
    'ManiphestSubpriorityController' => 'ManiphestController',
    'ManiphestTask' => array(
      'ManiphestDAO',
      'PhabricatorSubscribableInterface',
      'PhabricatorMarkupInterface',
      'PhabricatorPolicyInterface',
      'PhabricatorTokenReceiverInterface',
      'PhabricatorFlaggableInterface',
      'PhabricatorMentionableInterface',
      'PhrequentTrackableInterface',
      'PhabricatorCustomFieldInterface',
      'PhabricatorDestructibleInterface',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorProjectInterface',
      'PhabricatorSpacesInterface',
      'PhabricatorConduitResultInterface',
      'PhabricatorFulltextInterface',
    ),
    'ManiphestTaskAssignHeraldAction' => 'HeraldAction',
    'ManiphestTaskAssignOtherHeraldAction' => 'ManiphestTaskAssignHeraldAction',
    'ManiphestTaskAssignSelfHeraldAction' => 'ManiphestTaskAssignHeraldAction',
    'ManiphestTaskAssigneeHeraldField' => 'ManiphestTaskHeraldField',
    'ManiphestTaskAuthorHeraldField' => 'ManiphestTaskHeraldField',
    'ManiphestTaskAuthorPolicyRule' => 'PhabricatorPolicyRule',
    'ManiphestTaskClosedStatusDatasource' => 'PhabricatorTypeaheadDatasource',
    'ManiphestTaskDependedOnByTaskEdgeType' => 'PhabricatorEdgeType',
    'ManiphestTaskDependsOnTaskEdgeType' => 'PhabricatorEdgeType',
    'ManiphestTaskDescriptionHeraldField' => 'ManiphestTaskHeraldField',
    'ManiphestTaskDetailController' => 'ManiphestController',
    'ManiphestTaskEditBulkJobType' => 'PhabricatorWorkerBulkJobType',
    'ManiphestTaskEditController' => 'ManiphestController',
    'ManiphestTaskFulltextEngine' => 'PhabricatorFulltextEngine',
    'ManiphestTaskHasCommitEdgeType' => 'PhabricatorEdgeType',
    'ManiphestTaskHasMockEdgeType' => 'PhabricatorEdgeType',
    'ManiphestTaskHasRevisionEdgeType' => 'PhabricatorEdgeType',
    'ManiphestTaskHeraldField' => 'HeraldField',
    'ManiphestTaskHeraldFieldGroup' => 'HeraldFieldGroup',
    'ManiphestTaskListController' => 'ManiphestController',
    'ManiphestTaskListHTTPParameterType' => 'AphrontListHTTPParameterType',
    'ManiphestTaskListView' => 'ManiphestView',
    'ManiphestTaskMailReceiver' => 'PhabricatorObjectMailReceiver',
    'ManiphestTaskOpenStatusDatasource' => 'PhabricatorTypeaheadDatasource',
    'ManiphestTaskPHIDResolver' => 'PhabricatorPHIDResolver',
    'ManiphestTaskPHIDType' => 'PhabricatorPHIDType',
    'ManiphestTaskPriority' => 'ManiphestConstants',
    'ManiphestTaskPriorityDatasource' => 'PhabricatorTypeaheadDatasource',
    'ManiphestTaskPriorityHeraldAction' => 'HeraldAction',
    'ManiphestTaskPriorityHeraldField' => 'ManiphestTaskHeraldField',
    'ManiphestTaskQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'ManiphestTaskResultListView' => 'ManiphestView',
    'ManiphestTaskSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'ManiphestTaskStatus' => 'ManiphestConstants',
    'ManiphestTaskStatusDatasource' => 'PhabricatorTypeaheadDatasource',
    'ManiphestTaskStatusFunctionDatasource' => 'PhabricatorTypeaheadCompositeDatasource',
    'ManiphestTaskStatusHeraldAction' => 'HeraldAction',
    'ManiphestTaskStatusHeraldField' => 'ManiphestTaskHeraldField',
    'ManiphestTaskStatusTestCase' => 'PhabricatorTestCase',
    'ManiphestTaskTestCase' => 'PhabricatorTestCase',
    'ManiphestTaskTitleHeraldField' => 'ManiphestTaskHeraldField',
    'ManiphestTransaction' => 'PhabricatorApplicationTransaction',
    'ManiphestTransactionComment' => 'PhabricatorApplicationTransactionComment',
    'ManiphestTransactionEditor' => 'PhabricatorApplicationTransactionEditor',
    'ManiphestTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'ManiphestUpdateConduitAPIMethod' => 'ManiphestConduitAPIMethod',
    'ManiphestView' => 'AphrontView',
    'MetaMTAEmailTransactionCommand' => 'Phobject',
    'MetaMTAEmailTransactionCommandTestCase' => 'PhabricatorTestCase',
    'MetaMTAMailReceivedGarbageCollector' => 'PhabricatorGarbageCollector',
    'MetaMTAMailSentGarbageCollector' => 'PhabricatorGarbageCollector',
    'MetaMTAReceivedMailStatus' => 'Phobject',
    'MultimeterContext' => 'MultimeterDimension',
    'MultimeterControl' => 'Phobject',
    'MultimeterController' => 'PhabricatorController',
    'MultimeterDAO' => 'PhabricatorLiskDAO',
    'MultimeterDimension' => 'MultimeterDAO',
    'MultimeterEvent' => 'MultimeterDAO',
    'MultimeterEventGarbageCollector' => 'PhabricatorGarbageCollector',
    'MultimeterHost' => 'MultimeterDimension',
    'MultimeterLabel' => 'MultimeterDimension',
    'MultimeterSampleController' => 'MultimeterController',
    'MultimeterViewer' => 'MultimeterDimension',
    'NuanceConduitAPIMethod' => 'ConduitAPIMethod',
    'NuanceConsoleController' => 'NuanceController',
    'NuanceController' => 'PhabricatorController',
    'NuanceCreateItemConduitAPIMethod' => 'NuanceConduitAPIMethod',
    'NuanceDAO' => 'PhabricatorLiskDAO',
    'NuanceItem' => array(
      'NuanceDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorApplicationTransactionInterface',
    ),
    'NuanceItemEditController' => 'NuanceController',
    'NuanceItemEditor' => 'PhabricatorApplicationTransactionEditor',
    'NuanceItemPHIDType' => 'PhabricatorPHIDType',
    'NuanceItemQuery' => 'NuanceQuery',
    'NuanceItemTransaction' => 'NuanceTransaction',
    'NuanceItemTransactionComment' => 'PhabricatorApplicationTransactionComment',
    'NuanceItemTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'NuanceItemViewController' => 'NuanceController',
    'NuancePhabricatorFormSourceDefinition' => 'NuanceSourceDefinition',
    'NuanceQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'NuanceQueue' => array(
      'NuanceDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorApplicationTransactionInterface',
    ),
    'NuanceQueueDatasource' => 'PhabricatorTypeaheadDatasource',
    'NuanceQueueEditController' => 'NuanceController',
    'NuanceQueueEditor' => 'PhabricatorApplicationTransactionEditor',
    'NuanceQueueListController' => 'NuanceController',
    'NuanceQueuePHIDType' => 'PhabricatorPHIDType',
    'NuanceQueueQuery' => 'NuanceQuery',
    'NuanceQueueSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'NuanceQueueTransaction' => 'NuanceTransaction',
    'NuanceQueueTransactionComment' => 'PhabricatorApplicationTransactionComment',
    'NuanceQueueTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'NuanceQueueViewController' => 'NuanceController',
    'NuanceRequestor' => array(
      'NuanceDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorApplicationTransactionInterface',
    ),
    'NuanceRequestorEditController' => 'NuanceController',
    'NuanceRequestorEditor' => 'PhabricatorApplicationTransactionEditor',
    'NuanceRequestorPHIDType' => 'PhabricatorPHIDType',
    'NuanceRequestorQuery' => 'NuanceQuery',
    'NuanceRequestorSource' => 'NuanceDAO',
    'NuanceRequestorTransaction' => 'NuanceTransaction',
    'NuanceRequestorTransactionComment' => 'PhabricatorApplicationTransactionComment',
    'NuanceRequestorTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'NuanceRequestorViewController' => 'NuanceController',
    'NuanceSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'NuanceSource' => array(
      'NuanceDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorPolicyInterface',
    ),
    'NuanceSourceActionController' => 'NuanceController',
    'NuanceSourceCreateController' => 'NuanceController',
    'NuanceSourceDefaultEditCapability' => 'PhabricatorPolicyCapability',
    'NuanceSourceDefaultViewCapability' => 'PhabricatorPolicyCapability',
    'NuanceSourceDefinition' => 'Phobject',
    'NuanceSourceDefinitionTestCase' => 'PhabricatorTestCase',
    'NuanceSourceEditController' => 'NuanceController',
    'NuanceSourceEditor' => 'PhabricatorApplicationTransactionEditor',
    'NuanceSourceListController' => 'NuanceController',
    'NuanceSourceManageCapability' => 'PhabricatorPolicyCapability',
    'NuanceSourcePHIDType' => 'PhabricatorPHIDType',
    'NuanceSourceQuery' => 'NuanceQuery',
    'NuanceSourceSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'NuanceSourceTransaction' => 'NuanceTransaction',
    'NuanceSourceTransactionComment' => 'PhabricatorApplicationTransactionComment',
    'NuanceSourceTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'NuanceSourceViewController' => 'NuanceController',
    'NuanceTransaction' => 'PhabricatorApplicationTransaction',
    'OwnersConduitAPIMethod' => 'ConduitAPIMethod',
    'OwnersEditConduitAPIMethod' => 'PhabricatorEditEngineAPIMethod',
    'OwnersPackageReplyHandler' => 'PhabricatorMailReplyHandler',
    'OwnersQueryConduitAPIMethod' => 'OwnersConduitAPIMethod',
    'OwnersSearchConduitAPIMethod' => 'PhabricatorSearchEngineAPIMethod',
    'PHIDConduitAPIMethod' => 'ConduitAPIMethod',
    'PHIDInfoConduitAPIMethod' => 'PHIDConduitAPIMethod',
    'PHIDLookupConduitAPIMethod' => 'PHIDConduitAPIMethod',
    'PHIDQueryConduitAPIMethod' => 'PHIDConduitAPIMethod',
    'PHUI' => 'Phobject',
    'PHUIActionPanelExample' => 'PhabricatorUIExample',
    'PHUIActionPanelView' => 'AphrontTagView',
    'PHUIApplicationMenuView' => 'Phobject',
    'PHUIBadgeBoxView' => 'AphrontTagView',
    'PHUIBadgeExample' => 'PhabricatorUIExample',
    'PHUIBadgeMiniView' => 'AphrontTagView',
    'PHUIBadgeView' => 'AphrontTagView',
    'PHUIBigInfoView' => 'AphrontTagView',
    'PHUIBoxExample' => 'PhabricatorUIExample',
    'PHUIBoxView' => 'AphrontTagView',
    'PHUIButtonBarExample' => 'PhabricatorUIExample',
    'PHUIButtonBarView' => 'AphrontTagView',
    'PHUIButtonExample' => 'PhabricatorUIExample',
    'PHUIButtonView' => 'AphrontTagView',
    'PHUICalendarDayView' => 'AphrontView',
    'PHUICalendarListView' => 'AphrontTagView',
    'PHUICalendarMonthView' => 'AphrontView',
    'PHUICalendarWidgetView' => 'AphrontTagView',
    'PHUIColorPalletteExample' => 'PhabricatorUIExample',
    'PHUICrumbView' => 'AphrontView',
    'PHUICrumbsView' => 'AphrontView',
    'PHUIDiffInlineCommentDetailView' => 'PHUIDiffInlineCommentView',
    'PHUIDiffInlineCommentEditView' => 'PHUIDiffInlineCommentView',
    'PHUIDiffInlineCommentRowScaffold' => 'AphrontView',
    'PHUIDiffInlineCommentTableScaffold' => 'AphrontView',
    'PHUIDiffInlineCommentUndoView' => 'PHUIDiffInlineCommentView',
    'PHUIDiffInlineCommentView' => 'AphrontView',
    'PHUIDiffOneUpInlineCommentRowScaffold' => 'PHUIDiffInlineCommentRowScaffold',
    'PHUIDiffRevealIconView' => 'AphrontView',
    'PHUIDiffTableOfContentsItemView' => 'AphrontView',
    'PHUIDiffTableOfContentsListView' => 'AphrontView',
    'PHUIDiffTwoUpInlineCommentRowScaffold' => 'PHUIDiffInlineCommentRowScaffold',
    'PHUIDocumentExample' => 'PhabricatorUIExample',
    'PHUIDocumentSummaryView' => 'AphrontTagView',
    'PHUIDocumentView' => 'AphrontTagView',
    'PHUIDocumentViewPro' => 'AphrontTagView',
    'PHUIFeedStoryExample' => 'PhabricatorUIExample',
    'PHUIFeedStoryView' => 'AphrontView',
    'PHUIFormDividerControl' => 'AphrontFormControl',
    'PHUIFormFreeformDateControl' => 'AphrontFormControl',
    'PHUIFormIconSetControl' => 'AphrontFormControl',
    'PHUIFormInsetView' => 'AphrontView',
    'PHUIFormLayoutView' => 'AphrontView',
    'PHUIFormMultiSubmitControl' => 'AphrontFormControl',
    'PHUIFormPageView' => 'AphrontView',
    'PHUIHandleListView' => 'AphrontTagView',
    'PHUIHandleTagListView' => 'AphrontTagView',
    'PHUIHandleView' => 'AphrontView',
    'PHUIHeaderView' => 'AphrontTagView',
    'PHUIIconCircleView' => 'AphrontTagView',
    'PHUIIconExample' => 'PhabricatorUIExample',
    'PHUIIconView' => 'AphrontTagView',
    'PHUIImageMaskExample' => 'PhabricatorUIExample',
    'PHUIImageMaskView' => 'AphrontTagView',
    'PHUIInfoExample' => 'PhabricatorUIExample',
    'PHUIInfoPanelExample' => 'PhabricatorUIExample',
    'PHUIInfoPanelView' => 'AphrontView',
    'PHUIInfoView' => 'AphrontView',
    'PHUIListExample' => 'PhabricatorUIExample',
    'PHUIListItemView' => 'AphrontTagView',
    'PHUIListView' => 'AphrontTagView',
    'PHUIListViewTestCase' => 'PhabricatorTestCase',
    'PHUIMainMenuView' => 'AphrontView',
    'PHUIObjectBoxView' => 'AphrontView',
    'PHUIObjectItemListExample' => 'PhabricatorUIExample',
    'PHUIObjectItemListView' => 'AphrontTagView',
    'PHUIObjectItemView' => 'AphrontTagView',
    'PHUIPagedFormView' => 'AphrontView',
    'PHUIPagerView' => 'AphrontView',
    'PHUIPinboardItemView' => 'AphrontView',
    'PHUIPinboardView' => 'AphrontView',
    'PHUIPropertyGroupView' => 'AphrontTagView',
    'PHUIPropertyListExample' => 'PhabricatorUIExample',
    'PHUIPropertyListView' => 'AphrontView',
    'PHUIRemarkupPreviewPanel' => 'AphrontTagView',
    'PHUIRemarkupView' => 'AphrontView',
    'PHUISpacesNamespaceContextView' => 'AphrontView',
    'PHUIStatusItemView' => 'AphrontTagView',
    'PHUIStatusListView' => 'AphrontTagView',
    'PHUITagExample' => 'PhabricatorUIExample',
    'PHUITagView' => 'AphrontTagView',
    'PHUITextExample' => 'PhabricatorUIExample',
    'PHUITextView' => 'AphrontTagView',
    'PHUITimelineEventView' => 'AphrontView',
    'PHUITimelineExample' => 'PhabricatorUIExample',
    'PHUITimelineView' => 'AphrontView',
    'PHUITwoColumnView' => 'AphrontTagView',
    'PHUITypeaheadExample' => 'PhabricatorUIExample',
    'PHUIWorkboardView' => 'AphrontTagView',
    'PHUIWorkpanelView' => 'AphrontTagView',
    'PassphraseAbstractKey' => 'Phobject',
    'PassphraseConduitAPIMethod' => 'ConduitAPIMethod',
    'PassphraseController' => 'PhabricatorController',
    'PassphraseCredential' => array(
      'PassphraseDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorPolicyInterface',
      'PhabricatorFlaggableInterface',
      'PhabricatorSubscribableInterface',
      'PhabricatorDestructibleInterface',
      'PhabricatorSpacesInterface',
      'PhabricatorFulltextInterface',
    ),
    'PassphraseCredentialAuthorPolicyRule' => 'PhabricatorPolicyRule',
    'PassphraseCredentialConduitController' => 'PassphraseController',
    'PassphraseCredentialControl' => 'AphrontFormControl',
    'PassphraseCredentialCreateController' => 'PassphraseController',
    'PassphraseCredentialDestroyController' => 'PassphraseController',
    'PassphraseCredentialEditController' => 'PassphraseController',
    'PassphraseCredentialFulltextEngine' => 'PhabricatorFulltextEngine',
    'PassphraseCredentialListController' => 'PassphraseController',
    'PassphraseCredentialLockController' => 'PassphraseController',
    'PassphraseCredentialPHIDType' => 'PhabricatorPHIDType',
    'PassphraseCredentialPublicController' => 'PassphraseController',
    'PassphraseCredentialQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PassphraseCredentialRevealController' => 'PassphraseController',
    'PassphraseCredentialSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PassphraseCredentialTransaction' => 'PhabricatorApplicationTransaction',
    'PassphraseCredentialTransactionEditor' => 'PhabricatorApplicationTransactionEditor',
    'PassphraseCredentialTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PassphraseCredentialType' => 'Phobject',
    'PassphraseCredentialTypeTestCase' => 'PhabricatorTestCase',
    'PassphraseCredentialViewController' => 'PassphraseController',
    'PassphraseDAO' => 'PhabricatorLiskDAO',
    'PassphraseDefaultEditCapability' => 'PhabricatorPolicyCapability',
    'PassphraseDefaultViewCapability' => 'PhabricatorPolicyCapability',
    'PassphraseNoteCredentialType' => 'PassphraseCredentialType',
    'PassphrasePasswordCredentialType' => 'PassphraseCredentialType',
    'PassphrasePasswordKey' => 'PassphraseAbstractKey',
    'PassphraseQueryConduitAPIMethod' => 'PassphraseConduitAPIMethod',
    'PassphraseRemarkupRule' => 'PhabricatorObjectRemarkupRule',
    'PassphraseSSHGeneratedKeyCredentialType' => 'PassphraseSSHPrivateKeyCredentialType',
    'PassphraseSSHKey' => 'PassphraseAbstractKey',
    'PassphraseSSHPrivateKeyCredentialType' => 'PassphraseCredentialType',
    'PassphraseSSHPrivateKeyFileCredentialType' => 'PassphraseSSHPrivateKeyCredentialType',
    'PassphraseSSHPrivateKeyTextCredentialType' => 'PassphraseSSHPrivateKeyCredentialType',
    'PassphraseSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'PassphraseSecret' => 'PassphraseDAO',
    'PasteConduitAPIMethod' => 'ConduitAPIMethod',
    'PasteCreateConduitAPIMethod' => 'PasteConduitAPIMethod',
    'PasteCreateMailReceiver' => 'PhabricatorMailReceiver',
    'PasteDefaultEditCapability' => 'PhabricatorPolicyCapability',
    'PasteDefaultViewCapability' => 'PhabricatorPolicyCapability',
    'PasteEditConduitAPIMethod' => 'PhabricatorEditEngineAPIMethod',
    'PasteEmbedView' => 'AphrontView',
    'PasteInfoConduitAPIMethod' => 'PasteConduitAPIMethod',
    'PasteMailReceiver' => 'PhabricatorObjectMailReceiver',
    'PasteQueryConduitAPIMethod' => 'PasteConduitAPIMethod',
    'PasteReplyHandler' => 'PhabricatorApplicationTransactionReplyHandler',
    'PasteSearchConduitAPIMethod' => 'PhabricatorSearchEngineAPIMethod',
    'PeopleBrowseUserDirectoryCapability' => 'PhabricatorPolicyCapability',
    'PeopleCreateUsersCapability' => 'PhabricatorPolicyCapability',
    'PeopleUserLogGarbageCollector' => 'PhabricatorGarbageCollector',
    'Phabricator404Controller' => 'PhabricatorController',
    'PhabricatorAWSConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorAccessControlTestCase' => 'PhabricatorTestCase',
    'PhabricatorAccessLog' => 'Phobject',
    'PhabricatorAccessLogConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorAccountSettingsPanel' => 'PhabricatorSettingsPanel',
    'PhabricatorActionListView' => 'AphrontView',
    'PhabricatorActionView' => 'AphrontView',
    'PhabricatorActivitySettingsPanel' => 'PhabricatorSettingsPanel',
    'PhabricatorAdministratorsPolicyRule' => 'PhabricatorPolicyRule',
    'PhabricatorAjaxRequestExceptionHandler' => 'PhabricatorRequestExceptionHandler',
    'PhabricatorAlmanacApplication' => 'PhabricatorApplication',
    'PhabricatorAmazonAuthProvider' => 'PhabricatorOAuth2AuthProvider',
    'PhabricatorAnchorView' => 'AphrontView',
    'PhabricatorAphlictManagementDebugWorkflow' => 'PhabricatorAphlictManagementWorkflow',
    'PhabricatorAphlictManagementRestartWorkflow' => 'PhabricatorAphlictManagementWorkflow',
    'PhabricatorAphlictManagementStartWorkflow' => 'PhabricatorAphlictManagementWorkflow',
    'PhabricatorAphlictManagementStatusWorkflow' => 'PhabricatorAphlictManagementWorkflow',
    'PhabricatorAphlictManagementStopWorkflow' => 'PhabricatorAphlictManagementWorkflow',
    'PhabricatorAphlictManagementWorkflow' => 'PhabricatorManagementWorkflow',
    'PhabricatorAphlictSetupCheck' => 'PhabricatorSetupCheck',
    'PhabricatorAphrontBarUIExample' => 'PhabricatorUIExample',
    'PhabricatorAphrontViewTestCase' => 'PhabricatorTestCase',
    'PhabricatorAppSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorApplication' => array(
      'Phobject',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorApplicationApplicationPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorApplicationConfigOptions' => 'Phobject',
    'PhabricatorApplicationConfigurationPanel' => 'Phobject',
    'PhabricatorApplicationConfigurationPanelTestCase' => 'PhabricatorTestCase',
    'PhabricatorApplicationDatasource' => 'PhabricatorTypeaheadDatasource',
    'PhabricatorApplicationDetailViewController' => 'PhabricatorApplicationsController',
    'PhabricatorApplicationEditController' => 'PhabricatorApplicationsController',
    'PhabricatorApplicationEditHTTPParameterHelpView' => 'AphrontView',
    'PhabricatorApplicationEmailCommandsController' => 'PhabricatorApplicationsController',
    'PhabricatorApplicationLaunchView' => 'AphrontTagView',
    'PhabricatorApplicationPanelController' => 'PhabricatorApplicationsController',
    'PhabricatorApplicationQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorApplicationSearchController' => 'PhabricatorSearchBaseController',
    'PhabricatorApplicationSearchEngine' => 'Phobject',
    'PhabricatorApplicationSearchEngineTestCase' => 'PhabricatorTestCase',
    'PhabricatorApplicationSearchResultView' => 'Phobject',
    'PhabricatorApplicationStatusView' => 'AphrontView',
    'PhabricatorApplicationTestCase' => 'PhabricatorTestCase',
    'PhabricatorApplicationTransaction' => array(
      'PhabricatorLiskDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorDestructibleInterface',
    ),
    'PhabricatorApplicationTransactionComment' => array(
      'PhabricatorLiskDAO',
      'PhabricatorMarkupInterface',
      'PhabricatorPolicyInterface',
      'PhabricatorDestructibleInterface',
    ),
    'PhabricatorApplicationTransactionCommentEditController' => 'PhabricatorApplicationTransactionController',
    'PhabricatorApplicationTransactionCommentEditor' => 'PhabricatorEditor',
    'PhabricatorApplicationTransactionCommentHistoryController' => 'PhabricatorApplicationTransactionController',
    'PhabricatorApplicationTransactionCommentQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorApplicationTransactionCommentQuoteController' => 'PhabricatorApplicationTransactionController',
    'PhabricatorApplicationTransactionCommentRawController' => 'PhabricatorApplicationTransactionController',
    'PhabricatorApplicationTransactionCommentRemoveController' => 'PhabricatorApplicationTransactionController',
    'PhabricatorApplicationTransactionCommentView' => 'AphrontView',
    'PhabricatorApplicationTransactionController' => 'PhabricatorController',
    'PhabricatorApplicationTransactionDetailController' => 'PhabricatorApplicationTransactionController',
    'PhabricatorApplicationTransactionEditor' => 'PhabricatorEditor',
    'PhabricatorApplicationTransactionFeedStory' => 'PhabricatorFeedStory',
    'PhabricatorApplicationTransactionNoEffectException' => 'Exception',
    'PhabricatorApplicationTransactionNoEffectResponse' => 'AphrontProxyResponse',
    'PhabricatorApplicationTransactionPublishWorker' => 'PhabricatorWorker',
    'PhabricatorApplicationTransactionQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorApplicationTransactionRemarkupPreviewController' => 'PhabricatorApplicationTransactionController',
    'PhabricatorApplicationTransactionReplyHandler' => 'PhabricatorMailReplyHandler',
    'PhabricatorApplicationTransactionResponse' => 'AphrontProxyResponse',
    'PhabricatorApplicationTransactionShowOlderController' => 'PhabricatorApplicationTransactionController',
    'PhabricatorApplicationTransactionStructureException' => 'Exception',
    'PhabricatorApplicationTransactionTemplatedCommentQuery' => 'PhabricatorApplicationTransactionCommentQuery',
    'PhabricatorApplicationTransactionTextDiffDetailView' => 'AphrontView',
    'PhabricatorApplicationTransactionTransactionPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorApplicationTransactionValidationError' => 'Phobject',
    'PhabricatorApplicationTransactionValidationException' => 'Exception',
    'PhabricatorApplicationTransactionValidationResponse' => 'AphrontProxyResponse',
    'PhabricatorApplicationTransactionValueController' => 'PhabricatorApplicationTransactionController',
    'PhabricatorApplicationTransactionView' => 'AphrontView',
    'PhabricatorApplicationUninstallController' => 'PhabricatorApplicationsController',
    'PhabricatorApplicationsApplication' => 'PhabricatorApplication',
    'PhabricatorApplicationsController' => 'PhabricatorController',
    'PhabricatorApplicationsListController' => 'PhabricatorApplicationsController',
    'PhabricatorAsanaAuthProvider' => 'PhabricatorOAuth2AuthProvider',
    'PhabricatorAsanaConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorAsanaSubtaskHasObjectEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorAsanaTaskHasObjectEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorAuditActionConstants' => 'Phobject',
    'PhabricatorAuditAddCommentController' => 'PhabricatorAuditController',
    'PhabricatorAuditApplication' => 'PhabricatorApplication',
    'PhabricatorAuditCommentEditor' => 'PhabricatorEditor',
    'PhabricatorAuditCommitStatusConstants' => 'Phobject',
    'PhabricatorAuditController' => 'PhabricatorController',
    'PhabricatorAuditEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhabricatorAuditInlineComment' => array(
      'Phobject',
      'PhabricatorInlineCommentInterface',
    ),
    'PhabricatorAuditListController' => 'PhabricatorAuditController',
    'PhabricatorAuditListView' => 'AphrontView',
    'PhabricatorAuditMailReceiver' => 'PhabricatorObjectMailReceiver',
    'PhabricatorAuditManagementDeleteWorkflow' => 'PhabricatorAuditManagementWorkflow',
    'PhabricatorAuditManagementWorkflow' => 'PhabricatorManagementWorkflow',
    'PhabricatorAuditPreviewController' => 'PhabricatorAuditController',
    'PhabricatorAuditReplyHandler' => 'PhabricatorApplicationTransactionReplyHandler',
    'PhabricatorAuditStatusConstants' => 'Phobject',
    'PhabricatorAuditTransaction' => 'PhabricatorApplicationTransaction',
    'PhabricatorAuditTransactionComment' => 'PhabricatorApplicationTransactionComment',
    'PhabricatorAuditTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhabricatorAuditTransactionView' => 'PhabricatorApplicationTransactionView',
    'PhabricatorAuthAccountView' => 'AphrontView',
    'PhabricatorAuthApplication' => 'PhabricatorApplication',
    'PhabricatorAuthAuthFactorPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorAuthAuthProviderPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorAuthConduitAPIMethod' => 'ConduitAPIMethod',
    'PhabricatorAuthConfirmLinkController' => 'PhabricatorAuthController',
    'PhabricatorAuthController' => 'PhabricatorController',
    'PhabricatorAuthDAO' => 'PhabricatorLiskDAO',
    'PhabricatorAuthDisableController' => 'PhabricatorAuthProviderConfigController',
    'PhabricatorAuthDowngradeSessionController' => 'PhabricatorAuthController',
    'PhabricatorAuthEditController' => 'PhabricatorAuthProviderConfigController',
    'PhabricatorAuthFactor' => 'Phobject',
    'PhabricatorAuthFactorConfig' => 'PhabricatorAuthDAO',
    'PhabricatorAuthFactorTestCase' => 'PhabricatorTestCase',
    'PhabricatorAuthFinishController' => 'PhabricatorAuthController',
    'PhabricatorAuthHighSecurityRequiredException' => 'Exception',
    'PhabricatorAuthHighSecurityToken' => 'Phobject',
    'PhabricatorAuthInvite' => array(
      'PhabricatorUserDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorAuthInviteAccountException' => 'PhabricatorAuthInviteDialogException',
    'PhabricatorAuthInviteAction' => 'Phobject',
    'PhabricatorAuthInviteActionTableView' => 'AphrontView',
    'PhabricatorAuthInviteController' => 'PhabricatorAuthController',
    'PhabricatorAuthInviteDialogException' => 'PhabricatorAuthInviteException',
    'PhabricatorAuthInviteEngine' => 'Phobject',
    'PhabricatorAuthInviteException' => 'Exception',
    'PhabricatorAuthInviteInvalidException' => 'PhabricatorAuthInviteDialogException',
    'PhabricatorAuthInviteLoginException' => 'PhabricatorAuthInviteDialogException',
    'PhabricatorAuthInvitePHIDType' => 'PhabricatorPHIDType',
    'PhabricatorAuthInviteQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorAuthInviteRegisteredException' => 'PhabricatorAuthInviteException',
    'PhabricatorAuthInviteSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorAuthInviteTestCase' => 'PhabricatorTestCase',
    'PhabricatorAuthInviteVerifyException' => 'PhabricatorAuthInviteDialogException',
    'PhabricatorAuthInviteWorker' => 'PhabricatorWorker',
    'PhabricatorAuthLinkController' => 'PhabricatorAuthController',
    'PhabricatorAuthListController' => 'PhabricatorAuthProviderConfigController',
    'PhabricatorAuthLoginController' => 'PhabricatorAuthController',
    'PhabricatorAuthLoginHandler' => 'Phobject',
    'PhabricatorAuthMainMenuBarExtension' => 'PhabricatorMainMenuBarExtension',
    'PhabricatorAuthManagementCachePKCS8Workflow' => 'PhabricatorAuthManagementWorkflow',
    'PhabricatorAuthManagementLDAPWorkflow' => 'PhabricatorAuthManagementWorkflow',
    'PhabricatorAuthManagementListFactorsWorkflow' => 'PhabricatorAuthManagementWorkflow',
    'PhabricatorAuthManagementRecoverWorkflow' => 'PhabricatorAuthManagementWorkflow',
    'PhabricatorAuthManagementRefreshWorkflow' => 'PhabricatorAuthManagementWorkflow',
    'PhabricatorAuthManagementStripWorkflow' => 'PhabricatorAuthManagementWorkflow',
    'PhabricatorAuthManagementTrustOAuthClientWorkflow' => 'PhabricatorAuthManagementWorkflow',
    'PhabricatorAuthManagementUnlimitWorkflow' => 'PhabricatorAuthManagementWorkflow',
    'PhabricatorAuthManagementUntrustOAuthClientWorkflow' => 'PhabricatorAuthManagementWorkflow',
    'PhabricatorAuthManagementVerifyWorkflow' => 'PhabricatorAuthManagementWorkflow',
    'PhabricatorAuthManagementWorkflow' => 'PhabricatorManagementWorkflow',
    'PhabricatorAuthNeedsApprovalController' => 'PhabricatorAuthController',
    'PhabricatorAuthNeedsMultiFactorController' => 'PhabricatorAuthController',
    'PhabricatorAuthNewController' => 'PhabricatorAuthProviderConfigController',
    'PhabricatorAuthOldOAuthRedirectController' => 'PhabricatorAuthController',
    'PhabricatorAuthOneTimeLoginController' => 'PhabricatorAuthController',
    'PhabricatorAuthProvider' => 'Phobject',
    'PhabricatorAuthProviderConfig' => array(
      'PhabricatorAuthDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorAuthProviderConfigController' => 'PhabricatorAuthController',
    'PhabricatorAuthProviderConfigEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhabricatorAuthProviderConfigQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorAuthProviderConfigTransaction' => 'PhabricatorApplicationTransaction',
    'PhabricatorAuthProviderConfigTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhabricatorAuthQueryPublicKeysConduitAPIMethod' => 'PhabricatorAuthConduitAPIMethod',
    'PhabricatorAuthRegisterController' => 'PhabricatorAuthController',
    'PhabricatorAuthRevokeTokenController' => 'PhabricatorAuthController',
    'PhabricatorAuthSSHKey' => array(
      'PhabricatorAuthDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorDestructibleInterface',
    ),
    'PhabricatorAuthSSHKeyController' => 'PhabricatorAuthController',
    'PhabricatorAuthSSHKeyDeleteController' => 'PhabricatorAuthSSHKeyController',
    'PhabricatorAuthSSHKeyEditController' => 'PhabricatorAuthSSHKeyController',
    'PhabricatorAuthSSHKeyGenerateController' => 'PhabricatorAuthSSHKeyController',
    'PhabricatorAuthSSHKeyPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorAuthSSHKeyQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorAuthSSHKeyTableView' => 'AphrontView',
    'PhabricatorAuthSSHPublicKey' => 'Phobject',
    'PhabricatorAuthSession' => array(
      'PhabricatorAuthDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorAuthSessionEngine' => 'Phobject',
    'PhabricatorAuthSessionGarbageCollector' => 'PhabricatorGarbageCollector',
    'PhabricatorAuthSessionQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorAuthSetupCheck' => 'PhabricatorSetupCheck',
    'PhabricatorAuthStartController' => 'PhabricatorAuthController',
    'PhabricatorAuthTemporaryToken' => array(
      'PhabricatorAuthDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorAuthTemporaryTokenGarbageCollector' => 'PhabricatorGarbageCollector',
    'PhabricatorAuthTemporaryTokenQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorAuthTerminateSessionController' => 'PhabricatorAuthController',
    'PhabricatorAuthTryFactorAction' => 'PhabricatorSystemAction',
    'PhabricatorAuthUnlinkController' => 'PhabricatorAuthController',
    'PhabricatorAuthValidateController' => 'PhabricatorAuthController',
    'PhabricatorAuthenticationConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorAutoEventListener' => 'PhabricatorEventListener',
    'PhabricatorBadgeHasRecipientEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorBadgesApplication' => 'PhabricatorApplication',
    'PhabricatorBadgesArchiveController' => 'PhabricatorBadgesController',
    'PhabricatorBadgesBadge' => array(
      'PhabricatorBadgesDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorSubscribableInterface',
      'PhabricatorTokenReceiverInterface',
      'PhabricatorFlaggableInterface',
      'PhabricatorDestructibleInterface',
    ),
    'PhabricatorBadgesCommentController' => 'PhabricatorBadgesController',
    'PhabricatorBadgesController' => 'PhabricatorController',
    'PhabricatorBadgesCreateCapability' => 'PhabricatorPolicyCapability',
    'PhabricatorBadgesDAO' => 'PhabricatorLiskDAO',
    'PhabricatorBadgesDefaultEditCapability' => 'PhabricatorPolicyCapability',
    'PhabricatorBadgesEditController' => 'PhabricatorBadgesController',
    'PhabricatorBadgesEditEngine' => 'PhabricatorEditEngine',
    'PhabricatorBadgesEditRecipientsController' => 'PhabricatorBadgesController',
    'PhabricatorBadgesEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhabricatorBadgesIconSet' => 'PhabricatorIconSet',
    'PhabricatorBadgesListController' => 'PhabricatorBadgesController',
    'PhabricatorBadgesMailReceiver' => 'PhabricatorObjectMailReceiver',
    'PhabricatorBadgesPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorBadgesQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorBadgesRecipientsListView' => 'AphrontTagView',
    'PhabricatorBadgesRemoveRecipientsController' => 'PhabricatorBadgesController',
    'PhabricatorBadgesReplyHandler' => 'PhabricatorApplicationTransactionReplyHandler',
    'PhabricatorBadgesSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'PhabricatorBadgesSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorBadgesTransaction' => 'PhabricatorApplicationTransaction',
    'PhabricatorBadgesTransactionComment' => 'PhabricatorApplicationTransactionComment',
    'PhabricatorBadgesTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhabricatorBadgesViewController' => 'PhabricatorBadgesController',
    'PhabricatorBarePageUIExample' => 'PhabricatorUIExample',
    'PhabricatorBarePageView' => 'AphrontPageView',
    'PhabricatorBaseURISetupCheck' => 'PhabricatorSetupCheck',
    'PhabricatorBcryptPasswordHasher' => 'PhabricatorPasswordHasher',
    'PhabricatorBinariesSetupCheck' => 'PhabricatorSetupCheck',
    'PhabricatorBitbucketAuthProvider' => 'PhabricatorOAuth1AuthProvider',
    'PhabricatorBot' => 'PhabricatorDaemon',
    'PhabricatorBotChannel' => 'PhabricatorBotTarget',
    'PhabricatorBotDebugLogHandler' => 'PhabricatorBotHandler',
    'PhabricatorBotFeedNotificationHandler' => 'PhabricatorBotHandler',
    'PhabricatorBotFlowdockProtocolAdapter' => 'PhabricatorStreamingProtocolAdapter',
    'PhabricatorBotHandler' => 'Phobject',
    'PhabricatorBotLogHandler' => 'PhabricatorBotHandler',
    'PhabricatorBotMacroHandler' => 'PhabricatorBotHandler',
    'PhabricatorBotMessage' => 'Phobject',
    'PhabricatorBotObjectNameHandler' => 'PhabricatorBotHandler',
    'PhabricatorBotSymbolHandler' => 'PhabricatorBotHandler',
    'PhabricatorBotTarget' => 'Phobject',
    'PhabricatorBotUser' => 'PhabricatorBotTarget',
    'PhabricatorBotWhatsNewHandler' => 'PhabricatorBotHandler',
    'PhabricatorBritishEnglishTranslation' => 'PhutilTranslation',
    'PhabricatorBuiltinPatchList' => 'PhabricatorSQLPatchList',
    'PhabricatorBusyUIExample' => 'PhabricatorUIExample',
    'PhabricatorCacheDAO' => 'PhabricatorLiskDAO',
    'PhabricatorCacheGeneralGarbageCollector' => 'PhabricatorGarbageCollector',
    'PhabricatorCacheManagementPurgeWorkflow' => 'PhabricatorCacheManagementWorkflow',
    'PhabricatorCacheManagementWorkflow' => 'PhabricatorManagementWorkflow',
    'PhabricatorCacheMarkupGarbageCollector' => 'PhabricatorGarbageCollector',
    'PhabricatorCacheSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'PhabricatorCacheSetupCheck' => 'PhabricatorSetupCheck',
    'PhabricatorCacheSpec' => 'Phobject',
    'PhabricatorCacheTTLGarbageCollector' => 'PhabricatorGarbageCollector',
    'PhabricatorCaches' => 'Phobject',
    'PhabricatorCachesTestCase' => 'PhabricatorTestCase',
    'PhabricatorCalendarApplication' => 'PhabricatorApplication',
    'PhabricatorCalendarController' => 'PhabricatorController',
    'PhabricatorCalendarDAO' => 'PhabricatorLiskDAO',
    'PhabricatorCalendarEvent' => array(
      'PhabricatorCalendarDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorProjectInterface',
      'PhabricatorMarkupInterface',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorSubscribableInterface',
      'PhabricatorTokenReceiverInterface',
      'PhabricatorDestructibleInterface',
      'PhabricatorMentionableInterface',
      'PhabricatorFlaggableInterface',
      'PhabricatorSpacesInterface',
      'PhabricatorFulltextInterface',
    ),
    'PhabricatorCalendarEventCancelController' => 'PhabricatorCalendarController',
    'PhabricatorCalendarEventCommentController' => 'PhabricatorCalendarController',
    'PhabricatorCalendarEventDragController' => 'PhabricatorCalendarController',
    'PhabricatorCalendarEventEditController' => 'PhabricatorCalendarController',
    'PhabricatorCalendarEventEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhabricatorCalendarEventEmailCommand' => 'MetaMTAEmailTransactionCommand',
    'PhabricatorCalendarEventFulltextEngine' => 'PhabricatorFulltextEngine',
    'PhabricatorCalendarEventInvitee' => array(
      'PhabricatorCalendarDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorCalendarEventInviteeQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorCalendarEventJoinController' => 'PhabricatorCalendarController',
    'PhabricatorCalendarEventListController' => 'PhabricatorCalendarController',
    'PhabricatorCalendarEventMailReceiver' => 'PhabricatorObjectMailReceiver',
    'PhabricatorCalendarEventPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorCalendarEventQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorCalendarEventRSVPEmailCommand' => 'PhabricatorCalendarEventEmailCommand',
    'PhabricatorCalendarEventSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorCalendarEventTransaction' => 'PhabricatorApplicationTransaction',
    'PhabricatorCalendarEventTransactionComment' => 'PhabricatorApplicationTransactionComment',
    'PhabricatorCalendarEventTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhabricatorCalendarEventViewController' => 'PhabricatorCalendarController',
    'PhabricatorCalendarHoliday' => 'PhabricatorCalendarDAO',
    'PhabricatorCalendarHolidayTestCase' => 'PhabricatorTestCase',
    'PhabricatorCalendarIconSet' => 'PhabricatorIconSet',
    'PhabricatorCalendarRemarkupRule' => 'PhabricatorObjectRemarkupRule',
    'PhabricatorCalendarReplyHandler' => 'PhabricatorApplicationTransactionReplyHandler',
    'PhabricatorCalendarSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'PhabricatorCampfireProtocolAdapter' => 'PhabricatorStreamingProtocolAdapter',
    'PhabricatorCelerityApplication' => 'PhabricatorApplication',
    'PhabricatorCelerityTestCase' => 'PhabricatorTestCase',
    'PhabricatorChangeParserTestCase' => 'PhabricatorWorkingCopyTestCase',
    'PhabricatorChangesetResponse' => 'AphrontProxyResponse',
    'PhabricatorChatLogApplication' => 'PhabricatorApplication',
    'PhabricatorChatLogChannel' => array(
      'PhabricatorChatLogDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorChatLogChannelListController' => 'PhabricatorChatLogController',
    'PhabricatorChatLogChannelLogController' => 'PhabricatorChatLogController',
    'PhabricatorChatLogChannelQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorChatLogController' => 'PhabricatorController',
    'PhabricatorChatLogDAO' => 'PhabricatorLiskDAO',
    'PhabricatorChatLogEvent' => array(
      'PhabricatorChatLogDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorChatLogQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorChunkedFileStorageEngine' => 'PhabricatorFileStorageEngine',
    'PhabricatorClusterConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorCommentEditEngineExtension' => 'PhabricatorEditEngineExtension',
    'PhabricatorCommentEditField' => 'PhabricatorEditField',
    'PhabricatorCommentEditType' => 'PhabricatorEditType',
    'PhabricatorCommitBranchesField' => 'PhabricatorCommitCustomField',
    'PhabricatorCommitCustomField' => 'PhabricatorCustomField',
    'PhabricatorCommitMergedCommitsField' => 'PhabricatorCommitCustomField',
    'PhabricatorCommitRepositoryField' => 'PhabricatorCommitCustomField',
    'PhabricatorCommitSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorCommitTagsField' => 'PhabricatorCommitCustomField',
    'PhabricatorCommonPasswords' => 'Phobject',
    'PhabricatorConduitAPIController' => 'PhabricatorConduitController',
    'PhabricatorConduitApplication' => 'PhabricatorApplication',
    'PhabricatorConduitCertificateToken' => 'PhabricatorConduitDAO',
    'PhabricatorConduitConnectionLog' => 'PhabricatorConduitDAO',
    'PhabricatorConduitConsoleController' => 'PhabricatorConduitController',
    'PhabricatorConduitController' => 'PhabricatorController',
    'PhabricatorConduitDAO' => 'PhabricatorLiskDAO',
    'PhabricatorConduitEditField' => 'PhabricatorEditField',
    'PhabricatorConduitListController' => 'PhabricatorConduitController',
    'PhabricatorConduitLogController' => 'PhabricatorConduitController',
    'PhabricatorConduitLogQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorConduitLogSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorConduitMethodCallLog' => array(
      'PhabricatorConduitDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorConduitMethodQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorConduitRequestExceptionHandler' => 'PhabricatorRequestExceptionHandler',
    'PhabricatorConduitResultInterface' => 'PhabricatorPHIDInterface',
    'PhabricatorConduitSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorConduitSearchFieldSpecification' => 'Phobject',
    'PhabricatorConduitTestCase' => 'PhabricatorTestCase',
    'PhabricatorConduitToken' => array(
      'PhabricatorConduitDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorConduitTokenController' => 'PhabricatorConduitController',
    'PhabricatorConduitTokenEditController' => 'PhabricatorConduitController',
    'PhabricatorConduitTokenHandshakeController' => 'PhabricatorConduitController',
    'PhabricatorConduitTokenQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorConduitTokenTerminateController' => 'PhabricatorConduitController',
    'PhabricatorConduitTokensSettingsPanel' => 'PhabricatorSettingsPanel',
    'PhabricatorConfigAllController' => 'PhabricatorConfigController',
    'PhabricatorConfigApplication' => 'PhabricatorApplication',
    'PhabricatorConfigCacheController' => 'PhabricatorConfigController',
    'PhabricatorConfigCollectorsModule' => 'PhabricatorConfigModule',
    'PhabricatorConfigColumnSchema' => 'PhabricatorConfigStorageSchema',
    'PhabricatorConfigConfigPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorConfigController' => 'PhabricatorController',
    'PhabricatorConfigCoreSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'PhabricatorConfigDatabaseController' => 'PhabricatorConfigController',
    'PhabricatorConfigDatabaseIssueController' => 'PhabricatorConfigDatabaseController',
    'PhabricatorConfigDatabaseSchema' => 'PhabricatorConfigStorageSchema',
    'PhabricatorConfigDatabaseSource' => 'PhabricatorConfigProxySource',
    'PhabricatorConfigDatabaseStatusController' => 'PhabricatorConfigDatabaseController',
    'PhabricatorConfigDefaultSource' => 'PhabricatorConfigProxySource',
    'PhabricatorConfigDictionarySource' => 'PhabricatorConfigSource',
    'PhabricatorConfigEdgeModule' => 'PhabricatorConfigModule',
    'PhabricatorConfigEditController' => 'PhabricatorConfigController',
    'PhabricatorConfigEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhabricatorConfigEntry' => array(
      'PhabricatorConfigEntryDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorConfigEntryDAO' => 'PhabricatorLiskDAO',
    'PhabricatorConfigEntryQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorConfigFileSource' => 'PhabricatorConfigProxySource',
    'PhabricatorConfigGroupController' => 'PhabricatorConfigController',
    'PhabricatorConfigHTTPParameterTypesModule' => 'PhabricatorConfigModule',
    'PhabricatorConfigHistoryController' => 'PhabricatorConfigController',
    'PhabricatorConfigIgnoreController' => 'PhabricatorConfigController',
    'PhabricatorConfigIssueListController' => 'PhabricatorConfigController',
    'PhabricatorConfigIssueViewController' => 'PhabricatorConfigController',
    'PhabricatorConfigJSON' => 'Phobject',
    'PhabricatorConfigJSONOptionType' => 'PhabricatorConfigOptionType',
    'PhabricatorConfigKeySchema' => 'PhabricatorConfigStorageSchema',
    'PhabricatorConfigListController' => 'PhabricatorConfigController',
    'PhabricatorConfigLocalSource' => 'PhabricatorConfigProxySource',
    'PhabricatorConfigManagementDeleteWorkflow' => 'PhabricatorConfigManagementWorkflow',
    'PhabricatorConfigManagementGetWorkflow' => 'PhabricatorConfigManagementWorkflow',
    'PhabricatorConfigManagementListWorkflow' => 'PhabricatorConfigManagementWorkflow',
    'PhabricatorConfigManagementMigrateWorkflow' => 'PhabricatorConfigManagementWorkflow',
    'PhabricatorConfigManagementSetWorkflow' => 'PhabricatorConfigManagementWorkflow',
    'PhabricatorConfigManagementWorkflow' => 'PhabricatorManagementWorkflow',
    'PhabricatorConfigModule' => 'Phobject',
    'PhabricatorConfigModuleController' => 'PhabricatorConfigController',
    'PhabricatorConfigOption' => array(
      'Phobject',
      'PhabricatorMarkupInterface',
    ),
    'PhabricatorConfigOptionType' => 'Phobject',
    'PhabricatorConfigPHIDModule' => 'PhabricatorConfigModule',
    'PhabricatorConfigProxySource' => 'PhabricatorConfigSource',
    'PhabricatorConfigPurgeCacheController' => 'PhabricatorConfigController',
    'PhabricatorConfigRequestExceptionHandlerModule' => 'PhabricatorConfigModule',
    'PhabricatorConfigResponse' => 'AphrontStandaloneHTMLResponse',
    'PhabricatorConfigSchemaQuery' => 'Phobject',
    'PhabricatorConfigSchemaSpec' => 'Phobject',
    'PhabricatorConfigServerSchema' => 'PhabricatorConfigStorageSchema',
    'PhabricatorConfigSiteModule' => 'PhabricatorConfigModule',
    'PhabricatorConfigSiteSource' => 'PhabricatorConfigProxySource',
    'PhabricatorConfigSource' => 'Phobject',
    'PhabricatorConfigStackSource' => 'PhabricatorConfigSource',
    'PhabricatorConfigStorageSchema' => 'Phobject',
    'PhabricatorConfigTableSchema' => 'PhabricatorConfigStorageSchema',
    'PhabricatorConfigTransaction' => 'PhabricatorApplicationTransaction',
    'PhabricatorConfigTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhabricatorConfigValidationException' => 'Exception',
    'PhabricatorConfigVersionsModule' => 'PhabricatorConfigModule',
    'PhabricatorConfigWelcomeController' => 'PhabricatorConfigController',
    'PhabricatorConpherenceApplication' => 'PhabricatorApplication',
    'PhabricatorConpherencePreferencesSettingsPanel' => 'PhabricatorSettingsPanel',
    'PhabricatorConpherenceThreadPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorConsoleApplication' => 'PhabricatorApplication',
    'PhabricatorContentSource' => 'Phobject',
    'PhabricatorContentSourceView' => 'AphrontView',
    'PhabricatorContributedToObjectEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorController' => 'AphrontController',
    'PhabricatorCookies' => 'Phobject',
    'PhabricatorCoreConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorCountdown' => array(
      'PhabricatorCountdownDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorFlaggableInterface',
      'PhabricatorSubscribableInterface',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorTokenReceiverInterface',
      'PhabricatorSpacesInterface',
      'PhabricatorProjectInterface',
    ),
    'PhabricatorCountdownApplication' => 'PhabricatorApplication',
    'PhabricatorCountdownCommentController' => 'PhabricatorCountdownController',
    'PhabricatorCountdownController' => 'PhabricatorController',
    'PhabricatorCountdownCountdownPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorCountdownDAO' => 'PhabricatorLiskDAO',
    'PhabricatorCountdownDefaultEditCapability' => 'PhabricatorPolicyCapability',
    'PhabricatorCountdownDefaultViewCapability' => 'PhabricatorPolicyCapability',
    'PhabricatorCountdownDeleteController' => 'PhabricatorCountdownController',
    'PhabricatorCountdownEditController' => 'PhabricatorCountdownController',
    'PhabricatorCountdownEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhabricatorCountdownListController' => 'PhabricatorCountdownController',
    'PhabricatorCountdownMailReceiver' => 'PhabricatorObjectMailReceiver',
    'PhabricatorCountdownQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorCountdownRemarkupRule' => 'PhabricatorObjectRemarkupRule',
    'PhabricatorCountdownReplyHandler' => 'PhabricatorApplicationTransactionReplyHandler',
    'PhabricatorCountdownSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'PhabricatorCountdownSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorCountdownTransaction' => 'PhabricatorApplicationTransaction',
    'PhabricatorCountdownTransactionComment' => 'PhabricatorApplicationTransactionComment',
    'PhabricatorCountdownTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhabricatorCountdownView' => 'AphrontTagView',
    'PhabricatorCountdownViewController' => 'PhabricatorCountdownController',
    'PhabricatorCredentialsUsedByObjectEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorCursorPagedPolicyAwareQuery' => 'PhabricatorPolicyAwareQuery',
    'PhabricatorCustomField' => 'Phobject',
    'PhabricatorCustomFieldAttachment' => 'Phobject',
    'PhabricatorCustomFieldConfigOptionType' => 'PhabricatorConfigOptionType',
    'PhabricatorCustomFieldDataNotAvailableException' => 'Exception',
    'PhabricatorCustomFieldEditEngineExtension' => 'PhabricatorEditEngineExtension',
    'PhabricatorCustomFieldEditField' => 'PhabricatorEditField',
    'PhabricatorCustomFieldEditType' => 'PhabricatorEditType',
    'PhabricatorCustomFieldFulltextEngineExtension' => 'PhabricatorFulltextEngineExtension',
    'PhabricatorCustomFieldHeraldField' => 'HeraldField',
    'PhabricatorCustomFieldHeraldFieldGroup' => 'HeraldFieldGroup',
    'PhabricatorCustomFieldImplementationIncompleteException' => 'Exception',
    'PhabricatorCustomFieldIndexStorage' => 'PhabricatorLiskDAO',
    'PhabricatorCustomFieldList' => 'Phobject',
    'PhabricatorCustomFieldMonogramParser' => 'Phobject',
    'PhabricatorCustomFieldNotAttachedException' => 'Exception',
    'PhabricatorCustomFieldNotProxyException' => 'Exception',
    'PhabricatorCustomFieldNumericIndexStorage' => 'PhabricatorCustomFieldIndexStorage',
    'PhabricatorCustomFieldSearchEngineExtension' => 'PhabricatorSearchEngineExtension',
    'PhabricatorCustomFieldStorage' => 'PhabricatorLiskDAO',
    'PhabricatorCustomFieldStringIndexStorage' => 'PhabricatorCustomFieldIndexStorage',
    'PhabricatorCustomHeaderConfigType' => 'PhabricatorConfigOptionType',
    'PhabricatorDaemon' => 'PhutilDaemon',
    'PhabricatorDaemonBulkJobListController' => 'PhabricatorDaemonController',
    'PhabricatorDaemonBulkJobMonitorController' => 'PhabricatorDaemonController',
    'PhabricatorDaemonBulkJobViewController' => 'PhabricatorDaemonController',
    'PhabricatorDaemonConsoleController' => 'PhabricatorDaemonController',
    'PhabricatorDaemonController' => 'PhabricatorController',
    'PhabricatorDaemonDAO' => 'PhabricatorLiskDAO',
    'PhabricatorDaemonEventListener' => 'PhabricatorEventListener',
    'PhabricatorDaemonLog' => array(
      'PhabricatorDaemonDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorDaemonLogEvent' => 'PhabricatorDaemonDAO',
    'PhabricatorDaemonLogEventGarbageCollector' => 'PhabricatorGarbageCollector',
    'PhabricatorDaemonLogEventViewController' => 'PhabricatorDaemonController',
    'PhabricatorDaemonLogEventsView' => 'AphrontView',
    'PhabricatorDaemonLogGarbageCollector' => 'PhabricatorGarbageCollector',
    'PhabricatorDaemonLogListController' => 'PhabricatorDaemonController',
    'PhabricatorDaemonLogListView' => 'AphrontView',
    'PhabricatorDaemonLogQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorDaemonLogViewController' => 'PhabricatorDaemonController',
    'PhabricatorDaemonManagementDebugWorkflow' => 'PhabricatorDaemonManagementWorkflow',
    'PhabricatorDaemonManagementLaunchWorkflow' => 'PhabricatorDaemonManagementWorkflow',
    'PhabricatorDaemonManagementListWorkflow' => 'PhabricatorDaemonManagementWorkflow',
    'PhabricatorDaemonManagementLogWorkflow' => 'PhabricatorDaemonManagementWorkflow',
    'PhabricatorDaemonManagementReloadWorkflow' => 'PhabricatorDaemonManagementWorkflow',
    'PhabricatorDaemonManagementRestartWorkflow' => 'PhabricatorDaemonManagementWorkflow',
    'PhabricatorDaemonManagementStartWorkflow' => 'PhabricatorDaemonManagementWorkflow',
    'PhabricatorDaemonManagementStatusWorkflow' => 'PhabricatorDaemonManagementWorkflow',
    'PhabricatorDaemonManagementStopWorkflow' => 'PhabricatorDaemonManagementWorkflow',
    'PhabricatorDaemonManagementWorkflow' => 'PhabricatorManagementWorkflow',
    'PhabricatorDaemonOverseerModule' => 'PhutilDaemonOverseerModule',
    'PhabricatorDaemonReference' => 'Phobject',
    'PhabricatorDaemonTaskGarbageCollector' => 'PhabricatorGarbageCollector',
    'PhabricatorDaemonTasksTableView' => 'AphrontView',
    'PhabricatorDaemonsApplication' => 'PhabricatorApplication',
    'PhabricatorDaemonsSetupCheck' => 'PhabricatorSetupCheck',
    'PhabricatorDailyRoutineTriggerClock' => 'PhabricatorTriggerClock',
    'PhabricatorDashboard' => array(
      'PhabricatorDashboardDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorPolicyInterface',
      'PhabricatorFlaggableInterface',
      'PhabricatorDestructibleInterface',
      'PhabricatorProjectInterface',
    ),
    'PhabricatorDashboardAddPanelController' => 'PhabricatorDashboardController',
    'PhabricatorDashboardApplication' => 'PhabricatorApplication',
    'PhabricatorDashboardArchiveController' => 'PhabricatorDashboardController',
    'PhabricatorDashboardController' => 'PhabricatorController',
    'PhabricatorDashboardCopyController' => 'PhabricatorDashboardController',
    'PhabricatorDashboardDAO' => 'PhabricatorLiskDAO',
    'PhabricatorDashboardDashboardHasPanelEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorDashboardDashboardPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorDashboardEditController' => 'PhabricatorDashboardController',
    'PhabricatorDashboardHistoryController' => 'PhabricatorDashboardController',
    'PhabricatorDashboardInstall' => 'PhabricatorDashboardDAO',
    'PhabricatorDashboardInstallController' => 'PhabricatorDashboardController',
    'PhabricatorDashboardLayoutConfig' => 'Phobject',
    'PhabricatorDashboardListController' => 'PhabricatorDashboardController',
    'PhabricatorDashboardManageController' => 'PhabricatorDashboardController',
    'PhabricatorDashboardMovePanelController' => 'PhabricatorDashboardController',
    'PhabricatorDashboardPanel' => array(
      'PhabricatorDashboardDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorPolicyInterface',
      'PhabricatorCustomFieldInterface',
      'PhabricatorFlaggableInterface',
      'PhabricatorProjectInterface',
      'PhabricatorDestructibleInterface',
    ),
    'PhabricatorDashboardPanelArchiveController' => 'PhabricatorDashboardController',
    'PhabricatorDashboardPanelCoreCustomField' => array(
      'PhabricatorDashboardPanelCustomField',
      'PhabricatorStandardCustomFieldInterface',
    ),
    'PhabricatorDashboardPanelCustomField' => 'PhabricatorCustomField',
    'PhabricatorDashboardPanelEditController' => 'PhabricatorDashboardController',
    'PhabricatorDashboardPanelHasDashboardEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorDashboardPanelListController' => 'PhabricatorDashboardController',
    'PhabricatorDashboardPanelPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorDashboardPanelQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorDashboardPanelRenderController' => 'PhabricatorDashboardController',
    'PhabricatorDashboardPanelRenderingEngine' => 'Phobject',
    'PhabricatorDashboardPanelSearchApplicationCustomField' => 'PhabricatorStandardCustomField',
    'PhabricatorDashboardPanelSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorDashboardPanelSearchQueryCustomField' => 'PhabricatorStandardCustomField',
    'PhabricatorDashboardPanelTabsCustomField' => 'PhabricatorStandardCustomField',
    'PhabricatorDashboardPanelTransaction' => 'PhabricatorApplicationTransaction',
    'PhabricatorDashboardPanelTransactionEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhabricatorDashboardPanelTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhabricatorDashboardPanelType' => 'Phobject',
    'PhabricatorDashboardPanelViewController' => 'PhabricatorDashboardController',
    'PhabricatorDashboardQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorDashboardQueryPanelType' => 'PhabricatorDashboardPanelType',
    'PhabricatorDashboardRemarkupRule' => 'PhabricatorObjectRemarkupRule',
    'PhabricatorDashboardRemovePanelController' => 'PhabricatorDashboardController',
    'PhabricatorDashboardRenderingEngine' => 'Phobject',
    'PhabricatorDashboardSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'PhabricatorDashboardSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorDashboardTabsPanelType' => 'PhabricatorDashboardPanelType',
    'PhabricatorDashboardTextPanelType' => 'PhabricatorDashboardPanelType',
    'PhabricatorDashboardTransaction' => 'PhabricatorApplicationTransaction',
    'PhabricatorDashboardTransactionEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhabricatorDashboardTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhabricatorDashboardUninstallController' => 'PhabricatorDashboardController',
    'PhabricatorDashboardViewController' => 'PhabricatorDashboardController',
    'PhabricatorDataCacheSpec' => 'PhabricatorCacheSpec',
    'PhabricatorDataNotAttachedException' => 'Exception',
    'PhabricatorDatabaseSetupCheck' => 'PhabricatorSetupCheck',
    'PhabricatorDatasourceEditField' => 'PhabricatorTokenizerEditField',
    'PhabricatorDatasourceEditType' => 'PhabricatorPHIDListEditType',
    'PhabricatorDateTimeSettingsPanel' => 'PhabricatorSettingsPanel',
    'PhabricatorDebugController' => 'PhabricatorController',
    'PhabricatorDefaultRequestExceptionHandler' => 'PhabricatorRequestExceptionHandler',
    'PhabricatorDesktopNotificationsSettingsPanel' => 'PhabricatorSettingsPanel',
    'PhabricatorDestructionEngine' => 'Phobject',
    'PhabricatorDestructionEngineExtension' => 'Phobject',
    'PhabricatorDestructionEngineExtensionModule' => 'PhabricatorConfigModule',
    'PhabricatorDeveloperConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorDeveloperPreferencesSettingsPanel' => 'PhabricatorSettingsPanel',
    'PhabricatorDiffInlineCommentQuery' => 'PhabricatorApplicationTransactionCommentQuery',
    'PhabricatorDiffPreferencesSettingsPanel' => 'PhabricatorSettingsPanel',
    'PhabricatorDifferenceEngine' => 'Phobject',
    'PhabricatorDifferentialApplication' => 'PhabricatorApplication',
    'PhabricatorDifferentialAttachCommitWorkflow' => 'PhabricatorDifferentialManagementWorkflow',
    'PhabricatorDifferentialConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorDifferentialExtractWorkflow' => 'PhabricatorDifferentialManagementWorkflow',
    'PhabricatorDifferentialManagementWorkflow' => 'PhabricatorManagementWorkflow',
    'PhabricatorDifferentialRevisionTestDataGenerator' => 'PhabricatorTestDataGenerator',
    'PhabricatorDiffusionApplication' => 'PhabricatorApplication',
    'PhabricatorDiffusionConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorDisabledUserController' => 'PhabricatorAuthController',
    'PhabricatorDisplayPreferencesSettingsPanel' => 'PhabricatorSettingsPanel',
    'PhabricatorDisqusAuthProvider' => 'PhabricatorOAuth2AuthProvider',
    'PhabricatorDividerProfilePanel' => 'PhabricatorProfilePanel',
    'PhabricatorDivinerApplication' => 'PhabricatorApplication',
    'PhabricatorDoorkeeperApplication' => 'PhabricatorApplication',
    'PhabricatorDraft' => 'PhabricatorDraftDAO',
    'PhabricatorDraftDAO' => 'PhabricatorLiskDAO',
    'PhabricatorDrydockApplication' => 'PhabricatorApplication',
    'PhabricatorEdgeConfig' => 'PhabricatorEdgeConstants',
    'PhabricatorEdgeConstants' => 'Phobject',
    'PhabricatorEdgeCycleException' => 'Exception',
    'PhabricatorEdgeEditType' => 'PhabricatorPHIDListEditType',
    'PhabricatorEdgeEditor' => 'Phobject',
    'PhabricatorEdgeGraph' => 'AbstractDirectedGraph',
    'PhabricatorEdgeQuery' => 'PhabricatorQuery',
    'PhabricatorEdgeTestCase' => 'PhabricatorTestCase',
    'PhabricatorEdgeType' => 'Phobject',
    'PhabricatorEdgeTypeTestCase' => 'PhabricatorTestCase',
    'PhabricatorEdgesDestructionEngineExtension' => 'PhabricatorDestructionEngineExtension',
    'PhabricatorEditEngine' => array(
      'Phobject',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorEditEngineAPIMethod' => 'ConduitAPIMethod',
    'PhabricatorEditEngineCommentAction' => 'Phobject',
    'PhabricatorEditEngineConfiguration' => array(
      'PhabricatorSearchDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorEditEngineConfigurationDefaultCreateController' => 'PhabricatorEditEngineController',
    'PhabricatorEditEngineConfigurationDefaultsController' => 'PhabricatorEditEngineController',
    'PhabricatorEditEngineConfigurationDisableController' => 'PhabricatorEditEngineController',
    'PhabricatorEditEngineConfigurationEditController' => 'PhabricatorEditEngineController',
    'PhabricatorEditEngineConfigurationEditEngine' => 'PhabricatorEditEngine',
    'PhabricatorEditEngineConfigurationEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhabricatorEditEngineConfigurationIsEditController' => 'PhabricatorEditEngineController',
    'PhabricatorEditEngineConfigurationListController' => 'PhabricatorEditEngineController',
    'PhabricatorEditEngineConfigurationLockController' => 'PhabricatorEditEngineController',
    'PhabricatorEditEngineConfigurationPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorEditEngineConfigurationQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorEditEngineConfigurationReorderController' => 'PhabricatorEditEngineController',
    'PhabricatorEditEngineConfigurationSaveController' => 'PhabricatorEditEngineController',
    'PhabricatorEditEngineConfigurationSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorEditEngineConfigurationSortController' => 'PhabricatorEditEngineController',
    'PhabricatorEditEngineConfigurationTransaction' => 'PhabricatorApplicationTransaction',
    'PhabricatorEditEngineConfigurationTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhabricatorEditEngineConfigurationViewController' => 'PhabricatorEditEngineController',
    'PhabricatorEditEngineController' => 'PhabricatorApplicationTransactionController',
    'PhabricatorEditEngineExtension' => 'Phobject',
    'PhabricatorEditEngineExtensionModule' => 'PhabricatorConfigModule',
    'PhabricatorEditEngineListController' => 'PhabricatorEditEngineController',
    'PhabricatorEditEngineQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorEditEngineSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorEditEngineSelectCommentAction' => 'PhabricatorEditEngineCommentAction',
    'PhabricatorEditEngineTokenizerCommentAction' => 'PhabricatorEditEngineCommentAction',
    'PhabricatorEditField' => 'Phobject',
    'PhabricatorEditType' => 'Phobject',
    'PhabricatorEditor' => 'Phobject',
    'PhabricatorElasticFulltextStorageEngine' => 'PhabricatorFulltextStorageEngine',
    'PhabricatorElasticSearchSetupCheck' => 'PhabricatorSetupCheck',
    'PhabricatorEmailAddressesSettingsPanel' => 'PhabricatorSettingsPanel',
    'PhabricatorEmailFormatSettingsPanel' => 'PhabricatorSettingsPanel',
    'PhabricatorEmailLoginController' => 'PhabricatorAuthController',
    'PhabricatorEmailPreferencesSettingsPanel' => 'PhabricatorSettingsPanel',
    'PhabricatorEmailVerificationController' => 'PhabricatorAuthController',
    'PhabricatorEmbedFileRemarkupRule' => 'PhabricatorObjectRemarkupRule',
    'PhabricatorEmojiRemarkupRule' => 'PhutilRemarkupRule',
    'PhabricatorEmptyQueryException' => 'Exception',
    'PhabricatorEnv' => 'Phobject',
    'PhabricatorEnvTestCase' => 'PhabricatorTestCase',
    'PhabricatorEvent' => 'PhutilEvent',
    'PhabricatorEventEngine' => 'Phobject',
    'PhabricatorEventListener' => 'PhutilEventListener',
    'PhabricatorEventType' => 'PhutilEventType',
    'PhabricatorExampleEventListener' => 'PhabricatorEventListener',
    'PhabricatorExecFutureFileUploadSource' => 'PhabricatorFileUploadSource',
    'PhabricatorExtendingPhabricatorConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorExtensionsSetupCheck' => 'PhabricatorSetupCheck',
    'PhabricatorExternalAccount' => array(
      'PhabricatorUserDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorExternalAccountQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorExternalAccountsSettingsPanel' => 'PhabricatorSettingsPanel',
    'PhabricatorExtraConfigSetupCheck' => 'PhabricatorSetupCheck',
    'PhabricatorFacebookAuthProvider' => 'PhabricatorOAuth2AuthProvider',
    'PhabricatorFactAggregate' => 'PhabricatorFactDAO',
    'PhabricatorFactApplication' => 'PhabricatorApplication',
    'PhabricatorFactChartController' => 'PhabricatorFactController',
    'PhabricatorFactController' => 'PhabricatorController',
    'PhabricatorFactCountEngine' => 'PhabricatorFactEngine',
    'PhabricatorFactCursor' => 'PhabricatorFactDAO',
    'PhabricatorFactDAO' => 'PhabricatorLiskDAO',
    'PhabricatorFactDaemon' => 'PhabricatorDaemon',
    'PhabricatorFactEngine' => 'Phobject',
    'PhabricatorFactEngineTestCase' => 'PhabricatorTestCase',
    'PhabricatorFactHomeController' => 'PhabricatorFactController',
    'PhabricatorFactLastUpdatedEngine' => 'PhabricatorFactEngine',
    'PhabricatorFactManagementAnalyzeWorkflow' => 'PhabricatorFactManagementWorkflow',
    'PhabricatorFactManagementCursorsWorkflow' => 'PhabricatorFactManagementWorkflow',
    'PhabricatorFactManagementDestroyWorkflow' => 'PhabricatorFactManagementWorkflow',
    'PhabricatorFactManagementListWorkflow' => 'PhabricatorFactManagementWorkflow',
    'PhabricatorFactManagementStatusWorkflow' => 'PhabricatorFactManagementWorkflow',
    'PhabricatorFactManagementWorkflow' => 'PhabricatorManagementWorkflow',
    'PhabricatorFactRaw' => 'PhabricatorFactDAO',
    'PhabricatorFactSimpleSpec' => 'PhabricatorFactSpec',
    'PhabricatorFactSpec' => 'Phobject',
    'PhabricatorFactUpdateIterator' => 'PhutilBufferedIterator',
    'PhabricatorFeedApplication' => 'PhabricatorApplication',
    'PhabricatorFeedBuilder' => 'Phobject',
    'PhabricatorFeedConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorFeedController' => 'PhabricatorController',
    'PhabricatorFeedDAO' => 'PhabricatorLiskDAO',
    'PhabricatorFeedDetailController' => 'PhabricatorFeedController',
    'PhabricatorFeedListController' => 'PhabricatorFeedController',
    'PhabricatorFeedManagementRepublishWorkflow' => 'PhabricatorFeedManagementWorkflow',
    'PhabricatorFeedManagementWorkflow' => 'PhabricatorManagementWorkflow',
    'PhabricatorFeedQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorFeedSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorFeedStory' => array(
      'Phobject',
      'PhabricatorPolicyInterface',
      'PhabricatorMarkupInterface',
    ),
    'PhabricatorFeedStoryData' => 'PhabricatorFeedDAO',
    'PhabricatorFeedStoryNotification' => 'PhabricatorFeedDAO',
    'PhabricatorFeedStoryPublisher' => 'Phobject',
    'PhabricatorFeedStoryReference' => 'PhabricatorFeedDAO',
    'PhabricatorFile' => array(
      'PhabricatorFileDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorTokenReceiverInterface',
      'PhabricatorSubscribableInterface',
      'PhabricatorFlaggableInterface',
      'PhabricatorPolicyInterface',
      'PhabricatorDestructibleInterface',
    ),
    'PhabricatorFileBundleLoader' => 'Phobject',
    'PhabricatorFileChunk' => array(
      'PhabricatorFileDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorDestructibleInterface',
    ),
    'PhabricatorFileChunkIterator' => array(
      'Phobject',
      'Iterator',
    ),
    'PhabricatorFileChunkQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorFileCommentController' => 'PhabricatorFileController',
    'PhabricatorFileComposeController' => 'PhabricatorFileController',
    'PhabricatorFileController' => 'PhabricatorController',
    'PhabricatorFileDAO' => 'PhabricatorLiskDAO',
    'PhabricatorFileDataController' => 'PhabricatorFileController',
    'PhabricatorFileDeleteController' => 'PhabricatorFileController',
    'PhabricatorFileDropUploadController' => 'PhabricatorFileController',
    'PhabricatorFileEditController' => 'PhabricatorFileController',
    'PhabricatorFileEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhabricatorFileFilePHIDType' => 'PhabricatorPHIDType',
    'PhabricatorFileHasObjectEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorFileIconSetSelectController' => 'PhabricatorFileController',
    'PhabricatorFileImageMacro' => array(
      'PhabricatorFileDAO',
      'PhabricatorSubscribableInterface',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorFlaggableInterface',
      'PhabricatorTokenReceiverInterface',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorFileImageTransform' => 'PhabricatorFileTransform',
    'PhabricatorFileInfoController' => 'PhabricatorFileController',
    'PhabricatorFileLinkView' => 'AphrontView',
    'PhabricatorFileListController' => 'PhabricatorFileController',
    'PhabricatorFileQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorFileSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'PhabricatorFileSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorFileStorageBlob' => 'PhabricatorFileDAO',
    'PhabricatorFileStorageConfigurationException' => 'Exception',
    'PhabricatorFileStorageEngine' => 'Phobject',
    'PhabricatorFileStorageEngineTestCase' => 'PhabricatorTestCase',
    'PhabricatorFileTemporaryGarbageCollector' => 'PhabricatorGarbageCollector',
    'PhabricatorFileTestCase' => 'PhabricatorTestCase',
    'PhabricatorFileTestDataGenerator' => 'PhabricatorTestDataGenerator',
    'PhabricatorFileThumbnailTransform' => 'PhabricatorFileImageTransform',
    'PhabricatorFileTransaction' => 'PhabricatorApplicationTransaction',
    'PhabricatorFileTransactionComment' => 'PhabricatorApplicationTransactionComment',
    'PhabricatorFileTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhabricatorFileTransform' => 'Phobject',
    'PhabricatorFileTransformController' => 'PhabricatorFileController',
    'PhabricatorFileTransformListController' => 'PhabricatorFileController',
    'PhabricatorFileTransformTestCase' => 'PhabricatorTestCase',
    'PhabricatorFileUploadController' => 'PhabricatorFileController',
    'PhabricatorFileUploadDialogController' => 'PhabricatorFileController',
    'PhabricatorFileUploadException' => 'Exception',
    'PhabricatorFileUploadSource' => 'Phobject',
    'PhabricatorFileinfoSetupCheck' => 'PhabricatorSetupCheck',
    'PhabricatorFilesApplication' => 'PhabricatorApplication',
    'PhabricatorFilesApplicationStorageEnginePanel' => 'PhabricatorApplicationConfigurationPanel',
    'PhabricatorFilesBuiltinFile' => 'Phobject',
    'PhabricatorFilesComposeIconBuiltinFile' => 'PhabricatorFilesBuiltinFile',
    'PhabricatorFilesConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorFilesManagementCatWorkflow' => 'PhabricatorFilesManagementWorkflow',
    'PhabricatorFilesManagementCompactWorkflow' => 'PhabricatorFilesManagementWorkflow',
    'PhabricatorFilesManagementEnginesWorkflow' => 'PhabricatorFilesManagementWorkflow',
    'PhabricatorFilesManagementMigrateWorkflow' => 'PhabricatorFilesManagementWorkflow',
    'PhabricatorFilesManagementPurgeWorkflow' => 'PhabricatorFilesManagementWorkflow',
    'PhabricatorFilesManagementRebuildWorkflow' => 'PhabricatorFilesManagementWorkflow',
    'PhabricatorFilesManagementWorkflow' => 'PhabricatorManagementWorkflow',
    'PhabricatorFilesOnDiskBuiltinFile' => 'PhabricatorFilesBuiltinFile',
    'PhabricatorFilesOutboundRequestAction' => 'PhabricatorSystemAction',
    'PhabricatorFlag' => array(
      'PhabricatorFlagDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorFlagAddFlagHeraldAction' => 'HeraldAction',
    'PhabricatorFlagColor' => 'PhabricatorFlagConstants',
    'PhabricatorFlagConstants' => 'Phobject',
    'PhabricatorFlagController' => 'PhabricatorController',
    'PhabricatorFlagDAO' => 'PhabricatorLiskDAO',
    'PhabricatorFlagDeleteController' => 'PhabricatorFlagController',
    'PhabricatorFlagDestructionEngineExtension' => 'PhabricatorDestructionEngineExtension',
    'PhabricatorFlagEditController' => 'PhabricatorFlagController',
    'PhabricatorFlagListController' => 'PhabricatorFlagController',
    'PhabricatorFlagQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorFlagSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorFlagSelectControl' => 'AphrontFormControl',
    'PhabricatorFlaggableInterface' => 'PhabricatorPHIDInterface',
    'PhabricatorFlagsApplication' => 'PhabricatorApplication',
    'PhabricatorFlagsUIEventListener' => 'PhabricatorEventListener',
    'PhabricatorFulltextEngine' => 'Phobject',
    'PhabricatorFulltextEngineExtension' => 'Phobject',
    'PhabricatorFulltextEngineExtensionModule' => 'PhabricatorConfigModule',
    'PhabricatorFulltextIndexEngineExtension' => 'PhabricatorIndexEngineExtension',
    'PhabricatorFulltextStorageEngine' => 'Phobject',
    'PhabricatorFundApplication' => 'PhabricatorApplication',
    'PhabricatorGDSetupCheck' => 'PhabricatorSetupCheck',
    'PhabricatorGarbageCollector' => 'Phobject',
    'PhabricatorGarbageCollectorManagementCollectWorkflow' => 'PhabricatorGarbageCollectorManagementWorkflow',
    'PhabricatorGarbageCollectorManagementSetPolicyWorkflow' => 'PhabricatorGarbageCollectorManagementWorkflow',
    'PhabricatorGarbageCollectorManagementWorkflow' => 'PhabricatorManagementWorkflow',
    'PhabricatorGestureUIExample' => 'PhabricatorUIExample',
    'PhabricatorGitGraphStream' => 'PhabricatorRepositoryGraphStream',
    'PhabricatorGitHubAuthProvider' => 'PhabricatorOAuth2AuthProvider',
    'PhabricatorGlobalLock' => 'PhutilLock',
    'PhabricatorGlobalUploadTargetView' => 'AphrontView',
    'PhabricatorGoogleAuthProvider' => 'PhabricatorOAuth2AuthProvider',
    'PhabricatorHTTPParameterTypeTableView' => 'AphrontView',
    'PhabricatorHandleList' => array(
      'Phobject',
      'Iterator',
      'ArrayAccess',
      'Countable',
    ),
    'PhabricatorHandleObjectSelectorDataView' => 'Phobject',
    'PhabricatorHandlePool' => 'Phobject',
    'PhabricatorHandlePoolTestCase' => 'PhabricatorTestCase',
    'PhabricatorHandleQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorHandlesEditField' => 'PhabricatorPHIDListEditField',
    'PhabricatorHarbormasterApplication' => 'PhabricatorApplication',
    'PhabricatorHarbormasterConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorHash' => 'Phobject',
    'PhabricatorHashTestCase' => 'PhabricatorTestCase',
    'PhabricatorHelpApplication' => 'PhabricatorApplication',
    'PhabricatorHelpController' => 'PhabricatorController',
    'PhabricatorHelpDocumentationController' => 'PhabricatorHelpController',
    'PhabricatorHelpEditorProtocolController' => 'PhabricatorHelpController',
    'PhabricatorHelpKeyboardShortcutController' => 'PhabricatorHelpController',
    'PhabricatorHelpMainMenuBarExtension' => 'PhabricatorMainMenuBarExtension',
    'PhabricatorHeraldApplication' => 'PhabricatorApplication',
    'PhabricatorHighSecurityRequestExceptionHandler' => 'PhabricatorRequestExceptionHandler',
    'PhabricatorHomeApplication' => 'PhabricatorApplication',
    'PhabricatorHomeController' => 'PhabricatorController',
    'PhabricatorHomeMainController' => 'PhabricatorHomeController',
    'PhabricatorHomePreferencesSettingsPanel' => 'PhabricatorSettingsPanel',
    'PhabricatorHomeQuickCreateController' => 'PhabricatorHomeController',
    'PhabricatorHovercardEngineExtension' => 'Phobject',
    'PhabricatorHovercardEngineExtensionModule' => 'PhabricatorConfigModule',
    'PhabricatorHovercardUIExample' => 'PhabricatorUIExample',
    'PhabricatorHovercardView' => 'AphrontView',
    'PhabricatorHunksManagementMigrateWorkflow' => 'PhabricatorHunksManagementWorkflow',
    'PhabricatorHunksManagementWorkflow' => 'PhabricatorManagementWorkflow',
    'PhabricatorIDsSearchEngineExtension' => 'PhabricatorSearchEngineExtension',
    'PhabricatorIDsSearchField' => 'PhabricatorSearchField',
    'PhabricatorIRCProtocolAdapter' => 'PhabricatorProtocolAdapter',
    'PhabricatorIconRemarkupRule' => 'PhutilRemarkupRule',
    'PhabricatorIconSet' => 'Phobject',
    'PhabricatorIconSetEditField' => 'PhabricatorEditField',
    'PhabricatorIconSetIcon' => 'Phobject',
    'PhabricatorImageMacroRemarkupRule' => 'PhutilRemarkupRule',
    'PhabricatorImageTransformer' => 'Phobject',
    'PhabricatorImagemagickSetupCheck' => 'PhabricatorSetupCheck',
    'PhabricatorIndexEngine' => 'Phobject',
    'PhabricatorIndexEngineExtension' => 'Phobject',
    'PhabricatorIndexEngineExtensionModule' => 'PhabricatorConfigModule',
    'PhabricatorInfrastructureTestCase' => 'PhabricatorTestCase',
    'PhabricatorInlineCommentController' => 'PhabricatorController',
    'PhabricatorInlineCommentInterface' => 'PhabricatorMarkupInterface',
    'PhabricatorInlineCommentPreviewController' => 'PhabricatorController',
    'PhabricatorInlineSummaryView' => 'AphrontView',
    'PhabricatorInstructionsEditField' => 'PhabricatorEditField',
    'PhabricatorInternationalizationManagementExtractWorkflow' => 'PhabricatorInternationalizationManagementWorkflow',
    'PhabricatorInternationalizationManagementWorkflow' => 'PhabricatorManagementWorkflow',
    'PhabricatorInvalidConfigSetupCheck' => 'PhabricatorSetupCheck',
    'PhabricatorIteratedMD5PasswordHasher' => 'PhabricatorPasswordHasher',
    'PhabricatorIteratedMD5PasswordHasherTestCase' => 'PhabricatorTestCase',
    'PhabricatorJIRAAuthProvider' => 'PhabricatorOAuth1AuthProvider',
    'PhabricatorJavelinLinter' => 'ArcanistLinter',
    'PhabricatorJiraIssueHasObjectEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorJumpNavHandler' => 'Phobject',
    'PhabricatorKeyValueDatabaseCache' => 'PhutilKeyValueCache',
    'PhabricatorLDAPAuthProvider' => 'PhabricatorAuthProvider',
    'PhabricatorLegalpadApplication' => 'PhabricatorApplication',
    'PhabricatorLegalpadConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorLegalpadDocumentPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorLegalpadSignaturePolicyRule' => 'PhabricatorPolicyRule',
    'PhabricatorLibraryTestCase' => 'PhutilLibraryTestCase',
    'PhabricatorLinkProfilePanel' => 'PhabricatorProfilePanel',
    'PhabricatorLipsumArtist' => 'Phobject',
    'PhabricatorLipsumGenerateWorkflow' => 'PhabricatorLipsumManagementWorkflow',
    'PhabricatorLipsumManagementWorkflow' => 'PhabricatorManagementWorkflow',
    'PhabricatorLipsumMondrianArtist' => 'PhabricatorLipsumArtist',
    'PhabricatorLiskDAO' => 'LiskDAO',
    'PhabricatorLiskFulltextEngineExtension' => 'PhabricatorFulltextEngineExtension',
    'PhabricatorLiskSearchEngineExtension' => 'PhabricatorSearchEngineExtension',
    'PhabricatorLiskSerializer' => 'Phobject',
    'PhabricatorListFilterUIExample' => 'PhabricatorUIExample',
    'PhabricatorLocalDiskFileStorageEngine' => 'PhabricatorFileStorageEngine',
    'PhabricatorLocalTimeTestCase' => 'PhabricatorTestCase',
    'PhabricatorLocaleScopeGuard' => 'Phobject',
    'PhabricatorLocaleScopeGuardTestCase' => 'PhabricatorTestCase',
    'PhabricatorLogTriggerAction' => 'PhabricatorTriggerAction',
    'PhabricatorLogoutController' => 'PhabricatorAuthController',
    'PhabricatorLunarPhasePolicyRule' => 'PhabricatorPolicyRule',
    'PhabricatorMacroApplication' => 'PhabricatorApplication',
    'PhabricatorMacroAudioController' => 'PhabricatorMacroController',
    'PhabricatorMacroCommentController' => 'PhabricatorMacroController',
    'PhabricatorMacroConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorMacroController' => 'PhabricatorController',
    'PhabricatorMacroDatasource' => 'PhabricatorTypeaheadDatasource',
    'PhabricatorMacroDisableController' => 'PhabricatorMacroController',
    'PhabricatorMacroEditController' => 'PhabricatorMacroController',
    'PhabricatorMacroEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhabricatorMacroListController' => 'PhabricatorMacroController',
    'PhabricatorMacroMacroPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorMacroMailReceiver' => 'PhabricatorObjectMailReceiver',
    'PhabricatorMacroManageCapability' => 'PhabricatorPolicyCapability',
    'PhabricatorMacroMemeController' => 'PhabricatorMacroController',
    'PhabricatorMacroMemeDialogController' => 'PhabricatorMacroController',
    'PhabricatorMacroQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorMacroReplyHandler' => 'PhabricatorApplicationTransactionReplyHandler',
    'PhabricatorMacroSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorMacroTransaction' => 'PhabricatorApplicationTransaction',
    'PhabricatorMacroTransactionComment' => 'PhabricatorApplicationTransactionComment',
    'PhabricatorMacroTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhabricatorMacroViewController' => 'PhabricatorMacroController',
    'PhabricatorMailEmailHeraldField' => 'HeraldField',
    'PhabricatorMailEmailHeraldFieldGroup' => 'HeraldFieldGroup',
    'PhabricatorMailEmailSubjectHeraldField' => 'PhabricatorMailEmailHeraldField',
    'PhabricatorMailImplementationAdapter' => 'Phobject',
    'PhabricatorMailImplementationAmazonSESAdapter' => 'PhabricatorMailImplementationPHPMailerLiteAdapter',
    'PhabricatorMailImplementationMailgunAdapter' => 'PhabricatorMailImplementationAdapter',
    'PhabricatorMailImplementationPHPMailerAdapter' => 'PhabricatorMailImplementationAdapter',
    'PhabricatorMailImplementationPHPMailerLiteAdapter' => 'PhabricatorMailImplementationAdapter',
    'PhabricatorMailImplementationSendGridAdapter' => 'PhabricatorMailImplementationAdapter',
    'PhabricatorMailImplementationTestAdapter' => 'PhabricatorMailImplementationAdapter',
    'PhabricatorMailManagementListInboundWorkflow' => 'PhabricatorMailManagementWorkflow',
    'PhabricatorMailManagementListOutboundWorkflow' => 'PhabricatorMailManagementWorkflow',
    'PhabricatorMailManagementReceiveTestWorkflow' => 'PhabricatorMailManagementWorkflow',
    'PhabricatorMailManagementResendWorkflow' => 'PhabricatorMailManagementWorkflow',
    'PhabricatorMailManagementSendTestWorkflow' => 'PhabricatorMailManagementWorkflow',
    'PhabricatorMailManagementShowInboundWorkflow' => 'PhabricatorMailManagementWorkflow',
    'PhabricatorMailManagementShowOutboundWorkflow' => 'PhabricatorMailManagementWorkflow',
    'PhabricatorMailManagementVolumeWorkflow' => 'PhabricatorMailManagementWorkflow',
    'PhabricatorMailManagementWorkflow' => 'PhabricatorManagementWorkflow',
    'PhabricatorMailOutboundMailHeraldAdapter' => 'HeraldAdapter',
    'PhabricatorMailOutboundRoutingHeraldAction' => 'HeraldAction',
    'PhabricatorMailOutboundRoutingSelfEmailHeraldAction' => 'PhabricatorMailOutboundRoutingHeraldAction',
    'PhabricatorMailOutboundRoutingSelfNotificationHeraldAction' => 'PhabricatorMailOutboundRoutingHeraldAction',
    'PhabricatorMailOutboundStatus' => 'Phobject',
    'PhabricatorMailReceiver' => 'Phobject',
    'PhabricatorMailReceiverTestCase' => 'PhabricatorTestCase',
    'PhabricatorMailReplyHandler' => 'Phobject',
    'PhabricatorMailRoutingRule' => 'Phobject',
    'PhabricatorMailSetupCheck' => 'PhabricatorSetupCheck',
    'PhabricatorMailTarget' => 'Phobject',
    'PhabricatorMailgunConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorMainMenuBarExtension' => 'Phobject',
    'PhabricatorMainMenuSearchView' => 'AphrontView',
    'PhabricatorMainMenuView' => 'AphrontView',
    'PhabricatorManagementWorkflow' => 'PhutilArgumentWorkflow',
    'PhabricatorManiphestApplication' => 'PhabricatorApplication',
    'PhabricatorManiphestConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorManiphestTaskTestDataGenerator' => 'PhabricatorTestDataGenerator',
    'PhabricatorMarkupCache' => 'PhabricatorCacheDAO',
    'PhabricatorMarkupEngine' => 'Phobject',
    'PhabricatorMarkupOneOff' => array(
      'Phobject',
      'PhabricatorMarkupInterface',
    ),
    'PhabricatorMarkupPreviewController' => 'PhabricatorController',
    'PhabricatorMemeRemarkupRule' => 'PhutilRemarkupRule',
    'PhabricatorMentionRemarkupRule' => 'PhutilRemarkupRule',
    'PhabricatorMercurialGraphStream' => 'PhabricatorRepositoryGraphStream',
    'PhabricatorMetaMTAActor' => 'Phobject',
    'PhabricatorMetaMTAActorQuery' => 'PhabricatorQuery',
    'PhabricatorMetaMTAApplication' => 'PhabricatorApplication',
    'PhabricatorMetaMTAApplicationEmail' => array(
      'PhabricatorMetaMTADAO',
      'PhabricatorPolicyInterface',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorDestructibleInterface',
      'PhabricatorSpacesInterface',
    ),
    'PhabricatorMetaMTAApplicationEmailDatasource' => 'PhabricatorTypeaheadDatasource',
    'PhabricatorMetaMTAApplicationEmailEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhabricatorMetaMTAApplicationEmailHeraldField' => 'HeraldField',
    'PhabricatorMetaMTAApplicationEmailPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorMetaMTAApplicationEmailPanel' => 'PhabricatorApplicationConfigurationPanel',
    'PhabricatorMetaMTAApplicationEmailQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorMetaMTAApplicationEmailTransaction' => 'PhabricatorApplicationTransaction',
    'PhabricatorMetaMTAApplicationEmailTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhabricatorMetaMTAAttachment' => 'Phobject',
    'PhabricatorMetaMTAConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorMetaMTAController' => 'PhabricatorController',
    'PhabricatorMetaMTADAO' => 'PhabricatorLiskDAO',
    'PhabricatorMetaMTAEmailBodyParser' => 'Phobject',
    'PhabricatorMetaMTAEmailBodyParserTestCase' => 'PhabricatorTestCase',
    'PhabricatorMetaMTAEmailHeraldAction' => 'HeraldAction',
    'PhabricatorMetaMTAEmailOthersHeraldAction' => 'PhabricatorMetaMTAEmailHeraldAction',
    'PhabricatorMetaMTAEmailSelfHeraldAction' => 'PhabricatorMetaMTAEmailHeraldAction',
    'PhabricatorMetaMTAErrorMailAction' => 'PhabricatorSystemAction',
    'PhabricatorMetaMTAMail' => array(
      'PhabricatorMetaMTADAO',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorMetaMTAMailBody' => 'Phobject',
    'PhabricatorMetaMTAMailBodyTestCase' => 'PhabricatorTestCase',
    'PhabricatorMetaMTAMailHasRecipientEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorMetaMTAMailListController' => 'PhabricatorMetaMTAController',
    'PhabricatorMetaMTAMailPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorMetaMTAMailQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorMetaMTAMailSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorMetaMTAMailSection' => 'Phobject',
    'PhabricatorMetaMTAMailTestCase' => 'PhabricatorTestCase',
    'PhabricatorMetaMTAMailViewController' => 'PhabricatorMetaMTAController',
    'PhabricatorMetaMTAMailableDatasource' => 'PhabricatorTypeaheadCompositeDatasource',
    'PhabricatorMetaMTAMailableFunctionDatasource' => 'PhabricatorTypeaheadCompositeDatasource',
    'PhabricatorMetaMTAMailgunReceiveController' => 'PhabricatorMetaMTAController',
    'PhabricatorMetaMTAMemberQuery' => 'PhabricatorQuery',
    'PhabricatorMetaMTAPermanentFailureException' => 'Exception',
    'PhabricatorMetaMTAReceivedMail' => 'PhabricatorMetaMTADAO',
    'PhabricatorMetaMTAReceivedMailProcessingException' => 'Exception',
    'PhabricatorMetaMTAReceivedMailTestCase' => 'PhabricatorTestCase',
    'PhabricatorMetaMTASchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'PhabricatorMetaMTASendGridReceiveController' => 'PhabricatorMetaMTAController',
    'PhabricatorMetaMTAWorker' => 'PhabricatorWorker',
    'PhabricatorMetronomicTriggerClock' => 'PhabricatorTriggerClock',
    'PhabricatorMotivatorProfilePanel' => 'PhabricatorProfilePanel',
    'PhabricatorMultiColumnUIExample' => 'PhabricatorUIExample',
    'PhabricatorMultiFactorSettingsPanel' => 'PhabricatorSettingsPanel',
    'PhabricatorMultimeterApplication' => 'PhabricatorApplication',
    'PhabricatorMustVerifyEmailController' => 'PhabricatorAuthController',
    'PhabricatorMySQLConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorMySQLFileStorageEngine' => 'PhabricatorFileStorageEngine',
    'PhabricatorMySQLFulltextStorageEngine' => 'PhabricatorFulltextStorageEngine',
    'PhabricatorMySQLSetupCheck' => 'PhabricatorSetupCheck',
    'PhabricatorNamedQuery' => array(
      'PhabricatorSearchDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorNamedQueryQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorNavigationRemarkupRule' => 'PhutilRemarkupRule',
    'PhabricatorNeverTriggerClock' => 'PhabricatorTriggerClock',
    'PhabricatorNgramsIndexEngineExtension' => 'PhabricatorIndexEngineExtension',
    'PhabricatorNotificationBuilder' => 'Phobject',
    'PhabricatorNotificationClearController' => 'PhabricatorNotificationController',
    'PhabricatorNotificationClient' => 'Phobject',
    'PhabricatorNotificationConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorNotificationController' => 'PhabricatorController',
    'PhabricatorNotificationDestructionEngineExtension' => 'PhabricatorDestructionEngineExtension',
    'PhabricatorNotificationIndividualController' => 'PhabricatorNotificationController',
    'PhabricatorNotificationListController' => 'PhabricatorNotificationController',
    'PhabricatorNotificationPanelController' => 'PhabricatorNotificationController',
    'PhabricatorNotificationQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorNotificationSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorNotificationStatusController' => 'PhabricatorNotificationController',
    'PhabricatorNotificationStatusView' => 'AphrontTagView',
    'PhabricatorNotificationTestController' => 'PhabricatorNotificationController',
    'PhabricatorNotificationTestFeedStory' => 'PhabricatorFeedStory',
    'PhabricatorNotificationUIExample' => 'PhabricatorUIExample',
    'PhabricatorNotificationsApplication' => 'PhabricatorApplication',
    'PhabricatorNuanceApplication' => 'PhabricatorApplication',
    'PhabricatorOAuth1AuthProvider' => 'PhabricatorOAuthAuthProvider',
    'PhabricatorOAuth2AuthProvider' => 'PhabricatorOAuthAuthProvider',
    'PhabricatorOAuthAuthProvider' => 'PhabricatorAuthProvider',
    'PhabricatorOAuthClientAuthorization' => array(
      'PhabricatorOAuthServerDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorOAuthClientAuthorizationQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorOAuthClientController' => 'PhabricatorOAuthServerController',
    'PhabricatorOAuthClientDeleteController' => 'PhabricatorOAuthClientController',
    'PhabricatorOAuthClientEditController' => 'PhabricatorOAuthClientController',
    'PhabricatorOAuthClientListController' => 'PhabricatorOAuthClientController',
    'PhabricatorOAuthClientSecretController' => 'PhabricatorOAuthClientController',
    'PhabricatorOAuthClientViewController' => 'PhabricatorOAuthClientController',
    'PhabricatorOAuthResponse' => 'AphrontResponse',
    'PhabricatorOAuthServer' => 'Phobject',
    'PhabricatorOAuthServerAccessToken' => 'PhabricatorOAuthServerDAO',
    'PhabricatorOAuthServerApplication' => 'PhabricatorApplication',
    'PhabricatorOAuthServerAuthController' => 'PhabricatorOAuthServerController',
    'PhabricatorOAuthServerAuthorizationCode' => 'PhabricatorOAuthServerDAO',
    'PhabricatorOAuthServerAuthorizationsSettingsPanel' => 'PhabricatorSettingsPanel',
    'PhabricatorOAuthServerClient' => array(
      'PhabricatorOAuthServerDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorDestructibleInterface',
    ),
    'PhabricatorOAuthServerClientAuthorizationPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorOAuthServerClientPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorOAuthServerClientQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorOAuthServerClientSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorOAuthServerController' => 'PhabricatorController',
    'PhabricatorOAuthServerCreateClientsCapability' => 'PhabricatorPolicyCapability',
    'PhabricatorOAuthServerDAO' => 'PhabricatorLiskDAO',
    'PhabricatorOAuthServerScope' => 'Phobject',
    'PhabricatorOAuthServerTestCase' => 'PhabricatorTestCase',
    'PhabricatorOAuthServerTestController' => 'PhabricatorOAuthServerController',
    'PhabricatorOAuthServerTokenController' => 'PhabricatorOAuthServerController',
    'PhabricatorObjectHandle' => array(
      'Phobject',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorObjectHasAsanaSubtaskEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorObjectHasAsanaTaskEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorObjectHasContributorEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorObjectHasFileEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorObjectHasJiraIssueEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorObjectHasSubscriberEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorObjectHasUnsubscriberEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorObjectHasWatcherEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorObjectListQuery' => 'Phobject',
    'PhabricatorObjectListQueryTestCase' => 'PhabricatorTestCase',
    'PhabricatorObjectMailReceiver' => 'PhabricatorMailReceiver',
    'PhabricatorObjectMailReceiverTestCase' => 'PhabricatorTestCase',
    'PhabricatorObjectMentionedByObjectEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorObjectMentionsObjectEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorObjectQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorObjectRemarkupRule' => 'PhutilRemarkupRule',
    'PhabricatorObjectSelectorDialog' => 'Phobject',
    'PhabricatorObjectUsesCredentialsEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorOffsetPagedQuery' => 'PhabricatorQuery',
    'PhabricatorOneTimeTriggerClock' => 'PhabricatorTriggerClock',
    'PhabricatorOpcodeCacheSpec' => 'PhabricatorCacheSpec',
    'PhabricatorOwnerPathQuery' => 'Phobject',
    'PhabricatorOwnersApplication' => 'PhabricatorApplication',
    'PhabricatorOwnersArchiveController' => 'PhabricatorOwnersController',
    'PhabricatorOwnersConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorOwnersConfiguredCustomField' => array(
      'PhabricatorOwnersCustomField',
      'PhabricatorStandardCustomFieldInterface',
    ),
    'PhabricatorOwnersController' => 'PhabricatorController',
    'PhabricatorOwnersCustomField' => 'PhabricatorCustomField',
    'PhabricatorOwnersCustomFieldNumericIndex' => 'PhabricatorCustomFieldNumericIndexStorage',
    'PhabricatorOwnersCustomFieldStorage' => 'PhabricatorCustomFieldStorage',
    'PhabricatorOwnersCustomFieldStringIndex' => 'PhabricatorCustomFieldStringIndexStorage',
    'PhabricatorOwnersDAO' => 'PhabricatorLiskDAO',
    'PhabricatorOwnersDetailController' => 'PhabricatorOwnersController',
    'PhabricatorOwnersEditController' => 'PhabricatorOwnersController',
    'PhabricatorOwnersListController' => 'PhabricatorOwnersController',
    'PhabricatorOwnersOwner' => 'PhabricatorOwnersDAO',
    'PhabricatorOwnersPackage' => array(
      'PhabricatorOwnersDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorCustomFieldInterface',
      'PhabricatorDestructibleInterface',
      'PhabricatorConduitResultInterface',
      'PhabricatorFulltextInterface',
      'PhabricatorNgramsInterface',
    ),
    'PhabricatorOwnersPackageDatasource' => 'PhabricatorTypeaheadDatasource',
    'PhabricatorOwnersPackageEditEngine' => 'PhabricatorEditEngine',
    'PhabricatorOwnersPackageFulltextEngine' => 'PhabricatorFulltextEngine',
    'PhabricatorOwnersPackageFunctionDatasource' => 'PhabricatorTypeaheadCompositeDatasource',
    'PhabricatorOwnersPackageNameNgrams' => 'PhabricatorSearchNgrams',
    'PhabricatorOwnersPackageOwnerDatasource' => 'PhabricatorTypeaheadCompositeDatasource',
    'PhabricatorOwnersPackagePHIDType' => 'PhabricatorPHIDType',
    'PhabricatorOwnersPackageQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorOwnersPackageSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorOwnersPackageTestCase' => 'PhabricatorTestCase',
    'PhabricatorOwnersPackageTransaction' => 'PhabricatorApplicationTransaction',
    'PhabricatorOwnersPackageTransactionEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhabricatorOwnersPackageTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhabricatorOwnersPath' => 'PhabricatorOwnersDAO',
    'PhabricatorOwnersPathsController' => 'PhabricatorOwnersController',
    'PhabricatorOwnersPathsSearchEngineAttachment' => 'PhabricatorSearchEngineAttachment',
    'PhabricatorOwnersSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'PhabricatorOwnersSearchField' => 'PhabricatorSearchTokenizerField',
    'PhabricatorPHDConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorPHID' => 'Phobject',
    'PhabricatorPHIDConstants' => 'Phobject',
    'PhabricatorPHIDListEditField' => 'PhabricatorEditField',
    'PhabricatorPHIDListEditType' => 'PhabricatorEditType',
    'PhabricatorPHIDResolver' => 'Phobject',
    'PhabricatorPHIDType' => 'Phobject',
    'PhabricatorPHIDTypeTestCase' => 'PhutilTestCase',
    'PhabricatorPHIDsSearchField' => 'PhabricatorSearchField',
    'PhabricatorPHPASTApplication' => 'PhabricatorApplication',
    'PhabricatorPHPConfigSetupCheck' => 'PhabricatorSetupCheck',
    'PhabricatorPHPMailerConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorPagedFormUIExample' => 'PhabricatorUIExample',
    'PhabricatorPagerUIExample' => 'PhabricatorUIExample',
    'PhabricatorPassphraseApplication' => 'PhabricatorApplication',
    'PhabricatorPasswordAuthProvider' => 'PhabricatorAuthProvider',
    'PhabricatorPasswordHasher' => 'Phobject',
    'PhabricatorPasswordHasherTestCase' => 'PhabricatorTestCase',
    'PhabricatorPasswordHasherUnavailableException' => 'Exception',
    'PhabricatorPasswordSettingsPanel' => 'PhabricatorSettingsPanel',
    'PhabricatorPaste' => array(
      'PhabricatorPasteDAO',
      'PhabricatorSubscribableInterface',
      'PhabricatorTokenReceiverInterface',
      'PhabricatorFlaggableInterface',
      'PhabricatorMentionableInterface',
      'PhabricatorPolicyInterface',
      'PhabricatorProjectInterface',
      'PhabricatorDestructibleInterface',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorSpacesInterface',
      'PhabricatorConduitResultInterface',
    ),
    'PhabricatorPasteApplication' => 'PhabricatorApplication',
    'PhabricatorPasteArchiveController' => 'PhabricatorPasteController',
    'PhabricatorPasteConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorPasteContentSearchEngineAttachment' => 'PhabricatorSearchEngineAttachment',
    'PhabricatorPasteController' => 'PhabricatorController',
    'PhabricatorPasteDAO' => 'PhabricatorLiskDAO',
    'PhabricatorPasteEditController' => 'PhabricatorPasteController',
    'PhabricatorPasteEditEngine' => 'PhabricatorEditEngine',
    'PhabricatorPasteEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhabricatorPasteFilenameContextFreeGrammar' => 'PhutilContextFreeGrammar',
    'PhabricatorPasteListController' => 'PhabricatorPasteController',
    'PhabricatorPastePastePHIDType' => 'PhabricatorPHIDType',
    'PhabricatorPasteQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorPasteRawController' => 'PhabricatorPasteController',
    'PhabricatorPasteRemarkupRule' => 'PhabricatorObjectRemarkupRule',
    'PhabricatorPasteSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'PhabricatorPasteSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorPasteSnippet' => 'Phobject',
    'PhabricatorPasteTestDataGenerator' => 'PhabricatorTestDataGenerator',
    'PhabricatorPasteTransaction' => 'PhabricatorApplicationTransaction',
    'PhabricatorPasteTransactionComment' => 'PhabricatorApplicationTransactionComment',
    'PhabricatorPasteTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhabricatorPasteViewController' => 'PhabricatorPasteController',
    'PhabricatorPathSetupCheck' => 'PhabricatorSetupCheck',
    'PhabricatorPeopleAnyOwnerDatasource' => 'PhabricatorTypeaheadDatasource',
    'PhabricatorPeopleApplication' => 'PhabricatorApplication',
    'PhabricatorPeopleApproveController' => 'PhabricatorPeopleController',
    'PhabricatorPeopleCalendarController' => 'PhabricatorPeopleProfileController',
    'PhabricatorPeopleController' => 'PhabricatorController',
    'PhabricatorPeopleCreateController' => 'PhabricatorPeopleController',
    'PhabricatorPeopleDatasource' => 'PhabricatorTypeaheadDatasource',
    'PhabricatorPeopleDeleteController' => 'PhabricatorPeopleController',
    'PhabricatorPeopleDetailsProfilePanel' => 'PhabricatorProfilePanel',
    'PhabricatorPeopleDisableController' => 'PhabricatorPeopleController',
    'PhabricatorPeopleEmpowerController' => 'PhabricatorPeopleController',
    'PhabricatorPeopleExternalPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorPeopleHovercardEngineExtension' => 'PhabricatorHovercardEngineExtension',
    'PhabricatorPeopleInviteController' => 'PhabricatorPeopleController',
    'PhabricatorPeopleInviteListController' => 'PhabricatorPeopleInviteController',
    'PhabricatorPeopleInviteSendController' => 'PhabricatorPeopleInviteController',
    'PhabricatorPeopleLdapController' => 'PhabricatorPeopleController',
    'PhabricatorPeopleListController' => 'PhabricatorPeopleController',
    'PhabricatorPeopleLogQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorPeopleLogSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorPeopleLogsController' => 'PhabricatorPeopleController',
    'PhabricatorPeopleMainMenuBarExtension' => 'PhabricatorMainMenuBarExtension',
    'PhabricatorPeopleNewController' => 'PhabricatorPeopleController',
    'PhabricatorPeopleNoOwnerDatasource' => 'PhabricatorTypeaheadDatasource',
    'PhabricatorPeopleOwnerDatasource' => 'PhabricatorTypeaheadCompositeDatasource',
    'PhabricatorPeopleProfileController' => 'PhabricatorPeopleController',
    'PhabricatorPeopleProfileEditController' => 'PhabricatorPeopleProfileController',
    'PhabricatorPeopleProfilePanelEngine' => 'PhabricatorProfilePanelEngine',
    'PhabricatorPeopleProfilePictureController' => 'PhabricatorPeopleProfileController',
    'PhabricatorPeopleProfileViewController' => 'PhabricatorPeopleProfileController',
    'PhabricatorPeopleQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorPeopleRenameController' => 'PhabricatorPeopleController',
    'PhabricatorPeopleSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorPeopleTestDataGenerator' => 'PhabricatorTestDataGenerator',
    'PhabricatorPeopleTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhabricatorPeopleUserFunctionDatasource' => 'PhabricatorTypeaheadCompositeDatasource',
    'PhabricatorPeopleUserPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorPeopleWelcomeController' => 'PhabricatorPeopleController',
    'PhabricatorPersonaAuthProvider' => 'PhabricatorAuthProvider',
    'PhabricatorPhabricatorAuthProvider' => 'PhabricatorOAuth2AuthProvider',
    'PhabricatorPhameApplication' => 'PhabricatorApplication',
    'PhabricatorPhameBlogPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorPhamePostPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorPhluxApplication' => 'PhabricatorApplication',
    'PhabricatorPholioApplication' => 'PhabricatorApplication',
    'PhabricatorPholioConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorPholioMockTestDataGenerator' => 'PhabricatorTestDataGenerator',
    'PhabricatorPhortuneApplication' => 'PhabricatorApplication',
    'PhabricatorPhortuneManagementInvoiceWorkflow' => 'PhabricatorPhortuneManagementWorkflow',
    'PhabricatorPhortuneManagementWorkflow' => 'PhabricatorManagementWorkflow',
    'PhabricatorPhragmentApplication' => 'PhabricatorApplication',
    'PhabricatorPhrequentApplication' => 'PhabricatorApplication',
    'PhabricatorPhrequentConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorPhrictionApplication' => 'PhabricatorApplication',
    'PhabricatorPhrictionConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorPhurlApplication' => 'PhabricatorApplication',
    'PhabricatorPhurlConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorPhurlController' => 'PhabricatorController',
    'PhabricatorPhurlDAO' => 'PhabricatorLiskDAO',
    'PhabricatorPhurlLinkRemarkupRule' => 'PhutilRemarkupRule',
    'PhabricatorPhurlRemarkupRule' => 'PhabricatorObjectRemarkupRule',
    'PhabricatorPhurlSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'PhabricatorPhurlShortURLController' => 'PhabricatorPhurlController',
    'PhabricatorPhurlShortURLDefaultController' => 'PhabricatorPhurlController',
    'PhabricatorPhurlURL' => array(
      'PhabricatorPhurlDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorProjectInterface',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorSubscribableInterface',
      'PhabricatorTokenReceiverInterface',
      'PhabricatorDestructibleInterface',
      'PhabricatorMentionableInterface',
      'PhabricatorFlaggableInterface',
      'PhabricatorSpacesInterface',
    ),
    'PhabricatorPhurlURLAccessController' => 'PhabricatorPhurlController',
    'PhabricatorPhurlURLCommentController' => 'PhabricatorPhurlController',
    'PhabricatorPhurlURLCreateCapability' => 'PhabricatorPolicyCapability',
    'PhabricatorPhurlURLEditController' => 'PhabricatorPhurlController',
    'PhabricatorPhurlURLEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhabricatorPhurlURLListController' => 'PhabricatorPhurlController',
    'PhabricatorPhurlURLMailReceiver' => 'PhabricatorObjectMailReceiver',
    'PhabricatorPhurlURLPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorPhurlURLQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorPhurlURLReplyHandler' => 'PhabricatorApplicationTransactionReplyHandler',
    'PhabricatorPhurlURLSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorPhurlURLTransaction' => 'PhabricatorApplicationTransaction',
    'PhabricatorPhurlURLTransactionComment' => 'PhabricatorApplicationTransactionComment',
    'PhabricatorPhurlURLTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhabricatorPhurlURLViewController' => 'PhabricatorPhurlController',
    'PhabricatorPirateEnglishTranslation' => 'PhutilTranslation',
    'PhabricatorPlatformSite' => 'PhabricatorSite',
    'PhabricatorPolicies' => 'PhabricatorPolicyConstants',
    'PhabricatorPolicy' => array(
      'PhabricatorPolicyDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorDestructibleInterface',
    ),
    'PhabricatorPolicyApplication' => 'PhabricatorApplication',
    'PhabricatorPolicyAwareQuery' => 'PhabricatorOffsetPagedQuery',
    'PhabricatorPolicyAwareTestQuery' => 'PhabricatorPolicyAwareQuery',
    'PhabricatorPolicyCanEditCapability' => 'PhabricatorPolicyCapability',
    'PhabricatorPolicyCanJoinCapability' => 'PhabricatorPolicyCapability',
    'PhabricatorPolicyCanViewCapability' => 'PhabricatorPolicyCapability',
    'PhabricatorPolicyCapability' => 'Phobject',
    'PhabricatorPolicyCapabilityTestCase' => 'PhabricatorTestCase',
    'PhabricatorPolicyConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorPolicyConstants' => 'Phobject',
    'PhabricatorPolicyController' => 'PhabricatorController',
    'PhabricatorPolicyDAO' => 'PhabricatorLiskDAO',
    'PhabricatorPolicyDataTestCase' => 'PhabricatorTestCase',
    'PhabricatorPolicyEditController' => 'PhabricatorPolicyController',
    'PhabricatorPolicyEditEngineExtension' => 'PhabricatorEditEngineExtension',
    'PhabricatorPolicyEditField' => 'PhabricatorEditField',
    'PhabricatorPolicyException' => 'Exception',
    'PhabricatorPolicyExplainController' => 'PhabricatorPolicyController',
    'PhabricatorPolicyFilter' => 'Phobject',
    'PhabricatorPolicyInterface' => 'PhabricatorPHIDInterface',
    'PhabricatorPolicyManagementShowWorkflow' => 'PhabricatorPolicyManagementWorkflow',
    'PhabricatorPolicyManagementUnlockWorkflow' => 'PhabricatorPolicyManagementWorkflow',
    'PhabricatorPolicyManagementWorkflow' => 'PhabricatorManagementWorkflow',
    'PhabricatorPolicyPHIDTypePolicy' => 'PhabricatorPHIDType',
    'PhabricatorPolicyQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorPolicyRequestExceptionHandler' => 'PhabricatorRequestExceptionHandler',
    'PhabricatorPolicyRule' => 'Phobject',
    'PhabricatorPolicySearchEngineExtension' => 'PhabricatorSearchEngineExtension',
    'PhabricatorPolicyTestCase' => 'PhabricatorTestCase',
    'PhabricatorPolicyTestObject' => array(
      'Phobject',
      'PhabricatorPolicyInterface',
      'PhabricatorExtendedPolicyInterface',
    ),
    'PhabricatorPolicyType' => 'PhabricatorPolicyConstants',
    'PhabricatorPonderApplication' => 'PhabricatorApplication',
    'PhabricatorProfilePanel' => 'Phobject',
    'PhabricatorProfilePanelConfiguration' => array(
      'PhabricatorSearchDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorExtendedPolicyInterface',
      'PhabricatorApplicationTransactionInterface',
    ),
    'PhabricatorProfilePanelConfigurationQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorProfilePanelConfigurationTransaction' => 'PhabricatorApplicationTransaction',
    'PhabricatorProfilePanelConfigurationTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhabricatorProfilePanelEditEngine' => 'PhabricatorEditEngine',
    'PhabricatorProfilePanelEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhabricatorProfilePanelEngine' => 'Phobject',
    'PhabricatorProfilePanelIconSet' => 'PhabricatorIconSet',
    'PhabricatorProfilePanelPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorProject' => array(
      'PhabricatorProjectDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorFlaggableInterface',
      'PhabricatorPolicyInterface',
      'PhabricatorExtendedPolicyInterface',
      'PhabricatorCustomFieldInterface',
      'PhabricatorDestructibleInterface',
      'PhabricatorFulltextInterface',
      'PhabricatorConduitResultInterface',
    ),
    'PhabricatorProjectAddHeraldAction' => 'PhabricatorProjectHeraldAction',
    'PhabricatorProjectApplication' => 'PhabricatorApplication',
    'PhabricatorProjectArchiveController' => 'PhabricatorProjectController',
    'PhabricatorProjectBoardController' => 'PhabricatorProjectController',
    'PhabricatorProjectBoardImportController' => 'PhabricatorProjectBoardController',
    'PhabricatorProjectBoardReorderController' => 'PhabricatorProjectBoardController',
    'PhabricatorProjectBoardViewController' => 'PhabricatorProjectBoardController',
    'PhabricatorProjectColorsConfigOptionType' => 'PhabricatorConfigJSONOptionType',
    'PhabricatorProjectColumn' => array(
      'PhabricatorProjectDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorPolicyInterface',
      'PhabricatorDestructibleInterface',
    ),
    'PhabricatorProjectColumnDetailController' => 'PhabricatorProjectBoardController',
    'PhabricatorProjectColumnEditController' => 'PhabricatorProjectBoardController',
    'PhabricatorProjectColumnHideController' => 'PhabricatorProjectBoardController',
    'PhabricatorProjectColumnPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorProjectColumnPosition' => array(
      'PhabricatorProjectDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorProjectColumnPositionQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorProjectColumnQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorProjectColumnTransaction' => 'PhabricatorApplicationTransaction',
    'PhabricatorProjectColumnTransactionEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhabricatorProjectColumnTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhabricatorProjectConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorProjectConfiguredCustomField' => array(
      'PhabricatorProjectStandardCustomField',
      'PhabricatorStandardCustomFieldInterface',
    ),
    'PhabricatorProjectController' => 'PhabricatorController',
    'PhabricatorProjectCoreTestCase' => 'PhabricatorTestCase',
    'PhabricatorProjectCustomField' => 'PhabricatorCustomField',
    'PhabricatorProjectCustomFieldNumericIndex' => 'PhabricatorCustomFieldNumericIndexStorage',
    'PhabricatorProjectCustomFieldStorage' => 'PhabricatorCustomFieldStorage',
    'PhabricatorProjectCustomFieldStringIndex' => 'PhabricatorCustomFieldStringIndexStorage',
    'PhabricatorProjectDAO' => 'PhabricatorLiskDAO',
    'PhabricatorProjectDatasource' => 'PhabricatorTypeaheadDatasource',
    'PhabricatorProjectDescriptionField' => 'PhabricatorProjectStandardCustomField',
    'PhabricatorProjectDetailsProfilePanel' => 'PhabricatorProfilePanel',
    'PhabricatorProjectEditController' => 'PhabricatorProjectController',
    'PhabricatorProjectEditEngine' => 'PhabricatorEditEngine',
    'PhabricatorProjectEditPictureController' => 'PhabricatorProjectController',
    'PhabricatorProjectFulltextEngine' => 'PhabricatorFulltextEngine',
    'PhabricatorProjectHeraldAction' => 'HeraldAction',
    'PhabricatorProjectHeraldAdapter' => 'HeraldAdapter',
    'PhabricatorProjectHeraldFieldGroup' => 'HeraldFieldGroup',
    'PhabricatorProjectHistoryController' => 'PhabricatorProjectController',
    'PhabricatorProjectIconSet' => 'PhabricatorIconSet',
    'PhabricatorProjectIconsConfigOptionType' => 'PhabricatorConfigJSONOptionType',
    'PhabricatorProjectListController' => 'PhabricatorProjectController',
    'PhabricatorProjectListView' => 'AphrontView',
    'PhabricatorProjectLockController' => 'PhabricatorProjectController',
    'PhabricatorProjectLogicalAncestorDatasource' => 'PhabricatorTypeaheadCompositeDatasource',
    'PhabricatorProjectLogicalDatasource' => 'PhabricatorTypeaheadCompositeDatasource',
    'PhabricatorProjectLogicalOrNotDatasource' => 'PhabricatorTypeaheadCompositeDatasource',
    'PhabricatorProjectLogicalUserDatasource' => 'PhabricatorTypeaheadCompositeDatasource',
    'PhabricatorProjectLogicalViewerDatasource' => 'PhabricatorTypeaheadDatasource',
    'PhabricatorProjectMaterializedMemberEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorProjectMemberListView' => 'PhabricatorProjectUserListView',
    'PhabricatorProjectMemberOfProjectEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorProjectMembersAddController' => 'PhabricatorProjectController',
    'PhabricatorProjectMembersDatasource' => 'PhabricatorTypeaheadCompositeDatasource',
    'PhabricatorProjectMembersPolicyRule' => 'PhabricatorPolicyRule',
    'PhabricatorProjectMembersProfilePanel' => 'PhabricatorProfilePanel',
    'PhabricatorProjectMembersRemoveController' => 'PhabricatorProjectController',
    'PhabricatorProjectMembersViewController' => 'PhabricatorProjectController',
    'PhabricatorProjectMilestonesController' => 'PhabricatorProjectController',
    'PhabricatorProjectMoveController' => 'PhabricatorProjectController',
    'PhabricatorProjectNameContextFreeGrammar' => 'PhutilContextFreeGrammar',
    'PhabricatorProjectNoProjectsDatasource' => 'PhabricatorTypeaheadDatasource',
    'PhabricatorProjectObjectHasProjectEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorProjectOrUserDatasource' => 'PhabricatorTypeaheadCompositeDatasource',
    'PhabricatorProjectOrUserFunctionDatasource' => 'PhabricatorTypeaheadCompositeDatasource',
    'PhabricatorProjectPHIDResolver' => 'PhabricatorPHIDResolver',
    'PhabricatorProjectPanelController' => 'PhabricatorProjectController',
    'PhabricatorProjectProfileController' => 'PhabricatorProjectController',
    'PhabricatorProjectProfilePanelEngine' => 'PhabricatorProfilePanelEngine',
    'PhabricatorProjectProjectHasMemberEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorProjectProjectHasObjectEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorProjectProjectPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorProjectQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorProjectRemoveHeraldAction' => 'PhabricatorProjectHeraldAction',
    'PhabricatorProjectSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'PhabricatorProjectSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorProjectSearchField' => 'PhabricatorSearchTokenizerField',
    'PhabricatorProjectSilenceController' => 'PhabricatorProjectController',
    'PhabricatorProjectSilencedEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorProjectSlug' => 'PhabricatorProjectDAO',
    'PhabricatorProjectStandardCustomField' => array(
      'PhabricatorProjectCustomField',
      'PhabricatorStandardCustomFieldInterface',
    ),
    'PhabricatorProjectStatus' => 'Phobject',
    'PhabricatorProjectSubprojectsController' => 'PhabricatorProjectController',
    'PhabricatorProjectTestDataGenerator' => 'PhabricatorTestDataGenerator',
    'PhabricatorProjectTransaction' => 'PhabricatorApplicationTransaction',
    'PhabricatorProjectTransactionEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhabricatorProjectTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhabricatorProjectUIEventListener' => 'PhabricatorEventListener',
    'PhabricatorProjectUpdateController' => 'PhabricatorProjectController',
    'PhabricatorProjectUserFunctionDatasource' => 'PhabricatorTypeaheadCompositeDatasource',
    'PhabricatorProjectUserListView' => 'AphrontView',
    'PhabricatorProjectViewController' => 'PhabricatorProjectController',
    'PhabricatorProjectWatchController' => 'PhabricatorProjectController',
    'PhabricatorProjectWatcherListView' => 'PhabricatorProjectUserListView',
    'PhabricatorProjectWorkboardProfilePanel' => 'PhabricatorProfilePanel',
    'PhabricatorProjectsEditEngineExtension' => 'PhabricatorEditEngineExtension',
    'PhabricatorProjectsEditField' => 'PhabricatorTokenizerEditField',
    'PhabricatorProjectsFulltextEngineExtension' => 'PhabricatorFulltextEngineExtension',
    'PhabricatorProjectsMembershipIndexEngineExtension' => 'PhabricatorIndexEngineExtension',
    'PhabricatorProjectsPolicyRule' => 'PhabricatorPolicyRule',
    'PhabricatorProjectsSearchEngineAttachment' => 'PhabricatorSearchEngineAttachment',
    'PhabricatorProjectsSearchEngineExtension' => 'PhabricatorSearchEngineExtension',
    'PhabricatorProtocolAdapter' => 'Phobject',
    'PhabricatorPygmentSetupCheck' => 'PhabricatorSetupCheck',
    'PhabricatorQuery' => 'Phobject',
    'PhabricatorQueryConstraint' => 'Phobject',
    'PhabricatorQueryOrderItem' => 'Phobject',
    'PhabricatorQueryOrderTestCase' => 'PhabricatorTestCase',
    'PhabricatorQueryOrderVector' => array(
      'Phobject',
      'Iterator',
    ),
    'PhabricatorRateLimitRequestExceptionHandler' => 'PhabricatorRequestExceptionHandler',
    'PhabricatorRecaptchaConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorRecipientHasBadgeEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorRedirectController' => 'PhabricatorController',
    'PhabricatorRefreshCSRFController' => 'PhabricatorAuthController',
    'PhabricatorRegistrationProfile' => 'Phobject',
    'PhabricatorReleephApplication' => 'PhabricatorApplication',
    'PhabricatorReleephApplicationConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorRemarkupControl' => 'AphrontFormTextAreaControl',
    'PhabricatorRemarkupCowsayBlockInterpreter' => 'PhutilRemarkupBlockInterpreter',
    'PhabricatorRemarkupCustomBlockRule' => 'PhutilRemarkupBlockRule',
    'PhabricatorRemarkupCustomInlineRule' => 'PhutilRemarkupRule',
    'PhabricatorRemarkupEditField' => 'PhabricatorEditField',
    'PhabricatorRemarkupFigletBlockInterpreter' => 'PhutilRemarkupBlockInterpreter',
    'PhabricatorRemarkupUIExample' => 'PhabricatorUIExample',
    'PhabricatorRepositoriesSetupCheck' => 'PhabricatorSetupCheck',
    'PhabricatorRepository' => array(
      'PhabricatorRepositoryDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorPolicyInterface',
      'PhabricatorFlaggableInterface',
      'PhabricatorMarkupInterface',
      'PhabricatorDestructibleInterface',
      'PhabricatorProjectInterface',
      'PhabricatorSpacesInterface',
    ),
    'PhabricatorRepositoryAuditRequest' => array(
      'PhabricatorRepositoryDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorRepositoryBranch' => 'PhabricatorRepositoryDAO',
    'PhabricatorRepositoryCommit' => array(
      'PhabricatorRepositoryDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorFlaggableInterface',
      'PhabricatorProjectInterface',
      'PhabricatorTokenReceiverInterface',
      'PhabricatorSubscribableInterface',
      'PhabricatorMentionableInterface',
      'HarbormasterBuildableInterface',
      'PhabricatorCustomFieldInterface',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorFulltextInterface',
    ),
    'PhabricatorRepositoryCommitChangeParserWorker' => 'PhabricatorRepositoryCommitParserWorker',
    'PhabricatorRepositoryCommitData' => 'PhabricatorRepositoryDAO',
    'PhabricatorRepositoryCommitHeraldWorker' => 'PhabricatorRepositoryCommitParserWorker',
    'PhabricatorRepositoryCommitMessageParserWorker' => 'PhabricatorRepositoryCommitParserWorker',
    'PhabricatorRepositoryCommitOwnersWorker' => 'PhabricatorRepositoryCommitParserWorker',
    'PhabricatorRepositoryCommitPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorRepositoryCommitParserWorker' => 'PhabricatorWorker',
    'PhabricatorRepositoryCommitRef' => 'Phobject',
    'PhabricatorRepositoryConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorRepositoryDAO' => 'PhabricatorLiskDAO',
    'PhabricatorRepositoryDiscoveryEngine' => 'PhabricatorRepositoryEngine',
    'PhabricatorRepositoryEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhabricatorRepositoryEngine' => 'Phobject',
    'PhabricatorRepositoryGitCommitChangeParserWorker' => 'PhabricatorRepositoryCommitChangeParserWorker',
    'PhabricatorRepositoryGitCommitMessageParserWorker' => 'PhabricatorRepositoryCommitMessageParserWorker',
    'PhabricatorRepositoryGraphCache' => 'Phobject',
    'PhabricatorRepositoryGraphStream' => 'Phobject',
    'PhabricatorRepositoryManagementCacheWorkflow' => 'PhabricatorRepositoryManagementWorkflow',
    'PhabricatorRepositoryManagementDiscoverWorkflow' => 'PhabricatorRepositoryManagementWorkflow',
    'PhabricatorRepositoryManagementEditWorkflow' => 'PhabricatorRepositoryManagementWorkflow',
    'PhabricatorRepositoryManagementImportingWorkflow' => 'PhabricatorRepositoryManagementWorkflow',
    'PhabricatorRepositoryManagementListPathsWorkflow' => 'PhabricatorRepositoryManagementWorkflow',
    'PhabricatorRepositoryManagementListWorkflow' => 'PhabricatorRepositoryManagementWorkflow',
    'PhabricatorRepositoryManagementLookupUsersWorkflow' => 'PhabricatorRepositoryManagementWorkflow',
    'PhabricatorRepositoryManagementMarkImportedWorkflow' => 'PhabricatorRepositoryManagementWorkflow',
    'PhabricatorRepositoryManagementMirrorWorkflow' => 'PhabricatorRepositoryManagementWorkflow',
    'PhabricatorRepositoryManagementMovePathsWorkflow' => 'PhabricatorRepositoryManagementWorkflow',
    'PhabricatorRepositoryManagementParentsWorkflow' => 'PhabricatorRepositoryManagementWorkflow',
    'PhabricatorRepositoryManagementPullWorkflow' => 'PhabricatorRepositoryManagementWorkflow',
    'PhabricatorRepositoryManagementRefsWorkflow' => 'PhabricatorRepositoryManagementWorkflow',
    'PhabricatorRepositoryManagementReparseWorkflow' => 'PhabricatorRepositoryManagementWorkflow',
    'PhabricatorRepositoryManagementUpdateWorkflow' => 'PhabricatorRepositoryManagementWorkflow',
    'PhabricatorRepositoryManagementWorkflow' => 'PhabricatorManagementWorkflow',
    'PhabricatorRepositoryMercurialCommitChangeParserWorker' => 'PhabricatorRepositoryCommitChangeParserWorker',
    'PhabricatorRepositoryMercurialCommitMessageParserWorker' => 'PhabricatorRepositoryCommitMessageParserWorker',
    'PhabricatorRepositoryMirror' => array(
      'PhabricatorRepositoryDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorRepositoryMirrorEngine' => 'PhabricatorRepositoryEngine',
    'PhabricatorRepositoryMirrorPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorRepositoryMirrorQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorRepositoryParsedChange' => 'Phobject',
    'PhabricatorRepositoryPullEngine' => 'PhabricatorRepositoryEngine',
    'PhabricatorRepositoryPullLocalDaemon' => 'PhabricatorDaemon',
    'PhabricatorRepositoryPushEvent' => array(
      'PhabricatorRepositoryDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorRepositoryPushEventPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorRepositoryPushEventQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorRepositoryPushLog' => array(
      'PhabricatorRepositoryDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorRepositoryPushLogPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorRepositoryPushLogQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorRepositoryPushLogSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorRepositoryPushMailWorker' => 'PhabricatorWorker',
    'PhabricatorRepositoryPushReplyHandler' => 'PhabricatorMailReplyHandler',
    'PhabricatorRepositoryQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorRepositoryRefCursor' => array(
      'PhabricatorRepositoryDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorRepositoryRefCursorPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorRepositoryRefCursorQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorRepositoryRefEngine' => 'PhabricatorRepositoryEngine',
    'PhabricatorRepositoryRepositoryPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorRepositorySchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'PhabricatorRepositorySearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorRepositoryStatusMessage' => 'PhabricatorRepositoryDAO',
    'PhabricatorRepositorySvnCommitChangeParserWorker' => 'PhabricatorRepositoryCommitChangeParserWorker',
    'PhabricatorRepositorySvnCommitMessageParserWorker' => 'PhabricatorRepositoryCommitMessageParserWorker',
    'PhabricatorRepositorySymbol' => 'PhabricatorRepositoryDAO',
    'PhabricatorRepositoryTestCase' => 'PhabricatorTestCase',
    'PhabricatorRepositoryTransaction' => 'PhabricatorApplicationTransaction',
    'PhabricatorRepositoryTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhabricatorRepositoryType' => 'Phobject',
    'PhabricatorRepositoryURIIndex' => 'PhabricatorRepositoryDAO',
    'PhabricatorRepositoryURINormalizer' => 'Phobject',
    'PhabricatorRepositoryURINormalizerTestCase' => 'PhabricatorTestCase',
    'PhabricatorRepositoryURITestCase' => 'PhabricatorTestCase',
    'PhabricatorRepositoryVCSPassword' => 'PhabricatorRepositoryDAO',
    'PhabricatorRepositoryVersion' => 'Phobject',
    'PhabricatorRequestExceptionHandler' => 'AphrontRequestExceptionHandler',
    'PhabricatorResourceSite' => 'PhabricatorSite',
    'PhabricatorRobotsController' => 'PhabricatorController',
    'PhabricatorS3FileStorageEngine' => 'PhabricatorFileStorageEngine',
    'PhabricatorSMS' => 'PhabricatorSMSDAO',
    'PhabricatorSMSConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorSMSDAO' => 'PhabricatorLiskDAO',
    'PhabricatorSMSDemultiplexWorker' => 'PhabricatorSMSWorker',
    'PhabricatorSMSImplementationAdapter' => 'Phobject',
    'PhabricatorSMSImplementationTestBlackholeAdapter' => 'PhabricatorSMSImplementationAdapter',
    'PhabricatorSMSImplementationTwilioAdapter' => 'PhabricatorSMSImplementationAdapter',
    'PhabricatorSMSManagementListOutboundWorkflow' => 'PhabricatorSMSManagementWorkflow',
    'PhabricatorSMSManagementSendTestWorkflow' => 'PhabricatorSMSManagementWorkflow',
    'PhabricatorSMSManagementShowOutboundWorkflow' => 'PhabricatorSMSManagementWorkflow',
    'PhabricatorSMSManagementWorkflow' => 'PhabricatorManagementWorkflow',
    'PhabricatorSMSSendWorker' => 'PhabricatorSMSWorker',
    'PhabricatorSMSWorker' => 'PhabricatorWorker',
    'PhabricatorSQLPatchList' => 'Phobject',
    'PhabricatorSSHKeyGenerator' => 'Phobject',
    'PhabricatorSSHKeysSettingsPanel' => 'PhabricatorSettingsPanel',
    'PhabricatorSSHLog' => 'Phobject',
    'PhabricatorSSHPassthruCommand' => 'Phobject',
    'PhabricatorSSHWorkflow' => 'PhabricatorManagementWorkflow',
    'PhabricatorSavedQuery' => array(
      'PhabricatorSearchDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorSavedQueryQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorScheduleTaskTriggerAction' => 'PhabricatorTriggerAction',
    'PhabricatorScopedEnv' => 'Phobject',
    'PhabricatorSearchAbstractDocument' => 'Phobject',
    'PhabricatorSearchApplication' => 'PhabricatorApplication',
    'PhabricatorSearchApplicationSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorSearchApplicationStorageEnginePanel' => 'PhabricatorApplicationConfigurationPanel',
    'PhabricatorSearchAttachController' => 'PhabricatorSearchBaseController',
    'PhabricatorSearchBaseController' => 'PhabricatorController',
    'PhabricatorSearchCheckboxesField' => 'PhabricatorSearchField',
    'PhabricatorSearchConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorSearchController' => 'PhabricatorSearchBaseController',
    'PhabricatorSearchCustomFieldProxyField' => 'PhabricatorSearchField',
    'PhabricatorSearchDAO' => 'PhabricatorLiskDAO',
    'PhabricatorSearchDatasource' => 'PhabricatorTypeaheadCompositeDatasource',
    'PhabricatorSearchDatasourceField' => 'PhabricatorSearchTokenizerField',
    'PhabricatorSearchDateControlField' => 'PhabricatorSearchField',
    'PhabricatorSearchDateField' => 'PhabricatorSearchField',
    'PhabricatorSearchDeleteController' => 'PhabricatorSearchBaseController',
    'PhabricatorSearchDocument' => 'PhabricatorSearchDAO',
    'PhabricatorSearchDocumentField' => 'PhabricatorSearchDAO',
    'PhabricatorSearchDocumentFieldType' => 'Phobject',
    'PhabricatorSearchDocumentQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorSearchDocumentRelationship' => 'PhabricatorSearchDAO',
    'PhabricatorSearchDocumentTypeDatasource' => 'PhabricatorTypeaheadDatasource',
    'PhabricatorSearchEditController' => 'PhabricatorSearchBaseController',
    'PhabricatorSearchEngineAPIMethod' => 'ConduitAPIMethod',
    'PhabricatorSearchEngineAttachment' => 'Phobject',
    'PhabricatorSearchEngineExtension' => 'Phobject',
    'PhabricatorSearchEngineExtensionModule' => 'PhabricatorConfigModule',
    'PhabricatorSearchEngineTestCase' => 'PhabricatorTestCase',
    'PhabricatorSearchField' => 'Phobject',
    'PhabricatorSearchHovercardController' => 'PhabricatorSearchBaseController',
    'PhabricatorSearchIndexVersion' => 'PhabricatorSearchDAO',
    'PhabricatorSearchIndexVersionDestructionEngineExtension' => 'PhabricatorDestructionEngineExtension',
    'PhabricatorSearchManagementIndexWorkflow' => 'PhabricatorSearchManagementWorkflow',
    'PhabricatorSearchManagementInitWorkflow' => 'PhabricatorSearchManagementWorkflow',
    'PhabricatorSearchManagementWorkflow' => 'PhabricatorManagementWorkflow',
    'PhabricatorSearchNgrams' => 'PhabricatorSearchDAO',
    'PhabricatorSearchNgramsDestructionEngineExtension' => 'PhabricatorDestructionEngineExtension',
    'PhabricatorSearchOrderController' => 'PhabricatorSearchBaseController',
    'PhabricatorSearchOrderField' => 'PhabricatorSearchField',
    'PhabricatorSearchPreferencesSettingsPanel' => 'PhabricatorSettingsPanel',
    'PhabricatorSearchRelationship' => 'Phobject',
    'PhabricatorSearchResultView' => 'AphrontView',
    'PhabricatorSearchSelectController' => 'PhabricatorSearchBaseController',
    'PhabricatorSearchSelectField' => 'PhabricatorSearchField',
    'PhabricatorSearchStringListField' => 'PhabricatorSearchField',
    'PhabricatorSearchSubscribersField' => 'PhabricatorSearchTokenizerField',
    'PhabricatorSearchTextField' => 'PhabricatorSearchField',
    'PhabricatorSearchThreeStateField' => 'PhabricatorSearchField',
    'PhabricatorSearchTokenizerField' => 'PhabricatorSearchField',
    'PhabricatorSearchWorker' => 'PhabricatorWorker',
    'PhabricatorSecurityConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorSecuritySetupCheck' => 'PhabricatorSetupCheck',
    'PhabricatorSelectEditField' => 'PhabricatorEditField',
    'PhabricatorSendGridConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorSessionsSettingsPanel' => 'PhabricatorSettingsPanel',
    'PhabricatorSettingsAddEmailAction' => 'PhabricatorSystemAction',
    'PhabricatorSettingsAdjustController' => 'PhabricatorController',
    'PhabricatorSettingsApplication' => 'PhabricatorApplication',
    'PhabricatorSettingsMainController' => 'PhabricatorController',
    'PhabricatorSettingsMainMenuBarExtension' => 'PhabricatorMainMenuBarExtension',
    'PhabricatorSettingsPanel' => 'Phobject',
    'PhabricatorSetupCheck' => 'Phobject',
    'PhabricatorSetupCheckTestCase' => 'PhabricatorTestCase',
    'PhabricatorSetupIssue' => 'Phobject',
    'PhabricatorSetupIssueUIExample' => 'PhabricatorUIExample',
    'PhabricatorSetupIssueView' => 'AphrontView',
    'PhabricatorShortSite' => 'PhabricatorSite',
    'PhabricatorSimpleEditType' => 'PhabricatorEditType',
    'PhabricatorSite' => 'AphrontSite',
    'PhabricatorSlowvoteApplication' => 'PhabricatorApplication',
    'PhabricatorSlowvoteChoice' => 'PhabricatorSlowvoteDAO',
    'PhabricatorSlowvoteCloseController' => 'PhabricatorSlowvoteController',
    'PhabricatorSlowvoteCommentController' => 'PhabricatorSlowvoteController',
    'PhabricatorSlowvoteController' => 'PhabricatorController',
    'PhabricatorSlowvoteDAO' => 'PhabricatorLiskDAO',
    'PhabricatorSlowvoteDefaultViewCapability' => 'PhabricatorPolicyCapability',
    'PhabricatorSlowvoteEditController' => 'PhabricatorSlowvoteController',
    'PhabricatorSlowvoteEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhabricatorSlowvoteListController' => 'PhabricatorSlowvoteController',
    'PhabricatorSlowvoteMailReceiver' => 'PhabricatorObjectMailReceiver',
    'PhabricatorSlowvoteOption' => 'PhabricatorSlowvoteDAO',
    'PhabricatorSlowvotePoll' => array(
      'PhabricatorSlowvoteDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorPolicyInterface',
      'PhabricatorSubscribableInterface',
      'PhabricatorFlaggableInterface',
      'PhabricatorTokenReceiverInterface',
      'PhabricatorProjectInterface',
      'PhabricatorDestructibleInterface',
      'PhabricatorSpacesInterface',
    ),
    'PhabricatorSlowvotePollController' => 'PhabricatorSlowvoteController',
    'PhabricatorSlowvotePollPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorSlowvoteQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorSlowvoteReplyHandler' => 'PhabricatorApplicationTransactionReplyHandler',
    'PhabricatorSlowvoteSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'PhabricatorSlowvoteSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorSlowvoteTransaction' => 'PhabricatorApplicationTransaction',
    'PhabricatorSlowvoteTransactionComment' => 'PhabricatorApplicationTransactionComment',
    'PhabricatorSlowvoteTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhabricatorSlowvoteVoteController' => 'PhabricatorSlowvoteController',
    'PhabricatorSlug' => 'Phobject',
    'PhabricatorSlugTestCase' => 'PhabricatorTestCase',
    'PhabricatorSortTableUIExample' => 'PhabricatorUIExample',
    'PhabricatorSourceCodeView' => 'AphrontView',
    'PhabricatorSpaceEditField' => 'PhabricatorEditField',
    'PhabricatorSpacesApplication' => 'PhabricatorApplication',
    'PhabricatorSpacesArchiveController' => 'PhabricatorSpacesController',
    'PhabricatorSpacesCapabilityCreateSpaces' => 'PhabricatorPolicyCapability',
    'PhabricatorSpacesCapabilityDefaultEdit' => 'PhabricatorPolicyCapability',
    'PhabricatorSpacesCapabilityDefaultView' => 'PhabricatorPolicyCapability',
    'PhabricatorSpacesController' => 'PhabricatorController',
    'PhabricatorSpacesDAO' => 'PhabricatorLiskDAO',
    'PhabricatorSpacesEditController' => 'PhabricatorSpacesController',
    'PhabricatorSpacesInterface' => 'PhabricatorPHIDInterface',
    'PhabricatorSpacesListController' => 'PhabricatorSpacesController',
    'PhabricatorSpacesNamespace' => array(
      'PhabricatorSpacesDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorDestructibleInterface',
    ),
    'PhabricatorSpacesNamespaceDatasource' => 'PhabricatorTypeaheadDatasource',
    'PhabricatorSpacesNamespaceEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhabricatorSpacesNamespacePHIDType' => 'PhabricatorPHIDType',
    'PhabricatorSpacesNamespaceQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorSpacesNamespaceSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorSpacesNamespaceTransaction' => 'PhabricatorApplicationTransaction',
    'PhabricatorSpacesNamespaceTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhabricatorSpacesNoAccessController' => 'PhabricatorSpacesController',
    'PhabricatorSpacesRemarkupRule' => 'PhabricatorObjectRemarkupRule',
    'PhabricatorSpacesSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'PhabricatorSpacesSearchEngineExtension' => 'PhabricatorSearchEngineExtension',
    'PhabricatorSpacesSearchField' => 'PhabricatorSearchTokenizerField',
    'PhabricatorSpacesTestCase' => 'PhabricatorTestCase',
    'PhabricatorSpacesViewController' => 'PhabricatorSpacesController',
    'PhabricatorStandardCustomField' => 'PhabricatorCustomField',
    'PhabricatorStandardCustomFieldBlueprints' => 'PhabricatorStandardCustomFieldTokenizer',
    'PhabricatorStandardCustomFieldBool' => 'PhabricatorStandardCustomField',
    'PhabricatorStandardCustomFieldCredential' => 'PhabricatorStandardCustomField',
    'PhabricatorStandardCustomFieldDatasource' => 'PhabricatorStandardCustomFieldTokenizer',
    'PhabricatorStandardCustomFieldDate' => 'PhabricatorStandardCustomField',
    'PhabricatorStandardCustomFieldHeader' => 'PhabricatorStandardCustomField',
    'PhabricatorStandardCustomFieldInt' => 'PhabricatorStandardCustomField',
    'PhabricatorStandardCustomFieldLink' => 'PhabricatorStandardCustomField',
    'PhabricatorStandardCustomFieldPHIDs' => 'PhabricatorStandardCustomField',
    'PhabricatorStandardCustomFieldRemarkup' => 'PhabricatorStandardCustomField',
    'PhabricatorStandardCustomFieldSelect' => 'PhabricatorStandardCustomField',
    'PhabricatorStandardCustomFieldText' => 'PhabricatorStandardCustomField',
    'PhabricatorStandardCustomFieldTokenizer' => 'PhabricatorStandardCustomFieldPHIDs',
    'PhabricatorStandardCustomFieldUsers' => 'PhabricatorStandardCustomFieldTokenizer',
    'PhabricatorStandardPageView' => array(
      'PhabricatorBarePageView',
      'AphrontResponseProducerInterface',
    ),
    'PhabricatorStandardSelectCustomFieldDatasource' => 'PhabricatorTypeaheadDatasource',
    'PhabricatorStatusController' => 'PhabricatorController',
    'PhabricatorStatusUIExample' => 'PhabricatorUIExample',
    'PhabricatorStorageFixtureScopeGuard' => 'Phobject',
    'PhabricatorStorageManagementAPI' => 'Phobject',
    'PhabricatorStorageManagementAdjustWorkflow' => 'PhabricatorStorageManagementWorkflow',
    'PhabricatorStorageManagementDatabasesWorkflow' => 'PhabricatorStorageManagementWorkflow',
    'PhabricatorStorageManagementDestroyWorkflow' => 'PhabricatorStorageManagementWorkflow',
    'PhabricatorStorageManagementDumpWorkflow' => 'PhabricatorStorageManagementWorkflow',
    'PhabricatorStorageManagementProbeWorkflow' => 'PhabricatorStorageManagementWorkflow',
    'PhabricatorStorageManagementQuickstartWorkflow' => 'PhabricatorStorageManagementWorkflow',
    'PhabricatorStorageManagementRenamespaceWorkflow' => 'PhabricatorStorageManagementWorkflow',
    'PhabricatorStorageManagementShellWorkflow' => 'PhabricatorStorageManagementWorkflow',
    'PhabricatorStorageManagementStatusWorkflow' => 'PhabricatorStorageManagementWorkflow',
    'PhabricatorStorageManagementUpgradeWorkflow' => 'PhabricatorStorageManagementWorkflow',
    'PhabricatorStorageManagementWorkflow' => 'PhabricatorManagementWorkflow',
    'PhabricatorStoragePatch' => 'Phobject',
    'PhabricatorStorageSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'PhabricatorStorageSetupCheck' => 'PhabricatorSetupCheck',
    'PhabricatorStreamingProtocolAdapter' => 'PhabricatorProtocolAdapter',
    'PhabricatorStringListEditField' => 'PhabricatorEditField',
    'PhabricatorSubscribedToObjectEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorSubscribersEditField' => 'PhabricatorTokenizerEditField',
    'PhabricatorSubscribersQuery' => 'PhabricatorQuery',
    'PhabricatorSubscriptionTriggerClock' => 'PhabricatorTriggerClock',
    'PhabricatorSubscriptionsAddSelfHeraldAction' => 'PhabricatorSubscriptionsHeraldAction',
    'PhabricatorSubscriptionsAddSubscribersHeraldAction' => 'PhabricatorSubscriptionsHeraldAction',
    'PhabricatorSubscriptionsApplication' => 'PhabricatorApplication',
    'PhabricatorSubscriptionsEditController' => 'PhabricatorController',
    'PhabricatorSubscriptionsEditEngineExtension' => 'PhabricatorEditEngineExtension',
    'PhabricatorSubscriptionsEditor' => 'PhabricatorEditor',
    'PhabricatorSubscriptionsFulltextEngineExtension' => 'PhabricatorFulltextEngineExtension',
    'PhabricatorSubscriptionsHeraldAction' => 'HeraldAction',
    'PhabricatorSubscriptionsListController' => 'PhabricatorController',
    'PhabricatorSubscriptionsRemoveSelfHeraldAction' => 'PhabricatorSubscriptionsHeraldAction',
    'PhabricatorSubscriptionsRemoveSubscribersHeraldAction' => 'PhabricatorSubscriptionsHeraldAction',
    'PhabricatorSubscriptionsSearchEngineAttachment' => 'PhabricatorSearchEngineAttachment',
    'PhabricatorSubscriptionsSearchEngineExtension' => 'PhabricatorSearchEngineExtension',
    'PhabricatorSubscriptionsSubscribeEmailCommand' => 'MetaMTAEmailTransactionCommand',
    'PhabricatorSubscriptionsSubscribersPolicyRule' => 'PhabricatorPolicyRule',
    'PhabricatorSubscriptionsTransactionController' => 'PhabricatorController',
    'PhabricatorSubscriptionsUIEventListener' => 'PhabricatorEventListener',
    'PhabricatorSubscriptionsUnsubscribeEmailCommand' => 'MetaMTAEmailTransactionCommand',
    'PhabricatorSupportApplication' => 'PhabricatorApplication',
    'PhabricatorSyntaxHighlighter' => 'Phobject',
    'PhabricatorSyntaxHighlightingConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorSystemAction' => 'Phobject',
    'PhabricatorSystemActionEngine' => 'Phobject',
    'PhabricatorSystemActionGarbageCollector' => 'PhabricatorGarbageCollector',
    'PhabricatorSystemActionLog' => 'PhabricatorSystemDAO',
    'PhabricatorSystemActionRateLimitException' => 'Exception',
    'PhabricatorSystemApplication' => 'PhabricatorApplication',
    'PhabricatorSystemDAO' => 'PhabricatorLiskDAO',
    'PhabricatorSystemDestructionGarbageCollector' => 'PhabricatorGarbageCollector',
    'PhabricatorSystemDestructionLog' => 'PhabricatorSystemDAO',
    'PhabricatorSystemRemoveDestroyWorkflow' => 'PhabricatorSystemRemoveWorkflow',
    'PhabricatorSystemRemoveLogWorkflow' => 'PhabricatorSystemRemoveWorkflow',
    'PhabricatorSystemRemoveWorkflow' => 'PhabricatorManagementWorkflow',
    'PhabricatorSystemSelectEncodingController' => 'PhabricatorController',
    'PhabricatorSystemSelectHighlightController' => 'PhabricatorController',
    'PhabricatorTOTPAuthFactor' => 'PhabricatorAuthFactor',
    'PhabricatorTOTPAuthFactorTestCase' => 'PhabricatorTestCase',
    'PhabricatorTaskmasterDaemon' => 'PhabricatorDaemon',
    'PhabricatorTestApplication' => 'PhabricatorApplication',
    'PhabricatorTestCase' => 'PhutilTestCase',
    'PhabricatorTestController' => 'PhabricatorController',
    'PhabricatorTestDataGenerator' => 'Phobject',
    'PhabricatorTestNoCycleEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorTestStorageEngine' => 'PhabricatorFileStorageEngine',
    'PhabricatorTestWorker' => 'PhabricatorWorker',
    'PhabricatorTextAreaEditField' => 'PhabricatorEditField',
    'PhabricatorTextEditField' => 'PhabricatorEditField',
    'PhabricatorTime' => 'Phobject',
    'PhabricatorTimeGuard' => 'Phobject',
    'PhabricatorTimeTestCase' => 'PhabricatorTestCase',
    'PhabricatorTimezoneSetupCheck' => 'PhabricatorSetupCheck',
    'PhabricatorToken' => array(
      'PhabricatorTokenDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorTokenController' => 'PhabricatorController',
    'PhabricatorTokenCount' => 'PhabricatorTokenDAO',
    'PhabricatorTokenCountQuery' => 'PhabricatorOffsetPagedQuery',
    'PhabricatorTokenDAO' => 'PhabricatorLiskDAO',
    'PhabricatorTokenDestructionEngineExtension' => 'PhabricatorDestructionEngineExtension',
    'PhabricatorTokenGiveController' => 'PhabricatorTokenController',
    'PhabricatorTokenGiven' => array(
      'PhabricatorTokenDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorTokenGivenController' => 'PhabricatorTokenController',
    'PhabricatorTokenGivenEditor' => 'PhabricatorEditor',
    'PhabricatorTokenGivenFeedStory' => 'PhabricatorFeedStory',
    'PhabricatorTokenGivenQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorTokenLeaderController' => 'PhabricatorTokenController',
    'PhabricatorTokenQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorTokenReceiverQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorTokenTokenPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorTokenUIEventListener' => 'PhabricatorEventListener',
    'PhabricatorTokenizerEditField' => 'PhabricatorPHIDListEditField',
    'PhabricatorTokensApplication' => 'PhabricatorApplication',
    'PhabricatorTokensSettingsPanel' => 'PhabricatorSettingsPanel',
    'PhabricatorTooltipUIExample' => 'PhabricatorUIExample',
    'PhabricatorTransactions' => 'Phobject',
    'PhabricatorTransactionsApplication' => 'PhabricatorApplication',
    'PhabricatorTransactionsDestructionEngineExtension' => 'PhabricatorDestructionEngineExtension',
    'PhabricatorTransactionsFulltextEngineExtension' => 'PhabricatorFulltextEngineExtension',
    'PhabricatorTransformedFile' => 'PhabricatorFileDAO',
    'PhabricatorTranslationsConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorTriggerAction' => 'Phobject',
    'PhabricatorTriggerClock' => 'Phobject',
    'PhabricatorTriggerClockTestCase' => 'PhabricatorTestCase',
    'PhabricatorTriggerDaemon' => 'PhabricatorDaemon',
    'PhabricatorTrivialTestCase' => 'PhabricatorTestCase',
    'PhabricatorTwitchAuthProvider' => 'PhabricatorOAuth2AuthProvider',
    'PhabricatorTwitterAuthProvider' => 'PhabricatorOAuth1AuthProvider',
    'PhabricatorTypeaheadApplication' => 'PhabricatorApplication',
    'PhabricatorTypeaheadCompositeDatasource' => 'PhabricatorTypeaheadDatasource',
    'PhabricatorTypeaheadDatasource' => 'Phobject',
    'PhabricatorTypeaheadDatasourceController' => 'PhabricatorController',
    'PhabricatorTypeaheadFunctionHelpController' => 'PhabricatorTypeaheadDatasourceController',
    'PhabricatorTypeaheadInvalidTokenException' => 'Exception',
    'PhabricatorTypeaheadModularDatasourceController' => 'PhabricatorTypeaheadDatasourceController',
    'PhabricatorTypeaheadMonogramDatasource' => 'PhabricatorTypeaheadDatasource',
    'PhabricatorTypeaheadResult' => 'Phobject',
    'PhabricatorTypeaheadRuntimeCompositeDatasource' => 'PhabricatorTypeaheadCompositeDatasource',
    'PhabricatorTypeaheadTokenView' => 'AphrontTagView',
    'PhabricatorUIConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorUIExample' => 'Phobject',
    'PhabricatorUIExampleRenderController' => 'PhabricatorController',
    'PhabricatorUIExamplesApplication' => 'PhabricatorApplication',
    'PhabricatorUSEnglishTranslation' => 'PhutilTranslation',
    'PhabricatorUnitsTestCase' => 'PhabricatorTestCase',
    'PhabricatorUnsubscribedFromObjectEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorUser' => array(
      'PhabricatorUserDAO',
      'PhutilPerson',
      'PhabricatorPolicyInterface',
      'PhabricatorCustomFieldInterface',
      'PhabricatorDestructibleInterface',
      'PhabricatorSSHPublicKeyInterface',
      'PhabricatorFlaggableInterface',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorFulltextInterface',
    ),
    'PhabricatorUserBlurbField' => 'PhabricatorUserCustomField',
    'PhabricatorUserConfigOptions' => 'PhabricatorApplicationConfigOptions',
    'PhabricatorUserConfiguredCustomField' => array(
      'PhabricatorUserCustomField',
      'PhabricatorStandardCustomFieldInterface',
    ),
    'PhabricatorUserConfiguredCustomFieldStorage' => 'PhabricatorCustomFieldStorage',
    'PhabricatorUserCustomField' => 'PhabricatorCustomField',
    'PhabricatorUserCustomFieldNumericIndex' => 'PhabricatorCustomFieldNumericIndexStorage',
    'PhabricatorUserCustomFieldStringIndex' => 'PhabricatorCustomFieldStringIndexStorage',
    'PhabricatorUserDAO' => 'PhabricatorLiskDAO',
    'PhabricatorUserEditor' => 'PhabricatorEditor',
    'PhabricatorUserEditorTestCase' => 'PhabricatorTestCase',
    'PhabricatorUserEmail' => 'PhabricatorUserDAO',
    'PhabricatorUserEmailTestCase' => 'PhabricatorTestCase',
    'PhabricatorUserFulltextEngine' => 'PhabricatorFulltextEngine',
    'PhabricatorUserLog' => array(
      'PhabricatorUserDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorUserLogView' => 'AphrontView',
    'PhabricatorUserPHIDResolver' => 'PhabricatorPHIDResolver',
    'PhabricatorUserPreferences' => 'PhabricatorUserDAO',
    'PhabricatorUserProfile' => 'PhabricatorUserDAO',
    'PhabricatorUserProfileEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhabricatorUserRealNameField' => 'PhabricatorUserCustomField',
    'PhabricatorUserRolesField' => 'PhabricatorUserCustomField',
    'PhabricatorUserSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'PhabricatorUserSinceField' => 'PhabricatorUserCustomField',
    'PhabricatorUserStatusField' => 'PhabricatorUserCustomField',
    'PhabricatorUserTestCase' => 'PhabricatorTestCase',
    'PhabricatorUserTitleField' => 'PhabricatorUserCustomField',
    'PhabricatorUserTransaction' => 'PhabricatorApplicationTransaction',
    'PhabricatorUsersEditField' => 'PhabricatorTokenizerEditField',
    'PhabricatorUsersPolicyRule' => 'PhabricatorPolicyRule',
    'PhabricatorUsersSearchField' => 'PhabricatorSearchTokenizerField',
    'PhabricatorVCSResponse' => 'AphrontResponse',
    'PhabricatorVersionedDraft' => 'PhabricatorDraftDAO',
    'PhabricatorVeryWowEnglishTranslation' => 'PhutilTranslation',
    'PhabricatorViewerDatasource' => 'PhabricatorTypeaheadDatasource',
    'PhabricatorWatcherHasObjectEdgeType' => 'PhabricatorEdgeType',
    'PhabricatorWordPressAuthProvider' => 'PhabricatorOAuth2AuthProvider',
    'PhabricatorWorker' => 'Phobject',
    'PhabricatorWorkerActiveTask' => 'PhabricatorWorkerTask',
    'PhabricatorWorkerArchiveTask' => 'PhabricatorWorkerTask',
    'PhabricatorWorkerArchiveTaskQuery' => 'PhabricatorQuery',
    'PhabricatorWorkerBulkJob' => array(
      'PhabricatorWorkerDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorSubscribableInterface',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorDestructibleInterface',
    ),
    'PhabricatorWorkerBulkJobCreateWorker' => 'PhabricatorWorkerBulkJobWorker',
    'PhabricatorWorkerBulkJobEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhabricatorWorkerBulkJobPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorWorkerBulkJobQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhabricatorWorkerBulkJobSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhabricatorWorkerBulkJobTaskWorker' => 'PhabricatorWorkerBulkJobWorker',
    'PhabricatorWorkerBulkJobTestCase' => 'PhabricatorTestCase',
    'PhabricatorWorkerBulkJobTransaction' => 'PhabricatorApplicationTransaction',
    'PhabricatorWorkerBulkJobTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhabricatorWorkerBulkJobType' => 'Phobject',
    'PhabricatorWorkerBulkJobWorker' => 'PhabricatorWorker',
    'PhabricatorWorkerBulkTask' => 'PhabricatorWorkerDAO',
    'PhabricatorWorkerDAO' => 'PhabricatorLiskDAO',
    'PhabricatorWorkerDestructionEngineExtension' => 'PhabricatorDestructionEngineExtension',
    'PhabricatorWorkerLeaseQuery' => 'PhabricatorQuery',
    'PhabricatorWorkerManagementCancelWorkflow' => 'PhabricatorWorkerManagementWorkflow',
    'PhabricatorWorkerManagementExecuteWorkflow' => 'PhabricatorWorkerManagementWorkflow',
    'PhabricatorWorkerManagementFloodWorkflow' => 'PhabricatorWorkerManagementWorkflow',
    'PhabricatorWorkerManagementFreeWorkflow' => 'PhabricatorWorkerManagementWorkflow',
    'PhabricatorWorkerManagementRetryWorkflow' => 'PhabricatorWorkerManagementWorkflow',
    'PhabricatorWorkerManagementWorkflow' => 'PhabricatorManagementWorkflow',
    'PhabricatorWorkerPermanentFailureException' => 'Exception',
    'PhabricatorWorkerSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'PhabricatorWorkerTask' => 'PhabricatorWorkerDAO',
    'PhabricatorWorkerTaskData' => 'PhabricatorWorkerDAO',
    'PhabricatorWorkerTaskDetailController' => 'PhabricatorDaemonController',
    'PhabricatorWorkerTestCase' => 'PhabricatorTestCase',
    'PhabricatorWorkerTrigger' => array(
      'PhabricatorWorkerDAO',
      'PhabricatorDestructibleInterface',
      'PhabricatorPolicyInterface',
    ),
    'PhabricatorWorkerTriggerEvent' => 'PhabricatorWorkerDAO',
    'PhabricatorWorkerTriggerManagementFireWorkflow' => 'PhabricatorWorkerTriggerManagementWorkflow',
    'PhabricatorWorkerTriggerManagementWorkflow' => 'PhabricatorManagementWorkflow',
    'PhabricatorWorkerTriggerPHIDType' => 'PhabricatorPHIDType',
    'PhabricatorWorkerTriggerQuery' => 'PhabricatorPolicyAwareQuery',
    'PhabricatorWorkerYieldException' => 'Exception',
    'PhabricatorWorkingCopyDiscoveryTestCase' => 'PhabricatorWorkingCopyTestCase',
    'PhabricatorWorkingCopyPullTestCase' => 'PhabricatorWorkingCopyTestCase',
    'PhabricatorWorkingCopyTestCase' => 'PhabricatorTestCase',
    'PhabricatorXHPASTDAO' => 'PhabricatorLiskDAO',
    'PhabricatorXHPASTParseTree' => 'PhabricatorXHPASTDAO',
    'PhabricatorXHPASTViewController' => 'PhabricatorController',
    'PhabricatorXHPASTViewFrameController' => 'PhabricatorXHPASTViewController',
    'PhabricatorXHPASTViewFramesetController' => 'PhabricatorXHPASTViewController',
    'PhabricatorXHPASTViewInputController' => 'PhabricatorXHPASTViewPanelController',
    'PhabricatorXHPASTViewPanelController' => 'PhabricatorXHPASTViewController',
    'PhabricatorXHPASTViewRunController' => 'PhabricatorXHPASTViewController',
    'PhabricatorXHPASTViewStreamController' => 'PhabricatorXHPASTViewPanelController',
    'PhabricatorXHPASTViewTreeController' => 'PhabricatorXHPASTViewPanelController',
    'PhabricatorXHProfApplication' => 'PhabricatorApplication',
    'PhabricatorXHProfController' => 'PhabricatorController',
    'PhabricatorXHProfDAO' => 'PhabricatorLiskDAO',
    'PhabricatorXHProfProfileController' => 'PhabricatorXHProfController',
    'PhabricatorXHProfProfileSymbolView' => 'PhabricatorXHProfProfileView',
    'PhabricatorXHProfProfileTopLevelView' => 'PhabricatorXHProfProfileView',
    'PhabricatorXHProfProfileView' => 'AphrontView',
    'PhabricatorXHProfSample' => 'PhabricatorXHProfDAO',
    'PhabricatorXHProfSampleListController' => 'PhabricatorXHProfController',
    'PhabricatorYoutubeRemarkupRule' => 'PhutilRemarkupRule',
    'PhameBlog' => array(
      'PhameDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorMarkupInterface',
      'PhabricatorSubscribableInterface',
      'PhabricatorFlaggableInterface',
      'PhabricatorProjectInterface',
      'PhabricatorDestructibleInterface',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorConduitResultInterface',
    ),
    'PhameBlogArchiveController' => 'PhameBlogController',
    'PhameBlogController' => 'PhameController',
    'PhameBlogCreateCapability' => 'PhabricatorPolicyCapability',
    'PhameBlogEditConduitAPIMethod' => 'PhabricatorEditEngineAPIMethod',
    'PhameBlogEditController' => 'PhameBlogController',
    'PhameBlogEditEngine' => 'PhabricatorEditEngine',
    'PhameBlogEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhameBlogFeedController' => 'PhameBlogController',
    'PhameBlogListController' => 'PhameBlogController',
    'PhameBlogListView' => 'AphrontTagView',
    'PhameBlogManageController' => 'PhameBlogController',
    'PhameBlogProfilePictureController' => 'PhameBlogController',
    'PhameBlogQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhameBlogReplyHandler' => 'PhabricatorApplicationTransactionReplyHandler',
    'PhameBlogSearchConduitAPIMethod' => 'PhabricatorSearchEngineAPIMethod',
    'PhameBlogSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhameBlogSite' => 'PhameSite',
    'PhameBlogTransaction' => 'PhabricatorApplicationTransaction',
    'PhameBlogTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhameBlogViewController' => 'PhameLiveController',
    'PhameConstants' => 'Phobject',
    'PhameController' => 'PhabricatorController',
    'PhameDAO' => 'PhabricatorLiskDAO',
    'PhameDescriptionView' => 'AphrontTagView',
    'PhameDraftListView' => 'AphrontTagView',
    'PhameHomeController' => 'PhamePostController',
    'PhameLiveController' => 'PhameController',
    'PhameNextPostView' => 'AphrontTagView',
    'PhamePost' => array(
      'PhameDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorMarkupInterface',
      'PhabricatorFlaggableInterface',
      'PhabricatorProjectInterface',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorSubscribableInterface',
      'PhabricatorDestructibleInterface',
      'PhabricatorTokenReceiverInterface',
      'PhabricatorConduitResultInterface',
    ),
    'PhamePostCommentController' => 'PhamePostController',
    'PhamePostController' => 'PhameController',
    'PhamePostEditConduitAPIMethod' => 'PhabricatorEditEngineAPIMethod',
    'PhamePostEditController' => 'PhamePostController',
    'PhamePostEditEngine' => 'PhabricatorEditEngine',
    'PhamePostEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhamePostHistoryController' => 'PhamePostController',
    'PhamePostListController' => 'PhamePostController',
    'PhamePostListView' => 'AphrontTagView',
    'PhamePostMailReceiver' => 'PhabricatorObjectMailReceiver',
    'PhamePostMoveController' => 'PhamePostController',
    'PhamePostPublishController' => 'PhamePostController',
    'PhamePostQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhamePostReplyHandler' => 'PhabricatorApplicationTransactionReplyHandler',
    'PhamePostSearchConduitAPIMethod' => 'PhabricatorSearchEngineAPIMethod',
    'PhamePostSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhamePostTransaction' => 'PhabricatorApplicationTransaction',
    'PhamePostTransactionComment' => 'PhabricatorApplicationTransactionComment',
    'PhamePostTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhamePostViewController' => 'PhameLiveController',
    'PhameSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'PhameSite' => 'PhabricatorSite',
    'PhluxController' => 'PhabricatorController',
    'PhluxDAO' => 'PhabricatorLiskDAO',
    'PhluxEditController' => 'PhluxController',
    'PhluxListController' => 'PhluxController',
    'PhluxTransaction' => 'PhabricatorApplicationTransaction',
    'PhluxTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhluxVariable' => array(
      'PhluxDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorFlaggableInterface',
      'PhabricatorPolicyInterface',
    ),
    'PhluxVariableEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhluxVariablePHIDType' => 'PhabricatorPHIDType',
    'PhluxVariableQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhluxViewController' => 'PhluxController',
    'PholioActionMenuEventListener' => 'PhabricatorEventListener',
    'PholioController' => 'PhabricatorController',
    'PholioDAO' => 'PhabricatorLiskDAO',
    'PholioDefaultEditCapability' => 'PhabricatorPolicyCapability',
    'PholioDefaultViewCapability' => 'PhabricatorPolicyCapability',
    'PholioImage' => array(
      'PholioDAO',
      'PhabricatorMarkupInterface',
      'PhabricatorPolicyInterface',
    ),
    'PholioImagePHIDType' => 'PhabricatorPHIDType',
    'PholioImageQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PholioImageUploadController' => 'PholioController',
    'PholioInlineController' => 'PholioController',
    'PholioInlineListController' => 'PholioController',
    'PholioMock' => array(
      'PholioDAO',
      'PhabricatorMarkupInterface',
      'PhabricatorPolicyInterface',
      'PhabricatorSubscribableInterface',
      'PhabricatorTokenReceiverInterface',
      'PhabricatorFlaggableInterface',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorProjectInterface',
      'PhabricatorDestructibleInterface',
      'PhabricatorSpacesInterface',
      'PhabricatorMentionableInterface',
      'PhabricatorFulltextInterface',
    ),
    'PholioMockArchiveController' => 'PholioController',
    'PholioMockAuthorHeraldField' => 'PholioMockHeraldField',
    'PholioMockCommentController' => 'PholioController',
    'PholioMockDescriptionHeraldField' => 'PholioMockHeraldField',
    'PholioMockEditController' => 'PholioController',
    'PholioMockEditor' => 'PhabricatorApplicationTransactionEditor',
    'PholioMockEmbedView' => 'AphrontView',
    'PholioMockFulltextEngine' => 'PhabricatorFulltextEngine',
    'PholioMockHasTaskEdgeType' => 'PhabricatorEdgeType',
    'PholioMockHeraldField' => 'HeraldField',
    'PholioMockHeraldFieldGroup' => 'HeraldFieldGroup',
    'PholioMockImagesView' => 'AphrontView',
    'PholioMockListController' => 'PholioController',
    'PholioMockMailReceiver' => 'PhabricatorObjectMailReceiver',
    'PholioMockNameHeraldField' => 'PholioMockHeraldField',
    'PholioMockPHIDType' => 'PhabricatorPHIDType',
    'PholioMockQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PholioMockSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PholioMockThumbGridView' => 'AphrontView',
    'PholioMockViewController' => 'PholioController',
    'PholioRemarkupRule' => 'PhabricatorObjectRemarkupRule',
    'PholioReplyHandler' => 'PhabricatorApplicationTransactionReplyHandler',
    'PholioSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'PholioTransaction' => 'PhabricatorApplicationTransaction',
    'PholioTransactionComment' => 'PhabricatorApplicationTransactionComment',
    'PholioTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PholioTransactionView' => 'PhabricatorApplicationTransactionView',
    'PholioUploadedImageView' => 'AphrontView',
    'PhortuneAccount' => array(
      'PhortuneDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorPolicyInterface',
    ),
    'PhortuneAccountEditController' => 'PhortuneController',
    'PhortuneAccountEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhortuneAccountHasMemberEdgeType' => 'PhabricatorEdgeType',
    'PhortuneAccountListController' => 'PhortuneController',
    'PhortuneAccountPHIDType' => 'PhabricatorPHIDType',
    'PhortuneAccountQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhortuneAccountTransaction' => 'PhabricatorApplicationTransaction',
    'PhortuneAccountTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhortuneAccountViewController' => 'PhortuneController',
    'PhortuneAdHocCart' => 'PhortuneCartImplementation',
    'PhortuneAdHocProduct' => 'PhortuneProductImplementation',
    'PhortuneCart' => array(
      'PhortuneDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorPolicyInterface',
    ),
    'PhortuneCartAcceptController' => 'PhortuneCartController',
    'PhortuneCartCancelController' => 'PhortuneCartController',
    'PhortuneCartCheckoutController' => 'PhortuneCartController',
    'PhortuneCartController' => 'PhortuneController',
    'PhortuneCartEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhortuneCartImplementation' => 'Phobject',
    'PhortuneCartListController' => 'PhortuneController',
    'PhortuneCartPHIDType' => 'PhabricatorPHIDType',
    'PhortuneCartQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhortuneCartReplyHandler' => 'PhabricatorApplicationTransactionReplyHandler',
    'PhortuneCartSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhortuneCartTransaction' => 'PhabricatorApplicationTransaction',
    'PhortuneCartTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhortuneCartUpdateController' => 'PhortuneCartController',
    'PhortuneCartViewController' => 'PhortuneCartController',
    'PhortuneCharge' => array(
      'PhortuneDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhortuneChargeListController' => 'PhortuneController',
    'PhortuneChargePHIDType' => 'PhabricatorPHIDType',
    'PhortuneChargeQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhortuneChargeSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhortuneChargeTableView' => 'AphrontView',
    'PhortuneConstants' => 'Phobject',
    'PhortuneController' => 'PhabricatorController',
    'PhortuneCreditCardForm' => 'Phobject',
    'PhortuneCurrency' => 'Phobject',
    'PhortuneCurrencySerializer' => 'PhabricatorLiskSerializer',
    'PhortuneCurrencyTestCase' => 'PhabricatorTestCase',
    'PhortuneDAO' => 'PhabricatorLiskDAO',
    'PhortuneErrCode' => 'PhortuneConstants',
    'PhortuneLandingController' => 'PhortuneController',
    'PhortuneMemberHasAccountEdgeType' => 'PhabricatorEdgeType',
    'PhortuneMemberHasMerchantEdgeType' => 'PhabricatorEdgeType',
    'PhortuneMerchant' => array(
      'PhortuneDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorPolicyInterface',
    ),
    'PhortuneMerchantCapability' => 'PhabricatorPolicyCapability',
    'PhortuneMerchantController' => 'PhortuneController',
    'PhortuneMerchantEditController' => 'PhortuneMerchantController',
    'PhortuneMerchantEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhortuneMerchantHasMemberEdgeType' => 'PhabricatorEdgeType',
    'PhortuneMerchantInvoiceCreateController' => 'PhortuneMerchantController',
    'PhortuneMerchantListController' => 'PhortuneMerchantController',
    'PhortuneMerchantPHIDType' => 'PhabricatorPHIDType',
    'PhortuneMerchantQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhortuneMerchantSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhortuneMerchantTransaction' => 'PhabricatorApplicationTransaction',
    'PhortuneMerchantTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhortuneMerchantViewController' => 'PhortuneMerchantController',
    'PhortuneMonthYearExpiryControl' => 'AphrontFormControl',
    'PhortuneOrderTableView' => 'AphrontView',
    'PhortunePayPalPaymentProvider' => 'PhortunePaymentProvider',
    'PhortunePaymentMethod' => array(
      'PhortuneDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhortunePaymentMethodCreateController' => 'PhortuneController',
    'PhortunePaymentMethodDisableController' => 'PhortuneController',
    'PhortunePaymentMethodEditController' => 'PhortuneController',
    'PhortunePaymentMethodPHIDType' => 'PhabricatorPHIDType',
    'PhortunePaymentMethodQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhortunePaymentProvider' => 'Phobject',
    'PhortunePaymentProviderConfig' => array(
      'PhortuneDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhortunePaymentProviderConfigEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhortunePaymentProviderConfigQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhortunePaymentProviderConfigTransaction' => 'PhabricatorApplicationTransaction',
    'PhortunePaymentProviderConfigTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PhortunePaymentProviderPHIDType' => 'PhabricatorPHIDType',
    'PhortunePaymentProviderTestCase' => 'PhabricatorTestCase',
    'PhortuneProduct' => array(
      'PhortuneDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhortuneProductImplementation' => 'Phobject',
    'PhortuneProductListController' => 'PhabricatorController',
    'PhortuneProductPHIDType' => 'PhabricatorPHIDType',
    'PhortuneProductQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhortuneProductViewController' => 'PhortuneController',
    'PhortuneProviderActionController' => 'PhortuneController',
    'PhortuneProviderDisableController' => 'PhortuneMerchantController',
    'PhortuneProviderEditController' => 'PhortuneMerchantController',
    'PhortunePurchase' => array(
      'PhortuneDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhortunePurchasePHIDType' => 'PhabricatorPHIDType',
    'PhortunePurchaseQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhortuneSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'PhortuneStripePaymentProvider' => 'PhortunePaymentProvider',
    'PhortuneSubscription' => array(
      'PhortuneDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhortuneSubscriptionCart' => 'PhortuneCartImplementation',
    'PhortuneSubscriptionEditController' => 'PhortuneController',
    'PhortuneSubscriptionImplementation' => 'Phobject',
    'PhortuneSubscriptionListController' => 'PhortuneController',
    'PhortuneSubscriptionPHIDType' => 'PhabricatorPHIDType',
    'PhortuneSubscriptionProduct' => 'PhortuneProductImplementation',
    'PhortuneSubscriptionQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhortuneSubscriptionSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhortuneSubscriptionTableView' => 'AphrontView',
    'PhortuneSubscriptionViewController' => 'PhortuneController',
    'PhortuneSubscriptionWorker' => 'PhabricatorWorker',
    'PhortuneTestPaymentProvider' => 'PhortunePaymentProvider',
    'PhortuneWePayPaymentProvider' => 'PhortunePaymentProvider',
    'PhragmentBrowseController' => 'PhragmentController',
    'PhragmentCanCreateCapability' => 'PhabricatorPolicyCapability',
    'PhragmentConduitAPIMethod' => 'ConduitAPIMethod',
    'PhragmentController' => 'PhabricatorController',
    'PhragmentCreateController' => 'PhragmentController',
    'PhragmentDAO' => 'PhabricatorLiskDAO',
    'PhragmentFragment' => array(
      'PhragmentDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhragmentFragmentPHIDType' => 'PhabricatorPHIDType',
    'PhragmentFragmentQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhragmentFragmentVersion' => array(
      'PhragmentDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhragmentFragmentVersionPHIDType' => 'PhabricatorPHIDType',
    'PhragmentFragmentVersionQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhragmentGetPatchConduitAPIMethod' => 'PhragmentConduitAPIMethod',
    'PhragmentHistoryController' => 'PhragmentController',
    'PhragmentPatchController' => 'PhragmentController',
    'PhragmentPatchUtil' => 'Phobject',
    'PhragmentPolicyController' => 'PhragmentController',
    'PhragmentQueryFragmentsConduitAPIMethod' => 'PhragmentConduitAPIMethod',
    'PhragmentRevertController' => 'PhragmentController',
    'PhragmentSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'PhragmentSnapshot' => array(
      'PhragmentDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhragmentSnapshotChild' => array(
      'PhragmentDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhragmentSnapshotChildQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhragmentSnapshotCreateController' => 'PhragmentController',
    'PhragmentSnapshotDeleteController' => 'PhragmentController',
    'PhragmentSnapshotPHIDType' => 'PhabricatorPHIDType',
    'PhragmentSnapshotPromoteController' => 'PhragmentController',
    'PhragmentSnapshotQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhragmentSnapshotViewController' => 'PhragmentController',
    'PhragmentUpdateController' => 'PhragmentController',
    'PhragmentVersionController' => 'PhragmentController',
    'PhragmentZIPController' => 'PhragmentController',
    'PhrequentConduitAPIMethod' => 'ConduitAPIMethod',
    'PhrequentController' => 'PhabricatorController',
    'PhrequentDAO' => 'PhabricatorLiskDAO',
    'PhrequentListController' => 'PhrequentController',
    'PhrequentPopConduitAPIMethod' => 'PhrequentConduitAPIMethod',
    'PhrequentPushConduitAPIMethod' => 'PhrequentConduitAPIMethod',
    'PhrequentSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhrequentTimeBlock' => 'Phobject',
    'PhrequentTimeBlockTestCase' => 'PhabricatorTestCase',
    'PhrequentTimeSlices' => 'Phobject',
    'PhrequentTrackController' => 'PhrequentController',
    'PhrequentTrackingConduitAPIMethod' => 'PhrequentConduitAPIMethod',
    'PhrequentTrackingEditor' => 'PhabricatorEditor',
    'PhrequentUIEventListener' => 'PhabricatorEventListener',
    'PhrequentUserTime' => array(
      'PhrequentDAO',
      'PhabricatorPolicyInterface',
    ),
    'PhrequentUserTimeQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhrictionChangeType' => 'PhrictionConstants',
    'PhrictionConduitAPIMethod' => 'ConduitAPIMethod',
    'PhrictionConstants' => 'Phobject',
    'PhrictionContent' => array(
      'PhrictionDAO',
      'PhabricatorMarkupInterface',
    ),
    'PhrictionController' => 'PhabricatorController',
    'PhrictionCreateConduitAPIMethod' => 'PhrictionConduitAPIMethod',
    'PhrictionDAO' => 'PhabricatorLiskDAO',
    'PhrictionDeleteController' => 'PhrictionController',
    'PhrictionDiffController' => 'PhrictionController',
    'PhrictionDocument' => array(
      'PhrictionDAO',
      'PhabricatorPolicyInterface',
      'PhabricatorSubscribableInterface',
      'PhabricatorFlaggableInterface',
      'PhabricatorTokenReceiverInterface',
      'PhabricatorDestructibleInterface',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorFulltextInterface',
    ),
    'PhrictionDocumentAuthorHeraldField' => 'PhrictionDocumentHeraldField',
    'PhrictionDocumentContentHeraldField' => 'PhrictionDocumentHeraldField',
    'PhrictionDocumentController' => 'PhrictionController',
    'PhrictionDocumentFulltextEngine' => 'PhabricatorFulltextEngine',
    'PhrictionDocumentHeraldAdapter' => 'HeraldAdapter',
    'PhrictionDocumentHeraldField' => 'HeraldField',
    'PhrictionDocumentHeraldFieldGroup' => 'HeraldFieldGroup',
    'PhrictionDocumentPHIDType' => 'PhabricatorPHIDType',
    'PhrictionDocumentPathHeraldField' => 'PhrictionDocumentHeraldField',
    'PhrictionDocumentQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PhrictionDocumentStatus' => 'PhrictionConstants',
    'PhrictionDocumentTitleHeraldField' => 'PhrictionDocumentHeraldField',
    'PhrictionEditConduitAPIMethod' => 'PhrictionConduitAPIMethod',
    'PhrictionEditController' => 'PhrictionController',
    'PhrictionHistoryConduitAPIMethod' => 'PhrictionConduitAPIMethod',
    'PhrictionHistoryController' => 'PhrictionController',
    'PhrictionInfoConduitAPIMethod' => 'PhrictionConduitAPIMethod',
    'PhrictionListController' => 'PhrictionController',
    'PhrictionMoveController' => 'PhrictionController',
    'PhrictionNewController' => 'PhrictionController',
    'PhrictionRemarkupRule' => 'PhutilRemarkupRule',
    'PhrictionReplyHandler' => 'PhabricatorApplicationTransactionReplyHandler',
    'PhrictionSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'PhrictionSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PhrictionTransaction' => 'PhabricatorApplicationTransaction',
    'PhrictionTransactionComment' => 'PhabricatorApplicationTransactionComment',
    'PhrictionTransactionEditor' => 'PhabricatorApplicationTransactionEditor',
    'PhrictionTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PolicyLockOptionType' => 'PhabricatorConfigJSONOptionType',
    'PonderAddAnswerView' => 'AphrontView',
    'PonderAnswer' => array(
      'PonderDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorMarkupInterface',
      'PonderVotableInterface',
      'PhabricatorPolicyInterface',
      'PhabricatorFlaggableInterface',
      'PhabricatorSubscribableInterface',
      'PhabricatorDestructibleInterface',
    ),
    'PonderAnswerCommentController' => 'PonderController',
    'PonderAnswerEditController' => 'PonderController',
    'PonderAnswerEditor' => 'PonderEditor',
    'PonderAnswerHasVotingUserEdgeType' => 'PhabricatorEdgeType',
    'PonderAnswerHistoryController' => 'PonderController',
    'PonderAnswerMailReceiver' => 'PhabricatorObjectMailReceiver',
    'PonderAnswerPHIDType' => 'PhabricatorPHIDType',
    'PonderAnswerQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PonderAnswerReplyHandler' => 'PhabricatorApplicationTransactionReplyHandler',
    'PonderAnswerSaveController' => 'PonderController',
    'PonderAnswerStatus' => 'PonderConstants',
    'PonderAnswerTransaction' => 'PhabricatorApplicationTransaction',
    'PonderAnswerTransactionComment' => 'PhabricatorApplicationTransactionComment',
    'PonderAnswerTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PonderAnswerView' => 'AphrontTagView',
    'PonderConstants' => 'Phobject',
    'PonderController' => 'PhabricatorController',
    'PonderDAO' => 'PhabricatorLiskDAO',
    'PonderDefaultViewCapability' => 'PhabricatorPolicyCapability',
    'PonderEditor' => 'PhabricatorApplicationTransactionEditor',
    'PonderFooterView' => 'AphrontTagView',
    'PonderHelpfulSaveController' => 'PonderController',
    'PonderModerateCapability' => 'PhabricatorPolicyCapability',
    'PonderQuestion' => array(
      'PonderDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorMarkupInterface',
      'PhabricatorSubscribableInterface',
      'PhabricatorFlaggableInterface',
      'PhabricatorPolicyInterface',
      'PhabricatorTokenReceiverInterface',
      'PhabricatorProjectInterface',
      'PhabricatorDestructibleInterface',
      'PhabricatorSpacesInterface',
      'PhabricatorFulltextInterface',
    ),
    'PonderQuestionCommentController' => 'PonderController',
    'PonderQuestionEditController' => 'PonderController',
    'PonderQuestionEditor' => 'PonderEditor',
    'PonderQuestionFulltextEngine' => 'PhabricatorFulltextEngine',
    'PonderQuestionHistoryController' => 'PonderController',
    'PonderQuestionListController' => 'PonderController',
    'PonderQuestionMailReceiver' => 'PhabricatorObjectMailReceiver',
    'PonderQuestionPHIDType' => 'PhabricatorPHIDType',
    'PonderQuestionQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'PonderQuestionReplyHandler' => 'PhabricatorApplicationTransactionReplyHandler',
    'PonderQuestionSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'PonderQuestionStatus' => 'PonderConstants',
    'PonderQuestionStatusController' => 'PonderController',
    'PonderQuestionTransaction' => 'PhabricatorApplicationTransaction',
    'PonderQuestionTransactionComment' => 'PhabricatorApplicationTransactionComment',
    'PonderQuestionTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'PonderQuestionViewController' => 'PonderController',
    'PonderRemarkupRule' => 'PhabricatorObjectRemarkupRule',
    'PonderSchemaSpec' => 'PhabricatorConfigSchemaSpec',
    'PonderVote' => 'PonderConstants',
    'PonderVoteEditor' => 'PhabricatorEditor',
    'PonderVotingUserHasAnswerEdgeType' => 'PhabricatorEdgeType',
    'ProjectAddProjectsEmailCommand' => 'MetaMTAEmailTransactionCommand',
    'ProjectBoardTaskCard' => 'Phobject',
    'ProjectCanLockProjectsCapability' => 'PhabricatorPolicyCapability',
    'ProjectConduitAPIMethod' => 'ConduitAPIMethod',
    'ProjectCreateConduitAPIMethod' => 'ProjectConduitAPIMethod',
    'ProjectCreateProjectsCapability' => 'PhabricatorPolicyCapability',
    'ProjectDefaultEditCapability' => 'PhabricatorPolicyCapability',
    'ProjectDefaultJoinCapability' => 'PhabricatorPolicyCapability',
    'ProjectDefaultViewCapability' => 'PhabricatorPolicyCapability',
    'ProjectEditConduitAPIMethod' => 'PhabricatorEditEngineAPIMethod',
    'ProjectQueryConduitAPIMethod' => 'ProjectConduitAPIMethod',
    'ProjectRemarkupRule' => 'PhabricatorObjectRemarkupRule',
    'ProjectRemarkupRuleTestCase' => 'PhabricatorTestCase',
    'ProjectReplyHandler' => 'PhabricatorApplicationTransactionReplyHandler',
    'ProjectSearchConduitAPIMethod' => 'PhabricatorSearchEngineAPIMethod',
    'QueryFormattingTestCase' => 'PhabricatorTestCase',
    'ReleephAuthorFieldSpecification' => 'ReleephFieldSpecification',
    'ReleephBranch' => array(
      'ReleephDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorPolicyInterface',
    ),
    'ReleephBranchAccessController' => 'ReleephBranchController',
    'ReleephBranchCommitFieldSpecification' => 'ReleephFieldSpecification',
    'ReleephBranchController' => 'ReleephController',
    'ReleephBranchCreateController' => 'ReleephProductController',
    'ReleephBranchEditController' => 'ReleephBranchController',
    'ReleephBranchEditor' => 'PhabricatorEditor',
    'ReleephBranchHistoryController' => 'ReleephBranchController',
    'ReleephBranchNamePreviewController' => 'ReleephController',
    'ReleephBranchPHIDType' => 'PhabricatorPHIDType',
    'ReleephBranchPreviewView' => 'AphrontFormControl',
    'ReleephBranchQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'ReleephBranchSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'ReleephBranchTemplate' => 'Phobject',
    'ReleephBranchTransaction' => 'PhabricatorApplicationTransaction',
    'ReleephBranchTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'ReleephBranchViewController' => 'ReleephBranchController',
    'ReleephCommitFinder' => 'Phobject',
    'ReleephCommitFinderException' => 'Exception',
    'ReleephCommitMessageFieldSpecification' => 'ReleephFieldSpecification',
    'ReleephConduitAPIMethod' => 'ConduitAPIMethod',
    'ReleephController' => 'PhabricatorController',
    'ReleephDAO' => 'PhabricatorLiskDAO',
    'ReleephDefaultFieldSelector' => 'ReleephFieldSelector',
    'ReleephDependsOnFieldSpecification' => 'ReleephFieldSpecification',
    'ReleephDiffChurnFieldSpecification' => 'ReleephFieldSpecification',
    'ReleephDiffMessageFieldSpecification' => 'ReleephFieldSpecification',
    'ReleephDiffSizeFieldSpecification' => 'ReleephFieldSpecification',
    'ReleephFieldParseException' => 'Exception',
    'ReleephFieldSelector' => 'Phobject',
    'ReleephFieldSpecification' => array(
      'PhabricatorCustomField',
      'PhabricatorMarkupInterface',
    ),
    'ReleephGetBranchesConduitAPIMethod' => 'ReleephConduitAPIMethod',
    'ReleephIntentFieldSpecification' => 'ReleephFieldSpecification',
    'ReleephLevelFieldSpecification' => 'ReleephFieldSpecification',
    'ReleephOriginalCommitFieldSpecification' => 'ReleephFieldSpecification',
    'ReleephProductActionController' => 'ReleephProductController',
    'ReleephProductController' => 'ReleephController',
    'ReleephProductCreateController' => 'ReleephProductController',
    'ReleephProductEditController' => 'ReleephProductController',
    'ReleephProductEditor' => 'PhabricatorApplicationTransactionEditor',
    'ReleephProductHistoryController' => 'ReleephProductController',
    'ReleephProductListController' => 'ReleephController',
    'ReleephProductPHIDType' => 'PhabricatorPHIDType',
    'ReleephProductQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'ReleephProductSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'ReleephProductTransaction' => 'PhabricatorApplicationTransaction',
    'ReleephProductTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'ReleephProductViewController' => 'ReleephProductController',
    'ReleephProject' => array(
      'ReleephDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorPolicyInterface',
    ),
    'ReleephQueryBranchesConduitAPIMethod' => 'ReleephConduitAPIMethod',
    'ReleephQueryProductsConduitAPIMethod' => 'ReleephConduitAPIMethod',
    'ReleephQueryRequestsConduitAPIMethod' => 'ReleephConduitAPIMethod',
    'ReleephReasonFieldSpecification' => 'ReleephFieldSpecification',
    'ReleephRequest' => array(
      'ReleephDAO',
      'PhabricatorApplicationTransactionInterface',
      'PhabricatorPolicyInterface',
      'PhabricatorCustomFieldInterface',
    ),
    'ReleephRequestActionController' => 'ReleephRequestController',
    'ReleephRequestCommentController' => 'ReleephRequestController',
    'ReleephRequestConduitAPIMethod' => 'ReleephConduitAPIMethod',
    'ReleephRequestController' => 'ReleephController',
    'ReleephRequestDifferentialCreateController' => 'ReleephController',
    'ReleephRequestEditController' => 'ReleephBranchController',
    'ReleephRequestMailReceiver' => 'PhabricatorObjectMailReceiver',
    'ReleephRequestPHIDType' => 'PhabricatorPHIDType',
    'ReleephRequestQuery' => 'PhabricatorCursorPagedPolicyAwareQuery',
    'ReleephRequestReplyHandler' => 'PhabricatorApplicationTransactionReplyHandler',
    'ReleephRequestSearchEngine' => 'PhabricatorApplicationSearchEngine',
    'ReleephRequestStatus' => 'Phobject',
    'ReleephRequestTransaction' => 'PhabricatorApplicationTransaction',
    'ReleephRequestTransactionComment' => 'PhabricatorApplicationTransactionComment',
    'ReleephRequestTransactionQuery' => 'PhabricatorApplicationTransactionQuery',
    'ReleephRequestTransactionalEditor' => 'PhabricatorApplicationTransactionEditor',
    'ReleephRequestTypeaheadControl' => 'AphrontFormControl',
    'ReleephRequestTypeaheadController' => 'PhabricatorTypeaheadDatasourceController',
    'ReleephRequestView' => 'AphrontView',
    'ReleephRequestViewController' => 'ReleephBranchController',
    'ReleephRequestorFieldSpecification' => 'ReleephFieldSpecification',
    'ReleephRevisionFieldSpecification' => 'ReleephFieldSpecification',
    'ReleephSeverityFieldSpecification' => 'ReleephLevelFieldSpecification',
    'ReleephSummaryFieldSpecification' => 'ReleephFieldSpecification',
    'ReleephWorkCanPushConduitAPIMethod' => 'ReleephConduitAPIMethod',
    'ReleephWorkGetAuthorInfoConduitAPIMethod' => 'ReleephConduitAPIMethod',
    'ReleephWorkGetBranchCommitMessageConduitAPIMethod' => 'ReleephConduitAPIMethod',
    'ReleephWorkGetBranchConduitAPIMethod' => 'ReleephConduitAPIMethod',
    'ReleephWorkGetCommitMessageConduitAPIMethod' => 'ReleephConduitAPIMethod',
    'ReleephWorkNextRequestConduitAPIMethod' => 'ReleephConduitAPIMethod',
    'ReleephWorkRecordConduitAPIMethod' => 'ReleephConduitAPIMethod',
    'ReleephWorkRecordPickStatusConduitAPIMethod' => 'ReleephConduitAPIMethod',
    'RemarkupProcessConduitAPIMethod' => 'ConduitAPIMethod',
    'RepositoryConduitAPIMethod' => 'ConduitAPIMethod',
    'RepositoryCreateConduitAPIMethod' => 'RepositoryConduitAPIMethod',
    'RepositoryQueryConduitAPIMethod' => 'RepositoryConduitAPIMethod',
    'ShellLogView' => 'AphrontView',
    'SlowvoteConduitAPIMethod' => 'ConduitAPIMethod',
    'SlowvoteEmbedView' => 'AphrontView',
    'SlowvoteInfoConduitAPIMethod' => 'SlowvoteConduitAPIMethod',
    'SlowvoteRemarkupRule' => 'PhabricatorObjectRemarkupRule',
    'SubscriptionListDialogBuilder' => 'Phobject',
    'SubscriptionListStringBuilder' => 'Phobject',
    'TokenConduitAPIMethod' => 'ConduitAPIMethod',
    'TokenGiveConduitAPIMethod' => 'TokenConduitAPIMethod',
    'TokenGivenConduitAPIMethod' => 'TokenConduitAPIMethod',
    'TokenQueryConduitAPIMethod' => 'TokenConduitAPIMethod',
    'UserConduitAPIMethod' => 'ConduitAPIMethod',
    'UserDisableConduitAPIMethod' => 'UserConduitAPIMethod',
    'UserEnableConduitAPIMethod' => 'UserConduitAPIMethod',
    'UserFindConduitAPIMethod' => 'UserConduitAPIMethod',
    'UserQueryConduitAPIMethod' => 'UserConduitAPIMethod',
    'UserWhoAmIConduitAPIMethod' => 'UserConduitAPIMethod',
  ),
));
